 /**
 *                                                                             *
 * @file MyGraphicsScene.cpp                                                   *
 * Project name: HeKatE hqed                                                   *
 * $Id: ardGraphicsScene.cpp,v 1.7 2008-03-03 22:04:31 kinio Exp $              *
 * Author: Krzysztof Kaczor                                                    *
 * Date: 12.10.2007                                                            *
 *                                                                             *
 * Implementation by Krzysztof Kaczor <kinio4444@gmail.com>                    *
 *                                                                             *
 * Copyright (C) 2006-9 by the HeKatE Project                                  *
 *                                                                             *
 * HQEd has been develped within the HeKatE Project,                           *
 * see http://hekate.ia.agh.edu.pl                                             *
 *                                                                             *
 * This file is part of HQEd.                                                  *
 *                                                                             *
 * HQEd is free software: you can redistribute it and/or modify                *
 * it under the terms of the GNU General Public License as published by        *
 * the Free Software Foundation, either version 3 of the License, or           *
 * (at your option) any later version.                                         *
 *                                                                             *
 * HQEd is distributed in the hope that it will be useful,                     *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU General Public License for more details.                                *
 *                                                                             *
 * You should have received a copy of the GNU General Public License           *
 * along with HQEd.  If not, see <http://www.gnu.org/licenses/>.               *
 *                                                                             *
 * Comment: Plik zawierajacy definicje klasy pochodnej elementu bedacego       *
 * plotnem na ktorym rysowana jest scena                                       *
 *                                                                             *
 */

#include <QMessageBox>
#include <QtGui>
#include <QDialog>
#include <QObject>
#include <QKeyEvent>
#include <QGraphicsScene>
#include <QGraphicsView>

#include "../MainWin_ui.h"
#include "../Settings_ui.h"
#include "ardGraphicsScene.h"
// -----------------------------------------------------------------------------

/**
 * Konstruktor kalsy
 * @param QObject *parent - Wskanik do rodzica
 * @returns Brak zwracanych wartosci
*/
ardGraphicsScene::ardGraphicsScene(QObject* oParent) : QGraphicsScene(oParent)
{	
	setNewItem(NULL);
	
	fCurrentAngle = 0;
	fSelectedPropertyID = -1;
	fSelectedDependenciesID = -1;
}
// -----------------------------------------------------------------------------

/**
 * Destruktor klasy, kasuje dynamicznie zaalokowana pamiec
 * @returns Brak zwracanych wartosci
 */
ardGraphicsScene::~ardGraphicsScene(void)
{
}
// -----------------------------------------------------------------------------

/**
 * Pozwala ustawic obiekt na scenie ktorego pozycje trzeba ustawic
 * @param QGraphicsItem* _item - Wskanik do obiektu
 * @returns Brak zwracanych wartosci
*/
void ardGraphicsScene::setNewItem(QGraphicsItem* _item)
{
	fNewItem = _item;
}
// -----------------------------------------------------------------------------
	
/**
 * Zwraca wskaznik do obiektu na scenie ktorego pozycje trzeba ustawic
 * @returns Zwraca wskaznik do obiektu na scenie ktorego pozycje trzeba ustawic
*/
QGraphicsItem* ardGraphicsScene::newItem(void)
{
	return fNewItem;
}
// -----------------------------------------------------------------------------

/**
 * Resetuje wskaznik do obiektu ktory trzeba aktualnie ustawic na scenie
 * @returns Brak zwracanych wartosci
*/
void ardGraphicsScene::resetNewItem(void)
{
	setNewItem(NULL);
}
// -----------------------------------------------------------------------------

/**
 * Metoda wywolywana w momencie kiedy nastapil ruch kursora nad scena
 * @param QGraphicsSceneMouseEvent* mouseEvent - Wskanik do obiektu z parametrami zdarzenia
 * @returns Brak zwracanych wartosci
*/
void ardGraphicsScene::mouseMoveEvent(QGraphicsSceneMouseEvent* mouseEvent)
{
	// Ustawiamy bierzaca scene jako aktualna
	MainWin->setCurrentARDscene(this);

	// Jezeli mamy jakis nowy przedmiot to jest on priorytetm do tego aby zawsze byl widoczny
	if(fNewItem)
	{
		fNewItem->setPos(mouseEvent->scenePos());
		if(Settings->xttEnabledAutocentering())
			for(int i=0;i<views().count();i++)
				views().at(i)->ensureVisible(fNewItem);
	}
		
	// Jezeli cos chwycilismy myszka
	if((!fNewItem) && (mouseGrabberItem()))
	{
		if(Settings->xttEnabledAutocentering())
			for(int i=0;i<views().count();i++)
				views().at(i)->ensureVisible(mouseGrabberItem());
	}
		
	QGraphicsScene::mouseMoveEvent(mouseEvent);
}
// -----------------------------------------------------------------------------

/**
 * Metoda wywolywana w momencie kiedy nastapilo klikniecie na scenie
 * @param QGraphicsSceneMouseEvent* mouseEvent - Wskanik do obiektu z parametrami zdarzenia
 * @returns Brak zwracanych wartosci
*/
void ardGraphicsScene::mousePressEvent(QGraphicsSceneMouseEvent* mouseEvent)
{
	if(fNewItem)
		resetNewItem();
		
	QGraphicsScene::mousePressEvent(mouseEvent);
}
// -----------------------------------------------------------------------------

/**
 * Metoda wywolywana w momencie zmiany kata obrotu sceny
 * @param int angle - Wartosc konta docelowego
 * @returns Brak zwracanych wartoci
*/
void ardGraphicsScene::Rotate(int angle)
{
	QList<QGraphicsView*> views_list = views();
	
	for(int i=0;i<views_list.count();i++)
		views_list.at(i)->rotate(angle-fCurrentAngle);
		
     fCurrentAngle = angle;
	//rotateAngleEdit->setValue(angle);
}
// -----------------------------------------------------------------------------

/**
 * Metoda powiekszajaca scene
 * @returns Brak zwracanych wartoci
*/
void ardGraphicsScene::zin(void)
{
	double factor = (Settings->zoomFactorValue(MainWin->currentMode())/100.0)+1.0;
     scaleView(factor);
}
// -----------------------------------------------------------------------------

/**
 * Metoda pomniejszajace scene
 * @returns Brak zwracanych wartoci
*/
void ardGraphicsScene::zout(void)
{
	double factor = (Settings->zoomFactorValue(MainWin->currentMode())/100.0)+1.0;
     scaleView(1./factor);
}
// -----------------------------------------------------------------------------

/**
 * Metoda klasy robiaca zoom z zadanym powiekszeniem
 * @param qreal scaleFactor - Powiekszenie
 * @returns Brak zwracanych wartoci
*/
void ardGraphicsScene::scaleView(qreal scaleFactor)
{
	QList<QGraphicsView*> views_list = views();
	
	for(int i=0;i<views_list.count();i++)
	{
		qreal factor = views_list.at(i)->matrix().scale(scaleFactor, scaleFactor).mapRect(QRectF(0, 0, 1, 1)).width();
		if (factor < 0.007 || factor > 100)
			continue;
		views_list.at(i)->scale(scaleFactor, scaleFactor);
	}
}
// -----------------------------------------------------------------------------

/**
 * Metoda dostosowywujaca rozmiar sceny do rozmieszczenia elementow
 * @returns Brak zwracanych wartoci
*/
void ardGraphicsScene::AdaptSceneSize(void)
{
     const int Margin = 50;

     QList<QGraphicsItem*> items_list = items();

     float minLeft = 0;
     float minTop = 0;
     float maxLeft = 0;
     float maxTop = 0;

     for(int i=0;i<items_list.size();i++)
     {
          if(minLeft > items_list.at(i)->x())
               minLeft = items_list.at(i)->x();
          if(minTop > items_list.at(i)->y())
               minTop = items_list.at(i)->y();
          if(maxLeft < items_list.at(i)->x())
               maxLeft = items_list.at(i)->x();
          if(maxTop < items_list.at(i)->y())
               maxTop = items_list.at(i)->y();
     }

     minLeft -= Margin;
     minTop -= Margin;
     maxLeft += Margin;
     maxTop += Margin;

     setSceneRect(minLeft, minTop, maxLeft-minLeft, maxTop-minTop);
}
// -----------------------------------------------------------------------------

/**
 * Przesuwa elementy na scenie w gore
 * @returns Brak zwracanych wartoci
*/
void ardGraphicsScene::MoveItemsUp(void)
{	
	int moveDistance = Settings->ardMoveElementsDistance();
     setSceneRect(sceneRect().x(), sceneRect().y()+moveDistance, sceneRect().width(), sceneRect().height()-moveDistance);
}
// -----------------------------------------------------------------------------

/**
 * Przesuwa elementy na scenie w dol
 * @returns Brak zwracanych wartoci
*/
void ardGraphicsScene::MoveItemsDown(void)
{
	int moveDistance = Settings->ardMoveElementsDistance();
     setSceneRect(sceneRect().x(), sceneRect().y()-moveDistance, sceneRect().width(), sceneRect().height()+moveDistance);
}
// -----------------------------------------------------------------------------

/**
 * Rozszerza scene na lewo, przesuwajac elementy w prawo
 * @returns Brak zwracanych wartoci
*/
void ardGraphicsScene::MoveItemsRight(void)
{
	int moveDistance = Settings->ardMoveElementsDistance();
     setSceneRect(sceneRect().x()-moveDistance, sceneRect().y(), sceneRect().width()+moveDistance, sceneRect().height());
}
// -----------------------------------------------------------------------------

/**
 * Zweza scene od lewej, przesuwajac elementy w lewo
 * @returns Brak zwracanych wartoci
*/
void ardGraphicsScene::MoveItemsLeft(void)
{
	int moveDistance = Settings->ardMoveElementsDistance();
     setSceneRect(sceneRect().x()+moveDistance, sceneRect().y(), sceneRect().width()-moveDistance, sceneRect().height());
}
// -----------------------------------------------------------------------------
