 /**
 *                                                                             *
 * @file GLStartWidget.cpp                                                     *
 * Project name: HeKatE hqed                                                   *
 * $Id: GLStartWidget.cpp,v 1.7 2008-03-03 22:04:31 kinio Exp $                *
 * Author: Krzysztof Kaczor                                                    *
 * Date: 14.01.2008                                                            *
 *                                                                             *
 * Implementation by Krzysztof Kaczor <kinio4444@gmail.com>                    *
 *                                                                             *
 * Copyright (C) 2006-9 by the HeKatE Project                                  *
 *                                                                             *
 * HQEd has been develped within the HeKatE Project,                           *
 * see http://hekate.ia.agh.edu.pl                                             *
 *                                                                             *
 * This file is part of HQEd.                                                  *
 *                                                                             *
 * HQEd is free software: you can redistribute it and/or modify                *
 * it under the terms of the GNU General Public License as published by        *
 * the Free Software Foundation, either version 3 of the License, or           *
 * (at your option) any later version.                                         *
 *                                                                             *
 * HQEd is distributed in the hope that it will be useful,                     *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU General Public License for more details.                                *
 *                                                                             *
 * You should have received a copy of the GNU General Public License           *
 * along with HQEd.  If not, see <http://www.gnu.org/licenses/>.               *
 *                                                                             *
 * Comment: Plik zawierajacy definicje metod klasy obslugujacej obiekt         *
 * wspierajacy rysowanie w OpenGL                                              *
 *                                                                             *
 */

#include <QtGui>
#include <QtOpenGL>

#include <math.h>

#include "Settings_ui.h"
#include "GLStartWidget.h"
// -----------------------------------------------------------------------------

#define WALL_SIZE 10
// -----------------------------------------------------------------------------

/**
 * Konstruktor kalsy
 * @param QWidget *parent - Wskanik do rodzica
 * @returns Brak zwracanych wartosci
*/
GLStartWidget::GLStartWidget(QWidget *parent)
    : QGLWidget(parent)
{
	QTimer* timer = new QTimer(this);
     connect(timer, SIGNAL(timeout()), this, SLOT(renderScene()));
     timer->start(20);
	
	cubesJoined = false;
	runAppSend = false;
	backgroundPixmap = QPixmap::grabWindow(QApplication::desktop()->winId());
}
// -----------------------------------------------------------------------------

/**
 * Destruktor kalsy
 * @returns Brak zwracanych wartosci
*/
GLStartWidget::~GLStartWidget(void)
{
}
// -----------------------------------------------------------------------------

/**
 * Metoda tworzaca kontekst rysowania i przygotowywujaca parametry sceny
 * @returns Brak zwracanych wartosci
*/
void GLStartWidget::initializeGL(void)
{
	// Ustalanie wartosci zmiennych
	bkgAplpha = 0.0;
	d_bkgAlpha = 1.0;
	
	gl_x_min = -10.0;
	gl_x_max = +10.0;
	gl_y_min = -10.0;
	gl_y_max = +10.0;
	gl_z_min = 0.0;
	gl_z_max = +10.0;
	
	d_xrot1 = 16.0*((float)Settings->startScreenAnimationSpeed()/100.);
	d_yrot1 = 4.0*((float)Settings->startScreenAnimationSpeed()/100.);
	d_zrot1 = 8.0*((float)Settings->startScreenAnimationSpeed()/100.);
	d_xpos1 = -0.8*((float)Settings->startScreenAnimationSpeed()/100.);
	d_ypos1 = -1.6*((float)Settings->startScreenAnimationSpeed()/100.);
	
	d_xrot2 = 16.0*((float)Settings->startScreenAnimationSpeed()/100.);
	d_yrot2 = 4.0*((float)Settings->startScreenAnimationSpeed()/100.);
	d_zrot2 = 8.0*((float)Settings->startScreenAnimationSpeed()/100.);
	d_xpos2 = -0.8*((float)Settings->startScreenAnimationSpeed()/100.);
	d_ypos2 = -1.6*((float)Settings->startScreenAnimationSpeed()/100.);
	
	xpos1 = ypos1 = xrot1 = yrot1 = zrot1 = 0.0;
	xpos2 = ypos2 = xrot2 = yrot2 = zrot2 = 0.0;
	xpos1 = -20.0;
	xpos2 = 20.0;
	ypos1 = 20.0;
	ypos2 = 20.0;
	
	tmpVar = 0.0;
	

	if(Settings->startScreenBackgroundType() == 0)
		bkgwall = bindTexture(QPixmap(":/all_images/images/aghLogoFull.bmp"), GL_TEXTURE_2D);
	if(Settings->startScreenBackgroundType() == 1)
		bkgwall = bindTexture(backgroundPixmap.copy(fCopyArea), GL_TEXTURE_2D);
	
	eagle = bindTexture(QPixmap(":/all_images/images/aghLogo.bmp"), GL_TEXTURE_2D);
	eaglebw = bindTexture(QPixmap(":/all_images/images/aghLogoBW.bmp"), GL_TEXTURE_2D);
	xttt = bindTexture(QPixmap(":/all_images/images/xtt_apperance.bmp"), GL_TEXTURE_2D);
	ardt = bindTexture(QPixmap(":/all_images/images/ard_apperance.bmp"), GL_TEXTURE_2D);
	
	// Budowanie list
	buildLists();
	
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
	makeCurrent();
	glClearColor(1.0, 1.0, 1.0, 0.0);
}
// -----------------------------------------------------------------------------

/**
 * Metoda reagujaca na zmiane wymiarow sceny rysowania OpenGL
 * @param int _width - nowa szerokosc
 * @param int _height - nowa wysokosc
 * @returns Brak zwracanych wartosci
*/
void GLStartWidget::resizeGL(int width, int height)
{
	int side = qMin(width, height);
     glViewport((width - side) / 2, (height - side) / 2, side, side);

     glMatrixMode(GL_PROJECTION);
     glLoadIdentity();
	glFrustum(gl_x_min, gl_x_max, gl_y_min, gl_y_max, gl_z_min, gl_z_max);
	gluPerspective(45.0f, 1.0, 0.1f, 700.0f);
	
     glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	
	// Wlaczenie opcji OpenGL
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_TEXTURE_2D);
}
// -----------------------------------------------------------------------------

/**
 * Metoda rysujaca scene OpenGL
 * @returns Brak zwracanych wartosci
*/
void GLStartWidget::paintGL(void)
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	
	glLoadIdentity();
	glTranslatef(0.0, 0.0, -24.3);
	glPushMatrix();
	
	qglColor(QColor(255, 255, 255));
	glShadeModel(GL_SMOOTH);
	glEnable(GL_BLEND);
	glDisable(GL_DEPTH_TEST);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	
	// Rysowanie tla
	float ws = WALL_SIZE;
	QColor wallColor(255, 255, 255, 255);
	glBindTexture(GL_TEXTURE_2D, bkgwall);
	glBegin(GL_QUADS);
		qglColor(wallColor);	glTexCoord2f(0.0f, 0.0f);	glVertex3f(-ws, -ws, 0.0);
		qglColor(wallColor);	glTexCoord2f(0.0f, 1.0f);	glVertex3f(-ws, ws, 0.0);
		qglColor(wallColor);	glTexCoord2f(1.0f, 1.0f);	glVertex3f(ws, ws, 0.0);
		qglColor(wallColor);	glTexCoord2f(1.0f, 0.0f);	glVertex3f(ws, -ws, 0.0);
	glEnd();
	
	// Rysowanie orzelka AGH
	wallColor.setAlpha(255);
	ws = 1.5;
	float x = 10.0-ws;
	float y = 10.0-ws;
	
	glBlendFunc(GL_DST_COLOR, GL_ZERO); 
	glBindTexture(GL_TEXTURE_2D, eaglebw);	// Maska
	glBegin(GL_QUADS);
		qglColor(wallColor);	glTexCoord2f(0.0f, 0.0f);	glVertex3f(x-ws, y-ws, 0.0);
		qglColor(wallColor);	glTexCoord2f(0.0f, 1.0f);	glVertex3f(x-ws, y+ws, 0.0);
		qglColor(wallColor);	glTexCoord2f(1.0f, 1.0f);	glVertex3f(x+ws, y+ws, 0.0);
		qglColor(wallColor);	glTexCoord2f(1.0f, 0.0f);	glVertex3f(x+ws, y-ws, 0.0);
	glEnd();
	
	glBlendFunc(GL_ONE, GL_ONE);
	glBindTexture(GL_TEXTURE_2D, eagle);	// Wlasciwy obraz
	glBegin(GL_QUADS);
		qglColor(wallColor);	glTexCoord2f(0.0f, 0.0f);	glVertex3f(x-ws, y-ws, 0.0);
		qglColor(wallColor);	glTexCoord2f(0.0f, 1.0f);	glVertex3f(x-ws, y+ws, 0.0);
		qglColor(wallColor);	glTexCoord2f(1.0f, 1.0f);	glVertex3f(x+ws, y+ws, 0.0);
		qglColor(wallColor);	glTexCoord2f(1.0f, 0.0f);	glVertex3f(x+ws, y-ws, 0.0);
	glEnd();

	if(!cubesJoined)
	{
		// Rysowanie szescianow
		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
		
		glScalef(0.4, 0.4, 0.4);
		glTranslatef(xpos1, ypos1, 0.0);
		glRotatef(xrot1, 1.0, 0.0, 0.0);
		glRotatef(yrot1, 0.0, 1.0, 0.0);
		glRotatef(zrot1, 0.0, 0.0, 1.0);
		glCallList(cube1);
		
		glPopMatrix();
		glScalef(0.4, 0.4, 0.4);
		glTranslatef(xpos2, ypos2, 0.0);
		glRotatef(xrot2, 1.0, 0.0, 0.0);
		glRotatef(yrot2, 0.0, 1.0, 0.0);
		glRotatef(zrot2, 0.0, 0.0, 1.0);
		glCallList(cube2);
	}
	
	glDisable(GL_BLEND);
	glEnable(GL_DEPTH_TEST);
	
	if(cubesJoined)
	{
		GLfloat size = tmpVar;
	     static GLfloat lightPosition0[4] = {-size, -size, tmpVar, 1.0};
		static GLfloat specref[] = {1.0f, 1.0f, 1.0f, 1.0f};
	
		glLoadIdentity();
		
		glFrontFace(GL_CCW);
		glShadeModel(GL_SMOOTH);
	     glEnable(GL_LIGHTING);
	     glEnable(GL_LIGHT0);
		glEnable(GL_COLOR_MATERIAL);
		glColorMaterial(GL_FRONT, GL_AMBIENT_AND_DIFFUSE);
		glMaterialfv(GL_FRONT, GL_SPECULAR, specref);
		glMateriali(GL_FRONT, GL_SHININESS, 128);
		
	     glLightfv(GL_LIGHT0, GL_POSITION, lightPosition0);
		
		glTranslatef(0.0, 0.0, -24.3);
		glScalef(0.4, 0.4, 0.4);
		glTranslatef(xpos1, ypos1, 0.0);
		glRotatef(xrot1, 1.0, 0.0, 0.0);
		glRotatef(yrot1, 0.0, 1.0, 0.0);
		glRotatef(zrot1, 0.0, 0.0, 1.0);
		glCallList(cube3);
		
	     glDisable(GL_LIGHTING);
	     glDisable(GL_LIGHT0);
	}
}
// -----------------------------------------------------------------------------

/**
 * Metoda uaktualniajaca scene OpenGL
 * @returns Brak zwracanych wartosci
*/
void GLStartWidget::renderScene(void)
{
	if(((xrot1 + d_xrot1) > 360.) || ((xrot1 + d_xrot1) < 0.))
		d_xrot1 = -d_xrot1;
	xrot1 += d_xrot1;
	if(((yrot1 + d_yrot1) > 360.) || ((yrot1 + d_yrot1) < 0.))
		d_yrot1 = -d_yrot1;
	yrot1 += d_yrot1;
	if(((zrot1 + d_zrot1) > 360.) || ((zrot1 + d_zrot1) < 0.))
		d_zrot1 = -d_zrot1;
	zrot1 += d_zrot1;
	
	if(!cubesJoined)
	{
		if(qAbs(xpos1 + d_xpos1) > 20.)
			d_xpos1 = -d_xpos1;
		xpos1 += d_xpos1;
	}
	if(qAbs(ypos1 + d_ypos1) > 20.)
		d_ypos1 = -d_ypos1;
	ypos1 += d_ypos1;
	
	if(((xrot2 + d_xrot2) > 360.) || ((xrot2 + d_xrot2) < 0.))
		d_xrot2 = -d_xrot2;
	xrot2 += d_xrot2;
	if(((yrot2 + d_yrot2) > 360.) || ((yrot2 + d_yrot2) < 0.))
		d_yrot2 = -d_yrot2;
	yrot2 += d_yrot2;
	if(((zrot2 + d_zrot2) > 360.) || ((zrot2 + d_zrot2) < 0.))
		d_zrot2 = -d_zrot2;
	zrot2 += d_zrot2;
	
	if(!cubesJoined)
	{
		if(qAbs(xpos2 + d_xpos2) > 20.)
			d_xpos2 = -d_xpos2;
		xpos2 += d_xpos2;
	}
	if(qAbs(ypos2 + d_ypos2) > 20.)
	{
		d_ypos2 = -d_ypos2;
		cubesJoined = true;
	}
	ypos2 += d_ypos2;
	
	if((cubesJoined) && (ypos1 > 0.0))
	{
		d_ypos1 = 0.0;
		d_xrot1 = 0.0;
		d_yrot1 = 0.0;
		d_zrot1 = 0.0;
		QTimer* timer = new QTimer(this);
		connect(timer, SIGNAL(timeout()), this, SLOT(finish()));
		timer->start(1000);
		
	}
	
	if(((bkgAplpha + d_bkgAlpha) > 320.) || ((bkgAplpha + d_bkgAlpha) < 0.))
		d_bkgAlpha = -d_bkgAlpha;
	bkgAplpha += d_bkgAlpha;
	
	updateGL();
}
// -----------------------------------------------------------------------------

/**
 * Metoda reagujaca na wcisniecie klawisza na klawiaturze
 * @param QKeyEvent* event - obiekt rzechwoujacy parametry zdarzenia
 * @returns Brak zwracanych wartosci
*/
void GLStartWidget::keyPressEvent(QKeyEvent* event)
{
	float drot1 = 1.;

	if(event->key() == Qt::Key_Z)
		tmpVar -= drot1;
	if(event->key() == Qt::Key_X)
		tmpVar += drot1;
	if(event->key() == Qt::Key_M)
		QMessageBox ::question(NULL, "XTT Editor", "tmpVar = " + QString::number(tmpVar));
}
// -----------------------------------------------------------------------------

/**
 * Metoda tworzaca listy polecen OpenGL
 * @returns Brak zwracanych wartosci
*/
void GLStartWidget::buildLists(void)
{
	// Szeroksoc sciany
	float ws = WALL_SIZE / 2.;
	QColor wallColor(255, 255, 255, 200);
	
	// Tworzenie cube
	cube1 = glGenLists(1);
	glNewList(cube1, GL_COMPILE);
	
		glBindTexture(GL_TEXTURE_2D, xttt);
		
		glBegin(GL_QUADS);
			// Przednia sciana
			qglColor(wallColor);	glTexCoord2f(0.0f, 0.0f);	glVertex3f(-ws, -ws, ws);
			qglColor(wallColor);	glTexCoord2f(0.0f, 1.0f);	glVertex3f(-ws, ws, ws);
			qglColor(wallColor);	glTexCoord2f(1.0f, 1.0f);	glVertex3f(ws, ws, ws);
			qglColor(wallColor);	glTexCoord2f(1.0f, 0.0f);	glVertex3f(ws, -ws, ws);
			
			// Tylnia sciana
			qglColor(wallColor);	glTexCoord2f(0.0f, 0.0f);	glVertex3f(ws, -ws, -ws);
			qglColor(wallColor);	glTexCoord2f(0.0f, 1.0f);	glVertex3f(ws, ws, -ws);
			qglColor(wallColor);	glTexCoord2f(1.0f, 1.0f);	glVertex3f(-ws, ws, -ws);
			qglColor(wallColor);	glTexCoord2f(1.0f, 0.0f);	glVertex3f(-ws, -ws, -ws);
			
			// Gorna sciana
			qglColor(wallColor);	glTexCoord2f(0.0f, 0.0f);	glVertex3f(-ws, ws, ws);
			qglColor(wallColor);	glTexCoord2f(0.0f, 1.0f);	glVertex3f(-ws, ws, -ws);
			qglColor(wallColor);	glTexCoord2f(1.0f, 1.0f);	glVertex3f(ws, ws, -ws);
			qglColor(wallColor);	glTexCoord2f(1.0f, 0.0f);	glVertex3f(ws, ws, ws);
			
			// Dolna sciana
			qglColor(wallColor);							glVertex3f(-ws, -ws, -ws);
			qglColor(wallColor);							glVertex3f(-ws, -ws, ws);
			qglColor(wallColor);							glVertex3f(ws, -ws, ws);
			qglColor(wallColor);							glVertex3f(ws, -ws, -ws);
			
			// Lewa sciana
			qglColor(wallColor);							glVertex3f(-ws, -ws, -ws);
			qglColor(wallColor);							glVertex3f(-ws, -ws, ws);
			qglColor(wallColor);							glVertex3f(-ws, ws, ws);
			qglColor(wallColor);							glVertex3f(-ws, ws, -ws);
			
			// Prawa sciana
			qglColor(wallColor);							glVertex3f(ws, -ws, -ws);
			qglColor(wallColor);							glVertex3f(ws, -ws, ws);
			qglColor(wallColor);							glVertex3f(ws, ws, ws);
			qglColor(wallColor);							glVertex3f(ws, ws, -ws);
		glEnd();
		
	
     glEndList();     
	
	
	cube2 = glGenLists(1);
	glNewList(cube2, GL_COMPILE);
		
		glBindTexture(GL_TEXTURE_2D, ardt);
		glBegin(GL_QUADS);
		
			// Przednia sciana
			qglColor(wallColor);							glVertex3f(-ws, -ws, ws);
			qglColor(wallColor);							glVertex3f(-ws, ws, ws);
			qglColor(wallColor);							glVertex3f(ws, ws, ws);
			qglColor(wallColor);							glVertex3f(ws, -ws, ws);
			
			// Tylnia sciana
			qglColor(wallColor);							glVertex3f(ws, -ws, -ws);
			qglColor(wallColor);							glVertex3f(ws, ws, -ws);
			qglColor(wallColor);							glVertex3f(-ws, ws, -ws);
			qglColor(wallColor);							glVertex3f(-ws, -ws, -ws);
			
			// Gorna sciana
			qglColor(wallColor);							glVertex3f(-ws, ws, ws);
			qglColor(wallColor);							glVertex3f(-ws, ws, -ws);
			qglColor(wallColor);							glVertex3f(ws, ws, -ws);
			qglColor(wallColor);							glVertex3f(ws, ws, ws);
			
			// Dolna sciana
			qglColor(wallColor);	glTexCoord2f(0.0f, 0.0f);	glVertex3f(-ws, -ws, -ws);
			qglColor(wallColor);	glTexCoord2f(0.0f, 1.0f);	glVertex3f(-ws, -ws, ws);
			qglColor(wallColor);	glTexCoord2f(1.0f, 1.0f);	glVertex3f(ws, -ws, ws);
			qglColor(wallColor);	glTexCoord2f(1.0f, 0.0f);	glVertex3f(ws, -ws, -ws);
			
			// Lewa sciana
			qglColor(wallColor);	glTexCoord2f(0.0f, 0.0f);	glVertex3f(-ws, -ws, -ws);
			qglColor(wallColor);	glTexCoord2f(0.0f, 1.0f);	glVertex3f(-ws, -ws, ws);
			qglColor(wallColor);	glTexCoord2f(1.0f, 1.0f);	glVertex3f(-ws, ws, ws);
			qglColor(wallColor);	glTexCoord2f(1.0f, 0.0f);	glVertex3f(-ws, ws, -ws);
			
			// Prawa sciana
			qglColor(wallColor);	glTexCoord2f(0.0f, 0.0f);	glVertex3f(ws, -ws, -ws);
			qglColor(wallColor);	glTexCoord2f(0.0f, 1.0f);	glVertex3f(ws, -ws, ws);
			qglColor(wallColor);	glTexCoord2f(1.0f, 1.0f);	glVertex3f(ws, ws, ws);
			qglColor(wallColor);	glTexCoord2f(1.0f, 0.0f);	glVertex3f(ws, ws, -ws);
		glEnd();
	
     glEndList();   
	
	
	wallColor = QColor(125, 125, 125, 255);
	cube3 = glGenLists(1);
	glNewList(cube3, GL_COMPILE);
	
		static GLfloat specref[] = {1.0f, 1.0f, 1.0f, 1.0f};
		glEnable(GL_COLOR_MATERIAL);
		glColorMaterial(GL_FRONT, GL_AMBIENT_AND_DIFFUSE);
		glMaterialfv(GL_FRONT, GL_SPECULAR, specref);
		glMateriali(GL_FRONT, GL_SHININESS, 128);
		
		glBindTexture(GL_TEXTURE_2D, ardt);
		glBegin(GL_QUADS);
		
			// Przednia sciana
			glNormal3f(0.0, 0.0, 1.0);
			qglColor(wallColor);	glTexCoord2f(0.0f, 0.0f);	glVertex3f(-ws, -ws, ws);
			qglColor(wallColor);	glTexCoord2f(0.0f, 1.0f);	glVertex3f(-ws, ws, ws);
			qglColor(wallColor);	glTexCoord2f(1.0f, 1.0f);	glVertex3f(ws, ws, ws);
			qglColor(wallColor);	glTexCoord2f(1.0f, 0.0f);	glVertex3f(ws, -ws, ws);
			
			// Tylnia sciana
			glNormal3f(0.0, 0.0, -1.0);
			qglColor(wallColor);	glTexCoord2f(0.0f, 0.0f);	glVertex3f(ws, -ws, -ws);
			qglColor(wallColor);	glTexCoord2f(0.0f, 1.0f);	glVertex3f(ws, ws, -ws);
			qglColor(wallColor);	glTexCoord2f(1.0f, 1.0f);	glVertex3f(-ws, ws, -ws);
			qglColor(wallColor);	glTexCoord2f(1.0f, 0.0f);	glVertex3f(-ws, -ws, -ws);
			
			// Gorna sciana
			glNormal3f(0.0, 1.0, 0.0);
			qglColor(wallColor);	glTexCoord2f(0.0f, 0.0f);	glVertex3f(-ws, ws, ws);
			qglColor(wallColor);	glTexCoord2f(0.0f, 1.0f);	glVertex3f(-ws, ws, -ws);
			qglColor(wallColor);	glTexCoord2f(1.0f, 1.0f);	glVertex3f(ws, ws, -ws);
			qglColor(wallColor);	glTexCoord2f(1.0f, 0.0f);	glVertex3f(ws, ws, ws);
		glEnd();
			
		glBindTexture(GL_TEXTURE_2D, xttt);
		glBegin(GL_QUADS);
		
			// Dolna sciana
			glNormal3f(0.0, -1.0, 0.0);
			qglColor(wallColor);	glTexCoord2f(0.0f, 0.0f);	glVertex3f(-ws, -ws, -ws);
			qglColor(wallColor);	glTexCoord2f(1.0f, 0.0f);	glVertex3f(ws, -ws, -ws);
			qglColor(wallColor);	glTexCoord2f(1.0f, 1.0f);	glVertex3f(ws, -ws, ws);
			qglColor(wallColor);	glTexCoord2f(0.0f, 1.0f);	glVertex3f(-ws, -ws, ws);
			
			// Lewa sciana
			glNormal3f(-1.0, 0.0, 0.0);
			qglColor(wallColor);	glTexCoord2f(0.0f, 0.0f);	glVertex3f(-ws, -ws, -ws);
			qglColor(wallColor);	glTexCoord2f(1.0f, 0.0f);	glVertex3f(-ws, ws, -ws);
			qglColor(wallColor);	glTexCoord2f(1.0f, 1.0f);	glVertex3f(-ws, ws, ws);
			qglColor(wallColor);	glTexCoord2f(0.0f, 1.0f);	glVertex3f(-ws, -ws, ws);
			
			// Prawa sciana
			glNormal3f(1.0, 0.0, 0.0);
			qglColor(wallColor);	glTexCoord2f(0.0f, 0.0f);	glVertex3f(ws, -ws, -ws);
			qglColor(wallColor);	glTexCoord2f(1.0f, 0.0f);	glVertex3f(ws, ws, -ws);
			qglColor(wallColor);	glTexCoord2f(1.0f, 1.0f);	glVertex3f(ws, ws, ws);
			qglColor(wallColor);	glTexCoord2f(0.0f, 1.0f);	glVertex3f(ws, -ws, ws);
		glEnd();
	
     glEndList();  
}
// -----------------------------------------------------------------------------

/**
 * Metoda ustawiajaca obszar kopiowania pulpitu
 * @param QRect area - obszar kopiowania pulpitu
 * @returns Brak zwracanych wartosci
*/
void GLStartWidget::setCopyArea(QRect area)
{
	fCopyArea = area;
}
// -----------------------------------------------------------------------------

/**
 * Konczy dzialanie okienka
 * @returns Brak zwracanych wartosci
*/
void GLStartWidget::finish(void)
{
	if(runAppSend)
		return;

	runAppSend = true;
     //disconnect(NULL, NULL, this, SLOT(renderScene()));
	emit runApp();
}
// -----------------------------------------------------------------------------

