 /**
 *                                                                             *
 * @file MainWin_ui.h                                                          *
 * Project name: HeKatE hqed                                                   *
 * $Id: MainWin_ui.h,v 1.19 2008-03-03 22:04:31 kinio Exp $                     *
 * Author: Krzysztof Kaczor                                                    *
 * Date: 24.04.2007                                                            *
 *                                                                             *
 * Implementation by Krzysztof Kaczor <kinio4444@gmail.com>                    *
 *                                                                             *
 * Copyright (C) 2006-9 by the HeKatE Project                                  *
 *                                                                             *
 * HQEd has been develped within the HeKatE Project,                           *
 * see http://hekate.ia.agh.edu.pl                                             *
 *                                                                             *
 * This file is part of HQEd.                                                  *
 *                                                                             *
 * HQEd is free software: you can redistribute it and/or modify                *
 * it under the terms of the GNU General Public License as published by        *
 * the Free Software Foundation, either version 3 of the License, or           *
 * (at your option) any later version.                                         *
 *                                                                             *
 * HQEd is distributed in the hope that it will be useful,                     *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU General Public License for more details.                                *
 *                                                                             *
 * You should have received a copy of the GNU General Public License           *
 * along with HQEd.  If not, see <http://www.gnu.org/licenses/>.               *
 *                                                                             *
 * Comment: Plik zwierajacy definicje klasy obslugujacej okienko glowne        *
 * programu. CONTROLLER                                                        *
 *                                                                             *
 */

#ifndef MAINWINUI_H
#define MAINWINUI_H
// -----------------------------------------------------------------------------

#include <QDomDocument>

#include "../ui_MainWin.h"
// -----------------------------------------------------------------------------

#define ARD_MODE 0
#define XTT_MODE 1
// -----------------------------------------------------------------------------

class QDial;
class QMenu;
class QLabel;
class QSpinBox;
class QCheckBox;
class QListWidget;
class QGridLayout;
class QSpacerItem;
class QGraphicsView;
class QGraphicsScene;

class XTT_Table;

class sysTrayObject;
class MyGraphicsView;
class MyGraphicsScene;
class ardGraphicsView;
class ardGraphicsScene;
class ErrorsListWidget;
class ExecMsgListWidget;
// -----------------------------------------------------------------------------

class MainWin_ui : public QMainWindow, private Ui_MainWindow
{
    Q_OBJECT

private:
	
	/// Drukarka
	QPrinter* printer;
	
	/// Menu
	QMenu* toolBarMenu;
	QMenu* dockedWindows;

	/// Paski narzedzi
	QToolBar* fileToolBar;
     QToolBar* viewToolBar;
	QToolBar* tableToolBar;
	QToolBar* attributesToolBar;
	QToolBar* connectionsToolBar;
	QToolBar* executeToolBar;
	QToolBar* toolsToolBar;
	QToolBar* helpToolBar;
	
	/// Element ktory pozwala na rotacje sceny
	QDial* dial;
	
	/// Edit w ktorym wpisujemy aktualna wartosc rotacji sceny
	QSpinBox* rotateAngleEdit;
	
	/// Wskaznik do jednej z powyzszych, okresla ktora scena byla ostatni aktualna, lub jest aktualna
	ardGraphicsScene* fCurrentARDscene;
	
	/// Okresla czy pokazywac wskazowki dla ARD przy przelaczeniu trybu na ARD
	bool fShowARDtip;
	
	/// Okresla czy pokazywac wskazowki dla XTT przy przelaczeniu trybu na XTT
	bool fShowXTTtip;
	
	/// Okresla w jakim trybie pracuje program (0-edytor ARD, 1-edytor XTT)
	int appMode;

	/// Nazwa aktualnie otwartego pliku ARD
     QString fArdFileName;
	
     /// Nazwa aktualnie otwartego pliku XTT
     QString fXttFileName;

     /// Okresla czy aktualny prjekt jest plikiem
     bool isARDFile;
	
	/// Okresla czy aktualny projekt jest plikiem
     bool isXTTFile;
	
	/// Wywolywana w momencie zamkniecie okienka
	void closeEvent(QCloseEvent*);
	
	/// Tworzy okna dokowalne ARD
	void ard_createDockWindows(void);
	
	/// Tworzy okna dokowalne XTT
	void xtt_createDockWindows(void);
	
	/// Metoda tworzaca pasek przyciskow
	void xtt_createToolBars(void);
	
	/// Metoda tworzaca pasek przyciskow
	void xtt_createMenu(void);
	
	/// Tworzy pasek statusu
	void createStatusBar(void);
 
public:

	// ----------------- Obiekty -----------------
	
	/// Ekran dla sceny
	MyGraphicsView* xtt_graphicsView;
	ardGraphicsView* ard_modelView;
	ardGraphicsView* ard_tphView;
	
     /// Scena rysowania
     MyGraphicsScene* xtt_scene;
	ardGraphicsScene* ard_modelScene;
	ardGraphicsScene* ard_thpScene;
	
	
	/// Okna dokowalne
	QDockWidget* ard_tphDockWindow;
	QDockWidget* xtt_errorsDockWindow;
	QDockWidget* xtt_runModelMessagesDockWindow;
	
	/// Lista bledow wyswietlana w oknie dokowalnym bledow komorek
	ErrorsListWidget* xtt_errorsList;
	
	/// Lista obiektow okna dokowalnego bledow uruchomienia modelu
	QWidget* xtt_runModelMessagesDockWindowContents;
	QGridLayout* xtt_runModelMessagesDockWindowGridLayout;
	ExecMsgListWidget* xtt_runModelMessagesDockWindowMessagesList;
	QSpacerItem* xtt_runModelMessagesDockWindowSpacerItem;
	QCheckBox* xtt_runModelMessagesDockWindowInfoCheckBox;
	QCheckBox* xtt_runModelMessagesDockWindowWarningCheckBox;
	QCheckBox* xtt_runModelMessagesDockWindowErrorCheckBox;
	QLabel* xtt_runModelMessagesDockWindowLabel;
	
	/// Obiekt wyswietlajacy w zasobniku systemowym
	sysTrayObject* trayIcon;
	
	// ----------------- Metody -----------------
	
     /// Konstruktor klasy
     MainWin_ui(QMainWindow *parent = 0);
	
	/// Destruktor okienka
	~MainWin_ui(void);
	
	/// Zwraca tryb pracy aplikacji
	int currentMode(void);
	
	/// Ustawia bierzacy tryb pracy aplikacji
	void setCurrentMode(int);

     /// Wywolywana w momencie nacisniecia przycisku naklawiaturze
     void keyPressEvent(QKeyEvent*);
	
	/// Ustawia aktualna scene ARD
	void setCurrentARDscene(ardGraphicsScene*);
	
	/// Zwraca wskaznik do aktualnej sceny ARD
	ardGraphicsScene* currentARDscene(void) { return fCurrentARDscene; }

	/// Sprawdza modyfiakcje modelu ARD
	bool ARD_Changed(void);
	
	/// Sprawdza modyfiakcje modelu XTT
	bool XTT_Changed(void);
	
	/// Ustawia status modyfiakcji modelu ARD
	void setARD_Changed(bool);
	
	/// Ustawia status modyfiakcji modelu XTT
	void setXTT_Changed(bool);
	
     /// Tworzy nowy projekt XTT
     int CreateNewXTTProject(bool);
	
	/// Tworzy nowy projekt ARD
     int CreateNewARDProject(void);

     /// Uruchamia analize poszczegolnych wierszy
     int ExecuteRow(QList<XTT_Table*>*, XTT_Table*, bool*, QList<unsigned int>*, QList<QList<XTT_Table*>*>*, int, int, int, int);

     /// Okresla czy widok modelu ma byc czarno bialy
     bool ModelBW(void){ return actionB_W_view->isChecked(); }
	
	/// Zwraca czy wlaczony jest antialiasing
     bool antialiasingStatus(void){ return actionAntialiasing->isChecked(); }
	
	/// Zwraca czy wlaczony jest antialiasing dla tekstu
     bool textAntialiasingStatus(void){ return actionTextAntialiasing->isChecked(); }
	
	/// Zwraca czy wlaczona jest gladka transformacja bitmap
     bool smoothPixmapTransformStatus(void){ return actionSmooth_pixmap_transform->isChecked(); }
	
	/// Zwraca czy nalezy pokazac wskazowke ARD
	bool showARDtip(void) { return fShowARDtip; }
	
	/// Zwraca czy nalezy pokazac wskazowke XTT
	bool showXTTtip(void) { return fShowXTTtip; }
	
	/// Ustawia wartosc czy nalezy pokazac wskazowke ARD
	void setShowARDtip(bool _v) { fShowARDtip = _v; }
	
	/// Ustawia wartosc czy nalezy pokazac wskazowke ARD
	void setShowXTTtip(bool _v) { fShowXTTtip = _v; }
	
	/// Metoda wobslugujaca zdarzenie upuszczania pliku na obszar okienka
	void dragEnterEvent(QDragEnterEvent*);
	
	/// Metoda wobslugujaca zdarzenie upuszczania pliku na obszar okienka
	void dropEvent(QDropEvent*);
	
	/// Metoda wyszukujaca liste tabel inicjalizacyjnych dla kazdej tabeli
	void findInitialTables(XTT_Table*, QList<unsigned int>*, QList<QList<XTT_Table*>*>*, unsigned int);


public slots:

	/// Metoda wywolywana w momencie pokazania okienka
	void showEvent(QShowEvent*);

     /// Uruchamia model
     void ExecuteModel(void);

	/// Uruchamia okno dialogowe otwierania pliku ARD
     void openARDFileDialog(void);
	
     /// Uruchamia okno dialogowe otwierania pliku XTT
     void openXTTFileDialog(void);
	
	/// Otwiera plik
	void openFile(QString);
	
	/// Otwiera plik ATTML
	void openATTMLFile(void);

     /// Ustala nazwe pliku
     void saveAs(int);
	
	/// Ustawia zapis na format XTTML 1.0
	void saveAs_xttml_1_0(void);
	
	/// Ustawia zapis na format XTTML 2.0
	void saveAs_xttml_2_0(void);

     /// Zapisuje do pliku w formacie XTTML 1.0
     void saveToXTTML1_0(void);
	
	/// Zapisuje do pliku w formacie XTTML 2.0
     void saveToXTTML2_0(void);
	
	/// Zapisuje grupy i atrybuty do pliku w formacie ATTML 1.0
	void saveToATTML1_0(void);
	
	/// Pozwala na zapis sceny jako obraz typu svg
	void exportSceneToSVGfile(void);
	
	/// Pozwala na zapis sceny jako obraz typu BMP
	void exportSceneToBMPfile(void);
	
	/// Pozwala na zapis sceny jako obraz typu JPG
	void exportSceneToJPGfile(void);
	
	/// Pozwala na zapis sceny jako obraz typu PNG
	void exportSceneToPNGfile(void);
	
	/// Pozwala na zapis sceny jako obraz typu PPM
	void exportSceneToPPMfile(void);
	
	/// Pozwala na zapis sceny jako obraz typu XBM
	void exportSceneToXBMfile(void);
	
	/// Pozwala na zapis sceny jako obraz typu XPM
	void exportSceneToXPMfile(void);
	
	/// Pozwala na zapis sceny jako obraz
	void exportSceneAsPicture(QString);
	
	/// Pozwala na zapis sceny jako obraz
	void exportTPHasPicture(QString);
	
	/// Pozwala na zapis sceny jako obraz typu svg
	void exportTPHtoSVGfile(void);
	
	/// Pozwala na zapis sceny jako obraz typu BMP
	void exportTPHtoBMPfile(void);
	
	/// Pozwala na zapis sceny jako obraz typu JPG
	void exportTPHtoJPGfile(void);
	
	/// Pozwala na zapis sceny jako obraz typu PNG
	void exportTPHtoPNGfile(void);
	
	/// Pozwala na zapis sceny jako obraz typu PPM
	void exportTPHtoPPMfile(void);
	
	/// Pozwala na zapis sceny jako obraz typu XBM
	void exportTPHtoXBMfile(void);
	
	/// Pozwala na zapis sceny jako obraz typu XPM
	void exportTPHtoXPMfile(void);

     /// Wczytuje dane z pliku XTTML ver 1.0
     void loadFileXTTML1_0(QDomElement&);
	
	/// Wczytuje dane z pliku XTTML ver 2.0
	void loadFileXTTML2_0(QDomElement&);
	
	/// Wczytuje dane o grupach i atrybutach z pliku ATTML 1.0
	void loadFileATTML1_0(QDomElement&);
	
	/// Wczytuje dane z pliku ARDML
	void loadFileARDML1_0(QDomElement&);

     /// Tworzy nowy projekt z tymi samymi atrybutami
     void NewWhithOldAttributiesClick(void);

     /// Tworzy nowy projekt
     void NewClick(void);
	
	/// Przelacza palikacje w tryb edycji ARD
	void useARDmode(void);
	
	/// Przelacza palikacje w tryb edycji XTT
	void useXTTmode(void);

     /// Przelacza widok modelu z kolorowego na czarnobialy i na odwrot
     void SwitchColorMode(void);
	
	/// Resetuje obrot do 0 st
	void resetRotate(void);

     /// Pokazuje okno dodawania nowego atrybutu
     void AddNewAttribute(void);

     /// Pokazuje okno edycji atrybutu
     void EditAttribute(void);

     /// Pokazuje okno usuwania atrybutu
     void DeleteAttribute(void);

     /// Pokazuje okno menedzera grup
     void GroupManagerShow(void);

     /// Pokazuje okno menedzera atrybutow
     void AttributeManagerShow(void);

     /// Pokazuje okno edytora tabel w trybie dodawania nowej tabeli
     void TableEditorShow(void);

     /// Pokazuje okno edytora tabel w trybie edycji
     void TableEditorEdit(void);

     /// Pokazuje okno do usuwania tabel
     void DeleteTableShow(void);

     /// Pokazuje okienko edytora komorek
     void ShowCellEditor(void);

     /// Pokazuje okienko edytora polaczen
     void ShowConnEditor(void);

     /// Pokazuje okienko edytora polaczen w trybie edycji
     void ShowConnEditorInEditMode(void);

     /// Pokazuje okno usuwania polaczenia
     void ShowConnectionDeleteWindow(void);

     /// Pokazuje okienko menedzera polaczen
     void ShowConnectionManager(void);
	
	/// Wywolywana w momencie zmiany kata rotacji w polu edycji
	void angleEditorChange(int);

     /// Metoda uaktuaniajaca renderowanie scenys
     void UpdateRenderHints(void);

     /// Metoda zalanczajaca/wylanczajaca antyaliasing na scenie
     void ChangeAntialiasingStatus(void);

     /// Metoda konczaca dzialanie aplikacji
     void CloseForm(void);
	
	/// Drukuje model
	void printModel(void);
	
	/// Wywoluje ustawienia drukarki
	void printSetup(void);

     /// Wyswietla liste nieuzywanych atrybutow
     void ShowUnusedAttributes(void);

     /// Pokazuje info o programie
     void ShowAboutWin(void);
	
	/// Ustawia tryb kolorowy/czarnobialy
	void setColorMode(bool);
	
	/// Ustawia status antyaliasingu
	void setAntialiasingStatus(bool);
	
	/// Ustawia status antyaliasingu dla tekstu
	void setTextAntialiasingStatus(bool);
	
	/// Ustawia status gladkiej transformacji bitmap
	void setSmoothPixmapTransformStatus(bool);
	
	/// Usuwa polaczenie o podanym ID
	void deleteConnection(unsigned int);
	
	/// Usuwa tabele o podanym ID
	void deleteTable(unsigned int);
	
	/// Usuwa zaznaczone polaczenie
	void deleteSelectedConnection(void);
	
	/// Usuwa zaznaczona tabele
	void deleteSelectedTable(void);
	
	/// Pokazuje okienka ustawien
	void showConfigurationDialog(void);
};
// -----------------------------------------------------------------------------

/// Deklaracja deklaracji :) okienka glownego aplikacji
extern MainWin_ui* MainWin;

/// Funckja wczytujaca z lini parametrow XTTML wartosc o danej nazwie
QString XML_ReadValue(QString, QString);

/// Usuwa spacje na poczatku i koncu stringu
QString RemoveSpaceAtEnds(QString);
// -----------------------------------------------------------------------------

#endif
