 /**
 *                                                                             *
 * @file Settings_ui.h                                                         *
 * Project name: HeKatE hqed                                                   *
 * $Id: Settings_ui.h,v 1.15 2008-03-03 22:04:31 kinio Exp $                    *
 * Author: Krzysztof Kaczor                                                    *
 * Date: 06.11.2007                                                            *
 *                                                                             *
 * Implementation by Krzysztof Kaczor <kinio4444@gmail.com>                    *
 *                                                                             *
 * Copyright (C) 2006-9 by the HeKatE Project                                  *
 *                                                                             *
 * HQEd has been develped within the HeKatE Project,                           *
 * see http://hekate.ia.agh.edu.pl                                             *
 *                                                                             *
 * This file is part of HQEd.                                                  *
 *                                                                             *
 * HQEd is free software: you can redistribute it and/or modify                *
 * it under the terms of the GNU General Public License as published by        *
 * the Free Software Foundation, either version 3 of the License, or           *
 * (at your option) any later version.                                         *
 *                                                                             *
 * HQEd is distributed in the hope that it will be useful,                     *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU General Public License for more details.                                *
 *                                                                             *
 * You should have received a copy of the GNU General Public License           *
 * along with HQEd.  If not, see <http://www.gnu.org/licenses/>.               *
 *                                                                             *
 * Comment: Plik zawierajacy definicje klasy oblugujacej okno konfiguracji     *
 *                                                                             *
 */


#ifndef SettingsH
#define SettingsH
// -----------------------------------------------------------------------------

#include "../ui_Settings.h"
// -----------------------------------------------------------------------------

class QSettings;
// -----------------------------------------------------------------------------

class Settings_ui : public QWidget, private Ui_Settings
{
    Q_OBJECT
    
private:

	/// Konfiguracja aplikacji
	QSettings* conf;
     
public:

     /// Konstruktory klasy
     Settings_ui(QWidget *parent = 0);
	
	/// Metody zwracajaca wartosci opcji
	
	/// Zwraca czy nalzey pokazac ekran atartowy
	bool useStartScreen(void);
	
	/// Zwraca typ tla dla ekranu startowego
	bool startScreenBackgroundType(void);
	
	/// Zwraca szybkosc animacji w procentach w oknie startowym
	int startScreenAnimationSpeed(void);
	
	/// Zwraca kiedy trzeba pokazywac paski przewijania
	Qt::ScrollBarPolicy sceneScorllBarPolicy(void);
	
	/// Zwraca wartosc wspolczynnika powiekszenia/pomniejszenia sceny
	double zoomFactorValue(int);
	
	/// Zwraca czy nalezy uzyc obramowanych wezlow ARD i TPH
	bool useFramedNodes(void);
	
	/// Zwraca czy nalezy uzyc kolka myszki do zoomowania
	bool useZoomMouseWhell(int);
	
	/// Zwraca czy nalezy tworzyc automatycznie nazwe akronimu
	bool xttCreateAutoAcronym(void);
	
	/// Zwraca czy nalezy tworzyc automatyczna nazwe tabeli
	bool xttCreateTableAutoName(void);
	
	/// Zwraca czy nalezy pokazac widomosc o bledzie komorki
	bool xttShowErrorCellMessage(void);
	
	/// Zwraca czy nalezy podswietlic komorke z bledem
	bool xttHighlightErrorCell(void);
	
	/// Zwraca czy nalezy sprawdzac proste (nie bedace wrazeniami artymetycznymi) wartosci w komorce
	bool xttCheckAtomicValues(void);
	
	/// Zwraca czy nalezy uruchomic komorke podczas analizy kiedy wylaczony jest attr to attr comp
	bool xttExecuteCellWhenOff(void);
	
	/// Zwraca czy nalezy uruchomic komorke podczas analizy kiedy wlaczony jest attr to attr comp
	bool xttExecuteCellWhenOn(void);
	
	/// Zwraca czy nalezy wstawic tresc bledu do hint'a komorki
	bool xttShowErrorInToolTip(void);
	
	/// Zwraca domyslna wartosc wlaczonego porownywania atrubutow w komorce
	bool xttAttrToAttrComp(void);
	
	/// Zwraca czy nalezy samodzielnie rozpoznawac typ tabeli
	bool autoTableTypeRecognition(void);
	
	/// Zwraca czy nalezy pokazac ostrzezenie o niezgodnym typie tabeli
	bool messageAboutAmbiguousTableType(void);
	
	/// Zwraca czy nalezy pokazac ostrzezenie o niedostpnych wierszach
	bool massageAboutInaccesibleRow(void);
	
	/// Zwraca czy nalezy wyswietli mape rodzicow tabeli
	bool createMapMovement(void);
	
	/// Zwraca czy nalezy podswietlac komorke ktora jest aktualnie odapalana
	bool highlightCurrentCell(void);
	
	/// Zwraca czy nalezy po zakonczeniu analizy dac sygnal dzwiekowy
	bool beepOnFinish(void);
	
	/// Zwraca czy nalezy pokazac informacje o uruchomieniu komorki
	bool showRunCellStatus(void);
	
	/// Zwraca czy pokazac ikone w zasobniku systemowym
	bool showSystemTrayIcon(void);
	
	/// Zwraca czy pokazywac wskazowki
	bool showTrayTips(void);
	
	/// Zwraca czy wlaczone jest autocentrowanie elementow
	bool xttEnabledAutocentering(void);
	
	/// Zwraca czy scena modelu xtt powinna byc automatycznie powiekszana
	bool xttAutoSceneResize(void);
	
	// Zwraca czy pliki XML powinny byc formatowane w taki sposob aby mozna bylo zobaczyc ich hierarchie
	bool xmlAutoformating(void);
	
	/// Zwraca podstawe nazwy automatycznej dla tabeli
	QString xttTableBaseName(void);
	
	/// Zwraca string ktory jest wstawiany zamiast spacji w wartosichc atrybutu
	QString spaceReplacer(void);
	
	/// Zwraca domyslna szerokosc kolumny
	int xttDefaultColumnWidth(void);
	
	/// Zwraca domyslna wysokosc wiersza
	int xttDefaultRowHeight(void);
	
	/// Zwraca domyslna wysokosc naglowka
	int xttDefaultHeaderHeight(void);
	
	/// Okresla odelglosc o ile maja byc przesuwane elementy na scenie przy kliknieciu na przycisk przesuniecia
	int ardMoveElementsDistance(void);
	
	/// Okresla odelglosc o ile maja byc przesuwane elementy na scenie przy kliknieciu na przycisk przesuniecia
	int xttMoveElementsDistance(void);
	
	/// Zwraca odleglosc pomiedzy polaczeniami wychodzacymi z jednej tabeli
	int xttConnectionsDistance(void);
	
	/// Zwraca procent kompresji plikow graficznych bedacych screenami sceny modelu xtt
	int ardImageCompression(void);
	
	/// Zwraca procent kompresji plikow graficznych bedacych screenami sceny modelu xtt
	int xttImageCompression(void);
	
	/// Kolor tabeli
	QColor xttTableColor(void);
	
	/// Kolor zaznaczonej tabeli
	QColor xttSelectedTableColor(void);
	
	/// Kolor polaczenia
	QColor xttConnectionColor(void);
	
	/// Kolor zaznaczonego polaczenia
	QColor xttSelectedConnectionColor(void);
	
	/// Kolor polaczenia odcinajacego
	QColor xttCutConnectionColor(void);
	
	/// Kolor czcionki tabeli
	QColor xttTableFontColor(void);
	
	/// Kolor czcionki tytulu tabeli
	QColor xtTableTitleFontColor(void);
	
	/// Kolor czcionki polaczenia
	QColor xttConnectionLabelColor(void);
	
	/// Kolor tla sceny
	QColor xttSceneBackgroundColor(void);
	
	/// Kolor podswietlonej komorki
	QColor xttHoveredCellColor(void);
	
	/// Kolor komorki z bledem
	QColor xttErrorCellColor(void);
	
	/// Kolor komorki z bledem
	QColor xttTableBorderColor(void);
	
	/// Kolor zwyklego wezla TPH
	QColor ardTPHnodeColor(void);
	
	/// Kolor zaznaczonego/aktywnego wezla TPH
	QColor ardTPHselectedNodeColor(void);
	
	/// Kolor nietransformowalnego wezla TPH
	QColor ardTPHnotTransformatedNodeColor(void);
	
	/// Kolor czcionki wezla TPH
	QColor ardTPHfontColor(void);
	
	/// Kolor galezi (polaczenia) drzewa TPH
	QColor ardTPHbranchColor(void);
	
	/// Kolor sceny TPH
	QColor ardTPHsceneColor(void);
	
	/// Kolor wlasnosci ARD
	QColor ardPropertyColor(void);
	
	/// Kolor aktywnej wlasnosci ARD
	QColor ardSelectedPropertyColor(void);
	
	/// Kolor czcionki wlasnosci ARD
	QColor ardPropertyFontColor(void);
	
	/// Kolor czcionki etykiety poziomu
	QColor ardLevelLabelFontColor(void);
	
	/// Kolor linii zaleznosci ARD
	QColor ardDependencyColor(void);
	
	/// Kolor sceny ARD
	QColor ardSceneColor(void);
	
	/// Kolor marginesow ARD
	QColor ardMarginsColor(void);
	
	/// Kolor komorki z bledem
	QColor xttTableHeaderFontColor(void);
	
	/// Metoda zwracajaca wartosc czcionki okreslonego elementu
	QFont xttTableFont(void);
	
	/// Metoda zwracajaca wartosc czcionki okreslonego elementu
	QFont xttTableHeaderFont(void);
	
	/// Metoda zwracajaca wartosc czcionki okreslonego elementu
	QFont xttTableTitleFont(void);
	
	/// Metoda zwracajaca wartosc czcionki okreslonego elementu
	QFont xttConnectionLabelFont(void);
	
	/// Metoda zwracajaca wartosc czcionki okreslonego elementu
	QFont ardTPHFont(void);
	
	/// Metoda zwracajaca wartosc czcionki okreslonego elementu
	QFont ardARDdiagramFont(void);
	
	/// Metoda stosujaca ustawienia do aplikacji
	void applyOptions(void);
	
	/// Metoda zapisujaca ustawienia aplikacji
	void saveOptions(void);
	
	/// Metoda wczytujaca ustawienia aplikacji
	void loadOptions(void);
	
	/// Metoda pozwalajaca na stawienie opcji trybu kolorowego
	void setColorMode(bool, int);
	
	/// Metoda pozwalajaca na stawienie opcji trybu antyaliasingu
	void setAntialiasingMode(bool, int);
	
	/// Metoda pozwalajaca na stawienie opcji trybu antyaliasingu dla tekstu
	void setTextAntialiasingMode(bool, int);
	
	/// Metoda pozwalajaca na stawienie opcji trybu transformacji map bitowych
	void setSmoothPixmapTransformMode(bool, int);
	
	/// Zwraca tryb kolorow w danym trybie pracy aplikacji
	bool colorMode(int);
	
	/// Zwraca czy nalezy stosowac antyaliasing
	bool useAntialiasing(int);
	
	/// Zwraca czy nalezy stosowac antyaliasing dla tekstu
	bool useTextAntialiasing(int);
	
	/// Zwraca czy nalezy stosowac plynna transformacje bitmap
	bool useSmoothPixmapTransform(int);

public slots:

	/// Metoda wywolywana w momencie zmiany tresc pola edycji znaku zastepujacego spacje w wartosciach atrybutu
	void spaceCharacterChange(QString);

	/// Metoda wywolywana w momencie zmiany stanu checkBox-a odpowiedzialnego za pozwolenie n aautomatyczne
	// tworzenie nazwy tabeli
	void checkBox9changed(bool);
	
	/// Metoda wywolywana w momencie zmiany domyslnego statusu attr2attr comp
	void checkBox11changed(bool);
	
	/// Metoda wywolywana w momencie zmiany automatycznego rozpoznawania typu tabeli
	void checkBox15changed(bool);

	/// Metoda wywolywana w momencie zmiany wartosci faktora zoom
	void changeArdZoomFactorSpinBox(double);
	
	/// Metoda wywolywana w momencie zmiany wartosci faktora zoom
	void changeArdZoomFactorSliderBar(int);
	
	/// Metoda wywolywana w momencie zmiany wartosci faktora zoom
	void changeXttZoomFactorSpinBox(double);
	
	/// Metoda wywolywana w momencie zmiany wartosci faktora zoom
	void changeXttZoomFactorSliderBar(int);

	/// Metoda wywolywana w momencie klikniecia przycisku zmiany czcionki
	void changeTableFont(void);
	
	/// Metoda wywolywana w momencie klikniecia przycisku zmiany czcionki
	void changeTableHeaderFont(void);
	
	/// Metoda wywolywana w momencie klikniecia przycisku zmiany czcionki
	void changeTableTitleFont(void);
	
	/// Metoda wywolywana w momencie klikniecia przycisku zmiany czcionki
	void changeConnectionLabelFont(void);
	
	/// Metoda wywolywana w momencie klikniecia przycisku zmiany czcionki
	void changeTPHFont(void);
	
	/// Metoda wywolywana w momencie klikniecia przycisku zmiany czcionki
	void changeARDdiagramFont(void);

	/// Metoda wywolywana w momencie klikniecia przycisku zastosuj
	void applyButtonClick(void);
	
	/// Metoda wywolywana w momencie klikniecia przycisku Zamknij
	void closeButtonClick(void);

	/// Metoda wywolywana w momencie zmiany zaladki
	void currentTabChanged(int);
	
	/// Metody wywolywana w momencie klikniecia przycisku koloru
	void onSelectColor1(void);
	
	/// Metody wywolywana w momencie klikniecia przycisku koloru
	void onSelectColor2(void);
	
	/// Metody wywolywana w momencie klikniecia przycisku koloru
	void onSelectColor3(void);
	
	/// Metody wywolywana w momencie klikniecia przycisku koloru
	void onSelectColor4(void);
	
	/// Metody wywolywana w momencie klikniecia przycisku koloru
	void onSelectColor5(void);
	
	/// Metody wywolywana w momencie klikniecia przycisku koloru
	void onSelectColor6(void);
	
	/// Metody wywolywana w momencie klikniecia przycisku koloru
	void onSelectColor7(void);
	
	/// Metody wywolywana w momencie klikniecia przycisku koloru
	void onSelectColor8(void);
	
	/// Metody wywolywana w momencie klikniecia przycisku koloru
	void onSelectColor9(void);
	
	/// Metody wywolywana w momencie klikniecia przycisku koloru
	void onSelectColor10(void);
	
	/// Metody wywolywana w momencie klikniecia przycisku koloru
	void onSelectColor11(void);
	
	/// Metody wywolywana w momencie klikniecia przycisku koloru
	void onSelectColor12(void);
	
	/// Metody wywolywana w momencie klikniecia przycisku koloru
	void onSelectColor13(void);
	
	/// Metody wywolywana w momencie klikniecia przycisku koloru
	void onSelectColor14(void);
	
	/// Metody wywolywana w momencie klikniecia przycisku koloru
	void onSelectColor15(void);
	
	/// Metody wywolywana w momencie klikniecia przycisku koloru
	void onSelectColor16(void);
	
	/// Metody wywolywana w momencie klikniecia przycisku koloru
	void onSelectColor17(void);
	
	/// Metody wywolywana w momencie klikniecia przycisku koloru
	void onSelectColor18(void);
	
	/// Metody wywolywana w momencie klikniecia przycisku koloru
	void onSelectColor19(void);
	
	/// Metody wywolywana w momencie klikniecia przycisku koloru
	void onSelectColor20(void);
	
	/// Metody wywolywana w momencie klikniecia przycisku koloru
	void onSelectColor21(void);
	
	/// Metody wywolywana w momencie klikniecia przycisku koloru
	void onSelectColor22(void);
	
	/// Metody wywolywana w momencie klikniecia przycisku koloru
	void onSelectColor23(void);
	
	/// Metody wywolywana w momencie klikniecia przycisku koloru
	void onSelectColor24(void);
	
	/// Metody wywolywana w momencie klikniecia przycisku koloru
	void onSelectColor25(void);
	
	/// Metody wywolywana w momencie klikniecia przycisku koloru
	void onSelectColor26(void);
	
	/// Metody wywolywane w momencie przesuniecia suwaka przezroczystosci
	void onSlider1ChangePosition(int);
	
	/// Metody wywolywane w momencie przesuniecia suwaka przezroczystosci
	void onSlider2ChangePosition(int);
	
	/// Metody wywolywane w momencie przesuniecia suwaka przezroczystosci
	void onSlider3ChangePosition(int);
	
	/// Metody wywolywane w momencie przesuniecia suwaka przezroczystosci
	void onSlider4ChangePosition(int);
	
	/// Metody wywolywane w momencie przesuniecia suwaka przezroczystosci
	void onSlider5ChangePosition(int);
	
	/// Metody wywolywane w momencie przesuniecia suwaka przezroczystosci
	void onSlider6ChangePosition(int);
	
	/// Metody wywolywane w momencie przesuniecia suwaka przezroczystosci
	void onSlider7ChangePosition(int);
	
	/// Metody wywolywane w momencie przesuniecia suwaka przezroczystosci
	void onSlider8ChangePosition(int);
	
	/// Metody wywolywane w momencie przesuniecia suwaka przezroczystosci
	void onSlider9ChangePosition(int);
	
	/// Metody wywolywane w momencie przesuniecia suwaka przezroczystosci
	void onSlider10ChangePosition(int);
	
	/// Metody wywolywane w momencie przesuniecia suwaka przezroczystosci
	void onSlider11ChangePosition(int);
	
	/// Metody wywolywane w momencie przesuniecia suwaka przezroczystosci
	void onSlider12ChangePosition(int);
	
	/// Metody wywolywane w momencie przesuniecia suwaka przezroczystosci
	void onSlider13ChangePosition(int);
	
	/// Metody wywolywane w momencie przesuniecia suwaka kompresji obrazow
	void onSlider15ChangePosition(int);
	
	/// Metody wywolywane w momencie przesuniecia suwaka kompresji obrazow
	void onSlider16ChangePosition(int);
	
	/// Metody wywolywane w momencie przesuniecia suwaka przezroczystosci
	void onSlider17ChangePosition(int);
	
	/// Metody wywolywane w momencie przesuniecia suwaka przezroczystosci
	void onSlider18ChangePosition(int);
	
	/// Metody wywolywane w momencie przesuniecia suwaka przezroczystosci
	void onSlider19ChangePosition(int);
	
	/// Metody wywolywane w momencie przesuniecia suwaka przezroczystosci
	void onSlider20ChangePosition(int);
	
	/// Metody wywolywane w momencie przesuniecia suwaka przezroczystosci
	void onSlider21ChangePosition(int);
	
	/// Metody wywolywane w momencie przesuniecia suwaka przezroczystosci
	void onSlider22ChangePosition(int);
	
	/// Metody wywolywane w momencie przesuniecia suwaka przezroczystosci
	void onSlider23ChangePosition(int);
	
	/// Metody wywolywane w momencie przesuniecia suwaka przezroczystosci
	void onSlider24ChangePosition(int);
	
	/// Metody wywolywane w momencie przesuniecia suwaka przezroczystosci
	void onSlider25ChangePosition(int);
	
	/// Metody wywolywane w momencie przesuniecia suwaka przezroczystosci
	void onSlider26ChangePosition(int);
	
	/// Metody wywolywane w momencie przesuniecia suwaka przezroczystosci
	void onSlider27ChangePosition(int);
	
	/// Metody wywolywane w momencie przesuniecia suwaka przezroczystosci
	void onSlider28ChangePosition(int);
	
	/// Metody wywolywane w momencie przesuniecia suwaka przezroczystosci
	void onSlider29ChangePosition(int);
	
	/// Metoda wywolywana w momencie zmiany statusu pokazywania ekranu startowego
	void onCheckBox28checked(bool);
	
	/// Metoda wywolywana w momencie zmiany statusu pokazywania ikony w zasobniku systemowym
	void onCheckBox29checked(bool);
};
// -----------------------------------------------------------------------------

extern Settings_ui* Settings;
// -----------------------------------------------------------------------------

#endif
