 /**
 *                                                                             *
 * @file AttrEditor_ui.cpp                                                     *
 * Project name: HeKatE hqed                                                   *
 * $Id: AttributeEditor_ui.cpp,v 1.12 2008-03-03 22:04:31 kinio Exp $           *
 * Author: Krzysztof Kaczor                                                    *
 * Date: 24.04.2007                                                            *
 *                                                                             *
 * Implementation by Krzysztof Kaczor <kinio4444@gmail.com>                    *
 *                                                                             *
 * Copyright (C) 2006-9 by the HeKatE Project                                  *
 *                                                                             *
 * HQEd has been develped within the HeKatE Project,                           *
 * see http://hekate.ia.agh.edu.pl                                             *
 *                                                                             *
 * This file is part of HQEd.                                                  *
 *                                                                             *
 * HQEd is free software: you can redistribute it and/or modify                *
 * it under the terms of the GNU General Public License as published by        *
 * the Free Software Foundation, either version 3 of the License, or           *
 * (at your option) any later version.                                         *
 *                                                                             *
 * HQEd is distributed in the hope that it will be useful,                     *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU General Public License for more details.                                *
 *                                                                             *
 * You should have received a copy of the GNU General Public License           *
 * along with HQEd.  If not, see <http://www.gnu.org/licenses/>.               *
 *                                                                             *
 * Comment: Plik zawierajcy definicje funckji obslugujacych okienko            *
 * edytora atrybutow                                                           *
 *                                                                             *
 */

#include <QMessageBox>
#include <QtGui>
#include <QDialog>
#include <QObject>
#include <QKeyEvent>

#include "ValEdit_ui.h"
#include "AttributeEditor_ui.h"
#include "SelectGroup_ui.h"
#include "../Settings_ui.h"

#include "../../M/XTT/XTT_TableList.h"
#include "../../M/XTT/XTT_AttributeGroups.h"
// -----------------------------------------------------------------------------

AttrEditor* AttributeEditor;
// -----------------------------------------------------------------------------

/**
 * Konstruktor kalsy
 * @param QWidget *parent - Wskanik do rodzica
 * @returns Brak zwracanych wartosci
*/
AttrEditor::AttrEditor(QWidget*)
{
    setupUi(this);
    setWindowIcon(QIcon(":/all_images/images/mainico.bmp")); 

    // signals/slots mechanism in action
    connect(radioButton, SIGNAL(clicked()), this, SLOT(RadioButton1Click()));
    connect(radioButton_2, SIGNAL(clicked()), this, SLOT(RadioButton2Click()));
    connect(radioButton_3, SIGNAL(clicked()), this, SLOT(RadioButton3Click()));
    connect(radioButton_4, SIGNAL(clicked()), this, SLOT(RadioButton4Click()));
    connect(radioButton_5, SIGNAL(clicked()), this, SLOT(RadioButton5Click()));
    connect(radioButton_6, SIGNAL(clicked()), this, SLOT(RadioButton6Click()));
    connect(toolButton, SIGNAL(clicked()), this, SLOT(toolButton1Click()));
    connect(toolButton_2, SIGNAL(clicked()), this, SLOT(toolButton2Click()));
    connect(toolButton_3, SIGNAL(clicked()), this, SLOT(toolButton3Click()));
    connect(pushButton, SIGNAL(clicked()), this, SLOT(OKClick()));
    connect(comboBox, SIGNAL(currentIndexChanged(int)), this, SLOT(ComboBoxChange(int)));
    connect(comboBox_2, SIGNAL(currentIndexChanged(int)), this, SLOT(ComboBox2Change(int)));
    setWindowFlags(Qt::WindowMaximizeButtonHint);
}
// -----------------------------------------------------------------------------

/**
 * Konstruktor kalsy
 * @param bool New - Okresla czy dodajemy atrybut (true) czy edytujemy (false)
 * @param int GroupIndex - Index grupy z ktorej edytujemy argumenty
 * @param int AttrIndex - Index atrybutu ktory edytujemy
 * @param QWidget *parent - Wskanik do rodzica
 * @returns Brak zwracanych wartosci
*/
AttrEditor::AttrEditor(int attrID, bool New, QWidget*)
{
	setupUi(this);
	setWindowIcon(QIcon(":/all_images/images/mainico.bmp")); 
    
	int GroupIndex = -1;
	int AttrIndex = -1;
	for(unsigned int g=0;g<AttributeGroups->Count();++g)
	{
		int aindex = AttributeGroups->Group(g)->IndexOfID(attrID);
		if(aindex > -1)
		{
			GroupIndex = g;
			AttrIndex = aindex;
			break;
		}
	}
    
	// Wyszukiwanie atrybutu
    
	// signals/slots mechanism in action
	connect(radioButton, SIGNAL(clicked()), this, SLOT(RadioButton1Click()));
	connect(radioButton_2, SIGNAL(clicked()), this, SLOT(RadioButton2Click()));
	connect(radioButton_3, SIGNAL(clicked()), this, SLOT(RadioButton3Click()));
	connect(radioButton_4, SIGNAL(clicked()), this, SLOT(RadioButton4Click()));
	connect(radioButton_5, SIGNAL(clicked()), this, SLOT(RadioButton5Click()));
	connect(radioButton_6, SIGNAL(clicked()), this, SLOT(RadioButton6Click()));
	connect(pushButton, SIGNAL(clicked()), this, SLOT(OKClick()));
	connect(pushButton_3, SIGNAL(clicked()), this, SLOT(SrcGroupSelect()));
	connect(pushButton_4, SIGNAL(clicked()), this, SLOT(DstGroupSelect()));
	connect(toolButton, SIGNAL(clicked()), this, SLOT(toolButton1Click()));
	connect(toolButton_2, SIGNAL(clicked()), this, SLOT(toolButton2Click()));
	connect(toolButton_3, SIGNAL(clicked()), this, SLOT(toolButton3Click()));
	connect(comboBox, SIGNAL(currentIndexChanged(int)), this, SLOT(ComboBoxChange(int)));
	connect(comboBox, SIGNAL(editTextChanged(QString)), this, SLOT(comboBoxEditTextChange(QString)));
	connect(comboBox_2, SIGNAL(currentIndexChanged(int)), this, SLOT(ComboBox2Change(int)));
    
	connect(lineEdit_4, SIGNAL(textChanged(QString)), this, SLOT(onEnumEditor4TextChange(QString)));
	connect(lineEdit_5, SIGNAL(textChanged(QString)), this, SLOT(onEnumEditor5TextChange(QString)));
	connect(lineEdit_8, SIGNAL(textChanged(QString)), this, SLOT(onEnumEditor8TextChange(QString)));
    
	setWindowFlags(Qt::WindowMaximizeButtonHint);

	// Zapisanie parametrow edycji
	fNew = New;
	fAttrGroupIndex = GroupIndex;
	fAttrIndex = AttrIndex;

	pushButton_3->setText(AttributeGroups->CreatePath(fAttrGroupIndex));
	pushButton_4->setText(AttributeGroups->CreatePath(fAttrGroupIndex));

	ReadAttributesNames();
	comboBox->setCurrentIndex(AttrIndex);
	SelectedAttributeChanged(fAttrGroupIndex, fAttrIndex);
    
	// Aby zablokowac zakladki
	if(New)
		ComboBox2Change(0);
}
// -----------------------------------------------------------------------------

/**
 * Konstruktor kalsy
 * @param bool New - Okresla czy dodajemy atrybut (true) czy edytujemy (false)
 * @param int GroupIndex - Index grupy z ktorej edytujemy argumenty
 * @param int AttrIndex - Index atrybutu ktory edytujemy
 * @param QWidget *parent - Wskanik do rodzica
 * @returns Brak zwracanych wartosci
*/
AttrEditor::AttrEditor(bool New, int GroupIndex, int AttrIndex, QWidget*)
{
    setupUi(this);
    setWindowIcon(QIcon(":/all_images/images/mainico.bmp")); 
    
    // signals/slots mechanism in action
    connect(radioButton, SIGNAL(clicked()), this, SLOT(RadioButton1Click()));
    connect(radioButton_2, SIGNAL(clicked()), this, SLOT(RadioButton2Click()));
    connect(radioButton_3, SIGNAL(clicked()), this, SLOT(RadioButton3Click()));
    connect(radioButton_4, SIGNAL(clicked()), this, SLOT(RadioButton4Click()));
    connect(radioButton_5, SIGNAL(clicked()), this, SLOT(RadioButton5Click()));
    connect(radioButton_6, SIGNAL(clicked()), this, SLOT(RadioButton6Click()));
    connect(pushButton, SIGNAL(clicked()), this, SLOT(OKClick()));
    connect(pushButton_3, SIGNAL(clicked()), this, SLOT(SrcGroupSelect()));
    connect(pushButton_4, SIGNAL(clicked()), this, SLOT(DstGroupSelect()));
    connect(toolButton, SIGNAL(clicked()), this, SLOT(toolButton1Click()));
    connect(toolButton_2, SIGNAL(clicked()), this, SLOT(toolButton2Click()));
    connect(toolButton_3, SIGNAL(clicked()), this, SLOT(toolButton3Click()));
    connect(comboBox, SIGNAL(currentIndexChanged(int)), this, SLOT(ComboBoxChange(int)));
    connect(comboBox, SIGNAL(editTextChanged(QString)), this, SLOT(comboBoxEditTextChange(QString)));
    connect(comboBox_2, SIGNAL(currentIndexChanged(int)), this, SLOT(ComboBox2Change(int)));
    
    connect(lineEdit_4, SIGNAL(textChanged(QString)), this, SLOT(onEnumEditor4TextChange(QString)));
    connect(lineEdit_5, SIGNAL(textChanged(QString)), this, SLOT(onEnumEditor5TextChange(QString)));
    connect(lineEdit_8, SIGNAL(textChanged(QString)), this, SLOT(onEnumEditor8TextChange(QString)));
    
    setWindowFlags(Qt::WindowMaximizeButtonHint);

    // Zapisanie parametrow edycji
    fNew = New;
    fAttrGroupIndex = GroupIndex;
    fAttrIndex = AttrIndex;

    pushButton_3->setText(AttributeGroups->CreatePath(fAttrGroupIndex));
    pushButton_4->setText(AttributeGroups->CreatePath(fAttrGroupIndex));

    ReadAttributesNames();
    comboBox->setCurrentIndex(AttrIndex);
    SelectedAttributeChanged(fAttrGroupIndex, fAttrIndex);
    
    // Aby zablokowac zakladki
    if(New)
	ComboBox2Change(0);
}
// -----------------------------------------------------------------------------

/**
 * Metoda klasy AttrEditor, wywoywana w momencie zarejestrowania zmiany
 * zawartoci obiektu ComboBox
 * @param int Index - Index elementu na ktory obiekt zostal zmieniony
 * @returns Brak zwracanych wartoci
*/
void AttrEditor::ComboBoxChange(int Index)
{
     fAttrIndex = Index;
     SelectedAttributeChanged(fAttrGroupIndex, fAttrIndex);
}
// -----------------------------------------------------------------------------

/**
 * Metoda klasy AttrEditor, wywoywana w momencie zmiany tekstu w obiekcie lineEdit bedacym edytorem wartosci wyliczeniowych
 * @param QString _newText - nowa wartosc tekstu
 * @returns Brak zwracanych wartoci
*/
void AttrEditor::onEnumEditor4TextChange(QString _newText)
{
	// Opcje zastepowania
	QString spaceReplacer = Settings->spaceReplacer();
	
	// Wyszukiwanie spacji
	int from_index = 0;
	int space_index;
	int cp = lineEdit_4->cursorPosition();
	
	do
	{
		// Wyszukanie indexu spacji
		space_index = _newText.indexOf(" ", from_index);
		
		// Gdy spacja jest na samym poczatku to ja usuwamy
		if(space_index == 0)
			_newText = _newText.remove(0, 1);
		
		if((space_index > 0) && (_newText[space_index-1] != ';'))
			_newText = _newText.replace(space_index, 1, spaceReplacer);
		
		from_index = space_index+1;
	}
	while(space_index > -1);
	
	lineEdit_4->setText(_newText);
	lineEdit_4->setCursorPosition(cp+spaceReplacer.length());
}
// -----------------------------------------------------------------------------

/**
 * Metoda klasy AttrEditor, wywoywana w momencie zmiany tekstu w obiekcie lineEdit bedacym edytorem wartosci wyliczeniowych
 * @param QString _newText - nowa wartosc tekstu
 * @returns Brak zwracanych wartoci
*/
void AttrEditor::onEnumEditor5TextChange(QString _newText)
{
	// Opcje zastepowania
	QString spaceReplacer = Settings->spaceReplacer();
	
	// Wyszukiwanie spacji
	int from_index = 0;
	int space_index;
	int cp = lineEdit_5->cursorPosition();
	
	do
	{
		// Wyszukanie indexu spacji
		space_index = _newText.indexOf(" ", from_index);
		
		// Gdy spacja jest na samym poczatku to ja usuwamy
		if(space_index == 0)
			_newText = _newText.remove(0, 1);
		
		if((space_index > 0) && (_newText[space_index-1] != ';'))
			_newText = _newText.replace(space_index, 1, spaceReplacer);
		
		from_index = space_index+1;
	}
	while(space_index > -1);
	
	lineEdit_5->setText(_newText);
	lineEdit_5->setCursorPosition(cp+spaceReplacer.length());
}
// -----------------------------------------------------------------------------

/**
 * Metoda klasy AttrEditor, wywoywana w momencie zmiany tekstu w obiekcie lineEdit bedacym edytorem wartosci wyliczeniowych
 * @param QString _newText - nowa wartosc tekstu
 * @returns Brak zwracanych wartoci
*/
void AttrEditor::onEnumEditor8TextChange(QString _newText)
{
	// Opcje zastepowania
	QString spaceReplacer = Settings->spaceReplacer();
	
	// Wyszukiwanie spacji
	int from_index = 0;
	int space_index;
	int cp = lineEdit_8->cursorPosition();
	
	do
	{
		// Wyszukanie indexu spacji
		space_index = _newText.indexOf(" ", from_index);
		
		// Gdy spacja jest na samym poczatku to ja usuwamy
		if(space_index == 0)
			_newText = _newText.remove(0, 1);
		
		if((space_index > 0) && (_newText[space_index-1] != ';'))
			_newText = _newText.replace(space_index, 1, spaceReplacer);
		
		from_index = space_index+1;
	}
	while(space_index > -1);
	
	lineEdit_8->setText(_newText);
	lineEdit_8->setCursorPosition(cp+spaceReplacer.length());
}
// -----------------------------------------------------------------------------

/**
 * Metoda klasy AttrEditor, wywoywana w momencie zmiany tekstu w obiekcie comboBox - nazwa atrybutu tworzaca automatyczna nazwe akronimu
 * @param QString _newText - nowa wartosc tekstu
 * @returns Brak zwracanych wartoci
*/
void AttrEditor::comboBoxEditTextChange(QString _newText)
{
	// Jezeli nie mamy tworzyc automatycznej nazwy atrybutu
	if(!Settings->xttCreateAutoAcronym())
		return;

	// Jezeli string pusty to nic nie wstawiamy nic nie zmieniamy
	if(_newText == "")
		return;

	QString newAcronymBaseName = _newText;
	
	// Jezeli tekst jest zbyt dlugi
	if(newAcronymBaseName.length() > 4)
		newAcronymBaseName = newAcronymBaseName.remove(4, newAcronymBaseName.length()-4);
		
	int newAcronymIndex = 0;
	bool is_ok;
	QString newAcronymName;
	
	// Wyszukiwanie odpowiedniej nazwy dla akronimu
	do
	{
		is_ok = true;
		
		newAcronymIndex++;
		newAcronymName = QString::number(newAcronymIndex);
		
		// Poniewaz dlugosc nazwy akronium nie moze byc dluzsza niz 5 znakow to musimy jeszcze obciac
		// aktualna nazwe bazowa akronimu
		int lenSum = newAcronymName.length() + newAcronymBaseName.length();
		if(lenSum > 5)
			newAcronymBaseName = newAcronymBaseName.remove(newAcronymBaseName.length()-(lenSum-5), (lenSum-5));
			
		// Utworzenie nowej nazwy
		newAcronymName = newAcronymBaseName + newAcronymName;
		
		// Sprawdzanie czy w aktualnie wybranej grupie przeznaczenia nie wystepuje juz dany akronim lub nazwa
	     QString dgname = pushButton_4->text().split(".", QString::SkipEmptyParts).last();         // Nowa grupa
	     int sgindex = fAttrGroupIndex;                                       					// Index starej grupy
	     int dgindex = AttributeGroups->IndexOfName(dgname);                                       // Index nowej grupy

	     int sgai = fAttrIndex;                    										// Index atrybutu w starej grupie
	     int dgai = AttributeGroups->Group(dgindex)->IndexOfName(newAcronymName);                  // Index atrybutu w nowej grupie
	     int dgaci = AttributeGroups->Group(dgindex)->IndexOfAcronym(newAcronymName);              // Sprawdzanie czy nie istnieje taki akronim
		
		// Jezeli grupa docelowa jest taka sama jak bierzaca
		if(sgindex == dgindex)
		{
			// Jezeli istnieje juz taki akronim
			if((dgaci > -1) && (dgaci != sgai))
				is_ok = false;
				
			// Jezeli istnieje juz taka nazwa
			if((dgai > -1) && (dgai != sgai))
				is_ok = false;
		}
		
		// Jezeli grupa docelowa jest inna niz bierzaca
		if(sgindex != dgindex)
		{
			// Jezeli istnieje juz taki akronim
			if(dgaci > -1)
				is_ok = false;
				
			// Jezeli istnieje juz taka nazwa
			if(dgai > -1)
				is_ok = false;
		}
	}
	while(!is_ok);
	
	lineEdit->setText(newAcronymName);
}
// -----------------------------------------------------------------------------

/**
 * Metoda klasy AttrEditor, wywoywana w momencie zarejestrowania zmiany
 * zawartoci obiektu ComboBox_2
 * @param int Index - Index elementu na ktory obiekt zostal zmieniony
 * @returns Brak zwracanych wartoci
*/
void AttrEditor::ComboBox2Change(int Index)
{
     QStringList defvals;
     defvals.clear();
     defvals << "true" << "0" << "0.0" << "" << "";
     lineEdit_3->setText(defvals.at(Index));
	
	// Blokowanie zakladek
	for(int i=0;i<tabWidget->count();i++)
		tabWidget->setTabEnabled(i, Index==i);
	tabWidget->setCurrentIndex(Index);
}
// -----------------------------------------------------------------------------

/**
 * Metoda klasy AttrEditor wczytujaca nazwy atrybutow z danej grupy
 * @returns Brak zwracanych wartoci
*/
void AttrEditor::ReadAttributesNames(void)
{
     if(fNew)
     {
          lineEdit_3->setText("true");
          return;
     }

     comboBox->clear();
     comboBox->addItems(AttributeGroups->Group(fAttrGroupIndex)->AttributeNameList());
     comboBox->setCurrentIndex(fAttrIndex);
}
// -----------------------------------------------------------------------------

/**
 * Metoda klasy AttrEditor stawiajaca okienko zgodnie z podanymi parametrami
 * @param int GroupIndex - Index grupy atrybutu
 * @param int AttrIndex - Index atrybutu
 * @returns Brak zwracanych wartoci
*/
void AttrEditor::SelectedAttributeChanged(int GroupIndex, int AttrIndex)
{
     pushButton_3->setEnabled(!fNew);
     label_6->setEnabled(!fNew);

     // Sprawdzenie poprawnosci indexow
     if((GroupIndex == -1) || (AttrIndex == -1))
          return;

     // Ustawianie okienka zgodnie z parametrami atrybutu
     lineEdit->setText(AttributeGroups->Group(GroupIndex)->Item(AttrIndex)->Acronym());
     lineEdit_2->setText(AttributeGroups->Group(GroupIndex)->Item(AttrIndex)->Description());
     comboBox_2->setCurrentIndex(AttributeGroups->Group(GroupIndex)->Item(AttrIndex)->Type()-1);
     tabWidget->setCurrentIndex(AttributeGroups->Group(GroupIndex)->Item(AttrIndex)->Type()-1);
     lineEdit_3->setText(AttributeGroups->Group(GroupIndex)->Item(AttrIndex)->DefaultValue());
     if(AttributeGroups->Group(GroupIndex)->Item(AttrIndex)->RelationsType() == 0)
          radioButton_7->setChecked(true);
     if(AttributeGroups->Group(GroupIndex)->Item(AttrIndex)->RelationsType() == 1)
          radioButton_8->setChecked(true);
     if(AttributeGroups->Group(GroupIndex)->Item(AttrIndex)->RelationsType() == 2)
          radioButton_9->setChecked(true);


     // Ustawianie ograniczen :(

     // Gdy Integer
     if(AttributeGroups->Group(GroupIndex)->Item(AttrIndex)->Type() == 2)
     {
          // Ustawienie radio buttonow
          if(AttributeGroups->Group(GroupIndex)->Item(AttrIndex)->ConstraintsType() == 0)
          {
               radioButton->setChecked(true);
               RadioButton1Click();
          }
          if(AttributeGroups->Group(GroupIndex)->Item(AttrIndex)->ConstraintsType() == 1)
          {
               radioButton_2->setChecked(true);
               RadioButton2Click();
               checkBox->setChecked(!AttributeGroups->Group(GroupIndex)->Item(AttrIndex)->LoIgnoreConstraints());
               checkBox_2->setChecked(!AttributeGroups->Group(GroupIndex)->Item(AttrIndex)->HiIgnoreConstraints());
               comboBox_3->setCurrentIndex((int)AttributeGroups->Group(GroupIndex)->Item(AttrIndex)->LoBraceType());
               comboBox_4->setCurrentIndex((int)AttributeGroups->Group(GroupIndex)->Item(AttrIndex)->HiBraceType());
               spinBox->setValue((int)AttributeGroups->Group(GroupIndex)->Item(AttrIndex)->BottomConstraints());
               spinBox_2->setValue((int)AttributeGroups->Group(GroupIndex)->Item(AttrIndex)->UpConstraints());
               spinBox->setEnabled(checkBox->isChecked());
               comboBox_3->setEnabled(checkBox->isChecked());
               spinBox_2->setEnabled(checkBox_2->isChecked());
               comboBox_4->setEnabled(checkBox_2->isChecked());
          }
          if(AttributeGroups->Group(GroupIndex)->Item(AttrIndex)->ConstraintsType() == 2)
          {
               radioButton_3->setChecked(true);
               RadioButton3Click();
               lineEdit_4->setText(AttributeGroups->Group(GroupIndex)->Item(AttrIndex)->EnumerateConstraintsString());
          }
     }

     // Gdy Float
     if(AttributeGroups->Group(GroupIndex)->Item(AttrIndex)->Type() == 3)
     {
          // Ustawienie radio buttonow
          if(AttributeGroups->Group(GroupIndex)->Item(AttrIndex)->ConstraintsType() == 0)
          {
               radioButton_4->setChecked(true);
               RadioButton4Click();
          }
          if(AttributeGroups->Group(GroupIndex)->Item(AttrIndex)->ConstraintsType() == 1)
          {
               radioButton_5->setChecked(true);
               RadioButton5Click();
               checkBox_3->setChecked(!AttributeGroups->Group(GroupIndex)->Item(AttrIndex)->LoIgnoreConstraints());
               checkBox_4->setChecked(!AttributeGroups->Group(GroupIndex)->Item(AttrIndex)->HiIgnoreConstraints());
               comboBox_6->setCurrentIndex((int)AttributeGroups->Group(GroupIndex)->Item(AttrIndex)->LoBraceType());
               comboBox_5->setCurrentIndex((int)AttributeGroups->Group(GroupIndex)->Item(AttrIndex)->HiBraceType());
               doubleSpinBox->setValue(AttributeGroups->Group(GroupIndex)->Item(AttrIndex)->BottomConstraints());
               doubleSpinBox_2->setValue(AttributeGroups->Group(GroupIndex)->Item(AttrIndex)->UpConstraints());
               doubleSpinBox->setEnabled(checkBox_3->isChecked());
               comboBox_6->setEnabled(checkBox_3->isChecked());
               doubleSpinBox_2->setEnabled(checkBox_4->isChecked());
               comboBox_5->setEnabled(checkBox_4->isChecked());
          }
          if(AttributeGroups->Group(GroupIndex)->Item(AttrIndex)->ConstraintsType() == 2)
          {
               radioButton_6->setChecked(true);
               RadioButton6Click();
               lineEdit_5->setText(AttributeGroups->Group(GroupIndex)->Item(AttrIndex)->EnumerateConstraintsString());
          }
     }

     // Gdy Enumerative
     if(AttributeGroups->Group(GroupIndex)->Item(AttrIndex)->Type() == 5)
          lineEdit_8->setText(AttributeGroups->Group(GroupIndex)->Item(AttrIndex)->AttrValuesString());
}
// -----------------------------------------------------------------------------

/**
 * Metodoa klasy void AttrEditor, wywoywana w momencie kliknicia na obiekcie 
 * RadioButton1
 * @returns Brak zwracanych wartoci
*/
void AttrEditor::RadioButton1Click(void)
{
     tabWidget_2->setEnabled(false);
}
// -----------------------------------------------------------------------------

/**
 * Metodoa klasy void AttrEditor, wywoywana w momencie kliknicia na obiekcie 
 * RadioButton2
 * @returns Brak zwracanych wartoci
*/
void AttrEditor::RadioButton2Click(void)
{
     tabWidget_2->setEnabled(true);
	tabWidget_2->setTabEnabled(0, true);
	tabWidget_2->setTabEnabled(1, false);
     tabWidget_2->setCurrentIndex(0);
}
// -----------------------------------------------------------------------------

/**
 * Metodoa klasy void AttrEditor, wywoywana w momencie kliknicia na obiekcie 
 * RadioButton3
 * @returns Brak zwracanych wartoci
*/
void AttrEditor::RadioButton3Click(void)
{
     tabWidget_2->setEnabled(true);
	tabWidget_2->setTabEnabled(0, false);
	tabWidget_2->setTabEnabled(1, true);
     tabWidget_2->setCurrentIndex(1);
}
// -----------------------------------------------------------------------------

/**
 * Metodoa klasy void AttrEditor, wywoywana w momencie kliknicia na obiekcie 
 * RadioButton4
 * @returns Brak zwracanych wartoci
*/
void AttrEditor::RadioButton4Click(void)
{
     tabWidget_3->setEnabled(false);
}
// -----------------------------------------------------------------------------

/**
 * Metodoa klasy void AttrEditor, wywoywana w momencie kliknicia na obiekcie 
 * RadioButton5
 * @returns Brak zwracanych wartoci
*/
void AttrEditor::RadioButton5Click(void)
{
     tabWidget_3->setEnabled(true);
	tabWidget_3->setTabEnabled(0, true);
	tabWidget_3->setTabEnabled(1, false);
     tabWidget_3->setCurrentIndex(0);
}
// -----------------------------------------------------------------------------

/**
 * Metodoa klasy void AttrEditor, wywoywana w momencie kliknicia na obiekcie
 * RadioButton6
 * @returns Brak zwracanych wartoci
*/
void AttrEditor::RadioButton6Click(void)
{
     tabWidget_3->setEnabled(true);
	tabWidget_3->setTabEnabled(0, false);
	tabWidget_3->setTabEnabled(1, true);
     tabWidget_3->setCurrentIndex(1);
}
// -----------------------------------------------------------------------------

/**
 * Metodoa klasy void AttrEditor, wywoywana w momencie kliknicia na przycisku
 * pozwalajacym na wybranie grupy zrodlowej
 * @returns Brak zwracanych wartoci
*/
void AttrEditor::SrcGroupSelect(void)
{
     delete SelGroup;
     SelGroup = new SelectGroup_ui;
     connect(SelGroup, SIGNAL(ResultOK()), this, SLOT(OnSrcGroupChange()));
     SelGroup->show();
}
// -----------------------------------------------------------------------------

/**
 * Metodoa klasy void AttrEditor, wywoywana w momencie zmiany grupy zrodlowej
 * @returns Brak zwracanych wartoci
*/
void AttrEditor::OnSrcGroupChange(void)
{
     int pindex = AttributeGroups->IndexOfName(SelGroup->Result());
     if(pindex == -1)
          return;

     fAttrGroupIndex = pindex;
     pushButton_3->setText(AttributeGroups->CreatePath(fAttrGroupIndex));
     pushButton_4->setText(AttributeGroups->CreatePath(fAttrGroupIndex));
     ReadAttributesNames();
     SelectedAttributeChanged(fAttrGroupIndex, fAttrIndex);
}
// -----------------------------------------------------------------------------

/**
 * Metodoa klasy void AttrEditor, wywoywana w momencie kliknicia na przycisku
 * pozwalajacym na wybranie grupy zrodlowej
 * @returns Brak zwracanych wartoci
*/
void AttrEditor::DstGroupSelect(void)
{
     delete SelGroup;
     SelGroup = new SelectGroup_ui;
     connect(SelGroup, SIGNAL(ResultOK()), this, SLOT(OnDstGroupChange()));
     SelGroup->show();
}
// -----------------------------------------------------------------------------

/**
 * Metodoa klasy void AttrEditor, wywoywana w momencie zmiany grupy docelowej
 * @returns Brak zwracanych wartoci
*/
void AttrEditor::OnDstGroupChange(void)
{
     int pindex = AttributeGroups->IndexOfName(SelGroup->Result());
     if(pindex == -1)
          return;

     pushButton_4->setText(AttributeGroups->CreatePath(pindex));
}
// -----------------------------------------------------------------------------

/**
 * Slot klasy AttrEditor, aktywowany sygnalem kliknicia na obiekcie OK 
 * @returns Brak zwracanych wartoci
*/
void AttrEditor::OKClick(void)
{
     // WALIDACJA DANYCH
     // Index edytowanego atrybutu
     int curr_index = comboBox->currentIndex();

     // Kiedy zaden atrybut nie zostal wybrany
     if((curr_index == -1) && (!fNew))
     {
          const QString content = "There is no attrbiute selected.";
          QMessageBox::warning(NULL, "XTT Editor", content, QMessageBox::Ok);
          return;
     }

     // Wczytanie nowej nazwy i usuniecie spacji
     QString atrrNewName = comboBox->currentText();                                            // Nowa nazwa atrybutu
     for(;atrrNewName.contains(" ");)
          atrrNewName = atrrNewName.remove(atrrNewName.indexOf(" "), 1);

     // Wczytanie nowego akronimu i usuniecie spacji
     QString atrrNewAcronym = lineEdit->text();                                                // Nowy akronim atrybutu
     for(;atrrNewAcronym.contains(" ");)
          atrrNewAcronym = atrrNewAcronym.remove(atrrNewAcronym.indexOf(" "), 1);

     // Sprawdzanie czy podana nazwa juz nie istnieje
     QString sgname = pushButton_3->text().split(".", QString::SkipEmptyParts).last();         // Stara grupa
     QString dgname = pushButton_4->text().split(".", QString::SkipEmptyParts).last();         // Nowa grupa
     int sgindex = AttributeGroups->IndexOfName(sgname);                                       // Index starej grupy
     int dgindex = AttributeGroups->IndexOfName(dgname);                                       // Index nowej grupy

     if((sgindex == -1) || (dgindex == -1))
     {
          const QString content = "Incorrect source/destination group.";
          QMessageBox::critical(NULL, "XTT Editor", content, QMessageBox::Ok);
          return;
     }
     int sgai = fAttrIndex;                                                                    // Index atrybutu w starej grupie
     int dgai = AttributeGroups->Group(dgindex)->IndexOfName(atrrNewName);                     // Index atrybutu w nowej grupie
     int dgaci = AttributeGroups->Group(dgindex)->IndexOfAcronym(atrrNewName);                 // Sprawdzanie czy nie istnieje taki akronim

     if(((fNew) && ((dgai > -1) || (dgaci > -1))) ||
      ((!fNew) && (dgai > -1) && (sgindex == dgindex) && (sgai != dgai)) ||
      ((!fNew) && (dgai > -1) && (sgindex != dgindex)) ||
      ((!fNew) && (dgaci > -1) && (sgindex == dgindex) && (sgai != dgaci)) ||
      ((!fNew) && (dgaci > -1) && (sgindex != dgindex)))
     {
          const QString content = "The name/acronym like new name is already exists in this group.\nPlease give another name.";
          QMessageBox::information(NULL, "XTT Editor", content, QMessageBox::Ok);
          return;
     }

     // Sprawdzanie czy podany akronim juz nie istnieje
     dgai = AttributeGroups->Group(dgindex)->IndexOfAcronym(atrrNewAcronym);                   // Index atrybutu w nowej grupie
     dgaci = AttributeGroups->Group(dgindex)->IndexOfName(atrrNewAcronym);                     // Sprawdzanie czy nie istnieje taka nazwa jak akronim

     if(((fNew) && ((dgai > -1) || (dgaci > -1))) ||
      ((!fNew) && (dgai > -1) && (sgindex == dgindex) && (sgai != dgai)) ||
      ((!fNew) && (dgai > -1) && (sgindex != dgindex)) ||
      ((!fNew) && (dgaci > -1) && (sgindex == dgindex) && (sgai != dgaci)) ||
      ((!fNew) && (dgaci > -1) && (sgindex != dgindex)))
     {
          const QString content = "The name/acronym like acronym is already exists in this group.\nPlease give another acronym.";
          QMessageBox::information(NULL, "XTT Editor", content, QMessageBox::Ok);
          return;
     }

     // Sprawdzanie czy podano wszystkie wymagane dane
     // Czy podano nazwe
     if((atrrNewName == "") ||
          (atrrNewName.toLower() == XTT_Attribute::_not_definded_operator_) ||
          (atrrNewName.toLower() == XTT_Attribute::_any_operator_))
     {
          QMessageBox::information(NULL, "XTT Editor", "Icorrect name.", QMessageBox::Ok);
          return;
     }
     // Czy podano akronim
     if((atrrNewAcronym == "") ||
          (atrrNewAcronym.toLower() == XTT_Attribute::_not_definded_operator_) ||
          (atrrNewAcronym.toLower() == XTT_Attribute::_any_operator_))
     {
          QMessageBox::information(NULL, "XTT Editor", "Incorrect acronym.", QMessageBox::Ok);
          return;
     }

     // Sprawdzanie czy podane ograniczenia przedzialowe sa poprawne
     if((comboBox_2->currentIndex() == 1) && (radioButton_2->isChecked()))      // Jezeli liczba calkowita z ograniczeniami przedzalowymi
     {
          if((checkBox->isChecked()) && (checkBox_2->isChecked()) && (spinBox->value() > spinBox_2->value()))
          {
               QMessageBox::information(NULL, "XTT Editor", "Low limit must be less than hi limit.", QMessageBox::Ok);
               return;
          }
     }

     // Sprawdzanie czy podane ograniczenia przedzialowe sa poprawne
     if((comboBox_2->currentIndex() == 2) && (radioButton_5->isChecked()))      // Jezeli liczba zmiennoprzecinkowa z ograniczeniami przedzalowymi
     {
          if((checkBox_3->isChecked()) && (checkBox_4->isChecked()) && (doubleSpinBox->value() > doubleSpinBox_2->value()))
          {
               QMessageBox::information(NULL, "XTT Editor", "Low limit must be less than hi limit.", QMessageBox::Ok);
               return;
          }
     }

     // Sprawdzanie czy podane wartosci w ograniczenich wyliczeniowych sa odpowiedniego typu
     if((comboBox_2->currentIndex() == 1) && (radioButton_3->isChecked()))      // Jezeli liczba calkowita z ograniczeniem wyliczeniowym
     {
          // Utworzenie listy wartosci
          QStringList t = lineEdit_4->text().split(";", QString::SkipEmptyParts);

          // Sprawdzanie kazdej wartosci czy jest odpowiedniego typu
          for(int i=0;i<t.size();i++)
          {
               bool ok;
               t.at(i).toInt(&ok, 10);

               // Gdy konwersja jest nie udana to konczymy
               if(!ok)
               {
                    const QString content = "Value " + t.at(i) + " is not INTEGER type.";
                    QMessageBox::critical(NULL, "XTT Editor", content, QMessageBox::Ok);
                    return;
               }
          }
     }
     if((comboBox_2->currentIndex() == 2) && (radioButton_6->isChecked()))      // Jezeli liczba zmiennoprzecinkowa z ograniczeniem wyliczeniowym
     {
          // Utworzenie listy wartosci
          QStringList t = lineEdit_5->text().split(";", QString::SkipEmptyParts);

          // Sprawdzanie kazdej wartosci czy jest odpowiedniego typu
          for(int i=0;i<t.size();i++)
          {
               bool ok;
               t.at(i).toFloat(&ok);

               // Gdy konwersja jest nie udana to konczymy
               if(!ok)
               {
                    const QString content = "Value " + t.at(i) + " is not FLOAT type.";
                    QMessageBox::critical(NULL, "XTT Editor", content, QMessageBox::Ok);
                    return;
               }
          }
     }

     // KONIEC WALIDACJI DANYCH

     // Ustalanie adresu atrybutu
     XTT_Attribute* attr;

     if(!fNew)
     {
          // Zapisanie nowego indexu
          fAttrIndex = comboBox->currentIndex();
          
          // Kiedy index jest niepoprawny to tworzymy nowy argument
          if((fAttrIndex < 0) || (fAttrIndex >= (int)AttributeGroups->Group(fAttrGroupIndex)->Count()))
               fNew = true;

          // Jezeli wszystko ok to pobieramy adres z pojemnika
          if(!fNew)
               attr = AttributeGroups->Group(fAttrGroupIndex)->Item(fAttrIndex);
     }
     
     // Kiedy tworzymy nowy atrybut
     if(fNew)
          attr = new XTT_Attribute(AttributeGroups->Group(fAttrGroupIndex)->NextId());

     // Sprawdzanie czy trzeba ustawic tez wartosc
     bool setValue = false;
     if((int)attr->Type() != (comboBox_2->currentIndex()+1))
          setValue = true;

     attr->setName(atrrNewName);                                                // Ustawienie nazwy atrybutu
     attr->setAcronym(atrrNewAcronym);                                          // Ustawienie akronimu atrybutu
     attr->setDescription(lineEdit_2->text());                                  // Ustawienie opisu atrybutu
     attr->setType(comboBox_2->currentIndex()+1);                               // Ustawienie typu atrybutu
     if(radioButton_7->isChecked())
          attr->setRelationsType(0);
     if(radioButton_8->isChecked())
          attr->setRelationsType(1);
     if(radioButton_9->isChecked())
          attr->setRelationsType(2);

     // Ustawianie ograniczen
     attr->setConstraintsType(0);                                               // Poczatkowo przyjmujemy bez ograniczen
     if((comboBox_2->currentIndex() == 1) && (radioButton_2->isChecked()))      // Jezeli liczba calkowita z ograniczeniem przedzialowym
     {
          // Ustawiamy typ ograniczenia na przedzialowe
          attr->setConstraintsType(1);

          // Ustawiamy wartosci gornego i dolnego ograniczenia
          attr->setLoConstraints(spinBox->value());
          attr->setUpConstraints(spinBox_2->value());

          // Ustawiamy rodzaje nawiasow na koncach przedzialu
          attr->setLoBraceType((bool)comboBox_3->currentIndex());
          attr->setHiBraceType((bool)comboBox_4->currentIndex());

          // Ustawiamy ktore ograniczenia sa wazne, a ktore trzba ignorowac
          attr->setLoIgnoreConstraints(!checkBox->isChecked());
          attr->setHiIgnoreConstraints(!checkBox_2->isChecked());
     }
     if((comboBox_2->currentIndex() == 1) && (radioButton_3->isChecked()))      // Jezeli liczba calkowita z ograniczeniem wyliczeniowym
     {
          // Ustawiamy typ ograniczenia na wyliczeniowe
          attr->setConstraintsType(2);

          // Tworzymy liste dozwolonych wartosci atrybutu
          attr->setEnumerateConstraints(lineEdit_4->text().split(";", QString::SkipEmptyParts));
     }
     if((comboBox_2->currentIndex() == 2) && (radioButton_5->isChecked()))      // Jezeli liczba zmiennoprzecinkowa z ograniczeniem przedzialowym
     {
          // Ustawiamy typ ograniczenia na przedzialowe
          attr->setConstraintsType(1);

          // Ustawiamy wartosci gornego i dolnego ograniczenia
          attr->setLoConstraints(doubleSpinBox->value());
          attr->setUpConstraints(doubleSpinBox_2->value());

          // Ustawiamy rodzaje nawiasow na koncach przedzialu
          attr->setLoBraceType((bool)comboBox_6->currentIndex());
          attr->setHiBraceType((bool)comboBox_5->currentIndex());

          // Ustawiamy ktore ograniczenia sa wazne, a ktore trzba ignorowac
          attr->setLoIgnoreConstraints(!checkBox_3->isChecked());
          attr->setHiIgnoreConstraints(!checkBox_4->isChecked());
     }
     if((comboBox_2->currentIndex() == 2) && (radioButton_6->isChecked()))      // Jezeli liczba zmiennoprzecinkowa z ograniczeniem wyliczeniowym
     {
		// Ustawiamy typ ograniczenia na wyliczeniowe
          attr->setConstraintsType(2);
		
          // Tworzymy liste dozwolonych wartosci atrybutu
          attr->setEnumerateConstraints(lineEdit_5->text().split(";", QString::SkipEmptyParts));
     }

     // Gdy wybieramy typ wyliczeniowy, to musimy utworzyc liste wartosci dozwolonych
     if(comboBox_2->currentIndex() == 4)
     {
          // Towrzymy liste wartosci dozwolonych
          attr->setEnumValues(lineEdit_8->text().split(";", QString::SkipEmptyParts));
     }

     // Ustawienie wartosci domyslnej, jezeli sie nie uda to nie zamykamy jeszcze okienka
     if(!attr->setDefaultValue(lineEdit_3->text()))
          return;

     // Ustawienie wartosci jezeli tworzymy nowy, jezeli sie nie uda to nie zamykamy jeszcze okienka
     if(((fNew) || (setValue)) && (!attr->setValue(lineEdit_3->text())))
          return;

     // Zamkniecie okienka
     close();

     // Gdy nastepuje zmiana grupy to trzeba przeniesc atrybut
     if((!fNew) && (sgindex != dgindex))
     {
          // Dodawanie do nowej grupy
          AttributeGroups->Group(dgindex)->Add(attr);

          // Usuwanie ze starej grupy
          AttributeGroups->Group(sgindex)->Delete(sgai, false);                 // Usuwanie atrybutu
     }

     // Dodanie atrybutu tylko gdy jest on nowy
     if(fNew)
          AttributeGroups->Group(dgindex)->Add(attr);
		
	// Sortowanie atrybutow
	AttributeGroups->Group(dgindex)->sort();

     // Odswiezenie widoku tabel
	TableList->AnalyzeAllCells();
     TableList->RefreshAll();

     // Emisja sygnalu
     emit ClickOK();
}
// -----------------------------------------------------------------------------

/**
 * Metoda wywolywana w momencie edycji listy atrybutow
 * @returns Brak zwracanych wartoci
*/
void AttrEditor::toolButton1Click(void)
{
     delete ValEdit;
     ValEdit = new ValueEdit_ui(lineEdit_4->text());
     connect(ValEdit, SIGNAL(ClickOK()), this, SLOT(intEnumValsChange()));
     ValEdit->show();
}
// -----------------------------------------------------------------------------

/**
 * Metoda wywolywana w momencie edycji listy atrybutow
 * @returns Brak zwracanych wartoci
*/
void AttrEditor::toolButton2Click(void)
{
     delete ValEdit;
     ValEdit = new ValueEdit_ui(lineEdit_5->text());
     connect(ValEdit, SIGNAL(ClickOK()), this, SLOT(floatEnumValsChange()));
     ValEdit->show();
}
// -----------------------------------------------------------------------------

/**
 * Metoda wywolywana w momencie edycji listy atrybutow
 * @returns Brak zwracanych wartoci
*/
void AttrEditor::toolButton3Click(void)
{
     delete ValEdit;   
     ValEdit = new ValueEdit_ui(lineEdit_8->text());
     connect(ValEdit, SIGNAL(ClickOK()), this, SLOT(enumEnumValsChange()));
     ValEdit->show();
}
// -----------------------------------------------------------------------------

/**
 * Wywolywana w momencie zatwierdzenia zmiany wartosci
 * @returns Brak zwracanych wartoci
*/
void AttrEditor::intEnumValsChange(void)
{
     QString res = "";
     for(int i=0;i<ValEdit->values.size();i++)
          res = res + ValEdit->values.at(i) + ";";

     lineEdit_4->setText(res);
}
// -----------------------------------------------------------------------------

/**
 * Wywolywana w momencie zatwierdzenia zmiany wartosci
 * @returns Brak zwracanych wartoci
*/
void AttrEditor::floatEnumValsChange(void)
{
     QString res = "";
     for(int i=0;i<ValEdit->values.size();i++)
          res = res + ValEdit->values.at(i) + ";";

     lineEdit_5->setText(res);
}
// -----------------------------------------------------------------------------

/**
 * Wywolywana w momencie zatwierdzenia zmiany wartosci
 * @returns Brak zwracanych wartoci
*/
void AttrEditor::enumEnumValsChange(void)
{
     QString res = "";
     for(int i=0;i<ValEdit->values.size();i++)
          res = res + ValEdit->values.at(i) + ";";

     lineEdit_8->setText(res);
}
// -----------------------------------------------------------------------------

/**
 * Metoa wywolywana w momencie kiedy zostanie nacisniety jakis klawisz
 * @param QKeyEvent* event - Wskanik do obiektu przechwoujacego informacje o 
 * zdarzeniu
 * @returns Brak zwracanych wartoci
*/
void AttrEditor::keyPressEvent(QKeyEvent *event)
{
     switch (event->key())
     {
          case Qt::Key_Escape:
               close();
               break;

          case Qt::Key_Return:
               OKClick();
               break;

          default:
               QWidget::keyPressEvent(event);
     }
}
// -----------------------------------------------------------------------------
