 /**
 *                                                                             *
 * @file AttributeManager_ui.cpp                                               *
 * Project name: HeKatE hqed                                                   *
 * $Id: AttributeManager_ui.cpp,v 1.7 2008-03-03 22:04:31 kinio Exp $         *
 * Author: Krzysztof Kaczor                                                    *
 * Date: 29.04.2007                                                            *
 *                                                                             *
 * Implementation by Krzysztof Kaczor <kinio4444@gmail.com>                    *
 *                                                                             *
 * Copyright (C) 2006-9 by the HeKatE Project                                  *
 *                                                                             *
 * HQEd has been develped within the HeKatE Project,                           *
 * see http://hekate.ia.agh.edu.pl                                             *
 *                                                                             *
 * This file is part of HQEd.                                                  *
 *                                                                             *
 * HQEd is free software: you can redistribute it and/or modify                *
 * it under the terms of the GNU General Public License as published by        *
 * the Free Software Foundation, either version 3 of the License, or           *
 * (at your option) any later version.                                         *
 *                                                                             *
 * HQEd is distributed in the hope that it will be useful,                     *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU General Public License for more details.                                *
 *                                                                             *
 * You should have received a copy of the GNU General Public License           *
 * along with HQEd.  If not, see <http://www.gnu.org/licenses/>.               *
 *                                                                             *
 * Comment: Plik zawiera definicje metod obiektu obslugujacego okineko         *
 * managera atrybutow                                                          *
 *                                                                             *
 */

#include <QMessageBox>
#include <QtGui>
#include <QDialog>
#include <QObject>
#include <QKeyEvent>

#include "SelectGroup_ui.h"
#include "AttributeManager_ui.h"
#include "AttributeEditor_ui.h"
#include "SelectGroup_ui.h"
         
#include "../../M/XTT/XTT_AttributeGroups.h"
#include "../../M/XTT/XTT_TableList.h"
// -----------------------------------------------------------------------------

AttributeManager_ui* AttrManager;
// -----------------------------------------------------------------------------

/**
 * Konstruktor kalsy
 * @param QWidget *parent - Wskanik do rodzica
 * @returns Brak zwracanych wartosci
*/
AttributeManager_ui::AttributeManager_ui(QWidget*)
{
    setupUi(this);
    setWindowIcon(QIcon(":/all_images/images/mainico.bmp")); 

    connect(pushButton, SIGNAL(clicked()), this, SLOT(AddNewAttribute()));
    connect(pushButton_2, SIGNAL(clicked()), this, SLOT(EditAttribute()));
    connect(pushButton_3, SIGNAL(clicked()), this, SLOT(DeleteClick()));
    connect(pushButton_5, SIGNAL(clicked()), this, SLOT(NewGroupSelect()));
    connect(pushButton_6, SIGNAL(clicked()), this, SLOT(ReadAttributes()));
    connect(treeWidget, SIGNAL(itemClicked(QTreeWidgetItem*, int)), this, SLOT(TreeClick(QTreeWidgetItem*, int)));
    setWindowFlags(Qt::WindowMaximizeButtonHint);
    treeWidget->insertTopLevelItems(0, AttributeGroups->CreateTree());

    fGroupIndex = 0;
    ReadAttributes();

    // signals/slots mechanism in action
    //connect(pushButton, SIGNAL(clicked()), this, SLOT(DeleteClick()));
    //connect(pushButton_3, SIGNAL(clicked()), this, SLOT(SelectGroupClick()));
}
// -----------------------------------------------------------------------------

/**
 * Metoda wywoywana w momencie kliknicia na obiekcie Tree
 * @param QTreeWidgetItem* item - Wskanik do obiektu tree
 * @param int col - Kolumna w ktorej nastapilo klikniecie
 * @returns Brak zwracanych wartoci
*/
void AttributeManager_ui::TreeClick(QTreeWidgetItem* item, int col)
{
     // Zapisanie indexu kliknietej grupy
     fGroupIndex = AttributeGroups->IndexOfName(item->text(col));
     ReadAttributes();
}
// -----------------------------------------------------------------------------

/**
 * Metoda klasy AttributeManager_ui wczytujaca informacje o atrybutach z 
 * wybranje grupy
 * @returns Brak zwracanych wartoci
*/
void AttributeManager_ui::ReadAttributes(void)
{
     tableWidget->setRowCount(0);

     // Sprawdzanie czy index jest poprawny
     if((int)fGroupIndex == -1)
     {
          const QString content = "There is no group selected.";
          QMessageBox::information(NULL, "XTT Editor", content, QMessageBox::Ok);
          return;
     }

     // Wczytywanie informacji
     tableWidget->setRowCount(AttributeGroups->Group(fGroupIndex)->Count());
     QStringList types;
     types << "not defined" << "boolean" << "integer" << "float" << "symbolic" << "enumerate";

     for(unsigned int i=0;i<AttributeGroups->Group(fGroupIndex)->Count();i++)
     {
          QTableWidgetItem* _name = new QTableWidgetItem(AttributeGroups->Group(fGroupIndex)->Item(i)->Name());
          QTableWidgetItem* _aronym = new QTableWidgetItem(AttributeGroups->Group(fGroupIndex)->Item(i)->Acronym());
          QTableWidgetItem* _desc = new QTableWidgetItem(AttributeGroups->Group(fGroupIndex)->Item(i)->Description());
          QTableWidgetItem* _type = new QTableWidgetItem(types.at(AttributeGroups->Group(fGroupIndex)->Item(i)->Type()));
          QTableWidgetItem* _conrepr = new QTableWidgetItem(AttributeGroups->Group(fGroupIndex)->Item(i)->ConstraintsRepresentation());
          QTableWidgetItem* _defval = new QTableWidgetItem(AttributeGroups->Group(fGroupIndex)->Item(i)->DefaultValue());
          QTableWidgetItem* _val = new QTableWidgetItem(AttributeGroups->Group(fGroupIndex)->Item(i)->Value());

          tableWidget->setItem(i, 0, _name);
          tableWidget->setItem(i, 1, _aronym);
          tableWidget->setItem(i, 2, _desc);
          tableWidget->setItem(i, 3, _type);
          tableWidget->setItem(i, 4, _conrepr);
          tableWidget->setItem(i, 5, _defval);
          tableWidget->setItem(i, 6, _val);
     }
}
// -----------------------------------------------------------------------------

/**
 * Pokazuje okno edycji atrybutu. Dodatkowo konfiguruje je tak aby moc dodac 
 * nowy atrybut
 * @returns Brak zwracanych wartoci
*/
void AttributeManager_ui::AddNewAttribute(void)
{
     delete AttributeEditor;
     AttributeEditor = new AttrEditor(true, fGroupIndex);

     connect(AttributeEditor, SIGNAL(ClickOK()), this, SLOT(ReadAttributes()));
     
     AttributeEditor->show();
}
// -----------------------------------------------------------------------------

/**
 * Pokazuje okno edycji atrybutu. Dodatkowo konfiguruje je tak aby moc edytowac
 * atrybut
 * @returns Brak zwracanych wartoci
*/
void AttributeManager_ui::EditAttribute(void)
{
     // Wczytywanie indexu atrybutu
     if(tableWidget->currentRow() == -1)
     {
          QMessageBox::information(NULL, "XTT Editor", "There is no row selected.", QMessageBox::Ok);
          return;
     }

     QString name = tableWidget->item(tableWidget->currentRow(), 0)->text();
     int index = AttributeGroups->Group(fGroupIndex)->IndexOfName(name);

     delete AttributeEditor;
     AttributeEditor = new AttrEditor(false, fGroupIndex, index);

     connect(AttributeEditor, SIGNAL(ClickOK()), this, SLOT(ReadAttributes()));

     AttributeEditor->show();
}
// -----------------------------------------------------------------------------

/**
 * Metoda klasy AttributeManager_ui, wywoywana w momencie kliknicia na 
 * obiekcie Delete, usuwa wybrany atrybut
 * @returns Brak zwracanych wartoci
*/
void AttributeManager_ui::DeleteClick(void)
{
     // Kiedy nie jest zaznaczony zaden wiersz
     if((int)fGroupIndex == -1)
          return;

     // Wczytywanie indexu atrybutu
     if(tableWidget->currentRow() == -1)
     {
          QMessageBox::information(NULL, "XTT Editor", "There is no row selected.", QMessageBox::Ok);
          return;
     }

     QString name = tableWidget->item(tableWidget->currentRow(), 0)->text();
     int index = AttributeGroups->Group(fGroupIndex)->IndexOfName(name);

     // Gdy nie znaleziono
     if(index == -1)
     {
          QMessageBox::critical(NULL, "XTT Editor", "Can't find selected attribute.", QMessageBox::Ok);
          return;
     }

     // Potwierdzenie usuniecia
     const QString content = "Do you realy want to delete attribute named " + name + "?";
     if(QMessageBox::warning(NULL, "XTT Editor", content, QMessageBox::Yes | QMessageBox::No) == QMessageBox::No)
          return;

     // Inaczej kasujemy obiekt
     AttributeGroups->Group(fGroupIndex)->Delete(index);

     // Odswiezenie widoku
     ReadAttributes();

     // Odswiezenie widoku tabel
     TableList->RefreshAll();
}
// -----------------------------------------------------------------------------

/**
 * Metodoa klasy AttributeManager_ui, wywoywana w momencie kliknicia na
 * przycisku pozwalajacym na wybranie grupy atrybutu
 * @returns Brak zwracanych wartoci
*/
void AttributeManager_ui::NewGroupSelect(void)
{
     // Kiedy nie jest zaznaczony zaden wiersz
     if((int)fGroupIndex == -1)
          return;

     // Wczytywanie indexu atrybutu
     if(tableWidget->currentRow() == -1)
     {
          QMessageBox::information(NULL, "XTT Editor", "There is no row selected.", QMessageBox::Ok);
          return;
     }
     
     QString name = tableWidget->item(tableWidget->currentRow(), 0)->text();
     int fAttrIndex = AttributeGroups->Group(fGroupIndex)->IndexOfName(name);

     // Gdy nie znaleziono
     if(fAttrIndex == -1)
     {
          QMessageBox::critical(NULL, "XTT Editor", "Can't find selected attribute.", QMessageBox::Ok);
          return;
     }
     
     delete SelGroup;
     SelGroup = new SelectGroup_ui;
     connect(SelGroup, SIGNAL(ResultOK()), this, SLOT(NewGroupChange()));
     SelGroup->show();
}
// -----------------------------------------------------------------------------

/**
 * Metodoa klasy AttributeManager_ui, wywoywana w momencie zmiany grupy
 * atrybutu
 * @returns Brak zwracanych wartoci
*/
void AttributeManager_ui::NewGroupChange(void)
{
     QString name = tableWidget->item(tableWidget->currentRow(), 0)->text();
     int fAttrIndex = AttributeGroups->Group(fGroupIndex)->IndexOfName(name);

     // Gdy nie znaleziono
     if(fAttrIndex == -1)
     {
          QMessageBox::critical(NULL, "XTT Editor", "Can't find selected attribute.", QMessageBox::Ok);
          return;
     }

     int pindex = AttributeGroups->IndexOfName(SelGroup->Result());
     if(pindex == -1)
          return;

     // Sprawdzanie czy podana nazwa juz nie istnieje
     QString atrrNewName = name;                                                               // Nazwa atrybutu
     int sgindex = fGroupIndex;                                                                // Index starej grupy
     int dgindex = pindex;                                                                     // Index nowej grupy


     if((sgindex == -1) || (dgindex == -1))
     {
          const QString content = "Incorrect source/destination group.";
          QMessageBox::critical(NULL, "XTT Editor", content, QMessageBox::Ok);
          return;
     }
     int sgai = fAttrIndex;                                                                    // Index atrybutu w starej grupie
     int dgai = AttributeGroups->Group(dgindex)->IndexOfName(atrrNewName);                     // Index atrybutu w nowej grupie
     int dgaci = AttributeGroups->Group(dgindex)->IndexOfAcronym(atrrNewName);                 // Index atrybutu w nowej grupie o akronimie takim jak nazwa atrybutu

     if(((dgai > -1) && (sgindex == dgindex) && (sgai != dgai)) ||
      ((dgai > -1) && (sgindex != dgindex)) ||
      ((dgaci > -1) && (sgindex == dgindex) && (sgai != dgaci)) ||
      ((dgaci > -1) && (sgindex != dgindex)))
     {
          const QString content = "The name/acronym is already exists in this group.\nPlease give another name.";
          QMessageBox::information(NULL, "XTT Editor", content, QMessageBox::Ok);
          return;
     }

     // Sprawdzanie czy podany akronim juz nie istnieje
     QString atrrNewAcronym = AttributeGroups->Group(sgindex)->Item(fAttrIndex)->Acronym();    // Nowy akronim atrybutu
     dgai = AttributeGroups->Group(dgindex)->IndexOfAcronym(atrrNewAcronym);                   // Index atrybutu w nowej grupie
     dgaci = AttributeGroups->Group(dgindex)->IndexOfName(atrrNewAcronym);                     // Index atrybutu o nazwie jak akronim aktalnie przenoszonego

     if(((dgai > -1) && (sgindex == dgindex) && (sgai != dgai)) ||
      ((dgai > -1) && (sgindex != dgindex)) ||
      ((dgaci > -1) && (sgindex == dgindex) && (sgai != dgaci)) ||
      ((dgaci > -1) && (sgindex != dgindex)))
     {
          const QString content = "The name/acronym is already exists in this group.\nPlease give another acronym.";
          QMessageBox::information(NULL, "XTT Editor", content, QMessageBox::Ok);
          return;
     }

     // Gdy nastepuje zmiana grupy to trzeba przeniesc atrybut
     if(sgindex != dgindex)
     {
          // Dodawanie do nowej grupy
          AttributeGroups->Group(dgindex)->Add(AttributeGroups->Group(sgindex)->Item(fAttrIndex));

          // Usuwanie ze starej grupy
          AttributeGroups->Group(sgindex)->Delete(sgai, false);	// Usuwanie atrybutu
     }

     // Odswiezenie widoku
     ReadAttributes();

     // Odswiezenie widoku tabel
     TableList->RefreshAll();
}
// -----------------------------------------------------------------------------

/**
 * Metoa wywolywana w momencie kiedy zostanie nacisniety jakis klawisz
 * @param QKeyEvent* event - Wskanik do obiektu przechwoujacego informacje o 
 * zdarzeniu
 * @returns Brak zwracanych wartoci
*/
void AttributeManager_ui::keyPressEvent(QKeyEvent *event)
{
     switch (event->key())
     {
          case Qt::Key_Escape:
               close();
               break;

          case Qt::Key_Return:
               EditAttribute();
               break;

          default:
               QWidget::keyPressEvent(event);
     }
}
// -----------------------------------------------------------------------------
