 /**
 *                                                                             *
 * @file CellEditor_ui.cpp                                                     *
 * Project name: HeKatE hqed                                                   *
 * $Id: CellEditor_ui.cpp,v 1.12 2008-03-03 22:04:31 kinio Exp $                *
 * Author: Krzysztof Kaczor                                                    *
 * Date: 28.04.2007                                                            *
 *                                                                             *
 * Implementation by Krzysztof Kaczor <kinio4444@gmail.com>                    *
 *                                                                             *
 * Copyright (C) 2006-9 by the HeKatE Project                                  *
 *                                                                             *
 * HQEd has been develped within the HeKatE Project,                           *
 * see http://hekate.ia.agh.edu.pl                                             *
 *                                                                             *
 * This file is part of HQEd.                                                  *
 *                                                                             *
 * HQEd is free software: you can redistribute it and/or modify                *
 * it under the terms of the GNU General Public License as published by        *
 * the Free Software Foundation, either version 3 of the License, or           *
 * (at your option) any later version.                                         *
 *                                                                             *
 * HQEd is distributed in the hope that it will be useful,                     *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU General Public License for more details.                                *
 *                                                                             *
 * You should have received a copy of the GNU General Public License           *
 * along with HQEd.  If not, see <http://www.gnu.org/licenses/>.               *
 *                                                                             *
 * Comment: Plik zawierajacy definicje metod klasy obslugujacej okno           *
 * edycji komorek tabeli                                                       *
 *                                                                             *
 */

#include <QMessageBox>
#include <QtGui>
#include <QDialog>
#include <QObject>
#include <QCursor>
#include <QKeyEvent>

#include "../../M/XTT/XTT_AttributeGroups.h"
#include "../../M/XTT/XTT_TableList.h"
#include "../../M/XTT/XTT_Table.h"
#include "Hinter_ui.h"
#include "TableEditor_ui.h"
#include "CellEditor_ui.h"
#include "../Settings_ui.h"
// -----------------------------------------------------------------------------

CellEditor_ui* CellEdit;
// -----------------------------------------------------------------------------

/**
 * Konstruktor kalsy
 * @param QWidget *parent - Wskanik do rodzica
 * @returns Brak zwracanych wartosci
*/
CellEditor_ui::CellEditor_ui(QWidget*)
{
    setupUi(this);
    setWindowIcon(QIcon(":/all_images/images/mainico.bmp")); 
    
    connect(spinBox, SIGNAL(valueChanged(int)), this, SLOT(SpinButtonChange(int)));
    connect(tableWidget, SIGNAL(cellDoubleClicked(int, int)), this, SLOT(TableDoubleClick(int, int)));
    setWindowFlags(Qt::WindowMaximizeButtonHint);
}
// -----------------------------------------------------------------------------

/**
 * Konstruktor kalsy
 * @param int tableIndex - Index wczytywanej tabeli
 * @param QWidget *parent - Wskanik do rodzica
 * @returns Brak zwracanych wartosci
*/
CellEditor_ui::CellEditor_ui(int tableIndex, QWidget*)
{
    setupUi(this);
    setWindowIcon(QIcon(":/all_images/images/mainico.bmp")); 
    
    connect(spinBox, SIGNAL(valueChanged(int)), this, SLOT(SpinButtonChange(int)));
    connect(comboBox, SIGNAL(currentIndexChanged(int)), this, SLOT(TableChange(int))); 
    connect(pushButton, SIGNAL(clicked()), this, SLOT(ClickOK()));
    connect(pushButton_3, SIGNAL(clicked()), this, SLOT(EditTable()));
    connect(toolButton, SIGNAL(clicked()), this, SLOT(MoveDown()));
    connect(toolButton_2, SIGNAL(clicked()), this, SLOT(MoveUp()));
    connect(tableWidget, SIGNAL(cellDoubleClicked(int, int)), this, SLOT(TableDoubleClick(int, int)));
    setWindowFlags(Qt::WindowMaximizeButtonHint);

    ReadTabels();
    comboBox->setCurrentIndex(tableIndex);
}
// -----------------------------------------------------------------------------

/**
 * Metoda wczytujaca nazwy tabel
 * @returns Brak zwracanych wartoci
 */
void CellEditor_ui::ReadTabels(void)
{
     // Wczytywanie informacji o tabelach
     comboBox->clear();

     for(unsigned int i=0;i<TableList->Count();i++)
     {
          QString title = TableList->Table(i)->Title();
          QString desc = TableList->Table(i)->Description();
          QString line = title;
          if(desc != "")
               line += " - [" + desc + "]";
          comboBox->addItem(line);
     }
}
// -----------------------------------------------------------------------------

/**
 * Metoda klasy CellEditor_ui ustawiajaca okienko przy zmianie wybranej tabeli
 * @param int itemIndex - Index katualnie wysietlanej tabeli
 * @returns Brak zwracanych wartoci
*/
void CellEditor_ui::TableChange(int itemIndex)
{
     if(itemIndex == -1)
          return;

     XTT_Table* tbl = TableList->Table(itemIndex);

     // Ustawienie liczby wierszy i kolumn
     tableWidget->setColumnCount((int)tbl->ColCount());
     spinBox->setValue((int)tbl->RowCount());

     // Ustawianie naglowkow kolumn
     QTableWidgetItem** _attr = new QTableWidgetItem*[tbl->ColCount()];
     for(int i=0;i<tbl->ColCount();i++)
     {
          QString aname = AttributeGroups->CreatePath(tbl->ContextAtribute((unsigned int)i)) + tbl->ContextAtribute((unsigned int)i)->Name();
          if(aname == "#NAN")
          {
               QMessageBox::critical(NULL, "XTT Editor", "Missing pointer.", QMessageBox::Ok);
               return;
          }
          // Ustalanie contekstu i dodanie znacznika kontekstu
          QStringList ctx_smbl;
          ctx_smbl << "(?) " << "(+) " << "(-) " << "(->) ";
          aname = ctx_smbl.at(tbl->ColContext(i)-1) + aname;

          _attr[i] = new QTableWidgetItem(aname);
          tableWidget->setHorizontalHeaderItem(i, _attr[i]);
     }

     // Ustawianie tresci komorek
     QStringList ctx, atype;
     ctx << "conditional" << "assert" << "retract" << "action";
     atype << "boolean" << "integer" << "float" << "symbolic" << "enumerative";
     for(unsigned int i=0;i<tbl->RowCount();i++)
     {
          for(unsigned int j=0;j<tbl->Row(i)->Length();j++)
          {
               QTableWidgetItem* _content = new QTableWidgetItem(tbl->Row(i)->Cell(j)->Content());
               XTT_Attribute* attr = tbl->ContextAtribute(j);
               int context = tbl->ColContext(j);
               QString tt = "";
               tt = "Column information:\n\tContext: " + ctx.at(context-1) + "\n\tArgument name: " + attr->Name();
               tt += "\n\tArgument acronym: " + attr->Acronym();
               tt += "\n\tArgument type: " + atype.at(attr->Type()-1);
               tt += "\n\tArgument domain: " + attr->ConstraintsRepresentation();
               tt += "\nCell information:\n\tContent: \"" + tbl->Row(i)->Cell(j)->Content() + "\"";
               _content->setToolTip(tt);

               tableWidget->setItem(i, j, _content);
          }
     }

     // Ustawianie wymiarow kolumn
     for(int j=0;j<tbl->ColCount();j++)
          tableWidget->setColumnWidth((int)j, tbl->ColWidth(j));

     // Ustawianie wymiarow wierszy
     for(unsigned int j=0;j<tbl->RowCount();j++)
          tableWidget->setRowHeight((int)j, tbl->RowHeight(j));
}
// -----------------------------------------------------------------------------

/**
 * Metoda klasy, wywolujaca okienko edycji tabeli, attualnie wybranej w comboBox
 * @returns Brak zwracanych wartoci
*/
void CellEditor_ui::EditTable(void)
{
     if(comboBox->currentIndex() == -1)
     {
          const QString str2 = "There is no table selected.";
          QMessageBox::information(NULL, "XTT Editor", str2, QMessageBox::Ok);
          return;
     }


     delete TableEdit;
     TableEdit = new TableEditor_ui(false, comboBox->currentIndex());
     connect(TableEdit, SIGNAL(onClose()), this, SLOT(EditTableEvent()));
     TableEdit->show();
}
// -----------------------------------------------------------------------------

/**
 * Metoda wywolywana w momencie zatwoerdzenia edycji tabeli
 * @returns Brak zwracanych wartoci
*/
void CellEditor_ui::EditTableEvent(void)
{
     int cc = comboBox->currentIndex();
     ReadTabels();
     comboBox->setCurrentIndex(cc);
}
// -----------------------------------------------------------------------------

/**
 * Metoda klasy CellEditor_ui wywoywana w momencie kliknicia na przycisku
 * OK
 * @returns Brak zwracanych wartoci
*/
void CellEditor_ui::ClickOK(void)
{
     if(comboBox->currentIndex() == -1)
     {
          QMessageBox::information(NULL, "XTT Editor", "There is no table selected.", QMessageBox::Ok);
          return;
     }

     // Zapisanie wskaznika do tabeli
     XTT_Table* tbl = TableList->Table(comboBox->currentIndex());

     // Alokacja pamieci na komorki
     for(;tbl->RowCount() < (unsigned int)tableWidget->rowCount();)
          tbl->AddRow();
     for(;tbl->RowCount() > (unsigned int)tableWidget->rowCount();)
          tbl->DeleteRow(tbl->RowCount()-1);

     // Sprawdzanie poprawnosci wpisanych formul
     for(int i=0;i<tableWidget->rowCount();i++)
           for(int j=0;j<tableWidget->columnCount();j++)
           {
               // Jezeli nie poprawny formula
               if((!tbl->Row(i)->Cell(j)->Analyze(tableWidget->item(i, j)->text())) && (Settings->xttShowErrorCellMessage()))
               {
                    // Jezeli chcemy poprawic
                    if(QMessageBox::warning(NULL, "XTT Editor", tbl->Row(i)->Cell(j)->CellError() + "\nClick \"Retry\" for improve.\nClick \"Ignore\" for continue.", QMessageBox::Retry | QMessageBox::Ignore) == QMessageBox::Retry)
                    {
                         tableWidget->clearSelection();
                         tableWidget->item(i, j)->setSelected(true);
                         return;
                    }
               }
               tbl->Row(i)->Cell(j)->setContent(tableWidget->item(i, j)->text());
           }


     // Ustawianie wymiarow kolumn
     for(int j=0;j<tableWidget->columnCount();j++)
          tbl->setColWidth(j, tableWidget->columnWidth(j));

     // Ustawianie wymiarow wierszy
     for(int j=0;j<tableWidget->rowCount();j++)
          tbl->setRowHeight(j, tableWidget->rowHeight(j));

     // Odswiezenie widoku tabel
     TableList->Refresh(comboBox->currentIndex());

     close();
}
// -----------------------------------------------------------------------------

/**
 * Metoda wywolywana w momencie zminay wartosci w obiekcie spin button
 * @param int rowCount - Nowa wartosc, w tym wypadku liczba kolumn
 * @returns Brak zwracanych wartoci
*/
void CellEditor_ui::SpinButtonChange(int rowCount)
{
     int cc = tableWidget->rowCount();

     // Usuwanie nadmiarowycj komorek
     if(cc > rowCount)
     {
          for(int i=rowCount;i<cc;i++)
               for(int j=0;j<tableWidget->columnCount();j++)
                    delete tableWidget->item(i, j);
     }

     tableWidget->setRowCount(rowCount);

     if(cc >= rowCount)
          return;

     // Jezeli zwiekszamy to musimy zaalokowac miejsce na nowe komorki
     for(int i=cc;i<rowCount;i++)
     {
          for(int j=0;j<tableWidget->columnCount();j++)
          {
               QTableWidgetItem* _content = new QTableWidgetItem("");
               tableWidget->setItem(i, j, _content);
          }
		tableWidget->setRowHeight(i, Settings->xttDefaultRowHeight());
     }


     if(comboBox->currentIndex() == -1)
          return;

     // Wypiasnie zawartosci komorek, jezeli w zakresie
     XTT_Table* tbl = TableList->Table(comboBox->currentIndex());
     if(((unsigned int)rowCount <= tbl->RowCount()) && (rowCount > 0))
     {
          for(unsigned int j=0;j<tbl->Row(rowCount-1)->Length();j++)
          {
               QTableWidgetItem* _content = new QTableWidgetItem(tbl->Row(rowCount-1)->Cell(j)->Content());
               tableWidget->setItem(rowCount-1, j, _content);
			tableWidget->setRowHeight(j, tbl->RowHeight(j));
          }
     }
}
// -----------------------------------------------------------------------------

/**
 * Metoda przesuwajaca aktualny wiersz o jeden w dol
 * @returns Brak zwracanych wartoci
*/
void CellEditor_ui::MoveDown(void)
{
     int currRow = tableWidget->currentRow();
     int currCol = tableWidget->currentColumn();
     int tableIndex = comboBox->currentIndex();

     if(tableIndex == -1)
          return;

     if(TableList->Table(tableIndex)->ExchangeRows(currRow, currRow+1))
     {
          tableWidget->clearSelection();
          TableChange(tableIndex);
          tableWidget->setCurrentCell(currRow+1, currCol);
     }
}
// -----------------------------------------------------------------------------

/**
 * Metoda przesuwajaca aktualny wiersz o jeden w gore
 * @returns Brak zwracanych wartoci
*/
void CellEditor_ui::MoveUp(void)
{
     int currRow = tableWidget->currentRow();
     int currCol = tableWidget->currentColumn();
     int tableIndex = comboBox->currentIndex();

     if(tableIndex == -1)
          return;

     if(TableList->Table(tableIndex)->ExchangeRows(currRow, currRow-1))
     {
          tableWidget->clearSelection();
          TableChange(tableIndex);
          tableWidget->setCurrentCell(currRow-1, currCol);
     }
}
// -----------------------------------------------------------------------------

/**
 * Metoda wywolywana w momncie podwojnego klikniecia na komorke
 * @param int row - Wiersz komorki
 * @param int col - Kolumna komorki
 * @returns Brak zwracanych wartoci
*/
void CellEditor_ui::TableDoubleClick(int row, int col)
{
     delete Hinter;

     // Parametry uruchomienia okienka
     int tableIndex = comboBox->currentIndex();

     if(tableIndex == -1)
          return;

     int context = TableList->Table(tableIndex)->ColContext(col);
     XTT_Attribute* attr = TableList->Table(tableIndex)->ContextAtribute(col);
     XTT_Cell* cell = NULL;
     if(row < (int)TableList->Table(tableIndex)->RowCount())
          cell = TableList->Table(tableIndex)->Row(row)->Cell(col);
     XTT_Table* table = TableList->Table(tableIndex);

     Hinter = new Hinter_ui(context, table, attr, cell, tableWidget->item(row, col));
     Hinter->move(QCursor::pos());
     Hinter->show();
}
// -----------------------------------------------------------------------------

/**
 * Metoa wywolywana w momencie kiedy zostanie nacisniety jakis klawisz
 * @param QKeyEvent* event - Wskanik do obiektu przechwoujacego informacje o 
 * zdarzeniu
 * @returns Brak zwracanych wartoci
*/
void CellEditor_ui::keyPressEvent(QKeyEvent *event)
{
     switch (event->key())
     {
          case Qt::Key_Escape:
               close();
               break;

          case Qt::Key_Return:
               ClickOK();
               break;

          default:
               QWidget::keyPressEvent(event);
     }
}
// -----------------------------------------------------------------------------
