 /**
 *                                                                             *
 * @file ColumnEditor_ui.cpp                                                   *
 * Project name: HeKatE hqed                                                   *
 * $Id: ColumnEditor_ui.cpp,v 1.12 2008-03-03 22:04:32 kinio Exp $              *
 * Author: Krzysztof Kaczor                                                    *
 * Date: 28.04.2007                                                            *
 *                                                                             *
 * Implementation by Krzysztof Kaczor <kinio4444@gmail.com>                    *
 *                                                                             *
 * Copyright (C) 2006-9 by the HeKatE Project                                  *
 *                                                                             *
 * HQEd has been develped within the HeKatE Project,                           *
 * see http://hekate.ia.agh.edu.pl                                             *
 *                                                                             *
 * This file is part of HQEd.                                                  *
 *                                                                             *
 * HQEd is free software: you can redistribute it and/or modify                *
 * it under the terms of the GNU General Public License as published by        *
 * the Free Software Foundation, either version 3 of the License, or           *
 * (at your option) any later version.                                         *
 *                                                                             *
 * HQEd is distributed in the hope that it will be useful,                     *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU General Public License for more details.                                *
 *                                                                             *
 * You should have received a copy of the GNU General Public License           *
 * along with HQEd.  If not, see <http://www.gnu.org/licenses/>.               *
 *                                                                             *
 * Comment: Plik zawierajacy definicje metod klasy obslugujacej okno           *
 * edytora kolumny                                                             *
 *                                                                             *
 */

#include <QMessageBox>
#include <QtGui>
#include <QDialog>
#include <QObject>
#include <QKeyEvent>

#include "SelectGroup_ui.h"
#include "ColumnEditor_ui.h"

#include "../../M/XTT/XTT_Attribute.h"
#include "../../M/XTT/XTT_AttributeList.h"
#include "../../M/XTT/XTT_AttributeGroups.h"
// -----------------------------------------------------------------------------

ColumnEditor_ui* ColumnEdit;
// -----------------------------------------------------------------------------

/**
 * Konstruktor kalsy
 * @param QWidget *parent - Wskanik do rodzica
 * @returns Brak zwracanych wartosci
*/
ColumnEditor_ui::ColumnEditor_ui(QWidget*)
{
    setupUi(this);
    setWindowIcon(QIcon(":/all_images/images/mainico.bmp")); 
    
    setWindowFlags(Qt::WindowMaximizeButtonHint);
}
// -----------------------------------------------------------------------------

/**
 * Konstruktor klasy ColumnEditor_ui
 * @param bool _new - Wartosc okreslajaca czy jest tworzona nowa kolumna
 * @param QString _path - sceizka dostepu do atrybutu
 * @param int _cindex - Wartosc okreslajaca ktory element jest edytowany
 * @param QWidget *parent = 0 - Wskanik do rodzca
 * @returns Brak zwracanch wartosci
*/
ColumnEditor_ui::ColumnEditor_ui(bool _new, QString _path, int _cindex, QWidget*)
{
     setupUi(this);
	setWindowIcon(QIcon(":/all_images/images/mainico.bmp")); 

     connect(pushButton, SIGNAL(clicked()), this, SLOT(NewGroupSelect()));
     connect(pushButton_2, SIGNAL(clicked()), this, SLOT(ClickOk()));
     connect(pushButton_3, SIGNAL(clicked()), this, SLOT(ClickCancel()));
     setWindowFlags(Qt::WindowMaximizeButtonHint);

     fNew = _new;
     fcIndex = _cindex;
     fPath = _path;

     // Kiedy edycja
     if(!_new)
     {
          // Extract attribute name i path
          QStringList list = _path.split(".", QString::SkipEmptyParts);

          if(list.size() == 1)
          {
               ReadAttributes(0);

               // Nazwa atrybutu
               QString aname = list.at(list.size()-1);
               
               // Index atrybutu
               int aindex = AttributeGroups->Group(0)->IndexOfName(aname);
               if(aindex == -1)
               {
                    const QString content = "Critical error. Attribute \"" + aname + "\" not exists in group.";
                    QMessageBox::critical(NULL, "XTT Editor", content, QMessageBox::Ok);
               }
               comboBox->setCurrentIndex(aindex);
          }
          if(list.size() > 1)
          {
               // Nazwa grupy
               QString gname = list.at(list.size()-2);

               // Nazwa atrybutu
               QString aname = list.at(list.size()-1);

               // Index grupy
               int gindex = AttributeGroups->IndexOfName(gname);
               if(gindex == -1)
               {
                    const QString content = "Critical error. Group \"" + gname + "\" not exists.";
                    QMessageBox::critical(NULL, "XTT Editor", content, QMessageBox::Ok);
                    return;
               }

               // Wczytywanie atrybutow
               ReadAttributes(gindex);

               // Index atrybutu
               int aindex = AttributeGroups->Group(gindex)->IndexOfName(aname);
               if(aindex == -1)
               {
                    const QString content = "Critical error. Attribute \"" + aname + "\" not exists in group.";
                    QMessageBox::critical(NULL, "XTT Editor", content, QMessageBox::Ok);
               }
               comboBox->setCurrentIndex(aindex);
          }

          comboBox_2->setCurrentIndex(fcIndex);
     }
     // Kiedy nowy
     if(_new)
     {
          ReadAttributes(0);
     }
}
// -----------------------------------------------------------------------------

/**
 * Metoda klasy ColumnEditor_ui wywoywana w momencie kliknicia na przycisku OK
 * @returns Brak zwracanych wartoci
*/
void ColumnEditor_ui::ClickCancel(void)
{
     close();
}
// -----------------------------------------------------------------------------

/**
 * Metoda klasy ColumnEditor_ui wywoywana w momencie kliknicia na przycisku
 * OK
 * @returns Brak zwracanych wartoci
*/
void ColumnEditor_ui::ClickOk(void)
{
     // Sprawdzanie czy wybrany atrybut moze byc uzyty w danym kontekscie
     XTT_Attribute* attr = AttributeGroups->Attribute(pushButton->text() + "." + comboBox->currentText());

     if(attr == NULL)
     {
          const QString content = "Critical error. Selected attribute not exists.";
          QMessageBox::critical(NULL, "XTT Editor", content, QMessageBox::Ok);
          return;
     }

     int context = comboBox_2->currentIndex();

     // Sprawdzanie czy wszystkie dane zostaly podane
     if(comboBox->currentIndex() == -1)
     {
          const QString content = "There is no attribute selected.";
          QMessageBox::critical(NULL, "XTT Editor", content, QMessageBox::Ok);
          return;
     }
     if(comboBox_2->currentIndex() == -1)
     {
          const QString content = "There is no context selected.";
          QMessageBox::critical(NULL, "XTT Editor", content, QMessageBox::Ok);
          return;
     }

     // Sprawdzanie czy w wybranym kontekscie mozna uzyc danego atrybutu

     // Jezeli kontekst jest conditional i atrybut input
     if((context != 0) && (attr->RelationsType() == 0))
     {
          const QString content = "This is a input attribue which can be use only in conditional context.";
          QMessageBox::information(NULL, "XTT Editor", content, QMessageBox::Ok);
          return;
     }

     // Jezeli kontekst jest action a atrybut middle
     // if((context == 3) && (attr->RelationsType() == 1))
     // {
          // const QString content = "This is a middle attribue which can't be use in action context.";
          // QMessageBox::information(NULL, "XTT Editor", content, QMessageBox::Ok);
          // return;
     // }
     /*
     // Jezeli kontekst nie jest action a atrybut typu output
     if((context != 3) && (attr->RelationsType() == 2))
     {
          const QString content = "This is a output attribue whitch can be use only in decision context.";
          QMessageBox::information(NULL, "XTT Editor", content, QMessageBox::Ok);
          return;
     } */

     groupName = pushButton->text() + ".";
     if(groupName == "Global.")
          groupName = "";
     attrName = comboBox->currentText();
     ctxIndex = comboBox_2->currentIndex()+1;

     emit onClose();
     close();
}
// -----------------------------------------------------------------------------

/**
 * Metodoa klasy ColumnEditor_ui, wywoywana w momencie kliknicia na
 * przycisku pozwalajacym na wybranie grupy atrybutu
 * @returns Brak zwracanych wartoci
*/
void ColumnEditor_ui::NewGroupSelect(void)
{
     delete SelGroup;
     SelGroup = new SelectGroup_ui;
     connect(SelGroup, SIGNAL(ResultOK()), this, SLOT(NewGroupChange()));
     SelGroup->show();
}
// -----------------------------------------------------------------------------

/**
 * Metodoa klasy ColumnEditor_ui, wywoywana w momencie zmiany grupy
 * atrybutu
 * @returns Brak zwracanych wartoci
*/
void ColumnEditor_ui::NewGroupChange(void)
{
     int pindex = AttributeGroups->IndexOfName(SelGroup->Result());
     if(pindex == -1)
          return;
          
     ReadAttributes(pindex);
}
// -----------------------------------------------------------------------------

/**
 * Metoda klasy wczytujaca argumenty z danej grupy
 * @param int gindex - Index grupy z ktorej chcemy wczytac argumenty
 * @returns Brak zwracanych wartoci
*/
void ColumnEditor_ui::ReadAttributes(int gindex)
{
     // Sprawdzanie czy index jest poprawny
     if((int)gindex == -1)
     {
          const QString content = "There is no group selected.";
          QMessageBox::information(NULL, "XTT Editor", content, QMessageBox::Ok);
          return;
     }

     pushButton->setText(AttributeGroups->CreatePath(AttributeGroups->Group(gindex)->Name()));

     comboBox->clear();
     for(unsigned int i=0;i<AttributeGroups->Group(gindex)->Count();i++)
          comboBox->addItem(AttributeGroups->Group(gindex)->Item(i)->Name());
}
// -----------------------------------------------------------------------------

/**
 * Metoa wywolywana w momencie kiedy zostanie nacisniety jakis klawisz
 * @param QKeyEvent* event - Wskanik do obiektu przechwoujacego informacje o 
 * zdarzeniu
 * @returns Brak zwracanych wartoci
*/
void ColumnEditor_ui::keyPressEvent(QKeyEvent *event)
{
     switch (event->key())
     {
          case Qt::Key_Escape:
               ClickCancel();
               break;

          case Qt::Key_Return:
               ClickOk();
               break;

          default:
               QWidget::keyPressEvent(event);
     }
}
// -----------------------------------------------------------------------------
