 /**
 *                                                                             *
 * @file ConnectionEditor_ui.cpp                                               *
 * Project name: HeKatE hqed                                                   *
 * $Id: ConnectionEditor_ui.cpp,v 1.12 2008-03-03 22:04:32 kinio Exp $          *
 * Author: Krzysztof Kaczor                                                    *
 * Date: 28.04.2007                                                            *
 *                                                                             *
 * Implementation by Krzysztof Kaczor <kinio4444@gmail.com>                    *
 *                                                                             *
 * Copyright (C) 2006-9 by the HeKatE Project                                  *
 *                                                                             *
 * HQEd has been develped within the HeKatE Project,                           *
 * see http://hekate.ia.agh.edu.pl                                             *
 *                                                                             *
 * This file is part of HQEd.                                                  *
 *                                                                             *
 * HQEd is free software: you can redistribute it and/or modify                *
 * it under the terms of the GNU General Public License as published by        *
 * the Free Software Foundation, either version 3 of the License, or           *
 * (at your option) any later version.                                         *
 *                                                                             *
 * HQEd is distributed in the hope that it will be useful,                     *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU General Public License for more details.                                *
 *                                                                             *
 * You should have received a copy of the GNU General Public License           *
 * along with HQEd.  If not, see <http://www.gnu.org/licenses/>.               *
 *                                                                             *
 * Comment: Plik zawierajacy definicje metod klasy obslugujacej okno           *
 * edycji polaczenia                                                           *
 *                                                                             *
 */

#include <QMessageBox>
#include <QtGui>
#include <QDialog>
#include <QObject>
#include <QKeyEvent>

#include "../../M/XTT/XTT_ConnectionsList.h"
#include "../../M/XTT/XTT_AttributeGroups.h"
#include "../../M/XTT/XTT_TableList.h"
#include "../../M/XTT/XTT_Table.h"
#include "MyGraphicsScene.h"
#include "TableEditor_ui.h"
#include "ConnectionEditor_ui.h"
#include "../MainWin_ui.h"
// -----------------------------------------------------------------------------

ConnectionEditor_ui* ConnEdit;
// -----------------------------------------------------------------------------

/**
 * Konstruktor kalsy
 * @param QWidget *parent - Wskanik do rodzica
 * @returns Brak zwracanych wartosci
*/
ConnectionEditor_ui::ConnectionEditor_ui(QWidget*)
{
    setupUi(this);
    setWindowIcon(QIcon(":/all_images/images/mainico.bmp")); 
    
    connect(pushButton, SIGNAL(clicked()), this, SLOT(ClickOK()));
    setWindowFlags(Qt::WindowMaximizeButtonHint);
}
// -----------------------------------------------------------------------------

/**
 * Konstruktor klasy przyjmujaca parametry ustawiajace okienko
 * @param bool _new - Okresla czy tworzymy nowe polaczenie
 * @param int _index - Okresla index edytowanego polaczenia
 * @param QWidget *parent - Wskanik do rodzica
 * @returns Brak zwracanych wartosci
*/
ConnectionEditor_ui::ConnectionEditor_ui(bool _new, int _index, QWidget*)
{
     setupUi(this);
	setWindowIcon(QIcon(":/all_images/images/mainico.bmp")); 
	
     connect(pushButton, SIGNAL(clicked()), this, SLOT(ClickOK()));
     connect(comboBox, SIGNAL(currentIndexChanged(int)), this, SLOT(ComboBoxChange(int)));
     connect(comboBox_3, SIGNAL(currentIndexChanged(int)), this, SLOT(ComboBoxChange_3(int)));
     connect(comboBox_5, SIGNAL(currentIndexChanged(int)), this, SLOT(ComboBoxChange_5(int)));
     connect(comboBox_6, SIGNAL(currentIndexChanged(int)), this, SLOT(ComboBoxChange_6(int)));
     setWindowFlags(Qt::WindowMaximizeButtonHint);

     fNew = _new;
     fIndex = _index;

     groupBox_4->setDisabled(fNew);

     // Wczytywanie nazw tabel do comboboxow
     comboBox->addItems(TableList->TablesTitles());
     comboBox_3->addItems(TableList->TablesTitles());
     comboBox_5->addItems(TableList->TablesTitles());

     if(!fNew)
          setUpWindow();
     if(fNew)
          comboBox->setCurrentIndex(_index);
}
// -----------------------------------------------------------------------------

/**
 * Metoda ustawiajaca parametry okienka
 * @returns Brak zwracanych wartoci
*/
void ConnectionEditor_ui::setUpWindow(void)
{
     XTT_Connections* conn = ConnectionsList->Connection(fIndex);
     if(conn == NULL)
     {
          QMessageBox::critical(NULL, "XTT Editor", "Invalid connection index.", QMessageBox::Ok);
          return;
     }

     int tid = conn->SrcTable();
     int rid = conn->SrcRow();

     // Wyszukiwanie indexow
     int tindex = TableList->IndexOfID(tid);
     if(tindex == -1)
     {
          QMessageBox::critical(NULL, "XTT Editor", "Invalid table ID.", QMessageBox::Ok);
          return;
     }
     int rindex = -1;
     for(unsigned int i=0;i<TableList->Table(tindex)->RowCount();i++)
          if(TableList->Table(tindex)->Row(i)->RowId() == (unsigned int)rid)
          {
               rindex = i;
               break;
          }
     if(rindex == -1)
     {
          QMessageBox::critical(NULL, "XTT Editor", "Invalid row ID.", QMessageBox::Ok);
          return;
     }

     comboBox_5->setCurrentIndex(tindex);
     comboBox_6->setCurrentIndex(rindex);
}
// -----------------------------------------------------------------------------

/**
 * Metoda wywolywana w momencie zmiany zawartosci pola wyboru tabeli
 * @param int new_index - nowy index wartosci
 * @returns Brak zwracanych wartoci
*/
void ConnectionEditor_ui::ComboBoxChange(int new_index)
{
     comboBox_2->clear();
     
     // Pobranie indexu tabeli
     int tindex = new_index;
     if(tindex == -1)
          return;

     // Wczytywanie wierszy
     for(unsigned int i=0;i<TableList->Table(tindex)->RowCount();i++)
          comboBox_2->addItem(QString::number(i+1) + ". Row ID = " + QString::number(TableList->Table(tindex)->Row(i)->RowId()));
}
// -----------------------------------------------------------------------------

/**
 * Metoda wywolywana w momencie zmiany zawartosci pola wyboru tabeli
 * @param int new_index - nowy index wartosci
 * @returns Brak zwracanych wartoci
*/
void ConnectionEditor_ui::ComboBoxChange_3(int new_index)
{
     comboBox_4->clear();
     
     // Pobranie indexu tabeli
     int tindex = new_index;
     if(tindex == -1)
          return;

     // Wczytywanie wierszy
     for(unsigned int i=0;i<TableList->Table(tindex)->RowCount();i++)
          comboBox_4->addItem(QString::number(i+1) + ". Row ID = " + QString::number(TableList->Table(tindex)->Row(i)->RowId()));
}
// -----------------------------------------------------------------------------

/**
 * Metoda wywolywana w momencie zmiany zawartosci pola wyboru tabeli
 * @param int new_index - nowy index wartosci
 * @returns Brak zwracanych wartoci
*/
void ConnectionEditor_ui::ComboBoxChange_5(int new_index)
{
     if(fNew)
          return;
          
     comboBox_6->clear();
     
     // Pobranie indexu tabeli
     int tindex = new_index;
     if(tindex == -1)
          return;

     // Wczytywanie wierszy
     for(unsigned int i=0;i<TableList->Table(tindex)->RowCount();i++)
     {
          QString comment = " - CONNECTED";
          int tid = TableList->Table(tindex)->TableId();
          int rid = TableList->Table(tindex)->Row(i)->RowId();
          if(ConnectionsList->IndexOfOutConnections(tid, rid) == -1)
               comment = " - NOT CONNECTED";

          comboBox_6->addItem(QString::number(i+1) + ". Row ID = " + QString::number(TableList->Table(tindex)->Row(i)->RowId()) + comment);
     }
}
// -----------------------------------------------------------------------------

/**
 * Metoda wywolywana w momencie zmiany zawartosci pola wyboru tabeli
 * @param int new_index - nowy index wartosci
 * @returns Brak zwracanych wartoci
*/
void ConnectionEditor_ui::ComboBoxChange_6(int new_index)
{
     if((fNew) || (new_index == -1))
          return;

     int cti = comboBox_5->currentIndex();
     int cri = new_index;

     // Odblokowanie elementow
     pushButton->setEnabled(true);
     groupBox->setEnabled(true);
     groupBox_2->setEnabled(true);
     groupBox_3->setEnabled(true);

     // Jezeli nie poprawne indexy to blokujemy elemnty
     if((cti == -1) || (cri == -1))
     {
          pushButton->setDisabled(true);
          groupBox->setDisabled(true);
          groupBox_2->setDisabled(true);
          groupBox_3->setDisabled(true);
     }

     // Pobranie i elementow
     int tid = TableList->Table(cti)->TableId();
     int rid = TableList->Table(cti)->Row(cri)->RowId();
     int conn_index = ConnectionsList->IndexOfOutConnections(tid, rid);

     // Sprawdzanie czy dane polaczenie istnieje
     if(conn_index == -1)
     {
          pushButton->setDisabled(true);
          groupBox->setDisabled(true);
          groupBox_2->setDisabled(true);
          groupBox_3->setDisabled(true);
          return;
     }

     // Wyszukiwanie indexow elemetow polaczenia
     int dtid = ConnectionsList->Connection(conn_index)->DesTable();
     int drid = ConnectionsList->Connection(conn_index)->DesRow();
     int dti = TableList->IndexOfID(dtid);
     int dri = -1;
     if(dti == -1)
     {
          QMessageBox::critical(NULL, "XTT Editor", "Incorrect table ID.", QMessageBox::Ok);
          return;
     }
     // Wyszukiwanie indexu wiersza
     for(unsigned int i=0;i<TableList->Table(dti)->RowCount();i++)
          if((int)TableList->Table(dti)->Row(i)->RowId() == drid)
          {
               dri = i;
               break;
          }
     if(dri == -1)
     {
          QMessageBox::critical(NULL, "XTT Editor", "Incorrect row ID.", QMessageBox::Ok);
          return;
     }

     // Wpisujemy wsrtosci w pola okienka
     comboBox->setCurrentIndex(cti);
     comboBox_2->setCurrentIndex(cri);
     comboBox_3->setCurrentIndex(dti);
     comboBox_4->setCurrentIndex(dri);
     lineEdit->setText(ConnectionsList->Connection(conn_index)->Label());
     lineEdit_2->setText(ConnectionsList->Connection(conn_index)->Description());
     checkBox->setChecked(ConnectionsList->Connection(conn_index)->IsCut());
}
// -----------------------------------------------------------------------------

/**
 * Metoda klasy ConnectionEditor_ui wywoywana w momencie kliknicia na przycisku
 * OK
 * @returns Brak zwracanych wartoci
*/
void ConnectionEditor_ui::ClickOK(void)
{
     int stindex = comboBox->currentIndex();
     int srindex = comboBox_2->currentIndex();
     int dtindex = comboBox_3->currentIndex();
     int drindex = comboBox_4->currentIndex();

     int ctindex = comboBox_5->currentIndex();
     int crindex = comboBox_6->currentIndex();

     if(stindex == -1)
     {
          QMessageBox::information(NULL, "XTT Editor", "There is no source table selected.", QMessageBox::Ok);
          return;
     }

     if(dtindex == -1)
     {
          QMessageBox::information(NULL, "XTT Editor", "There is no destination table selected.", QMessageBox::Ok);
          return;
     }

     if(srindex == -1)
     {
          QMessageBox::information(NULL, "XTT Editor", "There is no source row selected.", QMessageBox::Ok);
          return;
     }

     if(drindex == -1)
     {
          QMessageBox::information(NULL, "XTT Editor", "There is no destination row selected.", QMessageBox::Ok);
          return;
     }

     // Sprawdzanie czy polaczenie nie wychodz z tabeli koncowej
     if(TableList->Table(stindex)->TableType() == 2)
     {
          QMessageBox::information(NULL, "XTT Editor", "Selected source table is halt table whitch can't have connection outside.", QMessageBox::Ok);
          return;
     }

     // Sprawdzanie czy polaczenie nie przychodzi do tabeli poczatkowej
     if(TableList->Table(dtindex)->TableType() == 0)
     {
          QMessageBox::information(NULL, "XTT Editor", "Selected destination table is initial table whitch can't have connection inside.", QMessageBox::Ok);
          return;
     }

     // Sprawdzanie czy nie istnieje juz polaczenie z tego miejsca
     int stid = TableList->Table(stindex)->TableId();
     int srid = TableList->Table(stindex)->Row(srindex)->RowId();
     int dtid = TableList->Table(dtindex)->TableId();
     int drid = TableList->Table(dtindex)->Row(drindex)->RowId();
     if(((fNew) && (ConnectionsList->IndexOfOutConnections(stid, srid) > -1)) ||
       ((!fNew) && (ConnectionsList->IndexOfOutConnections(stid, srid) > -1) && ((stindex != ctindex) || (srindex != crindex))))
     {
          QMessageBox::information(NULL, "XTT Editor", "Connection from this row is already exists. You can't create it.", QMessageBox::Ok);
          return;
     }

     // Zapisanie ustawien
     if(fNew)
     {
          ConnectionsList->Add();
          ConnectionsList->Connection(ConnectionsList->Count()-1)->setCut(checkBox->isChecked());
          ConnectionsList->Connection(ConnectionsList->Count()-1)->setLabel(lineEdit->text());
          ConnectionsList->Connection(ConnectionsList->Count()-1)->setDescription(lineEdit_2->text());
          ConnectionsList->Connection(ConnectionsList->Count()-1)->setSrcTable(stid);
          ConnectionsList->Connection(ConnectionsList->Count()-1)->setDesTable(dtid);
          ConnectionsList->Connection(ConnectionsList->Count()-1)->setSrcRow(srid);
          ConnectionsList->Connection(ConnectionsList->Count()-1)->setDesRow(drid);
		
		// Odswiezenie widoku polaczenia
		ConnectionsList->Connection(ConnectionsList->Count()-1)->Repaint();
		MainWin->xtt_scene->update();
     }

     if(!fNew)
     {
          int ctid = TableList->Table(ctindex)->TableId();
          int crid = TableList->Table(ctindex)->Row(crindex)->RowId();
          int conn_index =  ConnectionsList->IndexOfOutConnections(ctid, crid);

          if(conn_index == -1)
          {
               QMessageBox::critical(NULL, "XTT Editor", "Selected connection not exists.", QMessageBox::Ok);
               return;
          }

          ConnectionsList->Connection(conn_index)->setCut(checkBox->isChecked());
          ConnectionsList->Connection(conn_index)->setLabel(lineEdit->text());
          ConnectionsList->Connection(conn_index)->setDescription(lineEdit_2->text());
          ConnectionsList->Connection(conn_index)->setSrcTable(stid);
          ConnectionsList->Connection(conn_index)->setDesTable(dtid);
          ConnectionsList->Connection(conn_index)->setSrcRow(srid);
          ConnectionsList->Connection(conn_index)->setDesRow(drid);
		
		// Odswiezenie widoku polaczenia
		ConnectionsList->Connection(conn_index)->Repaint();
		MainWin->xtt_scene->update();
     }

     emit OnClose();
     close();
}
// -----------------------------------------------------------------------------

/**
 * Metoa wywolywana w momencie kiedy zostanie nacisniety jakis klawisz
 * @param QKeyEvent* event - Wskanik do obiektu przechwoujacego informacje o 
 * zdarzeniu
 * @returns Brak zwracanych wartoci
*/
void ConnectionEditor_ui::keyPressEvent(QKeyEvent *event)
{
     switch (event->key())
     {
          case Qt::Key_Escape:
               close();
               break;

          case Qt::Key_Return:
               ClickOK();
               break;

          default:
               QWidget::keyPressEvent(event);
     }
}
// -----------------------------------------------------------------------------
