 /**
 *                                                                             *
 * @file DeleteTable_ui.cpp                                                    *
 * Project name: HeKatE hqed                                                   *
 * $Id: DeleteTable_ui.cpp,v 1.12 2008-03-03 22:04:32 kinio Exp $               *
 * Author: Krzysztof Kaczor                                                    *
 * Date: 28.04.2007                                                            *
 *                                                                             *
 * Implementation by Krzysztof Kaczor <kinio4444@gmail.com>                    *
 *                                                                             *
 * Copyright (C) 2006-9 by the HeKatE Project                                  *
 *                                                                             *
 * HQEd has been develped within the HeKatE Project,                           *
 * see http://hekate.ia.agh.edu.pl                                             *
 *                                                                             *
 * This file is part of HQEd.                                                  *
 *                                                                             *
 * HQEd is free software: you can redistribute it and/or modify                *
 * it under the terms of the GNU General Public License as published by        *
 * the Free Software Foundation, either version 3 of the License, or           *
 * (at your option) any later version.                                         *
 *                                                                             *
 * HQEd is distributed in the hope that it will be useful,                     *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU General Public License for more details.                                *
 *                                                                             *
 * You should have received a copy of the GNU General Public License           *
 * along with HQEd.  If not, see <http://www.gnu.org/licenses/>.               *
 *                                                                             *
 * Comment: Plik zawierajacy definicje metod klasy obslugujacej okno           *
 * usuwania tabel                                                              *
 *                                                                             *
 */

#include <QMessageBox>
#include <QtGui>
#include <QDialog>
#include <QObject>
#include <QKeyEvent>

#include "../../M/XTT/XTT_TableList.h"
#include "../../M/XTT/XTT_Table.h"
#include "DeleteTable_ui.h"
// -----------------------------------------------------------------------------

DeleteTable_ui* DelTable;
// -----------------------------------------------------------------------------

/**
 * Konstruktor kalsy
 * @param QWidget *parent - Wskanik do rodzica
 * @returns Brak zwracanych wartosci
*/
DeleteTable_ui::DeleteTable_ui(QWidget*)
{
    setupUi(this);
    setWindowIcon(QIcon(":/all_images/images/mainico.bmp")); 
    
    connect(pushButton, SIGNAL(clicked()), this, SLOT(ClickDelete()));
    setWindowFlags(Qt::WindowMaximizeButtonHint);
    
    SetWindow();
}
// -----------------------------------------------------------------------------

/**
 * Konstruktor kalsy
 * @param QWidget *parent - Wskanik do rodzica
 * @param int tableindex - Index tabeli ktora chcemy usunac
 * @returns Brak zwracanych wartosci
*/
DeleteTable_ui::DeleteTable_ui(int tableindex, QWidget*)
{
    setupUi(this);
    setWindowIcon(QIcon(":/all_images/images/mainico.bmp")); 
    
    connect(pushButton, SIGNAL(clicked()), this, SLOT(ClickDelete()));
    setWindowFlags(Qt::WindowMaximizeButtonHint);
    
    SetWindow();
    listWidget->setCurrentRow(tableindex);
}
// -----------------------------------------------------------------------------

/**
 * Metoda klasy DeleteTable_ui ustawiajaca okienko
 * @returns Brak zwracanych wartoci
*/
void DeleteTable_ui::SetWindow(void)
{
     // Wczytywanie informacji o tabelach
     listWidget->clear();

     for(unsigned int i=0;i<TableList->Count();i++)
     {
          QString title = TableList->Table(i)->Title();
          QString desc = TableList->Table(i)->Description();
          QString line = title;
          if(desc != "")
               line += " - [" + desc + "]";
          listWidget->addItem(line);
     }
}
// -----------------------------------------------------------------------------

/**
 * Metoda klasy DeleteTable_ui wywoywana w momencie kliknicia na przycisku
 * Delete
 * @returns Brak zwracanych wartoci
*/
void DeleteTable_ui::ClickDelete(void)
{
     int row = listWidget->currentRow();

     // Jezeli nic nie wybrane
     if(row == -1)
          return;

     // Potwierdzenie usuniecia
     if(QMessageBox::question(NULL, "XTT Editor", "Do you really want to remove selected table?", QMessageBox::Yes | QMessageBox::No) == QMessageBox::No) 
          return;

     // Usuwanie
     TableList->Delete(row);

     // Odswiezenie okienka
     SetWindow();
}
// -----------------------------------------------------------------------------

/**
 * Metoa wywolywana w momencie kiedy zostanie nacisniety jakis klawisz
 * @param QKeyEvent* event - Wskanik do obiektu przechwoujacego informacje o 
 * zdarzeniu
 * @returns Brak zwracanych wartoci
*/
void DeleteTable_ui::keyPressEvent(QKeyEvent *event)
{
     switch (event->key())
     {
          case Qt::Key_Escape:
               close();
               break;

          case Qt::Key_Return:
               ClickDelete();
               break;

          default:
               QWidget::keyPressEvent(event);
     }
}
// -----------------------------------------------------------------------------
