 /**
 *                                                                             *
 * @file ErrorsListWidget.cpp                                                  *
 * Project name: HeKatE hqed                                                   *
 * $Id: ErrorsListWidget.cpp,v 1.12 2008-03-03 22:04:32 kinio Exp $             *
 * Author: Krzysztof Kaczor                                                    *
 * Date: 09.11.2007                                                            *
 *                                                                             *
 * Implementation by Krzysztof Kaczor <kinio4444@gmail.com>                    *
 *                                                                             *
 * Copyright (C) 2006-9 by the HeKatE Project                                  *
 *                                                                             *
 * HQEd has been develped within the HeKatE Project,                           *
 * see http://hekate.ia.agh.edu.pl                                             *
 *                                                                             *
 * This file is part of HQEd.                                                  *
 *                                                                             *
 * HQEd is free software: you can redistribute it and/or modify                *
 * it under the terms of the GNU General Public License as published by        *
 * the Free Software Foundation, either version 3 of the License, or           *
 * (at your option) any later version.                                         *
 *                                                                             *
 * HQEd is distributed in the hope that it will be useful,                     *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU General Public License for more details.                                *
 *                                                                             *
 * You should have received a copy of the GNU General Public License           *
 * along with HQEd.  If not, see <http://www.gnu.org/licenses/>.               *
 *                                                                             *
 * Comment: Plik zwierajacy definicje klasy bedacej kontrolka, lista bledow    *
 * w komorkach                                                                 *
 *                                                                             *
 */
 
#include <QList>
#include <QWidget>
#include <QString>
#include <QListWidget>
#include <QObject>
#include <QtGui>
#include <QMessageBox>
#include <QDialog>

#include "../../M/XTT/XTT_TableList.h"
#include "../../M/XTT/XTT_Table.h"
#include "../../M/XTT/XTT_Row.h"
#include "../../M/XTT/XTT_Cell.h"
#include "ErrorsListWidget.h"
//---------------------------------------------------------------------------

/**
 * Konstruktor klasy
 * @param QWidget *parent - Wskanik do rodzica
 * @returns Brak zwracanych wartosci
*/
ErrorsListWidget::ErrorsListWidget(QWidget* parent)
     :QListWidget(parent)
{
	connect(this, SIGNAL(currentRowChanged(int)), this, SLOT(onRowChange(int)));
	connect(this, SIGNAL(itemClicked(QListWidgetItem*)), this, SLOT(itemClick(QListWidgetItem*)));

	icons[0] = ":/all_images/images/critical.bmp";
	icons[1] = ":/all_images/images/warning.bmp";
	icons[2] = ":/all_images/images/information.bmp";
	msgTypes[0] = "Error: ";
	msgTypes[1] = "Warning: ";
	msgTypes[2] = "Information: ";

	// Dodanie informacji o liczbie bledow
	int n = fTablesIDs.size();
	if(n == 0)
	{
		addItem(msgTypes[2] + tr("No errors found."));
		item(count()-1)->setIcon(QIcon(icons[2]));
	}
}
// -----------------------------------------------------------------------------

/**
 * Metoda dodajaca element na liste
 * @param int _tid - Identyfikator tabeli
 * @param int _cid - Identyfikator komorki
 * @param QString _err - Tresc bledu 
 * @param int msgType - typ wiadomosci 0-error 1-warning 2-information
 * @returns Brak zwracanych wartosci
*/
void ErrorsListWidget::add(int _tid, int _cid, QString _err, int _msgType)
{
	if((_msgType < 0) || (_msgType > 2))
		return;
	
	// Wyszukanie nazwy tabeli
	int tindex = TableList->IndexOfID(_tid);
	if(tindex == -1)
		return;
		
	// Wyszukanie numeru wiersza i kolumny gdzie znajduje sie komorka
	int rindex = -1, cindex = -1;
	for(unsigned int r=0;r<TableList->Table(tindex)->RowCount();r++)
		if(TableList->Table(tindex)->Row(r)->indexOfCellID(_cid) > -1)
		{	
			rindex = (int)r;
			cindex = TableList->Table(tindex)->Row(r)->indexOfCellID(_cid);
			break;
		}
	if((rindex == -1) || (cindex == -1))
		return;
		
	// Usuwanie ostatniego obiektu ktorym jest info o liczbie bledow
	delete takeItem(count()-1);
		
	QString tname = TableList->Table(tindex)->Title();
	
	fTablesIDs.append(_tid);
	fCellsIDs.append(_cid);
	fCellsErrors.append(_err);
	fCells.append(TableList->Table(tindex)->Row(rindex)->Cell(cindex));
	
	addItem(msgTypes[_msgType] + "in table \'" + tname + "\' row " + QString::number(rindex+1) + ", col " + QString::number(cindex+1) + ": " + _err);
	item(count()-1)->setIcon(QIcon(icons[_msgType]));
	
	// Dodanie informacji o liczbie bledow
	int n = fTablesIDs.size();
	addItem(msgTypes[1] + tr("%n error(s) found.", "", n));
	item(count()-1)->setIcon(QIcon(icons[1]));
	QFont ifont = item(count()-1)->font();
	ifont.setBold(true);
	item(count()-1)->setFont(ifont);
	scrollToItem(item(count()-1));
}
// -----------------------------------------------------------------------------

/**
 * Metoda usuwajaca element
 * @param int _cid - Identyfikator komorki
 * @returns Brak zwracanych wartosci
*/
void ErrorsListWidget::deleteItem(int _tid, int _cid)
{
	int _index = -1;
	int _from = 0;
	int _indexof = -1;
	
	do
	{
		_indexof = fCellsIDs.indexOf(_cid, _from);
		if((_indexof > -1) && (fTablesIDs.at(_indexof) == _tid))
			_index = _indexof;
		_from = _indexof+1;
	}
	while((_index == -1) && (_indexof > -1));
	
	
	if(_index == -1)
		return;
	
	// Usuwanie z list
	fCells.removeAt(_index);
	fTablesIDs.removeAt(_index);
	fCellsIDs.removeAt(_index);
	fCellsErrors.removeAt(_index);
	
	// Usuwanie z obiektu
	delete takeItem(_index);
	
	// Usuwanie ostatniego obiektu ktorym jest info o liczbie bledow
	delete takeItem(count()-1);
	
	// Dodanie informacji o liczbie bledow
	int n = fTablesIDs.size();
	if(n == 0)
	{
		addItem(msgTypes[2] + tr("No errors found."));
		item(count()-1)->setIcon(QIcon(icons[2]));
	}
	if(n > 0)
	{
		addItem(msgTypes[1] + tr("%n error(s) found.", "", n));
		item(count()-1)->setIcon(QIcon(icons[1]));
	}
}
// -----------------------------------------------------------------------------

/**
 * Metoda usuwajaca element
 * @param int _cid - Identyfikator komorki
 * @returns Brak zwracanych wartosci
*/
void ErrorsListWidget::deleteAll(void)
{
	clear();
	addItem(msgTypes[2] + tr("No errors found."));
	item(count()-1)->setIcon(QIcon(icons[2]));
}
// -----------------------------------------------------------------------------

/**
 * Metoda wywolywana w momencie zmiany zaznaczenia
 * @param int _newRow - Index nowego wiersza
 * @returns Brak zwracanych wartosci
*/
void ErrorsListWidget::onRowChange(int _newRow)
{	
	int currSize = fCells.size();
	
	if((_newRow < 0) || (_newRow >= currSize))
		return;
	
	for(int i=0;i<fCells.size();i++)
		fCells.at(i)->setSelected(false);
		
	if(_newRow == currSize)
		return;
		
	fCells.at(_newRow)->setSelected();
}
// -----------------------------------------------------------------------------

/**
 * Metoda wywolywana w momencie klikniecia elementu
 * @param QListWidgetItem* itm - Wskaznik do kliknietego elementu
 * @returns Brak zwracanych wartosci
*/
void ErrorsListWidget::itemClick(QListWidgetItem* itm)
{
	onRowChange(row(itm));
}
// -----------------------------------------------------------------------------
