 /**
 *                                                                             *
 * @file ExecuteMessagesListWidget.cpp                                         *
 * Project name: HeKatE hqed                                                   *
 * $Id: ExecuteMessagesListWidget.cpp,v 1.12 2008-03-03 22:04:32 kinio Exp $    *
 * Author: Krzysztof Kaczor                                                    *
 * Date: 27.11.2007                                                            *
 *                                                                             *
 * Implementation by Krzysztof Kaczor <kinio4444@gmail.com>                    *
 *                                                                             *
 * Copyright (C) 2006-9 by the HeKatE Project                                  *
 *                                                                             *
 * HQEd has been develped within the HeKatE Project,                           *
 * see http://hekate.ia.agh.edu.pl                                             *
 *                                                                             *
 * This file is part of HQEd.                                                  *
 *                                                                             *
 * HQEd is free software: you can redistribute it and/or modify                *
 * it under the terms of the GNU General Public License as published by        *
 * the Free Software Foundation, either version 3 of the License, or           *
 * (at your option) any later version.                                         *
 *                                                                             *
 * HQEd is distributed in the hope that it will be useful,                     *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU General Public License for more details.                                *
 *                                                                             *
 * You should have received a copy of the GNU General Public License           *
 * along with HQEd.  If not, see <http://www.gnu.org/licenses/>.               *
 *                                                                             *
 * Comment: Plik zwierajacy definicje klasy bedacej kontrolka, lista           *
 * komuniktw podczas uruchomienie modelu                                      *
 *                                                                             *
 */
 
#include <QList>
#include <QWidget>
#include <QString>
#include <QListWidget>
#include <QObject>
#include <QtGui>
#include <QMessageBox>
#include <QDialog>

#include "../../M/XTT/XTT_TableList.h"
#include "../../M/XTT/XTT_Table.h"
#include "../../M/XTT/XTT_Row.h"
#include "../../M/XTT/XTT_Cell.h"
#include "../../V/XTT/GTable.h"
#include "ExecuteMessagesListWidget.h"
//---------------------------------------------------------------------------

/**
 * Konstruktor klasy
 * @param QWidget *parent - Wskanik do rodzica
 * @returns Brak zwracanych wartosci
*/
ExecMsgListWidget::ExecMsgListWidget(QWidget* parent)
     :QListWidget(parent)
{
	connect(this, SIGNAL(currentRowChanged(int)), this, SLOT(onRowChange(int)));
	connect(this, SIGNAL(itemClicked(QListWidgetItem*)), this, SLOT(itemClick(QListWidgetItem*)));

	icons[0] = ":/all_images/images/critical.bmp";
	icons[1] = ":/all_images/images/warning.bmp";
	icons[2] = ":/all_images/images/information.bmp";
	msgTypes[0] = "Error: ";
	msgTypes[1] = "Warning: ";
	msgTypes[2] = "Information: ";
	
	fShowInfo = true;				
	fShowError = true;				
	fShowWrng = true;
}
// -----------------------------------------------------------------------------

/**
 * Metoda dodajaca element na liste
 * @param int _tid - Identyfikator tabeli
 * @param int _rid - Identyfikator wiersza
 * @param int _cid - Identyfikator komorki
 * @param QString _err - Tresc bledu 
 * @param int msgType - typ wiadomosci 0-error 1-warning 2-information
 * @returns Brak zwracanych wartosci
*/
void ExecMsgListWidget::add(int _tid, int _rid, int _cid, QString _err, int _msgType)
{
	if((_msgType < 0) || (_msgType > 2))
		return;
		
	// Usuwanie ostatniego obiektu ktorym jest info o liczbie bledow
	delete takeItem(count()-1);
	
	XTT_Cell* cell = NULL;
	XTT_Row* row = NULL;
	XTT_Table* table = NULL;
	QString tname = "";
	
	// Wyszukanie tabeli
	int tindex = TableList->IndexOfID(_tid);
	if(tindex > -1)
	{
		table = TableList->Table(tindex);
		tname = table->Title();
		fTablesIDs.append(_tid);
	}
	
	// Tak czy owak dodoajemy wskaznik do tabeli, aby ilosc elementow w liscie pokrywala sie z liczba wiadomosci
	fTables.append(table);
	if(tindex == -1)
		fTablesIDs.append(-1);
	
	
	// Wyszukanie numeru wiersza gdzie znajduje sie komorka
	int rindex = -1, cindex = -1;
	if((tindex > -1) && (table != NULL))
	{
		for(unsigned int r=0;r<table->RowCount();r++)
			if(table->Row(r)->RowId() == (unsigned int)_rid)
			{	
				rindex = (int)r;
				row = table->Row(rindex);
				break;
			}
			
		if(rindex > -1)
			fRowIDs.append(_rid);
		if(rindex == -1)
			fRowIDs.append(-1);
	}
	if((tindex == -1) || (table == NULL))
		fRowIDs.append(-1);
	fRows.append(row);
	
	// Wyszukanie numeru kolumny gdzie znajduje sie komorka
	if((_cid > -1) && (rindex > -1) && (table != NULL))
	{
		cindex = row->indexOfCellID(_cid);
		cell = row->Cell(cindex);
		
		if(cindex > -1)
			fCellsIDs.append(_cid);
		if(cindex == -1)
			fCellsIDs.append(-1);
	}
	if((_cid == -1) || (rindex == -1) || (table == NULL))
		fCellsIDs.append(-1);
	fCells.append(cell);
	
	fMessages.append(_err);
	fMessagesTypes.append(_msgType);
	
	// Wyswietlenie informacji
	applyFilter();
}
// -----------------------------------------------------------------------------

/**
 * Metoda usuwajaca element
 * @returns Brak zwracanych wartosci
*/
void ExecMsgListWidget::deleteAll(void)
{
	clear();
	fTablesIDs.clear();		
	fRowIDs.clear();
	fCellsIDs.clear();			
	fMessagesTypes.clear();		
	fCells.clear();		
	fRows.clear();
	fTables.clear();		
	fMessages.clear();	
}
// -----------------------------------------------------------------------------

/**
 * Metoda wywolywana w momencie zmiany zaznaczenia
 * @param int _newRow - Index nowego wiersza
 * @returns Brak zwracanych wartosci
*/
void ExecMsgListWidget::onRowChange(int _newRow)
{	
	if((_newRow < 0) || (_newRow >= (count()-1)))
		return;
	
	int currSize = fCells.size();
	
	/// Index z listy wiadomosci
	int newRow = -1;
	for(int i=0;i<fMessages.size();i++)
		if(((fShowError) && (fMessagesTypes.at(i) == 0)) || ((fShowWrng) && (fMessagesTypes.at(i) == 1)) || ((fShowInfo) && (fMessagesTypes.at(i) == 2)))
		{
			newRow++;
			if(_newRow == newRow)
			{
				newRow = i;
				break;
			}
		}
	
	if((newRow < 0) || (newRow >= currSize))
		return;
	
	// Usuwanie zaznaczenia z tabel
	for(int i=0;i<fTables.size();i++)
		if(fTablesIDs.at(i) > -1)
			fTables.at(i)->GraphicTable()->setTableSelection(false);
			
	// Usuwanie zaznaczenia z komorek i wierszy
	for(int i=0;i<fRows.size();i++)
		if(fRowIDs.at(i) > -1)
			fRows.at(i)->setSelected(false);
	//for(int i=0;i<fCells.size();i++)
	//	fCells.at(i)->setSelected(false);
		
	if(fCellsIDs.at(newRow) > -1)
		fCells.at(newRow)->setSelected();
		
	if((fRowIDs.at(newRow) > -1) && (fCellsIDs.at(newRow) == -1))
		fRows.at(newRow)->setSelected();
		
	if((fTablesIDs.at(newRow) > -1) && (fRowIDs.at(newRow) == -1) && (fCellsIDs.at(newRow) == -1))
	{	
		fTables.at(newRow)->GraphicTable()->setTableSelection(true);
		fTables.at(newRow)->GraphicTable()->ensureVisible();
	}
}
// -----------------------------------------------------------------------------

/**
 * Metoda wywolywana w momencie klikniecia elementu
 * @param QListWidgetItem* itm - Wskaznik do kliknietego elementu
 * @returns Brak zwracanych wartosci
*/
void ExecMsgListWidget::itemClick(QListWidgetItem* itm)
{
	onRowChange(row(itm));
}
// -----------------------------------------------------------------------------

/**
 * Metoda ustawiajaca filter czy ma przepuszczac wiadomosci bledow
 * @param bool _val - okresla pokazywanie/niepokazywanie wiadomosci
 * @returns Brak zwracanych wartosci
*/
void ExecMsgListWidget::showErrors(bool _val)
{
	fShowError = _val;
	applyFilter();
}
// -----------------------------------------------------------------------------

/**
 * Metoda ustawiajaca filter czy ma przepuszczac wiadomosci ostrzezen
 * @param bool _val - okresla pokazywanie/niepokazywanie wiadomosci
 * @returns Brak zwracanych wartosci
*/
void ExecMsgListWidget::showWarnings(bool _val)
{
	fShowWrng = _val;
	applyFilter();
}
// -----------------------------------------------------------------------------

/**
 * Metoda ustawiajaca filter czy ma przepuszczac wiadomosci informacyjne
 * @param bool _val - okresla pokazywanie/niepokazywanie wiadomosci
 * @returns Brak zwracanych wartosci
*/
void ExecMsgListWidget::showInformations(bool _val)
{
	fShowInfo = _val;
	applyFilter();
}
// -----------------------------------------------------------------------------

/**
 * Metoda wyswietlajaca wiadomosci przefiltrowanego typu
 * @returns Brak zwracanych wartosci
*/
void ExecMsgListWidget::applyFilter(void)
{	
	setCurrentRow(-1);
	clear();
	int typesCount[3] = {0, 0, 0};
	
	for(int i=0;i<fMessages.size();i++)
	{
		typesCount[fMessagesTypes.at(i)]++;
		if(((fShowError) && (fMessagesTypes.at(i) == 0)) || ((fShowWrng) && (fMessagesTypes.at(i) == 1)) || ((fShowInfo) && (fMessagesTypes.at(i) == 2)))
		{
			addItem(messageContent(i));
			item(count()-1)->setIcon(QIcon(icons[fMessagesTypes.at(i)]));
		}
	}
		
	// Dodanie informacji o liczbie bledow
	// Ustalanie koloru czcionki
	QColor colors[3] = {Qt::red, Qt::blue, Qt::darkGreen};
	QColor fontColor;
	int lastIcon = 2;
	for(int i=2;i>=0;i--)
		if(typesCount[i] > 0)
		{
			fontColor = colors[i];
			lastIcon = i;
		}
	
	int n = fMessages.size();
	addItem(msgTypes[2] + QString::number(n) + " messages(s) found:\t" + QString::number(typesCount[0]) + " error(s),\t" + QString::number(typesCount[1]) + " warnings(s),\t"  + QString::number(typesCount[2]) + " informations(s).");
	QFont ifont = item(count()-1)->font();
	item(count()-1)->setIcon(QIcon(icons[lastIcon]));
	item(count()-1)->setFont(ifont);
	item(count()-1)->setForeground(QBrush(fontColor));
	scrollToItem(item(count()-1));
	
	setCurrentRow(0);
}
// -----------------------------------------------------------------------------

/**
 * Metoda tworzaca cala tresc wiadomosc poprzed dodanie do tresci wlasciwej parametrow wystapienia
 * @param int _index - index wiadomosci
 * @returns Tresc wiadomosci jako string
*/
QString ExecMsgListWidget::messageContent(int _index)
{
	if((_index < 0) || (_index >= fMessages.size()))
		return "";
		
		
	// Wyszukanie numeru wiersza gdzie znajduje sie komorka
	int rindex = -1, cindex = -1;
	if(fTablesIDs.at(_index) > -1)
	{
		for(unsigned int r=0;r<fTables.at(_index)->RowCount();r++)
			if(fTables.at(_index)->Row(r)->RowId() == (unsigned int)fRowIDs.at(_index))
			{	
				rindex = (int)r;
				break;
			}
	}
	
	// Wyszukanie numeru kolumny gdzie znajduje sie komorka
	if((rindex > -1) && (fTablesIDs.at(_index) > -1))
		cindex = fRows.at(_index)->indexOfCellID(fCellsIDs.at(_index));

	// Ustalanie tresci wiadomosci
	QString messageItem = msgTypes[fMessagesTypes.at(_index)];
	
	if(fTablesIDs.at(_index) > -1)
		messageItem += "in table \'" + fTables.at(_index)->Title() + "\'";
	if(fRowIDs.at(_index) > -1)
		messageItem += ", row " + QString::number(rindex+1);
	if(fCellsIDs.at(_index) > -1)
		messageItem += ", col " + QString::number(cindex+1);
	messageItem += ": " + fMessages.at(_index);
	
	return messageItem;
}
// -----------------------------------------------------------------------------
