 /**
 *                                                                             *
 * @file Hinter_ui.cpp                                                         *
 * Project name: HeKatE hqed                                                   *
 * $Id: Hinter_ui.cpp,v 1.12 2008-03-03 22:04:32 kinio Exp $                    *
 * Author: Krzysztof Kaczor                                                    *
 * Date: 28.04.2007                                                            *
 *                                                                             *
 * Implementation by Krzysztof Kaczor <kinio4444@gmail.com>                    *
 *                                                                             *
 * Copyright (C) 2006-9 by the HeKatE Project                                  *
 *                                                                             *
 * HQEd has been develped within the HeKatE Project,                           *
 * see http://hekate.ia.agh.edu.pl                                             *
 *                                                                             *
 * This file is part of HQEd.                                                  *
 *                                                                             *
 * HQEd is free software: you can redistribute it and/or modify                *
 * it under the terms of the GNU General Public License as published by        *
 * the Free Software Foundation, either version 3 of the License, or           *
 * (at your option) any later version.                                         *
 *                                                                             *
 * HQEd is distributed in the hope that it will be useful,                     *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU General Public License for more details.                                *
 *                                                                             *
 * You should have received a copy of the GNU General Public License           *
 * along with HQEd.  If not, see <http://www.gnu.org/licenses/>.               *
 *                                                                             *
 * Comment: Plik zawierajacy definicje metod klasy obslugujacej okno           *
 * wspomagajace towrzenie zawartosci komorki                                   *
 *                                                                             *
 */

#include <QMessageBox>
#include <QtGui>
#include <QDialog>
#include <QObject>
#include <QKeyEvent>
#include <QTableWidgetItem>
#include <QCompleter>
#include <QTextCursor>
#include <QScrollBar>

#include "../../M/XTT/XTT_Table.h"
#include "../../M/XTT/XTT_Row.h"
#include "../../M/XTT/XTT_Cell.h"
#include "../../M/XTT/XTT_AttributeList.h"
#include "../../M/XTT/XTT_AttributeGroups.h"
#include "../../M/XTT/XTT_Attribute.h"
#include "../MainWin_ui.h"
#include "../Settings_ui.h"
#include "MyLineEdit.h"
#include "Hinter_ui.h"
// -----------------------------------------------------------------------------

Hinter_ui* Hinter;
// -----------------------------------------------------------------------------

/**
 * Konstruktor kalsy
 * @param QWidget *parent - Wskanik do rodzica
 * @returns Brak zwracanych wartosci
*/
Hinter_ui::Hinter_ui(QWidget*)
{
    setupUi(this);
    setWindowIcon(QIcon(":/all_images/images/mainico.bmp")); 

    connect(toolButton, SIGNAL(clicked()), this, SLOT(OKClick()));
    connect(toolButton_2, SIGNAL(clicked()), this, SLOT(CancelClick()));

    setWindowModality(Qt::ApplicationModal);
    setWindowFlags(Qt::SplashScreen);
    setGeometry(geometry().x(), geometry().y(), (int)(geometry().width()*1.5), geometry().height());
    
    line_edit = NULL;
}
// -----------------------------------------------------------------------------

/**
 * Konstruktor klasy przyjmujacy parametry edytowaniego obiektu
 * @param int _colContext - Kontekst edytowanej kolumny
 * @param XTT_Table* _table - Wskanik do tabeli ktorej komorke edytujemy
 * @param XTT_Attribute* _attr - Wskanik do atrybutu ktory nalezy do komorki 
 * edytowanej
 * @param XTT_Cell* _cell - Wskanik do edytowanej komorki
 * @param QTableWidgetItem* _item - Wskanik do elementu edytowanego
 * @param QWidget *parent - Wskanik do rodzica
 * @returns Brak zwracanych wartosci
*/
Hinter_ui::Hinter_ui(int _colContext, XTT_Table* _table, XTT_Attribute* _attr, XTT_Cell* _cell, QTableWidgetItem* _item, QWidget*)
{
     setupUi(this);
	setWindowIcon(QIcon(":/all_images/images/mainico.bmp")); 
	setGeometry(geometry().x(), geometry().y(), (int)(geometry().width()*1.5), geometry().height());

     //delete lineEdit;
     lineEdit->setVisible(false);
     line_edit = new MyLineEdit(this);
     line_edit->setObjectName(QString::fromUtf8("line_edit"));
     line_edit->setGeometry(lineEdit->geometry());
     line_edit->setFrame(false);
     line_edit->setReadOnly(false);
     line_edit->setToolTip("Edit cell content\nPress down arrow for popup the list.");
     gridLayout->addWidget(line_edit, 3, 0, 1, 5);
     retranslateUi(this);

     connect(toolButton, SIGNAL(clicked()), this, SLOT(OKClick()));
     connect(toolButton_2, SIGNAL(clicked()), this, SLOT(CancelClick()));

     setWindowModality(Qt::ApplicationModal);
     setWindowFlags(Qt::SplashScreen);

     // Zapisanie parametrow
     fcolContext = _colContext;
     fattr = _attr;
     fitem = _item;
     fcell = _cell;
     ftable = _table;
	
	// Jezeli komorka jest NULL to tworzymy sobie nowa tyczasowa komorke
	if(fcell == NULL)
	{
		fcell = new XTT_Cell(_table->TableId(), 0, ftable->ColContext(fitem->column()));
		fcell->setAttrType(_attr);
		fcell->setEnableAttr2AttrComp(true);
		fcell->Analyze(_item->text());
		fcell->setContent(_item->text());
	}

     int context = 0;
     if((fcell == NULL) && (fitem != NULL))
          context = ftable->ColContext(fitem->column());
     if(fcell != NULL)
	{
          context = fcell->Context();
		checkBox->setChecked(fcell->enableAttr2AttrComp());
	}
	if((fcell == NULL) && (fitem == NULL))
	{
		QMessageBox::critical(NULL, "XTT Editor", "Unknown column context.", QMessageBox::Ok);
		close();
		return;
	}

     // Konfiguracja okienka

     // Ustawianie operatorow
     if(fcolContext == 4)
     {
          comboBox->clear();
          comboBox->addItem("=");
     }

	if(fcell == NULL)
		line_edit->setText(_item->text());
     if(fcell != NULL)
     {
          comboBox->setCurrentIndex(fcell->OperatorType()-1);
          line_edit->setText(fcell->Content());
     }

     if(comboBox->currentIndex() == -1)
               comboBox->setCurrentIndex(0);

     // Usuwanie operatora
     if((line_edit->text().length() > 0) && (line_edit->text().contains(comboBox->currentText(), Qt::CaseInsensitive)))
     {
          QString text = line_edit->text();
          
          text = text.remove(text.indexOf(comboBox->currentText(), 0, Qt::CaseInsensitive), comboBox->currentText().length());

          // Usuwanie spacji na poczatku
          for(;(text.length()) && (text[0] == ' ');)
               text = text.remove(0, 1);

          line_edit->setText(text);
     }

     // Usawienie komplitera
     // Tworzenie listy slow
     QStringList cmpl_word_list;

	// Jezeli jest dozwolone porownywanie atrybutow
	if(fcell->enableAttr2AttrComp())
	{
	     // Dodawanie atrybutow tego samego typu, bedacych w tej tabeli
	     for(unsigned int g=0;g<AttributeGroups->Count();g++)
	     {
	          if(AttributeGroups->Group(g)->Name() == "tmp")
	               continue;

	          for(unsigned int i=0;i<AttributeGroups->Group(g)->Count();i++)
	          {
	               // Jezeli jest tego samego typu i wystepuje w tabeli to dodajemy do do listy
	               if((AttributeGroups->Group(g)->Item(i)->Type() == fattr->Type()) &&
	                  (ftable->IndexOfAttributeID(AttributeGroups->Group(g)->Item(i)->Id()) > -1))
	               {
	                    cmpl_word_list << AttributeGroups->CreatePath(AttributeGroups->Group(g)->Item(i)) + AttributeGroups->Group(g)->Item(i)->Name();
	                    cmpl_word_list << AttributeGroups->CreatePath(AttributeGroups->Group(g)->Item(i)) + AttributeGroups->Group(g)->Item(i)->Acronym();
	               }
	          }
	     }
	}

     // Dodawanie nazw funckji dla atrybutow liczbowych
     if((fattr->Type() == 2) || (fattr->Type() == 3))
          cmpl_word_list << "sin()" << "cos()" << "tan()" << "abs()" << "root(,)" << "log()";
     if(fattr->Type() == 2)
          cmpl_word_list << "mod(,)";

     // Dodawanie elementow wyliczeniowych jezeli jest to typ wyliczeniowy
     if(fattr->Type() == 5)
     {
          for(int i=0;i<fattr->AttributeValues()->size();i++)
               cmpl_word_list << fattr->AttributeValues()->at(i);
     }

     // Dodawanie elementow wyliczeniowych jezeli jest ograniczenie wyliczeniowe
     if(fattr->ConstraintsType() == 2)
     {
          for(int i=0;i<fattr->EnumerateConstraints()->size();i++)
               cmpl_word_list << fattr->EnumerateConstraints()->at(i);
     }

     // Dodawanie wartosci typu boolean
     if(fattr->Type() == 1)
          cmpl_word_list << "true" << "false";

     // Dodanie operatora <nd>
     if((context == 1) || (context == 4))
          cmpl_word_list << XTT_Attribute::_not_definded_operator_;

     // Dodanie operatora <any>
     if((context == 1) || (context == 3))
          cmpl_word_list << XTT_Attribute::_any_operator_;

     cmpl = new QCompleter(cmpl_word_list, this);
     cmpl->setCaseSensitivity(Qt::CaseInsensitive);
     line_edit->setCompleter(cmpl);
}
// -----------------------------------------------------------------------------

/**
 * Destruktor klasy, kasuje dynamicznie zaalokowane kontrolki
 * @returns Brak zwracanych wartosci
 */
Hinter_ui::~Hinter_ui(void)
{
	if(line_edit)
		delete line_edit;
}
// -----------------------------------------------------------------------------

/**
 * Metoda zwalniajaca dynamicznie zaalokowana pamiec
 * @returns Brak zwracanych wartosci
 */
void Hinter_ui::freeDynamicAlocatedMemory(void)
{
	if(line_edit)
	{
		delete line_edit;
		line_edit = NULL;
	}
}
// -----------------------------------------------------------------------------

/**
 * Metoda klasy wywolywana w momencie klikniecia na przycisk OK
 * @returns Brak zwracanych wartoci
*/
void Hinter_ui::OKClick(void)
{
	// Jezeli znamy adres komorki to najpierw ustawiamy flage attr2attr comp, aby weryfikacja zawartosci
	// byla przeprowadzona z aktualna jej wartoscia
	if(fcell != NULL)
		fcell->setEnableAttr2AttrComp(checkBox->isChecked());
	
	QString res = comboBox->currentText() + " " + line_edit->text();
     if((fcell != NULL) && (!fcell->Analyze(res)) && (Settings->xttShowErrorCellMessage()))
     {
          QMessageBox::critical(NULL, "XTT Editor", fcell->CellError(false), QMessageBox::Ok);
          return;
     }
	if(fitem != NULL)
		fitem->setText(res);
	if(fcell != NULL)
		fcell->setContent(res);

     QStringList ctx, atype;
     ctx << "conditional" << "assert" << "retract" << "action";
     atype << "boolean" << "integer" << "float" << "symbolic" << "enumerative";
     //int context = ftable->ColContext(fitem->column());
	int context = fcolContext;
     QString tt = "";
     tt = "Column information:\n\tContext: " + ctx.at(context-1) + "\n\tArgument name: " + fattr->Name();
     tt += "\n\tArgument acronym: " + fattr->Acronym();
     tt += "\n\tArgument type: " + atype.at(fattr->Type()-1);
     tt += "\n\tArgument domain: " + fattr->ConstraintsRepresentation();
     if(fcell != NULL)
          tt += "\nCell information:\n\tContent: \"" + fcell->Content() + "\"";
     if(fcell == NULL)
          tt += "\nCell information:\n\tContent: \"" + res + "\"";
	if(fitem != NULL)
		fitem->setToolTip(tt);
     
	freeDynamicAlocatedMemory();
	
	emit ClickOK();
     close();
}
// -----------------------------------------------------------------------------

/**
 * Metoa wywolywana w momencie kiedy zostanie nacisniety jakis klawisz
 * @param QKeyEvent* event - Wskanik do obiektu przechwoujacego informacje o zdarzeniu
 * @returns Brak zwracanych wartoci
*/
void Hinter_ui::keyPressEvent(QKeyEvent *e)
{
     switch(e->key())
     {
          case Qt::Key_Escape:
               CancelClick();
               break;

          case Qt::Key_Return:
               OKClick();
               break;
               
          default:
               break;
     }
}
// -----------------------------------------------------------------------------

/**
 * Metoa wywolywana w momencie klkniecia przycisku cancel
 * @returns Brak zwracanych wartoci
*/
void Hinter_ui::CancelClick(void)
{
	if(fcell != NULL)
	{

		QString cnt = fcell->Content();
		fcell->Analyze(cnt);
		fcell->setContent(cnt);
	}
	
	freeDynamicAlocatedMemory();
	close();
}
// -----------------------------------------------------------------------------
