 /**
 *                                                                             *
 * @file MyGraphicsScene.h                                                     *
 * Project name: HeKatE hqed                                                   *
 * $Id: MyGraphicsScene.h,v 1.12 2008-03-03 22:04:32 kinio Exp $                *
 * Author: Krzysztof Kaczor                                                    *
 * Date: 12.10.2007                                                            *
 *                                                                             *
 * Implementation by Krzysztof Kaczor <kinio4444@gmail.com>                    *
 *                                                                             *
 * Copyright (C) 2006-9 by the HeKatE Project                                  *
 *                                                                             *
 * HQEd has been develped within the HeKatE Project,                           *
 * see http://hekate.ia.agh.edu.pl                                             *
 *                                                                             *
 * This file is part of HQEd.                                                  *
 *                                                                             *
 * HQEd is free software: you can redistribute it and/or modify                *
 * it under the terms of the GNU General Public License as published by        *
 * the Free Software Foundation, either version 3 of the License, or           *
 * (at your option) any later version.                                         *
 *                                                                             *
 * HQEd is distributed in the hope that it will be useful,                     *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU General Public License for more details.                                *
 *                                                                             *
 * You should have received a copy of the GNU General Public License           *
 * along with HQEd.  If not, see <http://www.gnu.org/licenses/>.               *
 *                                                                             *
 * Comment: Plik zawierajacy definicje klasy pochodnej elementu bedacego       *
 * plotnem na ktorym rysowana jest scena                                       *
 *                                                                             *
 */


#ifndef MYGRAPHICSSCENEH
#define MYGRAPHICSSCENEH
// -----------------------------------------------------------------------------

#include <QGraphicsScene>
// -----------------------------------------------------------------------------

class QGraphicsView;
class QGraphicsItem;
// -----------------------------------------------------------------------------

class MyGraphicsScene : public QGraphicsScene
{
    Q_OBJECT
    
private:

	/// Okresla obiekt ktory trzeba ustawic na scenie
	QGraphicsItem* fNewItem;
	
	/// Przechowuje aktualnie wybrana tabele
     int fSelectedTableID;

     /// Przechowuje aktualnie podswiwtlone polaczenie
     int fSelectedConnectionID;
	
	/// Bierzacy koat obraotu
     int fCurrentAngle;
	
protected:

	/// Metoda wywolywana w momencie ruchu kursorem nad scena
	void mouseMoveEvent(QGraphicsSceneMouseEvent*);
	
	/// Metoda wywolywana w momencie kliknecia przyciskiem na scene
	void mousePressEvent(QGraphicsSceneMouseEvent*);
     
public:

     /// Konstruktory klasy
     MyGraphicsScene(QObject* oParent = 0);
	
	/// Destruktor klasy
	~MyGraphicsScene(void);
	
	/// Pozwala ustawic wskaznik do obiektu ktory bedzie ustawiany na scenie
	void setNewItem(QGraphicsItem*);
	
	/// Zwraca wskaznik do obiektu ktory bedzie ustawiany na scenie
	QGraphicsItem* newItem(void);
	
	/// Resetuje wskaznik obiektu ktory trzeba ustawic na scenie
	void resetNewItem(void);
	
	/// Ustawia aktualnie wybrana tabele
     void setSelectedTableID(int _id){ fSelectedTableID = _id; }

     /// Zwraca wartosc aktualnie wybrane tabeli
     int SelectedTableID(void){ return fSelectedTableID; }

     /// Ustawia aktualnie wybrane polaczenie
     void setSelectedConnectionID(int _id){ fSelectedConnectionID = _id; }

     /// Zwraca wartosc aktualnie wybranego polaczenia
     int SelectedConnectionID(void){ return fSelectedConnectionID; }

     /// Zwraca aktualny kat obrotu sceny
     int currentAngle(void){ return fCurrentAngle; }
	
	/// Sporzada zoom z zadanym powikeszeniem
     void scaleView(qreal);
	
public slots:

	/// Wywolywana w momencie rotacji sceny
     void Rotate(int);
	
	/// Robi oddalenie sceny
     void zout(void);

     /// Robi zblizenie sceny
     void zin(void);
	
	/// Ustawia aktualny kat obrotu sceny
     void setCurrentAngle(int _ca){ fCurrentAngle = _ca; }
	
	/// Powieksza scene tak aby wszystkie elementy sie na niej mogly zmiescic
     void AdaptSceneSize(void);
	
	/// Przesuwa elementy w prawo
     void MoveItemsRight(void);

     /// Przesuwa elementy w lewo
     void MoveItemsLeft(void);

     /// Przesuwa elementy w gore
     void MoveItemsUp(void);

     /// Przesuwa elementy w dol
     void MoveItemsDown(void);
};
// -----------------------------------------------------------------------------
#endif
