 /**
 *                                                                             *
 * @file sysTrayObject.cpp                                                     *
 * Project name: HeKatE hqed                                                   *
 * $Id: sysTrayObject.cpp,v 1.2 2008-03-03 22:04:31 kinio Exp $                *
 * Author: Krzysztof Kaczor                                                    *
 * Date: 29.01.2008                                                            *
 *                                                                             *
 * Implementation by Krzysztof Kaczor <kinio4444@gmail.com>                    *
 *                                                                             *
 * Copyright (C) 2006-9 by the HeKatE Project                                  *
 *                                                                             *
 * HQEd has been develped within the HeKatE Project,                           *
 * see http://hekate.ia.agh.edu.pl                                             *
 *                                                                             *
 * This file is part of HQEd.                                                  *
 *                                                                             *
 * HQEd is free software: you can redistribute it and/or modify                *
 * it under the terms of the GNU General Public License as published by        *
 * the Free Software Foundation, either version 3 of the License, or           *
 * (at your option) any later version.                                         *
 *                                                                             *
 * HQEd is distributed in the hope that it will be useful,                     *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU General Public License for more details.                                *
 *                                                                             *
 * You should have received a copy of the GNU General Public License           *
 * along with HQEd.  If not, see <http://www.gnu.org/licenses/>.               *
 *                                                                             *
 * Comment: Plik zawierajacy definicje klasy pochodnej elementu bedacego       *
 * reprezentantem aplikacji w zasobniku systemowym.                            *
 *                                                                             *
 */

#include <QMessageBox>
#include <QtGui>
#include <QDialog>
#include <QObject>
#include <QKeyEvent>

#include <QTime>
#include <QAction>

#include "About_ui.h"
#include "MainWin_ui.h"
#include "sysTrayObject.h"
// -----------------------------------------------------------------------------

/**
 * Konstruktor kalsy
 * @param QObject *parent - Wskanik do rodzica
 * @returns Brak zwracanych wartosci
*/
sysTrayObject::sysTrayObject(QWidget* oParent) : QSystemTrayIcon(oParent)
{	
	setIcon(QIcon(":/all_images/images/mainico.bmp"));
	setToolTip(AboutWin->appVersion());
	
	// Tworzenie menu
	trayIconMenu = new QMenu(NULL);
	
	trayMenu_netxTip = new QAction(QIcon(":/all_images/images/blub.bmp"), tr("&Next tip"), NULL);
	trayMenu_minimize = new QAction(QIcon(":/all_images/images/minimize.bmp"), tr("&Minimize"), NULL);
	trayMenu_maximize = new QAction(QIcon(":/all_images/images/maximize.bmp"), tr("Ma&ximize"), NULL);
	trayMenu_restore = new QAction(QIcon(":/all_images/images/restore.bmp"), tr("&Restore"), NULL);
	trayMenu_options = new QAction(QIcon(":/all_images/images/settings.bmp"), tr("&Options"), NULL);
	trayMenu_about = new QAction(QIcon(":/all_images/images/About.bmp"), tr("&About"), NULL);
	trayMenu_quit = new QAction(QIcon(":/all_images/images/Quit.bmp"), tr("&Quit"), NULL);
	
	QFont nextTipFont = trayMenu_netxTip->font();
	nextTipFont.setBold(true);
	trayMenu_netxTip->setFont(nextTipFont);
	
	trayIconMenu->addAction(trayMenu_netxTip);
	trayIconMenu->addSeparator();
	trayIconMenu->addAction(trayMenu_minimize);
	trayIconMenu->addAction(trayMenu_maximize);
	trayIconMenu->addAction(trayMenu_restore);
	trayIconMenu->addSeparator();
	trayIconMenu->addAction(trayMenu_options);
	trayIconMenu->addSeparator();
	trayIconMenu->addAction(trayMenu_about);
	trayIconMenu->addSeparator();
	trayIconMenu->addAction(trayMenu_quit);
	
	connect(trayMenu_minimize, SIGNAL(triggered()), this, SLOT(onMinimize()));
	connect(trayMenu_maximize, SIGNAL(triggered()), this, SLOT(onMaximize()));
	connect(trayMenu_restore, SIGNAL(triggered()), this, SLOT(onRestore()));
	connect(trayMenu_options, SIGNAL(triggered()), this, SLOT(onOptions()));
	connect(trayMenu_about, SIGNAL(triggered()), this, SLOT(onAbout()));
	connect(trayMenu_quit, SIGNAL(triggered()), this, SLOT(onQuit()));
	connect(trayMenu_netxTip, SIGNAL(triggered()), this, SLOT(onNextTip()));
	connect(this, SIGNAL(messageClicked()), this, SLOT(onMessageClick()));
	connect(trayIconMenu, SIGNAL(aboutToShow()), this, SLOT(onShowMenu()));
	connect(this, SIGNAL(activated(QSystemTrayIcon::ActivationReason)), this, SLOT(onActivate(QSystemTrayIcon::ActivationReason)));
	
	setContextMenu(trayIconMenu);
}
// -----------------------------------------------------------------------------

sysTrayObject::~sysTrayObject(void)
{
	delete trayMenu_restore;
	delete trayMenu_minimize;
	delete trayMenu_maximize;
	delete trayMenu_options;
	delete trayMenu_about;
	delete trayMenu_quit;
	delete trayMenu_netxTip;
	delete trayIconMenu;
}
// -----------------------------------------------------------------------------

/**
 * Pokazuje losowa wskazowke na temat obslugi programu
 * @param int _appMode - okresla jaki tryb pracy aplikacji jest wymagany, jezeli  -1 to inne
 * @returns Brak zwracanych wartosci
*/
void sysTrayObject::drawTip(int _appMode)
{
	QStringList ardTips;
	QStringList xttTips;
	QStringList othersTips;
	
	ardTips << "If you want to open the file quickly, use drag-and-drop technique. The type of file will be regognized automatically";
	ardTips << "If you want to move the whole branch, choose the root of branch, press ctrl button and move it.";
	ardTips << "If you want to shorten/extend level, press the plus/minus button associated with the appropriate level.";
	ardTips << "If you want to switch to XTT mode, press the F12 button.";
	ardTips << "To open the file from command line, just type: \"hqed filename\". The type of file will be regognized automatically";
	
	xttTips << "If you want to open the file quickly, use drag-and-drop technique. The type of file will be regognized automatically";
	xttTips << "If you want to switch to ARD mode, press the F11 button.";
	xttTips << "To open the file from command line, just type: \"hqed filename\". The type of file will be regognized automatically";
	xttTips << "In order to create a new connection between rows, you can drag an appropriate input/output pin on the other an appropriate output/input pin.";
	xttTips << "If you want to check which of defined attributes are not used, choose the action: Atributes->Unused attributes...";
	
	othersTips << "Others Tip 1";
	othersTips << "Others Tip 2";
	othersTips << "Others Tip 3";
	othersTips << "Others Tip 4";
	
	// Losowanie wskazowki
	qsrand((unsigned int)QTime::currentTime().msec());
	if(_appMode == ARD_MODE)
		fLastTip = ardTips.at(qrand() % ardTips.size());
	if(_appMode == XTT_MODE)
		fLastTip = xttTips.at(qrand() % xttTips.size());
	if(_appMode == -1)
		fLastTip = othersTips.at(qrand() % othersTips.size());
}
// -----------------------------------------------------------------------------

/**
 * Pokazuje ostatnio wylosowana wskazowke
 * @returns brak zwracanych wartosci
*/
void sysTrayObject::showDrawedTip(void)
{
	int durationTime = 10000;
	QString title = "Tool Tip";
	showMessage(title, fLastTip, Information, durationTime);
}
// -----------------------------------------------------------------------------

/**
 * Metoda wywolywanaa w momencie klikniecia na wiadomosc
 * @returns brak zwracanych wartosci
*/
void sysTrayObject::onMessageClick(void)
{
	QMessageBox::information(NULL, "Tip", fLastTip, QMessageBox::Ok);
}
// -----------------------------------------------------------------------------

/**
 * Metoda wywolywanaa w momencie klikniecia na polecenie pokazania nastepnej wskazowki
 * @returns brak zwracanych wartosci
*/
void sysTrayObject::onNextTip(void)
{
	drawTip(MainWin->currentMode());
	showDrawedTip();
}
// -----------------------------------------------------------------------------

/**
 * Metoda wywolywana w momencie kliknecia wywolania polecenia przywrocenia okna
 * @returns brak zwracanych wartosci
*/
void sysTrayObject::onRestore(void)
{
	MainWin->showNormal();
}
// -----------------------------------------------------------------------------

/**
 * Metoda wywolywana w momencie kliknecia wywolania polecenia minimalizowania okna
 * @returns brak zwracanych wartosci
*/
void sysTrayObject::onMinimize(void)
{
	MainWin->showMinimized();
}
// -----------------------------------------------------------------------------

/**
 * Metoda wywolywana w momencie kliknecia wywolania polecenia maksymalizacji okna
 * @returns brak zwracanych wartosci
*/
void sysTrayObject::onMaximize(void)
{	
	MainWin->showMaximized();
}
// -----------------------------------------------------------------------------

/**
 * Metoda wywolywana w momencie kliknecia wywolania polecenia pokazania okna
 * @returns brak zwracanych wartosci
*/
void sysTrayObject::onOptions(void)
{
	MainWin->showConfigurationDialog();
}
// -----------------------------------------------------------------------------

/**
 * Metoda wywolywana w momencie kliknecia wywolania polecenia pokazania okna o porgramie
 * @returns brak zwracanych wartosci
*/
void sysTrayObject::onAbout(void)
{
	MainWin->ShowAboutWin();
}
// -----------------------------------------------------------------------------

/**
 * Metoda wywolywana w momencie kliknecia wywolania polecenia zamkniecia aplikacji
 * @returns brak zwracanych wartosci
*/
void sysTrayObject::onQuit(void)
{
	MainWin->CloseForm();
}
// -----------------------------------------------------------------------------

/**
 * Metoda wywolywana w momencie pokazywania menu kontekstowego
 * @returns brak zwracanych wartosci
*/
void sysTrayObject::onShowMenu(void)
{
	trayMenu_minimize->setEnabled(!MainWin->isMinimized());
	trayMenu_maximize->setEnabled(!MainWin->isMaximized());
	trayMenu_restore->setEnabled(MainWin->isMinimized() || MainWin->isMaximized());
}
// -----------------------------------------------------------------------------

/**
 * Metoda wywolana w momencie aktywacji ikony w zasobniku
 * @param QSystemTrayIcon::ActivationReason reason - zawiera powod w jaki sposob zostala aktywowana ikona w zasobniku
 * @returns brak zwracanych wartosci
*/
void sysTrayObject::onActivate(QSystemTrayIcon::ActivationReason reason)
{
	switch(reason) 
	{
		case QSystemTrayIcon::Trigger:
			contextMenu()->exec(QCursor::pos()); 
		break;
		
		case QSystemTrayIcon::DoubleClick:
		case QSystemTrayIcon::MiddleClick:
			onNextTip();
		break;
		
		default:
			contextMenu()->exec(QCursor::pos()); 
		break;
     }
}
// -----------------------------------------------------------------------------
