 /**
 *                                                                             *
 * @file ARD_AttributeList.cpp                                                 *
 * Project name: HeKatE hqed                                                   *
 * $Id: ARD_AttributeList.cpp,v 1.7 2008-03-03 22:04:33 kinio Exp $            *
 * Author: Krzysztof Kaczor                                                    *
 * Date: 22.12.2007                                                            *
 *                                                                             *
 * Implementation by Krzysztof Kaczor <kinio4444@gmail.com>                    *
 *                                                                             *
 * Copyright (C) 2006-9 by the HeKatE Project                                  *
 *                                                                             *
 * HQEd has been develped within the HeKatE Project,                           *
 * see http://hekate.ia.agh.edu.pl                                             *
 *                                                                             *
 * This file is part of HQEd.                                                  *
 *                                                                             *
 * HQEd is free software: you can redistribute it and/or modify                *
 * it under the terms of the GNU General Public License as published by        *
 * the Free Software Foundation, either version 3 of the License, or           *
 * (at your option) any later version.                                         *
 *                                                                             *
 * HQEd is distributed in the hope that it will be useful,                     *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU General Public License for more details.                                *
 *                                                                             *
 * You should have received a copy of the GNU General Public License           *
 * along with HQEd.  If not, see <http://www.gnu.org/licenses/>.               *
 *                                                                             *
 * Comment: Plik zawiera definicje metod klasy ARD_AttributeList               *
 *                                                                             *
 */

#include <QMessageBox>
#include <QtGui>
#include <QDialog>
#include <QObject>
#include <QStringList>

#include "../../C/MainWin_ui.h"
#include "../XTT/XTT_Attribute.h"
#include "ARD_Property.h"
#include "ARD_PropertyList.h"
#include "ARD_Attribute.h"
#include "ARD_AttributeList.h"
// -----------------------------------------------------------------------------

ARD_AttributeList* ardAttributeList;
// -----------------------------------------------------------------------------

/**
 * Konstruktor defaultowy klasy
 * @returns Brak zwracanych wartosci
*/
ARD_AttributeList::ARD_AttributeList(void) : QList<ARD_Attribute*>()
{
	fNextID = 0;
	fChanged = false;
}
// -----------------------------------------------------------------------------

/**
 * Destruktor defaultowy klasy
 * @returns Brak zwracanych wartosci
*/
ARD_AttributeList::~ARD_AttributeList(void)
{
	while(size() > 0)
	{
		delete takeAt(0);
	}
}
// -----------------------------------------------------------------------------

/**
 * Oproznia pojemnik
 * @returns wskaznik do nowo utworznonego atrybutu
*/
void ARD_AttributeList::flush(void)
{
	while(size() > 0)
	{
		delete takeAt(0);
	}
		
	fNextID = 0;
	fChanged = true;
}
// -----------------------------------------------------------------------------

/**
 * Dodaje nowy atrybut
 * @returns wskaznik do nowo utworznonego atrybutu
*/
ARD_Attribute* ARD_AttributeList::add(void)
{
	// Tworzenie nowego obiektu
	ARD_Attribute* tmp = new ARD_Attribute(++fNextID);
	add(tmp);
	
	return tmp;
}
// -----------------------------------------------------------------------------

/**
 * Dodaje nowy atrybut o podanym adresie
 * @param ARD_Attribute* _new - adres nowego atrybutu do dodania
 * @returns brak zwracanych wartosci
*/
void ARD_AttributeList::add(ARD_Attribute* _new)
{
	// Sprawdzamy czy taki atrybut juz nie istnieje
	if(contains(_new))
		return;
		
	append(_new);
}
// -----------------------------------------------------------------------------

/**
 * Usuwa wybrany atrybut
 * @param int _index - index atrybutu do usuniecia
 * @returns brak zwracanych wartosci
*/
void ARD_AttributeList::remove(int _index)
{
	// Sprawdzanie poprawnosci indexu
	if((_index < 0) || (_index >= size()))
		return;
		
	// Usuwanie z obiektow typu property
	QList<ARD_Property*>* pl = ardPropertyList->listOfAttributeID(at(_index)->id());
	for(int i=0;i<pl->size();++i)
		pl->at(i)->removeAt(pl->at(i)->indexOfID(at(_index)->id()));
	delete pl;
	
	// Usywanie obiektu
	delete takeAt(_index);
}
// -----------------------------------------------------------------------------

/**
 * Wyszukuje atrybuty na podstawie nazwy
 * @param QString _name - nazwa atrybutu
 * @returns index atrybutu o podanej nazwie, jezeli nie ma takiego atrybutu to -1
*/
int ARD_AttributeList::indexOfName(QString _name)
{
	for(int i=0;i<size();++i)
		if(at(i)->name() == _name)
			return i;
			
	return -1;
}
// -----------------------------------------------------------------------------

/**
 * Wyszukuje atrybuty na podstawie identyfikatora
 * @param unsigned int _id - wartosc identyfikatora
 * @returns index atrybutu o podanym identyfikatorze, jezeli nie ma takiego atrybutu to -1
*/
int ARD_AttributeList::indexOfID(unsigned int _id)
{
	for(int i=0;i<size();++i)
		if(at(i)->id() == _id)
			return i;
			
	return -1;
}
// -----------------------------------------------------------------------------

/**
 * Sortuje atrybuty rosnaco wg nazwy
 * @returns brak zwraanych wartosci
*/
void ARD_AttributeList::sort(void)
{
	for(int i=0;i<size();++i)
		for(int j=0;j<size()-i-1;++j)
			if(at(j)->name() > at(j+1)->name())
				swap(j, j+1);
				
	// Zapisanie statusu zmiany
	fChanged = true;
}
// -----------------------------------------------------------------------------
 
/**
 * Zwraca czy atrybuty byly modyfikowane
 * @returns true jezeli jakikolwiek element pojemnika byl modyfikowany, inaczej false
*/
bool ARD_AttributeList::changed(void)
{
	if(fChanged)
		return true;
		
	for(int i=0;i<size();++i)
		if(at(i)->changed())
			return true;
	
	return false;
}
// -----------------------------------------------------------------------------

/**
 * Ustawia status zmiany na zadana wartosc
 * @param bool _c - wartosc statusu
 * @returns brak zwraanych wartosci
*/
void ARD_AttributeList::setChanged(bool _c)
{
	fChanged = _c;
	
	for(int i=0;i<size();++i)
		at(i)->setChanged(_c);
}
// -----------------------------------------------------------------------------
