 /**
 *                                                                             *
 * @file ARD_Dependency.cpp                                                    *
 * Project name: HeKatE hqed                                                   *
 * $Id: ARD_Dependency.cpp,v 1.7 2008-03-03 22:04:33 kinio Exp $               *
 * Author: Krzysztof Kaczor                                                    *
 * Date: 26.12.2007                                                            *
 *                                                                             *
 * Implementation by Krzysztof Kaczor <kinio4444@gmail.com>                    *
 *                                                                             *
 * Copyright (C) 2006-9 by the HeKatE Project                                  *
 *                                                                             *
 * HQEd has been develped within the HeKatE Project,                           *
 * see http://hekate.ia.agh.edu.pl                                             *
 *                                                                             *
 * This file is part of HQEd.                                                  *
 *                                                                             *
 * HQEd is free software: you can redistribute it and/or modify                *
 * it under the terms of the GNU General Public License as published by        *
 * the Free Software Foundation, either version 3 of the License, or           *
 * (at your option) any later version.                                         *
 *                                                                             *
 * HQEd is distributed in the hope that it will be useful,                     *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU General Public License for more details.                                *
 *                                                                             *
 * You should have received a copy of the GNU General Public License           *
 * along with HQEd.  If not, see <http://www.gnu.org/licenses/>.               *
 *                                                                             *
 * Comment: Plik zawiera definicje metod klasy ARD_Dependency                  *
 *                                                                             *
 */

#include <QMessageBox>
#include <QtGui>
#include <QDialog>
#include <QObject>
#include <QStringList>

#include "../../C/MainWin_ui.h"
#include "../XTT/XTT_Attribute.h"
#include "ARD_Property.h"
#include "ARD_Dependency.h"
#include "ARD_Attribute.h"
// -----------------------------------------------------------------------------

/**
 * Konstruktor defaultowy klasy
 * @returns Brak zwracanych wartosci
*/
ARD_Dependency::ARD_Dependency(void)
{
	fID = 0;
	fIndependent = NULL;
	fDependent = NULL;
	fChanged = false;
}
// -----------------------------------------------------------------------------

/**
 * Konstruktor defaultowy klasy
 * @param unsigned int _id - Identyfikator atrybutu
 * @returns Brak zwracanych wartosci
*/
ARD_Dependency::ARD_Dependency(unsigned int _id)
{
	fID = _id;
	fIndependent = NULL;
	fDependent = NULL;
	fChanged = false;
}
// -----------------------------------------------------------------------------

/**
 * Zwraca identyfikator obiektu
 * @returns Zwraca identyfikator obiektu
*/
unsigned int ARD_Dependency::id(void)
{
	return fID;
}
// -----------------------------------------------------------------------------

/**
 * Zwraca status zmiany obiektu
 * @returns Zwraca status zmiany obiektu
*/
bool ARD_Dependency::changed(void)
{
	return fChanged;
}
// -----------------------------------------------------------------------------

/**
 * Zwraca wskaznik do obiektu niezaleznego
 * @returns Zwraca wskaznik do obiektu niezaleznego
*/
ARD_Property* ARD_Dependency::independent(void)
{
	return fIndependent;
}
// -----------------------------------------------------------------------------

/**
 * Zwraca wskaznik do obiektu zaleznego
 * @returns Zwraca wskaznik do obiektu zaleznego
*/
ARD_Property* ARD_Dependency::dependent(void)
{
	return fDependent;
}
// -----------------------------------------------------------------------------

/**
 * Ustawia status zmiany obiektu
 * @param unsigned int _id - nowa wartosc statusu
 * @returns Brak zwracanych wartosci
*/
void ARD_Dependency::setChanged(bool _c)
{
	fChanged = _c;
}
// -----------------------------------------------------------------------------

/**
 * Ustawia wskaznik do obiektu niezaleznego
 * @param ARD_Property* _ptr - wskaznik do obiektu
 * @returns Brak zwracanych wartosci
*/
bool ARD_Dependency::setIndependent(ARD_Property* _ptr)
{
	if(_ptr == NULL)
		return false;
		
	fIndependent = _ptr;
	return true;
}
// -----------------------------------------------------------------------------

/**
 * Ustawia wskaznik do obiektu zaleznego
 * @param ARD_Property* _ptr - wskaznik do obiektu
 * @returns Brak zwracanych wartosci
*/
bool ARD_Dependency::setDependent(ARD_Property* _ptr)
{
	if(_ptr == NULL)
		return false;
		
	fDependent = _ptr;
	return true;
}
// -----------------------------------------------------------------------------
