 /**
 *                                                                             *
 * @file ARD_DependencyList.h                                                  *
 * Project name: HeKatE hqed                                                   *
 * $Id: ARD_DependencyList.h,v 1.7 2008-03-03 22:04:33 kinio Exp $             *
 * Author: Krzysztof Kaczor                                                    *
 * Date: 26.12.2007                                                            *
 *                                                                             *
 * Implementation by Krzysztof Kaczor <kinio4444@gmail.com>                    *
 *                                                                             *
 * Copyright (C) 2006-9 by the HeKatE Project                                  *
 *                                                                             *
 * HQEd has been develped within the HeKatE Project,                           *
 * see http://hekate.ia.agh.edu.pl                                             *
 *                                                                             *
 * This file is part of HQEd.                                                  *
 *                                                                             *
 * HQEd is free software: you can redistribute it and/or modify                *
 * it under the terms of the GNU General Public License as published by        *
 * the Free Software Foundation, either version 3 of the License, or           *
 * (at your option) any later version.                                         *
 *                                                                             *
 * HQEd is distributed in the hope that it will be useful,                     *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU General Public License for more details.                                *
 *                                                                             *
 * You should have received a copy of the GNU General Public License           *
 * along with HQEd.  If not, see <http://www.gnu.org/licenses/>.               *
 *                                                                             *
 * Comment: Plik zwiera definicje obiektu bedacego pojemnikiem na zaleznosci   *
 * ARD                                                                         *
 */
 
#ifndef ARD_DEPENDENCYLISTH
#define ARD_DEPENDENCYLISTH
// -----------------------------------------------------------------------------

#include <QtCore/QString>
#include <QStringList>
#include <QDomDocument>
#include <QList>
// -----------------------------------------------------------------------------

class ARD_Property;
class ARD_Attribute;
class ARD_Dependency;
class XTT_Attribute;
// -----------------------------------------------------------------------------

class ARD_DependencyList : public QList<ARD_Dependency*>
{

private:

	/// Nastepny identyfikator atrybutu
	unsigned int fNextID;
	
	/// Okresla status czy zawartosc pojemnika byla zmieniona
	bool fChanged;
	
	/// Wyszukuje rodzicow dla danej wlasnosci, funckja jest prywatna ze wzgledu na jej rekurencyjnosc i niebezpieczenstwo
	/// podczas nieumiejetnego wywolania
	void findParents(QString, QList<ARD_Property*>*);
	
public:

	/// Konstruktor domyslny
	ARD_DependencyList(void);
	
	/// Destruktor
	~ARD_DependencyList(void);
	
	/// ----------------- Metody ustawiajace wartosci pol klasy -----------------
	
	/// Ustawia status zmiany na zadana wartosc
	void setChanged(bool);
	
	/// ----------------- Metody operacyjne -----------------
	
	/// Oproznia pojemnik
	void flush(void);
	
	/// Dodaje nowa zelznosc
	ARD_Dependency* add(void);
	
	/// Dodaje nowa zelznosc
	void add(ARD_Dependency*);
	
	/// Usuwa wybrana zelznosc
	void remove(int);
	
	/// Wyszukuje zelznosc po ID
	int indexOfID(unsigned int);
	
	/// Wyszukuje i zwraca liste zaleznosci o podanych wartoscich zrodla i przeznaczenia
	QList<ARD_Dependency*>* indexOfinout(ARD_Property*, ARD_Property*);
	
	/// Wyszukuje i zwraca liste zaleznosci o podanych wartoscich zrodla i przeznaczenia
	QList<ARD_Dependency*>* indexOfinout(QString, QString);
	
	/// Zwraca liste polaczen pomiedzy lisciami drzewa
	QList<ARD_Dependency*>* leafsDependencies(void);

	/// Wyszukuje rodzina dla danej wlasnosci
	QList<ARD_Property*>* findFamily(QString);
	
	/// Sprawdza zleanosc pomiedzy dwoa wlasnosciami
	bool checkDependence(QString _id, QString _dep, QList<ARD_Property*>* _checkedProperites = NULL);
	
	/// Zwraca czy zelznosci byly modyfikowane
	bool changed(void);
	
	/// Wcztuje dane z pliku w formacie ARDML w wersji tymczasowej
	bool fromARDML_wv(QStringList*);
	
	/// Wcztuje dane z pliku w formacie ARDML w wersji tymczasowej
	bool fromARDML_wv(QDomElement&);
};
// -----------------------------------------------------------------------------

extern ARD_DependencyList* ardDependencyList;
// -----------------------------------------------------------------------------
#endif
