 /**
 *                                                                             *
 * @file ARD_Property.h                                                        *
 * Project name: HeKatE hqed                                                   *
 * $Id: ARD_Property.h,v 1.7 2008-03-03 22:04:33 kinio Exp $                   *
 * Author: Krzysztof Kaczor                                                    *
 * Date: 23.12.2007                                                            *
 *                                                                             *
 * Implementation by Krzysztof Kaczor <kinio4444@gmail.com>                    *
 *                                                                             *
 * Copyright (C) 2006-9 by the HeKatE Project                                  *
 *                                                                             *
 * HQEd has been develped within the HeKatE Project,                           *
 * see http://hekate.ia.agh.edu.pl                                             *
 *                                                                             *
 * This file is part of HQEd.                                                  *
 *                                                                             *
 * HQEd is free software: you can redistribute it and/or modify                *
 * it under the terms of the GNU General Public License as published by        *
 * the Free Software Foundation, either version 3 of the License, or           *
 * (at your option) any later version.                                         *
 *                                                                             *
 * HQEd is distributed in the hope that it will be useful,                     *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU General Public License for more details.                                *
 *                                                                             *
 * You should have received a copy of the GNU General Public License           *
 * along with HQEd.  If not, see <http://www.gnu.org/licenses/>.               *
 *                                                                             *
 * Comment: Plik zwiera definicje obiektu bedacego poejdynczym obiektem        *
 * property ARD                                                                *
 */
 
#ifndef ARD_PROPERTYH
#define ARD_PROPERTYH
// -----------------------------------------------------------------------------

#include <QtCore/QString>
#include <QStringList>
#include <QList>
// -----------------------------------------------------------------------------

class ARD_Attribute;
class ARD_DependencyList;
class gTPHnode;
class gARDnode;
// -----------------------------------------------------------------------------

class ARD_Property : public QList<ARD_Attribute*>
{
private:
	
	/// Identyfikator obiektu
	QString fID;
	
	/// Identyfikator rodzica w diagramie TPH, jezeli zero to root
	QString fTPHparent;

	/// Okresla czy obiekt byl zmieniany
	bool fChanged;
	
	/// Obiekt beday graficzna reprezentacja wlasnosci na diagramie TPH
	gTPHnode* fGraphicTPHnode;
	
	/// Ustawia wartosc identyfikatora
	void setID(QString);
	
	/// Ustawia wartosc rodzica w diagramie tph
	void setTPHparent(QString);
	
public:

	/// Kostruktor defaultowy
	ARD_Property(void);
	
	/// Konstruktor przyjmujacy wartosc identyfikatora
	ARD_Property(QString);
	
	/// Destruktor
	~ARD_Property(void);
	
	/// ----------------- Metody zwracajaca wartosci pol klasy -----------------
	
	/// Zwraca identyfikator obiektu
	QString id(void);
	
	/// Zwraca identyfiaktor rodzica w drzewie TPH
	QString TPHparent(void);
	
	/// Zwaraca czy dany obiekt byl modyfikowany
	bool changed(void);
	
	/// Zwraca wskaznik do obiektu ktory graficznie reprezentuje dany obiekt
	gTPHnode* graphicTPHnode(void);
	
	/// ----------------- Metody ustawiajace wartosci pol klasy -----------------
	
	/// Ustawia wartosc statusu zmiany obiektu
	void setChanged(bool);
	
	/// --------------------------- Metody operacyjne ---------------------------
	
	/// Okresla czy obiekt moze byc dlaej transformowany
	bool canBeTransformated(void);
	
	/// Dodaje nowy atrybut do obiektu
	bool add(unsigned int);
	
	/// Dodaje nowy atrybut do obiektu
	bool add(ARD_Attribute*);
	
	/// Wyszukuje atrybuty na podstawie nazwy
	int indexOfName(QString);
	
	/// Wyszukuje atrybuty po ID
	int indexOfID(unsigned int);
	
	/// Sortuje atrybuty po nazwie
	void sort(void);
	
	// usuwanie atrybutu przez funckje removeAt(int)!!!
	
	/// Klasa musi miec dostep do funckji ustawiajacej rodzica
	friend class ARD_PropertyList;
};
// -----------------------------------------------------------------------------
#endif
