 /**
 *                                                                             *
 * @file XTT_Attribute.cpp                                                     *
 * Project name: HeKatE hqed                                                   *
 * $Id: XTT_Attribute.cpp,v 1.12 2008-03-03 22:04:33 kinio Exp $                *
 * Author: Krzysztof Kaczor                                                    *
 * Date: 24.04.2007                                                            *
 *                                                                             *
 * Implementation by Krzysztof Kaczor <kinio4444@gmail.com>                    *
 *                                                                             *
 * Copyright (C) 2006-9 by the HeKatE Project                                  *
 *                                                                             *
 * HQEd has been develped within the HeKatE Project,                           *
 * see http://hekate.ia.agh.edu.pl                                             *
 *                                                                             *
 * This file is part of HQEd.                                                  *
 *                                                                             *
 * HQEd is free software: you can redistribute it and/or modify                *
 * it under the terms of the GNU General Public License as published by        *
 * the Free Software Foundation, either version 3 of the License, or           *
 * (at your option) any later version.                                         *
 *                                                                             *
 * HQEd is distributed in the hope that it will be useful,                     *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU General Public License for more details.                                *
 *                                                                             *
 * You should have received a copy of the GNU General Public License           *
 * along with HQEd.  If not, see <http://www.gnu.org/licenses/>.               *
 *                                                                             *
 * Comment: Plik zawiera definicje metod klasy XTT_Attribute                   *
 *                                                                             *
 */

#include <QMessageBox>
#include <QtGui>
#include <QDialog>
#include <QObject>
#include <QStringList>

#include "../../C/MainWin_ui.h"
#include "XTT_Attribute.h"
// -----------------------------------------------------------------------------

QString XTT_Attribute::_not_definded_operator_ = "op:nd";
QString XTT_Attribute::_any_operator_ = "op:any";
// -----------------------------------------------------------------------------

/**
 * Konstruktor defaultowy klasy
 * @returns Brak zwracanych wartosci
*/
XTT_Attribute::XTT_Attribute(void)
{
     fAttrID = 0;
     fAttrName = "";
     fAttrAcronym = "";
     fAttrDesc = "";
     fAttrValue = XTT_Attribute::_not_definded_operator_;
     fDefAttrValue = XTT_Attribute::_not_definded_operator_;
     fRelationsType = 1;
     fAttrType = 0;
     fAttrConstrType = 0;
	fChanged = false;
     fIntervalConstraints = new double[2];
     fBraceType = new bool[2];
     fIgnoreConstraint = new bool[2];
     fAttrValues = new QStringList;
     fEnumerateConstraints = new QStringList;
}
// -----------------------------------------------------------------------------

/**
 * Konstruktor klasy XTT_Attribute
 * @param unsigned int _id - Identyfikator atrybutu
 * @returns Brak zwracanych wartosci
*/
XTT_Attribute::XTT_Attribute(unsigned int _id)
{
     fAttrID = _id;
     fAttrName = "";
     fAttrAcronym = "";
     fAttrDesc = "";
     fAttrValue = XTT_Attribute::_not_definded_operator_;
     fDefAttrValue = XTT_Attribute::_not_definded_operator_;
     fRelationsType = 1;
     fAttrType = 0;
     fAttrConstrType = 0;
	fChanged = false;
     fIntervalConstraints = new double[2];
     fBraceType = new bool[2];
     fIgnoreConstraint = new bool[2];
     fAttrValues = new QStringList;
     fEnumerateConstraints = new QStringList;
}
// -----------------------------------------------------------------------------

/**
 * Destruktor klasy
 * @returns Brak zwracanych wartosci
*/
XTT_Attribute::~XTT_Attribute(void)
{
     delete [] fIntervalConstraints;
     delete [] fBraceType;
     delete [] fIgnoreConstraint;
     delete fAttrValues;
     delete fEnumerateConstraints;
}
// -----------------------------------------------------------------------------

/**
 * Metoda ustawaiajaca nazwe atrybutu
 * @param QString _new_name - Nowa nazwa atrybutu
 * @returns Brak zwracanych wartoci
 */
void XTT_Attribute::setName(QString _new_name)
{
	fAttrName = _new_name; 
	fChanged = true;
}
// -----------------------------------------------------------------------------

/**
 * Metoda ustawiajaca acronim atrubutu
 * @param QString _new_arconym - Nowy akronim atrybutu
 * @returns Brak zwracanych wartoci
 */
void XTT_Attribute::setAcronym(QString _new_arconym)
{ 
	fAttrAcronym = _new_arconym;
	fChanged = true;
}
// -----------------------------------------------------------------------------

/**
 * Metoda ustawiajaca opis, konentarz atrybutu
 * @param QString _desc - Tresc opisu, komentarza
 * @returns Brak zwracanych wartoci
 */
void XTT_Attribute::setDescription(QString _desc)
{ 
	fAttrDesc = _desc;
	fChanged = true;
}
// -----------------------------------------------------------------------------

/**
 * Metoda ustawiajaca typ atrybutu
 * @param unsigned int _attr_type - Nowy typ atrybutu
 * @returns Brak zwracanych wartoci
 */
void XTT_Attribute::setType(unsigned int _attr_type)
{ 
	fAttrType = _attr_type; 
	FixValue();				// Poprawa wartosci na zgodne z ograniczeniami
	fChanged = true;
}
// -----------------------------------------------------------------------------

/**
 * Metoda ustawiajaca liste wartosci jakie moze przyjac w trybie wyliczeniowym
 * @param QStringList* _new_atrr_values - Wskanik do listy wartosci
 * @returns Brak zwracanych wartoci
 */
void XTT_Attribute::setEnumValues(QStringList* _new_atrr_values)
{ 
	fAttrValues->clear();
	
	// Dodawanie nowej wartosci
	for(int i=0;i<_new_atrr_values->size();i++)
	{
		QString val = _new_atrr_values->at(i);
		
		// Usuwanie spacji na koncach
		while((val.length() > 0) && (val[0] == ' '))
	     {
	          val = val.remove(0, 1);
	     }

	     while((val.length() > 0) && (val[val.length()-1] == ' '))
	     {
	          val = val.remove(val.length()-1, 1);
	     }
		
		if(val == "")
			continue;
		
		fAttrValues->append(val);
	}
	
	FixValue();				// Poprawa wartosci na zgodne z ograniczeniami
	fChanged = true;
}
// -----------------------------------------------------------------------------

/**
 * Metoda ustawiajaca liste wartosci jakie moze przyjac w trybie wyliczeniowym
 * @param QStringList _new_atrr_values - Lista wartosci
 * @returns Brak zwracanych wartoci
 */
void XTT_Attribute::setEnumValues(QStringList _new_atrr_values)
{ 
	fAttrValues->clear();
	
	// Dodawanie nowej wartosci
	for(int i=0;i<_new_atrr_values.size();i++)
	{
		QString val = _new_atrr_values.at(i);
		
		// Usuwanie spacji na koncach
		while((val.length() > 0) && (val[0] == ' '))
	     {
	          val = val.remove(0, 1);
	     }

	     while((val.length() > 0) && (val[val.length()-1] == ' '))
	     {
	          val = val.remove(val.length()-1, 1);
	     }
		
		if(val == "")
			continue;
		
		fAttrValues->append(val);
	}
	
	FixValue();				// Poprawa wartosci na zgodne z ograniczeniami
	fChanged = true;
}
// -----------------------------------------------------------------------------

/**
 * Metoda ustawiajaca typ ograniczen wartosci atrybutu
 * @param unsigned int _ct - Nowa wartosc typu ograniczen
 * @returns Brak zwracanych wartoci
 */
void XTT_Attribute::setConstraintsType(unsigned int _ct)
{ 
	fAttrConstrType = _ct; 
	FixValue();				// Poprawa wartosci na zgodne z ograniczeniami
	fChanged = true;
}
// -----------------------------------------------------------------------------

/**
 * Ustawia gorne ograniczenie atrybutu
 * @param double _nc - Wartosc gornego ograniczenia
 * @returns Brak zwracanych wartoci
 */
void XTT_Attribute::setUpConstraints(double _nc)
{ 
	fIntervalConstraints[1] = _nc; 
	FixValue();				// Poprawa wartosci na zgodne z ograniczeniami
	fChanged = true;
}
// -----------------------------------------------------------------------------

/**
 * Ustawia gorne dolne atrybutu
 * @param double _nc - Wartosc dolnego ograniczenia
 * @returns Brak zwracanych wartoci
 */
void XTT_Attribute::setLoConstraints(double _nc)
{ 
	fIntervalConstraints[0] = _nc; 
	FixValue();				// Poprawa wartosci na zgodne z ograniczeniami
	fChanged = true;
}
// -----------------------------------------------------------------------------

/**
 * Ustawia rodzaj nawiasu przy dolnym ograniczeniu
 * @param bool _nb - Wartosc rodzaju nowego nawiasu
 * @returns Brak zwracanych wartoci
 */
void XTT_Attribute::setLoBraceType(bool _nb)
{ 
	fBraceType[0] = _nb; 
	FixValue();				// Poprawa wartosci na zgodne z ograniczeniami
	fChanged = true;
}
// -----------------------------------------------------------------------------

/**
 * Ustawia rodzaj nawiasu przy gornym ograniczeniu
 * @param bool _nb - Wartosc rodzaju nowego nawiasu
 * @returns Brak zwracanych wartoci
 */
void XTT_Attribute::setHiBraceType(bool _nb)
{ 
	fBraceType[1] = _nb; 
	FixValue();				// Poprawa wartosci na zgodne z ograniczeniami
	fChanged = true;
}
// -----------------------------------------------------------------------------

/**
 * Ustawia wartosc okreslajaca waznosc dolnego ograniczenia
 * @param bool _lic - Nowa wartosc
 * @returns Brak zwracanych wartoci
 */
void XTT_Attribute::setLoIgnoreConstraints(bool _lic)
{ 
	fIgnoreConstraint[0] = _lic; 
	FixValue();				// Poprawa wartosci na zgodne z ograniczeniami
	fChanged = true;
}
// -----------------------------------------------------------------------------

/**
 * Ustawia wartosc okreslajaca waznosc gornego ograniczenia
 * @param bool _lic - Nowa wartosc
 * @returns Brak zwracanych wartoci
 */
void XTT_Attribute::setHiIgnoreConstraints(bool _hic)
{ 
	fIgnoreConstraint[1] = _hic; 
	FixValue();				// Poprawa wartosci na zgodne z ograniczeniami
	fChanged = true;
}
// -----------------------------------------------------------------------------

/**
 * Metoda ustawiajaca wartosci ograniczenia wyliczeniowego atrybutu
 * @param QStringList* _ec - Wskanik do listy wartosci
 * @returns Brak zwracanych wartoci
 */
void XTT_Attribute::setEnumerateConstraints(QStringList* _ec)
{ 
	fEnumerateConstraints->clear();
	
	// Dodawanie nowej wartosci
	for(int i=0;i<_ec->size();i++)
	{
		QString val = _ec->at(i);
		
		// Usuwanie spacji na koncach
		while((val.length() > 0) && (val[0] == ' '))
	     {
	          val = val.remove(0, 1);
	     }

	     while((val.length() > 0) && (val[val.length()-1] == ' '))
	     {
	          val = val.remove(val.length()-1, 1);
	     }
		
		if(val == "")
			continue;
		
		fEnumerateConstraints->append(val);
	}
	
	FixValue();				// Poprawa wartosci na zgodne z ograniczeniami
	fChanged = true;
}
// -----------------------------------------------------------------------------

/**
 * Metoda ustawiajaca wartosci ograniczenia wyliczeniowego atrybutu
 * @param QStringList _ec - Lista wartosci
 * @returns Brak zwracanych wartoci
 */
void XTT_Attribute::setEnumerateConstraints(QStringList _ec)
{ 
	fEnumerateConstraints->clear();
	
	// Dodawanie nowej wartosci
	for(int i=0;i<_ec.size();i++)
	{
		QString val = _ec.at(i);
		
		// Usuwanie spacji na koncach
		while((val.length() > 0) && (val[0] == ' '))
	     {
	          val = val.remove(0, 1);
	     }

	     while((val.length() > 0) && (val[val.length()-1] == ' '))
	     {
	          val = val.remove(val.length()-1, 1);
	     }
		
		if(val == "")
			continue;
		
		fEnumerateConstraints->append(val);
	}
	
	FixValue();				// Poprawa wartosci na zgodne z ograniczeniami
	fChanged = true;
}
// -----------------------------------------------------------------------------

/**
 * Metoda ustawiajaca znacznik czy dany obiekt byl modyfikowany
 * @param bool _c - Nowa wartosc statusu
 * @returns brak zwracanych wartosci
*/
void XTT_Attribute::setChanged(bool _c)
{ 
	fChanged = _c; 
}
// -----------------------------------------------------------------------------

/**
 * Metoda ustawiajaca wartosc tego atrybutu. Sprawdza czy podana wartosc zgadza
 * sie z typem i ograniczeniami
 * @param QString _dv - Nowa wartosc atrybutu podana jako string
 * @returns true gdy uda sie ustawic wartosc, false gdy jest niezgodna z
 * dziedzina
*/
bool XTT_Attribute::setDefaultValue(QString _dv)
{
     // Wyrowananie wielkosci znakow
     _dv = _dv.toLower();

     // Kiedy op:nd
     if(_dv.toLower() == XTT_Attribute::_not_definded_operator_)
     {
          fDefAttrValue = _dv;
          return true;
     }

     // Stringi uzywane w komunikatach
     const QString str1 = QObject::tr("XTT Editor");

     // Gdy bool
     if((fAttrType == 1) && (_dv != "true") && (_dv != "false"))
     {
          const QString str2 = fAttrName + "'s default value is out of domain.";
          QMessageBox::critical(NULL, str1.toUtf8(), str2.toUtf8(), QMessageBox::Ok);
          return false;
     }

     // Gdy integer
     if(fAttrType == 2)
     {
          bool ok;

          // Sprawdzanie czy na koncu liczby nie jest podany system liczbowy
          QString sysl = "10";
          if(_dv[_dv.length()-1] == ')')
          {
               // Wyzerowanie systemu
               sysl = "";
               
               // Wczytywanie wartosci systemu liczbowego
               for(int i=_dv.length()-2;((i>=0)&&(_dv[i]!='('));i--)
                    sysl = _dv[i] + sysl;
                    
               // Usuwanie liczby w nawiasie
               _dv = _dv.remove(_dv.length()-sysl.length()-2, sysl.length()+2);
          }


          int val = _dv.toInt(&ok, sysl.toInt());
          if(!ok)
          {
               const QString str2 = fAttrName + "'s default value is not INTEGER type.";
               QMessageBox::critical(NULL, str1, str2, QMessageBox::Ok);
               return false;
          }
          // Jezeli ok to konwersja na system dziesietny
          _dv = QString::number(val);

          // Sprawdzanie ograniczen

          // Gdy przedzialowe ograniczenie
          if(fAttrConstrType == 1)
          {
               // Gdy przekracza ograniczenia przedzialowe
               if(((val < fIntervalConstraints[0]) && (!fIgnoreConstraint[0])) || ((val > fIntervalConstraints[1]) && (!fIgnoreConstraint[1])))
               {
                    const QString str2 = fAttrName + "'s default value is out of range.";
                    QMessageBox::critical(NULL, str1, str2, QMessageBox::Ok);
                    return false;
               }
               // Gdy brak zgodnosci z nawiasem poczatkowym
               if(((fBraceType[0]) && (val == fIntervalConstraints[0]) && (!fIgnoreConstraint[0])) || ((fBraceType[1]) && (val == fIntervalConstraints[1]) && (!fIgnoreConstraint[1])))
               {
                    const QString str2 = fAttrName + "'s default value is out of range.";
                    QMessageBox::critical(NULL, str1, str2, QMessageBox::Ok);
                    return false;
               }
          }

          // Gdy zbior
          if(fAttrConstrType == 2)
          {
               // Sprawdzanie czy podana wartosc nalezy do zbioru
               if(!fEnumerateConstraints->contains(_dv, Qt::CaseInsensitive))
               {
                    // To moze jeszcze tak sie zdarzyc ze zamiast wartosci podano index
                    // tej wartosci
                    bool conv;
                    int _v = _dv.toInt(&conv, 10);
                    if((_v >= 0) && (_v < fEnumerateConstraints->size()))
                         _dv = fEnumerateConstraints->at(_v);

                    if((!conv) || (_v < 0) || (_v >= fEnumerateConstraints->size()))
                    {
                         const QString str2 = fAttrName + "'s default value is out of set.";
                         QMessageBox::critical(NULL, str1, str2, QMessageBox::Ok);
                         return false;
                    }
               }
          }
     }

     // Gdy float
     if(fAttrType == 3)
     {
          bool ok;
          float val = _dv.toFloat(&ok);
          if(!ok)
          {
               const QString str2 = fAttrName + "'s default value is not FLOAT type.";
               QMessageBox::critical(NULL, str1, str2, QMessageBox::Ok);
               return false;
          }

          // Sprawdzanie ograniczen

          // Gdy przedzialowe ograniczenie
          if(fAttrConstrType == 1)
          {
               // Gdy przekracza ograniczenia przedzialowe
               if(((val < fIntervalConstraints[0]) && (!fIgnoreConstraint[0])) || ((val > fIntervalConstraints[1]) && (!fIgnoreConstraint[1])))
               {
                    const QString str2 = fAttrName + "'s default value is out of range.";
                    QMessageBox::critical(NULL, str1, str2, QMessageBox::Ok);
                    return false;
               }
               // Gdy brak zgodnosci z nawiasem poczatkowym
               if(((fBraceType[0]) && (val == fIntervalConstraints[0]) && (!fIgnoreConstraint[0])) || ((fBraceType[1]) && (val == fIntervalConstraints[1]) && (!fIgnoreConstraint[1])))
               {
                    const QString str2 = fAttrName + "'s default value is out of range.";
                    QMessageBox::critical(NULL, str1, str2, QMessageBox::Ok);
                    return false;
               }
          }

          // Gdy zbior
          if(fAttrConstrType == 2)
          {
               // Sprawdzanie czy podana wartosc nalezy do zbioru
               if(!fEnumerateConstraints->contains(_dv, Qt::CaseInsensitive))
               {
                    // To moze jeszcze tak sie zdarzyc ze zamiast wartosci podano index
                    // tej wartosci
                    bool conv;
                    int _v = _dv.toInt(&conv, 10);
                    if((_v >= 0) && (_v < fEnumerateConstraints->size()))
                         _dv = fEnumerateConstraints->at(_v);

                    if((!conv) || (_v < 0) || (_v >= fEnumerateConstraints->size()))
                    {
                         const QString str2 = fAttrName + "'s default value is out of set.";
                         QMessageBox::critical(NULL, str1, str2, QMessageBox::Ok);
                         return false;
                    }
               }
          }
     }

     // Gdy symbolic
     if(fAttrType == 4)
     {
          if((_dv.toLower() == XTT_Attribute::_any_operator_) || (_dv.toLower() == XTT_Attribute::_not_definded_operator_))
          {
               const QString str2 = fAttrName + "'s default value can not contain predefined symbols.";
               QMessageBox::critical(NULL, str1, str2, QMessageBox::Ok);
               return false;
          }
     }

     // Gdy enumerative
     if(fAttrType == 5)
     {
          // Sprawdzanie czy podana wartosc nalezy do zbioru zadeklarowanego. Sprawdzanie ograniczen jest w
          // przypdaku tego typu zmiennych niepotrzebne
          if(!fAttrValues->contains(_dv, Qt::CaseInsensitive))
          {
               // To moze jeszcze tak sie zdarzyc ze zamiast wartosci podano index
               // tej wartosci
               bool conv;
               int _v = _dv.toInt(&conv, 10);
               if((_v >= 0) && (_v < fAttrValues->size()))
                    _dv = fAttrValues->at(_v);

               if((!conv) || (_v < 0) || (_v >= fAttrValues->size()))
               {
                    const QString str2 = fAttrName + "'s default value is out of set.";
                    QMessageBox::critical(NULL, str1, str2, QMessageBox::Ok);
                    return false;
               }
          }
     }

     // Gdy wszystko ok, to zapisujemy wartosc
     fDefAttrValue = _dv;
	fChanged = true;

     return true;
}
// -----------------------------------------------------------------------------

bool XTT_Attribute::checkValue(QString _v)
{
     // Wyrowananie wielkosci znakow
     _v = _v.toLower();

     // Kiedy op:nd
     if(_v.toLower() == XTT_Attribute::_not_definded_operator_)
          return true;

     // Gdy bool
     if((fAttrType == 1) && (_v != "true") && (_v != "false"))
          return false;

     // Gdy integer
     if(fAttrType == 2)
     {
          bool ok;

          // Sprawdzanie czy na koncu liczby nie jest podany system liczbowy
          QString sysl = "10";
          if(_v[_v.length()-1] == ')')
          {
               // Wyzerowanie systemu
               sysl = "";

               // Wczytywanie wartosci systemu liczbowego
               for(int i=_v.length()-2;((i>=0)&&(_v[i]!='('));i--)
                    sysl = _v[i] + sysl;
                    
               // Usuwanie liczby w nawiasie
               _v = _v.remove(_v.length()-sysl.length()-2, sysl.length()+2);
          }


          int val = _v.toInt(&ok, sysl.toInt());
          if(!ok)
               return false;
			
          // Jezeli ok to konwersja na system dziesietny
          _v = QString::number(val);

          // Sprawdzanie ograniczen

          // Gdy przedzialowe ograniczenie
          if(fAttrConstrType == 1)
          {
               // Gdy przekracza ograniczenia przedzialowe
               if(((val < fIntervalConstraints[0]) && (!fIgnoreConstraint[0])) || ((val > fIntervalConstraints[1]) && (!fIgnoreConstraint[1])))
                    return false;

               // Gdy brak zgodnosci z nawiasem poczatkowym
               if(((fBraceType[0]) && (val == fIntervalConstraints[0]) && (!fIgnoreConstraint[0])) || ((fBraceType[1]) && (val == fIntervalConstraints[1]) && (!fIgnoreConstraint[1])))
                    return false;
          }

          // Gdy zbior
          if(fAttrConstrType == 2)
          {
               // Sprawdzanie czy podana wartosc nalezy do zbioru
               if(!fEnumerateConstraints->contains(_v, Qt::CaseInsensitive))
               {
                    // To moze jeszcze tak sie zdarzyc ze zamiast wartosci podano index
                    // tej wartosci
                    bool conv;
                    int _vi = _v.toInt(&conv, 10);
                    if((_vi >= 0) && (_vi < fEnumerateConstraints->size()))
                         _v = fEnumerateConstraints->at(_vi);

                    if((!conv) || (_vi < 0) || (_vi >= fEnumerateConstraints->size()))
                         return false;
               }
          }
     }

     // Gdy float
     if(fAttrType == 3)
     {
          bool ok;
          float val = _v.toFloat(&ok);
          if(!ok)
               return false;

          // Sprawdzanie ograniczen

          // Gdy przedzialowe ograniczenie
          if(fAttrConstrType == 1)
          {
               // Gdy przekracza ograniczenia przedzialowe
               if(((val < fIntervalConstraints[0]) && (!fIgnoreConstraint[0])) || ((val > fIntervalConstraints[1]) && (!fIgnoreConstraint[1])))
                    return false;
               // Gdy brak zgodnosci z nawiasem poczatkowym
               if(((fBraceType[0]) && (val == fIntervalConstraints[0]) && (!fIgnoreConstraint[0])) || ((fBraceType[1]) && (val == fIntervalConstraints[1]) && (!fIgnoreConstraint[1])))
                    return false;
          }

          // Gdy zbior
          if(fAttrConstrType == 2)
          {
               // Sprawdzanie czy podana wartosc nalezy do zbioru
               if(!fEnumerateConstraints->contains(_v, Qt::CaseInsensitive))
               {
                    // To moze jeszcze tak sie zdarzyc ze zamiast wartosci podano index
                    // tej wartosci
                    bool conv;
                    int _vi = _v.toInt(&conv, 10);
                    if((_vi >= 0) && (_vi < fEnumerateConstraints->size()))
                         _v = fEnumerateConstraints->at(_vi);

                    if((!conv) || (_vi < 0) || (_vi >= fEnumerateConstraints->size()))
                         return false;
               }
          }
     }

     // Gdy symbolic
     if(fAttrType == 4)
     {
          if((_v == XTT_Attribute::_any_operator_) || (_v == XTT_Attribute::_not_definded_operator_))
               return false;
     }

     // Gdy enumerative
     if(fAttrType == 5)
     {
          // Sprawdzanie czy podana wartosc nalezy do zbioru zadeklarowanego. Sprawdzanie ograniczen jest w
          // przypdaku tego typu zmiennych niepotrzebne
          if(!fAttrValues->contains(_v, Qt::CaseInsensitive))
          {
               // To moze jeszcze tak sie zdarzyc ze zamiast wartosci podano index
               // tej wartosci
               bool conv;
               int _vi = _v.toInt(&conv, 10);
               if((_vi >= 0) && (_vi < fAttrValues->size()))
                    _v = fAttrValues->at(_vi);

               if((!conv) || (_vi < 0) || (_vi >= fAttrValues->size()))
                    return false;
          }
     }

     return true;
}
// -----------------------------------------------------------------------------

/**
 * Metoda poprawiajaca wartosc atrybutu jezeli przy zmianie ograniczen nie 
 * miescie sie ona w zakresie
 * @returns Brak zwracanych wartoci
 */
void XTT_Attribute::FixValue(void)
{
	// Poprawa wartosci domyslnej
	if(!checkValue(fDefAttrValue))
	{
		// Jezeli ograniczenie przedzailowe
		if(((fAttrType == 2) || (fAttrType == 3)) && (fAttrConstrType == 1))
		{
			// Gdy obydwa ograniczenia sa uwzgledniane
			if((!fIgnoreConstraint[0]) && (!fIgnoreConstraint[1]))
				fDefAttrValue = QString::number((fIntervalConstraints[1]+fIntervalConstraints[0])/2);
				
			// Gdy dolne ograniczenie jest uwzgledniane
			if((!fIgnoreConstraint[0]) && (fIgnoreConstraint[1]))
				fDefAttrValue = QString::number(fIntervalConstraints[0]+1);
				
			// Gdy gorne ograniczenie jest uwzgledniane
			if((fIgnoreConstraint[0]) && (!fIgnoreConstraint[1]))
				fDefAttrValue = QString::number(fIntervalConstraints[1]-1);
		}
		
		// Jezeli ograniczenie jako zbior elementow
		if(((fAttrType == 2) || (fAttrType == 3)) && (fAttrConstrType == 2))
		{
			if(fEnumerateConstraints->size() > 0)
				fDefAttrValue = fEnumerateConstraints->at(0);
		}
	}
	
	// Poprawa wartosci aktualnej
	if(!checkValue(fAttrValue))
	{
		// Jezeli ograniczenie przedzailowe
		if(((fAttrType == 2) || (fAttrType == 3)) && (fAttrConstrType == 1))
		{
			// Gdy obydwa ograniczenia sa uwzgledniane
			if((!fIgnoreConstraint[0]) && (!fIgnoreConstraint[1]))
				fAttrValue = QString::number((fIntervalConstraints[1]+fIntervalConstraints[0])/2);
				
			// Gdy dolne ograniczenie jest uwzgledniane
			if((!fIgnoreConstraint[0]) && (fIgnoreConstraint[1]))
				fAttrValue = QString::number(fIntervalConstraints[0]+1);
				
			// Gdy gorne ograniczenie jest uwzgledniane
			if((fIgnoreConstraint[0]) && (!fIgnoreConstraint[1]))
				fAttrValue = QString::number(fIntervalConstraints[1]-1);
		}
		
		// Jezeli ograniczenie jako zbior elementow
		if(((fAttrType == 2) || (fAttrType == 3)) && (fAttrConstrType == 2))
		{
			if(fEnumerateConstraints->size() > 0)
				fAttrValue = fEnumerateConstraints->at(0);
		}
	}
}
// -----------------------------------------------------------------------------


/**
 * Metoda ustawiajaca wartosc tego atrybutu. Sprawdza czy podana wartosc zgadza
 * sie z typem i ograniczeniami
 * @param QString _v - Nowa wartosc atrybutu podana jako string
 * @returns true gdy uda sie ustawic wartosc, false gdy jest niezgodna z
 * dziedzina
*/
bool XTT_Attribute::setValue(QString _v)
{
     // Wyrowananie wielkosci znakow
     _v = _v.toLower();

     // Kiedy op:nd
     if(_v.toLower() == XTT_Attribute::_not_definded_operator_)
     {
          fAttrValue = _v;
          return true;
     }

     // Stringi uzywane w komunikatach
     const QString str1 = QObject::tr("XTT Editor");

     // Gdy bool
     if((fAttrType == 1) && (_v != "true") && (_v != "false"))
     {
          const QString str2 = fAttrName + "'s value is out of domain.";
          QMessageBox::critical(NULL, str1, str2, QMessageBox::Ok);
          return false;
     }

     // Gdy integer
     if(fAttrType == 2)
     {
          bool ok;

          // Sprawdzanie czy na koncu liczby nie jest podany system liczbowy
          QString sysl = "10";
          if(_v[_v.length()-1] == ')')
          {
               // Wyzerowanie systemu
               sysl = "";

               // Wczytywanie wartosci systemu liczbowego
               for(int i=_v.length()-2;((i>=0)&&(_v[i]!='('));i--)
                    sysl = _v[i] + sysl;
                    
               // Usuwanie liczby w nawiasie
               _v = _v.remove(_v.length()-sysl.length()-2, sysl.length()+2);
          }


          int val = _v.toInt(&ok, sysl.toInt());
          if(!ok)
          {
               const QString str2 = fAttrName + "'s value \"" + _v + "\" is not INTEGER type.";
               QMessageBox::critical(NULL, str1, str2, QMessageBox::Ok);
               return false;
          }
          // Jezeli ok to konwersja na system dziesietny
          _v = QString::number(val);

          // Sprawdzanie ograniczen

          // Gdy przedzialowe ograniczenie
          if(fAttrConstrType == 1)
          {
               // Gdy przekracza ograniczenia przedzialowe
               if(((val < fIntervalConstraints[0]) && (!fIgnoreConstraint[0])) || ((val > fIntervalConstraints[1]) && (!fIgnoreConstraint[1])))
               {
                    const QString str2 = fAttrName + "'s value is out of range.";
                    QMessageBox::critical(NULL, str1, str2, QMessageBox::Ok);
                    return false;
               }
               // Gdy brak zgodnosci z nawiasem poczatkowym
               if(((fBraceType[0]) && (val == fIntervalConstraints[0]) && (!fIgnoreConstraint[0])) || ((fBraceType[1]) && (val == fIntervalConstraints[1]) && (!fIgnoreConstraint[1])))
               {
                    const QString str2 = fAttrName + "'s value is out of range.";
                    QMessageBox::critical(NULL, str1, str2, QMessageBox::Ok);
                    return false;
               }
          }

          // Gdy zbior
          if(fAttrConstrType == 2)
          {
               // Sprawdzanie czy podana wartosc nalezy do zbioru
               if(!fEnumerateConstraints->contains(_v, Qt::CaseInsensitive))
               {
                    // To moze jeszcze tak sie zdarzyc ze zamiast wartosci podano index
                    // tej wartosci
                    bool conv;
                    int _vi = _v.toInt(&conv, 10);
                    if((_vi >= 0) && (_vi < fEnumerateConstraints->size()))
                         _v = fEnumerateConstraints->at(_vi);

                    if((!conv) || (_vi < 0) || (_vi >= fEnumerateConstraints->size()))
                    {
                         const QString str2 = fAttrName + "'s value is out of set.";
                         QMessageBox::critical(NULL, str1, str2, QMessageBox::Ok);
                         return false;
                    }
               }
          }
     }

     // Gdy float
     if(fAttrType == 3)
     {
          bool ok;
          float val = _v.toFloat(&ok);
          if(!ok)
          {
               const QString str2 = fAttrName + "'s value is not FLOAT type.";
               QMessageBox::critical(NULL, str1, str2, QMessageBox::Ok);
               return false;
          }

          // Sprawdzanie ograniczen

          // Gdy przedzialowe ograniczenie
          if(fAttrConstrType == 1)
          {
               // Gdy przekracza ograniczenia przedzialowe
               if(((val < fIntervalConstraints[0]) && (!fIgnoreConstraint[0])) || ((val > fIntervalConstraints[1]) && (!fIgnoreConstraint[1])))
               {
                    const QString str2 = fAttrName + "'s value is out of range.";
                    QMessageBox::critical(NULL, str1, str2, QMessageBox::Ok);
                    return false;
               }
               // Gdy brak zgodnosci z nawiasem poczatkowym
               if(((fBraceType[0]) && (val == fIntervalConstraints[0]) && (!fIgnoreConstraint[0])) || ((fBraceType[1]) && (val == fIntervalConstraints[1]) && (!fIgnoreConstraint[1])))
               {
                    const QString str2 = fAttrName + "'s value is out of range.";
                    QMessageBox::critical(NULL, str1, str2, QMessageBox::Ok);
                    return false;
               }
          }

          // Gdy zbior
          if(fAttrConstrType == 2)
          {
               // Sprawdzanie czy podana wartosc nalezy do zbioru
               if(!fEnumerateConstraints->contains(_v, Qt::CaseInsensitive))
               {
                    // To moze jeszcze tak sie zdarzyc ze zamiast wartosci podano index
                    // tej wartosci
                    bool conv;
                    int _vi = _v.toInt(&conv, 10);
                    if((_vi >= 0) && (_vi < fEnumerateConstraints->size()))
                         _v = fEnumerateConstraints->at(_vi);

                    if((!conv) || (_vi < 0) || (_vi >= fEnumerateConstraints->size()))
                    {
                         const QString str2 = fAttrName + "'s value is out of set.";
                         QMessageBox::critical(NULL, str1, str2, QMessageBox::Ok);
                         return false;
                    }
               }
          }
     }

     // Gdy symbolic
     if(fAttrType == 4)
     {
          if((_v == XTT_Attribute::_any_operator_) || (_v == XTT_Attribute::_not_definded_operator_))
          {
               const QString str2 = fAttrName + "'s value can not contain predefined symbols.";
               QMessageBox::critical(NULL, str1, str2, QMessageBox::Ok);
               return false;
          }
     }

     // Gdy enumerative
     if(fAttrType == 5)
     {
          // Sprawdzanie czy podana wartosc nalezy do zbioru zadeklarowanego. Sprawdzanie ograniczen jest w
          // przypdaku tego typu zmiennych niepotrzebne
          if(!fAttrValues->contains(_v, Qt::CaseInsensitive))
          {
               // To moze jeszcze tak sie zdarzyc ze zamiast wartosci podano index
               // tej wartosci
               bool conv;
               int _vi = _v.toInt(&conv, 10);
               if((_vi >= 0) && (_vi < fAttrValues->size()))
                    _v = fAttrValues->at(_vi);

               if((!conv) || (_vi < 0) || (_vi >= fAttrValues->size()))
               {
                    const QString str2 = fAttrName + "'s value is out of set.";
                    QMessageBox::critical(NULL, str1, str2, QMessageBox::Ok);
                    return false;
               }
          }
     }

     // Gdy wszystko ok, to zapisujemy wartosc
     fAttrValue = _v;
	fChanged = true;

     return true;
}
// -----------------------------------------------------------------------------

/**
 * Metoda klasy XTT_Attribute konwertujaca liste wartosci na string
 * @returns Zwraca string wartosci oddzielony srednikami
*/
QString XTT_Attribute::EnumerateConstraintsString(void)
{
     QString res = "";
     for(int i=0;i<fEnumerateConstraints->size();i++)
          res = res + fEnumerateConstraints->at(i) + ";";

     return res;
}
// -----------------------------------------------------------------------------

/**
 * Metoda klasy XTT_Attribute konwertujaca liste wartosci na string
 * @returns Zwraca string wartosci oddzielony srednikami
*/
QString XTT_Attribute::AttrValuesString(void)
{
     QString res = "";
     for(int i=0;i<fAttrValues->size();i++)
          res = res + fAttrValues->at(i) + ";";

     return res;
}
// -----------------------------------------------------------------------------

/**
 * Metoda klasy XTT_Attribute tworzaca reprezentacje ograniczen atrybutu
 * @returns Zwraca reprezentacje ograniczen jako string
*/
QString XTT_Attribute::ConstraintsRepresentation(void)
{
     QString res = "";

     // Gdy bez ograniczen
     if(fAttrConstrType == 0)
          res = "No constraints definded";

     // Gdy ograniczenia przedzialowe
     if(fAttrConstrType == 1)
     {
          // Lewy nawias
          if(fBraceType[0])
               res += "(";
          if(!fBraceType[0])
               res += "[";

          // Dolna wartosc
          if(fIgnoreConstraint[0])
               res += "-inf";
          if(!fIgnoreConstraint[0])
               res += QString::number(fIntervalConstraints[0], 'f', 5);

          // Separator - srednik
          res += "; ";

          // Gorna wartosc
          if(fIgnoreConstraint[1])
               res += "inf";
          if(!fIgnoreConstraint[1])
               res += QString::number(fIntervalConstraints[1], 'f', 5);

          // Prawy nawias
          if(fBraceType[1])
               res += ")";
          if(!fBraceType[1])
               res += "]";
     }

     // Gdy ograniczenia wyliczeniowe
     if(fAttrConstrType == 2)
     {
          res = "{";
          for(int i=0;i<fEnumerateConstraints->size();i++)
               res += fEnumerateConstraints->at(i) + "; ";
          res = res.remove(res.length()-2, 2);
          res += "}";
     }

     // Gdy typ wyliczeniowy to tez trzeba podac dziedzine
     if(fAttrType == 5)
     {
          res = "{";
          for(int i=0;i<fAttrValues->size();i++)
               res += fAttrValues->at(i) + "; ";
          res = res.remove(res.length()-2, 2);
          res += "}";
     }

     return res;
}
// -----------------------------------------------------------------------------

/**
 * Metoda klasy XTT_Attribute pozwalajaca na ustawienie typu relacji atrybutu
 * @param unsigned int _rt - Typ relacji
 * @returns true jezeli podany typ jest poprawny inaczej false
*/
bool XTT_Attribute::setRelationsType(unsigned int _rt)
{
     if(((int)_rt < 0) || (_rt >= 3))
          return false;

     fRelationsType = _rt;
	fChanged = true;
	
     return true;
}
// -----------------------------------------------------------------------------

/**
 * Metoda zapisujaca wszystkie dane w formacie xttml 1.0
 * @returns Zwraca liste stringow w tym formacie
*/
QStringList* XTT_Attribute::XTTML_1_0(void)
{
     QStringList* res = new QStringList;
     res->clear();
     
     *res << "<attribute id=\"" + QString::number(fAttrID) + "\" relation=\"" +  QString::number(fRelationsType) + "\" type=\"" + QString::number(fAttrType) + "\">";
     *res << "\t<name>" + fAttrName + "</name>";
     *res << "\t<acronym>" + fAttrAcronym + "</acronym>";
     *res << "\t<description>" + fAttrDesc + "</description>";
     *res << "\t<defval>" + fDefAttrValue + "</defval>";
     *res << "\t<value>" + fAttrValue + "</value>";

     // Wartosci gdy jest to typ wyliczeniowy
     *res << "\t<enumerate_values>";
     for(int i=0;i<fAttrValues->size();i++)
          *res << "\t\t<ev>"+ fAttrValues->at(i) +"</ev>";
     *res << "\t</enumerate_values>";

     // Ograniczenia
     *res << "\t<constraints type=\"" + QString::number(fAttrConstrType) + "\">";

     *res << "\t\t<interval>";
     *res << "\t\t\t<from>" + QString::number(fIntervalConstraints[0]) + "</from>";
     *res << "\t\t\t<to>" + QString::number(fIntervalConstraints[1]) + "</to>";
     *res << "\t\t\t<left_brace_type>" + QString::number(fBraceType[0]?1:0) + "</left_brace_type>";
     *res << "\t\t\t<right_brace_type>" + QString::number(fBraceType[1]?1:0) + "</right_brace_type>";
     *res << "\t\t\t<ignore_left>" + QString::number(fIgnoreConstraint[0]?1:0) + "</ignore_left>";
     *res << "\t\t\t<ignore_right>" + QString::number(fIgnoreConstraint[1]?1:0) + "</ignore_right>";
     *res << "\t\t</interval>";

     *res << "\t\t<enumerate_constraints>";
     for(int i=0;i<fEnumerateConstraints->size();i++)
          *res << "\t\t\t<ec>"+ fEnumerateConstraints->at(i) +"</ec>";
     *res << "\t\t</enumerate_constraints>";

     *res << "\t</constraints>";
     
     *res << "</attribute>";

     return res;
}
// -----------------------------------------------------------------------------

#if QT_VERSION >= 0x040300
/**
 * Metoda zapisujaca wszystkie dane w formacie xttml 1.0
 * @param QXmlStreamWriter* _xmldoc - wskanik do dokumentu
 * @returns Brak zwracanych wartosci
*/
void XTT_Attribute::XTTML_1_0(QXmlStreamWriter* _xmldoc)
{
	_xmldoc->writeStartElement("attribute");
	_xmldoc->writeAttribute("id", QString::number(fAttrID));
	_xmldoc->writeAttribute("relation", QString::number(fRelationsType));
	_xmldoc->writeAttribute("type", QString::number(fAttrType));
	
		_xmldoc->writeTextElement("name", fAttrName);
		_xmldoc->writeTextElement("acronym", fAttrAcronym);
		_xmldoc->writeTextElement("description", fAttrDesc);
		_xmldoc->writeTextElement("defval", fDefAttrValue);
		_xmldoc->writeTextElement("value", fAttrValue);

	     // Wartosci gdy jest to typ wyliczeniowy
		_xmldoc->writeStartElement("enumerate_values");
	     for(int i=0;i<fAttrValues->size();i++)
			_xmldoc->writeTextElement("ev", fAttrValues->at(i));
	     _xmldoc->writeEndElement();

	     // Ograniczenia
		_xmldoc->writeStartElement("constraints");
		_xmldoc->writeAttribute("type", QString::number(fAttrConstrType));

			_xmldoc->writeStartElement("interval");
				_xmldoc->writeTextElement("from", QString::number(fIntervalConstraints[0]));
				_xmldoc->writeTextElement("to", QString::number(fIntervalConstraints[1]));
				_xmldoc->writeTextElement("left_brace_type", QString::number(fBraceType[0]?1:0));
				_xmldoc->writeTextElement("right_brace_type", QString::number(fBraceType[1]?1:0));
				_xmldoc->writeTextElement("ignore_left", QString::number(fIgnoreConstraint[0]?1:0));
				_xmldoc->writeTextElement("ignore_right", QString::number(fIgnoreConstraint[1]?1:0));
		     _xmldoc->writeEndElement();

			_xmldoc->writeStartElement("enumerate_constraints");
		     for(int i=0;i<fEnumerateConstraints->size();i++)
				_xmldoc->writeTextElement("ec", fEnumerateConstraints->at(i));
		     _xmldoc->writeEndElement();

	     _xmldoc->writeEndElement();
     
     _xmldoc->writeEndElement();
}
#endif
// -----------------------------------------------------------------------------

/**
 * Metoda wczytujaca dane o strybucie z pliku w formacie XTTML
 * @param QStringList* list - Wskanik do sekcji dotyczacej tego atrybutu
 * @returns true jezeli dane sa ok, ianczej false
*/
bool XTT_Attribute::FromXTTML_1_0(QStringList* list)
{
     // Usuwanie spacji i tabulatorow wierszach
     for(int i=0;i<list->size();i++)
     {
          QString line = list->at(i);
          while(line.contains("\t"))
          {
               int index = line.indexOf("\t");
               line = line.remove(index, 1);
          }
          list->replace(i, line);
     }

     // Zakladamy ze pierwszy wiersz jest naglowkiem
     if(!list->at(0).contains("<attribute", Qt::CaseInsensitive))
          return false;
     if(!list->contains("</attribute>", Qt::CaseInsensitive))
          return false;

     // Reszta zakladamy ze jest ok

     // Spis tagow
     QStringList tags;
     tags.clear();
     tags << "<attribute" << "<name>" << "<acronym>" << "<description>" << "<defval>" << "<value>" << "<enumerate_values>" << "<ev>";
     tags << "<constraints" << "<interval>" << "<from>" << "<to>" << "<left_brace_type>" << "<right_brace_type>" << "<ignore_left>" << "<ignore_right>" << "<enumerate_constraints>" << "<ec>";

     for(int line=0;line<list->size();line++)
     {
          // Sprawdzanie wystapienia tagu
          int action = -1;
          for(int tag=0;tag<tags.size();tag++)
               if(list->at(line).contains(tags.at(tag), Qt::CaseInsensitive))
                    action = tag;

          if(action == -1)
               continue;

          QString end_tag = tags.at(action);
          end_tag = end_tag.insert(1, "/");
          QString l = list->at(line);
          l = l.remove(l.indexOf(tags.at(action), Qt::CaseInsensitive), tags.at(action).length());
          l = l.mid(0, l.indexOf(end_tag, Qt::CaseInsensitive));

          // Wczytywanie parametrow w zaleznosci od wykrytego tagu
          if(action == 0)
          {
               fAttrID = (unsigned int)XML_ReadValue(list->at(line), "id").toInt();
               fRelationsType = (unsigned int)XML_ReadValue(list->at(line), "relation").toInt();
               fAttrType = (unsigned int)XML_ReadValue(list->at(line), "type").toInt();
          }
          if(action == 1)
               fAttrName = l;
          if(action == 2)
               fAttrAcronym = l;
          if(action == 3)
               fAttrDesc = l;
          if(action == 4)
               fDefAttrValue = l;
          if(action == 5)
               fAttrValue = l;
          if(action == 7)
               fAttrValues->append(l);
          if(action == 8)
               fAttrConstrType = (unsigned int)XML_ReadValue(list->at(line), "type").toInt();
          if(action == 10)
               fIntervalConstraints[0] = l.toDouble();
          if(action == 11)
               fIntervalConstraints[1] = l.toDouble();
          if(action == 12)
               fBraceType[0] = (bool)l.toInt();
          if(action == 13)
               fBraceType[1] = (bool)l.toInt();
          if(action == 14)
               fIgnoreConstraint[0] = (bool)l.toInt();
          if(action == 15)
               fIgnoreConstraint[1] = (bool)l.toInt();
          if(action == 17)
               fEnumerateConstraints->append(l);
     }

     return true;
}
// -----------------------------------------------------------------------------

/**
 * Metoda wczytujaca dane o strybucie z pliku w formacie XTTML
 * @param QDomElement& root - Referencja do sekcji dotyczacej tego atrybutu
 * @returns true jezeli dane sa ok, ianczej false
*/
bool XTT_Attribute::FromXTTML_1_0(QDomElement& root)
{
	if(root.tagName().toLower() != "attribute")
		return false;

	QDomElement DOMname = root.firstChildElement("name");
	QDomElement DOMacronym = root.firstChildElement("acronym");
	QDomElement DOMdesc = root.firstChildElement("description");
	QDomElement DOMdefval = root.firstChildElement("defval");
	QDomElement DOMval = root.firstChildElement("value");
	QDomElement DOMconstr = root.firstChildElement("constraints");
	QDomElement DOMevals = root.firstChildElement("enumerate_values");
	
	// Wczytanie wartosci wyliczeniowych atrybutu
	QDomElement DOMev = DOMevals.firstChildElement("ev");
		while(!DOMev.isNull())
		{	
			fAttrValues->append(DOMev.text());
			DOMev = DOMev.nextSiblingElement("ev");
		}

	fAttrID = (unsigned int)root.attribute("id", "0").toInt();
	fRelationsType = (unsigned int)root.attribute("relation", "0").toInt();
	fAttrType = (unsigned int)root.attribute("type", "0").toInt();

	fAttrName = DOMname.text();
	fAttrAcronym = DOMacronym.text();
	fAttrDesc = DOMdesc.text();
	fDefAttrValue = DOMdefval.text();
	fAttrValue = DOMval.text();
	fAttrConstrType = (unsigned int)DOMconstr.attribute("type", "0").toInt();
		QDomElement DOMinterval = DOMconstr.firstChildElement("interval");
			QDomElement DOMfrom = DOMinterval.firstChildElement("from");
			QDomElement DOMto = DOMinterval.firstChildElement("to");
			QDomElement DOMlbt = DOMinterval.firstChildElement("left_brace_type");
			QDomElement DOMrbt = DOMinterval.firstChildElement("right_brace_type");
			QDomElement DOMil = DOMinterval.firstChildElement("ignore_left");
			QDomElement DOMir = DOMinterval.firstChildElement("ignore_right");
			fIntervalConstraints[0] = DOMfrom.text().toDouble();
			fIntervalConstraints[1] = DOMto.text().toDouble();
			fBraceType[0] = (bool)DOMlbt.text().toInt();
			fBraceType[1] = (bool)DOMrbt.text().toInt();
			fIgnoreConstraint[0] = (bool)DOMil.text().toInt();
			fIgnoreConstraint[1] = (bool)DOMir.text().toInt();
		QDomElement DOMenumconstrs = DOMconstr.firstChildElement("enumerate_constraints");
			QDomElement DOMec = DOMenumconstrs.firstChildElement("ec");
			while(!DOMec.isNull())
			{	
				fEnumerateConstraints->append(DOMec.text());
				DOMec = DOMec.nextSiblingElement("ec");
			}

     return true;
}
// -----------------------------------------------------------------------------
