 /**
 *                                                                             *
 * @file XTT_Attribute.h                                                       *
 * Project name: HeKatE hqed                                                   *
 * $Id: XTT_Attribute.h,v 1.12 2008-03-03 22:04:33 kinio Exp $                  *
 * Author: Krzysztof Kaczor                                                    *
 * Date: 24.04.2007                                                            *
 *                                                                             *
 * Implementation by Krzysztof Kaczor <kinio4444@gmail.com>                    *
 *                                                                             *
 * Copyright (C) 2006-9 by the HeKatE Project                                  *
 *                                                                             *
 * HQEd has been develped within the HeKatE Project,                           *
 * see http://hekate.ia.agh.edu.pl                                             *
 *                                                                             *
 * This file is part of HQEd.                                                  *
 *                                                                             *
 * HQEd is free software: you can redistribute it and/or modify                *
 * it under the terms of the GNU General Public License as published by        *
 * the Free Software Foundation, either version 3 of the License, or           *
 * (at your option) any later version.                                         *
 *                                                                             *
 * HQEd is distributed in the hope that it will be useful,                     *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU General Public License for more details.                                *
 *                                                                             *
 * You should have received a copy of the GNU General Public License           *
 * along with HQEd.  If not, see <http://www.gnu.org/licenses/>.               *
 *                                                                             *
 * Comment: Plik zwiera definicje obiektu bedacego atrybutem, zmienna w        *
 * diagramch XTT.                                                              *
 *                                                                             *
 */

#ifndef XTT_ATTRIBUTEH
#define XTT_ATTRIBUTEH

// -----------------------------------------------------------------------------

#include <QtCore/QString>
#include <QStringList>
#include <QDomDocument>

#if QT_VERSION >= 0x040300
	#include <QXmlStreamWriter>
#endif
// -----------------------------------------------------------------------------

class XTT_Attribute
{
private:

     /// -------------------------- POLA OPISU ATRYBUTU -------------------------

     /// ID atrybutu, 0 - brak ID
     unsigned int fAttrID;

     /// Nazwa atrybutu
     QString fAttrName;

     /// Akronim atrybutu
     QString fAttrAcronym;
     
     /// Opis atrybutu
     QString fAttrDesc;

     /// Typ relacji atrybutu, 0-input, 1-middle, 2-output
     unsigned int fRelationsType;

     /// ------------------------- POLA WARTOSCI I TYPU -------------------------

     /// Wartosc atrybutu
     QString fAttrValue;

     /// Domyslna wartosc atrybutu
     QString fDefAttrValue;

     /// Typ atrybutu 0 - NotDefined, bool, integer, float, symbolic, enum
     unsigned int fAttrType;

     /// Wartosci w typie wyliczeniowym
     QStringList* fAttrValues;

     /// ----------------------- POLA DOTYCZACE OGRANICZEN ----------------------
     
     /// Okresla typ ograniczen atrybutu, 0 - bez ograniczen, 1 - range, 2 - set
     unsigned int fAttrConstrType;

     /// Okresla wartosc dolna i gorna przy ograniczeniach przedzialowych
     double* fIntervalConstraints;

     /// Okresla typ nawiasow przy ograniczeniach przedzialowych "()" - true, "[]" - false
     bool* fBraceType;

     /// Okresla czy nalezy ignorowac ograniczenie dolne/gorne przedzialowe
     bool* fIgnoreConstraint;

     /// Okresla zbior wartosci przy ograniczeniach wyliczeniowych
     QStringList* fEnumerateConstraints;
	
	/// ----------------------- POLA DOTYCZACE WLASCIWOSCI OBIEKTU ----------------------
	
	/// Okresla czy wlasciwosci obiektu zostaly zmienione od ostatniego zapisywania
	bool fChanged;

public:

	/// Zmianna statyczna definiujaca wyglad operatora not_defined
	static QString _not_definded_operator_;
	
	/// Zmianna statyczna definiujaca wyglad operatora any
	static QString _any_operator_;

     /// Konstruktor domyslny
     XTT_Attribute(void);

     /// Konstruktor przyjmujacy ID
     XTT_Attribute(unsigned int);

     /// Destruktor
     ~XTT_Attribute(void);
	
	/// Metoda sprawdzajaca czy da sie ustawic dana wartosc atrybutu
	bool checkValue(QString);

     /// ----------------------- Metody obslugujace pliki -----------------------

     /// Wstawia wszystkie dane w formacie xttml 1.0
     QStringList* XTTML_1_0(void);
	
	#if QT_VERSION >= 0x040300
	/// Zwraca format zapisu zgodny z XTTML 1.0
	void XTTML_1_0(QXmlStreamWriter*);
	#endif

     /// Wczytuje dane z pliku
     bool FromXTTML_1_0(QStringList*);
	
	/// Wczytuje dane z pliku
     bool FromXTTML_1_0(QDomElement&);
	
	/// -------------------------- Metody operacyjne ---------------------------
	
	/// Poprawia wartosc atrybutu tak aby przy zmianie ograniczen byla z 
	/// nimi zgdna
	void FixValue(void);

     /// ----------------- Metody zwracajaca wartosci pol klasy -----------------

     /// Zwraca identyfikator atrybutu
     unsigned int Id(void){ return fAttrID; }

     /// Zwraca nazwe atrybutu
     QString Name(void){ return fAttrName; }

     /// Zwraca akronim atrybutu
     QString Acronym(void){ return fAttrAcronym; }

     /// Zwraca opis atrybutu
     QString Description(void){ return fAttrDesc; }

     /// Zwraca wartosc atrybutu
     QString Value(void){ return fAttrValue.toLower(); }

     /// Zwraca domyslna wartosc atrybutu
     QString DefaultValue(void){ return fDefAttrValue; }

     /// Zwraca typ atrybutu
     unsigned int Type(void){ return fAttrType; }

     /// Zwraca typ relacji atrybutu
     unsigned int RelationsType(void){ return fRelationsType; }

     /// Zwraca wartosci atrybutu gdy ten jest typem wyliczeniowym, inaczej
     /// zwraca NULL
     QStringList* AttributeValues(void){ return fAttrValues; }

     /// Zwraca wartosci atrybutu gdy ten jest typem wyliczeniowym
     QStringList _AttributeValues(void){ return *fAttrValues; }

     /// Zwraca typ ograniczen
     unsigned int ConstraintsType(void) { return fAttrConstrType; }

     /// Zwraca gorne ograniczenie
     double UpConstraints(void){ return fIntervalConstraints[1]; }

     /// Zwraca dolne ograniczenie
     double BottomConstraints(void){ return fIntervalConstraints[0]; }

     /// Zwraca typ prawego nawiasu
     bool LoBraceType(void){ return fBraceType[0]; }

     /// Zwraca typ prawego nawiasu
     bool HiBraceType(void){ return fBraceType[1]; }

     /// Zwraca czy nalezy ignorowac dolne ograniczenie
     bool LoIgnoreConstraints(void){ return fIgnoreConstraint[0]; }

     /// Zwraca czy nalezy ignorowac gorne ograniczenie
     bool HiIgnoreConstraints(void){ return fIgnoreConstraint[1]; }

     /// Zwraca zbior wartosci ograniczen wyliczeniowych
     QStringList* EnumerateConstraints(void){ return fEnumerateConstraints; }

     /// Zwraca zbior wartosci ograniczen wyliczeniowych
     QStringList _EnumerateConstraints(void){ return *fEnumerateConstraints; }

     /// Zwraca liste wartosci w ograniczeniu wyliczeniowym jako string oddzielony srednikami
     QString EnumerateConstraintsString(void);

     /// Zwraca liste wartosci typu wyliczeniowego jako string
     QString AttrValuesString(void);

     /// Zwraca string bedacy reprezentacja ograniczen nalozonych na atrybut
     QString ConstraintsRepresentation(void);
	
	/// Zwraca czy obiekt zostal zmodyfikowany
	bool Changed(void){ return fChanged; }

     /// ---------------- Metody ustawiajace wartosci pol klasy -----------------

     /// Ustawia nazwe atrybutu
     void setName(QString);

     /// Ustawia nazwe atrybutu
     void setAcronym(QString);

     /// Ustawia opis strybutu
     void setDescription(QString);

     /// Ustawia wartosc atrybutu. Jezeli wartosc jest zgodna z typem i
     /// ograniczeniami to zwraca true, inaczej flase
     bool setValue(QString);

     /// Ustawia wartosc domyslna atrybutu. Jezeli wartosc jest zgodna z typem i
     /// ograniczeniami to zwraca true, inaczej flase
     bool setDefaultValue(QString);

     /// Ustawia typ relacji atrybutu
     bool setRelationsType(unsigned int);
	
	/// Ustawia typ atrybutu
     void setType(unsigned int);

     /// Ustawia wartosci atrybutu jakie moze przyjmowac w typie wyliczeniowym
     void setEnumValues(QStringList*);

     /// Ustawia wartosci atrybutu jakie moze przyjmowac w typie wyliczeniowym
     void setEnumValues(QStringList);

     /// Ustawia typ ograniczen atrybutu, 0 - brak ograniczen
     void setConstraintsType(unsigned int);

     /// Ustawia gorne ograniczenie przedzialowe
     void setUpConstraints(double);

     /// Ustawia dolne ograniczenie przedzialowe
     void setLoConstraints(double);

     /// Ustawia typ lewego nawiasu przy ograniczeniach przedzialowych
     void setLoBraceType(bool);

     /// Ustawia typ prawego nawiasu przy ograniczeniach przedzialowych
     void setHiBraceType(bool);

     /// Ustawia ignorowanie dolnego ograniczenia
     void setLoIgnoreConstraints(bool);

     /// Ustawia ignorowanie gornego ograniczenia
     void setHiIgnoreConstraints(bool);

     /// Ustawia wartosci ktore moze przyjac atrybut
     void setEnumerateConstraints(QStringList*);

     /// Ustawia wartosci ktore moze przyjac atrybut
     void setEnumerateConstraints(QStringList);
	
	/// Ustawia wartosc okreslajaca czy obiekt byl modyfikowany
	void setChanged(bool);
};

// -----------------------------------------------------------------------------
#endif
