 /**
 *                                                                             *
 * @file XTT_AttributeGroups.cpp                                               *
 * Project name: HeKatE hqed                                                   *
 * $Id: XTT_AttributeGroups.cpp,v 1.7 2008-03-03 22:04:33 kinio Exp $    *
 * Author: Krzysztof Kaczor                                                    *
 * Date: 26.04.2007                                                            *
 *                                                                             *
 * Implementation by Krzysztof Kaczor <kinio4444@gmail.com>                    *
 *                                                                             *
 * Copyright (C) 2006-9 by the HeKatE Project                                  *
 *                                                                             *
 * HQEd has been develped within the HeKatE Project,                           *
 * see http://hekate.ia.agh.edu.pl                                             *
 *                                                                             *
 * This file is part of HQEd.                                                  *
 *                                                                             *
 * HQEd is free software: you can redistribute it and/or modify                *
 * it under the terms of the GNU General Public License as published by        *
 * the Free Software Foundation, either version 3 of the License, or           *
 * (at your option) any later version.                                         *
 *                                                                             *
 * HQEd is distributed in the hope that it will be useful,                     *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU General Public License for more details.                                *
 *                                                                             *
 * You should have received a copy of the GNU General Public License           *
 * along with HQEd.  If not, see <http://www.gnu.org/licenses/>.               *
 *                                                                             *
 * Comment: Plik zawiera definicje metod obiektu przechowujacego               *
 * wszystkie grupy atrybutow                                                   *
 *                                                                             *
 */

#include <QMessageBox>
#include <QtGui>
#include <QDialog>
#include <QObject>
#include <QString>

#include <QStringList>

#include "../../C/MainWin_ui.h"
#include "XTT_Table.h"
#include "XTT_TableList.h"
#include "XTT_AttributeGroups.h"
// ---------------------------------------------------------------------------

XTT_AttributeGroups* AttributeGroups;
// ---------------------------------------------------------------------------

/**
 * Konstruktor domyslny klasy XTT_AttributeGroups
 * @returns Brak zwracanych wartosci
*/
XTT_AttributeGroups::XTT_AttributeGroups(void)
{
     fGroups = NULL;
     fCount = 0;
     fGId = 0;
     gAttrId = 0;
	fChanged = false;
     CreateGlobalVariablesGroup();
}
// ---------------------------------------------------------------------------

/**
 * Destruktor klasy XTT_AttributeGroups
 * @returns Brak zwracanych wartosci
*/
XTT_AttributeGroups::~XTT_AttributeGroups(void)
{
     if(fCount)
     {
          for(unsigned int i=0;i<fCount;i++)
               delete fGroups[i];
          delete [] fGroups;
     }
}
// ---------------------------------------------------------------------------

/**
 * Metoda zwracajaca wartosc okreslajaca czy dany obiekt oraz ktorykolkwiek z 
 * obiektow w pojemniku byl zmodyfikowany
 * @returns true jezeli nastapila modyfikacja, inaczej false
 */
bool XTT_AttributeGroups::Changed(void)
{
	for(unsigned int i=0;i<fCount;i++)
		if(fGroups[i]->Changed())
			return true;
			
	return fChanged;
}
// ---------------------------------------------------------------------------

/**
 * Metoda ustawiajaca wartosc zmienej okreslajacej modyfikacje wektora
 * @param bool _c - Nowa wartosc zmiennej
 * @returns Brak zwracanych wartoci
 */
void XTT_AttributeGroups::setChanged(bool _c)
{	
	for(unsigned int i=0;i<fCount;i++)
		fGroups[i]->setChanged(_c);
			
	fChanged = _c;
}
// ---------------------------------------------------------------------------

/**
 * Metoda klasy oprozniajaca pojemnik
 * @returns Brak zwracanych wartoci
*/
void XTT_AttributeGroups::Flush(void)
{
     if(fCount == 0)
          return;

     for(unsigned int i=0;i<fCount;i++)
          delete fGroups[i];
     delete [] fGroups;

     fGroups = NULL;
     fGId = 0;
     gAttrId = 0;
     fCount = 0;
	fChanged = true;

     // Te grupy musza byc zawsze
     CreateGlobalVariablesGroup();
}
//---------------------------------------------------------------------------

/**
 * Metoda klasy XTT_AttributeGroups tworzaca gupe do ktorej naleza atrybuty nie 
 * przydzielone do zadnej innej
 * @returns Brak zwracanych wartoci
*/
void XTT_AttributeGroups::CreateGlobalVariablesGroup(void)
{
     if(fCount > 0)
          return;

     Add();
     fGroups[0]->setName("Global");
     fGroups[0]->setDescription("This group contains a global variables, which not belong to groups.");

     // Grupa na atrybuty tymczasowe
     Add();
     fGroups[1]->setName("tmp");
     fGroups[1]->setDescription("This group contains temporary attributes.");
}
// ---------------------------------------------------------------------------

/**
 * Metoda klasy XTT_AttributeGroups tworzaca nowy tyczasowy argument dla 
 * przechowywania tymczasowych wynikow dzialan
 * @param unsigned int type - Typ jaki ma miec tymczasowy argument
 * @returns Index nowo utworzonego argumentu
*/
int XTT_AttributeGroups::CreateNewTmpArgument(unsigned int type)
{
     QString base_name = "tmp_";

     int aindex = 0;     // Index argumentu/nazwy

     // Wyszukiwanie indexu nawy
     for(;fGroups[1]->IndexOfName(base_name+QString::number(aindex))>-1;)
          aindex++;

     // Kiedy znajdziemy to tworzymy nowy atrybut o podanym typie z automatycznie nadawana nazwa
     fGroups[1]->Add();
     fGroups[1]->Item(fGroups[1]->Count()-1)->setType(type);
     fGroups[1]->Item(fGroups[1]->Count()-1)->setName(base_name+QString::number(aindex));
     fGroups[1]->Item(fGroups[1]->Count()-1)->setAcronym(base_name+QString::number(aindex));
	
	fChanged = true;

     // Zwracamy index nowoutworzonego atrybutu
     return fGroups[1]->Count()-1;
}
// ---------------------------------------------------------------------------

/**
 * Metoda klasy XTT_AttributeGroups wyszukuje grupe o podanej nazwie
 * @param QString _name - Nazwa wyszuiwanej grupy
 * @returns Wskaznik do grupy o podanej nazwie, jezeli taka nazwa nie istnieje 
 * to zwraca NULL
*/
XTT_AttributeList* XTT_AttributeGroups::Group(QString _name)
{
     for(unsigned int i=0;i<fCount;i++)
          if(fGroups[i]->Name() == _name)
               return fGroups[i];

     return NULL;
}
// ---------------------------------------------------------------------------

/**
 * Metoda klasy XTT_AttributeGroups, zwracajaca wskaznik do wybranej grupy
 * @param int _index - Index wybranej grupy
 * @returns Jezeli index jest poprawny zwraca wskaznik, jezeli niepoprawny to 
 * NULL
*/
XTT_AttributeList* XTT_AttributeGroups::Group(int _index)
{
     if((_index < 0) || (_index >= (int)fCount))
          return NULL;

     return fGroups[_index];
}
// ---------------------------------------------------------------------------

/**
 * Metodla klasy XTT_AttributeGroups wyszukujaca grupy na danym poziomie 
 * zagniezdzenia
 * @param QString _parent - Nazwa grupy rodzica lub string pusty gdy poziom 
 * najwyzszy
 * @returns Liste nazw grup w postaci listy stringow
*/
QStringList XTT_AttributeGroups::GroupList(QString _parent)
{
     // Lista zwracana
     QStringList res;
     res.clear();

     // Wyszukiwanie grup
     for(unsigned int i=0;i<fCount;i++)
     {
          if((_parent == "") && (fGroups[i]->Level() == 0))
               res << fGroups[i]->Name();
          if((fGroups[i]->Level() > 0) && (IndexOfID(fGroups[i]->ParentID()) > 0))
                    res << _IndexOfID(fGroups[i]->ParentID())->Name();
     }

     // Zwrocenie listy nazw
     return res;
}
// ---------------------------------------------------------------------------

/**
 * Metoda klasy XTT_AttributeGroups modyfikujaca ustawienia hierarchii
 * @param XTT_AttributeList* _group - Wskanik do grupy ktora chcemy przeniesc
 * @param XTT_AttributeList* _newParent - Wskanik do nowego rodzica
 * @returns Brak zwracanych wartoci
*/
void XTT_AttributeGroups::setGroupParent(XTT_AttributeList* _group, XTT_AttributeList* _newParent)
{
     if((_group == NULL) || (_newParent == NULL))
          return;

     _group->setParentId(_newParent->ID());       // Ustawienie wskaznika
     _group->setLevel(_newParent->Level()+1);     // Usawienie levelu
	
	fChanged = true;
}
// ---------------------------------------------------------------------------

/**
 * Metoda klasy XTT_AttributeGroups modyfikujaca ustawienia hierarchii
 * @param XTT_AttributeList* _group - Wskanik do grupy ktora chcemy przeniesc
 * @param unsigned int _newParentId - ID do nowego rodzica
 * @returns Brak zwracanych wartoci
*/
void XTT_AttributeGroups::setGroupParent(XTT_AttributeList* _group, unsigned int _newParentId)
{
     if((_group == NULL) || (_newParentId <= 0) || (IndexOfID(_newParentId) == -1))
          return;

     // Sprawdzanie czy rodzic i potomek to nie ta sama grupa
     int index = IndexOfID(_newParentId);
     if(_group->ID() == fGroups[index]->ID())
     {
          const QString content = "The group can't be parent for themselves.";
          QMessageBox::information(NULL, "XTT Editor", content, QMessageBox::Ok);
          return;
     }

     // Sprawdzanie czy podany nowy rodzic nie jest potomkiem aktualnej grupy
     while(fGroups[index]->Level() > 0)
     {
          // Sprawdzanie czy
          if(_group->ID() == fGroups[index]->ParentID())
          {
               const QString content = "You can't set child of this group as his parent.";
               QMessageBox::information(NULL, "XTT Editor", content, QMessageBox::Ok);
               return;
          }
          index = IndexOfID(fGroups[index]->ParentID());
     }

     _group->setParentId(_newParentId);                               // Ustawienie wskaznika
     RefreshLevelField(_group, _IndexOfID(_newParentId)->Level()+1);  // Usawienie levelu na najwyzszy i usaktalnienie pozostalych
	
	fChanged = true;
}
// ---------------------------------------------------------------------------

/**
 * Metoda klasy XTT_AttributeGroups, ustawiajaca wybrana grupe na szczycie 
 * drzewa
 * @param XTT_AttributeList* _group - Wskanik do wybranej grupy
 * @returns Brak zwracanych wartoci
*/
void XTT_AttributeGroups::setGroupAsRoot(XTT_AttributeList* _group)
{
     if(_group == NULL)
          return;

     _group->setParentId(0);                      // Ustawienie wskaznika na NULL - ze nie ma rodzicow
     RefreshLevelField(_group, 0);                // Usawienie levelu na najwyzszy i usaktalnienie pozostalych
	
	fChanged = true;
}
// ---------------------------------------------------------------------------

/**
 * Metoda klasy XTT_AttributeGroups odswierzajaca wartosci pola level po 
 * zmianie poziomu zagniezdzenia grupy
 * @param XTT_AttributeList* _start - Wskanik do grupy rodzica od ktorej 
 * nazlezy przenmerowac level
 * @returns Brak zwracanych wartoci
*/
void XTT_AttributeGroups::RefreshLevelField(XTT_AttributeList* _start, unsigned int _new_level)
{
     // Kiedy wskaznik jest jakis taki niepoprawny :)
     if(_start == NULL)
          return;

     // Odswierzanie watosci
     for(unsigned int i=0;i<fCount;i++)
          if(fGroups[i]->ParentID() == _start->ID())
               RefreshLevelField(fGroups[i], _new_level+1);

     // Odsiwiezenie bierzacej grupy
     _start->setLevel(_new_level);
	
	fChanged = true;
}
// ---------------------------------------------------------------------------

/**
 * Metoda klasy XTT_AttributeGroups dodajaca nowa grupe do pojemnika
 * @returns Brak zwracanych wartoci
*/
void XTT_AttributeGroups::Add(void)
{
     // Utworzenie nowej wiekszej tablicy wskaznikow
     XTT_AttributeList** tmp = new XTT_AttributeList*[fCount+1];

     // Kopiowanie zawartosci
     for(unsigned int i=0;i<fCount;i++)
          tmp[i] = fGroups[i];

     // Usuniecie starej tablicy wskaznikow
     if(fCount)
          delete [] fGroups;

     // Zapisanie lokalizacji
     fGroups = tmp;

     // Utworzenie nowego elementu
     fGroups[fCount] = new XTT_AttributeList(&gAttrId, ++fGId);

     // Powiekszenie rozmiaru
     fCount++;
	
	fChanged = true;
}
// ---------------------------------------------------------------------------

/**
 * Metodla klasy XTT_AttributeGroups usuwajaca grupe wraz z jej podgrupami
 * @param QString _name - Nazwa grupy do usuniecia
 * @returns Zwraca true, lub gdy podana nazwa jest niepoprawna to false
*/
bool XTT_AttributeGroups::Delete(QString _name)
{
     // Wyszukiwanie index
     int index = IndexOfName(_name);

     // Jezeli nie ma takiej grupy o podanej nazwie
     if(index == -1)
          return false;

     // Lista nazw grup ktore trzeba usunac
     QStringList names;
     names.clear();
     names << _name;

     // Wyznazanie nazw do usuniecia
     RemoveChildrens(fGroups[index]->ID(), names);

     // Usuwanie wszystkich grup z listy

     // Utworzenie nowej mniejszej tablicy wskaznikow
     XTT_AttributeList** tmp = new XTT_AttributeList*[fCount-names.size()];

     // Kopiowanie wskaznkow do nowej tablicy pomijajac wskazniki usuwane
     unsigned int tmp_i = 0;
     for(unsigned int i=0;i<fCount;i++)
          if(!names.contains(fGroups[i]->Name()))
               tmp[tmp_i++] = fGroups[i];

     // Kasowanie grup
     for(int i=0;i<names.size();i++)
     {
          int v = IndexOfName(names.at(i));
          if(v > -1)
               delete fGroups[v];
     }

     // Kasowanie starej tablicy wskaznikow
     if(fCount)
          delete [] fGroups;

     // Zapisanie nowej lokalizaji
     fGroups = tmp;

     // Pomniejszenie rozmiaru pojemnika
     fCount -= names.size();
	
	fChanged = true;

     // UFF
     return true;
}
// ---------------------------------------------------------------------------

/**
 * Metodla klasy XTT_AttributeGroups usuwajaca wszystkich potomkow danej grupy
 * @param XTT_AttributeList* _parent - Wskanik do grupy rodzica
 * @returns brak zwracanych wartosci
*/
void XTT_AttributeGroups::RemoveChildrens(unsigned int _parent, QStringList& names)
{
     // Wyszkiwanie grup majacych za potomka dana grupe
     for(unsigned int i=0;i<fCount;i++)
     {
          if(fGroups[i]->ParentID() == _parent)
          {
               names << fGroups[i]->Name();
               RemoveChildrens(fGroups[i]->ID(), names);
          }
     }
}
// ---------------------------------------------------------------------------

/**
 * Metoda klasy XTT_AttributeGroups wyszukujaca index grupy o podanej nazwie
 * @param QString _name - Nazwa grupy ktorej index chcemy wyszukac
 * @returns Index w pojemniku grupy o podanej nazwie o ile ona istnieje inaczej 
 * zwraca -1
*/
int XTT_AttributeGroups::IndexOfName(QString _name)
{
     for(unsigned int i=0;i<fCount;i++)
          if(fGroups[i]->Name() == _name)
               return (int)i;

     return -1;
}
// ---------------------------------------------------------------------------

/**
 * Metoda klasy XTT_AttributeGroups wyszukujaca wskaznik grupy o podanej nazwie
 * @param QString _name - Nazwa grupy ktorej wskaznik chcemy wyszukac
 * @returns Wskaznik do grupy o podanej nazwie o ile ona istnieje inaczej
 * zwraca NULL
*/
XTT_AttributeList* XTT_AttributeGroups::_IndexOfName(QString _name)
{
     int index = IndexOfName(_name);
     if(index > -1)
          return fGroups[index];

     return NULL;
}
// ---------------------------------------------------------------------------

/**
 * Metoda klasy XTT_AttributeGroups wyszukujaca index grupy o podanym ID
 * @param unsigned int _id - ID grupy ktorej index chcemy wyszukac
 * @returns Index w pojemniku grupy o podanej nazwie o ile ona istnieje inaczej
 * zwraca -1
*/
int XTT_AttributeGroups::IndexOfID(unsigned int _id)
{
     for(unsigned int i=0;i<fCount;i++)
          if(fGroups[i]->ID() == _id)
               return (int)i;

     return -1;
}
// ---------------------------------------------------------------------------

/**
 * Metoda klasy XTT_AttributeGroups wyszukujaca wskaznik grupy o podanym ID
 * @param unsigned int _id - ID grupy ktorej wskaznik chcemy wyszukac
 * @returns Wskaznik do grupy o podanej nazwie o ile ona istnieje inaczej
 * zwraca NULL
*/
XTT_AttributeList* XTT_AttributeGroups::_IndexOfID(unsigned int _id)
{
     int index = IndexOfID(_id);
     if(index > -1)
          return fGroups[index];

     return NULL;
}
// ---------------------------------------------------------------------------

/**
 * Metoda klasy XTT_AttributeGroups, sprawdzajaca czy jest zdefiniowny jaki 
 * kolwiek atrybut
 * @returns Zwraca false jezeli jest zdefiniowany jakikolwiek atrybut, inaczej 
 * true
*/
bool XTT_AttributeGroups::IsEmpty(void)
{
     for(unsigned int i=0;i<fCount;i++)
          if(fGroups[i]->Count())
               return false;

     return true;
}
// ---------------------------------------------------------------------------

/**
 * Meotda klasy QTreeWidgetItem, tworzaca drzewo nazw grup, ktore pokazuje 
 * zaleznosci pomiedzy grupami
 * @returns Drzewo nazw grup
*/
QList<QTreeWidgetItem*> XTT_AttributeGroups::CreateTree(void)
{
     QList<QTreeWidgetItem*> items;

     for(unsigned int i=0;i<fCount;i++)
     {
          // Niewyswietlamy grupy tymczasowej
          if(fGroups[i]->Name() == "tmp")
               continue;

          if(fGroups[i]->Level() == 0)
          {
               items.append(new QTreeWidgetItem((QTreeWidget*)0, QStringList(fGroups[i]->Name())));
               AddChilds(items.last(), fGroups[i]->ID());
          }

          //items.last()->addChild(new QTreeWidgetItem(QStringList(QString("child_item: %1"))));
          //items.last()->child(0)->addChild(new QTreeWidgetItem(QStringList(QString("child_child_item1: %1"))));
          //items.last()->child(0)->addChild(new QTreeWidgetItem(QStringList(QString("child_child_item2: %1"))));
          //items.last()->child(0)->addChild(new QTreeWidgetItem(QStringList(QString("child_child_item3: %1"))));
     }

     return items;
}
// ---------------------------------------------------------------------------

/**
 * Metoda klasy XTT_AttributeGroups tworzaca drzewo nazw grup atrybutow
 * @param QTreeWidgetItem* itm - Wskanik do drzewa
 * @param unsigned int _gId - ID grupy ktorej potomkow wyszukujemy
 * @returns Brak zwracanych wartoci
*/
void XTT_AttributeGroups::AddChilds(QTreeWidgetItem* itm, unsigned int _gId)
{
     for(unsigned int i=0;i<fCount;i++)
     {
          if(fGroups[i]->ParentID() == _gId)
          {
               itm->addChild(new QTreeWidgetItem(QStringList(fGroups[i]->Name())));
               AddChilds(itm->child(itm->childCount()-1), fGroups[i]->ID());
          }
     }
}
// ---------------------------------------------------------------------------

/**
 * Metoda klasy XTT_AttributeGroups tworzaca sciezke ostepu do danej grupy
 * @param unsigned int _id - ID grupy dla ktorej chcemy utworzyc sciezke dostepu
 * @returns Sciezke dostepu jako string
*/
QString XTT_AttributeGroups::_CreatePath(unsigned int _id)
{
     QString res = "";                                                          // Zwracany rezultat

     int curr_id = _id;                                                         // Bierzace id
     int index = IndexOfID(curr_id);                                            // Bierzacy ndex grupy

     // Jezeli grupa jest niepoprawna
     if(index == -1)
     {
          const QString content = "Application can't find requested group.";
          QMessageBox::critical(NULL, "XTT Editor", content, QMessageBox::Ok);
          return res;
     }

     res = fGroups[index]->Name();                                              // Uaktualnienie rezultatu

     // Generowanie scieki "w gore"
     while((index > -1) && (index < (int)fCount) && (fGroups[index]->Level() > 0))
     {
          for(unsigned int i=0;i<fCount;i++)
               if(fGroups[index]->ParentID() == fGroups[i]->ID())
               {
                    index = i;
                    break;
               }
          res = fGroups[index]->Name() + "." + res;
     }

     return res;
}
// ---------------------------------------------------------------------------

/**
 * Metoda klasy XTT_AttributeGroups tworzaca sciezke ostepu do danej grupy
 * @param QString _name - Nazwa grupy dla ktorej chcemy utworzyc sciezke dostepu
 * @returns Sciezke dostepu jako string
*/
QString XTT_AttributeGroups::CreatePath(QString _name)
{
     QString res = "";

     int index = IndexOfName(_name);
     if(index != -1)
          res = _CreatePath(fGroups[index]->ID());

     return res;
}
// ---------------------------------------------------------------------------

/**
 * Metoda klasy XTT_AttributeGroups tworzaca sciezke ostepu do danej grupy
 * @param int _index - Index grupy dla ktorej chcemy utworzyc sciezke dostepu
 * @returns Sciezke dostepu jako string
*/
QString XTT_AttributeGroups::CreatePath(int _index)
{
     QString res = "";

     if((_index > -1) && (_index < (int)fCount))
          res = _CreatePath(fGroups[_index]->ID());

     return res;
}
// ---------------------------------------------------------------------------

/**
 * Metoda kalsy sprawdzajaca czy sciezka i podany atrybut sa poprawne
 * @param QString _path - Scieka zakonczona atrybutem
 * @returns true jezeli sciezka jest poprawna inaczej false
*/
bool XTT_AttributeGroups::ExistsAttribute(QString _path)
{
     QStringList list = _path.split(".", QString::SkipEmptyParts);

     if(list.size() == 0)
          return false;

     // Usuwamy spacje na poczatku i koncu kazdego wiersza
     for(int i=0;i<list.size();i++)
     {
          QString line = list.at(i);
          for(;line[0] == ' ';)
               line = line.remove(0,1);
          for(;line[line.length()-1] == ' ';)
               line = line.remove(line.length()-1,1);
          list.replace(i, line);
     }

     // Gdy jest podany tylko atrybut, to sprawdzamy czy istnieje w Global
     if(list.size() == 1)
     {
          if((fGroups[0]->IndexOfName(list.at(0)) == -1) && (fGroups[0]->IndexOfAcronym(list.at(0)) == -1))
               return false;
          return true;
     }

     // Kiedy wieksza ilosc
     // Najpierw sprawdzamy czy podany atrybut istnieje w podanej grupie
     int index = IndexOfName(list.at(list.size()-2));
     if(index == -1)
          return false;
     if((fGroups[index]->IndexOfName(list.at(list.size()-1)) == -1) && (fGroups[index]->IndexOfAcronym(list.at(list.size()-1)) == -1))
          return false;
     
     // Sprawdzanie sciezki
     QString group_name;
     QString group_parent;
     int group_index;
     int group_parent_index;
     for(int i=list.size()-3;i>=0;i--)
     {
          group_name = list.at(i+1);
          group_parent = list.at(i);
          group_index = IndexOfName(group_name);
          group_parent_index = IndexOfName(group_parent);

          // Sprawdzanie istnienia grup
          if((group_index == -1) || (group_parent_index == -1))
               return false;

          // Sprawdzanie id rodzica
          if(fGroups[group_index]->ParentID() != fGroups[group_parent_index]->ID())
               return false;

     }

     // Sprawdzanie czy grupa na szczycie jest grupa glowna
     group_parent_index = IndexOfName(list.at(0));
     if(fGroups[group_parent_index]->Level() > 0)
          return false;
     return true;
}
// ---------------------------------------------------------------------------

/**
 * Metoda klasy XTT_AttributeGroups wyszukujaca atrybut o podanej sciezce 
 * dostepu i zwracajaca jego wskaznik
 * @param QString _path - Sciezka dostepu do atrybutu
 * @returns Wskazik do atrybutu podanego w sciezce dostepu
*/
XTT_Attribute* XTT_AttributeGroups::Attribute(QString _path)
{
     if(!ExistsAttribute(_path))
          return NULL;

     // Dalej nie musimy robic przechwytywania wyjatkow, poniewaz powyzsza funckja
     // to za nas zalatwia :)

     QStringList list = _path.split(".", QString::SkipEmptyParts);

     // Usuwamy spacje na poczatku i koncu kazdego wiersza
     for(int i=0;i<list.size();i++)
     {
          QString line = list.at(i);
          for(;line[0] == ' ';)
               line = line.remove(0,1);
          for(;line[line.length()-1] == ' ';)
               line = line.remove(line.length()-1,1);
          list.replace(i, line);
     }

     // Gdy jest podany tylko atrybut, to zwracamy jego index w Global
     if(list.size() == 1)
     {
          int index1 = fGroups[0]->IndexOfName(list.at(0));
          int index2 = fGroups[0]->IndexOfAcronym(list.at(0));
          int index = index1;
          if(index1 == -1)
               index = index2;
          return fGroups[0]->Item(index);
     }

     // Kiedy wieksza ilosc
     // Najpierw obliczamy index jego grupy
     int gindex = IndexOfName(list.at(list.size()-2));
     int index1 = fGroups[gindex]->IndexOfName(list.last());
     int index2 = fGroups[gindex]->IndexOfAcronym(list.last());
     int index = index1;
     if(index1 == -1)
          index = index2;

     // a teraz zwracamy jego wskaznik w danej grupie
     return fGroups[gindex]->Item(index);
}
// ---------------------------------------------------------------------------

/**
 * Metoda klasy XTT_AttributeGroups zwracajaca sciezke dostepu do atrybutu, bez
 * jego nazwy na podstawie jego adresu, oraz na koncu ostatniej grupy dodaje kropke
 * @param XTT_Attribute* _ptr - Wskanik do atrybutu
 * @returns String bedacy sciezka dostepu, lub string "#NAN" jezeli nie znajdzie
 * adresu
*/
QString XTT_AttributeGroups::CreatePath(XTT_Attribute* _ptr)
{
     QString res = "#NAN";

     for(unsigned int i=0;i<fCount;i++)
          for(unsigned int j=0;j<fGroups[i]->Count();j++)
               if(fGroups[i]->Item(j) == _ptr)
               {
                    res = CreatePath(i) + ".";
                    if(fGroups[i]->Name() == "Global")
                         res = "";
                    break;
               }

     return res;
}
// ---------------------------------------------------------------------------

/**
 * Metoda wyszukujaca atrybut o podanym ID
 * @param unsigned int _attrID - ID atrybutu
 * @returns Zwraca wskaznik do tego atrybutu
*/
XTT_Attribute* XTT_AttributeGroups::FindAttribute(unsigned int _attrID)
{
     XTT_Attribute* res = NULL;

     for(unsigned int i=0;i<fCount;i++)
          for(unsigned int j=0;j<fGroups[i]->Count();j++)
               if(fGroups[i]->Item(j)->Id() == _attrID)
               {
                    res = fGroups[i]->Item(j);
                    break;
               }

     return res;
}
// ---------------------------------------------------------------------------

/**
 * Metoda wyszukujaca grupe atrybutu
 * @param XTT_Attribute* _aptr - wskaznik do atrybutu
 * @returns Zwraca wskaznik do grupy w ktorej atrybut sie znajduje, jezeli nie znajdzie zwraca NULL
*/
XTT_AttributeList* XTT_AttributeGroups::AttributeGroup(XTT_Attribute* _aptr)
{
	XTT_AttributeList* res = NULL;

     for(unsigned int i=0;i<fCount;i++)
          for(unsigned int j=0;j<fGroups[i]->Count();j++)
               if(fGroups[i]->Item(j) == _aptr)
               {
                    res = fGroups[i];
                    break;
               }

     return res;
}
// ---------------------------------------------------------------------------

/**
 * Metoda wyszukujaca i zwracajaca liste atrybutow ktore nie zostaly jeszcze 
 * uzyte
 * @returns Liste atrybutow ktore nie zostaly jeszcze uzyte
*/
QStringList XTT_AttributeGroups::UnusedAttributesList(void)
{
     QStringList res;
     res.clear();

     for(unsigned int g=0;g<fCount;g++)
     {
          for(unsigned int a=0;a<fGroups[g]->Count();a++)
          {
               XTT_Attribute* _ptr = fGroups[g]->Item(a);
               
               // Wyszukujemy w tabelach czy istnieje powyzszy atrybut
               int index = -1;
               for(unsigned int t=0;t<TableList->Count();t++)
               {
                    index = TableList->Table(t)->IndexOfAttributeID(_ptr->Id());
                    if(index > -1)
                         t = TableList->Count();
               }

               // Gdy nie znaleziono tabeli w ktorej bylby uzyty dany atrybut, to
               // dodajemy go do listy nieuzywanych
               if(index == -1)
                    res << CreatePath(_ptr) + _ptr->Name();
          }
     }

     return res;
}
// ---------------------------------------------------------------------------

/**
 * Metoda zapisujaca wszystkie dane w formacie xttml 1.0
 * @returns Zwraca liste stringow w tym formacie
*/
QStringList* XTT_AttributeGroups::XTTML_1_0(void)
{
     QStringList* res = new QStringList;
     res->clear();

     *res << "<attr_group next_group_id=\"" + QString::number(fGId) + "\" next_attribute_id=\"" +  QString::number(gAttrId) + "\">";

     // Zapis atrybutow
     for(unsigned int a=0;a<fCount;a++)
     {
          // Jezeli grupa tmp, to nie zapisujemy jej
          if(a == 1)
               continue;
               
          QStringList* al = fGroups[a]->XTTML_1_0();
          for(int l=0;l<al->size();l++)
               *res << "\t" + al->at(l);
          delete al;
     }
     
     *res << "</attr_group>";

     return res;
}
// -----------------------------------------------------------------------------

#if QT_VERSION >= 0x040300
/**
 * Metoda zapisujaca wszystkie dane w formacie xttml 1.0
 * @param QXmlStreamWriter* _xmldoc - wskanik do dokumentu
 * @returns Brak zwracanych wartosci
*/
void XTT_AttributeGroups::XTTML_1_0(QXmlStreamWriter* _xmldoc)
{
	_xmldoc->writeStartElement("attr_group");
	_xmldoc->writeAttribute("next_group_id", QString::number(fGId));
	_xmldoc->writeAttribute("next_attribute_id", QString::number(gAttrId));

     // Zapis atrybutow
     for(unsigned int a=0;a<fCount;a++)
     {
          // Jezeli grupa tmp, to nie zapisujemy jej
          if(a == 1)
               continue;
               
          fGroups[a]->XTTML_1_0(_xmldoc);
     }
     
    _xmldoc->writeEndElement();
}
#endif
// -----------------------------------------------------------------------------

/**
 * Metoda zapisujaca wszystkie dane w formacie attml 1.0
 * @returns Zwraca liste stringow w tym formacie
*/
QStringList* XTT_AttributeGroups::ATTML_1_0(void)
{
     QStringList* res = new QStringList;
     res->clear();

	// Najpierw zapisujemy informacje o grupach
	*res << "<group_list>";
	for(unsigned int g=0;g<fCount;g++)
	{
		// Jezeli grupa global lub tmp, to nie zapisujemy jej
          if(g <= 1)
               continue;
			
		// Uzyskiwanie informacji o rodzicu
		int parentIndex = IndexOfID(fGroups[g]->ParentID());
		if((parentIndex == -1) && (fGroups[g]->ParentID() > 0))
		{
			QMessageBox::critical(NULL, "XTT_Editor", "Can't find parent group");
			return res;
		}
		QString parentName = (parentIndex==-1)?"":fGroups[parentIndex]->Name();
		
		*res << "\t<group name=\"" + fGroups[g]->Name() + "\" parent=\"" + parentName + "\">";
		if(fGroups[g]->Description() != "")
			*res << "\t\t<description>" + fGroups[g]->Description() + "</description>";
		*res << "\t</group>";
	}
	*res << "</group_list>";
	
	// Teraz zapisujemy informacje o atrybutach
	QStringList arelationstypes, atypes, acontraints;
	arelationstypes << "input" << "middle" << "output";
	atypes << "bool" << "integer" << "float" << "symbloic" << "enum";
	acontraints << "no" << "range" << "set";
	QStringList* enum_values, *aenumcontraints;
	
	*res << "<attribute_list>";
	for(unsigned int g=0;g<fCount;g++)
	{
		// Jezeli grupa tmp, to nie zapisujemy jej
          if(g == 1)
			continue;
			
		// Zapisywanie informacji o trybucie
		for(unsigned int a=0;a<fGroups[g]->Count();a++)
		{
			// Wczytywanie parametrow
			QString agroupname = (fGroups[g]->Name()=="Global")?"":fGroups[g]->Name();
			QString aname = fGroups[g]->Item(a)->Name();
			QString aacronym = fGroups[g]->Item(a)->Acronym();
			QString adesc = fGroups[g]->Item(a)->Description();
			QString arelationstype = arelationstypes.at(fGroups[g]->Item(a)->RelationsType());
			QString atype = atypes.at(fGroups[g]->Item(a)->Type()-1);
			QString acurrvalue = fGroups[g]->Item(a)->Value();
			QString adefvalue = fGroups[g]->Item(a)->DefaultValue();
			QString aconstraint = acontraints.at(fGroups[g]->Item(a)->ConstraintsType());
			QString alowconstraints = QString::number(fGroups[g]->Item(a)->BottomConstraints(), 'f');
			QString aupconstraints = QString::number(fGroups[g]->Item(a)->UpConstraints(), 'f');
			QString alowbrackettype = fGroups[g]->Item(a)->LoBraceType()?"round":"square";
			QString ahibrackettype = fGroups[g]->Item(a)->HiBraceType()?"round":"square";
			QString aignorelowconstr = fGroups[g]->Item(a)->LoIgnoreConstraints()?"true":"false";
			QString aignorehiconstr = fGroups[g]->Item(a)->HiIgnoreConstraints()?"true":"false";
			
			// Gdy jest typem wyliczeniowym to trzeba zapisac jego mozliwe wartosci
			enum_values = fGroups[g]->Item(a)->AttributeValues();
				
			// Zapisanie wartosci jezeli jest ograniczenie wyliczeniowe
			aenumcontraints = fGroups[g]->Item(a)->EnumerateConstraints();
			
			*res << "\t<xtt_attribute name=\"" + aname + "\" acronym=\"" + aacronym + "\" group=\"" + agroupname + "\" realtions=\"" + arelationstype + "\" type=\"" + atype + "\" defvalue=\"" + adefvalue + "\" currvalue=\"" + acurrvalue + "\" constraint=\"" + aconstraint + "\">";
			if(adesc != "")
				*res << "\t\t<description>" + adesc + "</description>";
			
			// Jezeli jest to typ wyliczeniowy
			if(fGroups[g]->Item(a)->Type() == 5)
			{
				*res << "\t\t<enum_values>";
				for(int i=0;i<enum_values->size();i++)
					*res << "\t\t\t<ev>" + enum_values->at(i) + "</ev>";
				*res << "\t\t</enum_values>";
			}
			
			// Gdy ograniczenie typu range
			if(fGroups[g]->Item(a)->ConstraintsType() == 1)
			{
				*res << "\t\t<low_constraint brackettype=\"" + alowbrackettype + "\" limit=\"" + alowconstraints + "\" ignore=\"" + aignorelowconstr + "\"/>";
				*res << "\t\t<hi_constraint brackettype=\"" + ahibrackettype + "\" limit=\"" + aupconstraints + "\" ignore=\"" + aignorehiconstr + "\"/>";
			}
			
			// Gdy ograniczenie typu set
			if(fGroups[g]->Item(a)->ConstraintsType() == 2)
			{
				*res << "\t\t<enum_constraint>";
				for(int i=0;i<aenumcontraints->size();i++)
					*res << "\t\t\t<ecv>" + aenumcontraints->at(i) + "</ecv>";
				*res << "\t\t</enum_constraint>";
			}
			
			*res << "\t</xtt_attribute>";
		}
	}
	*res << "</attribute_list>";

     return res;
}
// -----------------------------------------------------------------------------

#if QT_VERSION >= 0x040300
/**
 * Metoda zapisujaca wszystkie dane w formacie attml 1.0
 * @param QXmlStreamWriter* _xmldoc - wskanik do dokumentu
 * @returns Brak zwracanych wartosci
*/
void XTT_AttributeGroups::ATTML_1_0(QXmlStreamWriter* _xmldoc)
{
	// Najpierw zapisujemy informacje o grupach
	_xmldoc->writeStartElement("group_list");
	for(unsigned int g=0;g<fCount;g++)
	{
		// Jezeli grupa global lub tmp, to nie zapisujemy jej
          if(g <= 1)
               continue;
			
		// Uzyskiwanie informacji o rodzicu
		int parentIndex = IndexOfID(fGroups[g]->ParentID());
		if((parentIndex == -1) && (fGroups[g]->ParentID() > 0))
		{
			QMessageBox::critical(NULL, "XTT_Editor", "Can't find parent group");
			return;
		}
		QString parentName = (parentIndex==-1)?"":fGroups[parentIndex]->Name();
		
		_xmldoc->writeStartElement("group");
		_xmldoc->writeAttribute("name", fGroups[g]->Name());
		_xmldoc->writeAttribute("parent", parentName);
		if(fGroups[g]->Description() != "")
			_xmldoc->writeTextElement("description", fGroups[g]->Description());
		_xmldoc->writeEndElement();
	}
	_xmldoc->writeEndElement();
	
	// Teraz zapisujemy informacje o atrybutach
	QStringList arelationstypes, atypes, acontraints;
	arelationstypes << "input" << "middle" << "output";
	atypes << "bool" << "integer" << "float" << "symbloic" << "enum";
	acontraints << "no" << "range" << "set";
	QStringList* enum_values, *aenumcontraints;
	
	_xmldoc->writeStartElement("attribute_list");
	for(unsigned int g=0;g<fCount;g++)
	{
		// Jezeli grupa tmp, to nie zapisujemy jej
          if(g == 1)
			continue;
			
		// Zapisywanie informacji o trybucie
		for(unsigned int a=0;a<fGroups[g]->Count();a++)
		{
			// Wczytywanie parametrow
			QString agroupname = (fGroups[g]->Name()=="Global")?"":fGroups[g]->Name();
			QString aname = fGroups[g]->Item(a)->Name();
			QString aacronym = fGroups[g]->Item(a)->Acronym();
			QString adesc = fGroups[g]->Item(a)->Description();
			QString arelationstype = arelationstypes.at(fGroups[g]->Item(a)->RelationsType());
			QString atype = atypes.at(fGroups[g]->Item(a)->Type()-1);
			QString acurrvalue = fGroups[g]->Item(a)->Value();
			QString adefvalue = fGroups[g]->Item(a)->DefaultValue();
			QString aconstraint = acontraints.at(fGroups[g]->Item(a)->ConstraintsType());
			QString alowconstraints = QString::number(fGroups[g]->Item(a)->BottomConstraints(), 'f');
			QString aupconstraints = QString::number(fGroups[g]->Item(a)->UpConstraints(), 'f');
			QString alowbrackettype = fGroups[g]->Item(a)->LoBraceType()?"round":"square";
			QString ahibrackettype = fGroups[g]->Item(a)->HiBraceType()?"round":"square";
			QString aignorelowconstr = fGroups[g]->Item(a)->LoIgnoreConstraints()?"true":"false";
			QString aignorehiconstr = fGroups[g]->Item(a)->HiIgnoreConstraints()?"true":"false";
			
			// Gdy jest typem wyliczeniowym to trzeba zapisac jego mozliwe wartosci
			enum_values = fGroups[g]->Item(a)->AttributeValues();
				
			// Zapisanie wartosci jezeli jest ograniczenie wyliczeniowe
			aenumcontraints = fGroups[g]->Item(a)->EnumerateConstraints();
			
			_xmldoc->writeStartElement("xtt_attribute");
			_xmldoc->writeAttribute("name", aname);
			_xmldoc->writeAttribute("acronym", aacronym);
			_xmldoc->writeAttribute("group", agroupname);
			_xmldoc->writeAttribute("realtions", arelationstype);
			_xmldoc->writeAttribute("type", atype);
			_xmldoc->writeAttribute("defvalue", adefvalue);
			_xmldoc->writeAttribute("currvalue", acurrvalue);
			_xmldoc->writeAttribute("constraint", aconstraint);
			
			if(adesc != "")
				_xmldoc->writeTextElement("description", adesc);
			
			// Jezeli jest to typ wyliczeniowy
			if(fGroups[g]->Item(a)->Type() == 5)
			{
				_xmldoc->writeStartElement("enum_values");
				for(int i=0;i<enum_values->size();i++)
					_xmldoc->writeTextElement("ev", enum_values->at(i));
				_xmldoc->writeEndElement();
			}
			
			// Gdy ograniczenie typu range
			if(fGroups[g]->Item(a)->ConstraintsType() == 1)
			{
				_xmldoc->writeEmptyElement("low_constraint");
				_xmldoc->writeAttribute("brackettype", alowbrackettype);
				_xmldoc->writeAttribute("limit", alowconstraints);
				_xmldoc->writeAttribute("ignore", aignorelowconstr);
				
				_xmldoc->writeEmptyElement("hi_constraint");
				_xmldoc->writeAttribute("brackettype", ahibrackettype);
				_xmldoc->writeAttribute("limit", aupconstraints);
				_xmldoc->writeAttribute("ignore", aignorehiconstr);
			}
			
			// Gdy ograniczenie typu set
			if(fGroups[g]->Item(a)->ConstraintsType() == 2)
			{
				_xmldoc->writeStartElement("enum_constraint");
				for(int i=0;i<aenumcontraints->size();i++)
					_xmldoc->writeTextElement("ecv", aenumcontraints->at(i));
				_xmldoc->writeEndElement();
			}
			_xmldoc->writeEndElement();
		}
	}
	_xmldoc->writeEndElement();
}
#endif
// -----------------------------------------------------------------------------

/**
 * Metoda wczytujaca dane o strybucie z pliku w formacie XTTML
 * @param QStringList* list - Wskanik do sekcji dotyczacej tego atrybutu
 * @returns true jezeli dane sa ok, ianczej false
*/
bool XTT_AttributeGroups::FromXTTML_1_0(QStringList* list)
{
     bool res = true;
     
     // Usuwanie danych dotychczasowych
     if(fCount > 0)
     {
          for(unsigned int i=0;i<fCount;i++)
               delete fGroups[i];
          delete [] fGroups;

          fGroups = NULL;
          fGId = 0;
          gAttrId = 0;
          fCount = 0;
     }


     // Usuwanie spacji i tabulatorow wierszach
     for(int i=0;i<list->size();i++)
     {
          QString line = list->at(i);
          while(line.contains("\t"))
          {
               int index = line.indexOf("\t");
               line = line.remove(index, 1);
          }
          list->replace(i, line);
     }

     // Zakladamy ze pierwszy wiersz jest naglowkiem
     if(!list->at(0).contains("<attr_group", Qt::CaseInsensitive))
          return false;
     if(!list->contains("</attr_group>", Qt::CaseInsensitive))
          return false;

     // Reszta zakladamy ze jest ok

     // Spis tagow
     QStringList tags;
     tags.clear();
     tags << "<attr_group" << "<attr_list" << "</attr_list>";

     // Sluza do odpowiedniego kopiowania lini dla listy atrybutow
     bool copy = false;
     QStringList attribute_list;
     attribute_list.clear();

     for(int line=0;line<list->size();line++)
     {
          // Sprawdzanie wystapienia tagu
          int action = -1;
          for(int tag=0;tag<tags.size();tag++)
               if(list->at(line).contains(tags.at(tag), Qt::CaseInsensitive))
                    action = tag;

          // Kopiowanie lini danych atrybutu
          if(copy)
               attribute_list.append(list->at(line));

          if(action == -1)
               continue;

          QString end_tag = tags.at(action);
          end_tag = end_tag.insert(1, "/");
          QString l = list->at(line);
          l = l.remove(l.indexOf(tags.at(action), Qt::CaseInsensitive), tags.at(action).length());
          l = l.mid(0, l.indexOf(end_tag, Qt::CaseInsensitive));

          // Wczytywanie parametrow w zaleznosci od wykrytego tagu
          if(action == 0)
          {
               fGId = (unsigned int)XML_ReadValue(list->at(line), "next_group_id").toInt();
               gAttrId = (unsigned int)XML_ReadValue(list->at(line), "next_attribute_id").toInt();
          }
          if(action == 1)
          {
               // Jezeli tag rozpoczynajacy dane atrybutu, to uruchamiamy kopiowanie
               copy = true;
               attribute_list.append(list->at(line));
          }
          if(action == 2)
          {
               // Kiedy linia konczaca linie danych atrybutu
               // Dodajemy nowy atrybut
               Add();

               // Wczytujemy dane z pliku
               res = res && fGroups[fCount-1]->FromXTTML_1_0(&attribute_list);

               // Konczymy kopiowanie
               copy = false;

               // Czyscimy zawartosc danych atrybutu
               attribute_list.clear();

               // Jezeli wlasnie wczytalismy grupe pierwsza czyli global, to teraz musimy zalozyc grupe
               // tmp, bo nie jest ona wpisywana do pliku
               // Grupa na atrybuty tymczasowe
               if(fCount == 1)
               {
                    Add();
                    fGroups[1]->setName("tmp");
                    fGroups[1]->setDescription("This group contains temporary attributes.");
               }
          }
     }
	
	// Sortowanie atrybutow
	for(unsigned int gindex=0;gindex<fCount;gindex++)
		AttributeGroups->Group(gindex)->sort();

     return res;
}
// -----------------------------------------------------------------------------

/**
 * Metoda wczytujaca dane o strybucie z pliku w formacie XTTML
 * @param QDomElement& root - Referencja do sekcji dotyczacej tej grupy
 * @returns true jezeli dane sa ok, ianczej false
*/
bool XTT_AttributeGroups::FromXTTML_1_0(QDomElement& root)
{
	if(root.tagName().toLower() != "attr_group")
		return false;

     bool res = true;
     
     // Usuwanie danych dotychczasowych
     if(fCount > 0)
     {
          for(unsigned int i=0;i<fCount;i++)
               delete fGroups[i];
          delete [] fGroups;

          fGroups = NULL;
          fGId = 0;
          gAttrId = 0;
          fCount = 0;
     }
	
	// Wczytywanie informacji o pojemniku
	fGId = (unsigned int)root.attribute("next_group_id", "#NAN#").toInt();
	gAttrId = (unsigned int)root.attribute("next_attribute_id", "#NAN#").toInt();
		
	QDomElement DOMchild = root.firstChildElement("attr_list");
	while(!DOMchild.isNull())
     {
		// Kiedy linia konczaca linie danych atrybutu
		// Dodajemy nowy atrybut
		Add();

		// Wczytujemy dane z pliku
		res = res && fGroups[fCount-1]->FromXTTML_1_0(DOMchild);

		// Jezeli wlasnie wczytalismy grupe pierwsza czyli global, to teraz musimy zalozyc grupe
		// tmp, bo nie jest ona wpisywana do pliku
		// Grupa na atrybuty tymczasowe
		if(fCount == 1)
		{
			Add();
			fGroups[1]->setName("tmp");
			fGroups[1]->setDescription("This group contains temporary attributes.");
		}
		
		DOMchild = DOMchild.nextSiblingElement("attr_list");
     }
	
	// Sortowanie atrybutow
	for(unsigned int gindex=0;gindex<fCount;gindex++)
		AttributeGroups->Group(gindex)->sort();

     return res;
}
// -----------------------------------------------------------------------------

/**
 * Metoda wczytujaca dane o strybucie z pliku w formacie XTTML
 * @param QStringList* list - Wskanik do sekcji dotyczacej tego atrybutu
 * @returns true jezeli dane sa ok, ianczej false
*/
bool XTT_AttributeGroups::FromATTML_1_0(QStringList* list)
{
     bool res = true;
     
     // Usuwanie spacji i tabulatorow wierszach
     for(int i=0;i<list->size();i++)
     {
          QString line = list->at(i);
          while(line.contains("\t"))
          {
               int index = line.indexOf("\t");
               line = line.remove(index, 1);
          }
          list->replace(i, line);
     }

     // Zakladamy ze pierwszy wiersz jest naglowkiem
     if((!list->at(0).contains("<group_list>", Qt::CaseInsensitive)) && (!list->at(0).contains("<attribute_list>", Qt::CaseInsensitive)))
          return false;
     if((!list->contains("</group_list>", Qt::CaseInsensitive)) && (!list->contains("</attribute_list>", Qt::CaseInsensitive)))
          return false;
		
     // Reszta zakladamy ze jest ok
     // Gdy wcztujemy grupy
	if(list->at(0).contains("<group_list>", Qt::CaseInsensitive))
	{
		/*
		// Usuwanie danych dotychczasowych
	     if(fCount > 0)
	     {
	          for(unsigned int i=0;i<fCount;i++)
	               delete fGroups[i];
	          delete [] fGroups;

	          fGroups = NULL;
	          fGId = 0;
	          gAttrId = 0;
	          fCount = 0;
	     }
		
		// Utworzenie grupy atrybutow globalnych i tymczasowych
		CreateGlobalVariablesGroup();*/
	
	     QStringList tags;
	     tags.clear();
	     tags << "<group" << "<description>" << "</group>";
		
		XTT_AttributeList* group = NULL;
	     for(int line=0;line<list->size();line++)
	     {
			if(list->at(line) == "<group_list>")
				continue;
			
	          // Sprawdzanie wystapienia tagu
	          int action = -1;
	          for(int tag=0;tag<tags.size();tag++)
	               if(list->at(line).contains(tags.at(tag), Qt::CaseInsensitive))
	                    action = tag;

	          if(action == -1)
	               continue;

	          QString end_tag = tags.at(action);
	          end_tag = end_tag.insert(1, "/");
	          QString l = list->at(line);
	          l = l.remove(l.indexOf(tags.at(action), Qt::CaseInsensitive), tags.at(action).length());
	          l = l.mid(0, l.indexOf(end_tag, Qt::CaseInsensitive));

	          // Wczytywanie parametrow w zaleznosci od wykrytego tagu
	          if(action == 0)
	          {
	               QString gName = XML_ReadValue(list->at(line), "name");
	               QString gParent = XML_ReadValue(list->at(line), "parent");
				
				// Usuwanie spacji z nazwy
				for(;gName.contains(" ");)
					gName = gName.remove(gName.indexOf(" "), 1);
				if((gName == "#NAN#") || (gName == ""))
				{
					QMessageBox::critical(NULL, "XTT Editor", "Missing group name", QMessageBox::Ok);
					return false;
				}
				
				// Index grupy - sprawdzamy czy taka istnieje
				int group_index = IndexOfName(gName);
				
				// Dodawanie nowej grupy gdy ta nie istnieje
				if(group_index == -1)
				{
					Add();
					group_index = fCount-1;
				}
					
				group = fGroups[group_index];
				group->setName(gName);
				
				// Wyszukiwanie grupy rodzica
				// Usuwanie spacji z nazwy
				for(;gParent.contains(" ");)
					gParent = gParent.remove(gParent.indexOf(" "), 1);
				if((gParent != "#NAN#") && (gParent != ""))
				{
					int gindex = IndexOfName(gParent);
					if(gindex == -1)
					{
						QMessageBox::critical(NULL, "XTT Editor", "Not definded parent group " + gParent + ".\nPlease define this group first and try again.", QMessageBox::Ok);
						return false;
					}
					
					// Ustawianie rodzica grupy
					setGroupParent(group, fGroups[gindex]->ID());
				}
	          }
	          if(action == 1)
	          {
	               QString gDesc = l;
				if(group == NULL)
				{
					QMessageBox::critical(NULL, "XTT Editor", "Missplaced group description tag.", QMessageBox::Ok);
					return false;
				}
				group->setDescription(gDesc);
	          }
			if(action == 2)
	          {
	               group = NULL;
	          }

	     }
	}
	
	
	// Gdy wcztujemy atrybuty
	if(list->at(0).contains("<attribute_list>", Qt::CaseInsensitive))
	{
	     QStringList tags;
	     tags.clear();
	     tags << "<xtt_attribute" << "<description>" << "<enum_values>" << "<ev>" << "</enum_values>" << "<low_constraint" 
		     << "<hi_constraint" << "<enum_constraint>" << "<ecv>" << "</enum_constraint>" << "</xtt_attribute>";
		
		// Aktualnie wczytywany atrybut
		XTT_Attribute* attr = NULL;
		
		// Okresla czy w przypadku zdefiniowania atrybutu jako enum, zbior mozliwych wartosci zostal okreslony
		bool EnumerateValuesAreGiven = false;
		
		// Okresla czy w przypadku zdefiniowania ograniczen dziedziny atrybutu, dziedzina zostala okreslona
		bool ConstraintsValuesAreGiven = false;
		
		// Pojemniki na wartosci wyliczeniowe
		QStringList* enum_values, *aenumcontraints;
		
		// Tutaj bedziemy przechowywac wartosci atrybutu, bo musimy je ustawic na koncu wczytywania
		// inaczej moga powodowac bledy jezeli ograniczenia sa nie wczytane
		QString aCurrVal, aDefVal;
		
	     for(int line=0;line<list->size();line++)
	     {
			if(list->at(line) == "<group_list>")
				continue;
			
	          // Sprawdzanie wystapienia tagu
	          int action = -1;
	          for(int tag=0;tag<tags.size();tag++)
	               if(list->at(line).contains(tags.at(tag), Qt::CaseInsensitive))
	                    action = tag;

	          if(action == -1)
	               continue;

	          QString end_tag = tags.at(action);
	          end_tag = end_tag.insert(1, "/");
	          QString l = list->at(line);
	          l = l.remove(l.indexOf(tags.at(action), Qt::CaseInsensitive), tags.at(action).length());
	          l = l.mid(0, l.indexOf(end_tag, Qt::CaseInsensitive));

	          // Wczytywanie parametrow w zaleznosci od wykrytego tagu
	          if(action == 0)
	          {	
				// Tworzymy nowy atrybut
				attr = new XTT_Attribute(gAttrId++);
				
				// Teraz wczytujemy informacje o atrybutach
				QStringList arelationstypes, atypes, acontraints;
				arelationstypes << "input" << "middle" << "output";
				atypes << "bool" << "integer" << "float" << "symbloic" << "enum";
				acontraints << "no" << "range" << "set";
	
	               QString aName = XML_ReadValue(list->at(line), "name");
	               QString aAcronym = XML_ReadValue(list->at(line), "acronym");
				QString aGroup = XML_ReadValue(list->at(line), "group");
				QString aRelations = XML_ReadValue(list->at(line), "realtions");
				QString aType = XML_ReadValue(list->at(line), "type");
				QString aConstrType = XML_ReadValue(list->at(line), "constraint");
				aDefVal = XML_ReadValue(list->at(line), "defvalue");
				aCurrVal = XML_ReadValue(list->at(line), "currvalue");
				
				// Usuwanie spacji z wczytanych wartosci
				for(;aName.contains(" ");)
					aName = aName.remove(aName.indexOf(" "), 1);
				for(;aAcronym.contains(" ");)
					aAcronym = aAcronym.remove(aAcronym.indexOf(" "), 1);
				for(;aRelations.contains(" ");)
					aRelations = aRelations.remove(aRelations.indexOf(" "), 1);
				for(;aGroup.contains(" ");)
					aGroup = aGroup.remove(aGroup.indexOf(" "), 1);
				for(;aType.contains(" ");)
					aType = aType.remove(aType.indexOf(" "), 1);
				for(;aDefVal.contains(" ");)
					aDefVal = aDefVal.remove(aDefVal.indexOf(" "), 1);
				for(;aCurrVal.contains(" ");)
					aCurrVal = aCurrVal.remove(aCurrVal.indexOf(" "), 1);
				for(;aConstrType.contains(" ");)
					aConstrType = aConstrType.remove(aConstrType.indexOf(" "), 1);
					
				// Sprawdzanie podania nazwy
				if((aName == "#NAN#") || (aName == ""))
				{
					QMessageBox::critical(NULL, "XTT Editor", "Missing attribute name at line:\n" + list->at(line) + ".", QMessageBox::Ok);
					return false;
				}
				
				// Sprawdzanie podania akronimu
				if((aAcronym == "#NAN#") || (aAcronym == ""))
				{
					QMessageBox::critical(NULL, "XTT Editor", "Missing attribute acronym at line:\n" + list->at(line) + ".", QMessageBox::Ok);
					return false;
				}
				
				// Sprawdzanie podania proawidlowego typu relacji atrybutu
				if((aRelations == "#NAN#") || (aRelations == ""))
				{
					QMessageBox::critical(NULL, "XTT Editor", "Missing attribute relation at line:\n" + list->at(line) + ".", QMessageBox::Ok);
					return false;
				}
				if(!arelationstypes.contains(aRelations, Qt::CaseInsensitive))
				{
					QMessageBox::critical(NULL, "XTT Editor", "Incorrect relations type value: \"" + aRelations + "\" at line:\n" + list->at(line) + ".", QMessageBox::Ok);
					return false;
				}
				
				// Sprawdzanie podania proawidlowego typu atrybutu
				if((aType == "#NAN#") || (aType == ""))
				{
					QMessageBox::critical(NULL, "XTT Editor", "Missing attribute type at line:\n" + list->at(line) + ".", QMessageBox::Ok);
					return false;
				}
				if(!atypes.contains(aType, Qt::CaseInsensitive))
				{
					QMessageBox::critical(NULL, "XTT Editor", "Incorrect attribute type: \"" + aType + "\" at line:\n" + list->at(line) + ".", QMessageBox::Ok);
					return false;
				}
				
				// Jezeli nie podano typu ograniczen to uznajemy ze ich nie ma
				if((aConstrType == "#NAN#") || (aConstrType == ""))
				{
					aConstrType = "no";
				}
				
				// Sprawdzanie czy podano prawidlowe okreslenie ograniczen
				if((aConstrType != "#NAN#") && (aConstrType != "") && (!acontraints.contains(aConstrType, Qt::CaseInsensitive)))
				{
					QMessageBox::critical(NULL, "XTT Editor", "Incorrect attribute constraints defined: \"" + aConstrType + "\" at line:\n" + list->at(line) + ".", QMessageBox::Ok);
					return false;
				}
				
				// Sprawdzanie czy podano grupe, jezeli nie to ustawiamy na Global
				if((aGroup == "#NAN#") || (aGroup == ""))
				{
					aGroup = "Global";
				}
				
				// Sprawdzanie czy podano grupe ktora jest juz zdefiniowana
				if(IndexOfName(aGroup) == -1)
				{
					QMessageBox::critical(NULL, "XTT Editor", "Not defined group " + aGroup + " at line:\n" + list->at(line) + ".", QMessageBox::Ok);
					return false;
				}
				
				// Ustawienie grupy atrybutu
				int ion = fGroups[IndexOfName(aGroup)]->IndexOfName(aName);
				int ioa = fGroups[IndexOfName(aGroup)]->IndexOfAcronym(aAcronym);
				
				// Jezeli podany atrybut nie istnieje w danej grupie
				if((ion == -1) && (ioa == -1))
					fGroups[IndexOfName(aGroup)]->Add(attr);
					
				// Jezeli dany argument istnieje w tej grupie, to uaktualniamy jego parametry
				if((ion > -1) && (ioa > -1) && (ion == ioa))
				{
					delete attr;
					attr = fGroups[IndexOfName(aGroup)]->Item(ion);
				}
				
				// Jezeli istnieje argument o podanej nazwie lub akronimie i drugiego parametru nie ma takiego samego
				// to  wczytujemy nastepny atrybut
				if(((ion > -1) || (ioa > -1)) && (ion != ioa))
				{
					delete attr;
					attr = NULL;
					continue;
				}
				
				// Ustawianie parametrow atrybutu
				attr->setName(aName);
				attr->setAcronym(aAcronym);
				attr->setRelationsType((unsigned int)(arelationstypes.indexOf(aRelations.toLower())));
				attr->setType((unsigned int)(atypes.indexOf(aType.toLower())+1));
				attr->setConstraintsType((unsigned int)(acontraints.indexOf(aConstrType.toLower())));
				
				// Jezeli atrybut nie jest typu enum to ustawiamy flage podania wartosci wyliczenioeych na ture
				if(aType.toLower() != "enum")
					EnumerateValuesAreGiven = true;
				
				// Jezeli atrybut jest bez ograniczen to flage zdefiniowania dziedziny ustawiamy na true
				if(aConstrType.toLower() == "no")
					ConstraintsValuesAreGiven = true;
	          }
			
			// <description>
	          if(action == 1)
	          {
	               QString aDesc = l;
				if(attr == NULL)
				{
					QMessageBox::critical(NULL, "XTT Editor", "Missplaced attribute description tag.", QMessageBox::Ok);
					return false;
				}
				attr->setDescription(aDesc);
	          }
			
			// <enum_values>
			if(action == 2)
	          {
	               // Jezeli typ atrybutu zostal ustawiony na enumaerate
				if(attr == NULL)
					return false;	
				if(attr->Type() == 5)
					EnumerateValuesAreGiven = true;
				enum_values = new QStringList;
	          }
			
			// <ev> - gdy wczytujemy kolejne wartosci typu wyliczeniowego
			if(action == 3)
	          {
				if(attr == NULL)
					return false;	
				*enum_values << l;
			}
			
			// </enum_values>
			if(action == 4)
	          {
	               // Jezeli typ atrybutu zostal ustawiony na enumaerate
				if(attr == NULL)
					return false;	
				attr->setEnumValues(enum_values);
	          }
			
			// <low_constraint
			if(action == 5)
			{
				// Okreslenie mozliwych typow nawiasow na granicach przedzialu
				QStringList braceTypes;
				braceTypes << "square" << "round";
				
				if(attr == NULL)
					return false;	
				if(attr->ConstraintsType() == 1)
					ConstraintsValuesAreGiven = true;
					
				// Wczytywanie parametrow tagu
				QString bracetype = XML_ReadValue(list->at(line), "brackettype");
				QString limit = XML_ReadValue(list->at(line), "limit");
				QString ignore = XML_ReadValue(list->at(line), "ignore");
				
				// Usuwanie spacji z wczytanych wartosci
				for(;bracetype.contains(" ");)
					bracetype = bracetype.remove(bracetype.indexOf(" "), 1);
				for(;limit.contains(" ");)
					limit = limit.remove(limit.indexOf(" "), 1);
				for(;ignore.contains(" ");)
					ignore = ignore.remove(ignore.indexOf(" "), 1);
					
				// Sprawdzanie podania limitu
				if((limit == "#NAN#") || (limit == ""))
				{
					QMessageBox::critical(NULL, "XTT Editor", "Missing contraint attribute parameter \"limit\" at line:\n" + list->at(line) + ".", QMessageBox::Ok);
					return false;
				}
				
				// Sprawdzanie podania parametru ignore
				if((ignore == "#NAN#") || (ignore == ""))
				{
					QMessageBox::critical(NULL, "XTT Editor", "Missing contraint attribute parameter \"ignore\" at line:\n" + list->at(line) + ".", QMessageBox::Ok);
					return false;
				}
				
				// Sprawdzanie czy podana wartosc ignore jest poprawna
				if((ignore != "true") && (ignore != "false"))
				{
					QMessageBox::critical(NULL, "XTT Editor", "Incorrect contraint parameter \"ignore\" value at line:\n" + list->at(line) + ".", QMessageBox::Ok);
					return false;
				}
				
				// Sprawdzanie podania typu nawiasu
				if((bracetype == "#NAN#") || (bracetype == ""))
					bracetype = "round";
					
				// Jezeli podano typ nawiasu to sprawdzanie czy jest on wlasciwy
				if((bracetype != "#NAN#") && (bracetype != "") && (!braceTypes.contains(bracetype, Qt::CaseInsensitive)))
				{
					QMessageBox::critical(NULL, "XTT Editor", "Incorrect brace type: \"" + bracetype + "\" at line:\n" + list->at(line) + ".", QMessageBox::Ok);
					return false;
				}
				
				// Ustawianie wartosci parametrow
				attr->setLoBraceType((bool)braceTypes.indexOf(bracetype.toLower()));
				attr->setLoConstraints(limit.toDouble());
				attr->setLoIgnoreConstraints((ignore=="true")?true:false);
			}
			
			// <hi_constraint
			if(action == 6)
			{
				// Okreslenie mozliwych typow nawiasow na granicach przedzialu
				QStringList braceTypes;
				braceTypes << "square" << "round";
				
				if(attr == NULL)
					return false;	
				if(attr->ConstraintsType() == 1)
					ConstraintsValuesAreGiven = true;
					
				// Wczytywanie parametrow tagu
				QString bracetype = XML_ReadValue(list->at(line), "brackettype");
				QString limit = XML_ReadValue(list->at(line), "limit");
				QString ignore = XML_ReadValue(list->at(line), "ignore");
				
				// Usuwanie spacji z wczytanych wartosci
				for(;bracetype.contains(" ");)
					bracetype = bracetype.remove(bracetype.indexOf(" "), 1);
				for(;limit.contains(" ");)
					limit = limit.remove(limit.indexOf(" "), 1);
				for(;ignore.contains(" ");)
					ignore = ignore.remove(ignore.indexOf(" "), 1);
					
				// Sprawdzanie podania limitu
				if((limit == "#NAN#") || (limit == ""))
				{
					QMessageBox::critical(NULL, "XTT Editor", "Missing contraint attribute parameter \"limit\" at line:\n" + list->at(line) + ".", QMessageBox::Ok);
					return false;
				}
				
				// Sprawdzanie podania parametru ignore
				if((ignore == "#NAN#") || (ignore == ""))
				{
					QMessageBox::critical(NULL, "XTT Editor", "Missing contraint attribute parameter \"ignore\" at line:\n" + list->at(line) + ".", QMessageBox::Ok);
					return false;
				}
				
				// Sprawdzanie czy podana wartosc ignore jest poprawna
				if((ignore != "true") && (ignore != "false"))
				{
					QMessageBox::critical(NULL, "XTT Editor", "Incorrect contraint parameter \"ignore\" value at line:\n" + list->at(line) + ".", QMessageBox::Ok);
					return false;
				}
				
				// Sprawdzanie podania typu nawiasu
				if((bracetype == "#NAN#") || (bracetype == ""))
					bracetype = "round";
					
				// Jezeli podano typ nawiasu to sprawdzanie czy jest on wlasciwy
				if((bracetype != "#NAN#") && (bracetype != "") && (!braceTypes.contains(bracetype, Qt::CaseInsensitive)))
				{
					QMessageBox::critical(NULL, "XTT Editor", "Incorrect brace type: \"" + bracetype + "\" at line:\n" + list->at(line) + ".", QMessageBox::Ok);
					return false;
				}
				
				// Ustawianie wartosci parametrow
				attr->setHiBraceType((bool)braceTypes.indexOf(bracetype.toLower()));
				attr->setUpConstraints(limit.toDouble());
				attr->setHiIgnoreConstraints((ignore=="true")?true:false);
			}
			
			// <enum_constraint>
			if(action == 7)
	          {
	               // Jezeli typ ograniczen atrybutu zostal ustawiony na enumaerate
				if(attr == NULL)
					return false;	
				if(attr->ConstraintsType() == 2)
					ConstraintsValuesAreGiven = true;
				aenumcontraints = new QStringList;
	          }
			
			// <ecv> - gdy wczytujemy kolejne wartosci ograniczenia typu wyliczeniowego
			if(action == 8)
	          {
				if(attr == NULL)
					return false;	
				*aenumcontraints << l;
			}
			
			// <enum_constraint>
			if(action == 9)
	          {
	               // Jezeli typ atrybutu zostal ustawiony na enumaerate
				if(attr == NULL)
					return false;	
				attr->setEnumerateConstraints(aenumcontraints);
	          }
			
			if(action == 10)
	          {
				// Sprawdzanie czy podano dzeidzine atrybutu
				if(!ConstraintsValuesAreGiven)
				{
					QMessageBox::critical(NULL, "XTT Editor", "Not defined attribute \"" + attr->Name() + "\" domain.", QMessageBox::Ok);
					return false;
				}
				
				if((aDefVal != "#NAN#") && (aDefVal != ""))
					attr->setDefaultValue(aDefVal);
				if((aCurrVal != "#NAN#") && (aCurrVal != ""))
					attr->setValue(aCurrVal);
					
				attr = NULL;
	          }

	     }
		
		// Sortowanie atrybutow
		for(unsigned int gindex=0;gindex<fCount;gindex++)
			AttributeGroups->Group(gindex)->sort();
	}
	

     return res;
}
// -----------------------------------------------------------------------------

/**
 * Metoda wczytujaca dane o strybucie z pliku w formacie XTTML
 * @param QDomElement& root - referencja do obiektu przechwoujacego zawartosc XML
 * @returns true jezeli dane sa ok, ianczej false
*/
bool XTT_AttributeGroups::FromATTML_1_0(QDomElement& root)
{
     bool res = true;
		
     // Reszta zakladamy ze jest ok
     // Gdy wcztujemy grupy
	if(root.tagName().toLower() == "group_list")
	{
		/*
		// Usuwanie danych dotychczasowych
	     if(fCount > 0)
	     {
	          for(unsigned int i=0;i<fCount;i++)
	               delete fGroups[i];
	          delete [] fGroups;

	          fGroups = NULL;
	          fGId = 0;
	          gAttrId = 0;
	          fCount = 0;
	     }
		
		// Utworzenie grupy atrybutow globalnych i tymczasowych
		CreateGlobalVariablesGroup();*/
		
		XTT_AttributeList* group = NULL;
		QDomElement child = root.firstChildElement("group");
		while(!child.isNull()) 
		{
	          // Wczytywanie parametrow w zaleznosci od wykrytego tagu
			QString gName = child.attribute("name", "#NAN#");
			QString gParent = child.attribute("parent", "#NAN#");
			
			// Usuwanie spacji z nazwy
			for(;gName.contains(" ");)
				gName = gName.remove(gName.indexOf(" "), 1);
			if((gName == "#NAN#") || (gName == ""))
			{
				QMessageBox::critical(NULL, "XTT Editor", "Missing group name", QMessageBox::Ok);
				return false;
			}
			
			// Index grupy - sprawdzamy czy taka istnieje
			int group_index = IndexOfName(gName);
			
			// Dodawanie nowej grupy gdy ta nie istnieje
			if(group_index == -1)
			{
				Add();
				group_index = fCount-1;
			}
				
			group = fGroups[group_index];
			group->setName(gName);
			
			// Wyszukiwanie grupy rodzica
			// Usuwanie spacji z nazwy
			for(;gParent.contains(" ");)
				gParent = gParent.remove(gParent.indexOf(" "), 1);
			if((gParent != "#NAN#") && (gParent != ""))
			{
				int gindex = IndexOfName(gParent);
				if(gindex == -1)
				{
					QMessageBox::critical(NULL, "XTT Editor", "Not definded parent group " + gParent + ".\nPlease define this group first and try again.", QMessageBox::Ok);
					return false;
				}
				
				// Ustawianie rodzica grupy
				setGroupParent(group, fGroups[gindex]->ID());
			}

			// Wczytanie opisu grupy
			QString gDesc = child.firstChildElement("description").text();
			if(group == NULL)
			{
				QMessageBox::critical(NULL, "XTT Editor", "Missplaced group description tag.", QMessageBox::Ok);
				return false;
			}
			if(!gDesc.isEmpty())
				group->setDescription(gDesc);
			
			group = NULL;
			child = child.nextSiblingElement("group");
	     }
	}
	
	
	// Gdy wcztujemy atrybuty
	if(root.tagName().toLower() == "attribute_list")
	{
		// Aktualnie wczytywany atrybut
		XTT_Attribute* attr = NULL;
		
		// Okresla czy w przypadku zdefiniowania atrybutu jako enum, zbior mozliwych wartosci zostal okreslony
		bool EnumerateValuesAreGiven = false;
		
		// Okresla czy w przypadku zdefiniowania ograniczen dziedziny atrybutu, dziedzina zostala okreslona
		bool ConstraintsValuesAreGiven = false;
		
		// Pojemniki na wartosci wyliczeniowe
		QStringList* enum_values, *aenumcontraints;
		
		// Tutaj bedziemy przechowywac wartosci atrybutu, bo musimy je ustawic na koncu wczytywania
		// inaczej moga powodowac bledy jezeli ograniczenia sa nie wczytane
		QString aCurrVal, aDefVal;
		
		QDomElement child = root.firstChildElement("xtt_attribute");
	     while(!child.isNull()) 
	     {
	          // Wczytywanie parametrow w zaleznosci od wykrytego tagu
			// Tworzymy nowy atrybut
			attr = new XTT_Attribute(gAttrId++);
			
			// Teraz wczytujemy informacje o atrybutach
			QStringList arelationstypes, atypes, acontraints;
			arelationstypes << "input" << "middle" << "output";
			atypes << "bool" << "integer" << "float" << "symbloic" << "enum";
			acontraints << "no" << "range" << "set";

			QString aName = child.attribute("name", "#NAN#");
			QString aAcronym = child.attribute("acronym", "#NAN#");
			QString aGroup = child.attribute("group", "#NAN#");
			QString aRelations = child.attribute("realtions", "#NAN#");
			QString aType = child.attribute("type", "#NAN#");
			QString aConstrType = child.attribute("constraint", "#NAN#");
			aDefVal = child.attribute("defvalue", "#NAN#");
			aCurrVal = child.attribute("currvalue", "#NAN#");
			
			// Usuwanie spacji z wczytanych wartosci
			for(;aName.contains(" ");)
				aName = aName.remove(aName.indexOf(" "), 1);
			for(;aAcronym.contains(" ");)
				aAcronym = aAcronym.remove(aAcronym.indexOf(" "), 1);
			for(;aRelations.contains(" ");)
				aRelations = aRelations.remove(aRelations.indexOf(" "), 1);
			for(;aGroup.contains(" ");)
				aGroup = aGroup.remove(aGroup.indexOf(" "), 1);
			for(;aType.contains(" ");)
				aType = aType.remove(aType.indexOf(" "), 1);
			for(;aDefVal.contains(" ");)
				aDefVal = aDefVal.remove(aDefVal.indexOf(" "), 1);
			for(;aCurrVal.contains(" ");)
				aCurrVal = aCurrVal.remove(aCurrVal.indexOf(" "), 1);
			for(;aConstrType.contains(" ");)
				aConstrType = aConstrType.remove(aConstrType.indexOf(" "), 1);
				
			// Sprawdzanie podania nazwy
			if((aName == "#NAN#") || (aName == ""))
			{
				QMessageBox::critical(NULL, "XTT Editor", "Missing attribute name at line " + QString::number(child.lineNumber()) + ".", QMessageBox::Ok);
				return false;
			}
			
			// Sprawdzanie podania akronimu
			if((aAcronym == "#NAN#") || (aAcronym == ""))
			{
				QMessageBox::critical(NULL, "XTT Editor", "Missing attribute acronym at line " + QString::number(child.lineNumber()) + ".", QMessageBox::Ok);
				return false;
			}
			
			// Sprawdzanie podania proawidlowego typu relacji atrybutu
			if((aRelations == "#NAN#") || (aRelations == ""))
			{
				QMessageBox::critical(NULL, "XTT Editor", "Missing attribute relation at line " + QString::number(child.lineNumber()) + ".", QMessageBox::Ok);
				return false;
			}
			if(!arelationstypes.contains(aRelations, Qt::CaseInsensitive))
			{
				QMessageBox::critical(NULL, "XTT Editor", "Incorrect relations type value: \"" + aRelations + "\" at line " + QString::number(child.lineNumber()) + ".", QMessageBox::Ok);
				return false;
			}
			
			// Sprawdzanie podania proawidlowego typu atrybutu
			if((aType == "#NAN#") || (aType == ""))
			{
				QMessageBox::critical(NULL, "XTT Editor", "Missing attribute type at line " + QString::number(child.lineNumber()) + ".", QMessageBox::Ok);
				return false;
			}
			if(!atypes.contains(aType, Qt::CaseInsensitive))
			{
				QMessageBox::critical(NULL, "XTT Editor", "Incorrect attribute type: \"" + aType + "\" at line " + QString::number(child.lineNumber()) + ".", QMessageBox::Ok);
				return false;
			}
			
			// Jezeli nie podano typu ograniczen to uznajemy ze ich nie ma
			if((aConstrType == "#NAN#") || (aConstrType == ""))
			{
				aConstrType = "no";
			}
			
			// Sprawdzanie czy podano prawidlowe okreslenie ograniczen
			if((aConstrType != "#NAN#") && (aConstrType != "") && (!acontraints.contains(aConstrType, Qt::CaseInsensitive)))
			{
				QMessageBox::critical(NULL, "XTT Editor", "Incorrect attribute constraints defined: \"" + aConstrType + "\" at line " + QString::number(child.lineNumber()) + ".", QMessageBox::Ok);
				return false;
			}
			
			// Sprawdzanie czy podano grupe, jezeli nie to ustawiamy na Global
			if((aGroup == "#NAN#") || (aGroup == ""))
			{
				aGroup = "Global";
			}
			
			// Sprawdzanie czy podano grupe ktora jest juz zdefiniowana
			if(IndexOfName(aGroup) == -1)
			{
				QMessageBox::critical(NULL, "XTT Editor", "Not defined group " + aGroup + " at line " + QString::number(child.lineNumber()) + ".", QMessageBox::Ok);
				return false;
			}
			
			// Ustawienie grupy atrybutu
			int ion = fGroups[IndexOfName(aGroup)]->IndexOfName(aName);
			int ioa = fGroups[IndexOfName(aGroup)]->IndexOfAcronym(aAcronym);
			
			// Jezeli podany atrybut nie istnieje w danej grupie
			if((ion == -1) && (ioa == -1))
				fGroups[IndexOfName(aGroup)]->Add(attr);
				
			// Jezeli dany argument istnieje w tej grupie, to uaktualniamy jego parametry
			if((ion > -1) && (ioa > -1) && (ion == ioa))
			{
				delete attr;
				attr = fGroups[IndexOfName(aGroup)]->Item(ion);
			}
			
			// Jezeli istnieje argument o podanej nazwie lub akronimie i drugiego parametru nie ma takiego samego
			// to  wczytujemy nastepny atrybut
			if(((ion > -1) || (ioa > -1)) && (ion != ioa))
			{
				delete attr;
				attr = NULL;
				continue;
			}
			
			// Ustawianie parametrow atrybutu
			attr->setName(aName);
			attr->setAcronym(aAcronym);
			attr->setRelationsType((unsigned int)(arelationstypes.indexOf(aRelations.toLower())));
			attr->setType((unsigned int)(atypes.indexOf(aType.toLower())+1));
			attr->setConstraintsType((unsigned int)(acontraints.indexOf(aConstrType.toLower())));
			
			// Jezeli atrybut nie jest typu enum to ustawiamy flage podania wartosci wyliczenioeych na ture
			if(aType.toLower() != "enum")
				EnumerateValuesAreGiven = true;
			
			// Jezeli atrybut jest bez ograniczen to flage zdefiniowania dziedziny ustawiamy na true
			if(aConstrType.toLower() == "no")
				ConstraintsValuesAreGiven = true;
					
			
			// Wczytywanie opisu
			QDomElement description = child.firstChildElement("description");
	          if(!description.isNull())
	          {
	               QString aDesc = description.text();
				if(attr == NULL)
				{
					QMessageBox::critical(NULL, "XTT Editor", "Missplaced attribute description tag.", QMessageBox::Ok);
					return false;
				}
				attr->setDescription(aDesc);
	          }
			
			// <enum_values>
			QDomElement enmuvals = child.firstChildElement("enum_values");
	          if(!enmuvals.isNull())
	          {
	               // Jezeli typ atrybutu zostal ustawiony na enumaerate
				if(attr == NULL)
					return false;	
				if(attr->Type() == 5)
					EnumerateValuesAreGiven = true;
				enum_values = new QStringList;
				
				QDomElement enmuval = enmuvals.firstChildElement("ev");
				while(!enmuval.isNull())
				{
					*enum_values << enmuval.text();
					enmuval = enmuval.nextSiblingElement("ev");
				}
				
				if(attr == NULL)
					return false;	
				attr->setEnumValues(enum_values);
	          }
			
			// <low_constraint
			QDomElement lowconstr = child.firstChildElement("low_constraint");
	          if(!lowconstr.isNull())
	          {
				// Okreslenie mozliwych typow nawiasow na granicach przedzialu
				QStringList braceTypes;
				braceTypes << "square" << "round";
				
				if(attr == NULL)
					return false;	
				if(attr->ConstraintsType() == 1)
					ConstraintsValuesAreGiven = true;
					
				// Wczytywanie parametrow tagu
				QString bracetype = lowconstr.attribute("brackettype", "#NAN#");
				QString limit = lowconstr.attribute("limit", "#NAN#");
				QString ignore = lowconstr.attribute("ignore", "#NAN#");
				
				// Usuwanie spacji z wczytanych wartosci
				for(;bracetype.contains(" ");)
					bracetype = bracetype.remove(bracetype.indexOf(" "), 1);
				for(;limit.contains(" ");)
					limit = limit.remove(limit.indexOf(" "), 1);
				for(;ignore.contains(" ");)
					ignore = ignore.remove(ignore.indexOf(" "), 1);
					
				// Sprawdzanie podania limitu
				if((limit == "#NAN#") || (limit == ""))
				{
					QMessageBox::critical(NULL, "XTT Editor", "Missing contraint attribute parameter \"limit\" at line " + QString::number(lowconstr.lineNumber()) + ".", QMessageBox::Ok);
					return false;
				}
				
				// Sprawdzanie podania parametru ignore
				if((ignore == "#NAN#") || (ignore == ""))
				{
					QMessageBox::critical(NULL, "XTT Editor", "Missing contraint attribute parameter \"ignore\" at line " + QString::number(lowconstr.lineNumber()) + ".", QMessageBox::Ok);
					return false;
				}
				
				// Sprawdzanie czy podana wartosc ignore jest poprawna
				if((ignore != "true") && (ignore != "false"))
				{
					QMessageBox::critical(NULL, "XTT Editor", "Incorrect contraint parameter \"ignore\" value at line " + QString::number(lowconstr.lineNumber()) + ".", QMessageBox::Ok);
					return false;
				}
				
				// Sprawdzanie podania typu nawiasu
				if((bracetype == "#NAN#") || (bracetype == ""))
					bracetype = "round";
					
				// Jezeli podano typ nawiasu to sprawdzanie czy jest on wlasciwy
				if((bracetype != "#NAN#") && (bracetype != "") && (!braceTypes.contains(bracetype, Qt::CaseInsensitive)))
				{
					QMessageBox::critical(NULL, "XTT Editor", "Incorrect brace type: \"" + bracetype + "\" at line " + QString::number(lowconstr.lineNumber()) + ".", QMessageBox::Ok);
					return false;
				}
				
				// Ustawianie wartosci parametrow
				attr->setLoBraceType((bool)braceTypes.indexOf(bracetype.toLower()));
				attr->setLoConstraints(limit.toDouble());
				attr->setLoIgnoreConstraints((ignore=="true")?true:false);
			}
			
			// <hi_constraint
			QDomElement hiconstr = child.firstChildElement("hi_constraint");
	          if(!hiconstr.isNull())
			{
				// Okreslenie mozliwych typow nawiasow na granicach przedzialu
				QStringList braceTypes;
				braceTypes << "square" << "round";
				
				if(attr == NULL)
					return false;	
				if(attr->ConstraintsType() == 1)
					ConstraintsValuesAreGiven = true;
					
				// Wczytywanie parametrow tagu
				QString bracetype = hiconstr.attribute("brackettype", "#NAN#");
				QString limit = hiconstr.attribute("limit", "#NAN#");
				QString ignore = hiconstr.attribute("ignore", "#NAN#");
				
				// Usuwanie spacji z wczytanych wartosci
				for(;bracetype.contains(" ");)
					bracetype = bracetype.remove(bracetype.indexOf(" "), 1);
				for(;limit.contains(" ");)
					limit = limit.remove(limit.indexOf(" "), 1);
				for(;ignore.contains(" ");)
					ignore = ignore.remove(ignore.indexOf(" "), 1);
					
				// Sprawdzanie podania limitu
				if((limit == "#NAN#") || (limit == ""))
				{
					QMessageBox::critical(NULL, "XTT Editor", "Missing contraint attribute parameter \"limit\" at line " + QString::number(hiconstr.lineNumber()) + ".", QMessageBox::Ok);
					return false;
				}
				
				// Sprawdzanie podania parametru ignore
				if((ignore == "#NAN#") || (ignore == ""))
				{
					QMessageBox::critical(NULL, "XTT Editor", "Missing contraint attribute parameter \"ignore\" at line " + QString::number(hiconstr.lineNumber()) + ".", QMessageBox::Ok);
					return false;
				}
				
				// Sprawdzanie czy podana wartosc ignore jest poprawna
				if((ignore != "true") && (ignore != "false"))
				{
					QMessageBox::critical(NULL, "XTT Editor", "Incorrect contraint parameter \"ignore\" value at line " + QString::number(hiconstr.lineNumber()) + ".", QMessageBox::Ok);
					return false;
				}
				
				// Sprawdzanie podania typu nawiasu
				if((bracetype == "#NAN#") || (bracetype == ""))
					bracetype = "round";
					
				// Jezeli podano typ nawiasu to sprawdzanie czy jest on wlasciwy
				if((bracetype != "#NAN#") && (bracetype != "") && (!braceTypes.contains(bracetype, Qt::CaseInsensitive)))
				{
					QMessageBox::critical(NULL, "XTT Editor", "Incorrect brace type: \"" + bracetype + "\" at line " + QString::number(hiconstr.lineNumber()) + ".", QMessageBox::Ok);
					return false;
				}
				
				// Ustawianie wartosci parametrow
				attr->setHiBraceType((bool)braceTypes.indexOf(bracetype.toLower()));
				attr->setUpConstraints(limit.toDouble());
				attr->setHiIgnoreConstraints((ignore=="true")?true:false);
			}
			
			// <enum_constraint>
			QDomElement enumconstr = child.firstChildElement("enum_constraint");
	          if(!enumconstr.isNull())
	          {
	               // Jezeli typ atrybutu zostal ustawiony na enumaerate
				if(attr == NULL)
					return false;	
				if(attr->ConstraintsType() == 2)
					ConstraintsValuesAreGiven = true;
				aenumcontraints = new QStringList;
				
				QDomElement enmuval = enumconstr.firstChildElement("ecv");
				while(!enmuval.isNull())
				{
					*aenumcontraints << enmuval.text();
					enmuval = enmuval.nextSiblingElement("ecv");
				}
				
				if(attr == NULL)
					return false;	
				attr->setEnumerateConstraints(aenumcontraints);
	          }
			

			// Sprawdzanie czy podano dzeidzine atrybutu, weryfikacja wczytanych danych
			if(!ConstraintsValuesAreGiven)
			{
				QMessageBox::critical(NULL, "XTT Editor", "Not defined attribute \"" + attr->Name() + "\" domain.", QMessageBox::Ok);
				return false;
			}
			
			if((aDefVal != "#NAN#") && (aDefVal != ""))
				attr->setDefaultValue(aDefVal);
			if((aCurrVal != "#NAN#") && (aCurrVal != ""))
				attr->setValue(aCurrVal);
			
			attr = NULL;
			child = child.nextSiblingElement("xtt_attribute");

	     }
		
		// Sortowanie atrybutow
		for(unsigned int gindex=0;gindex<fCount;gindex++)
			AttributeGroups->Group(gindex)->sort();
	}
	

     return res;
}
// -----------------------------------------------------------------------------
