 /**
 *                                                                             *
 * @file XTT_AttributeGroups.h                                                 *
 * Project name: HeKatE hqed                                                   *
 * $Id: XTT_AttributeGroups.h,v 1.9 2008-03-03 22:04:33 kinio Exp $           *
 * Author: Krzysztof Kaczor                                                    *
 * Date: 26.04.2007                                                            *
 *                                                                             *
 * Implementation by Krzysztof Kaczor <kinio4444@gmail.com>                    *
 *                                                                             *
 * Copyright (C) 2006-9 by the HeKatE Project                                  *
 *                                                                             *
 * HQEd has been develped within the HeKatE Project,                           *
 * see http://hekate.ia.agh.edu.pl                                             *
 *                                                                             *
 * This file is part of HQEd.                                                  *
 *                                                                             *
 * HQEd is free software: you can redistribute it and/or modify                *
 * it under the terms of the GNU General Public License as published by        *
 * the Free Software Foundation, either version 3 of the License, or           *
 * (at your option) any later version.                                         *
 *                                                                             *
 * HQEd is distributed in the hope that it will be useful,                     *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU General Public License for more details.                                *
 *                                                                             *
 * You should have received a copy of the GNU General Public License           *
 * along with HQEd.  If not, see <http://www.gnu.org/licenses/>.               *
 *                                                                             *
 * Comment: Plik zawiera deklaracje klasy bedacej obiektem przechwujavym       *
 * grupy atrybutow                                                             *
 *                                                                             *
 */

#ifndef XTT_AttributeGroupsH
#define XTT_AttributeGroupsH
//---------------------------------------------------------------------------

#include <QTreeWidgetItem>
#include <QStringList>
#include <QDomDocument>

#if QT_VERSION >= 0x040300
	#include <QXmlStreamWriter>
#endif

#include "XTT_AttributeList.h"
//---------------------------------------------------------------------------

class XTT_AttributeGroups
{
private:

     /// Tablica grup atrubutow
     XTT_AttributeList** fGroups;

     /// Wielkosc tablicy
     unsigned int fCount;

     /// Nastepny identyfikator dla grupy
     unsigned int fGId;

     /// Nastepny identyfikator dla atrybutu
     unsigned int gAttrId;
	
	/// Okresla czy nastpila zmiana obiektu
	bool fChanged;

     /// Metoda tworzaca grupe zmiennych globalnych
     void CreateGlobalVariablesGroup(void);

     /// Prywata metoda rekurencyjna usuwajaca wszystkich potomkow danej grupy
     void RemoveChildrens(unsigned int, QStringList&);

     /// Prywatna metoda rekurencyjna tworzaca podrzewa galezi glownych drzewa nazw grup
     void AddChilds(QTreeWidgetItem*, unsigned int);

public:

     /// Konstruktor defaultowy
     XTT_AttributeGroups(void);

     /// Destruktor
     ~XTT_AttributeGroups(void);

     /// ----------------------- Metody obslugujace pliki -----------------------
	
	/// Zwraca format zapisu zgodny z ATTML 1.0
     QStringList* ATTML_1_0(void);
	
	#if QT_VERSION >= 0x040300
	/// Zwraca format zapisu zgodny z ATTML 1.0
	void ATTML_1_0(QXmlStreamWriter*);
	#endif
	
	/// Wstawia wszystkie dane w formacie xttml 1.0
     QStringList* XTTML_1_0(void);
	
	#if QT_VERSION >= 0x040300
	/// Zwraca format zapisu zgodny z ATTML 1.0
	void XTTML_1_0(QXmlStreamWriter*);
	#endif

     /// Wczytuje dane z pliku w formacie XTTML 1.0
     bool FromXTTML_1_0(QStringList*);
	
	/// Wczytuje dane z pliku w formacie XTTML 1.0
	bool FromXTTML_1_0(QDomElement&);
	
	/// Wczytuje dane z pliku w formacie ATTML 1.0
	bool FromATTML_1_0(QStringList*);
	
	/// Wczytuje dane z pliku w formacie ATTML 1.0
	bool FromATTML_1_0(QDomElement&);

     /// ----------------- Metody zwracajaca wartosci pol klasy -----------------

     /// Zwraca wielkosc pojemnika
     unsigned int Count(void){ return fCount; }

     /// Zwraca nastepne id dla grupy
     unsigned int GId(void){ return fGId; }

     /// Zwraca nastepne id dla atrybutu
     unsigned int AttrId(void){ return gAttrId; }

     /// Zwraca wskaznik wybranej grupy, jezeli podany index jest niepoprawny to
     /// zwraca NULL
     XTT_AttributeList* Group(int);

     /// Zwraca wskaznik wybranej grupy, jezeli podana nazwa jest niepoprawna to
     /// zwraca NULL
     XTT_AttributeList* Group(QString);

     /// Zwraca liste nazw grup
     QStringList GroupList(QString);
	
	/// Zwraca wartosc okreslajaca modyfikacje obiektu
	bool Changed(void);

     /// -------------------------- Metody operacyjne ---------------------------

     /// Dodaje nowa grupe
     void Add(void);

     /// Usuwa wybrana grupe, jezeli podany index jest niepoprawny to zwraca false
     /// inaczej true. Funckja taze usuwa wszystkie podgrupy danej grupy!!!
     bool Delete(QString);

     /// Ustawia rodzica dla danej grupy, dbajac o wszystkie z tym zwiazane ustawienia
     void setGroupParent(XTT_AttributeList*, XTT_AttributeList*);

     /// Ustawia rodzica dla danej grupy, dbajac o wszystkie z tym zwiazane ustawienia
     void setGroupParent(XTT_AttributeList*, unsigned int);

     /// Ustawia wybrana grupe jako grupe bez rodzicow, na samym szczycie drzewa
     void setGroupAsRoot(XTT_AttributeList*);

     /// Wyszukuje grupe o podanej nazwie zwraca jej index, lub -1 gdy nie znaleziono
     int IndexOfName(QString);

     /// Wyszukuje grupe o podanym ID zwraca jej index, lub -1 gdy nie znaleziono
     int IndexOfID(unsigned int);

     /// Wyszukuje grupe o podanej nazwie zwraca jej wskaznik, lub NULL gdy nie znaleziono
     XTT_AttributeList* _IndexOfName(QString);

     /// Wyszukuje grupe o podanym ID zwraca jej wskaznik, lub NULL gdy nie znaleziono
     XTT_AttributeList* _IndexOfID(unsigned int);

     /// Zwraca prawde jezeli suma atrybutow z wszystkich grup jest dodatnia
     bool IsEmpty(void);

     /// Tworzy i zwraca drzew nazw grup
     QList<QTreeWidgetItem*> CreateTree(void);

     /// Tworzy sciezke dostepu dla danej grupy
     QString CreatePath(int);

     /// Tworzy sciezke dostepu dla danej grupy
     QString CreatePath(QString);

     /// Tworzy sciezke dostepu do atrybutu o podnaym adresie
     QString CreatePath(XTT_Attribute*);

     /// Tworzy sciezke dostepu dla danej grupy
     QString _CreatePath(unsigned int);

     /// Oswieza wartosci poziomu zganiezdzenia
     void RefreshLevelField(XTT_AttributeList*, unsigned int);

     /// Sprawdza czy istnieje dany atrybut, razem ze sciezka
     bool ExistsAttribute(QString);
	
	/// Zwraca wskaznik do grupy z ktorej pochodzi atrybut
	XTT_AttributeList* AttributeGroup(XTT_Attribute*);

     /// Zwraca wskaznik do atrybutu o podanej sciezce dostepu
     XTT_Attribute* Attribute(QString);

     /// Wyszukuje atrybut o podanym ID
     XTT_Attribute* FindAttribute(unsigned int);

     /// Tworzy nowy tymczasowy argument
     int CreateNewTmpArgument(unsigned int);

     /// Zwraca liste atrybutow, ktore nie sa uzyte
     QStringList UnusedAttributesList(void);

     /// Oproznia vektor
     void Flush(void);
	
	/// Ustawia wartosc okreslajaca czy obiekty byly modyfikowane
	void setChanged(bool);
};
//---------------------------------------------------------------------------

/// Deklaracja wskaznika do grupy atrybutow
extern XTT_AttributeGroups* AttributeGroups;
//---------------------------------------------------------------------------
#endif
