 /**
 *                                                                             *
 * @file XTT_AttributeList.cpp                                                 *
 * Project name: HeKatE hqed                                                   *
 * $Id: XTT_AttributeList.cpp,v 1.10 2008-03-03 22:04:33 kinio Exp $           *
 * Author: Krzysztof Kaczor                                                    *
 * Date: 25.04.2007                                                            *
 *                                                                             *
 * Implementation by Krzysztof Kaczor <kinio4444@gmail.com>                    *
 *                                                                             *
 * Copyright (C) 2006-9 by the HeKatE Project                                  *
 *                                                                             *
 * HQEd has been develped within the HeKatE Project,                           *
 * see http://hekate.ia.agh.edu.pl                                             *
 *                                                                             *
 * This file is part of HQEd.                                                  *
 *                                                                             *
 * HQEd is free software: you can redistribute it and/or modify                *
 * it under the terms of the GNU General Public License as published by        *
 * the Free Software Foundation, either version 3 of the License, or           *
 * (at your option) any later version.                                         *
 *                                                                             *
 * HQEd is distributed in the hope that it will be useful,                     *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU General Public License for more details.                                *
 *                                                                             *
 * You should have received a copy of the GNU General Public License           *
 * along with HQEd.  If not, see <http://www.gnu.org/licenses/>.               *
 *                                                                             *
 * Comment: Plik zwiera definicje metod obiektu bdcego pojemnikiem na        *
 * atrybuty                                                                    *
 *                                                                             *
 */

#include <QMessageBox>
#include <QtGui>
#include <QDialog>
#include <QObject>

#include "../../C/MainWin_ui.h"
#include "XTT_Table.h"
#include "XTT_TableList.h"
#include "XTT_Attribute.h"
#include "XTT_AttributeList.h"
//---------------------------------------------------------------------------

// Deklaracja glownego zbiornika atrybutow
XTT_AttributeList* AttributeList;
//---------------------------------------------------------------------------

/**
 * Konstruktor defaultowy klasy XTT_AttributeList
 * @returns Brak zwracanych wartosci
*/
XTT_AttributeList::XTT_AttributeList(void)
{
     fId = 0;
     fList = NULL;
     fCount = 0;
     fName = "";
     fDescription = "";
     fNextId = new unsigned int;
     *fNextId = 0;
     fParentId = 0;
     fLevel = 0;
	fChanged = false;
}
//---------------------------------------------------------------------------

/**
 * Konstruktor klasy XTT_AttributeList
 * @param unsigned int _id - ID pojemnika
 * @returns Brak zwracanych wartosci
*/
XTT_AttributeList::XTT_AttributeList(unsigned int _id)
{
     fId = _id;
     fList = NULL;
     fCount = 0;
     fName = "";
     fDescription = "";
     fNextId = new unsigned int;
     *fNextId = 0;
     fParentId = 0;
     fLevel = 0;
	fChanged = false;
}
//---------------------------------------------------------------------------

/**
 * Konstruktor klasy XTT_AttributeList
 * @param unsigned int* _p_attr_id - Wskaznik do wartosci klejnego identyfikatora atrybutu
 * @param unsigned int _id - ID pojemnika
 * @param unsigned int _type - Typ pojemnika
 * @returns Brak zwracanych wartosci
*/
XTT_AttributeList::XTT_AttributeList(unsigned int* _p_attr_id, unsigned int _id)
{
     fId = _id;
     fList = NULL;
     fCount = 0;
     fName = "";
     fDescription = "";
     fNextId = _p_attr_id;
     fParentId = 0;
     fLevel = 0;
	fChanged = false;
}
//---------------------------------------------------------------------------

/**
 * Destruktor klasy XTT_AttributeList. Kasuje wszystkie dane alokowane 
 * dynamicznie
 * @returns Brak zwracanych wartosci
*/
XTT_AttributeList::~XTT_AttributeList(void)
{
     if(fCount > 0)
     {
		for(unsigned int i=0;i<fCount;i++)
			delete fList[i];
          delete [] fList;
     }
}
//---------------------------------------------------------------------------

/**
 * Metodla klasy XTT_AttributeList. Zwraca wskaznik do argumentu
 * @param int index - Index argumentu ktorego wskaznik chcemy otrzymac
 * @returns Jezeli index jest poprawny to zwraca wskaznik, jezeli nie to NULL
*/
XTT_Attribute* XTT_AttributeList::Item(int index)
{
     if((index < 0) || (index >= (int)fCount))
          return NULL;

     return fList[index];
}
//---------------------------------------------------------------------------

/**
 * Metoda ustawiajaca nazwe grupy
 * @param QString _nn - Nowa nazwa grupy
 * @returns Brak zwracanych wartoci
 */
void XTT_AttributeList::setName(QString _nn)
{ 
	// Usuwanie spacji
	while(_nn.indexOf(" ") > 0)
	{
		_nn = _nn.remove(_nn.indexOf(" "), 1);
	}

	fName = _nn; 
	
	// Zarejestrowanie modyfikacji obiektu
	fChanged = true;
}
//---------------------------------------------------------------------------

/**
 * Metoda pozwalajaca na ustawienie opisu dotyczacego grupy
 * @param QString _nd - Tresc opisu
 * @returns Brak zwracanych wartoci
 */
void XTT_AttributeList::setDescription(QString _nd)
{ 
	fDescription = _nd; 
	
	// Zarejestrowanie modyfikacji obiektu
	fChanged = true;
}
//---------------------------------------------------------------------------

/**
 * Metoda pozwalajaca na ustawienie rodzica grupy
 * @param unsigned int _newParent - Identyfikator nowego rodzica
 * @returns Brak zwracanych wartoci
 */
void XTT_AttributeList::setParentId(unsigned int _newParent)
{ 
	fParentId = _newParent; 
	
	// Zarejestrowanie modyfikacji obiektu
	fChanged = true;
}
//---------------------------------------------------------------------------

/**
 * Metoda ustawiajaca wartosc zmienej okreslajacej modyfikacje wektora
 * @param bool _c - Nowa wartosc zmiennej
 * @returns Brak zwracanych wartoci
 */
void XTT_AttributeList::setChanged(bool _c)
{
	for(unsigned int i=0;i<fCount;i++)
		fList[i]->setChanged(_c);
			
	fChanged = _c;
}
//---------------------------------------------------------------------------

/**
 * Metoda okreslajaca czy nastapila modyfikacja tego obiektu, jak i obiektow 
 * wewnatrz pojemnika
 * @returns true gdy nastapila modyfiakcja, inaczej false
 */
bool XTT_AttributeList::Changed(void)
{
	for(unsigned int i=0;i<fCount;i++)
		if(fList[i]->Changed())
			return true;
			
	return fChanged;
}
//---------------------------------------------------------------------------

/**
 * Dodaje miejsce i tworzy nowy obiekt argumentu
 * @returns Brak zwracanych wartoci
*/
void XTT_AttributeList::Add(void)
{
     // Wskazik tymczasowy
     XTT_Attribute** tmp = new XTT_Attribute*[fCount+1];

     // Przepisywanie zawartosci
     for(unsigned int i=0;i<fCount;i++)
          tmp[i] = fList[i];

     // Utowrzenie nowego obiektu
     tmp[fCount] = new XTT_Attribute((*fNextId)++);

     // Kiedy lista nie jest pusta to nalezy tez skaswac liste wskaznikow
     if(fCount)
          delete [] fList;

     // Powiekszenie rozmiaru pojemnika
     fCount++;

     // Zapisanie lokalizacji nowej tablicy
     fList = tmp;
	
	// Zarejestrowanie modyfikacji obiektu
	fChanged = true;
}
//---------------------------------------------------------------------------

/**
 * Dodaje nowe miejsce i przyporzadkowywuje je nowemu argumentowi
 * @param XTT_Attribute* _new - Wskanik do newgo argumentu ktory chcemy dodac 
 * do listy
 * @returns Brak zwracanych wartoci
*/
void XTT_AttributeList::Add(XTT_Attribute* _new)
{
     // Wskazik tymczasowy
     XTT_Attribute** tmp = new XTT_Attribute*[fCount+1];

     // Przepisywanie zawartosci
     for(unsigned int i=0;i<fCount;i++)
          tmp[i] = fList[i];

     // Utowrzenie nowego obiektu
     tmp[fCount] = _new;

     // Kiedy lista nie jest pusta to nalezy tez skaswac liste wskaznikow
     if(fCount)
          delete [] fList;

     // Powiekszenie rozmiaru pojemnika
     fCount++;

     // Zapisanie lokalizacji nowej tablicy
     fList = tmp;
	
	// Zarejestrowanie modyfikacji obiektu
	fChanged = true;
}
//---------------------------------------------------------------------------

/**
 * Metoda klasy XTT_AttributeList usuwajaca element z pojemnika
 * @param int index - Index elementu ktory chcemy usunac
 * @param bool delObj - Okresla czy ma zostac skasowany obiekt czy tylko wskaznik
 * @returns Prawde gdy index jest poprawny w przeciwnym wypadku false
*/
bool XTT_AttributeList::Delete(int index, bool delObj)
{
     if((index < 0) || (index >= (int)fCount))
          return false;

     // Wskazik tymczasowy
     XTT_Attribute** tmp = new XTT_Attribute*[fCount-1];

     // Przepisywanie zawartosci
     int tmp_i = 0;
     for(unsigned int i=0;i<fCount;i++)
          if((int)i != index)
               tmp[tmp_i++] = fList[i];

	// Jezeli trzeba skasowac obiekt
	if(delObj)
	{
		// Najpierw natepuje kasowanie kolumn zawierajacych dany argument
		// Przeszukuje tabele
		for(int t=(int)TableList->Count()-1;t>=0;t--)
		{
			// Kasowanie z tabeli kolumn
			int colIndex = TableList->Table(t)->IndexOfAttributeID(fList[index]->Id());
			while(colIndex > -1)
			{
				TableList->Table(t)->DeleteCol(colIndex);
				colIndex = TableList->Table(t)->IndexOfAttributeID(fList[index]->Id());

				// Jezeli tabela nie ma kolumn to ja usuwamy
				if(TableList->Table(t)->ColCount() == 0)
				{
					TableList->Delete(t);
					colIndex = -1;
				}
			}
		}
		delete fList[index];
	}

     // Kiedy lista nie jest pusta to nalezy tez skaswac liste wskaznikow
     if(fCount)
          delete [] fList;

     // Pomniejszenie rozmiaru pojemnika
     fCount--;

     // Zapisanie lokalizacji nowej tablicy
     fList = tmp;
	
	// Zarejestrowanie modyfikacji obiektu
	fChanged = true;

     // Jezeli tu jestesmy to wszystko ok
     return true;
}
//---------------------------------------------------------------------------

/**
 * Metodla klasy XTT_AttributeList, sortujaca atrybuty
 * @returns brak zwaracnych wartosci
*/
void XTT_AttributeList::sort(void)
{
	for(unsigned int i=0;i<fCount;i++)
	{
		for(unsigned j=0;j<fCount-i-1;j++)
		{
			if(fList[j]->Name() > fList[j+1]->Name())
			{
				XTT_Attribute* tmp = fList[j];
				fList[j] = fList[j+1];
				fList[j+1] = tmp;
			}
		}
	}
}
//---------------------------------------------------------------------------

/**
 * Metodla klasy XTT_AttributeList, generujaca liste nazw atrybutow w tym
 * pojemniku
 * @returns Liste nazw atrybutow
*/
QStringList XTT_AttributeList::AttributeNameList(void)
{
     QStringList res;
     for(unsigned int i=0;i<fCount;i++)
          res << fList[i]->Name();

     return res;
}
//---------------------------------------------------------------------------

/**
 * Metodla klasy XTT_AttributeList, generujaca liste akronimow atrybutow w tym
 * pojemniku
 * @returns Liste akronimow atrybutow
*/
QStringList XTT_AttributeList::AttributeAcronymList(void)
{
     QStringList res;
     for(unsigned int i=0;i<fCount;i++)
          res << fList[i]->Acronym();

     return res;
}
//---------------------------------------------------------------------------

/**
 * Metoda klasy XTT_AttributeList oprzniajaca zawartosc pojemnika
 * @returns Brak zwracanych wartoci
*/
void XTT_AttributeList::Flush(void)
{
     while(fCount)
     {
          // Uzywamy tej metody, gdyz zalezy nam na tym aby dabc o sposob
          // usuwania zalezny od typu pojemnika a ta metoda o ot dba
          Delete(0);
     }
	
	// Zarejestrowanie modyfikacji obiektu
	fChanged = true;
}
//---------------------------------------------------------------------------

/**
 * Metoda klasy XTT_AttributeList wyszukujaca atrybut o podanej nazwie
 * @param QString _name - Nazwa atrybutu ktorego szukamy
 * @returns Index w pojemniku gdy istnieje a gdy nie ma takiego w pojemiku to -1
*/
int XTT_AttributeList::IndexOfName(QString _name)
{
     for(unsigned int i=0;i<fCount;i++)
          if(fList[i]->Name() == _name)
               return (int)i;

     return -1;
}
//---------------------------------------------------------------------------

/**
 * Metoda klasy XTT_AttributeList wyszukujaca atrybut o podanym akronimie
 * @param QString _acronym - Akronim atrybutu ktorego szukamy
 * @returns Index w pojemniku gdy istnieje a gdy nie ma takiego w pojemiku to -1
*/
int XTT_AttributeList::IndexOfAcronym(QString _acronym)
{
     for(unsigned int i=0;i<fCount;i++)
          if(fList[i]->Acronym() == _acronym)
               return (int)i;

     return -1;
}
//---------------------------------------------------------------------------

/**
 * Metoda klasy XTT_AttributeList wyszukujaca atrybut o podanym ID
 * @param unsigned int _aID - identyfikator atrybutu ktorego szukamy
 * @returns index w pojemniku gdy istnieje a gdy nie ma takiego w pojemiku to -1
*/
int XTT_AttributeList::IndexOfID(unsigned int _aID)
{
     for(unsigned int i=0;i<fCount;i++)
          if(fList[i]->Id() == _aID)
               return (int)i;

     return -1;
}
//---------------------------------------------------------------------------

/**
 * Metoda klasy XTT_AttributeList, ustawiajaca poziom zagniezdzenia dla grupy
 * @param unsigned int _level - Nowy poziom zagniezedzenia
 * @returns Brak zwracanych wartoci
*/
void XTT_AttributeList::setLevel(unsigned int _level)
{
     fLevel = _level;

     // Jezeli jest to glowny poziom zagniezdzenia
     if(!fLevel)
     {
          // Automatycznie ustawiamy brak rodzicow
          fParentId = 0;
     }
	
	// Zarejestrowanie modyfikacji obiektu
	fChanged = true;
}
//---------------------------------------------------------------------------

/**
 * Metoda zapisujaca wszystkie dane w formacie xttml 1.0
 * @returns Zwraca liste stringow w tym formacie
*/
QStringList* XTT_AttributeList::XTTML_1_0(void)
{
     QStringList* res = new QStringList;
     res->clear();

     *res << "<attr_list parent_id=\"" + QString::number(fParentId) + "\" level=\"" +  QString::number(fLevel) + "\" list_id=\"" + QString::number(fId) + "\">";
     *res << "\t<group_name>" + fName + "</group_name>";
     *res << "\t<group_description>" + fDescription + "</group_description>";

     // Zapis atrybutow
     for(unsigned int a=0;a<fCount;a++)
     {
          QStringList* al = fList[a]->XTTML_1_0();
          for(int l=0;l<al->size();l++)
               *res << "\t" + al->at(l);
          delete al;
     }
     
     *res << "</attr_list>";

     return res;
}
// -----------------------------------------------------------------------------

#if QT_VERSION >= 0x040300
/**
 * Metoda zapisujaca wszystkie dane w formacie xttml 1.0
 * @param QXmlStreamWriter* _xmldoc - wskanik do dokumentu XML
 * @returns brak zwracanych wartosci
*/
void XTT_AttributeList::XTTML_1_0(QXmlStreamWriter* _xmldoc)
{
	_xmldoc->writeStartElement("attr_list");
	_xmldoc->writeAttribute("parent_id", QString::number(fParentId));
	_xmldoc->writeAttribute("level", QString::number(fLevel));
	_xmldoc->writeAttribute("list_id", QString::number(fId));
	
		_xmldoc->writeTextElement("group_name", fName);
		_xmldoc->writeTextElement("group_description", fDescription);

	     // Zapis atrybutow
	     for(unsigned int a=0;a<fCount;a++)
	          fList[a]->XTTML_1_0(_xmldoc);
     
     _xmldoc->writeEndElement();
}
#endif
// -----------------------------------------------------------------------------

/**
 * Metoda wczytujaca dane o strybucie z pliku w formacie XTTML
 * @param QStringList* list - Wskanik do sekcji dotyczacej tego atrybutu
 * @returns true jezeli dane sa ok, ianczej false
*/
bool XTT_AttributeList::FromXTTML_1_0(QStringList* list)
{
     bool res = true;
     
     // Usuwanie spacji i tabulatorow wierszach
     for(int i=0;i<list->size();i++)
     {
          QString line = list->at(i);
          while(line.contains("\t"))
          {
               int index = line.indexOf("\t");
               line = line.remove(index, 1);
          }
          list->replace(i, line);
     }

     // Zakladamy ze pierwszy wiersz jest naglowkiem
     if(!list->at(0).contains("<attr_list", Qt::CaseInsensitive))
          return false;
     if(!list->contains("</attr_list>", Qt::CaseInsensitive))
          return false;

     // Reszta zakladamy ze jest ok

     // Spis tagow
     QStringList tags;
     tags.clear();
     tags << "<attr_list" << "<attribute" << "</attribute>" << "<group_name>" << "<group_description>";

     // Sluza do odpowiedniego kopiowania lini dla atrybutu
     bool copy = false;
     QStringList attribute;
     attribute.clear();

     for(int line=0;line<list->size();line++)
     {
          // Sprawdzanie wystapienia tagu
          int action = -1;
          for(int tag=0;tag<tags.size();tag++)
               if(list->at(line).contains(tags.at(tag), Qt::CaseInsensitive))
                    action = tag;

          // Kopiowanie lini danych atrybutu
          if(copy)
               attribute.append(list->at(line));
               
          if(action == -1)
               continue;

          QString end_tag = tags.at(action);
          end_tag = end_tag.insert(1, "/");
          QString l = list->at(line);
          l = l.remove(l.indexOf(tags.at(action), Qt::CaseInsensitive), tags.at(action).length());
          l = l.mid(0, l.indexOf(end_tag, Qt::CaseInsensitive));

          // Wczytywanie parametrow w zaleznosci od wykrytego tagu
          if(action == 0)
          {
               fParentId = (unsigned int)XML_ReadValue(list->at(line), "parent_id").toInt();
               fLevel = (unsigned int)XML_ReadValue(list->at(line), "level").toInt();
               fId = (unsigned int)XML_ReadValue(list->at(line), "list_id").toInt();
          }
          if(action == 1)
          {
               // Jezeli tag rozpoczynajacy dane atrybutu, to uruchamiamy kopiowanie
               copy = true;
               attribute.append(list->at(line));
          }
          if(action == 2)
          {
               // Kiedy linia konczaca linie danych atrybutu
               // Dodajemy nowy atrybut
               Add();

               // Wczytujemy dane z pliku
               res = res && fList[fCount-1]->FromXTTML_1_0(&attribute);

               // Konczymy kopiowanie
               copy = false;
               
               // Czyscimy zawartosc danych atrybutu
               attribute.clear();
          }
          if(action == 3)
               fName = l;
          if(action == 4)
               fDescription = l;
     }

     //QStringList* h = XTTML_1_0();
     //for(int i=0;i<h->size();i++)
          //QMessageBox::warning(NULL, "Application", h->at(i));

     return res;
}
// -----------------------------------------------------------------------------

/**
 * Metoda wczytujaca dane o strybucie z pliku w formacie XTTML
 * @param QDomElement& root - Referencja do sekcji dotyczacej tego atrybutu
 * @returns true jezeli dane sa ok, ianczej false
*/
bool XTT_AttributeList::FromXTTML_1_0(QDomElement& root)
{
	if(root.tagName().toLower() != "attr_list")
		return false;
		
     bool res = true;
	
	QDomElement DOMgn = root.firstChildElement("group_name");
	QDomElement DOMgd = root.firstChildElement("group_description");
	QDomElement DOMatt = root.firstChildElement("attribute");
	
	fParentId = (unsigned int)root.attribute("parent_id", "0").toInt();
	fLevel = (unsigned int)root.attribute("level", "0").toInt();
	fId = (unsigned int)root.attribute("list_id", "0").toInt();
	
     fName = DOMgn.text();
     fDescription = DOMgd.text();

     while(!DOMatt.isNull())
     {
		// Kiedy linia konczaca linie danych atrybutu
		// Dodajemy nowy atrybut
		Add();

		// Wczytujemy dane z pliku
		res = res && fList[fCount-1]->FromXTTML_1_0(DOMatt);
		
		// Wczytanie kolejnego atrybutu
          DOMatt = DOMatt.nextSiblingElement("attribute");
     }

     return res;
}
// -----------------------------------------------------------------------------
