 /**
 *                                                                             *
 * @file XTT_AttributeList.h                                                   *
 * Project name: HeKatE hqed                                                   *
 * $Id: XTT_AttributeList.h,v 1.12 2008-03-03 22:04:34 kinio Exp $              *
 * Author: Krzysztof Kaczor                                                    *
 * Date: 25.04.2007                                                            *
 *                                                                             *
 * Implementation by Krzysztof Kaczor <kinio4444@gmail.com>                    *
 *                                                                             *
 * Copyright (C) 2006-9 by the HeKatE Project                                  *
 *                                                                             *
 * HQEd has been develped within the HeKatE Project,                           *
 * see http://hekate.ia.agh.edu.pl                                             *
 *                                                                             *
 * This file is part of HQEd.                                                  *
 *                                                                             *
 * HQEd is free software: you can redistribute it and/or modify                *
 * it under the terms of the GNU General Public License as published by        *
 * the Free Software Foundation, either version 3 of the License, or           *
 * (at your option) any later version.                                         *
 *                                                                             *
 * HQEd is distributed in the hope that it will be useful,                     *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU General Public License for more details.                                *
 *                                                                             *
 * You should have received a copy of the GNU General Public License           *
 * along with HQEd.  If not, see <http://www.gnu.org/licenses/>.               *
 *                                                                             *
 * Comment: Plik zawiera definicje klasy bedacej zbiornikiem atrybtutow        *
 *                                                                             *
 */


#ifndef XTT_AttributeListH
#define XTT_AttributeListH
//---------------------------------------------------------------------------

#include <QString>
#include <QStringList>
#include <QDomDocument>

#if QT_VERSION >= 0x040300
	#include <QXmlStreamWriter>
#endif

#include "XTT_Attribute.h"
//---------------------------------------------------------------------------

class XTT_AttributeList
{
private:

     /// Identyfikator tej grupy
     unsigned int fParentId;

     /// Poziom zagniezdzenia
     unsigned int fLevel;

     /// Tablica wskaznikow do obiektow atrybutow bedacych w tym pojemniku
     XTT_Attribute** fList;

     /// Oznacza liczba atrybutow
     unsigned int fCount;

     /// ID Listy
     unsigned int fId;

     /// Przyjmuje wartosc nastepnego ID nadawanego nowemu atrybutowi
     unsigned int* fNextId;

     /// Nazwa pojemnika
     QString fName;

     /// Opis pojemnika
     QString fDescription;
	
	/// Okresla czy nastapila modyfikacja obiektu
	bool fChanged;

public:

     /// Konstruktor defaultowy
     XTT_AttributeList(void);
     
     /// Konstruktor przyjmujacy ID pojemnika
     XTT_AttributeList(unsigned int);

     /// Konstruktor przyjmujacy jako argumnet ID i typ pojemnika
     XTT_AttributeList(unsigned int*, unsigned int);

     /// Destruktor
     ~XTT_AttributeList(void);

     /// ----------------------- Metody obslugujace pliki -----------------------

     /// Wstawia wszystkie dane w formacie xttml 1.0
     QStringList* XTTML_1_0(void);
	
	#if QT_VERSION >= 0x040300
	/// Wstawia wszystkie dane w formacie xttml 1.0
     void XTTML_1_0(QXmlStreamWriter*);
	#endif

     /// Wczytuje dane z pliku
     bool FromXTTML_1_0(QStringList*);
	
	/// Wczytuje dane z pliku
     bool FromXTTML_1_0(QDomElement&);

     
     /// ----------------- Metody zwracajaca wartosci pol klasy -----------------

     /// Zwraca wielkosc pojemnika
     unsigned int Count(void){ return fCount; }

     /// Zwraca ID listy
     unsigned int ID(void){ return fId; }

     /// Zwraca nastepny identyfikator dla argumentu i powieksza bierzacy
     unsigned int NextId(void){ return (*fNextId)++; }

     /// Zwraca nazwe pojemnika
     QString Name(void){ return fName; }

     /// Zwraca opis pojemnika
     QString Description(void){ return fDescription; }

     /// Zwraca wskaznik do wybranego atrybutu
     XTT_Attribute* Item(int);

     /// Zwraca wskaznik do rodzica, jezeli brak rodzica to zwraca null
     unsigned int ParentID(void){ return fParentId; }

     /// Zwaca poziom zganiezdzenia tej listy
     unsigned int Level(void){ return fLevel; }

     /// ---------------- Metody ustawiajace wartosci pol klasy -----------------

     /// Pozwala ustawic typ pojemnika
     void setContainerType(unsigned int);

     /// Pozwala ustawic nazwe pojemnika
     void setName(QString);

     /// Pozwala ustawic opis pojemnika
     void setDescription(QString);

     /// Ustawia wskaznik rodzica
     void setParentId(unsigned int);

     /// Ustawia poziom zagniezdzenia
     void setLevel(unsigned int);
	
	/// Metoda ustawiajaca zmienna okreslajaca zmiane obiektow
	void setChanged(bool);

     /// -------------------------- Metody operacyjne ---------------------------

     /// Pozwala dodac nowy element do zbiornika
     void Add(void);

     /// Pozwala dodac okreslony element do zbiornika
     void Add(XTT_Attribute*);

     /// Pozwala usunac wybrany element ze zbiornika, w zaleznosci od typu
     /// usuwa tylko wskaznik lub tez sam obiekt
     bool Delete(int, bool delObj = true);
	
	/// Sortuje liste atrybutow
	void sort(void);

     /// Tworzy liste nazw atrybutow z tego pojemnika
     QStringList AttributeNameList(void);

     /// Tworzy liste akronimow atrybutow z tego pojemnika
     QStringList AttributeAcronymList(void);

     /// Zwraca index atrybutu o podanej nazwie, jezeli nie ma takiej nazwy to zwraca -1
     int IndexOfName(QString);

     /// Zwraca index atrybutu o podanym akronimie, jezeli nie ma takiego to zwraca -1
     int IndexOfAcronym(QString);
	
	/// Zwraca index atrybutu o podanym ID, jezeli nie ma takiego to zwraca -1
	int IndexOfID(unsigned int);

     /// Oprozna zawartosc pojemnika
     void Flush(void);
	
	/// Zwraca czy obiekt oraz caly pojemnik byl zmodyfikowany
	bool Changed(void);
     
};
//---------------------------------------------------------------------------

extern XTT_AttributeList* AttributeList;
//---------------------------------------------------------------------------
#endif
