 /**
 *                                                                             *
 * @file MainWin_ui.cpp                                                        *
 * Project name: HeKatE hqed                                                   *
 * $Id: XTT_Cell.cpp,v 1.12 2008-03-03 22:04:34 kinio Exp $                     *
 * Author: Krzysztof Kaczor                                                    *
 * Date: 24.04.2007                                                            *
 *                                                                             *
 * Implementation by Krzysztof Kaczor <kinio4444@gmail.com>                    *
 *                                                                             *
 * Copyright (C) 2006-9 by the HeKatE Project                                  *
 *                                                                             *
 * HQEd has been develped within the HeKatE Project,                           *
 * see http://hekate.ia.agh.edu.pl                                             *
 *                                                                             *
 * This file is part of HQEd.                                                  *
 *                                                                             *
 * HQEd is free software: you can redistribute it and/or modify                *
 * it under the terms of the GNU General Public License as published by        *
 * the Free Software Foundation, either version 3 of the License, or           *
 * (at your option) any later version.                                         *
 *                                                                             *
 * HQEd is distributed in the hope that it will be useful,                     *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU General Public License for more details.                                *
 *                                                                             *
 * You should have received a copy of the GNU General Public License           *
 * along with HQEd.  If not, see <http://www.gnu.org/licenses/>.               *
 *                                                                             *
 * Comment: Plik zawierajcy definicje funckji obslugujacych okienko glowne     *
 * programu                                                                    *
 *                                                                             *
 */

#include <math.h>

#include <QMessageBox>
#include <QtGui>
#include <QDialog>
#include <QObject>

#include <QtGui>

#include "../../V/XTT/GTable.h"
#include "../../C/XTT/ErrorsListWidget.h"
#include "../../C/MainWin_ui.h"
#include "../../C/Settings_ui.h"
#include "XTT_TableList.h"
#include "XTT_Table.h"
#include "XTT_Attribute.h"
#include "XTT_AttributeGroups.h"
#include "XTT_Cell.h"
// -----------------------------------------------------------------------------

// Definicja zmiennych globalnych zawierajacych informacje o reprezentacji operatorow porownania w programie i pliku XML
const QStringList smbl_opers = (QStringList() << "=" << "!=" << ">=" << ">" << "<=" << "<" << "nin" << "in");
const QStringList text_opers = (QStringList() << "e" << "ne" << "ge" << "g" << "le" << "l" << "nin" << "in");
// -----------------------------------------------------------------------------

/**
 * Konstrukotr defaultowy klasy
 * @returns Brak zwracanych wartosci
*/
XTT_Cell::XTT_Cell(void)
{
     fParent = 0;
     fCellID = 0;
     fAttrType = NULL;
     fValue = "";
     fContent = "";
	fCellError = "";
     fValDefType = 0;
     fBrace = new bool[2];
     fBrace[0] = false;
     fBrace[1] = false;
     fBoundaries[0] = "";
     fBoundaries[1] = "";
     fAttrValue = new QStringList;
     fOperator = 0;
     fIgnore = false;
     fContext = 0;
	fChanged = false;
	fEnableAttr2AttrComp = Settings->xttAttrToAttrComp();
	fIsSelected = false;
}
// -----------------------------------------------------------------------------

/**
 * Konstruktor klasy
 * @param XTT_Table* _parent - Wskanik do tabeli rodzica tej komorki
 * @param unsigned int _id - Identyfikaotr komorki
 * @param int _context - Kontekst  wktorym wystepuje komorka
 * @returns Brak zwracanych wartosci
*/
XTT_Cell::XTT_Cell(unsigned int _parent, unsigned int _id, int _context)
{
     fParent = _parent;
     fCellID = _id;
     fAttrType = NULL;
     fValue = "";
     fContent = "";
	fCellError = "";
     fValDefType = 0;
     fBrace = new bool[2];
     fBrace[0] = false;
     fBrace[1] = false;
     fBoundaries[0] = "";
     fBoundaries[1] = "";
     fAttrValue = new QStringList;
     fOperator = 0;
     fIgnore = false;
     fContext = _context;
	fChanged = false;
	fEnableAttr2AttrComp = Settings->xttAttrToAttrComp();
	fIsSelected = false;
}
// -----------------------------------------------------------------------------

/**
 * Destruktor klasy
 * @returns Brak zwracanych wartosci
*/
XTT_Cell::~XTT_Cell(void)
{
	// Usuniecie z listy bledow wyswietlanej w oknie glownym, dokowalnym
	MainWin->xtt_errorsList->deleteItem(fParent, fCellID);
	
     delete [] fBrace;
     delete fAttrValue;
}
// -----------------------------------------------------------------------------

/**
 * Metoda klasy XTT_Cell ustawiajaca kontekst komorki
 * @param unsigned int _ctx - Index kontekstu komorki
 * @returns true jezeli index jest poprany naczej false
*/
bool XTT_Cell::setContext(unsigned int _ctx)
{
     if((((int)_ctx) < 0) || (_ctx > 4))
          return false;

     fContext = _ctx;
	
	fChanged = true;

     return true;
}
// -----------------------------------------------------------------------------

/**
 * Metoda ustawiajaca atrybut przypisany do komorki
 * @param XTT_Attribute* _xtt_attr_type - Wskanik do atrybutu
 * @returns Brak zwracanych wartoci
 */
void XTT_Cell::setAttrType(XTT_Attribute* _xtt_attr_type)
{
	fAttrType = _xtt_attr_type; 
	fChanged = true;
}
// -----------------------------------------------------------------------------

/**
 * Metoda ustawiajaca zawartosc komorki
 * @param QString _content - Tresc nowej zawartosci
 * @returns Brak zwracanych wartoci
 */
void XTT_Cell::setContent(QString _content)
{ 
	fContent = _content; 
	fChanged = true;
}
// -----------------------------------------------------------------------------

/**
 * Metoda opisujaca zawartosc komorki, opisuje rodzaj typu nawiasu otwierajacego
 * @param bool _bt - Wartosc typu nawiasu
 * @returns Brak zwracanych wartoci
 */
void XTT_Cell::setLoBrace(bool _bt)
{ 
	fBrace[0] = _bt; 
	fChanged = true;
}
// -----------------------------------------------------------------------------

/**
 * Metoda opisujaca zawartosc komorki, opisuje rodzaj typu nawiasu zamykajacego
 * @param bool _bt - Wartosc typu nawiasu
 * @returns Brak zwracanych wartoci
 */
void XTT_Cell::setHiBrace(bool _bt)
{ 
	fBrace[1] = _bt; 
	fChanged = true;
}
// -----------------------------------------------------------------------------

/**
 * Metoda ustawiajaca wartosc dolnego ograniczenia w przypadku definicji 
 * wartosci jako przedzial
 * @param QString _b - Wartosc dolnego ograniczenia
 * @returns Brak zwracanych wartoci
 */
void XTT_Cell::setLoBoundary(QString _b)
{ 
	fBoundaries[0] = _b; 
	fChanged = true;
}
// -----------------------------------------------------------------------------

/**
 * Metoda ustawiajaca wartosc gornego ograniczenia w przypadku definicji 
 * wartosci jako przedzial
 * @param QString _b - Wartosc gornego ograniczenia
 * @returns Brak zwracanych wartoci
 */
void XTT_Cell::setHiBoundary(QString _b)
{ 
	fBoundaries[1] = _b;
	fChanged = true;
}
// -----------------------------------------------------------------------------

/**
 * Metoda pozwalajaca ustawic typ operatora wystepujacego w komorce
 * @param unsigned int _new_oper_type - Wartosc typu operatora
 * @returns Brak zwracanych wartoci
 */
void XTT_Cell::setOperatorType(unsigned int _new_oper_type)
{ 
	fOperator = _new_oper_type; 
	fChanged = true;
}
// -----------------------------------------------------------------------------

/**
 * Metoda pozwalajaca na ustawienie przezroczystosci komorki (not implemented 
 * yet)
 * @param bool _i - Wartosc opisujaca czy komorka ma byc przezroczysta
 * @returns Brak zwracanych wartoci
 */
void XTT_Cell::setIgnore(bool _i)
{ 
	fIgnore = _i; 
	fChanged = true;
}
// -----------------------------------------------------------------------------

/**
 * Metoda ustawiajaca flage porownywania atrybutow
 * @param bool _nv - Nowa wartosc
 * @returns brak zwracanych wartosci
 */
void XTT_Cell::setEnableAttr2AttrComp(bool _nv)
{
	fEnableAttr2AttrComp = _nv;
}
// -----------------------------------------------------------------------------

/**
 * Metoda zwracajaca wartosc czy dany obiekt byl zmodyfikowany
 * @returns true jezeli modyfikacja byla, inaczej false
 */
bool XTT_Cell::Changed(void)
{
	return fChanged;
}
// -----------------------------------------------------------------------------

/**
 * Metoda ustawiajaca czy dany obiekt byl modyfikowany
 * @param bool _c - Nowa wartosc
 * @returns Brak zwracanych wartoci
 */
void XTT_Cell::setChanged(bool _c)
{	
	fChanged = _c;
}
// ---------------------------------------------------------------------------

/**
 * Metoda klasy zaznaczajaca/odznaczajaca komorke na scenie
 * @param bool _isSelected - okresla czy dana komorka jest zaznaczona
 * @returns Brak zwracanych wartoci
*/
void XTT_Cell::setSelected(bool _isSelected)
{
	fIsSelected = _isSelected;
	
	XTT_Table* parent = TableParent();
	if(parent == NULL)
		return;
		
	parent->GraphicTable()->ensureVisible();
}
// ---------------------------------------------------------------------------

/**
 * Zwraca czy dana kmorka jest zaznaczona czy nie
 * @returns true jezeli jest zaznaczona inaczej false
*/
bool XTT_Cell::isSelected(void)
{
	return fIsSelected;
}
// ---------------------------------------------------------------------------

/**
 * Metoda klasy wyszukujaca i zwracajaca wskazik do tabeli rodzica
 * @returns Jezeli taka tabela nie istnieje to NULL
*/
XTT_Table* XTT_Cell::TableParent(void)
{
     for(unsigned int i=0;i<TableList->Count();i++)
          if(TableList->Table(i)->TableId() == fParent)
               return TableList->Table(i);

     return NULL;
}
// -----------------------------------------------------------------------------

/**
 * Metoda klasy okreslajaca czy zawartosc komorki jest poprawna
 * @returns true jezeli jest poprawna inaczej flase
*/
bool XTT_Cell::isCorrectSyntax(void)
{
	return fCellError == "";
}
// -----------------------------------------------------------------------------

/**
 * Metoda klasy ustawiajaca tresc bledu komorki
 * @param QString _cellError - tresc ostatniego bledu komorki
 * @returns brak zwracanych wartosci
*/
void XTT_Cell::setCellError(QString _cellError)
{
	fCellError = _cellError;
	
	// Dodanie do listy bledow wyswietlanej w oknie glownym, dokowalnym
	MainWin->xtt_errorsList->add(fParent, fCellID, _cellError);
}
// -----------------------------------------------------------------------------

/**
 * Metoda klasy zwracajaca tresc ostatniego bledu komorki
 * @param bool addLocalizationInfo - Okresla czy do komunikatu o bledzie nalezy dodac informacje o lokalizacji bledu
 * @returns string zawierajacy tresc ostatniego bledu komorki
*/
QString XTT_Cell::CellError(bool addLocalizationInfo)
{
	if(!addLocalizationInfo)
		return fCellError;
		
	XTT_Table* parent = TableList->_IndexOfID(fParent);

	// Wyszukanie wiersza i kolumny tabeli w ktorych wystepuje komorka
	int cell_row = -1;
	int cell_col = -1;
	if(parent != NULL)
	{
		for(unsigned int row=0;row<parent->RowCount();row++)
			if(parent->Row(row)->indexOfCellID(fCellID) > -1)
			{
				cell_row = row;
				cell_col = parent->Row(row)->indexOfCellID(fCellID);
			}
	}
		
	if((cell_row == -1) || (cell_col == -1))
		return fCellError;
		
	QString res = fCellError + "\n";
	
	// Zapisanie naglowka tabeli
	QString tableTitle = parent->Title();
	
	// Zapisanie wspolrzednych tabeli
	QString cr = QString::number(cell_row+1);
	QString cc = QString::number(cell_col+1);
		
	// Dopisanie parametrow bledu
	res = res + "Error in table " + tableTitle + ", in row " + cr + ", column " + cc + ".";
	
	return res;
}
// -----------------------------------------------------------------------------

/**
 * Metoda klasy analizujaca i wykonujaca odpowiednie dzialanie
 * @param QString expression - Wyrazenie do analizy
 * @returns True jezeli wszystko ok, inaczej false
*/
bool XTT_Cell::Analyze(QString expression)
{
	// Usuniecie z listy bledow wyswietlanej w oknie glownym, dokowalnym
	MainWin->xtt_errorsList->deleteItem(fParent, fCellID);
	
	// Na poczatku kasujemy komunikat ostatniego bledu, poniewaz nie chcemy aby byl on wyswietlany
	// po analizie ktora moze zakonczyc sie pozytywnie
	fCellError = "";
	
	
     // Sprawdzanie czy komorka ma rodzica
     if(TableParent() == NULL)
     {
          setCellError("Cell hasn't got a parent.");
          return false;
     }
	
	// Sprawdzenie czy nie jest to ciag pusty
     if((expression == "") || (expression.length() == 0))
     {
          setCellError("Empty cell.");
          return false;
     }

     // Zdefiniowanie operatorow, uwaga wazna kolejnosc ich podawania!!!
     QStringList opers(smbl_opers);

     // Poczatkowe ustawienie
     fOperator = 0;

     // Wstepna obrobka wyrazenia
     // Usuwanie spacji na poczatku i koncu wyrazenia
     for(;(expression.length() > 0) && (expression[0] == ' ');)
          expression = expression.remove(0, 1);

     // Jezeli nie ma juz nic wiecej
     if(expression == "")
     {
          setCellError("Syntax error. No value defined.");
          return false;
     }

     // Usuwanie spacji na poczatku i koncu wyrazenia
     for(;(expression.length() > 0) && (expression[expression.length()-1] == ' ');)
          expression = expression.remove(expression.length()-1, 1);

     // Sprawdzanie wystapienie operatora
     for(int i=0;i<opers.size();i++)
          if(expression.indexOf(opers.at(i), 0, Qt::CaseInsensitive) == 0)
          {
               // Usuniecie znakow operatora
               expression = expression.remove(0, opers.at(i).length());

               // Usuniecie spacji po operatorze
               for(;(expression.length() > 0) && (expression[0] == ' ');)
                    expression = expression.remove(0, 1);

               // Zapisanie typu operatora
               fOperator = i+1;

               // Koniec petli
               break;
          }

     // Jezeli nie wykryto operatora zwracamy niepowodzenie
     if(fOperator == 0)
     {
          setCellError("Missing operator.");
          return false;
     }

     // Jezeli nie ma juz nic wiecej
     if(expression == "")
     {
          setCellError("Syntax error. No value defined.");
          return false;
     }

     // Jezeli wykryto to sprawdzamy czy moze nalezec do kontekstu
     // W kontekstach conditional, assert, retract dozwolone sa wszystkie typy
     // operatorow, tylko w kontekscie decision nie moga wystapic takie operatory
     // jak >, >=, <, <=, !=, nin, in natomiast moze wystapic operator =
     if((fOperator != 1) && (fContext == 4))
     {
          setCellError("Bad operator in context.");
          return false;
     }

     // Dlasza analiza

     // Sprawdzanie wsytapienia operatora <any>
     if(expression.contains(XTT_Attribute::_any_operator_, Qt::CaseInsensitive))
     {
          // Operator moze wystapic tylko w kontekscie conditional i retract
          if((fContext != 1) && ((fContext != 3)))
          {
               setCellError("Operator " + XTT_Attribute::_any_operator_ + " is not allowed in this context.");
               return false;
          }
          /*
          fContent = opers.at(fOperator-1) + " <any>";
          fValue = XTT_Attribute::_any_operator_;
          fValDefType = 1;
          return true;*/
     }

     // Sprawdzanie typu podanej wartosci, czy to jest atomic, set, range
     // Jezeli to jest set, lub range to teraz na pierwszej pozycji powinien
     // wsytapic nawias otwierajacy inabczej jest jest to typ atomic, jezeli jako
     // pierwszy znak mamy nawias ( lub [, to nalezy sprawdzic czy nie wysepuje
     // jeszcze srednik i taki sam nawias zamykajacy na koncu, wtedy jest to
     // przedzial range
     fValDefType = 0;

     if(((expression[0] == '(') || (expression[0] == '[')) &&
     (expression.count(";") == 1) &&
     ((expression[expression.length()-1] == ')') || (expression[expression.length()-1] == ']')))
     {
          fValDefType = 3;
          
          // Zapisanie typu nawiasow
          fBrace[0] = expression[0] == '(';
          fBrace[1] = expression[expression.length()-1] == ')';
     }

     if(fValDefType == 0)
     {
          if(expression[0] == '{')
               fValDefType = 2;
          else
               fValDefType = 1;
     }
	
	// Jezeli wysteouje wiecej niz jeden srednik to jest to napewno set
	if(((expression.count(";") > 1) && (fValDefType != 2)) || 
	   ((expression.count(";") == 1) && ((fValDefType == 0) || (fValDefType == 1))))
	{	
		setCellError("Value type seems to be \"set\". Maybe you forget the bracket \"{\"\nIf you want to define set you should use the following syntax:\n{expr1; expr2; ... ; exprn}");
		return false;
	}

     // Teraz sprawdzamy czy dla podanego typu wartosci moze istniec podany operator
     // w danym kontekscie. Rozpatrzymy tylko przypadki bledne aby zasygnalizowac
     // powstaly blad

     // Kontkest decyzyjny
     if(fContext == 1)
     {
          // Gdy to jest jeden z operatorow >=, >, <, <=, =, != i nie jest to
          // tpy atomic to blad
          if((fOperator >= 1) && (fOperator <= 6) && (fValDefType != 1))
          {
               setCellError("Bad operator. Allowed operators are only in, nin.");
               return false;
          }
     }

     // Kontkest assert i retract
     if((fContext == 2) || (fContext == 3))
     {
          if((fOperator >= 2) && (fOperator <= 6) && (fValDefType != 1))
          {
               setCellError("Bad operator. Allowed operators are only in, nin.");
               return false;
          }

          if((fOperator == 1) && (fValDefType != 1) && (fValDefType != 2))
          {
               setCellError("Bad operator. Allowed operators are only in, nin.");
               return false;
          }
     }

     // Kontkest action, tutaj sa dozwolone tylko set i atomic oraz operator =
     if(fContext == 4)
     {
          if((fOperator != 1) || (fValDefType == 3))
          {
               setCellError("Range is not allowed in context Action.");
               return false;
          }
     }
     

     // Wczytywanie argumentow, kazdy moze byc wyrazeniem arytmetycznym
     QStringList args;
     args.clear();

     // Gdy atomic, to cale wyrazenie
     if(fValDefType == 1)
          args << expression;

     // Gdy zbior
     if(fValDefType == 2)
     {
          if(expression[0] == '{')
               expression = expression.remove(0, 1);
          if(expression[expression.length()-1] == '}')
               expression = expression.remove(expression.length()-1, 1);

          // Wczytywanie argumentow, kolejnych elementow zbioru
          args = expression.split(";", QString::SkipEmptyParts, Qt::CaseSensitive);
     }

     // Gdy jest to przedzial
     if(fValDefType == 3)
     {
          expression = expression.remove(0, 1);
          expression = expression.remove(expression.length()-1, 1);
          args = expression.split(";", QString::SkipEmptyParts, Qt::CaseSensitive);
     }

     // Kiedy nie podamy argumentow
     if(!args.size())
     {
          setCellError("Expression syntax.");
          return false;
     }

     // Usuwanie spacji na koncach kazdego wyrazenia
     for(int i=0;i<args.size();i++)
     {
          for(;args.at(i)[0] == ' ';)
          {
               QString val = args.at(i);
               args.replace(i, val.remove(0, 1));
          }
          for(;args.at(i)[args.at(i).length()-1] == ' ';)
          {
               QString val = args.at(i);
               args.replace(i, val.remove(val.length()-1, 1));
          }
     }

     // Obliczanie wartosci wyrazen
     for(int i=0;i<args.size();i++)
     {
          QString expr = args.at(i);
          if(Calculate(expr) == "#NAN")
          {
               AttributeGroups->_IndexOfName("tmp")->Flush();
               return false;
          }
          args.replace(i, expr);
     }
	
	// ------ Weryfikacja typu atrybutu i jego dziedziny ------
	if(Settings->xttCheckAtomicValues())
	{
		for(int i=0;i<args.size();i++)
		{
			if((args.at(i).toLower() == XTT_Attribute::_not_definded_operator_) || (args.at(i).toLower() == XTT_Attribute::_any_operator_))
				continue;
					
			// Jezeli atrybut ma typ boolean
			if(fAttrType->Type() == 1)
			{
				if((args.at(i).toLower() != "true") && (args.at(i).toLower() != "false"))
				{
					setCellError("Value \'" + args.at(i) + "\' is out of attribute \'" + fAttrType->Name() + "\' domain.\nAllowed values are: false, true");
					return false;
				}
			}
			
			// Jezeli atrybut ma typ integer
			if(fAttrType->Type() == 2)
			{
				bool valok;
				args.at(i).toInt(&valok);
				
				if(!valok)
				{
					setCellError("Value \'" + args.at(i) + "\' is not integer type.");
					return false;
				}
			}
			
			// Jezeli atrybut ma typ float
			if(fAttrType->Type() == 3)
			{
				bool valok;
				args.at(i).toFloat(&valok);
				
				if(!valok)
				{
					setCellError("Value \'" + args.at(i) + "\' is not float type.");
					return false;
				}
			}
			
			// Jezeli atrybut ma typ symbolic - to nie musimy sprawdzac bo kazda wartosc jest dozwolona
			//if(fAttrType->Type() == 4)
			//{
			//}
			
			// Jezeli atrybut ma typ enumerative to sprawdzamy czy podana wartosc jest w zbiorze wyliczeniowym
			if(fAttrType->Type() == 5)
			{
				if(fAttrType->AttributeValues()->indexOf(args.at(i)) == -1)
				{
					setCellError("Value \'" + args.at(i) + "\' is out of attribute \'" + fAttrType->Name() + "\' domain.\nAllowed values are: " + fAttrType->ConstraintsRepresentation());
					return false;
				}
			}
		}
	}
	

     // Jezeli wszystko ok to zapisujemy wyliczone wartosci

     // Jezeli atomic, to przepisujemy wartosc do pola fValue
     if(fValDefType == 1)
          fValue = args.at(0);

     // Gdy typ wyliczeniowy
     if(fValDefType == 2)
     {
          fAttrValue->clear();
          for(int i=0;i<args.size();i++)
               fAttrValue->append(args.at(i));
     }

     // Gdy przedzial
     if(fValDefType == 3)
     {
          fBoundaries[0] = args.at(0);
          fBoundaries[1] = args.at(1);

          // Kiedy podamy takie same wartosci na koncach przedzialu
          if((fBoundaries[0] == fBoundaries[1]) && (fBoundaries[1].toLower() != XTT_Attribute::_any_operator_) && ((fBrace[0]) || (fBrace[1])))
          {
               setCellError("The limits can't be the same when the range is open.");
               return false;
          }
     }
	
	// Jezeli wylaczona jest porownywanie pomiedzy atrybutami to mozemy na robe odpalic komorke
	if(((!fEnableAttr2AttrComp) && (Settings->xttExecuteCellWhenOff())) || ((fEnableAttr2AttrComp) && (Settings->xttExecuteCellWhenOn())))
	{
		// Zapisanie bierzacej wartosci atrybutu
		QString attrCurVal = fAttrType->Value();
		
		// Odpalenie komorki
		int execRes = Execute(expression, false);
		
		// Przywrocenie poprzedniej wartosci atrybutu
		fAttrType->setValue(attrCurVal);
		
		// Jezeli odpalenie komorki sie nie udalo
		if(execRes == -1)
			return false;
	}
	
	// Zapisanie zawartosci komorki
     fContent = expression;

     return true;
}
// -----------------------------------------------------------------------------

/**
 * Metoda klasy XTT_Cell wykonujaca operacje zdefiniowane w komorce w zaleznosci od kontekstu
 * @param QString expr - Operacje do wykonania podane jako string
 * @param bool cellAnalyze - Okresla czy komorka powinna byc przeanalizowana przed odpaleniem, default = true
 * @returns -1 bd, 0 - false, 1 - true
*/
int XTT_Cell::Execute(QString expr, bool cellAnalyze)
{
     // Sprawdzenie wyrazenia pod kontem poprawnosci
	if(cellAnalyze)
	{
	     bool res = Analyze(expr);
	     if(!res)
	          return -1;
	}

     // Wykonywanie opercaji

     // Jezeli kontekst warunkowy
     if(fContext == 1)
     {
          // Sprawdzanie typu podanej wartosci

          // ---------------------------  Jezeli jest to atomic ---------------------------
          if(fValDefType == 1)
          {
               // Jezeli jest <any> i wartosc jest zdefiniowana to zwraca true
               if((fValue.toLower() == XTT_Attribute::_any_operator_) && (fAttrType->Value().toLower() != XTT_Attribute::_not_definded_operator_))
                    return 1;

               // Jezeli wartosc jest nie zdefiniowana w komorce i w atrybucie
               if((fValue.toLower() == XTT_Attribute::_not_definded_operator_) && (fAttrType->Value().toLower() == XTT_Attribute::_not_definded_operator_))
                    return 1;

               // Jezeli wartosc jest nie zdefiniowana to false
               if(fAttrType->Value().toLower() == XTT_Attribute::_not_definded_operator_)
                    return 0;

               // Sprawdzanie typu operatora

               // Operator =
               if((fOperator == 1) || (fOperator == 8))
               {
                    // Dla typu wyliczeniowego
                    if(fAttrType->Type() == 5)
                    {
                         int i1 = fAttrType->AttributeValues()->indexOf(fAttrType->Value());
                         int i2 = fAttrType->AttributeValues()->indexOf(fValue);
                         if((i1 == -1) || (i2 == -1))
                         {
                              setCellError("Unrecognized type, type must be enumerative.");
                              return -1;
                         }
                         if(i1 == i2)
                              return 1;
                         return 0;
                    }

                    // Dla pozostalych typow
                    
                    // Testowanie wartosci
                    if((fValue.toLower() == XTT_Attribute::_any_operator_) && (fAttrType->Value().toLower() != XTT_Attribute::_not_definded_operator_))
                         return 1;
                    // Testowanie wartosci
                    if(fAttrType->Value().toLower() == fValue.toLower())
                         return 1;
                    return 0;
               }
               
               // Operator !=
               if((fOperator == 2) || (fOperator == 7))
               {
                    // Dla typu wyliczeniowego
                    if(fAttrType->Type() == 5)
                    {
                         int i1 = fAttrType->AttributeValues()->indexOf(fAttrType->Value());
                         int i2 = fAttrType->AttributeValues()->indexOf(fValue);
                         if((i1 == -1) || (i2 == -1))
                         {
                              setCellError("Unrecognized type, type must be enumerative.");
                              return -1;
                         }
                         if(i1 != i2)
                              return 1;
                         return 0;
                    }

                    // Dla pozostalych typow
                    
                    // Testowanie wartosci
                    if((fValue.toLower() == XTT_Attribute::_any_operator_) || (fAttrType->Value().toLower() == XTT_Attribute::_not_definded_operator_))
                         return 1;
                    // Testowanie wartosci
                    if(fAttrType->Value() != fValue.toLower())
                         return 1;
                    return 0;
               }

               // Operator >=
               if(fOperator == 3)
               {
                    // Dla typu bool
                    if(fAttrType->Type() == 1)
                    {
                         if(fAttrType->Value() == "true")
                              return 1;
                         if(fAttrType->Value() == fValue.toLower())
                              return 1;
                         return 0;
                    }

                    // Dla typu liczbowego
                    if((fAttrType->Type() == 2) || (fAttrType->Type() == 3))
                    {
                         float a1 = fAttrType->Value().toFloat();
                         float a2 = fValue.toFloat();
                         if(a1 >= a2)
                              return 1;
                         return 0;
                    }

                    // Dla typu symbolicznego
                    if(fAttrType->Type() == 4)
                    {
                         if(fAttrType->Value() >= fValue.toLower())
                              return 1;
                         return 0;
                    }

                    // Dla typu wyliczeniowego
                    if(fAttrType->Type() == 5)
                    {
                         int i1 = fAttrType->AttributeValues()->indexOf(fAttrType->Value());
                         int i2 = fAttrType->AttributeValues()->indexOf(fValue);
                         if((i1 == -1) || (i2 == -1))
                         {
                              setCellError("Unrecognized type, type must be enumerative.");
                              return -1;
                         }
                         if(i1 >= i2)
                              return 1;
                         return 0;
                    }
               }

               // Operator >
               if(fOperator == 4)
               {
                    // Dla typu bool
                    if(fAttrType->Type() == 1)
                    {
                         if((fAttrType->Value() == "true") && (fValue == "false"))
                              return 1;
                         return 0;
                    }

                    // Dla typu liczbowego
                    if((fAttrType->Type() == 2) || (fAttrType->Type() == 3))
                    {
                         float a1 = fAttrType->Value().toFloat();
                         float a2 = fValue.toFloat();
                         if(a1 > a2)
                              return 1;
                         return 0;
                    }

                    // Dla typu symbolicznego
                    if(fAttrType->Type() == 4)
                    {
                         if(fAttrType->Value() > fValue)
                              return 1;
                         return 0;
                    }

                    // Dla typu wyliczeniowego
                    if(fAttrType->Type() == 5)
                    {
                         int i1 = fAttrType->AttributeValues()->indexOf(fAttrType->Value());
                         int i2 = fAttrType->AttributeValues()->indexOf(fValue);
                         if((i1 == -1) || (i2 == -1))
                         {
                              setCellError("Unrecognized type, type must be enumerative.");
                              return -1;
                         }
                         if(i1 > i2)
                              return 1;
                         return 0;
                    }
               }

               // Operator <=
               if(fOperator == 5)
               {
                    // Dla typu bool
                    if(fAttrType->Type() == 1)
                    {
                         if(fAttrType->Value() == "false")
                              return 1;
                         if(fAttrType->Value() == fValue)
                              return 1;
                         return 0;
                    }

                    // Dla typu liczbowego
                    if((fAttrType->Type() == 2) || (fAttrType->Type() == 3))
                    {
                         float a1 = fAttrType->Value().toFloat();
                         float a2 = fValue.toFloat();
                         if(a1 <= a2)
                              return 1;
                         return 0;
                    }

                    // Dla typu symbolicznego
                    if(fAttrType->Type() == 4)
                    {
                         if(fAttrType->Value() <= fValue)
                              return 1;
                         return 0;
                    }

                    // Dla typu wyliczeniowego
                    if(fAttrType->Type() == 5)
                    {
                         int i1 = fAttrType->AttributeValues()->indexOf(fAttrType->Value());
                         int i2 = fAttrType->AttributeValues()->indexOf(fValue);
                         if((i1 == -1) || (i2 == -1))
                         {
                              setCellError("Unrecognized type, type must be enumerative.");
                              return -1;
                         }
                         if(i1 <= i2)
                              return 1;
                         return 0;
                    }
               }

               // Operator <
               if(fOperator == 6)
               {
                    // Dla typu bool
                    if(fAttrType->Type() == 1)
                    {
                         if((fAttrType->Value() == "false") && (fValue == "true"))
                              return 1;
                         return 0;
                    }

                    // Dla typu liczbowego
                    if((fAttrType->Type() == 2) || (fAttrType->Type() == 3))
                    {
                         float a1 = fAttrType->Value().toFloat();
                         float a2 = fValue.toFloat();
                         if(a1 < a2)
                              return 1;
                         return 0;
                    }

                    // Dla typu symbolicznego
                    if(fAttrType->Type() == 4)
                    {
                         if(fAttrType->Value() < fValue)
                              return 1;
                         return 0;
                    }

                    // Dla typu wyliczeniowego
                    if(fAttrType->Type() == 5)
                    {
                         int i1 = fAttrType->AttributeValues()->indexOf(fAttrType->Value());
                         int i2 = fAttrType->AttributeValues()->indexOf(fValue);
                         if((i1 == -1) || (i2 == -1))
                         {
                              setCellError("Unrecognized type, type must be enumerative.");
                              return -1;
                         }
                         if(i1 < i2)
                              return 1;
                         return 0;
                    }
               }
          }//if(fValDefType == 1)

          
          // ---------------------------  Jezeli jest to set ---------------------------
          if(fValDefType == 2)
          {
               // context 1, set, nin
               if(fOperator == 7)
               {
                    //QString val_index = QString::number(fAttrType->AttributeValues()->indexOf(fAttrType->Value()));
                    int index = fAttrValue->indexOf(fAttrType->Value());

                    // Jezeli jest <any> oraz index > -1 to zwracamy 0, bo chocbysmy
                    // cokolwiek wstawili to zawsze bedzie
                    if((fAttrValue->indexOf(XTT_Attribute::_any_operator_)) && (index > -1))
                         return 0;

                    if(index > -1)
                         return 0;
                    if(index == -1)
                         return 1;
               }

               // context 1, set, in
               if(fOperator == 8)
               {
                    //QString val_index = QString::number(fAttrType->AttributeValues()->indexOf(fAttrType->Value()));
                    int index = fAttrValue->indexOf(fAttrType->Value());

                    // Jezeli jest any, to mozemy wstawic taka wartosc ze bedzie w zbiorze
                    if(fAttrValue->indexOf(XTT_Attribute::_any_operator_) > -1)
                         return 1;
                    
                    if(index == -1)
                         return 0;
                    if(index > -1)
                         return 1;
               }
          }//if(fValDefType == 2)

          // ---------------------------  Jezeli jest to range ---------------------------
          if(fValDefType == 3)
          {
               // Gdy bool
               if(fAttrType->Type() == 1)
               {
                    // context 1, range, nin
                    if(fOperator == 7)
                    {
                         // Jezeli w pierwszym jest true a w drugim false to juz nam nic nie pomoze
                         if((fBoundaries[1].toLower() == "false") && (fAttrType->Value().toLower() == "true"))
                              return 1;
                         if((fBoundaries[0].toLower() == "true") && (fAttrType->Value().toLower() == "false"))
                              return 1;

                         // Rozpatrzymy tylko przypadki kiedy sie zawiera
                         if((!fBrace[0]) && (fBoundaries[0].toLower() == "false") && (fAttrType->Value().toLower() == "false"))
                              return 0;
                         if((!fBrace[1]) && (fBoundaries[1].toLower() == "true") && (fAttrType->Value().toLower() == "true"))
                              return 0;
                         if((fBrace[1]) && (!fBrace[0]) && (fAttrType->Value().toLower() == "true") && ((fBoundaries[0].toLower() == XTT_Attribute::_any_operator_) || (fBoundaries[0].toLower() == "true")))
                              return 0;
                         if((fBrace[0]) && (!fBrace[1]) && (fAttrType->Value().toLower() == "false") && ((fBoundaries[1].toLower() == XTT_Attribute::_any_operator_) || (fBoundaries[1].toLower() == "false")))
                              return 0;
                         return 1;
                    }

                    // context 1, range, in
                    if(fOperator == 8)
                    {
                         // Jezeli w pierwszym jest true a w drugim false to juz nam nic nie pomoze
                         if((fBoundaries[1].toLower() == "false") && (fAttrType->Value().toLower() == "true"))
                              return 0;
                         if((fBoundaries[0].toLower() == "true") && (fAttrType->Value().toLower() == "false"))
                              return 0;

                              
                         // Rozpatrzymy tylko przypadki kiedy sie zawiera
                         if((!fBrace[0]) && (fBoundaries[0].toLower() == "false") && (fAttrType->Value().toLower() == "false"))
                              return 1;
                         if((!fBrace[1]) && (fBoundaries[1].toLower() == "true") && (fAttrType->Value().toLower() == "true"))
                              return 1;
                         if((fBrace[1]) && (!fBrace[0]) && (fAttrType->Value().toLower() == "true") && ((fBoundaries[0].toLower() == XTT_Attribute::_any_operator_) || (fBoundaries[0].toLower() == "true")))
                              return 1;
                         if((fBrace[0]) && (!fBrace[1]) && (fAttrType->Value().toLower() == "false") && ((fBoundaries[1].toLower() == XTT_Attribute::_any_operator_) || (fBoundaries[1].toLower() == "false")))
                              return 1;
                         return 0;
                    }
               }

               // Gdy float albo integer
               if((fAttrType->Type() == 2) || (fAttrType->Type() == 3))
               {
                    // context 1, range, nin
                    if(fOperator == 7)
                    {
                         bool ok1, ok2, valok;
                         float b[2];
                         float val1 = fAttrType->Value().toFloat(&valok);
                         if(!valok)
                         {
                              setCellError("Incorrect attribute " + fAttrType->Name() + " type.");
                              return -1;
                         }
                         
                         // Obsluga operatora <any>
                         if((fBoundaries[0].toLower() == XTT_Attribute::_any_operator_) && (fBoundaries[1].toLower() == XTT_Attribute::_any_operator_))
                              return 0;

                         // Kiedy lewy jest <any>
                         if((fBoundaries[0].toLower() == XTT_Attribute::_any_operator_) && (fBoundaries[1].toLower() != XTT_Attribute::_any_operator_))
                         {
                              b[1] = fBoundaries[1].toFloat(&ok2);
                              if(!ok2)
                              {
                                   setCellError("Right limit type is not numeric.");
                                   return -1;
                              }
                              if(((val1 < b[1]) && (fBrace[1])) || ((val1 <= b[1]) && (!fBrace[1])))
                                   return 0;
                              return 1;
                         }

                         // Kiedy prawy jest <any>
                         if((fBoundaries[1].toLower() == XTT_Attribute::_any_operator_) && (fBoundaries[0].toLower() != XTT_Attribute::_any_operator_))
                         {
                              b[0] = fBoundaries[0].toFloat(&ok1);
                              if(!ok1)
                              {
                                   setCellError("Left limit type is not numeric.");
                                   return -1;
                              }
                              if(((val1 > b[0]) && (fBrace[0])) || ((val1 >= b[0]) && (!fBrace[0])))
                                   return 0;
                              return 1;
                         }

                         b[0] = fBoundaries[0].toFloat(&ok1);
                         b[1] = fBoundaries[1].toFloat(&ok2);

                         if((!ok1) || (!ok2))
                         {
                              setCellError("Type is not numeric.");
                              return -1;
                         }

                         if((((val1 > b[0]) && (fBrace[0])) || ((val1 >= b[0]) && (!fBrace[0]))) &&
                            (((val1 < b[1]) && (fBrace[1])) || ((val1 <= b[1]) && (!fBrace[1]))))
                         {
                              return 0;
                         }
                         return 1;
                    }

                    // context 1, range, in
                    if(fOperator == 8)
                    {
                         bool ok1, ok2, valok;
                         float b[2];
                         float val1 = fAttrType->Value().toFloat(&valok);
                         if(!valok)
                         {
                              setCellError("Incorrect attribute " + fAttrType->Name() + " type.");
                              return -1;
                         }
                         
                         // Obsluga operatora <any>
                         if((fBoundaries[0].toLower() == XTT_Attribute::_any_operator_) && (fBoundaries[1].toLower() == XTT_Attribute::_any_operator_))
                              return 1;

                         // Kiedy lewy jest <any>
                         if((fBoundaries[0].toLower() == XTT_Attribute::_any_operator_) && (fBoundaries[1].toLower() != XTT_Attribute::_any_operator_))
                         {
                              b[1] = fBoundaries[1].toFloat(&ok2);
                              if(!ok2)
                              {
                                   setCellError("Right limit type is not numeric.");
                                   return -1;
                              }
                              if(((val1 < b[1]) && (fBrace[1])) || ((val1 <= b[1]) && (!fBrace[1])))
                                   return 1;
                              return 0;
                         }

                         // Kiedy prawy jest <any>
                         if((fBoundaries[1].toLower() == XTT_Attribute::_any_operator_) && (fBoundaries[0].toLower() != XTT_Attribute::_any_operator_))
                         {
                              b[0] = fBoundaries[0].toFloat(&ok1);
                              if(!ok1)
                              {
                                   setCellError("Left limit type is not numeric.");
                                   return -1;
                              }
                              if(((val1 > b[0]) && (fBrace[0])) || ((val1 >= b[0]) && (!fBrace[0])))
                                   return 1;
                              return 0;
                         }

                         b[0] = fBoundaries[0].toFloat(&ok1);
                         b[1] = fBoundaries[1].toFloat(&ok2);

                         if((!ok1) || (!ok2))
                         {
                              setCellError("Type is not numeric.");
                              return -1;
                         }

                         if((((val1 > b[0]) && (fBrace[0])) || ((val1 >= b[0]) && (!fBrace[0]))) &&
                            (((val1 < b[1]) && (fBrace[1])) || ((val1 <= b[1]) && (!fBrace[1]))))
                         {
                              return 1;
                         }
                         return 0;
                    }
               }
               
               // Gdy symbolic
               if(fAttrType->Type() == 4)
               {
                    // context 1, range, nin
                    if(fOperator == 7)
                    {
                         QString b[2];
                         QString val1 = fAttrType->Value();
                         
                         // Obsluga operatora <any>
                         if((fBoundaries[0].toLower() == XTT_Attribute::_any_operator_) && (fBoundaries[1].toLower() == XTT_Attribute::_any_operator_))
                              return 0;

                         // Kiedy lewy jest <any>
                         if((fBoundaries[0].toLower() == XTT_Attribute::_any_operator_) && (fBoundaries[1].toLower() != XTT_Attribute::_any_operator_))
                         {
                              b[1] = fBoundaries[1];
                              if(((val1 < b[1]) && (fBrace[1])) || ((val1 <= b[1]) && (!fBrace[1])))
                                   return 0;
                              return 1;
                         }

                         // Kiedy prawy jest <any>
                         if((fBoundaries[1].toLower() == XTT_Attribute::_any_operator_) && (fBoundaries[0].toLower() != XTT_Attribute::_any_operator_))
                         {
                              b[0] = fBoundaries[0];
                              if(((val1 > b[0]) && (fBrace[0])) || ((val1 >= b[0]) && (!fBrace[0])))
                                   return 0;
                              return 1;
                         }

                         b[0] = fBoundaries[0];
                         b[1] = fBoundaries[1];

                         if((((val1 > b[0]) && (fBrace[0])) || ((val1 >= b[0]) && (!fBrace[0]))) &&
                            (((val1 < b[1]) && (fBrace[1])) || ((val1 <= b[1]) && (!fBrace[1]))))
                         {
                              return 0;
                         }
                         return 1;
                    }

                    // context 1, range, in
                    if(fOperator == 8)
                    {
                         QString b[2];
                         QString val1 = fAttrType->Value();
                         
                         // Obsluga operatora <any>
                         if((fBoundaries[0].toLower() == XTT_Attribute::_any_operator_) && (fBoundaries[1].toLower() == XTT_Attribute::_any_operator_))
                              return 1;

                         // Kiedy lewy jest <any>
                         if((fBoundaries[0].toLower() == XTT_Attribute::_any_operator_) && (fBoundaries[1].toLower() != XTT_Attribute::_any_operator_))
                         {
                              b[1] = fBoundaries[1];
                              if(((val1 < b[1]) && (fBrace[1])) || ((val1 <= b[1]) && (!fBrace[1])))
                                   return 1;
                              return 0;
                         }

                         // Kiedy prawy jest <any>
                         if((fBoundaries[1].toLower() == XTT_Attribute::_any_operator_) && (fBoundaries[0].toLower() != XTT_Attribute::_any_operator_))
                         {
                              b[0] = fBoundaries[0];
                              if(((val1 > b[0]) && (fBrace[0])) || ((val1 >= b[0]) && (!fBrace[0])))
                                   return 1;
                              return 0;
                         }

                         b[0] = fBoundaries[0];
                         b[1] = fBoundaries[1];

                         if((((val1 > b[0]) && (fBrace[0])) || ((val1 >= b[0]) && (!fBrace[0]))) &&
                            (((val1 < b[1]) && (fBrace[1])) || ((val1 <= b[1]) && (!fBrace[1]))))
                         {
                              return 1;
                         }
                         return 0;
                    }
               }

               // Gdy enumerative
               if(fAttrType->Type() == 5)
               {
                    // context 1, range, nin
                    if(fOperator == 7)
                    {
                         //bool valok;
                         float b[2];
                         b[0] = fAttrType->AttributeValues()->indexOf(fBoundaries[0]);
                         b[1] = fAttrType->AttributeValues()->indexOf(fBoundaries[1]);
                         float val1 = fAttrType->AttributeValues()->indexOf(fAttrType->Value());
                         /*if(valok == -1)
                         {
                              setCellError("Incorrect attribute " + fAttrType->Name() + " type.");
                              return -1;
                         }*/
                         
                         // Obsluga operatora <any>
                         if((fBoundaries[0].toLower() == XTT_Attribute::_any_operator_) && (fBoundaries[1].toLower() == XTT_Attribute::_any_operator_))
                              return 0;

                         // Kiedy lewy jest <any>
                         if((fBoundaries[0].toLower() == XTT_Attribute::_any_operator_) && (fBoundaries[1].toLower() != XTT_Attribute::_any_operator_))
                         {
                              // Tak aby napewno wartosc byla mniejsza, gdyz dla typu wyliczeniowego najmniejsza moze byc 0
                              b[0] = 0;
                              if(b[1] == -1)
                              {
                                   setCellError("Right limit value is not correct.");
                                   return -1;
                              }
                              if((((val1 > b[0]) && (fBrace[0])) || ((val1 >= b[0]) && (!fBrace[0]))) &&
                                 (((val1 < b[1]) && (fBrace[1])) || ((val1 <= b[1]) && (!fBrace[1]))))
                              {
                                   return 0;
                              }
                              return 1;
                         }

                         // Kiedy prawy jest <any>
                         if((fBoundaries[1].toLower() == XTT_Attribute::_any_operator_) && (fBoundaries[0].toLower() != XTT_Attribute::_any_operator_))
                         {
                              // Tak aby napewno wartosc byla wieksza, gdyz dla typu wyliczeniowego najwieksza moze byc fAttrType->AttributeValues()->size()
                              b[1] = fAttrType->AttributeValues()->size()-1;
                              if(b[0] == -1)
                              {
                                   setCellError("Left limit value is not correct.");
                                   return -1;
                              }
                              if((((val1 > b[0]) && (fBrace[0])) || ((val1 >= b[0]) && (!fBrace[0]))) &&
                                 (((val1 < b[1]) && (fBrace[1])) || ((val1 <= b[1]) && (!fBrace[1]))))
                              {
                                   return 0;
                              }
                              return 1;
                         }

                         if((b[0] == -1) || (b[1] == -1))
                         {
                              setCellError("Type is not enumerative.");
                              return -1;
                         }

                         if((((val1 > b[0]) && (fBrace[0])) || ((val1 >= b[0]) && (!fBrace[0]))) &&
                            (((val1 < b[1]) && (fBrace[1])) || ((val1 <= b[1]) && (!fBrace[1]))))
                         {
                              return 0;
                         }
                         return 1;
                    }

                    // context 1, range, in
                    if(fOperator == 8)
                    {
                         //bool valok;
                         float b[2];
                         b[0] = fAttrType->AttributeValues()->indexOf(fBoundaries[0]);
                         b[1] = fAttrType->AttributeValues()->indexOf(fBoundaries[1]);
                         float val1 = fAttrType->AttributeValues()->indexOf(fAttrType->Value());
                         /*if(valok == -1)
                         {
                              setCellError("Incorrect attribute " + fAttrType->Name() + " type.");
                              return -1;
                         }*/
                         
                         // Obsluga operatora <any>
                         if((fBoundaries[0].toLower() == XTT_Attribute::_any_operator_) && (fBoundaries[1].toLower() == XTT_Attribute::_any_operator_))
                              return 1;

                         // Kiedy lewy jest <any>
                         if((fBoundaries[0].toLower() == XTT_Attribute::_any_operator_) && (fBoundaries[1].toLower() != XTT_Attribute::_any_operator_))
                         {
                              // Tak aby napewno wartosc byla mniejsza, gdyz dla typu wyliczeniowego najmniejsza moze byc 0
                              b[0] = 0;
                              if(b[1] == -1)
                              {
                                   setCellError("Right limit value is not correct.");
                                   return -1;
                              }
                              if((((val1 > b[0]) && (fBrace[0])) || ((val1 >= b[0]) && (!fBrace[0]))) &&
                                 (((val1 < b[1]) && (fBrace[1])) || ((val1 <= b[1]) && (!fBrace[1]))))
                              {
                                   return 1;
                              }
                              return 0;
                         }

                         // Kiedy prawy jest <any>
                         if((fBoundaries[1].toLower() == XTT_Attribute::_any_operator_) && (fBoundaries[0].toLower() != XTT_Attribute::_any_operator_))
                         {
                              // Tak aby napewno wartosc byla wieksza, gdyz dla typu wyliczeniowego najwieksza moze byc fAttrType->AttributeValues()->size()
                              b[1] = fAttrType->AttributeValues()->size()-1;
                              if(b[0] == -1)
                              {
                                   setCellError("Left limit value is not correct.");
                                   return -1;
                              }
                              if((((val1 > b[0]) && (fBrace[0])) || ((val1 >= b[0]) && (!fBrace[0]))) &&
                                 (((val1 < b[1]) && (fBrace[1])) || ((val1 <= b[1]) && (!fBrace[1]))))
                              {
                                   return 1;
                              }
                              return 0;
                         }

                         if((b[0] == -1) || (b[1] == -1))
                         {
                              setCellError("Type is not enumerative.");
                              return -1;
                         }

                         if((((val1 > b[0]) && (fBrace[0])) || ((val1 >= b[0]) && (!fBrace[0]))) &&
                            (((val1 < b[1]) && (fBrace[1])) || ((val1 <= b[1]) && (!fBrace[1]))))
                         {
                              return 1;
                         }
                         return 0;
                    }
               }
          }//if(fValDefType == 3)
     }// context

     if(fContext == 4)
     {
          // Gdy atomic
          if(fValDefType == 1)
          {
               // Ustawiamy wartosc
               if(!fAttrType->setValue(fValue))
			{
				setCellError("Cell execute error. The attribute value is incorrect.\nAttribute \"" + fAttrType->Name() + "\" domain: " + fAttrType->ConstraintsRepresentation() + "\nThe candidate for new value is \"" + fValue + "\".");
                    return -1;
			}
               return 1;
          }
     }

     return 1;
}
// -----------------------------------------------------------------------------

/**
 * Metoda klasy XTT_Cell zwracajaca jako string interpretzcje zapisu
 * @returns Zinterpretowany zapis jako string
*/
QString XTT_Cell::ValueRepresentation(void)
{
     QString res = "";

     // Zdefiniowanie operatorow, uwaga wazna kolejnosc ich podawania!!!
     QStringList opers;
     opers << "=" << "!=" << ">=" << ">" << "<=" << "<" << "nin" << "in";
     res = opers.at(fOperator-1) + " ";

     // Kiedy atomic
     if(fValDefType == 1)
     {
          res += fValue;
     }

     // Kiedy set
     if(fValDefType == 2)
     {
          res += "{";
          for(int i=0;i<fAttrValue->size();i++)
               res = res + fAttrValue->at(i) + "; ";
          if(fAttrValue->size() > 0)
               res = res.remove(res.length()-2,2);
          res = res + "}";
     }

     // Kiedy range
     if(fValDefType == 3)
     {
          if(fBrace[0])
               res += "(";
          if(!fBrace[0])
               res += "[";
          res += fBoundaries[0] + "; " + fBoundaries[1];
          if(fBrace[1])
               res += ")";
          if(!fBrace[1])
               res += "]";
     }

     return res;
}
// -----------------------------------------------------------------------------

/**
 * Metoda klasy XTT_Cell obliczajaca wartosc wyrazenia arytmetycznego
 * @param QString expression - Wyrazenie arytmetyczne w postaci stringu
 * @param int start_index - Index w wyrazeniu od ktorego nalezy zaczac przeszukiwanie
 * @returns Wynik dzialania w postaci stringu, jezeli blad to #NAN
*/
QString XTT_Cell::Calculate(QString& expression, int start_index)
{
     QString res = "#NAN";

     // Jezeli ciag zawiera <any> to zwracamy <any>
     if(expression.contains(XTT_Attribute::_any_operator_, Qt::CaseInsensitive))
     {
          expression =  XTT_Attribute::_any_operator_;
          return expression;
     }

     // Jezeli ciag zawiera <any> to zwracamy <any>
     if(expression.contains(XTT_Attribute::_not_definded_operator_, Qt::CaseInsensitive))
     {
          expression =  XTT_Attribute::_not_definded_operator_;
          return expression;
     }

     // Usuwanie spacji z wyrazenia
     if((start_index == 0) && (expression.contains(" ")))
          expression = expression.remove(expression.indexOf(" "), 1);

     // Wyszukiwanie nawiasow, jezeli znajdziemy nawias to uruchamiamy jeszcze
     // raz rekurencyjnie ta funckje
     for(int i=start_index;i<expression.length();i++)
     {
          // Kiedy na koncu stringu jest otwieranie nawiasu
          if((expression[i] == '(') && (i >= (expression.length()-1)))
               return res;
               
          if((expression[i] == '(') && (i < (expression.length()-1)))
          {
               if(Calculate(expression, i+1) == "#NAN")
                    return res;
          }
     }

     // Wyszukiwanie konca obliczen
     int end_index = -1;
     for(int i=start_index;i<expression.length();i++)
     {
          if(expression[i] == ')')
          {
               end_index = i;
               break;
          }
          if((i == (expression.length()-1)) && (start_index == 0))
          {
               end_index = i+1;
               break;
          }
     }

     // Jezeli nie znajdziemy nawiasu zamykajacego ale w nie pustym stringu, bo
     // pusty string tez moze byc rezultatem
     if((end_index == -1) && (expression != ""))
     {
          setCellError("There is a lack of right brace.");
          return "#NAN";
     }

     // Jezeli znajdzeimy to wyodrebniamy sobie substring dla latwiejsze analizy
     // Expr nie zawiera juz ani jednego nawiasu zamykajacego i otwierajacego
     QString expr = expression.mid(start_index, end_index-start_index);

     // Analiza nowego krotkiego wyrazenia

     // Definicja nazw funckji matematycznych jakie obsluguje program (PRIORITY HIGH)
     QStringList func_H;
     func_H << "sin" << "cos" << "tan" << "abs" << "root" << "^" << "!" << "log" << "mod";

     // Definicja nazw funckji matematycznych jakie obsluguje program (PRIORITY NORMAL)
     QStringList func_N;
     func_N << "*" << "/";

     // Definicja nazw funckji matematycznych jakie obsluguje program (PRIORITY LOW)
     QStringList func_L;
     func_L << "+" << "-";

     // *************** WYSZUKIWANIE FUNKCJI O NAJWYZSZYM POZIOMIE *************
     if(sin_func(func_H.at(0), expr) == "#NAN")
          return "#NAN";
     if(cos_func(func_H.at(1), expr) == "#NAN")
          return "#NAN";
     if(tan_func(func_H.at(2), expr) == "#NAN")
          return "#NAN";
     if(abs_func(func_H.at(3), expr) == "#NAN")
          return "#NAN";
     if(root_func(func_H.at(4), expr) == "#NAN")
          return "#NAN";
     if(pow_func(func_H.at(5), expr) == "#NAN")
          return "#NAN";
     if(fac_func(func_H.at(6), expr) == "#NAN")
          return "#NAN";
     if(log_func(func_H.at(7), expr) == "#NAN")
          return "#NAN";
     if(mod_func(func_H.at(8), expr) == "#NAN")
          return "#NAN";
     // ************************************************************************
     

     // ***************** WYSZUKIWANIE FUNKCJI O SREDNIM POZIOMIE **************
     int i1 = expr.indexOf(func_N.at(0));
     int i2 = expr.indexOf(func_N.at(1));
     while((i1 > -1) || (i2 > -1))
     {
          if(((i1 < i2) && (i1 != -1)) || ((i1 > -1) && (i2 == -1)))
               if(mul_func(func_N.at(0), expr) == "#NAN")
                    return "#NAN";
          if(((i2 < i1) && (i2 != -1)) || ((i2 > -1) && (i1 == -1)))
               if(div_func(func_N.at(1), expr) == "#NAN")
                    return "#NAN";

          // Ponowne wyszukanie dzialan
          i1 = expr.indexOf(func_N.at(0));
          i2 = expr.indexOf(func_N.at(1));
     }
     // ************************************************************************

     // ****************** WYSZUKIWANIE FUNKCJI O NISKIM POZIOMIE **************

     // Aby wykonywanie tych dzialan bylo od lewej do prawej nalezy sprawdzac
     // ktore wczesniej wystepuje:
     i1 = expr.indexOf(func_L.at(0));
     i2 = expr.indexOf(func_L.at(1), 1);
     while((i1 > -1) || (i2 > -1))
     {
          if(((i1 < i2) && (i1 != -1)) || ((i1 > -1) && (i2 == -1)))
               if(add_func(func_L.at(0), expr) == "#NAN")
                    return "#NAN";
          if(((i2 < i1) && (i2 != -1)) || ((i2 > -1) && (i1 == -1)))
               if(sub_func(func_L.at(1), expr) == "#NAN")
                    return "#NAN";

          // Ponowne wyszukanie dzialan
          i1 = expr.indexOf(func_L.at(0));
          i2 = expr.indexOf(func_L.at(1), 1);
     }
     // ************************************************************************


     // Kiedy sprawdzimy wszystkie funckje to pozniej jeszcze trzeba sprawdzic
     // czy nie jest to przypadkiem wyraz wolny czyli jest typu naszego
     // argumentu
     res = expr;

     // Jezeli jako wyrazenie zostala podana sama nazwa atrybutu, to funckje
     // nie zamienia jej na konkretna wartosc, dlatego trezba to zrobic to
     // tutaj, i tylko w momencie kiedy jestesmy w pierwszym wywolaniu czyli teraz
     // bedziemy konczyc obliczenia. W bardziej zagniezdzonych moga wystapic
     // nazwy zmiennych tymczasowych wiec zamiana na wartosc jest nie wskazana
     // bo zostanie wyswietlony komunikat o bledzie typu atrybutow
     XTT_Attribute* attr = AttributeGroups->Attribute(res);
	
	if((!fEnableAttr2AttrComp) && (attr != NULL))
	{	
		if(AttributeGroups->AttributeGroup(attr)->Name() != "tmp")
		{
			setCellError("Not allowed attribute to attribute comparsion.");
               return "#NAN";
		}
	}

     // Jezeli taki atrybut istnieje to:
     if((start_index == 0) && (attr != NULL))
     {
          // Sprawdzamy typ atrbutu czy jest taki sam jak naszego zrdlowego
          if(attr->Type() != fAttrType->Type())
          {
               setCellError("Inconsistent argument type \"" + res + "\".");
               return "#NAN";
          }

          if((TableParent()->IndexOfAttributeID(attr->Id()) == -1) && (AttributeGroups->CreatePath(attr) != "tmp."))
          {
               setCellError("Undefined argument \"" + res + "\" occurence.");
               return "#NAN";
          }

          // Jezeli poprawny to zapisujemy wartosc
          res = attr->Value();

          // Oprozniamy grupe atrybutow tymczasowych
          AttributeGroups->_IndexOfName("tmp")->Flush();
     }
     
     // Jezeli tutaj jestesmy to znaczy ze dzialanie sie udalo i teraz
     // trzeba zastapi analizowany pod string wynikiem
     if(start_index > 0)
          expression = expression.replace(start_index-1, end_index-start_index+2, res);
     if(start_index == 0)
          expression = res;
                               
     return res;
}
// -----------------------------------------------------------------------------

/**
 * Metoda obliczajaca z wyrazenia wszystkie sinusy
 * @param QString& expr - Referencja do wyrazenia
 * @returns Zwraca zmodyfikowane wyrazenie
*/
QString XTT_Cell::sin_func(QString fname, QString& expr)
{
     // Zmienna przechowujaca tymczasowy wynik dzialania funckji
     QString tmp_res = "";

     while(expr.contains(fname, Qt::CaseInsensitive))
     {
          // Index pierwszego znaku nazwy funckji
          int index = expr.indexOf(fname, 0, Qt::CaseInsensitive);
          
          // Przesuniecie na pierwszy znak argumentu
          int aindex = index + fname.length();

          // Wczytywanie argumentu
          QString arg = "";
          for(int i=aindex;((i<expr.length())&&(expr[i]!='+')&&((expr[i]!='-')||((expr[i]=='-')&&((i==aindex)||(expr[i-1]==' '))))&&(expr[i]!='*')&&(expr[i]!='/'));i++)
               arg = arg + expr[i];

          // Sprawdanie czy istnieje podany argumnet
          XTT_Attribute* attr = AttributeGroups->Attribute(arg);
		
		// Jezeli nie jest wlaczony tryb porownywania atrybutow to ustawiamy atrybut na NULL
		if((!fEnableAttr2AttrComp) && (attr != NULL))
		{	
			if(AttributeGroups->AttributeGroup(attr)->Name() != "tmp")
				attr = NULL;
		}

          // Kiedy nie jest to atrybut zdefiniowany w programie to probujemy go
          // konwertowac na liczbe typu float
          if(attr == NULL)
          {
               // Jezeli nie to sprawdzamy czy jest to liczba float
               // Sprawdzamy float bo w tym zbiorze zawiera sie integer
               bool is_float;
               float val = arg.toFloat(&is_float);

               // Jezeli tez nie liczba to mamy niezidentyfikowany argument
               if(!is_float)
               {
                    setCellError("Undefined argument \"" + arg + "\".");
                    return "#NAN";
               }
               // Jezeli jednak liczba to obliczamy wartosc funckji
               if(is_float)
                    tmp_res = QString::number(sin(val), 'f');
          }

          // Jezeli jest to konkretny argument to sprawdzamy czy jest typu liczbowego
          if(attr != NULL)
          {
               // Sprawdzanie czy atrybut istneieje w danej tabeli
               if(TableParent()->IndexOfAttributeID(attr->Id()) == -1)
               {
                    setCellError("Undefined argument \"" + arg + "\" occurence.");
                    return "#NAN";
               }

               if(attr->Value().toLower() == XTT_Attribute::_not_definded_operator_)
               {
                    setCellError("Not defined value in argument \"" + arg + "\".");
                    return "#NAN";
               }
               
               // Gdy jest typu float albo integer
               if((attr->Type() == 2) || (attr->Type() == 3))
                    tmp_res = QString::number(sin(attr->Value().toFloat()), 'f');
               if((attr->Type() != 2) && (attr->Type() != 3))
               {
                    setCellError("Incorrect attribute type \"" + arg + "\".");
                    return "#NAN";
               }
          }
          expr = expr.replace(index, fname.length()+arg.length(), tmp_res);
     }
     return expr;
}
// -----------------------------------------------------------------------------

/**
 * Metoda obliczajaca z wyrazenia wszystkie cosinusy
 * @param QString& expr - Referencja do wyrazenia
 * @returns Zwraca zmodyfikowane wyrazenie
*/
QString XTT_Cell::cos_func(QString fname, QString& expr)
{
     // Zmienna przechowujaca tymczasowy wynik dzialania funckji
     QString tmp_res = "";

     while(expr.contains(fname, Qt::CaseInsensitive))
     {
          // Index pierwszego znaku nazwy funckji
          int index = expr.indexOf(fname, 0, Qt::CaseInsensitive);
          
          // Przesuniecie na pierwszy znak argumentu
          int aindex = index + fname.length();

          // Wczytywanie argumentu
          QString arg = "";
          for(int i=aindex;((i<expr.length())&&(expr[i]!='+')&&((expr[i]!='-')||((expr[i]=='-')&&((i==aindex)||(expr[i-1]==' '))))&&(expr[i]!='*')&&(expr[i]!='/'));i++)
               arg = arg + expr[i];

          // Sprawdanie czy istnieje podany argumnet
          XTT_Attribute* attr = AttributeGroups->Attribute(arg);
		
		// Jezeli nie jest wlaczony tryb porownywania atrybutow to ustawiamy atrybut na NULL
		if((!fEnableAttr2AttrComp) && (attr != NULL))
		{	
			if(AttributeGroups->AttributeGroup(attr)->Name() != "tmp")
				attr = NULL;
		}

          // Kiedy nie jest to atrybut zdefiniowany w programie to probujemy go
          // konwertowac na liczbe typu float
          if(attr == NULL)
          {
               // Jezeli nie to sprawdzamy czy jest to liczba float
               // Sprawdzamy float bo w tym zbiorze zawiera sie integer
               bool is_float;
               float val = arg.toFloat(&is_float);

               // Jezeli tez nie liczba to mamy niezidentyfikowany argument
               if(!is_float)
               {
                    setCellError("Undefined argument \"" + arg + "\".");
                    return "#NAN";
               }
               // Jezeli jednak liczba to obliczamy wartosc funckji
               if(is_float)
                    tmp_res = QString::number(cos(val), 'f');
          }

          // Jezeli jest to konkretny argument to sprawdzamy czy jest typu liczbowego
          if(attr != NULL)
          {
               // Sprawdzanie czy atrybut istneieje w danej tabeli
               if(TableParent()->IndexOfAttributeID(attr->Id()) == -1)
               {
                    setCellError("Undefined argument \"" + arg + "\" occurence.");
                    return "#NAN";
               }

               if(attr->Value().toLower() == XTT_Attribute::_not_definded_operator_)
               {
                    setCellError("Not defined value in argument \"" + arg + "\".");
                    return "#NAN";
               }
               
               // Gdy jest typu float albo integer
               if((attr->Type() == 2) || (attr->Type() == 3))
                    tmp_res = QString::number(cos(attr->Value().toFloat()), 'f');
               if((attr->Type() != 2) && (attr->Type() != 3))
               {
                    setCellError("Incorrect attribute type \"" + arg + "\".");
                    return "#NAN";
               }
          }
          expr = expr.replace(index, fname.length()+arg.length(), tmp_res);
     }
     return expr;
}
// -----------------------------------------------------------------------------

/**
 * Metoda obliczajaca z wyrazenia wszystkie tangensy
 * @param QString& expr - Referencja do wyrazenia
 * @returns Zwraca zmodyfikowane wyrazenie
*/
QString XTT_Cell::tan_func(QString fname, QString& expr)
{
     // Zmienna przechowujaca tymczasowy wynik dzialania funckji
     QString tmp_res = "";

     while(expr.contains(fname, Qt::CaseInsensitive))
     {
          // Index pierwszego znaku nazwy funckji
          int index = expr.indexOf(fname, 0, Qt::CaseInsensitive);
          
          // Przesuniecie na pierwszy znak argumentu
          int aindex = index + fname.length();

          // Wczytywanie argumentu
          QString arg = "";
          for(int i=aindex;((i<expr.length())&&(expr[i]!='+')&&((expr[i]!='-')||((expr[i]=='-')&&((i==aindex)||(expr[i-1]==' '))))&&(expr[i]!='*')&&(expr[i]!='/'));i++)
               arg = arg + expr[i];

          // Sprawdanie czy istnieje podany argumnet
          XTT_Attribute* attr = AttributeGroups->Attribute(arg);
		
		// Jezeli nie jest wlaczony tryb porownywania atrybutow to ustawiamy atrybut na NULL
		if((!fEnableAttr2AttrComp) && (attr != NULL))
		{	
			if(AttributeGroups->AttributeGroup(attr)->Name() != "tmp")
				attr = NULL;
		}

          // Kiedy nie jest to atrybut zdefiniowany w programie to probujemy go
          // konwertowac na liczbe typu float
          if(attr == NULL)
          {
               // Jezeli nie to sprawdzamy czy jest to liczba float
               // Sprawdzamy float bo w tym zbiorze zawiera sie integer
               bool is_float;
               float val = arg.toFloat(&is_float);

               // Jezeli tez nie liczba to mamy niezidentyfikowany argument
               if(!is_float)
               {
                    setCellError("Undefined argument \"" + arg + "\".");
                    return "#NAN";
               }
               // Jezeli jednak liczba to obliczamy wartosc funckji
               if(is_float)
                    tmp_res = QString::number(tan(val), 'f');
          }

          // Jezeli jest to konkretny argument to sprawdzamy czy jest typu liczbowego
          if(attr != NULL)
          {
               // Sprawdzanie czy atrybut istneieje w danej tabeli
               if(TableParent()->IndexOfAttributeID(attr->Id()) == -1)
               {
                    setCellError("Undefined argument \"" + arg + "\" occurence.");
                    return "#NAN";
               }
               
               if(attr->Value().toLower() == XTT_Attribute::_not_definded_operator_)
               {
                    setCellError("Not defined value in argument \"" + arg + "\".");
                    return "#NAN";
               }
               
               // Gdy jest typu float albo integer
               if((attr->Type() == 2) || (attr->Type() == 3))
                    tmp_res = QString::number(tan(attr->Value().toFloat()), 'f');
               if((attr->Type() != 2) && (attr->Type() != 3))
               {
                    setCellError("Incorrect attribute type \"" + arg + "\".");
                    return "#NAN";
               }
          }
          expr = expr.replace(index, fname.length()+arg.length(), tmp_res);
     }
     return expr;
}
// -----------------------------------------------------------------------------

/**
 * Metoda obliczajaca z wyrazenia wszystkie wartosci bezwzgledne
 * @param QString& expr - Referencja do wyrazenia
 * @returns Zwraca zmodyfikowane wyrazenie
*/
QString XTT_Cell::abs_func(QString fname, QString& expr)
{
     // Zmienna przechowujaca tymczasowy wynik dzialania funckji
     QString tmp_res = "";

     while(expr.contains(fname, Qt::CaseInsensitive))
     {
          // Index pierwszego znaku nazwy funckji
          int index = expr.indexOf(fname, 0, Qt::CaseInsensitive);
          
          // Przesuniecie na pierwszy znak argumentu
          int aindex = index + fname.length();

          // Wczytywanie argumentu
          QString arg = "";
          for(int i=aindex;((i<expr.length())&&(expr[i]!='+')&&((expr[i]!='-')||((expr[i]=='-')&&((i==aindex)||(expr[i-1]==' '))))&&(expr[i]!='*')&&(expr[i]!='/'));i++)
               arg = arg + expr[i];

          // Sprawdanie czy istnieje podany argumnet
          XTT_Attribute* attr = AttributeGroups->Attribute(arg);
		
		// Jezeli nie jest wlaczony tryb porownywania atrybutow to ustawiamy atrybut na NULL
		if((!fEnableAttr2AttrComp) && (attr != NULL))
		{	
			if(AttributeGroups->AttributeGroup(attr)->Name() != "tmp")
				attr = NULL;
		}

          // Kiedy nie jest to atrybut zdefiniowany w programie to probujemy go
          // konwertowac na liczbe typu float
          if(attr == NULL)
          {
               // Jezeli nie to sprawdzamy czy jest to liczba float
               // Sprawdzamy float bo w tym zbiorze zawiera sie integer
               bool is_float;
               float val = arg.toFloat(&is_float);

               // Jezeli tez nie liczba to mamy niezidentyfikowany argument
               if(!is_float)
               {
                    setCellError("Undefined argument \"" + arg + "\".");
                    return "#NAN";
               }
               // Jezeli jednak liczba to obliczamy wartosc funckji
               if(is_float)
               {
                    if(val < .0)
                         val = -val;
                    tmp_res = QString::number(val);
               }
          }

          // Jezeli jest to konkretny argument to sprawdzamy czy jest typu liczbowego
          if(attr != NULL)
          {
               // Sprawdzanie czy atrybut istneieje w danej tabeli
               if(TableParent()->IndexOfAttributeID(attr->Id()) == -1)
               {
                    setCellError("Undefined argument \"" + arg + "\" occurence.");
                    return "#NAN";
               }
               
               if(attr->Value().toLower() == XTT_Attribute::_not_definded_operator_)
               {
                    setCellError("Not defined value in argument \"" + arg + "\".");
                    return "#NAN";
               }
               
               // Gdy jest typu integer
               if(attr->Type() == 2)
               {
                    int val = attr->Value().toInt();
                    if(val < 0)
                         val = -val;
                    tmp_res = QString::number(val);
               }

               // Gdy jest typu float
               if(attr->Type() == 3)
               {
                    float val = attr->Value().toFloat();
                    if(val < .0)
                         val = -val;
                    tmp_res = QString::number((double)val, 'f', 6);
               }
               if((attr->Type() != 2) && (attr->Type() != 3))
               {
                    setCellError("Incorrect attribute type \"" + arg + "\".");
                    return "#NAN";
               }
          }
          expr = expr.replace(index, fname.length()+arg.length(), tmp_res);
     }
     return expr;
}
// -----------------------------------------------------------------------------

/**
 * Metoda obliczajaca z wyrazenia wszystkie pierwiastki dowolnego stopnia
 * @param QString& expr - Referencja do wyrazenia
 * @returns Zwraca zmodyfikowane wyrazenie
*/
QString XTT_Cell::root_func(QString fname, QString& expr)
{
     // Zmienna przechowujaca tymczasowy wynik dzialania funckji
     QString tmp_res = "";

     // Wartosci argumentow jako stringi
     QString arg1val = "";
     QString arg2val = "";
     
     while(expr.contains(fname, Qt::CaseInsensitive))
     {
          // Index pierwszego znaku nazwy funckji
          int index = expr.indexOf(fname, 0, Qt::CaseInsensitive);
          
          // Przesuniecie na pierwszy znak argumentu
          int aindex = index + fname.length();

          // Wczytywanie argumentu
          QString arg = "";
          for(int i=aindex;((i<expr.length())&&(expr[i]!='+')&&((expr[i]!='-')||((expr[i]=='-')&&(((i==aindex)||(expr[i-1]==' '))||((i>0)&&((expr[i-1]==',')||(expr[i-1]==' '))))))&&(expr[i]!='*')&&(expr[i]!='/'));i++)
               arg = arg + expr[i];

          // Poniewaz ta funckja przyjmuje dwa argumenty oddzielone przecinkiem
          // to najpierw nalezy sprawdzic czy sa one podane i sa ok
          QStringList arglist = arg.split(",", QString::SkipEmptyParts);
          if(arglist.size() != 2)
          {
               setCellError("Incorect argument count in function sqrt/2.");
               return "#NAN";
          }

          // Sprawdanie czy istnieje podany argumnet
          XTT_Attribute* attr1 = AttributeGroups->Attribute(arglist.at(0));
          XTT_Attribute* attr2 = AttributeGroups->Attribute(arglist.at(1));
		
		// Jezeli nie jest wlaczony tryb porownywania atrybutow to ustawiamy atrybut na NULL
		if((!fEnableAttr2AttrComp) && (attr1 != NULL))
		{	
			if(AttributeGroups->AttributeGroup(attr1)->Name() != "tmp")
				attr1 = NULL;
		}
		if((!fEnableAttr2AttrComp) && (attr2 != NULL))
		{	
			if(AttributeGroups->AttributeGroup(attr2)->Name() != "tmp")
				attr2 = NULL;
		}

          // Kiedy nie jest to atrybut zdefiniowany w programie to probujemy go
          // konwertowac na liczbe typu float
          if(attr1 == NULL)
          {
               // Jezeli nie to sprawdzamy czy jest to liczba float
               // Sprawdzamy float bo w tym zbiorze zawiera sie integer
               bool is_float;
               arglist.at(0).toFloat(&is_float);

               // Jezeli tez nie liczba to mamy niezidentyfikowany argument
               if(!is_float)
               {
                    setCellError("Undefined argument \"" + arglist.at(0) + "\".");
                    return "#NAN";
               }
               // Jezeli jednak liczba to obliczamy wartosc funckji
               if(is_float)
                    arg1val = arglist.at(0);
          }

          // Jezeli jest to konkretny argument to sprawdzamy czy jest typu liczbowego
          if(attr1 != NULL)
          {
               // Sprawdzanie czy atrybut istneieje w danej tabeli
               if(TableParent()->IndexOfAttributeID(attr1->Id()) == -1)
               {
                    setCellError("Undefined argument \"" + arglist.at(0) + "\" occurence.");
                    return "#NAN";
               }

               if(attr1->Value().toLower() == XTT_Attribute::_not_definded_operator_)
               {
                    setCellError("Not defined value in argument \"" + arglist.at(0) + "\".");
                    return "#NAN";
               }
               
               // Gdy jest typu float albo integer
               if((attr1->Type() == 2) || (attr1->Type() == 3))
                    arg1val = attr1->Value();
               if((attr1->Type() != 2) && (attr1->Type() != 3))
               {
                    setCellError("Incorrect attribute type \"" + arglist.at(0) + "\".");
                    return "#NAN";
               }
          }

          // Kiedy nie jest to atrybut zdefiniowany w programie to probujemy go
          // konwertowac na liczbe typu float
          if(attr2 == NULL)
          {
               // Jezeli nie to sprawdzamy czy jest to liczba float
               // Sprawdzamy float bo w tym zbiorze zawiera sie integer
               bool is_float;
               arglist.at(1).toFloat(&is_float);

               // Jezeli tez nie liczba to mamy niezidentyfikowany argument
               if(!is_float)
               {
                    setCellError("Undefined argument \"" + arglist.at(1) + "\".");
                    return "#NAN";
               }
               // Jezeli jednak liczba to obliczamy wartosc funckji
               if(is_float)
                    arg2val = arglist.at(1);
          }

          // Jezeli jest to konkretny argument to sprawdzamy czy jest typu liczbowego
          if(attr2 != NULL)
          {
               // Sprawdzanie czy atrybut istneieje w danej tabeli
               if(TableParent()->IndexOfAttributeID(attr2->Id()) == -1)
               {
                    setCellError("Undefined argument \"" + arglist.at(1) + "\" occurence.");
                    return "#NAN";
               }

               if(attr2->Value().toLower() == XTT_Attribute::_not_definded_operator_)
               {
                    setCellError("Not defined value in argument \"" + arglist.at(1) + "\".");
                    return "#NAN";
               }
               
               // Gdy jest typu float albo integer
               if((attr2->Type() == 2) || (attr2->Type() == 3))
                    arg2val = attr2->Value();
               if((attr2->Type() != 2) && (attr2->Type() != 3))
               {
                    setCellError("Incorrect attribute type \"" + arglist.at(1) + "\".");
                    return "#NAN";
               }
          }

          float a1 = arg1val.toFloat();                // Stopien pierwiastka
          float a2 = arg2val.toFloat();                // Liczba podpierwiastkowa

          // Sprawdzanie czy stopien nie jest rowny zero
          if(a1 == .0)
          {
               setCellError("Degree of root can't be equal zero.");
               return "#NAN";
          }

          // Sprawdzanie czy liczba podpierwiastkowa nie jest mniejsza od zera
          if(a2 < .0)
          {
               setCellError("Base of root can't be less than zero.");
               return "#NAN";
          }

          float root = pow(a2, 1./a1);
          tmp_res = QString::number(root, 'f');

          expr = expr.replace(index, fname.length()+arg.length(), tmp_res);
     }
     return expr;
}
// -----------------------------------------------------------------------------

/**
 * Metoda obliczajaca z wyrazenia wszystkie potegi
 * @param QString& expr - Referencja do wyrazenia
 * @returns Zwraca zmodyfikowane wyrazenie
*/
QString XTT_Cell::pow_func(QString fname, QString& expr)
{
     // Zmienna przechowujaca tymczasowy wynik dzialania funckji
     QString tmp_res = "";

     // Wartosci argumentow jako stringi
     QString arg1val = "";
     QString arg2val = "";

     // Dlugosci wczytanych argumentow
     int a1len;
     int a2len;
     
     while(expr.contains(fname, Qt::CaseInsensitive))
     {
          // Index pierwszego znaku nazwy funckji
          int index = expr.indexOf(fname, 0, Qt::CaseInsensitive);
          
          // Przesuniecie na pierwszy znak argumentu
          int aindex = index + fname.length();

          // Wczytywanie lewego argumentu
          QString stop_char = "+-/*";
          QString arg1 = "";
          if(index > 0)
               for(int i=index-1;((i>=0)&&(expr[i]!='+')&&((expr[i]!='-')||((expr[i]=='-')&&((i==0)||((i>0)&&(stop_char.contains(expr[i-1]))))))&&(expr[i]!='*')&&(expr[i]!='/'));i--)
                    arg1 = expr[i] + arg1;

          // Wczytywanie prawego argumentu
          QString arg2 = "";
          for(int i=aindex;((i<expr.length())&&(expr[i]!='+')&&((expr[i]!='-')||((expr[i]=='-')&&((i==aindex)||(expr[i-1]==' '))))&&(expr[i]!='*')&&(expr[i]!='/'));i++)
               arg2 = arg2 + expr[i];

          // Zapisujemy dlugosci wczytanych atrbutow
          a1len = arg1.length();
          a2len = arg2.length();

          // Usuwanie spacji w wartosciach
          for(;arg1.contains(" ");) arg1 = arg1.remove(arg1.indexOf(" "), 1);
          for(;arg2.contains(" ");) arg2 = arg2.remove(arg2.indexOf(" "), 1);

          // Sprawdanie czy istnieje podany argumnet
          XTT_Attribute* attr1 = AttributeGroups->Attribute(arg1);
          XTT_Attribute* attr2 = AttributeGroups->Attribute(arg2);
		
		// Jezeli nie jest wlaczony tryb porownywania atrybutow to ustawiamy atrybut na NULL
		if((!fEnableAttr2AttrComp) && (attr1 != NULL))
		{	
			if(AttributeGroups->AttributeGroup(attr1)->Name() != "tmp")
				attr1 = NULL;
		}
		if((!fEnableAttr2AttrComp) && (attr2 != NULL))
		{	
			if(AttributeGroups->AttributeGroup(attr2)->Name() != "tmp")
				attr2 = NULL;
		}

          // Kiedy nie jest to atrybut zdefiniowany w programie to probujemy go
          // konwertowac na liczbe typu float
          if(attr1 == NULL)
          {
               // Jezeli nie to sprawdzamy czy jest to liczba float
               // Sprawdzamy float bo w tym zbiorze zawiera sie integer
               bool is_float;
               arg1.toFloat(&is_float);

               // Jezeli tez nie liczba to mamy niezidentyfikowany argument
               if(!is_float)
               {
                    setCellError("Undefined argument \"" + arg1 + "\".");
                    return "#NAN";
               }
               // Jezeli jednak liczba to obliczamy wartosc funckji
               if(is_float)
                    arg1val = arg1;
          }

          // Jezeli jest to konkretny argument to sprawdzamy czy jest typu liczbowego
          if(attr1 != NULL)
          {
               // Sprawdzanie czy atrybut istneieje w danej tabeli
               if(TableParent()->IndexOfAttributeID(attr1->Id()) == -1)
               {
                    setCellError("Undefined argument \"" + arg1 + "\" occurence.");
                    return "#NAN";
               }

               if(attr1->Value().toLower() == XTT_Attribute::_not_definded_operator_)
               {
                    setCellError("Not defined value in argument \"" + arg1 + "\".");
                    return "#NAN";
               }
               
               // Gdy jest typu float albo integer
               if((attr1->Type() == 2) || (attr1->Type() == 3))
                    arg1val = attr1->Value();
               if((attr1->Type() != 2) && (attr1->Type() != 3))
               {
                    setCellError("Incorrect attribute type \"" + arg1 + "\".");
                    return "#NAN";
               }
          }

          // Kiedy nie jest to atrybut zdefiniowany w programie to probujemy go
          // konwertowac na liczbe typu float
          if(attr2 == NULL)
          {
               // Jezeli nie to sprawdzamy czy jest to liczba float
               // Sprawdzamy float bo w tym zbiorze zawiera sie integer
               bool is_float;
               arg2.toFloat(&is_float);

               // Jezeli tez nie liczba to mamy niezidentyfikowany argument
               if(!is_float)
               {
                    setCellError("Undefined argument \"" + arg2 + "\".");
                    return "#NAN";
               }
               // Jezeli jednak liczba to obliczamy wartosc funckji
               if(is_float)
                    arg2val = arg2;
          }

          // Jezeli jest to konkretny argument to sprawdzamy czy jest typu liczbowego
          if(attr2 != NULL)
          {
               // Sprawdzanie czy atrybut istneieje w danej tabeli
               if(TableParent()->IndexOfAttributeID(attr2->Id()) == -1)
               {
                    setCellError("Undefined argument \"" + arg2 + "\" occurence.");
                    return "#NAN";
               }

               if(attr2->Value().toLower() == XTT_Attribute::_not_definded_operator_)
               {
                    setCellError("Not defined value in argument \"" + arg2 + "\".");
                    return "#NAN";
               }

               // Gdy jest typu float albo integer
               if((attr2->Type() == 2) || (attr2->Type() == 3))
                    arg2val = attr2->Value();
               if((attr2->Type() != 2) && (attr2->Type() != 3))
               {
                    setCellError("Incorrect attribute type \"" + arg2 + "\".");
                    return "#NAN";
               }
          }

          float a1 = arg1val.toFloat();                // Podstawa potegi
          float a2 = arg2val.toFloat();                // Wykladnik potegi

          float powres = pow(a1, a2);
          tmp_res = QString::number(powres, 'f');
          expr = expr.replace(index-a1len, fname.length()+a1len+a2len, tmp_res);
          //expr = expr.replace(index-arg1.length(), fname.length()+arg1.length()+arg2.length(), tmp_res);
     }
     return expr;
}
// -----------------------------------------------------------------------------

/**
 * Metoda obliczajaca z wyrazenia wszystkie dzielenia modulo
 * @param QString& expr - Referencja do wyrazenia
 * @returns Zwraca zmodyfikowane wyrazenie
*/
QString XTT_Cell::mod_func(QString fname, QString& expr)
{
     //QMessageBox::critical(NULL, "XTT Editor", "Running mod function with fname = " + fname + ", expr = " + expr, QMessageBox::Ok);

     // Zmienna przechowujaca tymczasowy wynik dzialania funckji
     QString tmp_res = "";

     // Wartosci argumentow jako stringi
     QString arg1val = "";
     QString arg2val = "";
     
     while(expr.contains(fname, Qt::CaseInsensitive))
     {
          // Index pierwszego znaku nazwy funckji
          int index = expr.indexOf(fname, 0, Qt::CaseInsensitive);
          
          // Przesuniecie na pierwszy znak argumentu
          int aindex = index + fname.length();

          // Wczytywanie argumentu
          QString arg = "";
          for(int i=aindex;((i<expr.length())&&(expr[i]!='+')&&((expr[i]!='-')||((expr[i]=='-')&&(((i==aindex)||(expr[i-1]==' '))||((i>0)&&((expr[i-1]==',')||(expr[i-1]==' '))))))&&(expr[i]!='*')&&(expr[i]!='/'));i++)
               arg = arg + expr[i];

          // Poniewaz ta funckja przyjmuje dwa argumenty oddzielone przecinkiem
          // to najpierw nalezy sprawdzic czy sa one podane i sa ok
          QStringList arglist = arg.split(",", QString::SkipEmptyParts);
          if(arglist.size() != 2)
          {
               setCellError("Incorect argument count in function modulo/2.");
               return "#NAN";
          }

          // Sprawdanie czy istnieje podany argumnet
          XTT_Attribute* attr1 = AttributeGroups->Attribute(arglist.at(0));
          XTT_Attribute* attr2 = AttributeGroups->Attribute(arglist.at(1));
		
		// Jezeli nie jest wlaczony tryb porownywania atrybutow to ustawiamy atrybut na NULL
		if((!fEnableAttr2AttrComp) && (attr1 != NULL))
		{	
			if(AttributeGroups->AttributeGroup(attr1)->Name() != "tmp")
				attr1 = NULL;
		}
		if((!fEnableAttr2AttrComp) && (attr2 != NULL))
		{	
			if(AttributeGroups->AttributeGroup(attr2)->Name() != "tmp")
				attr2 = NULL;
		}

          // Kiedy nie jest to atrybut zdefiniowany w programie to probujemy go
          // konwertowac na liczbe typu float
          if(attr1 == NULL)
          {
               // Jezeli nie to sprawdzamy czy jest to liczba float
               // Sprawdzamy float bo w tym zbiorze zawiera sie integer
               bool is_int;
               arglist.at(0).toInt(&is_int);

               // Jezeli tez nie liczba to mamy niezidentyfikowany argument
               if(!is_int)
               {
                    setCellError("Incorrect attribute type \"" + arglist.at(0) + "\".\nModulo function require INTEGER argument type.");
                    return "#NAN";
               }
               // Jezeli jednak liczba to obliczamy wartosc funckji
               if(is_int)
                    arg1val = arglist.at(0);
          }

          // Jezeli jest to konkretny argument to sprawdzamy czy jest typu liczbowego
          if(attr1 != NULL)
          {
               // Sprawdzanie czy atrybut istneieje w danej tabeli
               if(TableParent()->IndexOfAttributeID(attr1->Id()) == -1)
               {
                    setCellError("Undefined argument \"" + arglist.at(0) + "\" occurence.");
                    return "#NAN";
               }

               if(attr1->Value().toLower() == XTT_Attribute::_not_definded_operator_)
               {
                    setCellError("Not defined value in argument \"" + arglist.at(0) + "\".");
                    return "#NAN";
               }
               
               // Gdy jest typu float albo integer
               if(attr1->Type() == 2)
                    arg1val = attr1->Value();
               if(attr1->Type() != 2)
               {
                    setCellError("Incorrect attribute type \"" + arglist.at(0) + "\".\nModulo function require INTEGER argument type.");
                    return "#NAN";
               }
          }

          // Kiedy nie jest to atrybut zdefiniowany w programie to probujemy go
          // konwertowac na liczbe typu float
          if(attr2 == NULL)
          {
               // Jezeli nie to sprawdzamy czy jest to liczba float
               // Sprawdzamy float bo w tym zbiorze zawiera sie integer
               bool is_int;
               arglist.at(1).toInt(&is_int);

               // Jezeli tez nie liczba to mamy niezidentyfikowany argument
               if(!is_int)
               {
                    setCellError("Incorrect attribute type \"" + arglist.at(1) + "\".\nModulo function require INTEGER argument type.");
                    return "#NAN";
               }
               // Jezeli jednak liczba to obliczamy wartosc funckji
               if(is_int)
                    arg2val = arglist.at(1);
          }

          // Jezeli jest to konkretny argument to sprawdzamy czy jest typu liczbowego
          if(attr2 != NULL)
          {
               // Sprawdzanie czy atrybut istneieje w danej tabeli
               if(TableParent()->IndexOfAttributeID(attr2->Id()) == -1)
               {
                    setCellError("Undefined argument \"" + arglist.at(1) + "\" occurence.");
                    return "#NAN";
               }

               if(attr2->Value().toLower() == XTT_Attribute::_not_definded_operator_)
               {
                    setCellError("Not defined value in argument \"" + arglist.at(1) + "\".");
                    return "#NAN";
               }

               // Gdy jest typu  integer
               if(attr2->Type() == 2)
                    arg2val = attr2->Value();
               if(attr2->Type() != 2)
               {
                    setCellError("Incorrect attribute type \"" + arglist.at(1) + "\".\nModulo function require INTEGER argument type.");
                    return "#NAN";
               }
          }

          int a1 = arg1val.toInt();                // Podstawa potegi
          int a2 = arg2val.toInt();                // Wykladnik potegi

          int res = a1%a2;
          tmp_res = QString::number((double)res, 'f', 0);
          expr = expr.replace(index, fname.length()+arg.length(), tmp_res);
          //expr = expr.replace(index-arg1.length(), fname.length()+arg1.length()+arg2.length(), tmp_res);
     }
     return expr;
}
// -----------------------------------------------------------------------------

/**
 * Metoda obliczajaca z wyrazenia wszystkie silnie
 * @param QString& expr - Referencja do wyrazenia
 * @returns Zwraca zmodyfikowane wyrazenie
*/
QString XTT_Cell::fac_func(QString fname, QString& expr)
{
     // Zmienna przechowujaca tymczasowy wynik dzialania funckji
     QString tmp_res = "";

     // Wartosci argumentow jako stringi
     QString arg1val = "";

     while(expr.contains(fname, Qt::CaseInsensitive))
     {
          // Index pierwszego znaku nazwy funckji
          int index = expr.indexOf(fname, 0, Qt::CaseInsensitive);

          // Wczytywanie lewego argumentu
          QString stop_char = "+-/*";
          QString arg1 = "";
          if(index > 0)
               for(int i=index-1;((i>=0)&&(expr[i]!='+')&&((expr[i]!='-')||((expr[i]=='-')&&((i==0)||((i>0)&&(stop_char.contains(expr[i-1]))))))&&(expr[i]!='*')&&(expr[i]!='/'));i--)
                    arg1 = expr[i] + arg1;

          // Sprawdanie czy istnieje podany argumnet
          XTT_Attribute* attr1 = AttributeGroups->Attribute(arg1);
		
		// Jezeli nie jest wlaczony tryb porownywania atrybutow to ustawiamy atrybut na NULL
		if((!fEnableAttr2AttrComp) && (attr1 != NULL))
		{	
			if(AttributeGroups->AttributeGroup(attr1)->Name() != "tmp")
				attr1 = NULL;
		}

          // Kiedy nie jest to atrybut zdefiniowany w programie to probujemy go
          // konwertowac na liczbe typu float
          if(attr1 == NULL)
          {
               // Jezeli nie to sprawdzamy czy jest to liczba float
               // Sprawdzamy float bo w tym zbiorze zawiera sie integer
               bool is_int;
               arg1.toFloat(&is_int);

               // Jezeli tez nie liczba to mamy niezidentyfikowany argument
               if(!is_int)
               {
                    setCellError("Undefined argument \"" + arg1 + "\".");
                    return "#NAN";
               }
               // Jezeli jednak liczba to obliczamy wartosc funckji
               if(is_int)
                    arg1val = arg1;
          }

          // Jezeli jest to konkretny argument to sprawdzamy czy jest typu liczbowego
          if(attr1 != NULL)
          {
               // Sprawdzanie czy atrybut istneieje w danej tabeli
               if(TableParent()->IndexOfAttributeID(attr1->Id()) == -1)
               {
                    setCellError("Undefined argument \"" + arg1 + "\" occurence.");
                    return "#NAN";
               }

               if(attr1->Value().toLower() == XTT_Attribute::_not_definded_operator_)
               {
                    setCellError("Not defined value in argument \"" + arg1 + "\".");
                    return "#NAN";
               }
               
               // Gdy jest typu float albo integer
               if(attr1->Type() == 2)
                    arg1val = attr1->Value();
               if(attr1->Type() != 2)
               {
                    setCellError("Incorrect attribute type \"" + arg1 + "\".");
                    return "#NAN";
               }
          }



          int a1 = arg1val.toInt();                // Podstawa potegi
          if(a1 < 0)
          {
               setCellError("Factorial argument \"" + arg1 + "\" can't be less than zero.");
               return "#NAN";
          }

          unsigned int fac = 1;

          for(int i=1;i<=a1;i++)
               fac *= i;

          tmp_res = QString::number((double)fac, 'f', 0);

          expr = expr.replace(index-arg1.length(), fname.length()+arg1.length(), tmp_res);
     }
     return expr;
}
// -----------------------------------------------------------------------------

/**
 * Metoda obliczajaca z wyrazenia wszystkie logarytmy
 * @param QString& expr - Referencja do wyrazenia
 * @returns Zwraca zmodyfikowane wyrazenie
*/
QString XTT_Cell::log_func(QString fname, QString& expr)
{
     // Zmienna przechowujaca tymczasowy wynik dzialania funckji
     QString tmp_res = "";

     while(expr.contains(fname, Qt::CaseInsensitive))
     {
          // Index pierwszego znaku nazwy funckji
          int index = expr.indexOf(fname, 0, Qt::CaseInsensitive);
          
          // Przesuniecie na pierwszy znak argumentu
          int aindex = index + fname.length();

          // Wczytywanie argumentu
          QString arg = "";
          for(int i=aindex;((i<expr.length())&&(expr[i]!='+')&&((expr[i]!='-')||((expr[i]=='-')&&((i==aindex)||(expr[i-1]==' '))))&&(expr[i]!='*')&&(expr[i]!='/'));i++)
               arg = arg + expr[i];

          // Sprawdanie czy istnieje podany argumnet
          XTT_Attribute* attr = AttributeGroups->Attribute(arg);
		
		// Jezeli nie jest wlaczony tryb porownywania atrybutow to ustawiamy atrybut na NULL
		if((!fEnableAttr2AttrComp) && (attr != NULL))
		{	
			if(AttributeGroups->AttributeGroup(attr)->Name() != "tmp")
				attr = NULL;
		}

          // Kiedy nie jest to atrybut zdefiniowany w programie to probujemy go
          // konwertowac na liczbe typu float
          if(attr == NULL)
          {
               // Jezeli nie to sprawdzamy czy jest to liczba float
               // Sprawdzamy float bo w tym zbiorze zawiera sie integer
               bool is_float;
               float val = arg.toFloat(&is_float);

               // Jezeli tez nie liczba to mamy niezidentyfikowany argument
               if(!is_float)
               {
                    setCellError("Undefined argument \"" + arg + "\".");
                    return "#NAN";
               }
               // Jezeli jednak liczba to obliczamy wartosc funckji
               if(is_float)
               {
                    if(val < .0)
                    {
                         setCellError("Logarithm argument \"" + arg + "\" can't be less than zero.");
                         return "#NAN";
                    }
                    tmp_res = QString::number(log(val), 'f');
               }
          }

          // Jezeli jest to konkretny argument to sprawdzamy czy jest typu liczbowego
          if(attr != NULL)
          {
               // Sprawdzanie czy atrybut istneieje w danej tabeli
               if(TableParent()->IndexOfAttributeID(attr->Id()) == -1)
               {
                    setCellError("Undefined argument \"" + arg + "\" occurence.");
                    return "#NAN";
               }

               if(attr->Value().toLower() == XTT_Attribute::_not_definded_operator_)
               {
                    setCellError("Not defined value in argument \"" + arg + "\".");
                    return "#NAN";
               }
               
               // Gdy jest typu float albo integer
               if((attr->Type() == 2) || (attr->Type() == 3))
                    tmp_res = QString::number(log(attr->Value().toFloat()), 'f');
               if((attr->Type() != 2) && (attr->Type() != 3))
               {
                    setCellError("Incorrect attribute type \"" + arg + "\".");
                    return "#NAN";
               }
          }
          expr = expr.replace(index, fname.length()+arg.length(), tmp_res);
     }
     return expr;
}
// -----------------------------------------------------------------------------

/**
 * Metoda obliczajaca z wyrazenia wszystkie iloczyny
 * @param QString& expr - Referencja do wyrazenia
 * @returns Zwraca zmodyfikowane wyrazenie
*/
QString XTT_Cell::mul_func(QString fname, QString& expr)
{
     // Zmienna przechowujaca tymczasowy wynik dzialania funckji
     QString tmp_res = "";

     // Wartosci argumentow jako stringi
     QString arg1val = "";
     QString arg2val = "";

     // Mozemy uogolnic typ liczbowy bo mnozenie jest da obu dobrze zdefiniowane
     int arg1type;            // 0-bool, 1-liczba, 2-symbolic, 3-enumerate 
     int arg2type;            // 0-bool, 1-liczba, 2-symbolic, 3-enumerate

     // Dlugosci wczytanych argumentow
     int a1len;
     int a2len;


          // Index pierwszego znaku nazwy funckji
          int index = expr.indexOf(fname, 0, Qt::CaseInsensitive);
          
          // Przesuniecie na pierwszy znak argumentu
          int aindex = index + fname.length();

          // Wczytywanie lewego argumentu
          QString stop_char = "+-/*";
          QString arg1 = "";
          if(index > 0)
               for(int i=index-1;((i>=0)&&(expr[i]!='+')&&((expr[i]!='-')||((expr[i]=='-')&&((i==0)||((i>0)&&(stop_char.contains(expr[i-1]))))))&&(expr[i]!='*')&&(expr[i]!='/'));i--)
                    arg1 = expr[i] + arg1;

          // Wczytywanie prawego argumentu
          QString arg2 = "";
          for(int i=aindex;((i<expr.length())&&(expr[i]!='+')&&((expr[i]!='-')||((expr[i]=='-')&&((i==aindex)||(expr[i-1]==' '))))&&(expr[i]!='*')&&(expr[i]!='/'));i++)
               arg2 = arg2 + expr[i];

          // Zapisujemy dlugosci wczytanych atrbutow
          a1len = arg1.length();
          a2len = arg2.length();

          // Usuwanie spacji w wartosciach
          for(;arg1.contains(" ");) arg1 = arg1.remove(arg1.indexOf(" "), 1);
          for(;arg2.contains(" ");) arg2 = arg2.remove(arg2.indexOf(" "), 1);

          // Sprawdanie czy istnieje podany argumnet
          XTT_Attribute* attr1 = AttributeGroups->Attribute(arg1);
          XTT_Attribute* attr2 = AttributeGroups->Attribute(arg2);
		
		// Jezeli nie jest wlaczony tryb porownywania atrybutow to ustawiamy atrybut na NULL
		if((!fEnableAttr2AttrComp) && (attr1 != NULL))
		{	
			if(AttributeGroups->AttributeGroup(attr1)->Name() != "tmp")
				attr1 = NULL;
		}
		if((!fEnableAttr2AttrComp) && (attr2 != NULL))
		{	
			if(AttributeGroups->AttributeGroup(attr2)->Name() != "tmp")
				attr2 = NULL;
		}

          // Kiedy nie jest to atrybut zdefiniowany w programie to probujemy go
          // konwertowac na liczbe typu float
          if(attr1 == NULL)
          {
               // Gdy bool
               if(fAttrType->Type() == 1)
               {
                    // Jezeli podane wartosci jako liczby
                    if(arg1 == "0") arg1 = "false";
                    if(arg1 == "1") arg1 = "true";

                    // Jezeli nie jest to wartosc boolowska to blad
                    if((arg1 != "true") && (arg1 != "false"))
                    {
                         setCellError("Undefined argument 1 \"" + arg1 + "\".");
                         return "#NAN";
                    }

                    // Inaczej zapisujemy typ i wartosc
                    arg1type = 0;
                    arg1val = arg1;
               }

               // Gdy integer lub float
               if((fAttrType->Type() == 2) || (fAttrType->Type() == 3))
               {
                    // Jezeli nie to sprawdzamy czy jest to liczba float
                    // Sprawdzamy float bo w tym zbiorze zawiera sie integer
                    bool is_float;
                    arg1.toFloat(&is_float);

                    // Jezeli tez nie liczba to mamy niezidentyfikowany argument
                    if(!is_float)
                    {
                         setCellError("Undefined argument 1 \"" + arg1 + "\".");
                         return "#NAN";
                    }
                    // Jezeli jednak liczba to zapisujemy typ i wartosc argumentu
                    if(is_float)
                    {
                         arg1type = 1;
                         arg1val = arg1;
                    }
               }

               // Gdy symbolic
               if(fAttrType->Type() == 4)
               {
                    // Inaczej zapisujemy typ i wartosc
                    arg1type = 2;
                    arg1val = arg1;
               }

               // Gdy enumerate
               if(fAttrType->Type() == 5)
               {
                    // Sprawdzanie czy argument jest w dziedzinie
                    int index = fAttrType->AttributeValues()->indexOf(arg1);

                    // Jezeli poza dziedzina
                    if(index == -1)
                    {
                         setCellError("Undefined argument 1 \"" + arg1 + "\".");
                         return "#NAN";
                    }

                    // Inaczej zapisujemy typ i wartosc
                    arg1type = 3;
                    arg1val = QString::number(index);

                    // Utworzenie nowego tymczasowego atrybutu i zapisanie wyniku
                    int tmp_i = AttributeGroups->CreateNewTmpArgument(5);
                    AttributeGroups->_IndexOfName("tmp")->Item(tmp_i)->setEnumValues(fAttrType->_AttributeValues());
                    AttributeGroups->_IndexOfName("tmp")->Item(tmp_i)->setValue(arg1);
                    attr1 = AttributeGroups->_IndexOfName("tmp")->Item(tmp_i);
               }
          }

          // Jezeli jest to konkretny argument to sprawdzamy czy jest typu liczbowego
          if(attr1 != NULL)
          {
               // Sprawdzanie czy atrybut istneieje w danej tabeli
               if((TableParent()->IndexOfAttributeID(attr1->Id()) == -1) && (AttributeGroups->CreatePath(attr1) != "tmp."))
               {
                    setCellError("Undefined argument \"" + arg1 + "\" occurence.");
                    return "#NAN";
               }

               if(attr1->Value().toLower() == XTT_Attribute::_not_definded_operator_)
               {
                    setCellError("Not defined value \"" + arg1 + "\".");
                    return "#NAN";
               }
               // Zpisanie wartosci argumentu
               arg1val = attr1->Value();

               // Ustalenie typu argumentu
               if(attr1->Type() == 1)
                    arg1type = 0;
               if((attr1->Type() == 2) || (attr1->Type() == 3))
                    arg1type = 1;
               if(attr1->Type() == 4)
                    arg1type = 2;
               if(attr1->Type() == 5)
                    arg1type = 3;

          }

          // Kiedy nie jest to atrybut zdefiniowany w programie to probujemy go
          // konwertowac na liczbe typu float
          if(attr2 == NULL)
          {
               // Gdy bool
               if(fAttrType->Type() == 1)
               {
                    // Jezeli podane wartosci jako liczby
                    if(arg2 == "0") arg2 = "false";
                    if(arg2 == "1") arg2 = "true";

                    // Jezeli nie jest to wartosc boolowska to blad
                    if((arg2 != "true") && (arg2 != "false"))
                    {
                         setCellError("Undefined argument 2 \"" + arg2 + "\".");
                         return "#NAN";
                    }

                    // Inaczej zapisujemy typ i wartosc
                    arg2type = 0;
                    arg2val = arg2;
               }

               // Gdy integer lub float
               if((fAttrType->Type() == 2) || (fAttrType->Type() == 3))
               {
                    // Jezeli nie to sprawdzamy czy jest to liczba float
                    // Sprawdzamy float bo w tym zbiorze zawiera sie integer
                    bool is_float;
                    arg2.toFloat(&is_float);

                    // Jezeli tez nie liczba to mamy niezidentyfikowany argument
                    if(!is_float)
                    {
                         setCellError("Undefined argument 2 \"" + arg2 + "\".");
                         return "#NAN";
                    }
                    // Jezeli jednak liczba to zapisujemy typ i wartosc argumentu
                    if(is_float)
                    {
                         arg2type = 1;
                         arg2val = arg2;
                    }
               }

               // Gdy symbolic
               if(fAttrType->Type() == 4)
               {
                    // Inaczej zapisujemy typ i wartosc
                    arg2type = 2;
                    arg2val = arg2;
               }

               // Gdy enumerate
               if(fAttrType->Type() == 5)
               {
                    // Sprawdzanie czy argument jest w dziedzinie
                    int index = fAttrType->AttributeValues()->indexOf(arg2);

                    // Jezeli poza dziedzina
                    if(index == -1)
                    {
                         setCellError("Undefined argument 2 \"" + arg2 + "\".");
                         return "#NAN";
                    }

                    // Inaczej zapisujemy typ i wartosc
                    arg2type = 3;
                    arg2val = QString::number(index);

                    // Utworzenie nowego tymczasowego atrybutu i zapisanie wyniku
                    int tmp_i = AttributeGroups->CreateNewTmpArgument(5);
                    AttributeGroups->_IndexOfName("tmp")->Item(tmp_i)->setEnumValues(fAttrType->_AttributeValues());
                    AttributeGroups->_IndexOfName("tmp")->Item(tmp_i)->setValue(arg2);
                    attr2 = AttributeGroups->_IndexOfName("tmp")->Item(tmp_i);
               }
          }

          // Jezeli jest to konkretny argument to sprawdzamy czy jest typu liczbowego
          if(attr2 != NULL)
          {
               // Sprawdzanie czy atrybut istneieje w danej tabeli
               if((TableParent()->IndexOfAttributeID(attr2->Id()) == -1) && (AttributeGroups->CreatePath(attr2) != "tmp."))
               {
                    setCellError("Undefined argument \"" + arg2 + "\" occurence.");
                    return "#NAN";
               }

               if(attr2->Value().toLower() == XTT_Attribute::_not_definded_operator_)
               {
                    setCellError("Not defined value in argument 2 \"" + arg2 + "\".");
                    return "#NAN";
               }

               // Zpisanie wartosci argumentu
               arg2val = attr2->Value();

               // Ustalenie typu argumentu
               if(attr2->Type() == 1)
                    arg2type = 0;
               if((attr2->Type() == 2) || (attr2->Type() == 3))
                    arg2type = 1;
               if(attr2->Type() == 4)
                    arg2type = 2;
               if(attr2->Type() == 5)
                    arg2type = 3;
          }

          // Jezeli argumenty nie sa typu takiego jaki atrybut przypisany do
          // komorki
          if(((arg2type == 0) && (fAttrType->Type() != 1)) ||
             ((arg2type == 1) && (fAttrType->Type() != 2) && (fAttrType->Type() != 3)) ||
             ((arg2type == 2) && (fAttrType->Type() != 4)) ||
             ((arg2type == 3) && (fAttrType->Type() != 5)))
          {
               setCellError("Result of operation is unsiutable.");
               return "#NAN";
          }

          // Jezeli argumenty sa roznych typow
          if(arg1type != arg2type)
          {
               setCellError("Inconsistent arguments for product function.");
               return "#NAN";
          }

          // ------ Obliczanie wartosci dzialania w zaleznosi od typu argumentu
          // Kiedy boolean
          if(arg1type == 0)
          {
               tmp_res = "false";
               if((arg1val == arg2val) && (arg1val == "true"))
                    tmp_res = "true";

               // Utworzenie nowego tymczasowego atrybutu i zapisanie wyniku
               int tmp_i = AttributeGroups->CreateNewTmpArgument(1);
               AttributeGroups->_IndexOfName("tmp")->Item(tmp_i)->setValue(tmp_res);
               tmp_res = "tmp." + AttributeGroups->_IndexOfName("tmp")->Item(tmp_i)->Name();
          }
          // Kiedy liczba
          if(arg1type == 1)
          {
               tmp_res = "";
               float v1 = arg1val.toFloat();
               float v2 = arg2val.toFloat();
               int precision = 6;

               bool isInt1, isInt2;
               arg1val.toInt(&isInt1);
               arg2val.toInt(&isInt2);
               if(isInt1 && isInt2)
                    precision = 0;

               double r = v1*v2;
               QString sr = QString::number(r, 'f', precision);
               tmp_res = sr;

               // Utworzenie nowego tymczasowego atrybutu i zapisanie wyniku
               //int tmp_i = AttributeGroups->CreateNewTmpArgument(2);
               //AttributeGroups->_IndexOfName("tmp")->Item(tmp_i)->setValue(tmp_res);
               //tmp_res = "tmp." + AttributeGroups->_IndexOfName("tmp")->Item(tmp_i)->Name();
          }
          // Kiedy symbolic
          if(arg1type == 2)
          {
               tmp_res = "";
               for(int i=0;i<arg1val.length();i++)
               {
                    if(arg2val.contains(arg1val[i]))
                         tmp_res = tmp_res + arg1val[i];
               }

               // Utworzenie nowego tymczasowego atrybutu i zapisanie wyniku
               int tmp_i = AttributeGroups->CreateNewTmpArgument(4);
               AttributeGroups->_IndexOfName("tmp")->Item(tmp_i)->setValue(tmp_res);
               tmp_res = "tmp." + AttributeGroups->_IndexOfName("tmp")->Item(tmp_i)->Name();
          }
          // Kiedy enumerate
          if(arg1type == 3)
          {
               // Sprawdzanie czy wskazniki nie sa NULL
               if((attr1 == NULL) || (attr2 == NULL))
               {
                    setCellError("Critical Error!!!\nInconsistent argument type.");
                    return "#NAN";
               }

               tmp_res = "";
               int i1 = attr1->AttributeValues()->indexOf(attr1->Value());
               int i2 = attr2->AttributeValues()->indexOf(attr2->Value());

               // Sprawdzanie czy wartosci argumentow sa rzeczywicie dobre
               if((i1 == -1) || (i2 == -1))
               {
                    setCellError("Critical Error!!!\nOne of product function argument has got a value out of domain.");
                    return "#NAN";
               }

               // Wykonanie dzialania
               tmp_res = QString::number(i1*i2);

               // Utworzenie nowego tymczasowego atrybutu i zapisanie wyniku
               int tmp_i = AttributeGroups->CreateNewTmpArgument(5);
               AttributeGroups->_IndexOfName("tmp")->Item(tmp_i)->setEnumValues(fAttrType->_AttributeValues());
               AttributeGroups->_IndexOfName("tmp")->Item(tmp_i)->setValue(tmp_res);
               tmp_res = "tmp." + AttributeGroups->_IndexOfName("tmp")->Item(tmp_i)->Name();
          }

          expr = expr.replace(index-a1len, fname.length()+a1len+a2len, tmp_res);

     return expr;
}
// -----------------------------------------------------------------------------

/**
 * Metoda obliczajaca z wyrazenia wszystkie ilorazy
 * @param QString& expr - Referencja do wyrazenia
 * @returns Zwraca zmodyfikowane wyrazenie
*/
QString XTT_Cell::div_func(QString fname, QString& expr)
{
     // Zmienna przechowujaca tymczasowy wynik dzialania funckji
     QString tmp_res = "";

     // Wartosci argumentow jako stringi
     QString arg1val = "";
     QString arg2val = "";

     // Mozemy uogolnic typ liczbowy bo mnozenie jest da obu dobrze zdefiniowane
     int arg1type = -1;       // 0-bool, 1-liczba, 2-symbolic, 3-enumerate
     int arg2type = -1;       // 0-bool, 1-liczba, 2-symbolic, 3-enumerate

     // Dlugosci wczytanych argumentow
     int a1len;
     int a2len;

          // Index pierwszego znaku nazwy funckji
          int index = expr.indexOf(fname, 0, Qt::CaseInsensitive);
          
          // Przesuniecie na pierwszy znak argumentu
          int aindex = index + fname.length();

          // Wczytywanie lewego argumentu
          QString stop_char = "+-/*";
          QString arg1 = "";
          if(index > 0)
               for(int i=index-1;((i>=0)&&(expr[i]!='+')&&((expr[i]!='-')||((expr[i]=='-')&&((i==0)||((i>0)&&(stop_char.contains(expr[i-1]))))))&&(expr[i]!='*')&&(expr[i]!='/'));i--)
                    arg1 = expr[i] + arg1;

          // Wczytywanie prawego argumentu
          QString arg2 = "";
          for(int i=aindex;((i<expr.length())&&(expr[i]!='+')&&((expr[i]!='-')||((expr[i]=='-')&&((i==aindex)||(expr[i-1]==' '))))&&(expr[i]!='*')&&(expr[i]!='/'));i++)
               arg2 = arg2 + expr[i];

          // Zapisujemy dlugosci wczytanych atrbutow
          a1len = arg1.length();
          a2len = arg2.length();

          // Usuwanie spacji w wartosciach
          for(;arg1.contains(" ");) arg1 = arg1.remove(arg1.indexOf(" "), 1);
          for(;arg2.contains(" ");) arg2 = arg2.remove(arg2.indexOf(" "), 1);

          // Sprawdanie czy istnieje podany argumnet
          XTT_Attribute* attr1 = AttributeGroups->Attribute(arg1);
          XTT_Attribute* attr2 = AttributeGroups->Attribute(arg2);
		
		// Jezeli nie jest wlaczony tryb porownywania atrybutow to ustawiamy atrybut na NULL
		if((!fEnableAttr2AttrComp) && (attr1 != NULL))
		{	
			if(AttributeGroups->AttributeGroup(attr1)->Name() != "tmp")
				attr1 = NULL;
		}
		if((!fEnableAttr2AttrComp) && (attr2 != NULL))
		{	
			if(AttributeGroups->AttributeGroup(attr2)->Name() != "tmp")
				attr2 = NULL;
		}

          // Kiedy nie jest to atrybut zdefiniowany w programie to probujemy go
          // konwertowac na liczbe typu float
          if(attr1 == NULL)
          {
               // Gdy bool
               if(fAttrType->Type() == 1)
               {
                    // Jezeli podane wartosci jako liczby
                    if(arg1 == "0") arg1 = "false";
                    if(arg1 == "1") arg1 = "true";

                    // Jezeli nie jest to wartosc boolowska to blad
                    if((arg1 != "true") && (arg1 != "false"))
                    {
                         setCellError("Undefined argument 1 \"" + arg1 + "\".");
                         return "#NAN";
                    }

                    // Inaczej zapisujemy typ i wartosc
                    arg1type = 0;
                    arg1val = arg1;
               }

               // Gdy integer lub float
               if((fAttrType->Type() == 2) || (fAttrType->Type() == 3))
               {
                    // Jezeli nie to sprawdzamy czy jest to liczba float
                    // Sprawdzamy float bo w tym zbiorze zawiera sie integer
                    bool is_float;
                    arg1.toFloat(&is_float);

                    // Jezeli tez nie liczba to mamy niezidentyfikowany argument
                    if(!is_float)
                    {
                         setCellError("Undefined argument 1 \"" + arg1 + "\".");
                         return "#NAN";
                    }
                    // Jezeli jednak liczba to zapisujemy typ i wartosc argumentu
                    if(is_float)
                    {
                         arg1type = 1;
                         arg1val = arg1;
                    }
               }

               // Gdy symbolic
               if(fAttrType->Type() == 4)
               {
                    // Inaczej zapisujemy typ i wartosc
                    arg1type = 2;
                    arg1val = arg1;
               }

               // Gdy enumerate
               if(fAttrType->Type() == 5)
               {
                    // Sprawdzanie czy argument jest w dziedzinie
                    int index = fAttrType->AttributeValues()->indexOf(arg1);

                    // Jezeli poza dziedzina
                    if(index == -1)
                    {
                         setCellError("Undefined argument 1 \"" + arg1 + "\".");
                         return "#NAN";
                    }

                    // Inaczej zapisujemy typ i wartosc
                    arg1type = 3;
                    arg1val = QString::number(index);

                    // Utworzenie nowego tymczasowego atrybutu i zapisanie wyniku
                    int tmp_i = AttributeGroups->CreateNewTmpArgument(5);
                    AttributeGroups->_IndexOfName("tmp")->Item(tmp_i)->setEnumValues(fAttrType->_AttributeValues());
                    AttributeGroups->_IndexOfName("tmp")->Item(tmp_i)->setValue(arg1);
                    attr1 = AttributeGroups->_IndexOfName("tmp")->Item(tmp_i);
               }
          }

          // Jezeli jest to konkretny argument to sprawdzamy czy jest typu liczbowego
          if(attr1 != NULL)
          {
               // Sprawdzanie czy atrybut istneieje w danej tabeli
               if((TableParent()->IndexOfAttributeID(attr1->Id()) == -1) && (AttributeGroups->CreatePath(attr1) != "tmp."))
               {
                    setCellError("Undefined argument \"" + arg1 + "\" occurence.");
                    return "#NAN";
               }

               if(attr1->Value().toLower() == XTT_Attribute::_not_definded_operator_)
               {
                    setCellError("Not defined value in argument 1 \"" + arg1 + "\".");
                    return "#NAN";
               }

               // Gdy jest typu float albo integer
               if((attr1->Type() == 2) || (attr1->Type() == 3))
               {
                    arg1val = attr1->Value();
                    arg1type = 1;
               }
               if((attr1->Type() != 2) && (attr1->Type() != 3))
               {
                    setCellError("Incorrect attribute type 1 \"" + arg1 + "\".");
                    return "#NAN";
               }

          }

          // Kiedy nie jest to atrybut zdefiniowany w programie to probujemy go
          // konwertowac na liczbe typu float
          if(attr2 == NULL)
          {
               // Gdy bool
               if(fAttrType->Type() == 1)
               {
                    // Jezeli podane wartosci jako liczby
                    if(arg2 == "0") arg2 = "false";
                    if(arg2 == "1") arg2 = "true";

                    // Jezeli nie jest to wartosc boolowska to blad
                    if((arg2 != "true") && (arg2 != "false"))
                    {
                         setCellError("Undefined argument 2 \"" + arg2 + "\".");
                         return "#NAN";
                    }

                    // Inaczej zapisujemy typ i wartosc
                    arg2type = 0;
                    arg2val = arg2;
               }

               // Gdy integer lub float
               if((fAttrType->Type() == 2) || (fAttrType->Type() == 3))
               {
                    // Jezeli nie to sprawdzamy czy jest to liczba float
                    // Sprawdzamy float bo w tym zbiorze zawiera sie integer
                    bool is_float;
                    arg2.toFloat(&is_float);

                    // Jezeli tez nie liczba to mamy niezidentyfikowany argument
                    if(!is_float)
                    {
                         setCellError("Undefined argument 2 \"" + arg2 + "\".");
                         return "#NAN";
                    }
                    // Jezeli jednak liczba to zapisujemy typ i wartosc argumentu
                    if(is_float)
                    {
                         arg2type = 1;
                         arg2val = arg2;
                    }
               }

               // Gdy symbolic
               if(fAttrType->Type() == 4)
               {
                    // Inaczej zapisujemy typ i wartosc
                    arg2type = 2;
                    arg2val = arg2;
               }

               // Gdy enumerate
               if(fAttrType->Type() == 5)
               {
                    // Sprawdzanie czy argument jest w dziedzinie
                    int index = fAttrType->AttributeValues()->indexOf(arg2);

                    // Jezeli poza dziedzina
                    if(index == -1)
                    {
                         setCellError("Undefined argument 2 \"" + arg2 + "\".");
                         return "#NAN";
                    }

                    // Inaczej zapisujemy typ i wartosc
                    arg2type = 3;
                    arg2val = QString::number(index);

                    // Utworzenie nowego tymczasowego atrybutu i zapisanie wyniku
                    int tmp_i = AttributeGroups->CreateNewTmpArgument(5);
                    AttributeGroups->_IndexOfName("tmp")->Item(tmp_i)->setEnumValues(fAttrType->_AttributeValues());
                    AttributeGroups->_IndexOfName("tmp")->Item(tmp_i)->setValue(arg2);
                    attr2 = AttributeGroups->_IndexOfName("tmp")->Item(tmp_i);
               }
          }

          // Jezeli jest to konkretny argument to sprawdzamy czy jest typu liczbowego
          if(attr2 != NULL)
          {
               // Sprawdzanie czy atrybut istneieje w danej tabeli
               if((TableParent()->IndexOfAttributeID(attr2->Id()) == -1) && (AttributeGroups->CreatePath(attr2) != "tmp."))
               {
                    setCellError("Undefined argument \"" + arg2 + "\" occurence.");
                    return "#NAN";
               }

               if(attr2->Value().toLower() == XTT_Attribute::_not_definded_operator_)
               {
                    setCellError("Not defined value in argument 2 \"" + arg2 + "\".");
                    return "#NAN";
               }

               // Gdy jest typu float albo integer
               if((attr2->Type() == 2) || (attr2->Type() == 3))
               {
                    arg2val = attr2->Value();
                    arg2type = 1;
               }
               if((attr2->Type() != 2) && (attr2->Type() != 3))
               {
                    setCellError("Incorrect attribute type 2 \"" + arg2 + "\".");
                    return "#NAN";
               }
          }

          // Jezeli argumenty nie sa typu takiego jaki atrybut przypisany do
          // komorki
          if(((arg2type == 0) && (fAttrType->Type() != 1)) ||
             ((arg2type == 1) && (fAttrType->Type() != 2) && (fAttrType->Type() != 3)) ||
             ((arg2type == 2) && (fAttrType->Type() != 4)) ||
             ((arg2type == 3) && (fAttrType->Type() != 5)))
          {
               setCellError("Result of operation is unsiutable.");
               return "#NAN";
          }

          // Jezeli argumenty sa roznych typow
          if(arg1type != arg2type)
          {
               setCellError("Inconsistent arguments for product function.");
               return "#NAN";
          }

          // ------ Obliczanie wartosci dzialania w zaleznosi od typu argumentu
          // Kiedy boolean
          if(arg1type == 0)
          {
               tmp_res = "false";
               if((arg1val == arg2val) && (arg1val == "true"))
                    tmp_res = "true";

               // Utworzenie nowego tymczasowego atrybutu i zapisanie wyniku
               int tmp_i = AttributeGroups->CreateNewTmpArgument(1);
               AttributeGroups->_IndexOfName("tmp")->Item(tmp_i)->setValue(tmp_res);
               tmp_res = "tmp." + AttributeGroups->_IndexOfName("tmp")->Item(tmp_i)->Name();
          }
          // Kiedy liczba
          if(arg1type == 1)
          {
               tmp_res = "";
               float v1 = arg1val.toFloat();
               float v2 = arg2val.toFloat();
               if(v2 == .0)
               {
                    setCellError("Division by zero.");
                    return "#NAN";
               }

               int precision = 6;

               bool isInt1, isInt2;
               arg1val.toInt(&isInt1);
               arg2val.toInt(&isInt2);
               if(isInt1 && isInt2)
                    precision = 0;

               double r = v1/v2;
               QString sr = QString::number(r, 'f', precision);
               tmp_res = sr;
          }
          // Kiedy symbolic
          if(arg1type == 2)
          {
               tmp_res = "";
               for(int i=0;i<arg1val.length();i++)
               {
                    if(arg2val.contains(arg1val[i]))
                         tmp_res = tmp_res + arg1val[i];
               }
               // Utworzenie nowego tymczasowego atrybutu i zapisanie wyniku
               int tmp_i = AttributeGroups->CreateNewTmpArgument(4);
               AttributeGroups->_IndexOfName("tmp")->Item(tmp_i)->setValue(tmp_res);
               tmp_res = "tmp." + AttributeGroups->_IndexOfName("tmp")->Item(tmp_i)->Name();
          }
          // Kiedy enumerate
          if(arg1type == 3)
          {
               // Sprawdzanie czy wskazniki nie sa NULL
               if((attr1 == NULL) || (attr2 == NULL))
               {
                    setCellError("Critical Error!!!\nInconsistent argument type.");
                    return "#NAN";
               }

               tmp_res = "";
               int i1 = attr1->AttributeValues()->indexOf(attr1->Value());
               int i2 = attr2->AttributeValues()->indexOf(attr2->Value());

               // Sprawdzanie czy wartosci argumentow sa rzeczywicie dobre
               if((i1 == -1) || (i2 == -1))
               {
                    setCellError("Critical Error!!!\nOne of product function argument has got a value out of domain.");
                    return "#NAN";
               }

               // Wykonanie dzialania
               tmp_res = QString::number(i1/i2);

               // Utworzenie nowego tymczasowego atrybutu i zapisanie wyniku
               int tmp_i = AttributeGroups->CreateNewTmpArgument(5);
               AttributeGroups->_IndexOfName("tmp")->Item(tmp_i)->setEnumValues(fAttrType->_AttributeValues());
               AttributeGroups->_IndexOfName("tmp")->Item(tmp_i)->setValue(tmp_res);
               tmp_res = "tmp." + AttributeGroups->_IndexOfName("tmp")->Item(tmp_i)->Name();
          }

          expr = expr.replace(index-a1len, fname.length()+a1len+a2len, tmp_res);

     return expr;
}
// -----------------------------------------------------------------------------

/**
 * Metoda obliczajaca z wyrazenia wszystkie sumy
 * @param QString& expr - Referencja do wyrazenia
 * @returns Zwraca zmodyfikowane wyrazenie
*/
QString XTT_Cell::add_func(QString fname, QString& expr)
{
     // Zmienna przechowujaca tymczasowy wynik dzialania funckji
     QString tmp_res = "";

     // Wartosci argumentow jako stringi
     QString arg1val = "";
     QString arg2val = "";

     // Mozemy uogolnic typ liczbowy bo mnozenie jest da obu dobrze zdefiniowane
     int arg1type;            // 0-bool, 1-liczba, 2-symbolic, 3-enumerate 
     int arg2type;            // 0-bool, 1-liczba, 2-symbolic, 3-enumerate

     // Dlugosci wczytanych argumentow
     int a1len;
     int a2len;

          // Index pierwszego znaku nazwy funckji
          int index = expr.indexOf(fname, 0, Qt::CaseInsensitive);
          
          // Przesuniecie na pierwszy znak argumentu
          int aindex = index + fname.length();

          // Wczytywanie lewego argumentu
          QString stop_char = "+-/*";
          QString arg1 = "";
          if(index > 0)
               for(int i=index-1;((i>=0)&&(expr[i]!='+')&&((expr[i]!='-')||((expr[i]=='-')&&((i==0)||((i>0)&&(stop_char.contains(expr[i-1]))))))&&(expr[i]!='*')&&(expr[i]!='/'));i--)
                    arg1 = expr[i] + arg1;

          // Wczytywanie prawego argumentu
          QString arg2 = "";
          for(int i=aindex;((i<expr.length())&&(expr[i]!='+')&&((expr[i]!='-')||((expr[i]=='-')&&((i==aindex)||(expr[i-1]==' '))))&&(expr[i]!='*')&&(expr[i]!='/'));i++)
               arg2 = arg2 + expr[i];

          // Zapisujemy dlugosci wczytanych atrbutow
          a1len = arg1.length();
          a2len = arg2.length();

          // Usuwanie spacji w wartosciach
          for(;arg1.contains(" ");) arg1 = arg1.remove(arg1.indexOf(" "), 1);
          for(;arg2.contains(" ");) arg2 = arg2.remove(arg2.indexOf(" "), 1);

          // Sprawdanie czy istnieje podany argumnet
          XTT_Attribute* attr1 = AttributeGroups->Attribute(arg1);
          XTT_Attribute* attr2 = AttributeGroups->Attribute(arg2);
		
		// Jezeli nie jest wlaczony tryb porownywania atrybutow to ustawiamy atrybut na NULL
		if((!fEnableAttr2AttrComp) && (attr1 != NULL))
		{	
			if(AttributeGroups->AttributeGroup(attr1)->Name() != "tmp")
				attr1 = NULL;
		}
		if((!fEnableAttr2AttrComp) && (attr2 != NULL))
		{	
			if(AttributeGroups->AttributeGroup(attr2)->Name() != "tmp")
				attr2 = NULL;
		}

          // Kiedy nie jest to atrybut zdefiniowany w programie to probujemy go
          // konwertowac na liczbe typu float
          if(attr1 == NULL)
          {
               // Gdy bool
               if(fAttrType->Type() == 1)
               {
                    // Jezeli podane wartosci jako liczby
                    if(arg1 == "0") arg1 = "false";
                    if(arg1 == "1") arg1 = "true";

                    // Jezeli nie jest to wartosc boolowska to blad
                    if((arg1 != "true") && (arg1 != "false"))
                    {
                         setCellError("Undefined argument 1 \"" + arg1 + "\".");
                         return "#NAN";
                    }

                    // Inaczej zapisujemy typ i wartosc
                    arg1type = 0;
                    arg1val = arg1;
               }

               // Gdy integer lub float
               if((fAttrType->Type() == 2) || (fAttrType->Type() == 3))
               {
                    // Jezeli nie to sprawdzamy czy jest to liczba float
                    // Sprawdzamy float bo w tym zbiorze zawiera sie integer
                    bool is_float;
                    arg1.toFloat(&is_float);

                    // Jezeli tez nie liczba to mamy niezidentyfikowany argument
                    if(!is_float)
                    {
                         setCellError("Undefined argument 1 \"" + arg1 + "\".");
                         return "#NAN";
                    }
                    // Jezeli jednak liczba to zapisujemy typ i wartosc argumentu
                    if(is_float)
                    {
                         arg1type = 1;
                         arg1val = arg1;
                    }
               }

               // Gdy symbolic
               if(fAttrType->Type() == 4)
               {
                    // Inaczej zapisujemy typ i wartosc
                    arg1type = 2;
                    arg1val = arg1;
               }

               // Gdy enumerate
               if(fAttrType->Type() == 5)
               {
                    // Sprawdzanie czy argument jest w dziedzinie
                    int index = fAttrType->AttributeValues()->indexOf(arg1);

                    // Jezeli poza dziedzina
                    if(index == -1)
                    {
                         setCellError("Undefined argument 1 \"" + arg1 + "\".");
                         return "#NAN";
                    }

                    // Inaczej zapisujemy typ i wartosc
                    arg1type = 3;
                    arg1val = QString::number(index);

                    // Utworzenie nowego tymczasowego atrybutu i zapisanie wyniku
                    int tmp_i = AttributeGroups->CreateNewTmpArgument(5);
                    AttributeGroups->_IndexOfName("tmp")->Item(tmp_i)->setEnumValues(fAttrType->_AttributeValues());
                    AttributeGroups->_IndexOfName("tmp")->Item(tmp_i)->setValue(arg1);
                    attr1 = AttributeGroups->_IndexOfName("tmp")->Item(tmp_i);
               }
          }

          // Jezeli jest to konkretny argument to sprawdzamy czy jest typu liczbowego
          if(attr1 != NULL)
          {
               // Sprawdzanie czy atrybut istneieje w danej tabeli
               if((TableParent()->IndexOfAttributeID(attr1->Id()) == -1) && (AttributeGroups->CreatePath(attr1) != "tmp."))
               {
                    setCellError("Undefined argument \"" + arg1 + "\" occurence.");
                    return "#NAN";
               }

               if(attr1->Value().toLower() == XTT_Attribute::_not_definded_operator_)
               {
                    setCellError("Not defined value in argument 1 \"" + arg1 + "\".");
                    return "#NAN";
               }
               // Zpisanie wartosci argumentu
               arg1val = attr1->Value();

               // Ustalenie typu argumentu
               if(attr1->Type() == 1)
                    arg1type = 0;
               if((attr1->Type() == 2) || (attr1->Type() == 3))
                    arg1type = 1;
               if(attr1->Type() == 4)
                    arg1type = 2;
               if(attr1->Type() == 5)
                    arg1type = 3;

          }

          // Kiedy nie jest to atrybut zdefiniowany w programie to probujemy go
          // konwertowac na liczbe typu float
          if(attr2 == NULL)
          {
               // Gdy bool
               if(fAttrType->Type() == 1)
               {
                    // Jezeli podane wartosci jako liczby
                    if(arg2 == "0") arg2 = "false";
                    if(arg2 == "1") arg2 = "true";

                    // Jezeli nie jest to wartosc boolowska to blad
                    if((arg2 != "true") && (arg2 != "false"))
                    {
                         setCellError("Undefined argument 2 \"" + arg2 + "\".");
                         return "#NAN";
                    }

                    // Inaczej zapisujemy typ i wartosc
                    arg2type = 0;
                    arg2val = arg2;
               }

               // Gdy integer lub float
               if((fAttrType->Type() == 2) || (fAttrType->Type() == 3))
               {
                    // Jezeli nie to sprawdzamy czy jest to liczba float
                    // Sprawdzamy float bo w tym zbiorze zawiera sie integer
                    bool is_float;
                    arg2.toFloat(&is_float);

                    // Jezeli tez nie liczba to mamy niezidentyfikowany argument
                    if(!is_float)
                    {
                         setCellError("Undefined argument 2 \"" + arg2 + "\".");
                         return "#NAN";
                    }
                    // Jezeli jednak liczba to zapisujemy typ i wartosc argumentu
                    if(is_float)
                    {
                         arg2type = 1;
                         arg2val = arg2;
                    }
               }

               // Gdy symbolic
               if(fAttrType->Type() == 4)
               {
                    // Inaczej zapisujemy typ i wartosc
                    arg2type = 2;
                    arg2val = arg2;
               }

               // Gdy enumerate
               if(fAttrType->Type() == 5)
               {
                    // Sprawdzanie czy argument jest w dziedzinie
                    int index = fAttrType->AttributeValues()->indexOf(arg2);

                    // Jezeli poza dziedzina
                    if(index == -1)
                    {
                         setCellError("Undefined argument 2 \"" + arg2 + "\".");
                         return "#NAN";
                    }

                    // Inaczej zapisujemy typ i wartosc
                    arg2type = 3;
                    arg2val = QString::number(index);

                    // Utworzenie nowego tymczasowego atrybutu i zapisanie wyniku
                    int tmp_i = AttributeGroups->CreateNewTmpArgument(5);
                    AttributeGroups->_IndexOfName("tmp")->Item(tmp_i)->setEnumValues(fAttrType->_AttributeValues());
                    AttributeGroups->_IndexOfName("tmp")->Item(tmp_i)->setValue(arg2);
                    attr2 = AttributeGroups->_IndexOfName("tmp")->Item(tmp_i);
               }
          }

          // Jezeli jest to konkretny argument to sprawdzamy czy jest typu liczbowego
          if(attr2 != NULL)
          {
               // Sprawdzanie czy atrybut istneieje w danej tabeli
               if((TableParent()->IndexOfAttributeID(attr2->Id()) == -1) && (AttributeGroups->CreatePath(attr2) != "tmp."))
               {
                    setCellError("Undefined argument \"" + arg2 + "\" occurence.");
                    return "#NAN";
               }

               if(attr2->Value().toLower() == XTT_Attribute::_not_definded_operator_)
               {
                    setCellError("Not defined value in argument 2 \"" + arg2 + "\".");
                    return "#NAN";
               }

               // Zpisanie wartosci argumentu
               arg2val = attr2->Value();

               // Ustalenie typu argumentu
               if(attr2->Type() == 1)
                    arg2type = 0;
               if((attr2->Type() == 2) || (attr2->Type() == 3))
                    arg2type = 1;
               if(attr2->Type() == 4)
                    arg2type = 2;
               if(attr2->Type() == 5)
                    arg2type = 3;
          }

          // Jezeli argumenty nie sa typu takiego jaki atrybut przypisany do
          // komorki
          if(((arg2type == 0) && (fAttrType->Type() != 1)) ||
             ((arg2type == 1) && (fAttrType->Type() != 2) && (fAttrType->Type() != 3)) ||
             ((arg2type == 2) && (fAttrType->Type() != 4)) ||
             ((arg2type == 3) && (fAttrType->Type() != 5)))
          {
               setCellError("Result of operation is unsiutable.");
               return "#NAN";
          }

          // Jezeli argumenty sa roznych typow
          if(arg1type != arg2type)
          {
               setCellError("Inconsistent arguments for product function.");
               return "#NAN";
          }

          // ------ Obliczanie wartosci dzialania w zaleznosi od typu argumentu
          // Kiedy boolean
          if(arg1type == 0)
          {
               tmp_res = "true";
               if((arg1val == arg2val) && (arg1val == "false"))
                    tmp_res = "true";

               // Utworzenie nowego tymczasowego atrybutu i zapisanie wyniku
               int tmp_i = AttributeGroups->CreateNewTmpArgument(1);
               AttributeGroups->_IndexOfName("tmp")->Item(tmp_i)->setValue(tmp_res);
               tmp_res = "tmp." + AttributeGroups->_IndexOfName("tmp")->Item(tmp_i)->Name();
          }
          // Kiedy liczba
          if(arg1type == 1)
          {
               tmp_res = "";
               float v1 = arg1val.toFloat();
               float v2 = arg2val.toFloat();
               int precision = 6;

               bool isInt1, isInt2;
               arg1val.toInt(&isInt1);
               arg2val.toInt(&isInt2);
               if(isInt1 && isInt2)
                    precision = 0;

               double r = v1+v2;
               QString sr = QString::number(r, 'f', precision);
               tmp_res = sr;
          }
          // Kiedy symbolic
          if(arg1type == 2)
          {
               tmp_res = arg1val + arg2val;

               // Utworzenie nowego tymczasowego atrybutu i zapisanie wyniku
               int tmp_i = AttributeGroups->CreateNewTmpArgument(4);
               AttributeGroups->_IndexOfName("tmp")->Item(tmp_i)->setValue(tmp_res);
               tmp_res = "tmp." + AttributeGroups->_IndexOfName("tmp")->Item(tmp_i)->Name();
          }
          // Kiedy enumerate
          if(arg1type == 3)
          {
               // Sprawdzanie czy wskazniki nie sa NULL
               if((attr1 == NULL) || (attr2 == NULL))
               {
                    setCellError("Critical Error!\nInconsistent argument type.");
                    return "#NAN";
               }

               tmp_res = "";
               int i1 = attr1->AttributeValues()->indexOf(attr1->Value());
               int i2 = attr2->AttributeValues()->indexOf(attr2->Value());

               // Sprawdzanie czy wartosci argumentow sa rzeczywicie dobre
               if((i1 == -1) || (i2 == -1))
               {
                    setCellError("Critical Error!\nOne of product function argument has got a value out of domain.");
                    return "#NAN";
               }

               // Wykonanie dzialania
               tmp_res = QString::number(i1+i2);

               // Utworzenie nowego tymczasowego atrybutu i zapisanie wyniku
               int tmp_i = AttributeGroups->CreateNewTmpArgument(5);
               AttributeGroups->_IndexOfName("tmp")->Item(tmp_i)->setEnumValues(fAttrType->_AttributeValues());
               AttributeGroups->_IndexOfName("tmp")->Item(tmp_i)->setValue(tmp_res);
               tmp_res = "tmp." + AttributeGroups->_IndexOfName("tmp")->Item(tmp_i)->Name();
          }

          expr = expr.replace(index-a1len, fname.length()+a1len+a2len, tmp_res);

     return expr;
}
// -----------------------------------------------------------------------------

/**
 * Metoda obliczajaca z wyrazenia wszystkie roznice
 * @param QString& expr - Referencja do wyrazenia
 * @returns Zwraca zmodyfikowane wyrazenie
*/
QString XTT_Cell::sub_func(QString fname, QString& expr)
{
     // Zmienna przechowujaca tymczasowy wynik dzialania funckji
     QString tmp_res = "";

     // Wartosci argumentow jako stringi
     QString arg1val = "";
     QString arg2val = "";

     // Mozemy uogolnic typ liczbowy bo mnozenie jest da obu dobrze zdefiniowane
     int arg1type;            // 0-bool, 1-liczba, 2-symbolic, 3-enumerate 
     int arg2type;            // 0-bool, 1-liczba, 2-symbolic, 3-enumerate

     // Dlugosci wczytanych argumentow
     int a1len;
     int a2len;


          // Index pierwszego znaku nazwy funckji
          int index = expr.indexOf(fname, 1, Qt::CaseInsensitive);
          
          // Przesuniecie na pierwszy znak argumentu
          int aindex = index + fname.length();

          // Wczytywanie lewego argumentu
          QString stop_char = "+-/*";
          QString arg1 = "";
          if(index > 0)
               for(int i=index-1;((i>=0)&&(expr[i]!='+')&&((expr[i]!='-')||((expr[i]=='-')&&((i==0)||((i>0)&&(stop_char.contains(expr[i-1]))))))&&(expr[i]!='*')&&(expr[i]!='/'));i--)
                         arg1 = expr[i] + arg1;

          // Wczytywanie prawego argumentu
          QString arg2 = "";
          for(int i=aindex;((i<expr.length())&&(expr[i]!='+')&&((expr[i]!='-')||((expr[i]=='-')&&((i==aindex)||(expr[i-1]==' '))))&&(expr[i]!='*')&&(expr[i]!='/'));i++)
                    arg2 = arg2 + expr[i];

          // Zapisujemy dlugosci wczytanych atrbutow
          a1len = arg1.length();
          a2len = arg2.length();

          // Usuwanie spacji w wartosciach
          for(;arg1.contains(" ");) arg1 = arg1.remove(arg1.indexOf(" "), 1);
          for(;arg2.contains(" ");) arg2 = arg2.remove(arg2.indexOf(" "), 1);

          // Sprawdanie czy istnieje podany argumnet
          XTT_Attribute* attr1 = AttributeGroups->Attribute(arg1);
          XTT_Attribute* attr2 = AttributeGroups->Attribute(arg2);
		
		// Jezeli nie jest wlaczony tryb porownywania atrybutow to ustawiamy atrybut na NULL
		if((!fEnableAttr2AttrComp) && (attr1 != NULL))
		{	
			if(AttributeGroups->AttributeGroup(attr1)->Name() != "tmp")
				attr1 = NULL;
		}
		if((!fEnableAttr2AttrComp) && (attr2 != NULL))
		{	
			if(AttributeGroups->AttributeGroup(attr2)->Name() != "tmp")
				attr2 = NULL;
		}

          // Kiedy nie jest to atrybut zdefiniowany w programie to probujemy go
          // konwertowac na watosc w typie atrybutu przypisanego do komorki (zrdolwego)
          if(attr1 == NULL)
          {
               // Gdy bool
               if(fAttrType->Type() == 1)
               {
                    // Jezeli podane wartosci jako liczby
                    if(arg1 == "0") arg1 = "false";
                    if(arg1 == "1") arg1 = "true";

                    // Jezeli nie jest to wartosc boolowska to blad
                    if((arg1 != "true") && (arg1 != "false"))
                    {
                         setCellError("Undefined argument 1 \"" + arg1 + "\".");
                         return "#NAN";
                    }

                    // Inaczej zapisujemy typ i wartosc
                    arg1type = 0;
                    arg1val = arg1;
               }

               // Gdy integer lub float
               if((fAttrType->Type() == 2) || (fAttrType->Type() == 3))
               {
                    // Jezeli nie to sprawdzamy czy jest to liczba float
                    // Sprawdzamy float bo w tym zbiorze zawiera sie integer
                    bool is_float;
                    arg1.toFloat(&is_float);

                    // Jezeli tez nie liczba to mamy niezidentyfikowany argument
                    if(!is_float)
                    {
                         setCellError("Undefined argument 1 \"" + arg1 + "\".");
                         return "#NAN";
                    }
                    // Jezeli jednak liczba to zapisujemy typ i wartosc argumentu
                    if(is_float)
                    {
                         arg1type = 1;
                         arg1val = arg1;
                    }
               }

               // Gdy symbolic
               if(fAttrType->Type() == 4)
               {
                    // Inaczej zapisujemy typ i wartosc
                    arg1type = 2;
                    arg1val = arg1;
               }

               // Gdy enumerate
               if(fAttrType->Type() == 5)
               {
                    // Sprawdzanie czy argument jest w dziedzinie
                    int index = fAttrType->AttributeValues()->indexOf(arg1);

                    // Jezeli poza dziedzina
                    if(index == -1)
                    {
                         setCellError("Undefined argument 1 \"" + arg1 + "\".");
                         return "#NAN";
                    }

                    // Inaczej zapisujemy typ i wartosc
                    arg1type = 3;
                    arg1val = QString::number(index);

                    // Utworzenie nowego tymczasowego atrybutu i zapisanie wyniku
                    int tmp_i = AttributeGroups->CreateNewTmpArgument(5);
                    AttributeGroups->_IndexOfName("tmp")->Item(tmp_i)->setEnumValues(fAttrType->_AttributeValues());
                    AttributeGroups->_IndexOfName("tmp")->Item(tmp_i)->setValue(arg1);
                    attr1 = AttributeGroups->_IndexOfName("tmp")->Item(tmp_i);
               }
          }

          // Jezeli jest to konkretny argument to sprawdzamy czy jest typu liczbowego
          if(attr1 != NULL)
          {
               // Sprawdzanie czy atrybut istneieje w danej tabeli
               if((TableParent()->IndexOfAttributeID(attr1->Id()) == -1) && (AttributeGroups->CreatePath(attr1) != "tmp."))
               {
                    setCellError("Undefined argument \"" + arg1 + "\" occurence.");
                    return "#NAN";
               }
               
               if(attr1->Value().toLower() == XTT_Attribute::_not_definded_operator_)
               {
                    setCellError("Not defined value in argument 1 \"" + arg1 + "\".");
                    return "#NAN";
               }
               // Zpisanie wartosci argumentu
               arg1val = attr1->Value();

               // Ustalenie typu argumentu
               if(attr1->Type() == 1)
                    arg1type = 0;
               if((attr1->Type() == 2) || (attr1->Type() == 3))
                    arg1type = 1;
               if(attr1->Type() == 4)
                    arg1type = 2;
               if(attr1->Type() == 5)
                    arg1type = 3;

          }

          // Kiedy nie jest to atrybut zdefiniowany w programie to probujemy go
          // konwertowac na liczbe typu float
          if(attr2 == NULL)
          {
               // Gdy bool
               if(fAttrType->Type() == 1)
               {
                    // Jezeli podane wartosci jako liczby
                    if(arg2 == "0") arg2 = "false";
                    if(arg2 == "1") arg2 = "true";

                    // Jezeli nie jest to wartosc boolowska to blad
                    if((arg2 != "true") && (arg2 != "false"))
                    {
                         setCellError("Undefined argument 2 \"" + arg2 + "\".");
                         return "#NAN";
                    }

                    // Inaczej zapisujemy typ i wartosc
                    arg2type = 0;
                    arg2val = arg2;
               }

               // Gdy integer lub float
               if((fAttrType->Type() == 2) || (fAttrType->Type() == 3))
               {
                    // Jezeli nie to sprawdzamy czy jest to liczba float
                    // Sprawdzamy float bo w tym zbiorze zawiera sie integer
                    bool is_float;
                    arg2.toFloat(&is_float);

                    // Jezeli tez nie liczba to mamy niezidentyfikowany argument
                    if(!is_float)
                    {
                         setCellError("Undefined argument 2 \"" + arg2 + "\".");
                         return "#NAN";
                    }
                    // Jezeli jednak liczba to zapisujemy typ i wartosc argumentu
                    if(is_float)
                    {
                         arg2type = 1;
                         arg2val = arg2;
                    }
               }

               // Gdy symbolic
               if(fAttrType->Type() == 4)
               {
                    // Inaczej zapisujemy typ i wartosc
                    arg2type = 2;
                    arg2val = arg2;
               }

               // Gdy enumerate
               if(fAttrType->Type() == 5)
               {
                    // Sprawdzanie czy argument jest w dziedzinie
                    int index = fAttrType->AttributeValues()->indexOf(arg2);

                    // Jezeli poza dziedzina
                    if(index == -1)
                    {
                         setCellError("Undefined argument 2 \"" + arg2 + "\".");
                         return "#NAN";
                    }

                    // Inaczej zapisujemy typ i wartosc
                    arg2type = 3;
                    arg2val = QString::number(index);

                    // Utworzenie nowego tymczasowego atrybutu i zapisanie wyniku
                    int tmp_i = AttributeGroups->CreateNewTmpArgument(5);
                    AttributeGroups->_IndexOfName("tmp")->Item(tmp_i)->setEnumValues(fAttrType->_AttributeValues());
                    AttributeGroups->_IndexOfName("tmp")->Item(tmp_i)->setValue(arg2);
                    attr2 = AttributeGroups->_IndexOfName("tmp")->Item(tmp_i);
               }
          }

          // Jezeli jest to konkretny argument to sprawdzamy czy jest typu liczbowego
          if(attr2 != NULL)
          {
               // Sprawdzanie czy atrybut istneieje w danej tabeli
               if((TableParent()->IndexOfAttributeID(attr2->Id()) == -1) && (AttributeGroups->CreatePath(attr2) != "tmp."))
               {
                    setCellError("Undefined argument \"" + arg2 + "\" occurence.");
                    return "#NAN";
               }

               if(attr2->Value().toLower() == XTT_Attribute::_not_definded_operator_)
               {
                    setCellError("Not defined value in argument 2 \"" + arg2 + "\".");
                    return "#NAN";
               }

               // Zpisanie wartosci argumentu
               arg2val = attr2->Value();

               // Ustalenie typu argumentu
               if(attr2->Type() == 1)
                    arg2type = 0;
               if((attr2->Type() == 2) || (attr2->Type() == 3))
                    arg2type = 1;
               if(attr2->Type() == 4)
                    arg2type = 2;
               if(attr2->Type() == 5)
                    arg2type = 3;
          }

          // Jezeli argumenty nie sa typu takiego jaki atrybut przypisany do
          // komorki
          if(((arg2type == 0) && (fAttrType->Type() != 1)) ||
             ((arg2type == 1) && (fAttrType->Type() != 2) && (fAttrType->Type() != 3)) ||
             ((arg2type == 2) && (fAttrType->Type() != 4)) ||
             ((arg2type == 3) && (fAttrType->Type() != 5)))
          {
               setCellError("Result of operation is unsiutable.");
               return "#NAN";
          }

          // Jezeli argumenty sa roznych typow
          if(arg1type != arg2type)
          {
               setCellError("Inconsistent arguments for product function.");
               return "#NAN";
          }

          // ------ Obliczanie wartosci dzialania w zaleznosi od typu argumentu
          // Kiedy boolean
          if(arg1type == 0)
          {
               tmp_res = "true";
               if(arg1val == arg2val)
                    tmp_res = "false";

               // Utworzenie nowego tymczasowego atrybutu i zapisanie wyniku
               int tmp_i = AttributeGroups->CreateNewTmpArgument(1);
               AttributeGroups->_IndexOfName("tmp")->Item(tmp_i)->setValue(tmp_res);
               tmp_res = "tmp." + AttributeGroups->_IndexOfName("tmp")->Item(tmp_i)->Name();
          }
          // Kiedy liczba
          if(arg1type == 1)
          {
               tmp_res = "";
               float v1 = arg1val.toFloat();
               float v2 = arg2val.toFloat();
               int precision = 6;

               bool isInt1, isInt2;
               arg1val.toInt(&isInt1);
               arg2val.toInt(&isInt2);
               if(isInt1 && isInt2)
                    precision = 0;

               double r = v1-v2;
               QString sr = QString::number(r, 'f', precision);
               tmp_res = sr;
          }
          // Kiedy symbolic
          if(arg1type == 2)
          {
               tmp_res = arg1val;
               for(int i=0;i<arg2val.length();i++)
                    if(tmp_res.contains(arg2val[i]))
                         tmp_res = tmp_res.remove(tmp_res.indexOf(arg2val[i]), 1);
                         
               // Utworzenie nowego tymczasowego atrybutu i zapisanie wyniku
               int tmp_i = AttributeGroups->CreateNewTmpArgument(4);
               AttributeGroups->_IndexOfName("tmp")->Item(tmp_i)->setValue(tmp_res);
               tmp_res = "tmp." + AttributeGroups->_IndexOfName("tmp")->Item(tmp_i)->Name();
          }
          // Kiedy enumerate
          if(arg1type == 3)
          {
               // Sprawdzanie czy wskazniki nie sa NULL
               if((attr1 == NULL) || (attr2 == NULL))
               {
                    setCellError("Critical Error!!!\nInconsistent argument type.");
                    return "#NAN";
               }

               tmp_res = "";
               int i1 = attr1->AttributeValues()->indexOf(attr1->Value());
               int i2 = attr2->AttributeValues()->indexOf(attr2->Value());

               // Sprawdzanie czy wartosci argumentow sa rzeczywicie dobre
               if((i1 == -1) || (i2 == -1))
               {
                    setCellError("Critical Error!!!\nOne of product function argument has got a value out of domain.");
                    return "#NAN";
               }

               // Wykonanie dzialania
               tmp_res = QString::number(i1-i2);
               
               // Utworzenie nowego tymczasowego atrybutu i zapisanie wyniku
               int tmp_i = AttributeGroups->CreateNewTmpArgument(5);
               AttributeGroups->_IndexOfName("tmp")->Item(tmp_i)->setEnumValues(fAttrType->_AttributeValues());
               AttributeGroups->_IndexOfName("tmp")->Item(tmp_i)->setValue(tmp_res);
               tmp_res = "tmp." + AttributeGroups->_IndexOfName("tmp")->Item(tmp_i)->Name();
          }

          expr = expr.replace(index-a1len, fname.length()+a1len+a2len, tmp_res);

     return expr;
}
// -----------------------------------------------------------------------------

/**
 * Metoda zapisujaca wszystkie dane w formacie xttml 1.0
 * @returns Zwraca liste stringow w tym formacie
*/
QStringList* XTT_Cell::XTTML_1_0(void)
{
     QStringList* res = new QStringList;
     res->clear();
	
	QString a2a = "true";
	if(!fEnableAttr2AttrComp)
		a2a = "false";
	
	QString ccontent = fContent;
	
	// Usuwanie spacji z poczatku
	for(;ccontent.indexOf(" ") == 0;)
		ccontent = ccontent.remove(ccontent.indexOf(" "), 1);
		
	int operApperIndex = -1;
	
	if(fOperator > 0)
		operApperIndex = ccontent.indexOf(smbl_opers.at(fOperator-1), 0, Qt::CaseInsensitive);
	
	if((fOperator > 0) && (operApperIndex == 0))
		ccontent = ccontent.replace(operApperIndex, smbl_opers.at(fOperator-1).length(), text_opers.at(fOperator-1));
     
     *res << "<cell id=\"" + QString::number(fCellID) + "\" parent=\"" +  QString::number(fParent) + "\" attr2attr=\"" + a2a + "\" value_type=\"" + QString::number(fValDefType) + "\" operator=\"" + QString::number(fOperator) + "\" context=\"" + QString::number(fContext) + "\" ignore=\"" + QString::number(fIgnore?1:0) + "\" cell_attribute=\"" + QString::number(fAttrType->Id()) + "\">";
     *res << "\t<content>" + ccontent + "</content>";
     *res << "\t<value>" + fValue + "</value>";

     *res << "\t<values_set>";
     for(int i=0;i<fAttrValue->size();i++)
          *res << "\t\t<vs>"+ fAttrValue->at(i) +"</vs>";
     *res << "\t</values_set>";

     *res << "\t<brace_left>" + QString::number(fBrace[0]?1:0) + "</brace_left>";
     *res << "\t<brace_right>" + QString::number(fBrace[1]?1:0) + "</brace_right>";
     *res << "\t<boundaries_left>" + fBoundaries[0] + "</boundaries_left>";
     *res << "\t<boundaries_right>" + fBoundaries[1] + "</boundaries_right>";
     
     *res << "</cell>";

     return res;
}
// -----------------------------------------------------------------------------

#if QT_VERSION >= 0x040300
/**
 * Metoda zapisujaca wszystkie dane w formacie xttml 1.0
 * @param QXmlStreamWriter* _xmldoc - wskanik do dokumentu XML
 * @returns brak zwracanych wartosci
*/
void XTT_Cell::XTTML_1_0(QXmlStreamWriter* _xmldoc)
{
	QString a2a = "true";
	if(!fEnableAttr2AttrComp)
		a2a = "false";
	
	QString ccontent = fContent;
	
	// Usuwanie spacji z poczatku
	for(;ccontent.indexOf(" ") == 0;)
		ccontent = ccontent.remove(ccontent.indexOf(" "), 1);
		
	int operApperIndex = -1;
	
	if(fOperator > 0)
		operApperIndex = ccontent.indexOf(smbl_opers.at(fOperator-1), 0, Qt::CaseInsensitive);
	
	if((fOperator > 0) && (operApperIndex == 0))
		ccontent = ccontent.replace(operApperIndex, smbl_opers.at(fOperator-1).length(), text_opers.at(fOperator-1));
     
	_xmldoc->writeStartElement("cell");
	_xmldoc->writeAttribute("id", QString::number(fCellID));
	_xmldoc->writeAttribute("parent", QString::number(fParent));
	_xmldoc->writeAttribute("attr2attr", a2a);
	_xmldoc->writeAttribute("value_type", QString::number(fValDefType));
	_xmldoc->writeAttribute("operator", QString::number(fOperator));
	_xmldoc->writeAttribute("context", QString::number(fContext));
	_xmldoc->writeAttribute("ignore", QString::number(fIgnore?1:0));
	_xmldoc->writeAttribute("cell_attribute", QString::number(fAttrType->Id()));
	
		_xmldoc->writeTextElement("content", ccontent);
		_xmldoc->writeTextElement("value", fValue);

		_xmldoc->writeStartElement("values_set");
		for(int i=0;i<fAttrValue->size();i++)
			_xmldoc->writeTextElement("vs", fAttrValue->at(i));
		_xmldoc->writeEndElement();

		_xmldoc->writeTextElement("brace_left", QString::number(fBrace[0]?1:0));
		_xmldoc->writeTextElement("brace_right", QString::number(fBrace[1]?1:0));
		_xmldoc->writeTextElement("boundaries_left", fBoundaries[0]);
		_xmldoc->writeTextElement("boundaries_right", fBoundaries[1]);
     
     _xmldoc->writeEndElement();
}
#endif
// -----------------------------------------------------------------------------

/**
 * Metoda zapisujaca wszystkie dane w formacie xttml 2.0
 * @returns Zwraca liste stringow w tym formacie
*/
QStringList* XTT_Cell::XTTML_2_0(void)
{
	QStringList* res = new QStringList;
     res->clear();
	
	QString ccontent = fContent;
	QString coperator = "";
	
	// Usuwanie spacji z poczatku
	for(;ccontent.indexOf(" ") == 0;)
		ccontent = ccontent.remove(ccontent.indexOf(" "), 1);
		
	int operApperIndex = -1;
	
	if(fOperator > 0)
		operApperIndex = ccontent.indexOf(smbl_opers.at(fOperator-1), 0, Qt::CaseInsensitive);
	
	if((fOperator > 0) && (operApperIndex == 0))
	{
		ccontent = ccontent.remove(operApperIndex, smbl_opers.at(fOperator-1).length());
		coperator = text_opers.at(fOperator-1);
	}
	
	// Usuwanie spacji z poczatku
	for(;ccontent.indexOf(" ") == 0;)
		ccontent = ccontent.remove(ccontent.indexOf(" "), 1);
	
	
	QString a2a = "true";
	if(!fEnableAttr2AttrComp)
		a2a = "false";
	
     *res << "<cell  operator=\"" + coperator + "content=\"" + ccontent + "\" attr2attr=\"" + a2a + "\"/>";
	
	return res;
}
// -----------------------------------------------------------------------------

#if QT_VERSION >= 0x040300
/**
 * Metoda zapisujaca wszystkie dane w formacie xttml 2.0
 * @param QXmlStreamWriter* _xmldoc - wskanik do dokumentu XML
 * @returns brak zwracanych wartosci
*/
void XTT_Cell::XTTML_2_0(QXmlStreamWriter* _xmldoc)
{
	QString ccontent = fContent;
	QString coperator = "";
	
	int operApperIndex = -1;
	
	if(fOperator > 0)
		operApperIndex = ccontent.indexOf(smbl_opers.at(fOperator-1), 0, Qt::CaseInsensitive);
	
	if((fOperator > 0) && (operApperIndex == 0))
	{
		ccontent = ccontent.remove(operApperIndex, smbl_opers.at(fOperator-1).length());
		coperator = text_opers.at(fOperator-1);
	}
	
	// Usuwanie spacji z poczatku
	for(;ccontent.indexOf(" ") == 0;)
		ccontent = ccontent.remove(ccontent.indexOf(" "), 1);
	
	QString a2a = "true";
	if(!fEnableAttr2AttrComp)
		a2a = "false";
		
	_xmldoc->writeEmptyElement("cell");
	_xmldoc->writeAttribute("operator", coperator);
	_xmldoc->writeAttribute("content", ccontent);
	_xmldoc->writeAttribute("attr2attr", a2a);
}
#endif
// -----------------------------------------------------------------------------

/**
 * Metoda wczytujaca dane o strybucie z pliku w formacie XTTML 1.0
 * @param QStringList* list - Wskanik do sekcji dotyczacej tego atrybutu
 * @returns true jezeli dane sa ok, ianczej false
*/
bool XTT_Cell::FromXTTML_1_0(QStringList* list)
{
     // Usuwanie spacji i tabulatorow wierszach
     for(int i=0;i<list->size();i++)
     {
          QString line = list->at(i);
          while(line.contains("\t"))
          {
               int index = line.indexOf("\t");
               line = line.remove(index, 1);
          }
          list->replace(i, line);
     }

     // Zakladamy ze pierwszy wiersz jest naglowkiem
     if(!list->at(0).contains("<cell", Qt::CaseInsensitive))
          return false;
     if(!list->contains("</cell>", Qt::CaseInsensitive))
          return false;

     // Reszta zakladamy ze jest ok

     // Spis tagow
     QStringList tags;
     tags.clear();
     tags  << "<content>" << "<value>" << "<values_set>" << "<vs>" << "<brace_left>" << "<brace_right>" << "<boundaries_left>" << "<boundaries_right>" << "<cell";

     for(int line=0;line<list->size();line++)
     {
          // Sprawdzanie wystapienia tagu
          int action = -1;
          for(int tag=0;tag<tags.size();tag++)
               if(list->at(line).contains(tags.at(tag), Qt::CaseInsensitive))
                    action = tag;

          if(action == -1)
               continue;

          QString end_tag = tags.at(action);
          end_tag = end_tag.insert(1, "/");
          QString l = list->at(line);
          l = l.remove(l.indexOf(tags.at(action), Qt::CaseInsensitive), tags.at(action).length());
          l = l.mid(0, l.indexOf(end_tag, Qt::CaseInsensitive));

          // Wczytywanie parametrow w zaleznosci od wykrytego tagu
          if(action == 0)
		{
			QString cnt = l;
		
			// Usuwanie spacji z tresci
			for(;cnt.indexOf(" ", 0) == 0;)
				cnt = cnt.remove(cnt.indexOf(" "), 1);
			
			// Zamiana operatora tekstowego na symboliczny
			for(int i=0;i<text_opers.size();i++)
				if(cnt.indexOf(text_opers.at(i), 0, Qt::CaseInsensitive) == 0)
					cnt = cnt.replace(cnt.indexOf(text_opers.at(i), 0, Qt::CaseInsensitive), text_opers.at(i).length(), smbl_opers.at(i));
			
			fContent = cnt;
		}
          if(action == 1)
               fValue = l;
          if(action == 3)
               fAttrValue->append(l);
          if(action == 4)
               fBrace[0] = (bool)l.toInt();
          if(action == 5)
               fBrace[1] = (bool)l.toInt();
          if(action == 6)
               fBoundaries[0] = l;
          if(action == 7)
               fBoundaries[1] = l;
          if(action == 8)
          {
               fCellID = (unsigned int)XML_ReadValue(list->at(line), "id").toInt();
               fParent = (unsigned int)XML_ReadValue(list->at(line), "parent").toInt();
               fValDefType = (unsigned int)XML_ReadValue(list->at(line), "value_type").toInt();
               fOperator = (unsigned int)XML_ReadValue(list->at(line), "operator").toInt();
               fContext = (unsigned int)XML_ReadValue(list->at(line), "context").toInt();
               unsigned int aid = (unsigned int)XML_ReadValue(list->at(line), "cell_attribute").toInt();
			fEnableAttr2AttrComp = (bool)XML_ReadValue(list->at(line), "attr2attr").toInt();
			
               fAttrType = AttributeGroups->FindAttribute(aid);
               if(fAttrType == NULL)
               {
                    QMessageBox::critical(NULL, "XTT Editor", "Error attribute ID.", QMessageBox::Ok);
                    return false;
               }
          }
     }
     
     //QStringList* h = XTTML_1_0();
     //for(int i=0;i<h->size();i++)
     //QMessageBox::warning(NULL, "Application", h->at(i));

     return true;
}
// -----------------------------------------------------------------------------

/**
 * Metoda wczytujaca dane o strybucie z pliku w formacie XTTML 1.0
 * @param QDomElement& root - Referencja do sekcji dotyczacej tego atrybutu
 * @returns true jezeli dane sa ok, ianczej false
*/
bool XTT_Cell::FromXTTML_1_0(QDomElement& root)
{
	if(root.tagName().toLower() != "cell")
		return false;
	
	fCellID = (unsigned int)root.attribute("id", "0").toInt();
	fParent = (unsigned int)root.attribute("parent", "0").toInt();
	fValDefType = (unsigned int)root.attribute("value_type", "0").toInt();
	fOperator = (unsigned int)root.attribute("operator", "0").toInt();
	fContext = (unsigned int)root.attribute("context", "0").toInt();
	unsigned int aid = (unsigned int)root.attribute("cell_attribute", "0").toInt();
	fEnableAttr2AttrComp = (bool)root.attribute("attr2attr", "0").toInt();
	
	fAttrType = AttributeGroups->FindAttribute(aid);
	if(fAttrType == NULL)
	{
		QMessageBox::critical(NULL, "XTT Editor", "Error attribute ID = " + QString::number(aid) + ". Can't find attribute.", QMessageBox::Ok);
		return false;
	}
	
	QDomElement DOMcont = root.firstChildElement("content");
	QDomElement DOMval = root.firstChildElement("value");
	QDomElement DOMvalset = root.firstChildElement("values_set");
	QDomElement DOMbl = root.firstChildElement("brace_left");
	QDomElement DOMbr = root.firstChildElement("brace_right");
	QDomElement DOMbndl = root.firstChildElement("boundaries_left");
	QDomElement DOMbndr = root.firstChildElement("boundaries_right");
	
	fValue = DOMval.text();
	fBrace[0] = (bool)DOMbl.text().toInt();
	fBrace[1] = (bool)DOMbr.text().toInt();
	fBoundaries[0] = DOMbndl.text();
	fBoundaries[1] = DOMbndr.text();
	
	QDomElement DOMvs = DOMvalset.firstChildElement("vs");
     while(!DOMvs.isNull())
     {
		fAttrValue->append(DOMvs.text());
		DOMvs = DOMvs.nextSiblingElement("vs");
     }
	
	QString cnt = DOMcont.text();
	// Usuwanie spacji z tresci
	for(;cnt.indexOf(" ", 0) == 0;)
		cnt = cnt.remove(cnt.indexOf(" "), 1);
	
	// Zamiana operatora tekstowego na symboliczny
	for(int i=0;i<text_opers.size();i++)
		if(cnt.indexOf(text_opers.at(i), 0, Qt::CaseInsensitive) == 0)
			cnt = cnt.replace(cnt.indexOf(text_opers.at(i), 0, Qt::CaseInsensitive), text_opers.at(i).length(), smbl_opers.at(i));
			
	fContent = cnt;

     return true;
}
// -----------------------------------------------------------------------------

/**
 * Metoda wczytujaca dane o strybucie z pliku w formacie XTTML 2.0
 * @param QStringList* list - Wskanik do sekcji dotyczacej tego atrybutu
 * @returns true jezeli dane sa ok, ianczej false
*/
bool XTT_Cell::FromXTTML_2_0(QStringList* list)
{
     // Usuwanie spacji i tabulatorow wierszach
     for(int i=0;i<list->size();i++)
     {
          QString line = list->at(i);
          while(line.contains("\t"))
          {
               int index = line.indexOf("\t");
               line = line.remove(index, 1);
          }
          list->replace(i, line);
     }

     // Zakladamy ze pierwszy wiersz jest naglowkiem
     if(!list->at(0).contains("<cell", Qt::CaseInsensitive))
          return false;

     // Reszta zakladamy ze jest ok
     // Spis tagow
     QStringList tags;
     tags.clear();
     tags  << "<cell";

     for(int line=0;line<list->size();line++)
     {
          // Sprawdzanie wystapienia tagu
          int action = -1;
          for(int tag=0;tag<tags.size();tag++)
               if(list->at(line).contains(tags.at(tag), Qt::CaseInsensitive))
                    action = tag;

          if(action == -1)
               continue;

          QString end_tag = tags.at(action);
          end_tag = end_tag.insert(1, "/");
          QString l = list->at(line);
          l = l.remove(l.indexOf(tags.at(action), Qt::CaseInsensitive), tags.at(action).length());
          l = l.mid(0, l.indexOf(end_tag, Qt::CaseInsensitive));

          // Wczytywanie parametrow w zaleznosci od wykrytego tagu
          if(action == 0)
          {
			QString cnt = XML_ReadValue(list->at(line), "content"); 
			QString a2a = XML_ReadValue(list->at(line), "attr2attr"); 
			if(cnt == "#NAN#")
			{
				QMessageBox::critical(NULL, "XTT Editor", "Missing cell parameter \"content\" at line: " + list->at(line) + ".", QMessageBox::Ok);
				return false;
			}
			
			// Usuwanie spacji z tresci
			for(;cnt.indexOf(" ", 0) == 0;)
				cnt = cnt.remove(cnt.indexOf(" "), 1);
			for(;a2a.contains(" ");)
				a2a = a2a.remove(a2a.indexOf(" "), 1);
			
			// Zamiana operatora tekstowego na symboliczny
			for(int i=0;i<text_opers.size();i++)
				if(cnt.indexOf(text_opers.at(i), 0, Qt::CaseInsensitive) == 0)
					cnt = cnt.replace(cnt.indexOf(text_opers.at(i), 0, Qt::CaseInsensitive), text_opers.at(i).length(), smbl_opers.at(i));
			
			fContent = cnt;
			
			fEnableAttr2AttrComp = true;
			if(a2a.toLower() == "false")
				fEnableAttr2AttrComp = false;
				
			if((a2a.toLower() != "true") && (a2a.toLower() != "false") && (a2a != "#NAN#"))
			{
				QMessageBox::critical(NULL, "XTT Editor", "Incorrect cell parameter \"attr2attr\" at line: " + list->at(line) + ".\nAllowed values are only \"true\", \"false\"", QMessageBox::Ok);
				return false;
			}
          }
     }
     
     //QStringList* h = XTTML_1_0();
     //for(int i=0;i<h->size();i++)
     //QMessageBox::warning(NULL, "Application", h->at(i));

     return true;
}
// -----------------------------------------------------------------------------

/**
 * Metoda wczytujaca dane o strybucie z pliku w formacie XTTML 2.0
 * @param QDomDocument& root - Wskanik do sekcji dotyczacej tego atrybutu
 * @returns true jezeli dane sa ok, ianczej false
*/
bool XTT_Cell::FromXTTML_2_0(QDomElement& root)
{
	if(root.tagName().toLower() != "cell")
		return false;

	QString opr = root.attribute("operator", "0");
	QString cnt = root.attribute("content", "#NAN#");
	QString a2a = root.attribute("attr2attr", "#NAN#");
	
	if(opr == "0")
	{
		QMessageBox::critical(NULL, "XTT Editor", "Missing cell parameter \"operator\" at line " + QString::number(root.lineNumber()) + ".", QMessageBox::Ok);
		return false;
	}
	if(cnt == "#NAN#")
	{
		QMessageBox::critical(NULL, "XTT Editor", "Missing cell parameter \"content\" at line " + QString::number(root.lineNumber()) + ".", QMessageBox::Ok);
		return false;
	}
	
	// Usuwanie spacji z tresci
	for(;opr.indexOf(" ", 0) == 0;)
		opr = opr.remove(opr.indexOf(" "), 1);
	for(;cnt.indexOf(" ", 0) == 0;)
		cnt = cnt.remove(cnt.indexOf(" "), 1);
	for(;a2a.contains(" ");)
		a2a = a2a.remove(a2a.indexOf(" "), 1);
	
	// Zamiana operatora tekstowego na symboliczny
	for(int i=0;i<text_opers.size();i++)
		if(opr.indexOf(text_opers.at(i), 0, Qt::CaseInsensitive) == 0)
			opr = smbl_opers.at(i);
	
	fContent = opr + " " + cnt;
	
	fEnableAttr2AttrComp = true;
	if(a2a.toLower() == "false")
		fEnableAttr2AttrComp = false;
		
	if((a2a.toLower() != "true") && (a2a.toLower() != "false") && (a2a != "#NAN#"))
	{
		QMessageBox::critical(NULL, "XTT Editor", "Incorrect cell parameter \"attr2attr\" at line " + QString::number(root.lineNumber()) + ".\nAllowed values are only \"true\", \"false\"", QMessageBox::Ok);
		return false;
	}

     return true;
}
// -----------------------------------------------------------------------------

