 /**
 *                                                                             *
 * @file XTT_Cell.h                                                            *
 * Project name: HeKatE hqed                                                   *
 * $Id: XTT_Cell.h,v 1.12 2008-03-03 22:04:34 kinio Exp $                       *
 * Author: Krzysztof Kaczor                                                    *
 * Date: 24.04.2007                                                            *
 *                                                                             *
 * Implementation by Krzysztof Kaczor <kinio4444@gmail.com>                    *
 *                                                                             *
 * Copyright (C) 2006-9 by the HeKatE Project                                  *
 *                                                                             *
 * HQEd has been develped within the HeKatE Project,                           *
 * see http://hekate.ia.agh.edu.pl                                             *
 *                                                                             *
 * This file is part of HQEd.                                                  *
 *                                                                             *
 * HQEd is free software: you can redistribute it and/or modify                *
 * it under the terms of the GNU General Public License as published by        *
 * the Free Software Foundation, either version 3 of the License, or           *
 * (at your option) any later version.                                         *
 *                                                                             *
 * HQEd is distributed in the hope that it will be useful,                     *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU General Public License for more details.                                *
 *                                                                             *
 * You should have received a copy of the GNU General Public License           *
 * along with HQEd.  If not, see <http://www.gnu.org/licenses/>.               *
 *                                                                             *
 * Comment: Plik zaweira definicje obiektu bedacego komorka tabeli XTT         *
 *                                                                             *
 */

#ifndef XTT_CELL_H
#define XTT_CELL_H
// -----------------------------------------------------------------------------

#include <QtCore/QString>
#include <QStringList>
#include <QDomDocument>

#if QT_VERSION >= 0x040300
	#include <QXmlStreamWriter>
#endif

#include "XTT_Attribute.h"
// -----------------------------------------------------------------------------

class XTT_Table;
// -----------------------------------------------------------------------------

class XTT_Cell
{
private:

     /// Tabela rodzic komorki
     unsigned int fParent;

     /// Identyfiaktor komorki, 0 - not definded
     unsigned int fCellID;

     /// Atrybutu jaki moze byc wpisany w komorke
     XTT_Attribute* fAttrType;

     /// Okresla typ okreslenia wartosci atrybutu, 0 - not definded, 1 - atomic, 2 - set, 3 - interval
     int fValDefType;

     /// Przechowuje podana zawartosc komorki
     QString fContent;

     /// Okresla zawartosc komorki jezeli wartosc jest zdefiniowana jako atomic
     QString fValue;

     /// W przypadku definicji jako przedzial, okresla typ nawiasow
     bool* fBrace;

     /// W przypadku definicji jako przedzial, zawiera wartosc dolna i gorna ograniczenia
     QString fBoundaries[2];

     /// Wartosc, wartosci, zbior wartosci atrybutu
     QStringList* fAttrValue;

     /// Typ operatora przy atrybucie
     /// 0 - not defined
     /// 1 - =
     /// 2 - !=
     /// 3 - >=
     /// 4 - >
     /// 5 - <=
     /// 6 - <
     /// 7 - nin
     /// 8 - in
     unsigned int fOperator;

     /// Okresla czy komorka moze byc ignorowana
     bool fIgnore;

     /// Zwraca identyfikator kontekstu w jakim znajduje sie komorka
     /// 0 - not defined
     /// 1 - decision
     /// 2 - assert
     /// 3 - retract
     /// 4 - action
     int fContext;
	
	/// Okresla czy ma zostac wlaczony tryb porownywania atrybutow
	bool fEnableAttr2AttrComp;

     /// Zawiera ostatni komunikat o bledzie
     QString fCellError;
	
	/// Metoda ustawiajaca blad komorki
	void setCellError(QString);

     /// Metody prywatne obliczajace wartosci funckji
     QString sin_func(QString, QString&);
     QString cos_func(QString, QString&);
     QString tan_func(QString, QString&);
     QString abs_func(QString, QString&);
     QString root_func(QString, QString&);
     QString pow_func(QString, QString&);
     QString fac_func(QString, QString&);
     QString mul_func(QString, QString&);
     QString div_func(QString, QString&);
     QString add_func(QString, QString&);
     QString sub_func(QString, QString&);
     QString log_func(QString, QString&);
     QString mod_func(QString, QString&);
	
	/// Okresla czy dany obiekt zostal zmodyfikowany
	bool fChanged;
	
	/// Okresla czy komorka jest zaznaczona
	bool fIsSelected;

public:

     /// Konstruktor domyslny
     XTT_Cell(void);

     /// Konstruktor przyjmujacy rodzica, wartosc ID i kontekstu
     XTT_Cell(unsigned int, unsigned int, int);

     /// Destruktor komorki
     ~XTT_Cell(void);

     /// ----------------------- Metody obslugujace pliki -----------------------

     /// Wstawia wszystkie dane w formacie xttml 1.0
     QStringList* XTTML_1_0(void);
	
	#if QT_VERSION >= 0x040300
	/// Wstawia wszystkie dane w formacie xttml 1.0
     void XTTML_1_0(QXmlStreamWriter*);
	#endif
	
	/// Wstawia wszystkie dane w formacie xttml 2.0
     QStringList* XTTML_2_0(void);
	
	#if QT_VERSION >= 0x040300
	/// Wstawia wszystkie dane w formacie xttml 2.0
     void XTTML_2_0(QXmlStreamWriter*);
	#endif

     /// Wczytuje dane z pliku XTTML 1.0
     bool FromXTTML_1_0(QStringList*);
	
	/// Wczytuje dane z pliku XTTML 1.0
     bool FromXTTML_1_0(QDomElement&);
	
	/// Wczytuje dane z pliku XTTML 2.0
	bool FromXTTML_2_0(QStringList*);
	
	/// Wczytuje dane z pliku XTTML 2.0
	bool FromXTTML_2_0(QDomElement&);

     /// ----------------- Metody zwracajaca wartosci pol klasy -----------------

     /// Zwraca wskaznik rodzica komorki
     unsigned int Parent(void){ return fParent; }

     /// Zwraca identyfikator komorki
     unsigned int CellId(void){ return fCellID; }

     /// Zwraca zawartosc komorki
     QString Content(void){ return fContent; }

     /// Zwraca typ okreslenia wartosci atrybutu
     int ValueDefinitionType(void){ return fValDefType; }

     /// Zwraca type nawiasu rozpoczynajacego
     bool LoBraceType(void){ return fBrace[0]; }

     /// Zwraca type nawiasu zamykajacego
     bool HiBraceType(void){ return fBrace[1]; }

     /// Zwraca dolna wartosc ograniczenia w przypadku takiej definicji wartosci
     QString LoBoundary(void){ return fBoundaries[0]; }

     /// Zwraca gorna wartosc ograniczenia w przypadku takiej definicji wartosci
     QString HiBoundary(void){ return fBoundaries[1]; }

     /// Zwraca zbior stringow definiujacych watrosc
     QStringList* AttrValue(void){ return fAttrValue; }

     /// Zwraca waskaznik do atrybutu jaki moze zawierac komorka
     XTT_Attribute* AttrType(void){ return fAttrType; }

     /// Zwraca typ operatora przy atrybucie
     unsigned int OperatorType(void){ return fOperator; }

     /// Zwraca prawde/falsz okreslajaca czy dana komorka moze byc ignorowana
     bool Ignore(void){ return fIgnore; }

     /// Zwraca context w jakim pracuje komorka
     int Context(void) { return fContext; }
	
	/// Zwraca czy w komorce jest wlaczony tryb porownywania atrybutow
	bool enableAttr2AttrComp(void){ return fEnableAttr2AttrComp; }

     /// Zwraca wartosc bledu
     QString CellError(bool addLocalizationInfo = true);
	
	/// Zwraca wartosc czy dany pojmnik zostal zmodyfikowany
	bool Changed(void);
	
	/// Okresla czy komorka ma poprawna zawartosc
	bool isCorrectSyntax(void);
	
	/// Zaznacza dana komorke i przenosi do niej na scenie
	void setSelected(bool _isSelected = true);
	
	/// Zwraca czy dana komorka jest zaznaczona
	bool isSelected(void);

     /// ---------------- Metody ustawiajace wartosci pol klasy -----------------

     /// Pozwala ustawic typ atrybutu
     void setAttrType(XTT_Attribute*);

     /// Ustawia typ definicji wartosci, dodatkowo ta metoda sprawdza czy dany
     /// typ definicji moze zostac uzyty w danym kontekscie, jezeli nie to
     /// zwraca falsz, jezeli tak to zwraca prawde
     bool setValueDefinitionType(int);

     /// Ustawia zawartosc komorki
     void setContent(QString);

     /// Ustawia kontekst komorki
     bool setContext(unsigned int);

     /// Pozwala ustawic typ nawiasu otwierajacego
     void setLoBrace(bool);

     /// Pozwala ustawic typ nawiasu zamykajacego
     void setHiBrace(bool);

     /// Pozwala ustawic dolna wartosc ograniczenia w przypadku definicji wartosci jako przedzial
     void setLoBoundary(QString);

     /// Pozwala ustawic gorna wartosc ograniczenia w przypadku definicji wartosci jako przedzial
     void setHiBoundary(QString);

     /// Pozwala ustawic wartosc, wartosci, przedzial wartosci arybutu zgodnego
     /// z typem definicji okreslonej przez pole fValDefType. Metoda ta sprawdza
     /// czy ustawiana wartosc jest zgodna z typem definicji wartosci
     bool setAttrValue(QStringList);

     /// Pozwala ustawic typ operatora przy wartosci w komorce
     void setOperatorType(unsigned int);

     /// Pozwala ustawic znacznik ignorowania komorki
     void setIgnore(bool);
	
	/// Ustawia falge porownywania atrybutow
	void setEnableAttr2AttrComp(bool);

     /// Ustawienie kontekstu moze sie odbyc tylko w momencie tworzenia obiektu
     /// komorki, w przeciwnym wypadku pozniejsza jego zmiana moze prowadzic
     /// do wielu bledow w apliakcji
	
	/// Ustawia wartosc czy dany pojmnik zostal zmodyfikowany
	void setChanged(bool);

     /// -----------------------------  Inne metody -----------------------------

     /// Zwraca string bedacy reprezentacja wartosci, zbioru, badz przedzialu
     /// dodajac odowiednie nawiasy i operator
     QString ValueRepresentation(void);

     /// Metoda analizujaca skladnie
     bool Analyze(QString);

     /// Oblicza wartosc wyrazenia
     QString Calculate(QString&, int start_index = 0);

     /// Wykonywuje dana operacje
     int Execute(QString, bool cellAnalyze = true);

     /// Zwraca wskaznik do rodzica tabeli
     XTT_Table* TableParent(void);
};
// -----------------------------------------------------------------------------
#endif
