/**
 *                                                                             *
 * @file XTT_Connections.cpp                                                   *
 * Project name: HeKatE hqed                                                   *
 * $Id: XTT_Connections.cpp,v 1.12 2008-03-03 22:04:34 kinio Exp $              *
 * Author: Krzysztof Kaczor                                                    *
 * Date: 04.05.2007                                                            *
 *                                                                             *
 * Implementation by Krzysztof Kaczor <kinio4444@gmail.com>                    *
 *                                                                             *
 * Copyright (C) 2006-9 by the HeKatE Project                                  *
 *                                                                             *
 * HQEd has been develped within the HeKatE Project,                           *
 * see http://hekate.ia.agh.edu.pl                                             *
 *                                                                             *
 * This file is part of HQEd.                                                  *
 *                                                                             *
 * HQEd is free software: you can redistribute it and/or modify                *
 * it under the terms of the GNU General Public License as published by        *
 * the Free Software Foundation, either version 3 of the License, or           *
 * (at your option) any later version.                                         *
 *                                                                             *
 * HQEd is distributed in the hope that it will be useful,                     *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU General Public License for more details.                                *
 *                                                                             *
 * You should have received a copy of the GNU General Public License           *
 * along with HQEd.  If not, see <http://www.gnu.org/licenses/>.               *
 *                                                                             *
 * Comment: Plik zawierajacy definicje metod obiektu bedacego                  *
 * polaczeniami miedzy tabelami                                                *
 *                                                                             *
 */

#include <QMessageBox>
#include <QtGui>
#include <QDialog>
#include <QObject>
 
#include "../../C/MainWin_ui.h"
#include "../../V/XTT/GConnection.h"
#include "../../V/XTT/GWire.h"  

#include "XTT_Table.h"
#include "XTT_TableList.h"
#include "XTT_Connections.h"
//---------------------------------------------------------------------------

/**
 * Konstruktor domyslny klasy
 * @returns Brak zwracanych wartosci
*/
XTT_Connections::XTT_Connections(void)
{
     fConnID = 0;
     fConnCut = false;
     fConnLabel = "";
     fConnDesc = "";
     fSrcTablID = 0;
     fDesTablID = 0;
     fSrcRowID = 0;
     fDesRowID = 0;
	fChanged = false;
     fGConn = NULL;
}
//---------------------------------------------------------------------------

/**
 * Konstruktor klasy
 * @param unsigned int _id - ID polaczenia
 * @returns Brak zwracanych wartosci
*/
XTT_Connections::XTT_Connections(unsigned int _id)
{
     fConnID = _id;
     fConnCut = false;
     fConnLabel = "";
     fConnDesc = "";
     fSrcTablID = 0;
     fDesTablID = 0;
     fSrcRowID = 0;
     fDesRowID = 0;
	fChanged = false;
	fGConn = NULL;
}
//---------------------------------------------------------------------------

/**
 * Destruktor klasy
 * @returns Brak zwracanych wartosci
*/
XTT_Connections::~XTT_Connections(void)
{
     delete fGConn;
}
//---------------------------------------------------------------------------

/**
 * Metoda pozwalajaca ustawic status modyfiakcji obiektu
 * @param bool _c - Nowa wartosc statusu
 * @returns Brak zwracanych wartoci
 */
void XTT_Connections::setChanged(bool _c)
{
	fChanged = _c;
}
//---------------------------------------------------------------------------

/**
 * Metoda klasy ustawiajaca wlasciwosci polaczenia
 * @param bool _cut - Nowa wartosc okreslajaca czy polaczenie przerywa powroty
 * @returns Brak zwracanych wartoci
*/
void XTT_Connections::setCut(bool _cut)
{
     fConnCut = _cut;
	fChanged = true;
     Repaint();
}
//---------------------------------------------------------------------------

/**
 * Metoda ustawiajaca etykiete polaczenia
 * @param QString _label - Nowa wartosc etykiety
 * @returns Brak zwracanych wartoci
*/
void XTT_Connections::setLabel(QString _label)
{
     fConnLabel = _label;
	fChanged = true;
     Repaint();
}
//---------------------------------------------------------------------------

/**
 * Metoda pozwalajaca ustawic opis polaczenia
 * @param QString _desc - Nowa wartosc opisu polaczenia
 * @returns Brak zwracanych wartoci
*/
void XTT_Connections::setDescription(QString _desc)
{
     fConnDesc = _desc;
	fChanged = true;
     Repaint();
}
//---------------------------------------------------------------------------

/**
 * Metoda pozwalajaca ustawic ID tabeli zrodlowej polaczenia
 * @param unsigned int _st - ID tabeli zrodlowej
 * @returns Brak zwracanych wartoci
*/
void XTT_Connections::setSrcTable(unsigned int _st)
{
     fSrcTablID = _st;
	fChanged = true;
     Repaint();
}
//---------------------------------------------------------------------------

/**
 * Metoda pozwalajaca ustawic ID tabeli docelowej polaczenia
 * @param unsigned int _dt - ID tabeli docelowej
 * @returns Brak zwracanych wartoci
*/
void XTT_Connections::setDesTable(unsigned int _dt)
{
     fDesTablID = _dt;
	fChanged = true;
     Repaint();
}
//---------------------------------------------------------------------------

/**
 * Metoda pozwalajaca ustawic ID wiersza zrodlowego
 * @param unsigned int _dt - ID wiersza zrodlowego
 * @returns Brak zwracanych wartoci
*/
void XTT_Connections::setSrcRow(unsigned int _sr)
{
     fSrcRowID = _sr;
	fChanged = true;
     Repaint();
}
//---------------------------------------------------------------------------

/**
 * Metoda pozwalajaca ustawic ID wiersza docelowego
 * @param unsigned int _dt - ID wiersza docelowego
 * @returns Brak zwracanych wartoci
*/
void XTT_Connections::setDesRow(unsigned int _dr)
{
     fDesRowID = _dr;
	fChanged = true;
     Repaint();
}
//---------------------------------------------------------------------------

/**
 * Metoda odswiezajaca rysunek polaczenia na ekranie
 * @returns Brak zwracanych wartoci
*/
void XTT_Connections::Repaint(void)
{
     if((fSrcTablID > 0) && (fDesTablID > 0) && (fSrcRowID > 0) && (fDesRowID > 0))
	{
		if(fGConn == NULL)
			fGConn = new GConnection(this);
          fGConn->RePaint();
	}
}
//---------------------------------------------------------------------------

/**
 * Metoda odswiezajaca parametry lokalizacyjne polaczenia
 * @returns Brak zwracanych wartoci
*/
void XTT_Connections::Reposition(void)
{
	fGConn->setLocalizationParameters();
}
//---------------------------------------------------------------------------

/**
 * Metoda zapisujaca wszystkie dane w formacie xttml 1.0
 * @returns Zwraca liste stringow w tym formacie
*/
QStringList* XTT_Connections::XTTML_1_0(void)
{
     QStringList* res = new QStringList;
     res->clear();
     
     *res << "<xtt_connection id=\"" + QString::number(fConnID) + "\" cut=\"" +  QString::number(fConnCut?1:0) + "\" source_table=\"" + QString::number(fSrcTablID) + "\" destination_table=\"" + QString::number(fDesTablID) + "\" source_row=\"" + QString::number(fSrcRowID) + "\" destination_row=\"" + QString::number(fDesRowID) + "\">";
     *res << "\t<label>" + fConnLabel + "</label>";
     *res << "\t<description>" + fConnDesc + "</description>";
     *res << "</xtt_connection>";

     return res;
}
// -----------------------------------------------------------------------------

#if QT_VERSION >= 0x040300
/**
 * Metoda zapisujaca wszystkie dane w formacie xttml 1.0
 * @param QXmlStreamWriter* _xmldoc - wskanik do dokumentu XML
 * @returns brak zwracanych wartosci
*/
void XTT_Connections::XTTML_1_0(QXmlStreamWriter* _xmldoc)
{
	_xmldoc->writeStartElement("xtt_connection");
	_xmldoc->writeAttribute("id", QString::number(fConnID));
	_xmldoc->writeAttribute("cut", QString::number(fConnCut?1:0));
	_xmldoc->writeAttribute("source_table", QString::number(fSrcTablID));
	_xmldoc->writeAttribute("destination_table", QString::number(fDesTablID));
	_xmldoc->writeAttribute("source_row", QString::number(fSrcRowID));
	_xmldoc->writeAttribute("destination_row", QString::number(fDesRowID));
		_xmldoc->writeTextElement("label", fConnLabel);
		_xmldoc->writeTextElement("description", fConnDesc);
     _xmldoc->writeEndElement();
}
#endif
// -----------------------------------------------------------------------------

/**
 * Metoda zapisujaca wszystkie dane w formacie xttml 2.0
 * @returns Zwraca liste stringow w tym formacie
*/
QStringList* XTT_Connections::XTTML_2_0(void)
{
     QStringList* res = new QStringList;
     res->clear();
     
	QString iscconncut = fConnCut?"true":"false";
	
	// Wczytywanie nazw tabel rodzicow
	int srcTableIndex = TableList->IndexOfID(fSrcTablID);
	int desTableIndex = TableList->IndexOfID(fDesTablID);
	if((srcTableIndex == -1) || (desTableIndex == -1))
		return res;
	QString srcTableName = TableList->Table(srcTableIndex)->Title();
	QString desTableName = TableList->Table(desTableIndex)->Title();
	
	// Wyszukiwanie indexow wierszy poczatkowego i koncowego
	int srcRowIndex = -1;
	int desRowIndex = -1;
	for(unsigned int i=0;i<TableList->Table(srcTableIndex)->RowCount();i++)
		if(TableList->Table(srcTableIndex)->Row(i)->RowId() == fSrcRowID)
			srcRowIndex = i;
	for(unsigned int i=0;i<TableList->Table(desTableIndex)->RowCount();i++)
		if(TableList->Table(desTableIndex)->Row(i)->RowId() == fDesRowID)
			desRowIndex = i;
			
	if((srcRowIndex == -1) || (desRowIndex == -1))
		return res;
	
	// src_row i destination_row - to sa indexy wierszy nie ID !!!
     *res << "<xtt_connection cut=\"" + iscconncut + "\" source_table_name=\"" + srcTableName + "\" destination_table_name=\"" + desTableName + "\" source_row=\"" + QString::number(srcRowIndex) + "\" destination_row=\"" + QString::number(desRowIndex) + "\">";
	if(fConnLabel != "")
		*res << "\t<label>" + fConnLabel + "</label>";
	if(fConnDesc != "")
		*res << "\t<description>" + fConnDesc + "</description>";
     *res << "</xtt_connection>";

     return res;
}
// -----------------------------------------------------------------------------

#if QT_VERSION >= 0x040300
/**
 * Metoda zapisujaca wszystkie dane w formacie xttml 2.0
 * @param QXmlStreamWriter* _xmldoc - wskanik do dokumentu XML
 * @returns brak zwracanych wartosci
*/
void XTT_Connections::XTTML_2_0(QXmlStreamWriter* _xmldoc)
{
	QString iscconncut = fConnCut?"true":"false";
	
	// Wczytywanie nazw tabel rodzicow
	int srcTableIndex = TableList->IndexOfID(fSrcTablID);
	int desTableIndex = TableList->IndexOfID(fDesTablID);
	if((srcTableIndex == -1) || (desTableIndex == -1))
		return;
	QString srcTableName = TableList->Table(srcTableIndex)->Title();
	QString desTableName = TableList->Table(desTableIndex)->Title();
	
	// Wyszukiwanie indexow wierszy poczatkowego i koncowego
	int srcRowIndex = -1;
	int desRowIndex = -1;
	for(unsigned int i=0;i<TableList->Table(srcTableIndex)->RowCount();i++)
		if(TableList->Table(srcTableIndex)->Row(i)->RowId() == fSrcRowID)
			srcRowIndex = i;
	for(unsigned int i=0;i<TableList->Table(desTableIndex)->RowCount();i++)
		if(TableList->Table(desTableIndex)->Row(i)->RowId() == fDesRowID)
			desRowIndex = i;
			
	if((srcRowIndex == -1) || (desRowIndex == -1))
		return;
	
	// src_row i destination_row - to sa indexy wierszy nie ID !!!
	_xmldoc->writeStartElement("xtt_connection");
	_xmldoc->writeAttribute("cut", iscconncut);
	_xmldoc->writeAttribute("source_table_name", srcTableName);
	_xmldoc->writeAttribute("destination_table_name", desTableName);
	_xmldoc->writeAttribute("source_row", QString::number(srcRowIndex));
	_xmldoc->writeAttribute("destination_row", QString::number(desRowIndex));
	if(fConnLabel != "")
		_xmldoc->writeTextElement("label", fConnLabel);
	if(fConnDesc != "")
		_xmldoc->writeTextElement("description", fConnDesc);
     _xmldoc->writeEndElement();
}
#endif
// -----------------------------------------------------------------------------

/**
 * Metoda wczytujaca dane o strybucie z pliku w formacie XTTML 1.0
 * @param QStringList* list - Wskanik do 
 * sekcji dotyczacej tego atrybutu
 * @returns true jezeli dane sa ok, ianczej false
*/
bool XTT_Connections::FromXTTML_1_0(QStringList* list)
{
     // Usuwanie spacji i tabulatorow wierszach
     for(int i=0;i<list->size();i++)
     {
          QString line = list->at(i);
          while(line.contains("\t"))
          {
               int index = line.indexOf("\t");
               line = line.remove(index, 1);
          }
          list->replace(i, line);
     }

     // Zakladamy ze pierwszy wiersz jest naglowkiem
     if(!list->at(0).contains("<xtt_connection", Qt::CaseInsensitive))
          return false;
     if(!list->contains("</xtt_connection>", Qt::CaseInsensitive))
          return false;

     // Reszta zakladamy ze jest ok

     // Spis tagow
     QStringList tags;
     tags.clear();
     tags  << "<xtt_connection" << "<label>" << "<description>";

     for(int line=0;line<list->size();line++)
     {
          // Sprawdzanie wystapienia tagu
          int action = -1;
          for(int tag=0;tag<tags.size();tag++)
               if(list->at(line).contains(tags.at(tag), Qt::CaseInsensitive))
                    action = tag;

          if(action == -1)
               continue;

          QString end_tag = tags.at(action);
          end_tag = end_tag.insert(1, "/");
          QString l = list->at(line);
          l = l.remove(l.indexOf(tags.at(action), Qt::CaseInsensitive), tags.at(action).length());
          l = l.mid(0, l.indexOf(end_tag, Qt::CaseInsensitive));

          // Wczytywanie parametrow w zaleznosci od wykrytego tagu
          if(action == 0)
          {
               fConnID = (unsigned int)XML_ReadValue(list->at(line), "id").toInt();
               fConnCut = (bool)XML_ReadValue(list->at(line), "cut").toInt();
               fSrcTablID = (unsigned int)XML_ReadValue(list->at(line), "source_table").toInt();
               fDesTablID = (unsigned int)XML_ReadValue(list->at(line), "destination_table").toInt();
               fSrcRowID = (unsigned int)XML_ReadValue(list->at(line), "source_row").toInt();
               fDesRowID = (unsigned int)XML_ReadValue(list->at(line), "destination_row").toInt();
          }
          if(action == 1)
               fConnLabel = l;
          if(action == 2)
               fConnDesc = l;
     }

     Repaint();
     
     //QStringList* h = XTTML_1_0();
     //for(int i=0;i<h->size();i++)
     //QMessageBox::warning(NULL, "Application", h->at(i));

     return true;
}
// -----------------------------------------------------------------------------

/**
 * Metoda wczytujaca dane o strybucie z pliku w formacie XTTML 1.0
 * @param QDomElement& root - Referencja do sekcji dotyczacej tego atrybutu
 * @returns true jezeli dane sa ok, ianczej false
*/
bool XTT_Connections::FromXTTML_1_0(QDomElement& root)
{
	if(root.tagName().toLower() != "xtt_connection")
		return false;
	
	fConnID = (unsigned int)root.attribute("id", "0").toInt();
	fConnCut = (bool)root.attribute("cut", "0").toInt();
	fSrcTablID = (unsigned int)root.attribute("source_table", "0").toInt();
	fDesTablID = (unsigned int)root.attribute("destination_table", "0").toInt();
	fSrcRowID = (unsigned int)root.attribute("source_row", "0").toInt();
	fDesRowID = (unsigned int)root.attribute("destination_row", "0").toInt();
	
	QDomElement DOMlabel = root.firstChildElement("label");
	QDomElement DOMdesc = root.firstChildElement("description");
			
	fConnLabel = DOMlabel.text();
	fConnDesc = DOMdesc.text();

	// Ofswiezenie widoku
     Repaint();

     return true;
}
// -----------------------------------------------------------------------------

/**
 * Metoda wczytujaca dane o strybucie z pliku w formacie XTTML 2.0
 * @param QStringList* list - Wskanik do 
 * sekcji dotyczacej tego atrybutu
 * @returns true jezeli dane sa ok, ianczej false
*/
bool XTT_Connections::FromXTTML_2_0(QStringList* list)
{
     // Usuwanie spacji i tabulatorow wierszach
     for(int i=0;i<list->size();i++)
     {
          QString line = list->at(i);
          while(line.contains("\t"))
          {
               int index = line.indexOf("\t");
               line = line.remove(index, 1);
          }
          list->replace(i, line);
     }

     // Zakladamy ze pierwszy wiersz jest naglowkiem
     if(!list->at(0).contains("<xtt_connection", Qt::CaseInsensitive))
          return false;
     if(!list->contains("</xtt_connection>", Qt::CaseInsensitive))
          return false;

     // Reszta zakladamy ze jest ok

     // Spis tagow
     QStringList tags;
     tags.clear();
     tags  << "<xtt_connection" << "<label>" << "<description>";

     for(int line=0;line<list->size();line++)
     {
          // Sprawdzanie wystapienia tagu
          int action = -1;
          for(int tag=0;tag<tags.size();tag++)
               if(list->at(line).contains(tags.at(tag), Qt::CaseInsensitive))
                    action = tag;

          if(action == -1)
               continue;

          QString end_tag = tags.at(action);
          end_tag = end_tag.insert(1, "/");
          QString l = list->at(line);
          l = l.remove(l.indexOf(tags.at(action), Qt::CaseInsensitive), tags.at(action).length());
          l = l.mid(0, l.indexOf(end_tag, Qt::CaseInsensitive));

          // Wczytywanie parametrow w zaleznosci od wykrytego tagu
          if(action == 0)
          {
               QString sCut = XML_ReadValue(list->at(line), "cut");
               QString sSTN = XML_ReadValue(list->at(line), "source_table_name");
               QString sDTN = XML_ReadValue(list->at(line), "destination_table_name");
               QString sSR = XML_ReadValue(list->at(line), "source_row");
               QString sDR = XML_ReadValue(list->at(line), "destination_row");
			
			// Usuwanie spacji z wczytanych wartosci
			for(;sCut.contains(" ");)
				sCut = sCut.remove(sCut.indexOf(" "), 1);
			for(;sSTN.contains(" ");)
				sSTN = sSTN.remove(sSTN.indexOf(" "), 1);
			for(;sDTN.contains(" ");)
				sDTN = sDTN.remove(sDTN.indexOf(" "), 1);
			for(;sSR.contains(" ");)
				sSR = sSR.remove(sSR.indexOf(" "), 1);
			for(;sDR.contains(" ");)
				sDR = sDR.remove(sDR.indexOf(" "), 1);
				
			// Sprawdzanie poprawnosci wczytanych wartosci parametrow
			
			bool bCut = false;		// Nie wymagany, default=false
			QString stName = "";	// Wymagany
			QString dtName = "";	// Wymagany
			int iSRow = -1;		// Wymagany
			int iDRow = -1;		// Wymagany
			
			// Sprawdzanie podania czy polaczenie jest odcinajace nawroty, jezeli jest nie podane to przyjmujemy false
			if((sCut != "") && (sCut != "#NAN#") && (sCut.toLower() != "true") && (sCut.toLower() != "false"))
				QMessageBox::information(NULL, "XTT Editor", "Incorrent connection parameter \"cut\" at line: " + list->at(line) + ".\nParameter will be set as \"false\"", QMessageBox::Ok);
			bCut = (sCut.toLower() == "true");
			fConnCut = bCut;
				
				
				
			// Jezeli nie podano nazwy tabeli zrodlowej
			if((sSTN == "") || (sSTN == "#NAN#"))
			{
				QMessageBox::critical(NULL, "XTT Editor", "Missing connection parameter \"source_table_name\" at line: " + list->at(line) + ".", QMessageBox::Ok);
				return false;
			}
			// Natomiast jezeli podano to ustawianie identyfikatora
			int SrcTableIndex = TableList->IndexOfTitle(sSTN);
			if(SrcTableIndex == -1)
			{
				QMessageBox::critical(NULL, "XTT Editor", "Undefined table name \"" + sSTN + "\" at line: " + list->at(line) + ".", QMessageBox::Ok);
				return false;
			}
			if(SrcTableIndex > -1)
				fSrcTablID = TableList->Table(SrcTableIndex)->TableId();
				
				
				
			// Jezeli nie podano nazwy tabeli docelowej
			if((sDTN == "") || (sDTN == "#NAN#"))
			{
				QMessageBox::critical(NULL, "XTT Editor", "Missing connection parameter \"destination_table_name\" at line: " + list->at(line) + ".", QMessageBox::Ok);
				return false;
			}
			// Natomiast jezeli podano to ustawianie identyfikatora
			int DesTableIndex = TableList->IndexOfTitle(sDTN);
			if(DesTableIndex == -1)
			{
				QMessageBox::critical(NULL, "XTT Editor", "Undefined table name \"" + sDTN + "\" at line: " + list->at(line) + ".", QMessageBox::Ok);
				return false;
			}
			if(DesTableIndex > -1)
				fDesTablID = TableList->Table(DesTableIndex)->TableId();
			
			
			// Sprawdzanie czy podano index wiersza zrodlowego
			if((sSR == "") || (sSR == "#NAN#"))
			{
				QMessageBox::critical(NULL, "XTT Editor", "Missing connection parameter \"source_row\" at line: " + list->at(line) + ".", QMessageBox::Ok);
				return false;
			}
			// Proby konwersji jezeli podano
			if((sSR != "") && (sSR != "#NAN#"))
			{
				bool ok;
				iSRow = sSR.toInt(&ok);
				if(!ok)
				{
					QMessageBox::critical(NULL, "XTT Editor", sSR + " is not integer type.\nIncorrect connection parameter \"source_row\" type: \"" + sSR + "\" at line: " + list->at(line) + ".", QMessageBox::Ok);
					return false;
				}
				
				// Jezeli konwersja sie udala, to robujemy uzyskac identyfikator zadanego wiersza
				if((iSRow >= (int)TableList->Table(SrcTableIndex)->RowCount()) || (iSRow < 0))
				{
					QMessageBox::critical(NULL, "XTT Editor", "Incorrect source row index value: " + QString::number(iSRow) + ". Table " + TableList->Table(SrcTableIndex)->Title() + " has only " + QString::number(TableList->Table(SrcTableIndex)->RowCount()) + " rows.\nError at line: " + list->at(line) + ".", QMessageBox::Ok);
					return false;
				}
				
				// Jezeli jest ok to zapisujemy ID wiersza
				fSrcRowID = TableList->Table(SrcTableIndex)->Row(iSRow)->RowId();
			}
			
			
			// Sprawdzanie czy podano index wiersza docelowego
			if((sDR == "") || (sDR == "#NAN#"))
			{
				QMessageBox::critical(NULL, "XTT Editor", "Missing connection parameter \"destination_row\" at line: " + list->at(line) + ".", QMessageBox::Ok);
				return false;
			}
			// Proby konwersji jezeli podano
			if((sDR != "") && (sDR != "#NAN#"))
			{
				bool ok;
				iDRow = sDR.toInt(&ok);
				if(!ok)
				{
					QMessageBox::critical(NULL, "XTT Editor", sDR + " is not integer type.\nIncorrect connection parameter \"destination_row\" type: \"" + sDR + "\" at line: " + list->at(line) + ".", QMessageBox::Ok);
					return false;
				}
				
				// Jezeli konwersja sie udala, to robujemy uzyskac identyfikator zadanego wiersza
				if((iDRow >= (int)TableList->Table(DesTableIndex)->RowCount()) || (iDRow < 0))
				{
					QMessageBox::critical(NULL, "XTT Editor", "Incorrect destination row index value: " + QString::number(iDRow) + ". Table " + TableList->Table(DesTableIndex)->Title() + " has only " + QString::number(TableList->Table(DesTableIndex)->RowCount()) + " rows.\nError at line: " + list->at(line) + ".", QMessageBox::Ok);
					return false;
				}
				
				// Jezeli jest ok to zapisujemy ID wiersza
				fDesRowID = TableList->Table(DesTableIndex)->Row(iDRow)->RowId();
			}
          }
          if(action == 1)
               fConnLabel = l;
          if(action == 2)
               fConnDesc = l;
     }

     Repaint();
     
     //QStringList* h = XTTML_1_0();
     //for(int i=0;i<h->size();i++)
     //QMessageBox::warning(NULL, "Application", h->at(i));

     return true;
}
// -----------------------------------------------------------------------------

/**
 * Metoda wczytujaca dane o strybucie z pliku w formacie XTTML 2.0
 * @param QDomElement& root - Wskanik do sekcji dotyczacej tego atrybutu
 * @returns true jezeli dane sa ok, ianczej false
*/
bool XTT_Connections::FromXTTML_2_0(QDomElement& root)
{
	if(root.tagName().toLower() != "xtt_connection")
		return false;

	QString sCut = root.attribute("cut", "#NAN#");
	QString sSTN = root.attribute("source_table_name", "#NAN#");
	QString sDTN = root.attribute("destination_table_name", "#NAN#");
	QString sSR = root.attribute("source_row", "#NAN#");
	QString sDR = root.attribute("destination_row", "#NAN#");
	
	// Usuwanie spacji z wczytanych wartosci
	for(;sCut.contains(" ");)
		sCut = sCut.remove(sCut.indexOf(" "), 1);
	for(;sSTN.contains(" ");)
		sSTN = sSTN.remove(sSTN.indexOf(" "), 1);
	for(;sDTN.contains(" ");)
		sDTN = sDTN.remove(sDTN.indexOf(" "), 1);
	for(;sSR.contains(" ");)
		sSR = sSR.remove(sSR.indexOf(" "), 1);
	for(;sDR.contains(" ");)
		sDR = sDR.remove(sDR.indexOf(" "), 1);
		
	// Sprawdzanie poprawnosci wczytanych wartosci parametrow
	
	bool bCut = false;		// Nie wymagany, default=false
	QString stName = "";	// Wymagany
	QString dtName = "";	// Wymagany
	int iSRow = -1;		// Wymagany
	int iDRow = -1;		// Wymagany
	
	// Sprawdzanie podania czy polaczenie jest odcinajace nawroty, jezeli jest nie podane to przyjmujemy false
	if((sCut != "") && (sCut != "#NAN#") && (sCut.toLower() != "true") && (sCut.toLower() != "false"))
		QMessageBox::information(NULL, "XTT Editor", "Incorrent connection parameter \"cut\" at line " + QString::number(root.lineNumber()) + ".\nParameter will be set as \"false\"", QMessageBox::Ok);
	bCut = (sCut.toLower() == "true");
	fConnCut = bCut;
		
	// Jezeli nie podano nazwy tabeli zrodlowej
	if((sSTN == "") || (sSTN == "#NAN#"))
	{
		QMessageBox::critical(NULL, "XTT Editor", "Missing connection parameter \"source_table_name\" at line " + QString::number(root.lineNumber()) + ".", QMessageBox::Ok);
		return false;
	}
	// Natomiast jezeli podano to ustawianie identyfikatora
	int SrcTableIndex = TableList->IndexOfTitle(sSTN);
	if(SrcTableIndex == -1)
	{
		QMessageBox::critical(NULL, "XTT Editor", "Undefined table name \"" + sSTN + "\" at line " + QString::number(root.lineNumber()) + ".", QMessageBox::Ok);
		return false;
	}
	if(SrcTableIndex > -1)
		fSrcTablID = TableList->Table(SrcTableIndex)->TableId();
		
		
		
	// Jezeli nie podano nazwy tabeli docelowej
	if((sDTN == "") || (sDTN == "#NAN#"))
	{
		QMessageBox::critical(NULL, "XTT Editor", "Missing connection parameter \"destination_table_name\" at line " + QString::number(root.lineNumber()) + ".", QMessageBox::Ok);
		return false;
	}
	// Natomiast jezeli podano to ustawianie identyfikatora
	int DesTableIndex = TableList->IndexOfTitle(sDTN);
	if(DesTableIndex == -1)
	{
		QMessageBox::critical(NULL, "XTT Editor", "Undefined table name \"" + sDTN + "\" at line " + QString::number(root.lineNumber()) + ".", QMessageBox::Ok);
		return false;
	}
	if(DesTableIndex > -1)
		fDesTablID = TableList->Table(DesTableIndex)->TableId();
	
	
	// Sprawdzanie czy podano index wiersza zrodlowego
	if((sSR == "") || (sSR == "#NAN#"))
	{
		QMessageBox::critical(NULL, "XTT Editor", "Missing connection parameter \"source_row\" at line " + QString::number(root.lineNumber()) + ".", QMessageBox::Ok);
		return false;
	}
	// Proby konwersji jezeli podano
	if((sSR != "") && (sSR != "#NAN#"))
	{
		bool ok;
		iSRow = sSR.toInt(&ok);
		if(!ok)
		{
			QMessageBox::critical(NULL, "XTT Editor", sSR + " is not integer type.\nIncorrect connection parameter \"source_row\" type: \"" + sSR + "\" at line " + QString::number(root.lineNumber()) + ".", QMessageBox::Ok);
			return false;
		}
		
		// Jezeli konwersja sie udala, to robujemy uzyskac identyfikator zadanego wiersza
		if((iSRow >= (int)TableList->Table(SrcTableIndex)->RowCount()) || (iSRow < 0))
		{
			QMessageBox::critical(NULL, "XTT Editor", "Incorrect source row index value: " + QString::number(iSRow) + ". Table " + TableList->Table(SrcTableIndex)->Title() + " has only " + QString::number(TableList->Table(SrcTableIndex)->RowCount()) + " rows.\nError at line " + QString::number(root.lineNumber()) + ".", QMessageBox::Ok);
			return false;
		}
		
		// Jezeli jest ok to zapisujemy ID wiersza
		fSrcRowID = TableList->Table(SrcTableIndex)->Row(iSRow)->RowId();
	}
	
	
	// Sprawdzanie czy podano index wiersza docelowego
	if((sDR == "") || (sDR == "#NAN#"))
	{
		QMessageBox::critical(NULL, "XTT Editor", "Missing connection parameter \"destination_row\" at line " + QString::number(root.lineNumber()) + ".", QMessageBox::Ok);
		return false;
	}
	// Proby konwersji jezeli podano
	if((sDR != "") && (sDR != "#NAN#"))
	{
		bool ok;
		iDRow = sDR.toInt(&ok);
		if(!ok)
		{
			QMessageBox::critical(NULL, "XTT Editor", sDR + " is not integer type.\nIncorrect connection parameter \"destination_row\" type: \"" + sDR + "\" at line " + QString::number(root.lineNumber()) + ".", QMessageBox::Ok);
			return false;
		}
		
		// Jezeli konwersja sie udala, to robujemy uzyskac identyfikator zadanego wiersza
		if((iDRow >= (int)TableList->Table(DesTableIndex)->RowCount()) || (iDRow < 0))
		{
			QMessageBox::critical(NULL, "XTT Editor", "Incorrect destination row index value: " + QString::number(iDRow) + ". Table " + TableList->Table(DesTableIndex)->Title() + " has only " + QString::number(TableList->Table(DesTableIndex)->RowCount()) + " rows.\nError at line " + QString::number(root.lineNumber()) + ".", QMessageBox::Ok);
			return false;
		}
		
		// Jezeli jest ok to zapisujemy ID wiersza
		fDesRowID = TableList->Table(DesTableIndex)->Row(iDRow)->RowId();
	}
	
	QString label = root.firstChildElement("label").text();	
	QString desc = root.firstChildElement("description").text();	
	if(!label.isEmpty())
		fConnLabel = label;
	if(!desc.isEmpty())
		fConnDesc = desc;

     Repaint();
     
     //QStringList* h = XTTML_1_0();
     //for(int i=0;i<h->size();i++)
     //QMessageBox::warning(NULL, "Application", h->at(i));

     return true;
}
// -----------------------------------------------------------------------------

