 /**
 *                                                                             *
 * @file XTT_Connections.h                                                     *
 * Project name: HeKatE hqed                                                   *
 * $Id: XTT_Connections.h,v 1.12 2008-03-03 22:04:34 kinio Exp $                *
 * Author: Krzysztof Kaczor                                                    *
 * Date: 24.04.2007                                                            *
 *                                                                             *
 * Implementation by Krzysztof Kaczor <kinio4444@gmail.com>                    *
 *                                                                             *
 * Copyright (C) 2006-9 by the HeKatE Project                                  *
 *                                                                             *
 * HQEd has been develped within the HeKatE Project,                           *
 * see http://hekate.ia.agh.edu.pl                                             *
 *                                                                             *
 * This file is part of HQEd.                                                  *
 *                                                                             *
 * HQEd is free software: you can redistribute it and/or modify                *
 * it under the terms of the GNU General Public License as published by        *
 * the Free Software Foundation, either version 3 of the License, or           *
 * (at your option) any later version.                                         *
 *                                                                             *
 * HQEd is distributed in the hope that it will be useful,                     *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU General Public License for more details.                                *
 *                                                                             *
 * You should have received a copy of the GNU General Public License           *
 * along with HQEd.  If not, see <http://www.gnu.org/licenses/>.               *
 *                                                                             *
 * Comment: Plik zawiera definicje obiektu bedacego polaczeniem miedzy         *
 * tabelami                                                                    *
 *                                                                             *
 */

#ifndef XTT_CONNECTIONSH
#define XTT_CONNECTIONSH
// -----------------------------------------------------------------------------

#include <QtCore/QString>
#include <QDomDocument>

#if QT_VERSION >= 0x040300
	#include <QXmlStreamWriter>
#endif
// -----------------------------------------------------------------------------

class GConnection;
class GWire;
// -----------------------------------------------------------------------------

class XTT_Connections
{
private:

     /// ID polaczenia
     unsigned int fConnID;

     /// Okresla czy jest to polaczenie ktore blokuje nawroty
     bool fConnCut;

     /// Etykieta poczenia
     QString fConnLabel;

     /// Opis poczenia
     QString fConnDesc;

     /// Tablica poczatkowa
     unsigned int fSrcTablID;

     /// Tablica koncowa
     unsigned int fDesTablID;

     /// Wiersz tablicy poczatkowej
     unsigned int fSrcRowID;

     /// Wiersz tablicy koncowej
     unsigned int fDesRowID;

     /// Obiekt symbolizujacy dane polaczenie na scenie
     GConnection* fGConn;
	
	/// Okresla modyfikacje obiektu
	bool fChanged;

public:

     /// Konstruktor domyslny
     XTT_Connections(void);

     /// Konstruktor przyjmujacy wartosc ID
     XTT_Connections(unsigned int);

     /// Destruktor
     ~XTT_Connections(void);

     //// ----------------------- Metody obslugujace pliki -----------------------

     // Wstawia wszystkie dane w formacie xttml 1.0
     QStringList* XTTML_1_0(void);
	
	#if QT_VERSION >= 0x040300
	/// Wstawia wszystkie dane w formacie xttml 1.0
     void XTTML_1_0(QXmlStreamWriter*);
	#endif
	
	// Wstawia wszystkie dane w formacie xttml 2.0
     QStringList* XTTML_2_0(void);
	
	#if QT_VERSION >= 0x040300
	/// Wstawia wszystkie dane w formacie xttml 2.0
     void XTTML_2_0(QXmlStreamWriter*);
	#endif

     /// Wczytuje dane z pliku XTTML 1.0
     bool FromXTTML_1_0(QStringList*);
	
	/// Wczytuje dane z pliku XTTML 1.0
     bool FromXTTML_1_0(QDomElement&);
	
	/// Wczytuje dane z pliku XTTML 2.0
     bool FromXTTML_2_0(QStringList*);
	
	/// Wczytuje dane z pliku XTTML 2.0
	bool FromXTTML_2_0(QDomElement&);

     /// ----------------- Metody zwracajaca wartosci pol klasy -----------------

     /// Zwraca identyfikator polaczenia
     unsigned int ConnId(void){ return fConnID; }

     /// Zwraca ID tabeli poczatkowej
     unsigned int SrcTable(void){return fSrcTablID; }

     /// Zwraca ID tabeli koncowej
     unsigned int DesTable(void){return fDesTablID; }

     /// Zwraca ID wiersza w tabeli poczatkowej
     unsigned int SrcRow(void){return fSrcRowID; }

     /// Zwraca ID wiersza w tabeli koncowej
     unsigned int DesRow(void){return fDesRowID; }

     /// Zwraca wartosc pola fConnCut
     bool IsCut(void){ return fConnCut; }

     /// Zwraca opis polaczenia
     QString Description(void){ return fConnDesc; }

     /// Zwraca etykiete polaczenia
     QString Label(void){ return fConnLabel; }
	
	/// Zwraca status modyfiakcji obiektu
	bool Changed(void){ return fChanged; }

     /// ---------------- Metody ustawiajace wartosci pol klasy -----------------

     /// Pozwala ustawic czy polaczenie jest odcinajace czy nie
     void setCut(bool);

     /// Pozwala zmienic etykiete polaczenia
     void setLabel(QString);

     /// Pozwala zmienic opis polaczenia
     void setDescription(QString);

     /// Pozwala ustawic ID tabeli poczatkowej
     void setSrcTable(unsigned int);

     /// Pozwala ustawic ID tabeli koncowej
     void setDesTable(unsigned int);

     /// Pozwala ustawic ID wiersza w tabeli poczatkowej
     void setSrcRow(unsigned int);

     /// Pozwala ustawic ID wiersza w tabeli koncowej
     void setDesRow(unsigned int);
	
	/// Usatwia status modyfiakcji obiektu
	void setChanged(bool);

     /// -------------------------- Metody operacyjne ---------------------------

     /// Metoda ustawiajaca parametry lokalizacyjne polaczen
     void Reposition(void);
	
	/// Metoda rysujaaca polaczenie na ekranie
     void Repaint(void);
};
// -----------------------------------------------------------------------------
#endif 
