 /**
 *                                                                             *
 * @file XTT_ConnectionsList.cpp                                               *
 * Project name: HeKatE hqed                                                   *
 * $Id: XTT_ConnectionsList.cpp,v 1.12 2008-03-03 22:04:34 kinio Exp $          *
 * Author: Krzysztof Kaczor                                                    *
 * Date: 04.05.2007                                                            *
 *                                                                             *
 * Implementation by Krzysztof Kaczor <kinio4444@gmail.com>                    *
 *                                                                             *
 * Copyright (C) 2006-9 by the HeKatE Project                                  *
 *                                                                             *
 * HQEd has been develped within the HeKatE Project,                           *
 * see http://hekate.ia.agh.edu.pl                                             *
 *                                                                             *
 * This file is part of HQEd.                                                  *
 *                                                                             *
 * HQEd is free software: you can redistribute it and/or modify                *
 * it under the terms of the GNU General Public License as published by        *
 * the Free Software Foundation, either version 3 of the License, or           *
 * (at your option) any later version.                                         *
 *                                                                             *
 * HQEd is distributed in the hope that it will be useful,                     *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU General Public License for more details.                                *
 *                                                                             *
 * You should have received a copy of the GNU General Public License           *
 * along with HQEd.  If not, see <http://www.gnu.org/licenses/>.               *
 *                                                                             *
 * Comment: Zwiera definicje metod obiekt bedacego polaczeniem tabel           *
 *                                                                             *
 */
 
#include <QMessageBox>
#include <QtGui>
#include <QDialog>
#include <QObject>

#include "../../C/MainWin_ui.h"
#include "XTT_ConnectionsList.h"
//---------------------------------------------------------------------------

XTT_ConnectionsList* ConnectionsList;
//---------------------------------------------------------------------------

/**
 * Konstruktor domyslny obiektu
 * @returns Brak zwracanych wartosci
*/
XTT_ConnectionsList::XTT_ConnectionsList(void)
{
     fConnections = NULL;
     fCount = 0;
     fNextConID = 0;
}
//---------------------------------------------------------------------------

/**
 * Destruktor obiektu
 * @returns Brak zwracanych wartosci
*/
XTT_ConnectionsList::~XTT_ConnectionsList(void)
{
     if(fCount > 0)
     {
          for(unsigned int i=0;i<fCount;i++)
               delete fConnections[i];
          delete [] fConnections;
     }
}
//---------------------------------------------------------------------------

/**
 * Metoda ustawiajaca status modyfiakcji obiektu
 * @param bool _c - Nowa wartosc statusu
 * @returns Brak zwracanych wartoci
 */
void XTT_ConnectionsList::setChanged(bool _c)
{
	for(unsigned int i=0;i<fCount;i++)
		fConnections[i]->setChanged(_c);
			
	fChanged = _c;
}
//---------------------------------------------------------------------------

/**
 * Metoda klasy oprozniajaca pojemnik
 * @returns Brak zwracanych wartoci
*/
void XTT_ConnectionsList::Flush(void)
{
     if(fCount == 0)
          return;

     for(unsigned int i=0;i<fCount;i++)
          delete fConnections[i];
     delete [] fConnections;
	
	// Rejestracja zmian
	fChanged = true;

     fCount = 0;
     fNextConID = 0;
}
//---------------------------------------------------------------------------

/**
 * Metoda klasy zwracajaca wskaznik do wybranej tabeli
 * @param unsigned int _index - Index wybranej tabeli
 * @returns Wskaznik do wybranej tabeli, jezeli index jest niepoprawny to 
 * zwraca NULL
*/
XTT_Connections* XTT_ConnectionsList::Connection(unsigned int _index)
{
     if((((int)_index) < 0) || (_index >= fCount))
          return NULL;

     return fConnections[_index];
}
//---------------------------------------------------------------------------

/**
 * Metoda zwraca wartosc statusu modyfiakcji obiektu
 * @returns true jezeli obiekt jest zmodyfikowany, inaczej false
 */
bool XTT_ConnectionsList::Changed(void)
{
	for(unsigned int i=0;i<fCount;i++)
		if(fConnections[i]->Changed())
			return true;
			
	return fChanged;
}
//---------------------------------------------------------------------------

/**
 * Metoda klasy dodajaca nowa tabele do pojemnika
 * @returns Brak zwracanych wartoci
*/
void XTT_ConnectionsList::Add(void)
{
     XTT_Connections** tmp = new XTT_Connections*[fCount+1];

     // Kopiowanie zawartosci
     for(unsigned int i=0;i<fCount;i++)
          tmp[i] = fConnections[i];

     // Usuniecie starej tablicy wskaznikow
     if(fCount)
          delete [] fConnections;

     // Zapisanie nowej lokalizacji
     fConnections = tmp;

     // Utworenie nowego elementu
     fConnections[fCount] = new XTT_Connections(++fNextConID);

     // Powiekszenie pojemnika
     fCount++;
	
	// Rejestracja zmian
	fChanged = true;
}
//---------------------------------------------------------------------------

/**
 * Metoda klasy dodajaca nowa tabele do pojemnika
 * @param unsigned XTT_Connections* _newConn - Wskaznik do nowego polaczenia
 * @returns Brak zwracanych wartoci
*/
void XTT_ConnectionsList::Add(XTT_Connections* _newConn)
{
	XTT_Connections** tmp = new XTT_Connections*[fCount+1];

     // Kopiowanie zawartosci
     for(unsigned int i=0;i<fCount;i++)
          tmp[i] = fConnections[i];

     // Usuniecie starej tablicy wskaznikow
     if(fCount)
          delete [] fConnections;

     // Zapisanie nowej lokalizacji
     fConnections = tmp;

     // Utworenie nowego elementu
     fConnections[fCount] = _newConn;

     // Powiekszenie pojemnika
     fCount++;
	
	// Rejestracja zmian
	fChanged = true;
}
//---------------------------------------------------------------------------

/**
 * Metoda klasy usuwajaca wybrana tabele z pojemnika
 * @param unsigned int _index - Index wybranej tabeli
 * @returns true jezeli inex jest ok, inaczej false
*/
bool XTT_ConnectionsList::Delete(unsigned int _index)
{
     if((((int)_index) < 0) || (_index >= fCount))
          return false;

     XTT_Connections** tmp = new XTT_Connections*[fCount-1];

     // Kopiowanie zawartosci
     int tmp_i = 0;
     for(unsigned int i=0;i<fCount;i++)
          if(i != _index)
               tmp[tmp_i++] = fConnections[i];

     // Usuniecie starej tablicy wskaznikow
     if(fCount)
     {
          delete fConnections[_index];
          delete [] fConnections;
     }

     // Zapisanie nowej lokalizacji
     fConnections = tmp;

     // Pomniejszenie pojemnika
     fCount--;
	
	// Rejestracja zmian
	fChanged = true;

     return true;
}
//---------------------------------------------------------------------------

/**
 * Metoda klasy zwracajaca index tabeli o podanym ID
 * @param unsigned int _id - ID wybranej tabeli
 * @returns Index w pojemniku, jezeli nie znajdzie to -1
*/
int XTT_ConnectionsList::IndexOfID(unsigned int _id)
{
     for(unsigned int i=0;i<fCount;i++)
          if(fConnections[i]->ConnId() == _id)
               return i;

     return -1;
}
//---------------------------------------------------------------------------

/**
 * Metoda klasy zwracajaca wskaznik do tabeli o podanym ID
 * @param unsigned int _id - ID wybranej tabeli
 * @returns Wskaznik do tabeli, jezeli nie znajdzie to NULL
*/
XTT_Connections* XTT_ConnectionsList::_IndexOfID(unsigned int _id)
{
     int index = IndexOfID(_id);
     if(index == -1)
          return NULL;

     return fConnections[index];
}
//---------------------------------------------------------------------------

/**
 * Metoa klasy wyszukujaca i zwracajaca polaczenie wychodzace z podanej tabeli 
 * i podanego wiersza
 * @param unsigned int _tid - ID tabeli
 * @param unsigned int _rid - ID wiersza
 * @returns Index w pojemniku polaczenia, jezeli nie znajdzie to -1
*/
int XTT_ConnectionsList::IndexOfOutConnections(unsigned int _tid, unsigned int _rid)
{
     for(unsigned int i=0;i<fCount;i++)
          if((fConnections[i]->SrcTable() == _tid) && (fConnections[i]->SrcRow() == _rid))
               return i;

     return -1;
}
//---------------------------------------------------------------------------

/**
 * Metoa klasy wyszukujaca i zwracajaca polaczenie wychodzace z podanej tabeli
 * i podanego wiersza
 * @param unsigned int _tid - ID tabeli
 * @param unsigned int _rid - ID wiersza
 * @returns Wskaznik polaczenia, jezeli nie znajdzie to NULL
*/
XTT_Connections* XTT_ConnectionsList::_IndexOfOutConnections(unsigned int _tid, unsigned int _rid)
{
     int index = IndexOfOutConnections(_tid, _rid);
     if(index == -1)
          return NULL;

     return fConnections[index];
}
//---------------------------------------------------------------------------

/**
 * Metoa klasy wyszukujaca i zwracajaca polaczenie wchodzace do podanej tabeli i podanego wiersza
 * @param unsigned int _tid - ID tabeli
 * @param unsigned int _rid - ID wiersza
 * @returns Index w pojemniku polaczenia, jezeli nie znajdzie to -1
*/
QList<int>* XTT_ConnectionsList::IndexOfInConnections(unsigned int _tid, unsigned int _rid)
{
	QList<int>* res = new QList<int>;

     for(unsigned int i=0;i<fCount;i++)
          if((fConnections[i]->DesTable() == _tid) && (fConnections[i]->DesRow() == _rid))
               res->append(fConnections[i]->ConnId());

     return res;
}
//---------------------------------------------------------------------------

/**
 * Metoa klasy wyszukujaca i zwracajaca polaczenie wchodzace z podanej tabeli i podanego wiersza
 * @param unsigned int _tid - ID tabeli
 * @param unsigned int _rid - ID wiersza
 * @returns Wskaznik polaczenia, jezeli nie znajdzie to NULL
*/
QList<XTT_Connections*>* XTT_ConnectionsList::_IndexOfInConnections(unsigned int _tid, unsigned int _rid)
{
     QList<int>* list = IndexOfInConnections(_tid, _rid);
	QList<XTT_Connections*>* res = new QList<XTT_Connections*>;
	
	for(int i=0;i<list->size();i++)
		res->append(_IndexOfID(list->at(i)));
		
	delete list;
     return res;
}
//---------------------------------------------------------------------------

/**
 * Metoa wyszukuje i zwraca liste polaczen wychodzacych z danej tabeli
 * @param unsigned int _tid - ID tabeli
 * @returns Wskaznik zwraca liste polaczen wychodzacych z danej tabeli
*/
QList<XTT_Connections*>* XTT_ConnectionsList::listOfOutConnections(unsigned int _tid)
{
	QList<XTT_Connections*>* res = new QList<XTT_Connections*>;
	
	for(unsigned int i=0;i<fCount;i++)
          if(fConnections[i]->SrcTable() == _tid)
               res->append(fConnections[i]);
			
	return res;
}
//---------------------------------------------------------------------------

/**
 * Usuwa wszystkie polaczenia wychodzace z wiersza o danym ID
 * @param unsigned int _rid - ID wiersza
 * @returns Brak zwracanych wartoci
*/
void XTT_ConnectionsList::RemoveOutConnectionsWithRowID(unsigned int _rid)
{
     for(int i=fCount-1;i>=0;i--)
          if(fConnections[i]->SrcRow() == _rid)
               Delete(i);
}
//---------------------------------------------------------------------------

/**
 * Usuwa wszystkie polaczenia przychodzace do wiersza o danym ID
 * @param unsigned int _rid - ID wiersza
 * @returns Brak zwracanych wartoci
*/
void XTT_ConnectionsList::RemoveInConnectionsWithRowID(unsigned int _rid)
{
     for(int i=fCount-1;i>=0;i--)
          if(fConnections[i]->DesRow() == _rid)
               Delete(i);
}
//---------------------------------------------------------------------------

/**
 * Usuwa wszystkie polaczenia z wierszem o danym ID
 * @param unsigned int _rid - ID wiersza
 * @returns Brak zwracanych wartoci
*/
void XTT_ConnectionsList::RemoveConnectionsWithRowID(unsigned int _rid)
{
     for(int i=fCount-1;i>=0;i--)
          if((fConnections[i]->SrcRow() == _rid) || (fConnections[i]->DesRow() == _rid))
               Delete(i);
}
//---------------------------------------------------------------------------

/**
 * Odswieza wszystkie polaczenia powiazane z dana tabela
 * @param unsigned int _tid - ID danej tabeli
 * @returns Brak zwracanych wartoci
*/
void XTT_ConnectionsList::RepaintConnectionRelatedWithTable(unsigned int _tid)
{
     for(unsigned int i=0;i<fCount;i++)
          if((fConnections[i]->SrcTable() == _tid) || (fConnections[i]->DesTable() == _tid))
		{
			fConnections[i]->Reposition();
               fConnections[i]->Repaint();
		}
}
//---------------------------------------------------------------------------

/**
 * Metoda zapisujaca wszystkie dane w formacie xttml 1.0
 * @returns Zwraca liste stringow w tym formacie
*/
QStringList* XTT_ConnectionsList::XTTML_1_0(void)
{
     QStringList* res = new QStringList;
     res->clear();

     *res << "<xtt_list_connections next_connection_id=\"" + QString::number(fNextConID) + "\">";

     // Zapis atrybutow
     for(unsigned int a=0;a<fCount;a++)
     {
          QStringList* al = fConnections[a]->XTTML_1_0();
          for(int l=0;l<al->size();l++)
               *res << "\t" + al->at(l);
          delete al;
     }
     
     *res << "</xtt_list_connections>";

     return res;
}
// -----------------------------------------------------------------------------

#if QT_VERSION >= 0x040300
/**
 * Metoda zapisujaca wszystkie dane w formacie xttml 1.0
 * @param QXmlStreamWriter* _xmldoc - wskanik do dokumentu XML
 * @returns brak zwracanych wartosci
*/
void XTT_ConnectionsList::XTTML_1_0(QXmlStreamWriter* _xmldoc)
{
	_xmldoc->writeStartElement("xtt_list_connections");
	_xmldoc->writeAttribute("next_connection_id", QString::number(fNextConID));

     // Zapis atrybutow
     for(unsigned int a=0;a<fCount;a++)
          fConnections[a]->XTTML_1_0(_xmldoc);
     
     _xmldoc->writeEndElement();
}
#endif
// -----------------------------------------------------------------------------

/**
 * Metoda zapisujaca wszystkie dane w formacie xttml 2.0
 * @returns Zwraca liste stringow w tym formacie
*/
QStringList* XTT_ConnectionsList::XTTML_2_0(void)
{
     QStringList* res = new QStringList;
     res->clear();

     *res << "<xtt_list_connections>";

     // Zapis atrybutow
     for(unsigned int a=0;a<fCount;a++)
     {
          QStringList* al = fConnections[a]->XTTML_2_0();
          for(int l=0;l<al->size();l++)
               *res << "\t" + al->at(l);
          delete al;
     }
     
     *res << "</xtt_list_connections>";

     return res;
}
// -----------------------------------------------------------------------------

#if QT_VERSION >= 0x040300
/**
 * Metoda zapisujaca wszystkie dane w formacie xttml 1.0
 * @param QXmlStreamWriter* _xmldoc - wskanik do dokumentu XML
 * @returns brak zwracanych wartosci
*/
void XTT_ConnectionsList::XTTML_2_0(QXmlStreamWriter* _xmldoc)
{
	_xmldoc->writeStartElement("xtt_list_connections");
	
     // Zapis atrybutow
     for(unsigned int a=0;a<fCount;a++)
          fConnections[a]->XTTML_2_0(_xmldoc);
     
     _xmldoc->writeEndElement();
}
#endif
// -----------------------------------------------------------------------------

/**
 * Metoda wczytujaca dane o strybucie z pliku w formacie XTTML 1.0
 * @param QStringList* list - Wskanik do sekcji dotyczacej tego atrybutu
 * @returns true jezeli dane sa ok, ianczej false
*/
bool XTT_ConnectionsList::FromXTTML_1_0(QStringList* list)
{
     bool res = true;
     
     // Wyczyszczenie zawartosci pojemnika
     Flush();
     
     // Usuwanie spacji i tabulatorow wierszach
     for(int i=0;i<list->size();i++)
     {
          QString line = list->at(i);
          while(line.contains("\t"))
          {
               int index = line.indexOf("\t");
               line = line.remove(index, 1);
          }
          list->replace(i, line);
     }

     // Zakladamy ze pierwszy wiersz jest naglowkiem
     if(!list->at(0).contains("<xtt_list_connections", Qt::CaseInsensitive))
          return false;
     if(!list->contains("</xtt_list_connections>", Qt::CaseInsensitive))
          return false;

     // Reszta zakladamy ze jest ok

     // Spis tagow
     QStringList tags;
     tags.clear();
     tags << "<xtt_list_connections" << "<xtt_connection" << "</xtt_connection>";

     // Sluza do odpowiedniego kopiowania lini dla listy atrybutow
     bool copy = false;
     QStringList buf;
     buf.clear();

     for(int line=0;line<list->size();line++)
     {
          // Sprawdzanie wystapienia tagu
          int action = -1;
          for(int tag=0;tag<tags.size();tag++)
               if(list->at(line).contains(tags.at(tag), Qt::CaseInsensitive))
                    action = tag;

          // Kopiowanie lini danych
          if(copy)
               buf.append(list->at(line));

          if(action == -1)
               continue;

          QString end_tag = tags.at(action);
          end_tag = end_tag.insert(1, "/");
          QString l = list->at(line);
          l = l.remove(l.indexOf(tags.at(action), Qt::CaseInsensitive), tags.at(action).length());
          l = l.mid(0, l.indexOf(end_tag, Qt::CaseInsensitive));

          // Wczytywanie parametrow w zaleznosci od wykrytego tagu
          if(action == 0)
               fNextConID = (unsigned int)XML_ReadValue(list->at(line), "next_connection_id").toInt();
          if(action == 1)
          {
               // Jezeli tag rozpoczynajacy dane, to uruchamiamy kopiowanie
               copy = true;
               buf.append(list->at(line));
          }
          if(action == 2)
          {
               // Kiedy linia konczaca linie danych
               // Dodajemy nowy atrybut
               Add();

               // Wczytujemy dane z pliku
               res = res && fConnections[fCount-1]->FromXTTML_1_0(&buf);

               // Konczymy kopiowanie
               copy = false;

               // Czyscimy zawartosc danych
               buf.clear();
          }
     }

     return res;
}
// -----------------------------------------------------------------------------

/**
 * Metoda wczytujaca dane o strybucie z pliku w formacie XTTML 1.0
 * @param QDomElement& root - Referencja do sekcji dotyczacej tego atrybutu
 * @returns true jezeli dane sa ok, ianczej false
*/
bool XTT_ConnectionsList::FromXTTML_1_0(QDomElement& root)
{
	if(root.tagName().toLower() != "xtt_list_connections")
		return false;
		
     bool res = true;
     
     // Wyczyszczenie zawartosci pojemnika
     Flush();
	
	fNextConID = (unsigned int)root.attribute("next_connection_id", "0").toInt();

	QDomElement DOMconn = root.firstChildElement("xtt_connection");
     while(!DOMconn.isNull())
     {
		// Kiedy linia konczaca linie danych
		// Dodajemy nowy atrybut
		Add();

		// Wczytujemy dane z pliku
		res = res && fConnections[fCount-1]->FromXTTML_1_0(DOMconn);
		
		// Wczytanie nastepnego polaczenia
		DOMconn = DOMconn.nextSiblingElement("xtt_connection");
     }

     return res;
}
// -----------------------------------------------------------------------------

/**
 * Metoda wczytujaca dane o strybucie z pliku w formacie XTTML 2.0
 * @param QStringList* list - Wskanik do sekcji dotyczacej tego atrybutu
 * @returns true jezeli dane sa ok, ianczej false
*/
bool XTT_ConnectionsList::FromXTTML_2_0(QStringList* list)
{
     bool res = true;
     
     // Wyczyszczenie zawartosci pojemnika
     Flush();
     
     // Usuwanie spacji i tabulatorow wierszach
     for(int i=0;i<list->size();i++)
     {
          QString line = list->at(i);
          while(line.contains("\t"))
          {
               int index = line.indexOf("\t");
               line = line.remove(index, 1);
          }
          list->replace(i, line);
     }

     // Zakladamy ze pierwszy wiersz jest naglowkiem
     if(!list->at(0).contains("<xtt_list_connections>", Qt::CaseInsensitive))
          return false;
     if(!list->contains("</xtt_list_connections>", Qt::CaseInsensitive))
          return false;

     // Reszta zakladamy ze jest ok

     // Spis tagow
     QStringList tags;
     tags.clear();
     tags << "<xtt_list_connections>" << "<xtt_connection" << "</xtt_connection>";

     // Sluza do odpowiedniego kopiowania lini dla listy atrybutow
     bool copy = false;
     QStringList buf;
     buf.clear();

     for(int line=0;line<list->size();line++)
     {
          // Sprawdzanie wystapienia tagu
          int action = -1;
          for(int tag=0;tag<tags.size();tag++)
               if(list->at(line).contains(tags.at(tag), Qt::CaseInsensitive))
                    action = tag;

          // Kopiowanie lini danych
          if(copy)
               buf.append(list->at(line));

          if(action == -1)
               continue;

          QString end_tag = tags.at(action);
          end_tag = end_tag.insert(1, "/");
          QString l = list->at(line);
          l = l.remove(l.indexOf(tags.at(action), Qt::CaseInsensitive), tags.at(action).length());
          l = l.mid(0, l.indexOf(end_tag, Qt::CaseInsensitive));

          // Wczytywanie parametrow w zaleznosci od wykrytego tagu
          if(action == 1)
          {
               // Jezeli tag rozpoczynajacy dane, to uruchamiamy kopiowanie
               copy = true;
               buf.append(list->at(line));
          }
          if(action == 2)
          {
               // Kiedy linia konczaca linie danych
               // Dodajemy nowy atrybut
               Add();

               // Wczytujemy dane z pliku
               res = res && fConnections[fCount-1]->FromXTTML_2_0(&buf);

               // Konczymy kopiowanie
               copy = false;

               // Czyscimy zawartosc danych
               buf.clear();
          }
     }

     return res;
}
// -----------------------------------------------------------------------------

/**
 * Metoda wczytujaca dane o strybucie z pliku w formacie XTTML 2.0
 * @param QDomElement& root - Wskanik do sekcji dotyczacej tego atrybutu
 * @returns true jezeli dane sa ok, ianczej false
*/
bool XTT_ConnectionsList::FromXTTML_2_0(QDomElement& root)
{
	if(root.tagName().toLower() != "xtt_list_connections")
		return false;

     bool res = true;
     
     // Wyczyszczenie zawartosci pojemnika
     Flush();
	
	QDomElement child = root.firstChildElement("xtt_connection");
     while(!child.isNull())
     {
		// Kiedy linia konczaca linie danych
		// Dodajemy nowy atrybut
		Add();

		// Wczytujemy dane z pliku
		res = res && fConnections[fCount-1]->FromXTTML_2_0(child);

		// Wczytywanie nastepnego polaczenia
		child = child.nextSiblingElement("xtt_connection");
     }

     return res;
}
// -----------------------------------------------------------------------------
