 /**
 *                                                                             *
 * @file XTT_ConnectionsList.h                                                 *
 * Project name: HeKatE hqed                                                   *
 * $Id: XTT_ConnectionsList.h,v 1.12 2008-03-03 22:04:34 kinio Exp $            *
 * Author: Krzysztof Kaczor                                                    *
 * Date: 04.05.2007                                                            *
 *                                                                             *
 * Implementation by Krzysztof Kaczor <kinio4444@gmail.com>                    *
 *                                                                             *
 * Copyright (C) 2006-9 by the HeKatE Project                                  *
 *                                                                             *
 * HQEd has been develped within the HeKatE Project,                           *
 * see http://hekate.ia.agh.edu.pl                                             *
 *                                                                             *
 * This file is part of HQEd.                                                  *
 *                                                                             *
 * HQEd is free software: you can redistribute it and/or modify                *
 * it under the terms of the GNU General Public License as published by        *
 * the Free Software Foundation, either version 3 of the License, or           *
 * (at your option) any later version.                                         *
 *                                                                             *
 * HQEd is distributed in the hope that it will be useful,                     *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU General Public License for more details.                                *
 *                                                                             *
 * You should have received a copy of the GNU General Public License           *
 * along with HQEd.  If not, see <http://www.gnu.org/licenses/>.               *
 *                                                                             *
 * Comment: Zawiera definicje ciala klasy bedacej polaczeniem tabel            *
 *                                                                             *
 */


#ifndef XTT_ConnectionsListH
#define XTT_ConnectionsListH
//---------------------------------------------------------------------------

#include <QDomDocument>

#if QT_VERSION >= 0x040300
	#include <QXmlStreamWriter>
#endif

#include "XTT_Connections.h"
//---------------------------------------------------------------------------

class XTT_ConnectionsList
{
private:

     /// Tablica wskaznkow do polaczen
     XTT_Connections** fConnections;

     /// Wielkosc zbiornika
     unsigned int fCount;

     /// Nastepne id polaczenia
     unsigned int fNextConID;
	
	/// Okresla modyfikacje obiektu
	bool fChanged;

public:

     /// Konstruktor
     XTT_ConnectionsList(void);

     /// Destruktor
     ~XTT_ConnectionsList(void);

     /// ----------------------- Metody obslugujace pliki -----------------------

     /// Wstawia wszystkie dane w formacie xttml 1.0
     QStringList* XTTML_1_0(void);
	
	#if QT_VERSION >= 0x040300
	/// Wstawia wszystkie dane w formacie xttml 1.0
     void XTTML_1_0(QXmlStreamWriter*);
	#endif
	
	/// Wstawia wszystkie dane w formacie xttml 2.0
	QStringList* XTTML_2_0(void);
	
	#if QT_VERSION >= 0x040300
	/// Wstawia wszystkie dane w formacie xttml 1.0
     void XTTML_2_0(QXmlStreamWriter*);
	#endif

     /// Wczytuje dane z pliku XTTML 1.0
     bool FromXTTML_1_0(QStringList*);
	
	/// Wczytuje dane z pliku XTTML 1.0
     bool FromXTTML_1_0(QDomElement&);
	
	/// Wczytuje dane z pliku XTTML 2.0
     bool FromXTTML_2_0(QStringList*);
	
	/// Wczytuje dane z pliku XTTML 2.0
	bool FromXTTML_2_0(QDomElement&);

     /// ----------------- Metody zwracajaca wartosci pol klasy -----------------

     /// Zwraca nastepne id polaczenia
     unsigned int NextConID(void){ return fNextConID; }

     /// Zwraca rozmiar pojemnika
     unsigned int Count(void){ return fCount; }

     /// Zwraca wskaznik do polaczenia
     XTT_Connections* Connection(unsigned int);
	
	/// Zwraca wartosc statusu modyfikcji obiektu
	bool Changed(void);

     /// -------------------------- Metody operacyjne ---------------------------
	
	/// Ustaiwa status modyfiakcji obiektu
	void setChanged(bool);

     /// Dodaje nowe polaczenie
     void Add(void);
	
	/// Dodaje nowe polaczenie
     void Add(XTT_Connections*);

     /// Usuwa wybrane polaczenie
     bool Delete(unsigned int);

     /// Oproznia bufor
     void Flush(void);

     /// Zwraca index polaczenia o podanym ID
     int IndexOfID(unsigned int);

     /// Zwraca wskaznik polaczenia o podanym ID
     XTT_Connections* _IndexOfID(unsigned int);

     /// Zwraca index polaczenia do podnaej tabeli i podanego wiersza
     int IndexOfOutConnections(unsigned int, unsigned int);

     /// Zwraca index polaczenia do podnaej tabeli i podanego wiersza
     QList<int>* IndexOfInConnections(unsigned int, unsigned int);

     /// Zwraca wskaznik polaczenia do podnaej tabeli i podanego wiersza
     XTT_Connections* _IndexOfOutConnections(unsigned int, unsigned int);

     /// Zwraca wskaznik polaczenia do podnaej tabeli i podanego wiersza
     QList<XTT_Connections*>* _IndexOfInConnections(unsigned int, unsigned int);
	
	/// Zwraca liste polaczen wychodzacych z danej tabeli
	QList<XTT_Connections*>* listOfOutConnections(unsigned int);

     /// Usuwa wszystkie polaczenia z danym wieszem
     void RemoveConnectionsWithRowID(unsigned int);
	
	/// Usuwa wszystkie polaczenia z danym wieszem
     void RemoveOutConnectionsWithRowID(unsigned int);
	
	/// Usuwa wszystkie polaczenia z danym wieszem
     void RemoveInConnectionsWithRowID(unsigned int);

     /// Odswieza polaczenia z dana tabela
     void RepaintConnectionRelatedWithTable(unsigned int);
};
//---------------------------------------------------------------------------

extern XTT_ConnectionsList* ConnectionsList;
//---------------------------------------------------------------------------
#endif
