 /**
 *                                                                             *
 * @file XTT_Row.cpp                                                           *
 * Project name: HeKatE hqed                                                   *
 * $Id: XTT_Row.cpp,v 1.12 2008-03-03 22:04:34 kinio Exp $                      *
 * Author: Krzysztof Kaczor                                                    *
 * Date: 03.05.2007                                                            *
 *                                                                             *
 * Implementation by Krzysztof Kaczor <kinio4444@gmail.com>                    *
 *                                                                             *
 * Copyright (C) 2006-9 by the HeKatE Project                                  *
 *                                                                             *
 * HQEd has been develped within the HeKatE Project,                           *
 * see http://hekate.ia.agh.edu.pl                                             *
 *                                                                             *
 * This file is part of HQEd.                                                  *
 *                                                                             *
 * HQEd is free software: you can redistribute it and/or modify                *
 * it under the terms of the GNU General Public License as published by        *
 * the Free Software Foundation, either version 3 of the License, or           *
 * (at your option) any later version.                                         *
 *                                                                             *
 * HQEd is distributed in the hope that it will be useful,                     *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU General Public License for more details.                                *
 *                                                                             *
 * You should have received a copy of the GNU General Public License           *
 * along with HQEd.  If not, see <http://www.gnu.org/licenses/>.               *
 *                                                                             *
 * Comment: Plik zawiera definicje metod obiektu row bedacego wierszem         *
 * tabeli                                                                      *
 *                                                                             *
 */

#include <QMessageBox>
#include <QtGui>
#include <QDialog>
#include <QObject>

#include "../../C/MainWin_ui.h"
#include "XTT_Row.h"
//---------------------------------------------------------------------------

/**
 * Konstruktor defaultowy klasy XTT_Row
 * @returns Brak zwracanych wartosci
*/
XTT_Row::XTT_Row(void)
{
     fCells = NULL;
     fLabel = "";
     fRowLength = 0;
     fRowID = 0;
     fNextCellId = NULL;
     fParent = 0;
	fChanged = false;
}
//---------------------------------------------------------------------------

/**
 * Konstruktor klasy XTT_Row ustawiajacy od razy identyfikatory
 * @param unsigned int _parent_id - Identyfikator rodzica wiersz, tabeli
 * @param unsigned int _id - Identyfikaotr wiersza
 * @returns Brak zwracanych wartosci
*/
XTT_Row::XTT_Row(unsigned int _parent_id, unsigned int _id, unsigned int* next_cell_id)
{
     fCells = NULL;
     fLabel = "";
     fRowLength = 0;
     fRowID = _id;
     fNextCellId = next_cell_id;
     fParent = _parent_id;
	fChanged = false;
}
//---------------------------------------------------------------------------

/**
 * Destrkutor klasy
 * @returns Brak zwracanych wartosci
*/
XTT_Row::~XTT_Row(void)
{
     // Usuwanie polaczen dolaczonych do tego wiersz
     ConnectionsList->RemoveConnectionsWithRowID(fRowID);

     if(!fRowLength)
          return;

     for(unsigned int i=0;i<fRowLength;i++)
          delete fCells[i];
     delete [] fCells;
}
//---------------------------------------------------------------------------

/**
 * Metoda klasy XTT_Row zwracajaca wskaznik do wybranej komorki
 * @param int index - Index wybranej komorki
 * @returns Wskaznik do wybranej komorki, jezeli index jest niepoprawny zwraca 
 * NULL
*/
XTT_Cell* XTT_Row::Cell(int index)
{
     // Sprawdzanie czy index jest ok
     if((index < 0) || (index >= (int)fRowLength))
          return NULL;

     return fCells[index];
}
//---------------------------------------------------------------------------

/**
 * Metoda okreslajaca czy dany wiersz byl zmodyfikowany
 * @returns true jezeli byl modyfikowany, inaczej false
 */
bool XTT_Row::Changed(void)
{
	for(unsigned int i=0;i<fRowLength;i++)
		if(fCells[i]->Changed())
			return true;
			
	return fChanged;
}
//---------------------------------------------------------------------------

/**
 * Metoda ustawiajaca wartosc modyfikacji obiektu
 * @param bool _c - Nowa wartosc
 * @returns Brak zwracanych wartoci
 */
void XTT_Row::setChanged(bool _c)
{
	for(unsigned int i=0;i<fRowLength;i++)
		fCells[i]->setChanged(_c);
			
	fChanged = _c;
}
//---------------------------------------------------------------------------

/**
 * Metoda klasy XTT_Row wstawiajaca wiersz w danym miejscu
 * @param unsigned int index - Index miejsca w ktore nalezy wstawic komorke
 * @param int context - Kontekst jaki bedzie podany komorce
 * @returns true jezeli wszystko ok, naczej false
*/
bool XTT_Row::Insert(unsigned int index, int context)
{
     // Sprawdzanie czy index jest ok
     if((((int)index) < 0) || (index > fRowLength))
          return false;

     XTT_Cell** tmp = new XTT_Cell*[fRowLength+1];
     int tmp_i = 0;

     // Kopiowanie starej tablicy
     for(unsigned int i=0;i<=fRowLength;i++)
          if(i != index)
               tmp[i] = fCells[tmp_i++];

     // Usuwanie strej tablicy, jesli nie pusta
     if(fRowLength)
          delete [] fCells;

     // Zapisanie lokalizacji
     fCells = tmp;

     // Utworzenie nowego obiektu
     fCells[index] = new XTT_Cell(fParent, ++(*fNextCellId), context);

     // Powiekszenie tablicy
     fRowLength++;
	
	// Zarejestrowanie zmiany obiektu
	fChanged = true;

     // Zwrocenie ze wszystko sie udalo
     return true;
}
//---------------------------------------------------------------------------

/**
 * Metoda klasy XTT_Row usuwajaca dana komorke
 * @param unsigned int index - Index komorki kora trzeba usunac
 * @returns true jeseli wszystko ok, inaczej false
*/
bool XTT_Row::Delete(unsigned int index)
{
     // Sprawdzanie czy index jest ok
     if((((int)index) < 0) || (index >= fRowLength))
          return false;

     XTT_Cell** tmp = new XTT_Cell*[fRowLength-1];
     int tmp_i = 0;

     // Kopiowanie starej tablicy
     for(unsigned int i=0;i<fRowLength;i++)
          if(i != index)
               tmp[tmp_i++] = fCells[i];

     // Usuwanie strej tablicy, jesli nie pusta
     if(fRowLength)
     {
          delete fCells[index];
          delete [] fCells;
     }

     // Zapisanie lokalizacji
     fCells = tmp;

     // Pomniejszenie tablicy
     fRowLength--;
	
	// Zarejestrowanie zmiany obiektu
	fChanged = true;

     // Zwrocenie ze wszystko sie udalo
     return true;
}
//---------------------------------------------------------------------------

/**
 * Metoda klasy XTT_Row pozwalajaca na zamiane ze soba dwoch komorek
 * @param unsigned int _index1 - Index pierwszej komorki
 * @param unsigned int _index2 - Index drugiej komorki
 * @returns true jezeli podane ndexy sa poprawne, inaczej false
*/
bool XTT_Row::Exchange(unsigned int _index1, unsigned int _index2)
{
     if((((int)_index1) < 0) || (_index1 >= fRowLength))
          return false;
     if((((int)_index2) < 0) || (_index2 >= fRowLength))
          return false;

     XTT_Cell* tmp = fCells[_index1];
     fCells[_index1] = fCells[_index2];
     fCells[_index2] = tmp;
	
	// Zarejestrowanie zmiany obiektu
	fChanged = true;

     return true;
}
//---------------------------------------------------------------------------

/**
 * Metoda klasy XTT_Row zwracajaca wskaznik do polaczenia wychodzacego z danego 
 * wiersza
 * @returns Wskaznik do polaczenia, jezeli nie istnieje to zwraca NULL
*/
XTT_Connections* XTT_Row::OutConnection(void)
{
     return ConnectionsList->_IndexOfOutConnections(fParent, fRowID);
}
//---------------------------------------------------------------------------

/**
 * Metoda klasy XTT_Row zwracajaca wskaznik do polaczenia wchodzacego do danego wiersza
 * @returns Wskaznik do polaczenia, jezeli nie istnieje to zwraca NULL
*/
QList<XTT_Connections*>* XTT_Row::InConnection(void)
{
     return ConnectionsList->_IndexOfInConnections(fParent, fRowID);
}
//---------------------------------------------------------------------------

/**
 * Metoda klasy XTT_Cell, ustawiajaca wskazik do komorki
 * @param int index - Index na ktore miejsce trzeba wskaznik ustawic
 * @param XTT_Cell* ptr - Wskanik do komorki
 * @returns true jezeli wszystko ok, inaczej false
*/
bool XTT_Row::setCell(int index, XTT_Cell* ptr)
{
     if((index < 0) || ((unsigned int)index >= fRowLength))
          return false;

     fCells[index] = ptr;
	
	// Zarejestrowanie zmiany obiektu
	fChanged = true;

     return true;
}
//---------------------------------------------------------------------------

/**
 * Metoda ustawiajaca etykiete wiersza
 * @param QString _label - Tresc etykiety
 * @returns Brak zwracanych wartoci
 */
void XTT_Row::setLabel(QString _label)
{ 
	fLabel = _label;
	
	// Zarejestrowanie zmiany obiektu
	fChanged = true;
}
//---------------------------------------------------------------------------

/**
 * Metoda zaznaczajaca/odznaczajaca wiersz tabeli
 * @param bool _sel - okresla czy dany wiersz ma byc zaznaczony (true) czy nalezy usunac zaznaczenie
 * @returns Brak zwracanych wartoci
 */
void XTT_Row::setSelected(bool _sel)
{
	for(unsigned int i=0;i<fRowLength;i++)
		fCells[i]->setSelected(_sel);
}
//---------------------------------------------------------------------------

/**
 * Metoda wyszukujaca w wierszu komorke o podanym id
 * @param int _id - identyfikator komorki
 * @returns index w ierszu, lub gdy komorka o pondamy id nie wystepuje -1
 */
int XTT_Row::indexOfCellID(int _id)
{
	int res_ = -1;
	
	for(unsigned int i=0;i<fRowLength;i++)
		if(fCells[i]->CellId() == (unsigned int)_id)
			return (int)i;
		
	return res_;
}
//---------------------------------------------------------------------------

/**
 * Metoda zapisujaca wszystkie dane w formacie xttml 1.0
 * @returns Zwraca liste stringow w tym formacie
*/
QStringList* XTT_Row::XTTML_1_0(void)
{
     QStringList* res = new QStringList;
     res->clear();

     *res << "<row row_id=\"" + QString::number(fRowID) + "\" table_id=\"" +  QString::number(fParent) + "\">";
	if(fLabel != "")
		*res << "\t<label>" + fLabel + "</label>";

     // Zapis atrybutow
     for(unsigned int a=0;a<fRowLength;a++)
     {
          QStringList* al = fCells[a]->XTTML_1_0();
          for(int l=0;l<al->size();l++)
               *res << "\t" + al->at(l);
          delete al;
     }
     
     *res << "</row>";

     return res;
}
// -----------------------------------------------------------------------------

#if QT_VERSION >= 0x040300
/**
 * Metoda zapisujaca wszystkie dane w formacie xttml 1.0
 * @param QXmlStreamWriter* _xmldoc - wskanik do dokumentu
 * @returns brak zwracanych wartosci
*/
void XTT_Row::XTTML_1_0(QXmlStreamWriter* _xmldoc)
{
	_xmldoc->writeStartElement("row");
	_xmldoc->writeAttribute("row_id", QString::number(fRowID));
	_xmldoc->writeAttribute("table_id", QString::number(fParent));
	if(fLabel != "")
		_xmldoc->writeTextElement("label", fLabel);

     // Zapis atrybutow
     for(unsigned int a=0;a<fRowLength;a++)
		fCells[a]->XTTML_1_0(_xmldoc);
     
     _xmldoc->writeEndElement();
}
#endif
// -----------------------------------------------------------------------------

/**
 * Metoda zapisujaca wszystkie dane w formacie xttml 2.0
 * @returns Zwraca liste stringow w tym formacie
*/
QStringList* XTT_Row::XTTML_2_0(void)
{
     QStringList* res = new QStringList;
     res->clear();

     *res << "<row label=\"" + fLabel + "\">";

     // Zapis atrybutow
     for(unsigned int a=0;a<fRowLength;a++)
     {
          QStringList* al = fCells[a]->XTTML_2_0();
          for(int l=0;l<al->size();l++)
               *res << "\t" + al->at(l);
          delete al;
     }
     
     *res << "</row>";

     return res;
}
// -----------------------------------------------------------------------------

#if QT_VERSION >= 0x040300
/**
 * Metoda zapisujaca wszystkie dane w formacie xttml 2.0
 * @param QXmlStreamWriter* _xmldoc - wskanik do dokumentu
 * @returns brak zwracanych wartosci
*/
void XTT_Row::XTTML_2_0(QXmlStreamWriter* _xmldoc)
{
	_xmldoc->writeStartElement("row");
	_xmldoc->writeAttribute("label", fLabel);

     // Zapis atrybutow
     for(unsigned int a=0;a<fRowLength;a++)
		fCells[a]->XTTML_2_0(_xmldoc);
     
     _xmldoc->writeEndElement();
}
#endif
// -----------------------------------------------------------------------------

/**
 * Metoda wczytujaca dane o strybucie z pliku w formacie XTTML 1.0
 * @param QStringList* list - Wskanik do sekcji dotyczacej tego atrybutu
 * @returns true jezeli dane sa ok, ianczej false
*/
bool XTT_Row::FromXTTML_1_0(QStringList* list)
{
     bool res = true;

     // Usuwanie spacji i tabulatorow wierszach
     for(int i=0;i<list->size();i++)
     {
          QString line = list->at(i);
          while(line.contains("\t"))
          {
               int index = line.indexOf("\t");
               line = line.remove(index, 1);
          }
          list->replace(i, line);
     }

     // Zakladamy ze pierwszy wiersz jest naglowkiem
     if(!list->at(0).contains("<row", Qt::CaseInsensitive))
          return false;
     if(!list->contains("</row>", Qt::CaseInsensitive))
          return false;

     // Reszta zakladamy ze jest ok

     // Spis tagow
     QStringList tags;
     tags.clear();
     tags << "<row" << "<cell" << "</cell>" << "<label>";

     // Sluza do odpowiedniego kopiowania lini dla listy atrybutow
     bool copy = false;
     QStringList buf;
     buf.clear();

     for(int line=0;line<list->size();line++)
     {
          // Sprawdzanie wystapienia tagu
          int action = -1;
          for(int tag=0;tag<tags.size();tag++)
               if(list->at(line).contains(tags.at(tag), Qt::CaseInsensitive))
                    action = tag;

          // Kopiowanie lini danych
          if(copy)
               buf.append(list->at(line));

          if(action == -1)
               continue;

          QString end_tag = tags.at(action);
          end_tag = end_tag.insert(1, "/");
          QString l = list->at(line);
          l = l.remove(l.indexOf(tags.at(action), Qt::CaseInsensitive), tags.at(action).length());
          l = l.mid(0, l.indexOf(end_tag, Qt::CaseInsensitive));

          // Wczytywanie parametrow w zaleznosci od wykrytego tagu
          if(action == 0)
          {
               fRowID = (unsigned int)XML_ReadValue(list->at(line), "row_id").toInt();
               fParent = (unsigned int)XML_ReadValue(list->at(line), "table_id").toInt();
          }
          if(action == 1)
          {
               // Jezeli tag rozpoczynajacy dane, to uruchamiamy kopiowanie
               copy = true;
               buf.append(list->at(line));
          }
          if(action == 2)
          {
               // Kiedy linia konczaca linie danych
               // Dodajemy nowy atrybut
               Insert(fRowLength, 1);

               // Wczytujemy dane z pliku
               res = res && fCells[fRowLength-1]->FromXTTML_1_0(&buf);

               // Konczymy kopiowanie
               copy = false;

               // Czyscimy zawartosc danych
               buf.clear();
          }
          if(action == 3)
               fLabel = l;     
     }

     return res;
}
// -----------------------------------------------------------------------------

/**
 * Metoda wczytujaca dane o strybucie z pliku w formacie XTTML 1.0
 * @param QStringList* list - Wskanik do sekcji dotyczacej tego atrybutu
 * @returns true jezeli dane sa ok, ianczej false
*/
bool XTT_Row::FromXTTML_1_0(QDomElement& root)
{
	if(root.tagName().toLower() != "row")
		return false;
		
     bool res = true;
	
	fRowID = (unsigned int)root.attribute("row_id", "0").toInt();
     fParent = (unsigned int)root.attribute("table_id", "0").toInt();
	
	QDomElement DOMlabel = root.firstChildElement("label");
	fLabel = DOMlabel.text();

	QDomElement DOMcell = root.firstChildElement("cell");
     while(!DOMcell.isNull())
     {
		// Kiedy linia konczaca linie danych
		// Dodajemy nowy atrybut
		Insert(fRowLength, 1);

		// Wczytujemy dane z pliku
		res = res && fCells[fRowLength-1]->FromXTTML_1_0(DOMcell); 
		
		// Wczytujemy koleja komorke
		DOMcell = DOMcell.nextSiblingElement("cell");
     }

     return res;
}
// -----------------------------------------------------------------------------

/**
 * Metoda wczytujaca dane o strybucie z pliku w formacie XTTML 2.0
 * @param QStringList* list - Wskanik do sekcji dotyczacej tego atrybutu
 * @returns true jezeli dane sa ok, ianczej false
*/
bool XTT_Row::FromXTTML_2_0(QStringList* list)
{
     bool res = true;

     // Usuwanie spacji i tabulatorow wierszach
     for(int i=0;i<list->size();i++)
     {
          QString line = list->at(i);
          while(line.contains("\t"))
          {
               int index = line.indexOf("\t");
               line = line.remove(index, 1);
          }
          list->replace(i, line);
     }

     // Zakladamy ze pierwszy wiersz jest naglowkiem
     if(!list->at(0).contains("<row", Qt::CaseInsensitive))
          return false;
     if(!list->contains("</row>", Qt::CaseInsensitive))
          return false;

     // Reszta zakladamy ze jest ok

     // Spis tagow
     QStringList tags;
     tags.clear();
     tags << "<row" << "<cell";

     // Sluza do odpowiedniego kopiowania lini dla listy atrybutow
     QStringList buf;
     buf.clear();

     for(int line=0;line<list->size();line++)
     {
          // Sprawdzanie wystapienia tagu
          int action = -1;
          for(int tag=0;tag<tags.size();tag++)
               if(list->at(line).contains(tags.at(tag), Qt::CaseInsensitive))
                    action = tag;


          if(action == -1)
               continue;

          QString end_tag = tags.at(action);
          end_tag = end_tag.insert(1, "/");
          QString l = list->at(line);
          l = l.remove(l.indexOf(tags.at(action), Qt::CaseInsensitive), tags.at(action).length());
          l = l.mid(0, l.indexOf(end_tag, Qt::CaseInsensitive));

          // Wczytywanie parametrow w zaleznosci od wykrytego tagu
          if(action == 0)
          {
			QString rlabel = XML_ReadValue(list->at(line), "label");
					
			if((rlabel != "") && (rlabel != "#NAN#"))		// label nie jest wymagane
				fLabel = rlabel;
          }
          if(action == 1)
          {
               buf.append(list->at(line));
			
			// Kiedy linia konczaca linie danych
               // Dodajemy nowy atrybut
               Insert(fRowLength, 1);

               // Wczytujemy dane z pliku
               res = res && fCells[fRowLength-1]->FromXTTML_2_0(&buf);

               // Czyscimy zawartosc danych
               buf.clear();
          }   
     }

     return res;
}
// -----------------------------------------------------------------------------

/**
 * Metoda wczytujaca dane o strybucie z pliku w formacie XTTML 2.0
 * @param QDomElement& root - Referencja do sekcji dotyczacej tego atrybutu
 * @returns true jezeli dane sa ok, ianczej false
*/
bool XTT_Row::FromXTTML_2_0(QDomElement& root)
{
	if(root.tagName().toLower() != "row")
		return false;

     bool res = true;

     // Reszta zakladamy ze jest ok

     // Spis tagow
     QStringList tags;
     tags.clear();
     tags << "<row" << "<cell";
	
	QString rlabel = root.attribute("label", "#NAN#");		
	if((rlabel != "") && (rlabel != "#NAN#"))		// label nie jest wymagane
		fLabel = rlabel;

	QDomElement child = root.firstChildElement("cell");
     while(!child.isNull())
     {
		// Kiedy linia konczaca linie danych
		// Dodajemy nowy atrybut
		Insert(fRowLength, 1);

		// Wczytujemy dane z pliku
		res = res && fCells[fRowLength-1]->FromXTTML_2_0(child);
			
		// Wczytywanie kolejnej komorki
		child = child.nextSiblingElement("cell");
     }

     return res;
}
// -----------------------------------------------------------------------------
