 /**
 *                                                                             *
 * @file XTT_Row.h                                                             *
 * Project name: HeKatE hqed                                                   *
 * $Id: XTT_Row.h,v 1.12 2008-03-03 22:04:34 kinio Exp $                        *
 * Author: Krzysztof Kaczor                                                    *
 * Date: 24.04.2007                                                            *
 *                                                                             *
 * Implementation by Krzysztof Kaczor <kinio4444@gmail.com>                    *
 *                                                                             *
 * Copyright (C) 2006-9 by the HeKatE Project                                  *
 *                                                                             *
 * HQEd has been develped within the HeKatE Project,                           *
 * see http://hekate.ia.agh.edu.pl                                             *
 *                                                                             *
 * This file is part of HQEd.                                                  *
 *                                                                             *
 * HQEd is free software: you can redistribute it and/or modify                *
 * it under the terms of the GNU General Public License as published by        *
 * the Free Software Foundation, either version 3 of the License, or           *
 * (at your option) any later version.                                         *
 *                                                                             *
 * HQEd is distributed in the hope that it will be useful,                     *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU General Public License for more details.                                *
 *                                                                             *
 * You should have received a copy of the GNU General Public License           *
 * along with HQEd.  If not, see <http://www.gnu.org/licenses/>.               *
 *                                                                             *
 * Comment: Plik zawiera definicje obiektu bedacego wierszem tabeli            *
 *                                                                             *
 */

#ifndef XTT_ROWH
#define XTT_ROWH
// -----------------------------------------------------------------------------

#include <QtCore/QString>
#include <QDomDocument>

#if QT_VERSION >= 0x040300
	#include <QXmlStreamWriter>
#endif

#include "XTT_Cell.h"
#include "XTT_ConnectionsList.h"
// -----------------------------------------------------------------------------

class XTT_Row
{
private:

     /// Tabela rodzic
     unsigned int fParent;

     /// Nastepna wartosc identyfikatora komorki
     unsigned int* fNextCellId;

     /// Identyfikator wiersza
     unsigned int fRowID;

     /// Dlugosc wiersza
     unsigned int fRowLength;

     /// Komrki wiersza
     XTT_Cell** fCells;

     /// Etykieta wiersza
     QString fLabel;
	
	/// Okresla czy wiersz byl modyfikowany
	bool fChanged;

public:

     /// Konstruktor domyslny
     XTT_Row(void);

     /// Konstruktor przyjmujacy id rodzica i wartosc identyfikatora
     XTT_Row(unsigned int, unsigned int, unsigned int*);

     /// Destrutor
     ~XTT_Row(void);

     /// ----------------------- Metody obslugujace pliki -----------------------

     /// Wstawia wszystkie dane w formacie xttml 1.0
     QStringList* XTTML_1_0(void);
	
	#if QT_VERSION >= 0x040300
	/// Zwraca format zapisu zgodny z XTTML 1.0
	void XTTML_1_0(QXmlStreamWriter*);
	#endif
	
	/// Wstawia wszystkie dane w formacie xttml 2.0
     QStringList* XTTML_2_0(void);

	#if QT_VERSION >= 0x040300
	/// Zwraca format zapisu zgodny z XTTML 2.0
	void XTTML_2_0(QXmlStreamWriter*);
	#endif

     /// Wczytuje dane z pliku XTTML 1.0
     bool FromXTTML_1_0(QStringList*);
	
	/// Wczytuje dane z pliku XTTML 1.0
     bool FromXTTML_1_0(QDomElement&);
	
	/// Wczytuje dane z pliku XTTML 2.0
	bool FromXTTML_2_0(QStringList*);
	
	/// Wczytuje dane z pliku XTTML 2.0
	bool FromXTTML_2_0(QDomElement&);

     /// ----------------- Metody zwracajaca wartosci pol klasy -----------------

     /// Zwraca wskaznik rodzica komorki
     unsigned int Parent(void){ return fParent; }

     /// Zwraca ID wiersza
     unsigned int RowId(void){ return fRowID; }

     /// Zwraca dlugosc wiersza
     unsigned int Length(void){ return fRowLength; }

     /// Zwraca waskaznik do wybranej komorki wiersza
     XTT_Cell* Cell(int);

     /// Zwraca etykiete tabeli
     QString Label(void){ return fLabel; }
	
	/// Zwraca czy dany wiersz byl modyfikowany
	bool Changed(void);

     /// ---------------- Metody ustawiajace wartosci pol klasy -----------------

     /// Ustawia etykiete tabeli
     void setLabel(QString);

     /// Pozwala ustawic wskznik do komorki
     bool setCell(int, XTT_Cell*);
	
	/// Ustawia wartosc okreslajaca modyfikacje wiersza
	void setChanged(bool);

     /// -------------------------- Metody operacyjne ---------------------------

     /// Metoda wstawiajaca komorke w zadane miejsce, jezeli wstawienie komorki
     /// jest mozliwe to zwraca true inaczej false
     bool Insert(unsigned int, int);

     /// Usuwa wybrna komorke wiersza, jezeli usuniecie jest mozliwe to zwraca
     /// true, inaczej false
     bool Delete(unsigned int);

     /// Pozwala na zamiane elementow ze soba
     bool Exchange(unsigned int, unsigned int);

     /// Zwraca wskaznik do polaczenia wychodzacego z tego wiersza
     XTT_Connections* OutConnection(void);

     /// Zwraca wskaznik do polaczenia wchodzacego do tego wiersza
     QList<XTT_Connections*>* InConnection(void);
	
	/// Zwraca index komorki o podanym id
	int indexOfCellID(int);
	
	/// Ustawia zaznaczenie wiersza
	void setSelected(bool _sel = true);
};
// -----------------------------------------------------------------------------
#endif
