 /**
 *                                                                             *
 * @file XTT_Table.cpp                                                         *
 * Project name: HeKatE hqed                                                   *
 * $Id: XTT_Table.cpp,v 1.12 2008-03-03 22:04:34 kinio Exp $                    *
 * Author: Krzysztof Kaczor                                                    *
 * Date: 03.05.2007                                                            *
 *                                                                             *
 * Implementation by Krzysztof Kaczor <kinio4444@gmail.com>                    *
 *                                                                             *
 * Copyright (C) 2006-9 by the HeKatE Project                                  *
 *                                                                             *
 * HQEd has been develped within the HeKatE Project,                           *
 * see http://hekate.ia.agh.edu.pl                                             *
 *                                                                             *
 * This file is part of HQEd.                                                  *
 *                                                                             *
 * HQEd is free software: you can redistribute it and/or modify                *
 * it under the terms of the GNU General Public License as published by        *
 * the Free Software Foundation, either version 3 of the License, or           *
 * (at your option) any later version.                                         *
 *                                                                             *
 * HQEd is distributed in the hope that it will be useful,                     *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU General Public License for more details.                                *
 *                                                                             *
 * You should have received a copy of the GNU General Public License           *
 * along with HQEd.  If not, see <http://www.gnu.org/licenses/>.               *
 *                                                                             *
 * Comment: Plik zawierajacy definicje metod obiektu bedacego tabela           *
 *                                                                             *
 */

#include <QMessageBox>
#include <QtGui>
#include <QDialog>
#include <QObject>

#include "../../C/MainWin_ui.h"
#include "../../C/Settings_ui.h"
#include "../../V/XTT/GTable.h"
#include "XTT_AttributeGroups.h"
#include "XTT_ConnectionsList.h"
#include "XTT_Table.h"
#include "XTT_TableList.h"
//---------------------------------------------------------------------------

/**
 * Konstruktor defaultowy klasy
 * @returns Brak zwracanych wartosci
*/
XTT_Table::XTT_Table(void)
{
     fNextRowId = NULL;
     fTableID = 0;
     fTitle = "";
     fDescription = "";
     fnContextSize = new unsigned int[4];
     for(int i=0;i<4;i++)
          fnContextSize[i] = 0;
     fContextAtributes = NULL;
     fRowCount = 0;
     fRows = NULL;
     fPoint.setX(0);
     fPoint.setY(0);
     fWidth = NULL;
     fHeight = NULL;
     fChanged = false;
     fLevel = 0;
     fParent = 0;
     fTableType = 1;
     fAbstractColID = NULL;     
     fNextAbstractID = 1;
     fNextCellId = 0;
     fHeaderHeight = Settings->xttDefaultHeaderHeight();
     fGTable = new GTable(this);
}
//---------------------------------------------------------------------------

/**
 * Konstruktor klasy XTT_Table ustawiajacy wartosc identyfikatora tabeli
 * @param unsigned int _tid - Wartosc identyfikatora tabeli
 * @param unsigned int* _prid - Wskaznik do wartosci przechowujacej nastepne ID
 * wiersza
 * @returns Brak zwracanych wartosci
*/
XTT_Table::XTT_Table(unsigned int _tid, unsigned int* _prid)
{
     fNextRowId = _prid;
     fTableID = _tid;
     fTitle = "";
     fDescription = "";
     fnContextSize = new unsigned int[4];
     for(int i=0;i<4;i++)
          fnContextSize[i] = 0;
     fContextAtributes = NULL;
     fRowCount = 0;
     fRows = NULL;
     fPoint.setX(0);
     fPoint.setY(0);
     fWidth = NULL;
     fHeight = NULL;
     fChanged = false;
     fLevel = 0;
     fParent = 0;
     fTableType = 1;
     fAbstractColID = NULL;
     fNextAbstractID = 1;
     fNextCellId = 0;
     fHeaderHeight = Settings->xttDefaultHeaderHeight();
     fGTable = new GTable(this);
}
//---------------------------------------------------------------------------

/**
 * Destrutkor klasy
 * @returns Brak zwracanych wartosci
*/
XTT_Table::~XTT_Table(void)
{
     // Usuwanie zaalokowanych obiektow
     if(ColCount())
     {
          delete [] fContextAtributes;
          delete [] fWidth;
          delete [] fHeight;
     }

     if(fRowCount)
     {
          for(unsigned int i=0;i<fRowCount;i++)
               delete fRows[i];
          delete [] fRows;
     }

     delete [] fnContextSize;
	if(fGTable != NULL)
		delete fGTable;
}
//---------------------------------------------------------------------------

/**
 * Metoda zwracajaca ilosc kolmn tabeli
 * @returns Ilosc kolumn tabeli
*/
int XTT_Table::ColCount(void)
{
     return fnContextSize[0] + fnContextSize[1] + fnContextSize[2] + fnContextSize[3];
}
//---------------------------------------------------------------------------

/**
 * Metoda klasy XTT_Table zwracajaca wielkosc wybranego kontekstu
 * @param unsigned int _cindex - Index kontekstu
 * @returns Zwraca rozmiar kontekstu
*/
int XTT_Table::ContextSize(unsigned int _cindex)
{
     // Sprawdzanie indexu
     if((_cindex < 1) || (_cindex > 4))
          return -1;

     return (int)fnContextSize[_cindex-1];
}
//---------------------------------------------------------------------------

/**
 * Metoda klasy, okreslajaca ktory kontekst jest ostatni nie pusty
 * @returns Zwraca index kontekstu, jezeli wszystkie puste to -1
*/
int XTT_Table::LastNotEmptyContext(void)
{
     int res = 3;
     while((res >= 0) && (fnContextSize[res] == 0))
     {
          res--;
     }

     return res+1;
}
//---------------------------------------------------------------------------

/**
 * Metoda klasy XTT_Table zwracajaca wskaznik do atrybutu w wybranej kolumnie
 * @param unsigned int _index - Index atrybutu
 * @returns Wskaznik do atrybutu, w przypadku kiedy index jest niepoprawny NULL
*/
XTT_Attribute* XTT_Table::ContextAtribute(unsigned int _index)
{
     // Sprawdzanie indexu
     if((((int)_index) < 0) || (_index >= (unsigned int)ColCount()))
          return NULL;

     return fContextAtributes[_index];
}
//---------------------------------------------------------------------------

/**
 * Metoda klasy XTT_Table zwracajaca wskaznik do wybranego wiersza tabeli
 * @param unsigned int _index - Index wiersza
 * @returns Wskaznik do wiersza, w przypadku kiedy index jest niepoprawny NULL
*/
XTT_Row* XTT_Table::Row(unsigned int _index)
{
     // Sprawdzanie indexu
     if((((int)_index) < 0) || (_index >= fRowCount))
          return NULL;

     return fRows[_index];
}
//---------------------------------------------------------------------------

/**
 * Metoda zwracajaca czy dana tabela zostala zmodyfikowana
 * @returns true jezeli tabela zostala zmodyfikowana, inaczej false
 */
bool XTT_Table::Changed(void)
{
	for(unsigned int i=0;i<fRowCount;i++)
		if(fRows[i]->Changed())
			return true;
		
	return fChanged;
}
//---------------------------------------------------------------------------

/**
 * Metoda klasy XTT_Table zmieniajaca tytul tabeli i zaznaczajaca ze zostala 
 * zmieniona
 * @param QString _title - Nowy tytul tabeli
 * @returns Brak zwracanych wartoci
*/
void XTT_Table::setTitle(QString _title)
{
     fTitle = _title;
     fChanged = true;
}
//---------------------------------------------------------------------------

/**
 * Metoda klasy XTT_Table zmieniajaca opis tabeli i zaznaczajaca ze zostala
 * zmieniona
 * @param QString _desc - Nowy opis tabeli
 * @returns Brak zwracanych wartoci
*/
void XTT_Table::setDescription(QString _desc)
{
     fDescription = _desc;
     fChanged = true;
}
//---------------------------------------------------------------------------

/**
 * Metoda klasy XTT_Table zmieniajaca polozenie tabeli i zaznaczajaca ze zostala
 * zmieniona
 * @param QPoint _point - Nowe polozenie tabeli
 * @returns Brak zwracanych wartoci
*/
bool XTT_Table::setLocalization(QPoint _point)
{
     fPoint = _point;
     fChanged = true;
     fGTable->RePosition();
	
     // Odswiezenie polaczen
     ConnectionsList->RepaintConnectionRelatedWithTable(fTableID);
	
     return true;
}
//---------------------------------------------------------------------------

/**
 * Metoda ustawiajaca typ tabeli
 * @param unsigned int _tt - Nowa wartosc typu. opis w header
 * @returns Brak zwracanych wartoci
 */
void XTT_Table::setTableType(unsigned int _tt)
{ 
	fTableType = _tt;
	fChanged = true;
}
//---------------------------------------------------------------------------

/**
 * Metoda ustawiajaca poziom zagniezdzenia tabeli
 * @param unsigned int _l - Nowa wartosc poziomu zagniezdzenia
 * @returns Brak zwracanych wartoci
 */
void XTT_Table::setLevel(unsigned int _l)
{ 
	fLevel = _l; 
	fChanged = true;
}
//---------------------------------------------------------------------------

/**
 * Metoda ustawiajaca rodzica tabeli
 * @param unsigned int _p - Identyfiaktor rodzica
 * @returns Brak zwracanych wartoci
 */
void XTT_Table::setParent(unsigned int _p)
{ 
	fParent = _p;
	fChanged = true;
}
//---------------------------------------------------------------------------

/**
 * Metoda ustawiajaca wartosc okreslajaca czy dana tabela zostala zmodyfikowana
 * @param bool _changed - Nowa wartosc
 * @returns Brak zwracanych wartoci
 */
void XTT_Table::setChanged(bool _changed)
{ 
	for(unsigned int i=0;i<fRowCount;i++)
		fRows[i]->setChanged(_changed);
		
	fChanged = _changed;
}
//---------------------------------------------------------------------------

/**
 * Metoda ustawiajaca wysokosc naglowka
 * @param unsigned int _hh - Nowa wysokosc naglowka
 * @returns Brak zwracanych wartoci
 */
void XTT_Table::setHeaderHeight(unsigned int _hh)
{ 
	fHeaderHeight = _hh; 
	fChanged = true;
}
//---------------------------------------------------------------------------

/**
 * Metoda klasy XTT_Table zwracajaca sumaryczna szerokosc tabeli
 * @returns Zwraca sumaryczna szerokosc tabeli
*/
unsigned int XTT_Table::Width(void)
{
     unsigned int res = 0;
     for(int i=0;i<ColCount();i++)
          res += fWidth[i];

     // Na podwojna linie
     /*
     // Kiedy istnieje granica pomiedzy conditional i assert lub retract
     if((fnContextSize[0] > 0) && ((fnContextSize[1] > 0) || (fnContextSize[2] > 0)))
          res += 2;

     // Kiedy istnieje granica pomiedzy action i assert lub retract
     if((fnContextSize[3] > 0) && ((fnContextSize[1] > 0) || (fnContextSize[2] > 0)))
          res += 2;

     // Kiedy tylko wystepuja konteksty conditional i action
     if((fnContextSize[1] > 0) && (fnContextSize[1] == 0) && (fnContextSize[2] == 0) && (fnContextSize[3] > 0))
          res += 2;  */

     return res;
}
//---------------------------------------------------------------------------

/**
 * Metoda klasy XTT_Table zwracajaca sumaryczna wysokosc tabeli
 * @returns Zwraca sumaryczna wysokosc tabeli
*/
unsigned int XTT_Table::Height(void)
{
     unsigned int res = fHeaderHeight;
     for(unsigned int i=0;i<fRowCount;i++)
          res += fHeight[i];

     return res;
}
//---------------------------------------------------------------------------

/**
 * Metoda klasy XTT_Table zwracajaca szerokosc danej kolumny
 * @param unsigned int _index - Index kolumny ktorej szerokosc chcemy poznac
 * @returns Szerokosc, jezeli index jest nie poprawny 0
*/
unsigned int XTT_Table::ColWidth(unsigned int _index)
{
     // Sprawdzanie indexu
     if((((int)_index) < 0) || (_index >= (unsigned int)ColCount()))
          return 0;

     return fWidth[_index];
}
//---------------------------------------------------------------------------

/**
 * Metoda klasy XTT_Table zwracajaca wysokosc danego wiersza
 * @param unsigned int _index - Index wiersza ktorego wysokosc chcemy poznac
 * @returns Wysokosc, jezeli index jest nie poprawny 0
*/
unsigned int XTT_Table::RowHeight(unsigned int _index)
{
     // Sprawdzanie indexu
     if((((int)_index) < 0) || (_index >= fRowCount))
          return 0;

     return fHeight[_index];
}
//---------------------------------------------------------------------------

/**
 * Metoda klasy XTT_Table zmieniajaca szerokosc tabeli i zaznaczajaca ze zostala
 * zmieniona
 * @param unsigned int _width - Nowa szerokosc tabeli
 * @returns Brak zwracanych wartoci
*/
bool XTT_Table::setColWidth(unsigned int _index, unsigned int _width)
{
     // Sprawdzanie indexu
     if((((int)_index) < 0) || (_index >= (unsigned int)ColCount()))
          return false;

     fWidth[_index] = _width;
     fChanged = true;

     return true;
}
//---------------------------------------------------------------------------

/**
 * Metoda klasy XTT_Table zmieniajaca wysokosc tabeli i zaznaczajaca ze zostala
 * zmieniona
 * @param unsigned int _height - Nowa wysokosc tabeli
 * @returns Brak zwracanych wartoci
*/
bool XTT_Table::setRowHeight(unsigned int _index, unsigned int _height)
{
     // Sprawdzanie indexu
     if((((int)_index) < 0) || (_index >= fRowCount))
          return false;

     fHeight[_index] = _height;
     fChanged = true;

     return true;
}
//---------------------------------------------------------------------------

/**
 * Metoda klasy XTT_Table zmieniajaca przypisany do kolumny atrybut
 * @param unsigned int _colIndex - Index kolumny ktorej atrybut chcemy zmienic
 * @param XTT_Attribute* _newAttr - Wskanik do nowego atrybutu
 * @returns true jezeli index jest poprawny i wskaznik nie NULL, inaczej false
*/
bool XTT_Table::setContextAtributes(unsigned int _colIndex, XTT_Attribute* _newAttr)
{
     if((((int)_colIndex) < 0) || (_colIndex >= (unsigned int)ColCount()))
          return false;
     if(_newAttr == NULL)
          return false;

     try
     {
          fContextAtributes[_colIndex] = _newAttr;
          for(unsigned int i=0;i<fRowCount;i++)
               fRows[i]->Cell(_colIndex)->setAttrType(_newAttr);
     }
     catch(...)
     {
          return false;
     }

     // Ze sie zmienilo
     fChanged = true;

     return true;
}
//---------------------------------------------------------------------------

/**
 * Metoda klasy XTT_Table zmieniajaca atrybut przypisany do komorki
 * @param unsigned int _context - Index kontekstu w ktorym chcemy zmienic 
 * atrybut
 * @param unsigned int _colIndex - Index kolumny  w kontekscie ktory chcemy 
 * zmienic
 * @param XTT_Attribute* _newAttr - Wskanik do nowego atrybutu
 * @returns true jezeli wszystkie dane wejsciowe sa poprane inaczej false
*/
bool XTT_Table::setContextAtributes(unsigned int _context, unsigned int _colIndex, XTT_Attribute* _newAttr)
{
     // Jezeli kontekst jest niepoprawnie okreslony
     if((_context < 1) || (_context > 4))
          return false;

     // Jezeli index kolumny jest niepopranwnie okreslony
     if((((int)_colIndex) < 0) || (_colIndex >= fnContextSize[_context]))
          return false;

     // Jezeli wskaznik nie poazujacy na cos konkretnego :)
     if(_newAttr == NULL)
          return false;

     // Wyszukanie docelowej kolumny
     int col_ind = 0;
     for(unsigned int i=0;i<(_context-1);i++)
          col_ind += fnContextSize[i];
     col_ind += _colIndex;


     // Ustawienie nowego atrybutu, w razie jakis bledow zwracamy false
     try
     {
          for(unsigned int i=0;i<fRowCount;i++)
               fRows[i]->Cell(col_ind)->setAttrType(_newAttr);
     }
     catch(...)
     {
          return false;
     }

     // Ze sie zmienilo
     fChanged = true;

     return true;
}
//---------------------------------------------------------------------------

/**
 * Metoda klasy XTT_Table zwracajaca kontekst kolumny o podanym indexie
 * @param unsigned int _cindex - Index kolumny kotrej kontekst chcemy poznac
 * @returns Kontekst kolumny, jezeli index jest niepopranwy to -1
*/
int XTT_Table::ColContext(int _cindex)
{
     // Sprawdzanie indexu
     if((_cindex < 0) || (_cindex >= ColCount()))
          return -1;

     // Wyszukiwanie kontekstu
     int res = 0;
     for(int i=0;i<4;i++)
     {
          res += fnContextSize[i];
          if(res > _cindex)
               return i+1;
     }

     return -1;
}
//---------------------------------------------------------------------------

/**
 * Metoda zwracajaca index kolumny w pojemniku
 * @param unsigned int _context - Index kontekstu w ktorym wystepuje dana 
 * kolumna
 * @param unsigned int _index - Index kolumny w danym kontekscie
 * @returns Index kolumny w pojemniku, jezeli podane indexy sa niepoprawne 
 * zwraca -1
*/
int XTT_Table::ColIndex(unsigned int _context, unsigned int _index)
{
     // Jezeli kontekst jest niepoprawnie okreslony
     if((_context < 1) || (_context > 4))
          return -1;

     // Jezeli index kolumny jest niepopranwnie okreslony
     if((((int)_index) < 0) || (_index >= fnContextSize[_context]))
          return -1;

     // Sumowanie kolumn z poprzedzajacych konykstow
     int res = 0;
     for(unsigned int i=0;i<(_context-1);i++)
          res += fnContextSize[i];

     // Przesuniecie o index
     res += _index;

     return res;
}
//---------------------------------------------------------------------------

/**
 * Metoda klasy XTT_Table zamieniajaca ze soba kolumny z jednego kontekstu
 * @param unsigned int _index1 - Index pierwszej kolumny
 * @param unsigned int _index2 - Index drugiej kolumny
 * @returns true jezeli wszystko ok, inaczej false
*/
bool XTT_Table::Exchange(unsigned int _index1, unsigned int _index2)
{
     // Sprawdzanie indexu
     if((((int)_index1) < 0) || (_index1 >= (unsigned int)ColCount()))
          return false;

     // Sprawdzanie indexu
     if((((int)_index2) < 0) || (_index2 >= (unsigned int)ColCount()))
          return false;

     // Zamienianie poszczegolnych komorek
     bool ok = true;
     for(unsigned int i=0;i<fRowCount;i++)
          ok = ok && fRows[i]->Exchange(_index1, _index2);

     // Zmiana identyfikatorow
     unsigned int tid = fAbstractColID[_index1];
     fAbstractColID[_index1] = fAbstractColID[_index2];
     fAbstractColID[_index2] = tid;

     // Ze sie zmienilo
     fChanged = true;
	
	// Odswiezenie widoku tabeli
	fGTable->RePosition();

     return ok;
}
//---------------------------------------------------------------------------

/**
 * Metoda klasy XTT_Table dodajaca nowy wiersz do tabeli
 * @returns Brak zwracanych wartoci
*/
void XTT_Table::AddRow(void)
{
     // Alokacja tymczasowej tablicy
     XTT_Row** tmp = new XTT_Row*[fRowCount+1];
     unsigned int* htmp = new unsigned int[fRowCount+1];

     // Kopiowanie wskaznikow
     for(unsigned int i=0;i<fRowCount;i++)
     {
          tmp[i] = fRows[i];
          htmp[i] = fHeight[i];
     }

     // Skasowanie strej za malej tablicy elementow
     if(fRowCount)
     {
          delete [] fHeight;
          delete [] fRows;
     }

     // Zapisanie nowej lokalizacji
     fRows = tmp;
     fHeight = htmp;

     // Utworzenie nowego elementu
     fRows[fRowCount] = new XTT_Row(fTableID, ++(*fNextRowId), &fNextCellId);
     fHeight[fRowCount] = Settings->xttDefaultRowHeight();

     // Ustawienie komorek w nowym wierszu
     for(unsigned int i=0;i<(unsigned int)ColCount();i++)
     {
          fRows[fRowCount]->Insert(i, ColContext(i));
          fRows[fRowCount]->Cell(i)->setAttrType(fContextAtributes[i]);
          fRows[fRowCount]->Cell(i)->setContent("");
     }

     // Powiekszenie pojemnika
     fRowCount++;
	
	fGTable->ReCreate();
	fGTable->Update();

     // Ze sie zmienilo
     fChanged = true;
}
//---------------------------------------------------------------------------

/**
 * Metoda klasy kasujaca wybrany wiersz
 * @param unsigned int _index - Index wierszka do skakowania
 * @returns true jezeli kasowanie przebiegnie pomyslnie, inaczej flase
 */
bool XTT_Table::DeleteRow(unsigned int _index)
{
     // Alokacja tymczasowej tablicy
     XTT_Row** tmp = new XTT_Row*[fRowCount-1];
     unsigned int* htmp = new unsigned int[fRowCount-1];

     // Kopiowanie wskaznikow
     int tmp_i = 0;
     for(unsigned int i=0;i<fRowCount;i++)
          if(i != _index)
          {
               tmp[tmp_i] = fRows[i];
               htmp[tmp_i++] = fHeight[i];
          }

     // Skasowanie strej za malej tablicy elementow
     if(fRowCount)
     {
          delete fRows[_index];
          delete [] fHeight;
          delete [] fRows;
     }

     // Zapisanie nowej lokalizacji
     fRows = tmp;
     fHeight = htmp;

     // Powiekszenie pojemnika
     fRowCount--;

     // Ze sie zmienilo
     fChanged = true;
	
	// Odsiezenie widoku
	TableList->Refresh(TableList->IndexOfID(fTableID));

     return true;
}
//---------------------------------------------------------------------------

/**
 * Metoda klasy XTT_Table wstawiajaca nowa kolumne do tabeli
 * @param unsigned int _index - Index kolumny w tabeli
 * @returns true jezeli podany index jest poprawny inaczej false
*/
bool XTT_Table::InsertCol(unsigned int _index)
{
     // Sprawdzanie indexu
     if((((int)_index) < 0) || (_index > (unsigned int)ColCount()))
          return false;

     // Alokacja pamieci na nowa szerokosc
     unsigned int* wtmp = new unsigned int[ColCount()+1];
     unsigned int* aidtmp = new unsigned int[ColCount()+1];
     XTT_Attribute** ftmp = new XTT_Attribute*[ColCount()+1];

     // Kopiowanie szerokosci
     int tmp_i = 0;
     for(int i=0;i<=ColCount();i++)
          if(i != (int)_index)
          {
               wtmp[i] = fWidth[tmp_i];
               aidtmp[i] = fAbstractColID[tmp_i];
               ftmp[i] = fContextAtributes[tmp_i++];
          }

     // Skasowanie starej tablicy szerokosci
     if(ColCount())
     {
          delete [] fWidth;
          delete [] fContextAtributes;
          delete [] fAbstractColID;
     }

     // Zapisanie nowej okalizacji tablicy
     fWidth = wtmp;
     fContextAtributes = ftmp;
     fAbstractColID = aidtmp;

     // Nowa wartosc domyslna szerokosci
     fWidth[_index] = Settings->xttDefaultColumnWidth();
     fAbstractColID[_index] = fNextAbstractID++;

     // Dodawanie nowej kolumny
     int ctx = LastNotEmptyContext();
     if(_index < (unsigned int)ColCount())
          ctx = ColContext(_index);
          
     for(unsigned int i=0;i<fRowCount;i++)
     {
          fRows[i]->Insert(_index, ctx);
          fRows[i]->Cell(_index)->setAttrType(fContextAtributes[_index]);
          fRows[i]->Cell(_index)->setContent("");
     }
     
     // Powiekszenie kontekstu
     fnContextSize[ctx-1]++;

     // Ze sie zmienilo
     fChanged = true;
     
     return true;
}
//---------------------------------------------------------------------------

/**
 * Metoda klasy XTT_Table wstawiajaca nowy wiersz w zadane miejsce
 * @param unsigned int _index - Index wiersza w tabeli
 * @returns true jezeli podany index jest poprawny inaczej false
*/
bool XTT_Table::insertRow(int _index)
{
	// Jezeli ondex wstawiania jest niepoprawny
	if((_index < 0) || (_index > (int)fRowCount))	
		return false;
		
	// Jezeli wszystko ok to:
	
	// Dodajemy wiersz
	AddRow();
	
	// Przesuwamy go na zadana pozycje
	for(int ci=(int)fRowCount-1;ci>_index;ci--)
		ExchangeRows(ci, ci-1);
	
	// Odsiezenie widoku
	TableList->Refresh(TableList->IndexOfID(fTableID));
		
	return true;
}
//---------------------------------------------------------------------------

/**
 * Metoda klasy XTT_Table usuwajaca dana kolumne
 * @param unsigned int _index - Index kloumny do usuniecia
 * @returns true jezeli wszystko ok, flase jezeli index jest niepoprawny
*/
bool XTT_Table::DeleteCol(unsigned int _index)
{
     // Sprawdzanie indexu
     if((((int)_index) < 0) || (_index >= (unsigned int)ColCount()))
          return false;

     // Zapisanie ilosc kolumn
     int cc = ColCount();

     // Pomniejszenie kontekstu
     fnContextSize[ColContext(_index)-1]--;

     // Alokacja pamieci na nowa szerokosc
     unsigned int* wtmp = new unsigned int[cc-1];
     unsigned int* aidtmp = new unsigned int[cc-1];
     XTT_Attribute** ftmp = new XTT_Attribute*[cc-1];

     // Kopiowanie szerokosci
     int tmp_i = 0;
     for(int i=0;i<cc;i++)
          if(i != (int)_index)
          {
               wtmp[tmp_i] = fWidth[i];
               aidtmp[tmp_i] = fAbstractColID[i];
               ftmp[tmp_i++] = fContextAtributes[i];
          }

     // Skasowanie starej tablicy szerokosci
     if(cc)
     {
          delete [] fWidth;
          delete [] fContextAtributes;
          delete [] fAbstractColID;
     }

     // Zapisanie nowej okalizacji tablicy
     fWidth = wtmp;
     fContextAtributes = ftmp;
     fAbstractColID = aidtmp;

     // Usuwanie kolumny
     for(unsigned int i=0;i<fRowCount;i++)
          fRows[i]->Delete(_index);

     // Ze sie zmienilo
     fChanged = true;
	
	// Odsiezenie widoku
	TableList->Refresh(TableList->IndexOfID(fTableID));
     
     return true;
}
//---------------------------------------------------------------------------

/**
 * Metoda klasy XTT_Table wstawiajaca kolumne o nowym kotekscie
 * @param unsigned int _ctx - Index kontekstu
 * @returns true jezeli wstawi, false jezeli juz kolmna o tym kontekscie 
 * istnieje
*/
bool XTT_Table::InsertNewContext(unsigned int _ctx)
{
     // Sprawdzanie indexu
     if((_ctx < 1) || (_ctx > 4))
          return false;

     if(fnContextSize[_ctx-1] > 0)
          return false;

     // Wyszukiwanie indexu wstawienia
     unsigned int _index = 0;
     for(unsigned int i=0;i<(_ctx-1);i++)
          _index += fnContextSize[i];

     // Alokacja pamieci na nowa szerokosc
     unsigned int* wtmp = new unsigned int[ColCount()+1];
     unsigned int* aidtmp = new unsigned int[ColCount()+1];
     XTT_Attribute** ftmp = new XTT_Attribute*[ColCount()+1];

     // Kopiowanie szerokosci
     int tmp_i = 0;
     for(int i=0;i<=ColCount();i++)
          if(i != (int)_index)
          {
               wtmp[i] = fWidth[tmp_i];
               aidtmp[i] = fAbstractColID[tmp_i];
               ftmp[i] = fContextAtributes[tmp_i++];
          }

     // Skasowanie starej tablicy szerokosci
     if(ColCount())
     {
          delete [] fWidth;
          delete [] fContextAtributes;
          delete [] fAbstractColID;
     }

     // Zapisanie nowej lokalizacji tablicy
     fWidth = wtmp;
     fContextAtributes = ftmp;
     fAbstractColID = aidtmp;

     // Nowa wartosc domyslna szerokosci
     fWidth[_index] = 120;
     fAbstractColID[_index] = fNextAbstractID++;

     // Dodawanie nowej kolumny
     for(unsigned int i=0;i<fRowCount;i++)
     {
          fRows[i]->Insert(_index, _ctx);
          fRows[i]->Cell(_index)->setAttrType(fContextAtributes[_index]);
          fRows[i]->Cell(_index)->setContent("");
     }
     
     // Powiekszenie kontekstu
     fnContextSize[_ctx-1]++;

     // Ze sie zmienilo
     fChanged = true;
     
     return true;
}
//---------------------------------------------------------------------------

/**
 * Metoda klasy XTT_Table, przenoszaca wybrana kolumne w wybrane miejsce
 * @param unsigned int from - index kolumny zrodlowej - ta ktora chcemy 
 * przeniesc
 * @param unsigned int to - index miejsca docelowego
 * @returns true jezeli wszystko sie uda, inaczej false
*/
bool XTT_Table::Move(unsigned int from, unsigned int to)
{
     // Sprawdzanie indexow
     if((((int)from) < 0) || (from >= (unsigned int)ColCount()))
          return false;
     if((((int)to) < 0) || (to >= (unsigned int)ColCount()))
          return false;
     if(from == to)
          return true;

     // Przyrost
     int dx = 1;
     if(from > to)
          dx = -1;

     // Przenoszenie
     bool ok = true;
     for(unsigned int r=0;r<fRowCount;r++)                               // Dla kazdego wiersza
     {
          // Wskaznik do komorki zrodlowej
          XTT_Cell* ptr = fRows[r]->Cell(from);
          
          for(unsigned int i=from;i!=to;i+=dx)
               ok = ok && fRows[r]->setCell(i, fRows[r]->Cell(i+dx));
          ok = ok && fRows[r]->setCell(to, ptr);
     }

     // Przenoszenie ID i atrybutow
     unsigned int tmp_col_id = fAbstractColID[from];
     XTT_Attribute* tmp_c_a = fContextAtributes[from];
     for(unsigned int i=from;i!=to;i+=dx)
     {
          fAbstractColID[i] = fAbstractColID[i+dx];
          fContextAtributes[i] = fContextAtributes[i+dx];
     }
     fAbstractColID[to] = tmp_col_id;
     fContextAtributes[to] = tmp_c_a;

     return ok;
}
//---------------------------------------------------------------------------

/**
 * Metoda klasy XTT_Table pozwalajaca na zmiene kontekstu ccalej kolumny
 * @param unsigned int _col_index - Index kolumny
 * @param unsigned int _ctx - Nowy kontekst
 * @returns Brak zwracanych wartoci
*/
void XTT_Table::setColContext(unsigned int _col_index, unsigned int _ctx)
{
     // Sprawdzanie indexow
     if((((int)_col_index) < 0) || (_col_index >= (unsigned int)ColCount()))
          return;
     if((_ctx < 1) || (_ctx >= 5))
          return;

     fnContextSize[ColContext(_col_index)-1]--;
     fnContextSize[_ctx-1]++;

     for(unsigned int i=0;i<fRowCount;i++)
          fRows[i]->Cell(_col_index)->setContext(_ctx);
}
//---------------------------------------------------------------------------

/**
 * Metoda klasy XTT_Table ustawiajaca rozmiae kontekstu
 * @param unsigned int _ctx - Index kontesktu
 * @param unsigned int _size - Nowy rozmiar kontekstu
 * @returns Brak zwracanych wartoci
*/
void XTT_Table::setContextSize(unsigned int _ctx, unsigned int _size)
{
     if((_ctx < 1) || (_ctx >= 5))
          return;

     fnContextSize[_ctx-1] = _size;
}
//---------------------------------------------------------------------------

/**
 * Metoda klasy XTT_Table wyszukujaca index kolumny o zadanym id
 * @param unsigned int _a_c_id - zadane id kolumny
 * @returns Index kolumny, lub -1 kiedy nie znajdzie
*/
int XTT_Table::IndexOfColId(unsigned int _a_c_id)
{
     for(int i=0;i<ColCount();i++)
          if(fAbstractColID[i] == _a_c_id)
               return i;

     return -1;
}
//---------------------------------------------------------------------------

/**
 * Metoda klasy XTT_Table zwracajaca id kolumny o zadanym indexie
 * @param unsigned int _index - index kolumny
 * @returns id kolumny lub -1 jezeli index jest niepoprawny
*/
int XTT_Table::AbstractColID(unsigned int _index)
{
     if((((int)_index) < 0) || (_index >= (unsigned int)ColCount()))
          return -1;

     return fAbstractColID[_index];
}
//---------------------------------------------------------------------------

/**
 * Metoda klasy wyszukujaca i zwracajaca index w tablicy argumentow 
 * wsyteoujacych w danej tabeli argumentu o podanym ID
 * @param unsigned int id - ID danego argumentu
 * @returns Index jezeli id wystepuje -1 jezeli nie wsytepuje
*/
int XTT_Table::IndexOfAttributeID(unsigned int id)
{
     for(int i=0;i<ColCount();i++)
          if(fContextAtributes[i]->Id() == id)
               return i;

     return -1;
}
//---------------------------------------------------------------------------

/**
 * Metoda klasy zamieniajaca ze soba dwa wiersze miejscami
 * @param unsigned int _index1 - Index pierwszego wiersza
 * @param unsigned int _index2 - Index drugiego wiersza
 * @param bool _keepConnections - okresla czy nalezy odswiezyc polaczenia
 * @returns true jezeli indexy sa poprawne inaczej false
*/
bool XTT_Table::ExchangeRows(unsigned int _index1, unsigned int _index2, bool _keepConnections)
{
     if((((int)_index1) < 0) || (_index1 >= fRowCount))
          return false;
     if((((int)_index2) < 0) || (_index2 >= fRowCount))
          return false;
	
	// Zamiana wskaznikow do wierszy
     XTT_Row* tmp = fRows[_index1];
     fRows[_index1] = fRows[_index2];
     fRows[_index2] = tmp;
	
	// Zamiana wysokosci wierszy
	unsigned int tmph = fHeight[_index1];
	fHeight[_index1] = fHeight[_index2];
	fHeight[_index2] = tmph;
	
	// Jezeli nalezy odswiezyc polaczenia
	if(_keepConnections)
		ConnectionsList->RepaintConnectionRelatedWithTable(fTableID);
		
	// Odswiezenie widoku tabeli
	fGTable->RePosition();

     return true;
}
//---------------------------------------------------------------------------

/**
 * Metoda zwracajaca index pierwszego wiersza do ktorego jest podlaczone jakies polaczenie
 * @returns Zwraca index pierwszego wiersza do ktorego jest podlaczone jakies polaczenie, jezeli nie ma polaczen przychodzacych to -1
*/
int XTT_Table::firstConnectionRowIndex(void)
{	
	int res = -1;
	
	for(unsigned int i=0;i<RowCount();i++)
	{
		QList<XTT_Connections*>* conn_list = Row(i)->InConnection();
		if(conn_list->size() > 0)
			res = i;
		delete conn_list;
		
		if(res > -1)
			return res;
	}
	
	return res;
}
//---------------------------------------------------------------------------

/**
 * Metoda zwracajaca liste polaczen przychodzacych do tej tabeli
 * @returns Zwraca liste polaczen przychodzacych do tej tabeli
*/
QList<XTT_Connections*>* XTT_Table::inConnectionsList(void)
{
	QList<XTT_Connections*>* res = new QList<XTT_Connections*>;
	
	for(unsigned int i=0;i<RowCount();i++)
	{
		QList<XTT_Connections*>* conn_list = Row(i)->InConnection();
		(*res) << (*conn_list);
		delete conn_list;
	}
	
	return res;
}
//---------------------------------------------------------------------------

/**
 * Metoda zwracajaca liste polaczen wychodzacych z tej tabeli
 * @returns Zwraca liste polaczen wychodzacych z tej tabeli
*/
QList<XTT_Connections*>* XTT_Table::outConnectionsList(void)
{
	return ConnectionsList->listOfOutConnections(fTableID);
}
//---------------------------------------------------------------------------

/**
 * Metoda zapisujaca wszystkie dane w formacie xttml 1.0
 * @returns Zwraca liste stringow w tym formacie
*/
QStringList* XTT_Table::XTTML_1_0(void)
{
     QStringList* res = new QStringList;
     res->clear();

     *res << "<xtt_table table_id=\"" + QString::number(fTableID) + "\" next_cell_id=\"" +  QString::number(fNextCellId) + "\" table_type=\"" +  QString::number(fTableType) + "\" next_abstract_id=\"" +  QString::number(fNextAbstractID) + "\">";
     *res << "\t<name>" + fTitle + "</name>";
	if(fDescription != "")
		*res << "\t<description>" + fDescription + "</description>";

     // Zapis wierszy
     for(unsigned int a=0;a<fRowCount;a++)
     {
          QStringList* al = fRows[a]->XTTML_1_0();
          for(int l=0;l<al->size();l++)
               *res << "\t" + al->at(l);
          delete al;
     }

     // Zapis rozmiarow kontekstow
     *res << "\t<context_size>";
     for(unsigned int a=0;a<4;a++)
               *res << "\t\t<cs>" + QString::number(fnContextSize[a]) + "</cs>";
     *res << "\t</context_size>";

     // Zapis atrybotow
     *res << "\t<context_attributes>";
     for(unsigned int a=0;a<(unsigned int)ColCount();a++)
               *res << "\t\t<ca>" + QString::number(fContextAtributes[a]->Id()) + "</ca>";
     *res << "\t</context_attributes>";

     // Zapis abstrakcyjnych id
     *res << "\t<abstract_id>";
     for(unsigned int a=0;a<(unsigned int)ColCount();a++)
               *res << "\t\t<ai>" + QString::number(fAbstractColID[a]) + "</ai>";
     *res << "\t</abstract_id>";

     // Zapis lokalizacji
     *res << "\t<localization xpos=\"" + QString::number(fPoint.x()) + "\" ypos=\"" +  QString::number(fPoint.y()) + "\" header_height=\"" +  QString::number(fHeaderHeight) + "\" level=\"" +  QString::number(fLevel) + "\" parent=\"" +  QString::number(fParent) + "\" changed=\"" +  QString::number(fChanged?1:0) + "\">";

     // Zapis szerokosci kolumn
     *res << "\t\t<col_widths>";
     for(unsigned int a=0;a<(unsigned int)ColCount();a++)
               *res << "\t\t\t<cw>" + QString::number(fWidth[a]) + "</cw>";
     *res << "\t\t</col_widths>";

     // Zapis wysokosci wierszy kolumn
     *res << "\t\t<nrow_heights>";
     for(unsigned int a=0;a<fRowCount;a++)
               *res << "\t\t\t<rh>" + QString::number(fHeight[a]) + "</rh>";
     *res << "\t\t</nrow_heights>";

     *res << "\t</localization>";

     *res << "</xtt_table>";

     return res;
}
// -----------------------------------------------------------------------------

#if QT_VERSION >= 0x040300
/**
 * Metoda zapisujaca wszystkie dane w formacie xttml 1.0
 * @param QXmlStreamWriter* _xmldoc - wskanik do dokumentu
 * @returns brak zwracanych wartosci
*/
void XTT_Table::XTTML_1_0(QXmlStreamWriter* _xmldoc)
{
	_xmldoc->writeStartElement("xtt_table");
	_xmldoc->writeAttribute("table_id", QString::number(fTableID));
	_xmldoc->writeAttribute("next_cell_id", QString::number(fNextCellId));
	_xmldoc->writeAttribute("table_type", QString::number(fTableType));
	_xmldoc->writeAttribute("next_abstract_id", QString::number(fNextAbstractID));

		_xmldoc->writeTextElement("name", fTitle);
		if(fDescription != "")
			_xmldoc->writeTextElement("description", fDescription);

	     // Zapis wierszy
	     for(unsigned int a=0;a<fRowCount;a++)
			fRows[a]->XTTML_1_0(_xmldoc);

	     // Zapis rozmiarow kontekstow
		_xmldoc->writeStartElement("context_size");
	     for(unsigned int a=0;a<4;a++)
			_xmldoc->writeTextElement("cs", QString::number(fnContextSize[a]));
	     _xmldoc->writeEndElement();

	     // Zapis atrybotow
		_xmldoc->writeStartElement("context_attributes");
	     for(unsigned int a=0;a<(unsigned int)ColCount();a++)
			_xmldoc->writeTextElement("ca", QString::number(fContextAtributes[a]->Id()));
	     _xmldoc->writeEndElement();

	     // Zapis abstrakcyjnych id
		_xmldoc->writeStartElement("abstract_id");
	     for(unsigned int a=0;a<(unsigned int)ColCount();a++)
			_xmldoc->writeTextElement("ai", QString::number(fAbstractColID[a]));
	     _xmldoc->writeEndElement();

		// Zapis lokalizacji
		_xmldoc->writeStartElement("localization");
		_xmldoc->writeAttribute("xpos", QString::number(fPoint.x()));
		_xmldoc->writeAttribute("ypos", QString::number(fPoint.y()));
		_xmldoc->writeAttribute("header_height", QString::number(fHeaderHeight));
		_xmldoc->writeAttribute("level", QString::number(fLevel));
		_xmldoc->writeAttribute("parent", QString::number(fParent));
		_xmldoc->writeAttribute("changed", QString::number(fChanged?1:0));

		     // Zapis szerokosci kolumn
			_xmldoc->writeStartElement("col_widths");
		     for(unsigned int a=0;a<(unsigned int)ColCount();a++)
				_xmldoc->writeTextElement("cw", QString::number(fWidth[a]));
		     _xmldoc->writeEndElement();

		     // Zapis wysokosci wierszy
			_xmldoc->writeStartElement("nrow_heights");
		     for(unsigned int a=0;a<fRowCount;a++)
				_xmldoc->writeTextElement("rh", QString::number(fHeight[a]));
		     _xmldoc->writeEndElement();

	     _xmldoc->writeEndElement();

     _xmldoc->writeEndElement();
}
#endif
// -----------------------------------------------------------------------------

/**
 * Metoda zapisujaca wszystkie dane w formacie xttml 2.0
 * @returns Zwraca liste stringow w tym formacie
*/
QStringList* XTT_Table::XTTML_2_0(void)
{
     QStringList* res = new QStringList;
     res->clear();
	
	QStringList tableTypes;
	tableTypes << "initial" << "middle" << "halt";

     *res << "<xtt_table table_type=\"" +  tableTypes.at(fTableType) + "\" name=\"" + fTitle + "\">";
     if(fDescription != "")
		*res << "\t<description>" + fDescription + "</description>";

     // Zapis wierszy
     for(unsigned int a=0;a<fRowCount;a++)
     {
          QStringList* al = fRows[a]->XTTML_2_0();
          for(int l=0;l<al->size();l++)
               *res << "\t" + al->at(l);
          delete al;
     }

     // Zapis rozmiarow kontekstow
     *res << "\t<context_size conditional=\"" + QString::number(fnContextSize[0]) + "\" assert=\"" + QString::number(fnContextSize[1]) + "\" retract=\"" + QString::number(fnContextSize[2]) + "\" action=\"" + QString::number(fnContextSize[3]) + "\"/>";

     // Zapis atrybotow
     *res << "\t<column_attributes>";
     for(unsigned int a=0;a<(unsigned int)ColCount();a++)
          *res << "\t\t<ca>" + AttributeGroups->CreatePath(fContextAtributes[a]) + fContextAtributes[a]->Name() + "</ca>";
     *res << "\t</column_attributes>";

     // Zapis lokalizacji
     *res << "\t<localization xpos=\"" + QString::number(fPoint.x()) + "\" ypos=\"" +  QString::number(fPoint.y()) + "\" header_height=\"" +  QString::number(fHeaderHeight) + "\" level=\"" +  QString::number(fLevel) + "\" parent=\"" +  QString::number(fParent) + "\">";

     // Zapis szerokosci kolumn
     *res << "\t\t<col_widths>";
     for(unsigned int a=0;a<(unsigned int)ColCount();a++)
               *res << "\t\t\t<cw>" + QString::number(fWidth[a]) + "</cw>";
     *res << "\t\t</col_widths>";

     // Zapis wysokosci wierszy kolumn
     *res << "\t\t<nrow_heights>";
     for(unsigned int a=0;a<fRowCount;a++)
               *res << "\t\t\t<rh>" + QString::number(fHeight[a]) + "</rh>";
     *res << "\t\t</nrow_heights>";

     *res << "\t</localization>";

     *res << "</xtt_table>";

     return res;
}
// -----------------------------------------------------------------------------

#if QT_VERSION >= 0x040300
/**
 * Metoda zapisujaca wszystkie dane w formacie xttml 2.0
 * @param QXmlStreamWriter* _xmldoc - wskanik do dokumentu
 * @returns brak zwracanych wartosci
*/
void XTT_Table::XTTML_2_0(QXmlStreamWriter* _xmldoc)
{
	QStringList tableTypes;
	tableTypes << "initial" << "middle" << "halt";

	_xmldoc->writeStartElement("xtt_table");
	_xmldoc->writeAttribute("table_type", tableTypes.at(fTableType));
	_xmldoc->writeAttribute("name", fTitle);
	
	     if(fDescription != "")
			_xmldoc->writeTextElement("description", fDescription);

	     // Zapis wierszy
	     for(unsigned int a=0;a<fRowCount;a++)
	          fRows[a]->XTTML_2_0(_xmldoc);

	     // Zapis rozmiarow kontekstow
		_xmldoc->writeEmptyElement("context_size");
		_xmldoc->writeAttribute("conditional", QString::number(fnContextSize[0]));
		_xmldoc->writeAttribute("assert", QString::number(fnContextSize[1]));
		_xmldoc->writeAttribute("retract", QString::number(fnContextSize[2]));
		_xmldoc->writeAttribute("action", QString::number(fnContextSize[3]));

	     // Zapis atrybotow
		_xmldoc->writeStartElement("column_attributes");
	     for(unsigned int a=0;a<(unsigned int)ColCount();a++)
			_xmldoc->writeTextElement("ca", AttributeGroups->CreatePath(fContextAtributes[a]) + fContextAtributes[a]->Name());
	     _xmldoc->writeEndElement();

	     // Zapis lokalizacji
		_xmldoc->writeStartElement("localization");
		_xmldoc->writeAttribute("xpos", QString::number(fPoint.x()));
		_xmldoc->writeAttribute("ypos", QString::number(fPoint.y()));
		_xmldoc->writeAttribute("header_height", QString::number(fHeaderHeight));
		_xmldoc->writeAttribute("level", QString::number(fLevel));
		_xmldoc->writeAttribute("parent", QString::number(fParent));

		     // Zapis szerokosci kolumn
			_xmldoc->writeStartElement("col_widths");
		     for(unsigned int a=0;a<(unsigned int)ColCount();a++)
				_xmldoc->writeTextElement("cw", QString::number(fWidth[a]));
		     _xmldoc->writeEndElement();

		     // Zapis wysokosci wierszy kolumn
			_xmldoc->writeStartElement("nrow_heights");
		     for(unsigned int a=0;a<fRowCount;a++)
				_xmldoc->writeTextElement("rh", QString::number(fHeight[a]));
		     _xmldoc->writeEndElement();

	     _xmldoc->writeEndElement();

     _xmldoc->writeEndElement();
}
#endif
// -----------------------------------------------------------------------------

/**
 * Metoda wczytujaca dane o strybucie z pliku w formacie XTTML 1.0
 * @param QStringList* list - Wskanik do sekcji dotyczacej tego atrybutu
 * @returns true jezeli dane sa ok, ianczej false
*/
bool XTT_Table::FromXTTML_1_0(QStringList* list)      
{
     bool res = true;

     // Usuwanie spacji i tabulatorow wierszach
     for(int i=0;i<list->size();i++)
     {
          QString line = list->at(i);
          while(line.contains("\t"))
          {
               int index = line.indexOf("\t");
               line = line.remove(index, 1);
          }
          list->replace(i, line);
     }

     // Zakladamy ze pierwszy wiersz jest naglowkiem
     if(!list->at(0).contains("<xtt_table", Qt::CaseInsensitive))
          return false;
     if(!list->contains("</xtt_table>", Qt::CaseInsensitive))
          return false;

     // Reszta zakladamy ze jest ok

     // Spis tagow
     QStringList tags;
     tags.clear();
     tags << "<xtt_table" << "<row" << "</row>" << "<name>" << "<description>" << "<context_size>" << "<cs>" << "<context_attributes>" << "<ca>";
     tags << "<abstract_id>" << "<ai>" << "<localization" << "<col_widths>" << "<cw>" << "<nrow_heights>" << "<rh>" << "</context_size>";

     // Sluza do odpowiedniego kopiowania lini dla listy atrybutow
     bool copy = false;
     QStringList buf;
     buf.clear();

     // Okresla ktra wartosc wczytujemy z wartosci indexowanych, jak np fContextAtributes, lub fnContextSize
     int index;
     unsigned int* cs = new unsigned int[4];

     for(int line=0;line<list->size();line++)
     {
          //QMessageBox::warning(NULL, "tworzenie tabeli", list->at(line));
          // Sprawdzanie wystapienia tagu
          int action = -1;
          for(int tag=0;tag<tags.size();tag++)
               if(list->at(line).contains(tags.at(tag), Qt::CaseInsensitive))
                    action = tag;

          // Kopiowanie lini danych                               
          if(copy)
               buf.append(list->at(line));

          if(action == -1)
               continue;

          QString l = list->at(line);
          l = l.remove(l.indexOf(tags.at(action), Qt::CaseInsensitive), tags.at(action).length());
          l = l.mid(0, l.indexOf("<", Qt::CaseInsensitive));


          // Wczytywanie parametrow w zaleznosci od wykrytego tagu
          if(action == 0)
          {
               fTableID = (unsigned int)XML_ReadValue(list->at(line), "table_id").toInt();
               fNextCellId = (unsigned int)XML_ReadValue(list->at(line), "next_cell_id").toInt();
               fTableType = (unsigned int)XML_ReadValue(list->at(line), "table_type").toInt();
               fNextAbstractID = (unsigned int)XML_ReadValue(list->at(line), "next_abstract_id").toInt();
          }
          if(action == 1)
          {
               // Jezeli tag rozpoczynajacy dane, to uruchamiamy kopiowanie
               copy = true;
               buf.append(list->at(line));
          }                                                                                    
          if(action == 2)
          {
               // Kiedy linia konczaca linie danych
               // Dodajemy nowy atrybut
               AddRow();

               // Wczytujemy dane z pliku
               res = res && fRows[fRowCount-1]->FromXTTML_1_0(&buf);

               // Konczymy kopiowanie
               copy = false;

               // Czyscimy zawartosc danych
               buf.clear();
          }
          if(action == 3)
               fTitle = l;
          if(action == 4)
               fDescription = l;
          if((action == 5) || (action == 7)  || (action == 9)  || (action == 12) || (action == 14))
               index = 0;
          if(action == 6)
          {
               cs[index] = l.toInt();
               //QMessageBox::warning(NULL, fTitle, "zapis rozmiaru kontekstu = " + l + ", index = " + QString::number(index) + ", line = " + list->at(line) + ", teraz cs = " + QString::number(fnContextSize[index]));
               index++;
          } 
          if(action == 8)
          {
               XTT_Attribute* attr;
               int id = l.toInt();
               attr = AttributeGroups->FindAttribute(id);
               if(attr == NULL)
               {
                    QMessageBox::critical(NULL, "XTT Editor", "Error attribute ID when table was read.\nID = " + l, QMessageBox::Ok);
                    return false;
               }

               fContextAtributes[index++] = attr;
               //QMessageBox::warning(NULL, "Application", "wczytywanie atrybutow zakonczone");
          }
          if(action == 10)
               fAbstractColID[index++] = (unsigned int)l.toInt();
          if(action == 11)
          {
               fPoint.setX(XML_ReadValue(list->at(line), "xpos").toInt());
               fPoint.setY(XML_ReadValue(list->at(line), "ypos").toInt());
               fHeaderHeight = (unsigned int)XML_ReadValue(list->at(line), "header_height").toInt();
               fLevel = (unsigned int)XML_ReadValue(list->at(line), "level").toInt();
               fParent = (unsigned int)XML_ReadValue(list->at(line), "parent").toInt();
               fChanged = (bool)XML_ReadValue(list->at(line), "changed").toInt();
          }
          if(action == 13)
               fWidth[index++] = (unsigned int)l.toInt();
          if(action == 15)
               fHeight[index++] = (unsigned int)l.toInt();
          if(action == 16)
          {
               // Obliczanie ilosci kolumn kolumn
               unsigned int sum = 0;
               for(int i=0;i<4;i++)
                    sum += cs[i];

               // Alokacja pamieci na nowe parametry
               fWidth = new unsigned int[sum];
               fAbstractColID = new unsigned int[sum];
               fContextAtributes = new XTT_Attribute*[sum];

               // Ustawianie wartosci domyslnych kolumn
               for(unsigned int i=0;i<sum;i++)
               {
                    fWidth[i] = 120;
                    fAbstractColID[i] = fNextAbstractID++;
               }
               //delete [] fnContextSize;
               fnContextSize = cs;
          }
     }

     // Odswiezenie widoku
     delete fGTable;
     fGTable = new GTable(this);
     fGTable->ReCreate();
     fGTable->Update();
     fGTable->RePosition();

     return res;
}
// -----------------------------------------------------------------------------

/**
 * Metoda wczytujaca dane o strybucie z pliku w formacie XTTML 1.0
 * @param QDomElement& root - Referencja do sekcji dotyczacej tego atrybutu
 * @returns true jezeli dane sa ok, ianczej false
*/
bool XTT_Table::FromXTTML_1_0(QDomElement& root)      
{
	if(root.tagName().toLower() != "xtt_table")
		return false;
		
     bool res = true;
	
	fTableID = (unsigned int)root.attribute("table_id", "0").toInt();
	fNextCellId = (unsigned int)root.attribute("next_cell_id", "0").toInt();
	fTableType = (unsigned int)root.attribute("table_type", "0").toInt();
	fNextAbstractID = (unsigned int)root.attribute("next_abstract_id", "0").toInt();
	
	QDomElement DOMtname = root.firstChildElement("name");
	QDomElement DOMtdesc = root.firstChildElement("description");
	fTitle = DOMtname.text();
	fDescription = DOMtdesc.text();
	
	QDomElement DOMrow = root.firstChildElement("row");
	while(!DOMrow.isNull())
	{
		// Kiedy linia konczaca linie danych
		// Dodajemy nowy atrybut
		AddRow();

		// Wczytujemy dane z pliku
		res = res && fRows[fRowCount-1]->FromXTTML_1_0(DOMrow);
			
		// Wczytanie kolejnego wiersza
		DOMrow = DOMrow.nextSiblingElement("row");
	}
	
	unsigned int* cs = new unsigned int[4];
	
	// Wczytywanie rozmiarow kontekstow
	int index = 0;
	QDomElement DOMctxsize = root.firstChildElement("context_size");
	QDomElement DOMcs = DOMctxsize.firstChildElement("cs");
	while(!DOMcs.isNull())
	{
		if(index < 4)
			cs[index++] = DOMcs.text().toInt();
		
		// Wczytanie kolejnego wiersza
		DOMcs = DOMcs.nextSiblingElement("cs");
	}
	// Obliczanie ilosci kolumn kolumn
	unsigned int sum = 0;
	for(int i=0;i<4;i++)
		sum += cs[i];

	// Alokacja pamieci na nowe parametry
	fWidth = new unsigned int[sum];
	fAbstractColID = new unsigned int[sum];
	fContextAtributes = new XTT_Attribute*[sum];

	// Ustawianie wartosci domyslnych kolumn
	for(unsigned int i=0;i<sum;i++)
	{
		fWidth[i] = 120;
		fAbstractColID[i] = fNextAbstractID++;
	}
	//delete [] fnContextSize;
	fnContextSize = cs;
	
	// Wczytywanie atrybutow kolumn
	index = 0;
	QDomElement DOMcontext_attr = root.firstChildElement("context_attributes");
	QDomElement DOMca = DOMcontext_attr.firstChildElement("ca");
	while(!DOMca.isNull())
	{
		XTT_Attribute* attr;
		int id = DOMca.text().toInt();
		attr = AttributeGroups->FindAttribute(id);
		if(attr == NULL)
		{
			QMessageBox::critical(NULL, "XTT Editor", "Error attribute ID when table was read.\nID = " + DOMca.text(), QMessageBox::Ok);
			return false;
		}

		fContextAtributes[index++] = attr;
		
		// Wczytanie kolejnego identyfikatora atrybutu
		DOMca = DOMca.nextSiblingElement("ca");
	}
	
	// Wczytywanie identyfikatorow
	index = 0;
	QDomElement DOMabstractid = root.firstChildElement("abstract_id");
	QDomElement DOMai = DOMabstractid.firstChildElement("ai");
	while(!DOMai.isNull())
	{
		fAbstractColID[index++] = (unsigned int)DOMai.text().toInt();
	
		// Wczytanie kolejnego identyfikatora atrybutu
		DOMai = DOMai.nextSiblingElement("ai");
	}
             

	// Wczytywanie lokalizacji i rozmiarow
	QDomElement DOMgeometry = root.firstChildElement("localization");
	fPoint.setX(DOMgeometry.attribute("xpos", "0").toInt());
	fPoint.setY(DOMgeometry.attribute("ypos", "0").toInt());
	fHeaderHeight = (unsigned int)DOMgeometry.attribute("header_height", "0").toInt();
	fLevel = (unsigned int)DOMgeometry.attribute("level", "0").toInt();
	fParent = (unsigned int)DOMgeometry.attribute("parent", "0").toInt();
	fChanged = (bool)DOMgeometry.attribute("changed", "0").toInt();
	
		QDomElement DOMcolwidths = DOMgeometry.firstChildElement("col_widths");
			QDomElement DOMcw = DOMcolwidths.firstChildElement("cw");
		QDomElement DOMrowheights = DOMgeometry.firstChildElement("nrow_heights");
			QDomElement DOMrh = DOMrowheights.firstChildElement("rh");
			
			index = 0;
			while(!DOMcw.isNull())
			{
				fWidth[index++] = (unsigned int)DOMcw.text().toInt();
				DOMcw = DOMcw.nextSiblingElement("cw");
			}
			
			index = 0;
			while(!DOMrh.isNull())
			{
				fHeight[index++] = (unsigned int)DOMrh.text().toInt();
				DOMrh = DOMrh.nextSiblingElement("rh");
			}

     // Odswiezenie widoku
     delete fGTable;
     fGTable = new GTable(this);
     fGTable->ReCreate();
     fGTable->Update();
     fGTable->RePosition();

     return res;
}
// -----------------------------------------------------------------------------

/**
 * Metoda wczytujaca dane o strybucie z pliku w formacie XTTML 2.0
 * @param QStringList* list - Wskanik do sekcji dotyczacej tego atrybutu
 * @returns true jezeli dane sa ok, ianczej false
*/
bool XTT_Table::FromXTTML_2_0(QStringList* list)      
{
	// Resetowanie ilosci kontekstow
	fnContextSize[0] = 0;
	fnContextSize[1] = 0;
	fnContextSize[2] = 0;
	fnContextSize[3] = 0;

     bool res = true;

     // Usuwanie spacji i tabulatorow wierszach
     for(int i=0;i<list->size();i++)
     {
          QString line = list->at(i);
          while(line.contains("\t"))
          {
               int index = line.indexOf("\t");
               line = line.remove(index, 1);
          }
          list->replace(i, line);
     }

     // Zakladamy ze pierwszy wiersz jest naglowkiem
     if(!list->at(0).contains("<xtt_table", Qt::CaseInsensitive))
          return false;
     if(!list->contains("</xtt_table>", Qt::CaseInsensitive))
          return false;

     // Reszta zakladamy ze jest ok

     // Spis tagow
     QStringList tags;
     tags.clear();
     tags << "<xtt_table" << "<row" << "</row>" << "<description>" << "<context_size" << "<column_attributes>" << "<ca>";
	tags << "<localization" << "<col_widths>" << "<cw>" << "<nrow_heights>" << "<rh>" << "</xtt_table>";
	
	QStringList tableTypes;
	tableTypes << "initial" << "middle" << "halt";

     // Sluza do odpowiedniego kopiowania lini dla listy atrybutow
     bool copy = false;
     QStringList buf;
     buf.clear();

     // Okresla ktra wartosc wczytujemy z wartosci indexowanych, jak np fContextAtributes, lub fnContextSize
     int index;

     for(int line=0;line<list->size();line++)
     {
          //QMessageBox::warning(NULL, "tworzenie tabeli", list->at(line));
          // Sprawdzanie wystapienia tagu
          int action = -1;
          for(int tag=0;tag<tags.size();tag++)
               if(list->at(line).contains(tags.at(tag), Qt::CaseInsensitive))
                    action = tag;

          // Kopiowanie lini danych                               
          if(copy)
               buf.append(list->at(line));

          if(action == -1)
               continue;
			
		//QMessageBox::critical(NULL, "XTT Editor", "Action = " + QString::number(action), QMessageBox::Ok);

          QString l = list->at(line);
          l = l.remove(l.indexOf(tags.at(action), Qt::CaseInsensitive), tags.at(action).length());
          l = l.mid(0, l.indexOf("<", Qt::CaseInsensitive));


          // Wczytywanie parametrow w zaleznosci od wykrytego tagu
		
		// "<xtt_table"
          if(action == 0)
          {
			QString tableType = XML_ReadValue(list->at(line), "table_type");
			QString tableTitle = XML_ReadValue(list->at(line), "name");
			
			// Usuwanie spacji z wczytanych wartosci
			for(;tableType.contains(" ");)
				tableType = tableType.remove(tableType.indexOf(" "), 1);
			for(;tableTitle.contains(" ");)
				tableTitle = tableTitle.remove(tableTitle.indexOf(" "), 1);
			
			// Oba parametry sa wymagane wiec sprawdzanie czy wystepuja
			if((tableType == "") || (tableType == "#NAN#"))
			{
				QMessageBox::critical(NULL, "XTT Editor", "Missing table parameter \"table_type\" at line: " + list->at(line) + ".", QMessageBox::Ok);
				return false;
			}
			
			// Jezeli okreslono typ tabeli to trzeba sprawdzic czy poprawnie
			if(!tableTypes.contains(tableType, Qt::CaseInsensitive))
			{
				QMessageBox::critical(NULL, "XTT Editor", "Incorrect table type: \"" + tableType + "\" at line:\n" + list->at(line) + ".", QMessageBox::Ok);
				return false;
			}
			
			// Sprawdzanie podania nazwy tabeli
			if((tableTitle == "") || (tableTitle == "#NAN#"))
			{
				QMessageBox::critical(NULL, "XTT Editor", "Missing table parameter \"name\" at line: " + list->at(line) + ".", QMessageBox::Ok);
				return false;
			}
			
			fTitle = tableTitle;
			fTableType = tableTypes.indexOf(tableType.toLower());
          }
		
		// "<row"
          if(action == 1)
          {
               // Jezeli tag rozpoczynajacy dane, to uruchamiamy kopiowanie
               copy = true;
               buf.append(list->at(line));
          }    

		// "</row>"
          if(action == 2)
          {
               // Kiedy linia konczaca linie danych
               // Dodajemy nowy wiersz
               AddRow();

               // Wczytujemy dane z pliku
               res = res && fRows[fRowCount-1]->FromXTTML_2_0(&buf);

               // Konczymy kopiowanie
               copy = false;

               // Czyscimy zawartosc danych
               buf.clear();
          }
		
		// "<description>"
          if(action == 3)
               fDescription = l;
			
		// "<context_size"
          if(action == 4)
		{
               QString sConditional = XML_ReadValue(list->at(line), "conditional");
			QString sAssert = XML_ReadValue(list->at(line), "assert");
			QString sRetract = XML_ReadValue(list->at(line), "retract");
			QString sAction = XML_ReadValue(list->at(line), "action");
			
			// Podanie nie jest wymagane, jezeli parametr nie jest podany to przyjmujemy wielkosc za 0
			// Usuwanie spacji z wczytanych wartosci
			for(;sConditional.contains(" ");)
				sConditional = sConditional.remove(sConditional.indexOf(" "), 1);
			for(;sAssert.contains(" ");)
				sAssert = sAssert.remove(sAssert.indexOf(" "), 1);
			for(;sRetract.contains(" ");)
				sRetract = sRetract.remove(sRetract.indexOf(" "), 1);
			for(;sAction.contains(" ");)
				sAction = sAction.remove(sAction.indexOf(" "), 1);
				
			int iConditional = 0;
			int iAssert = 0;
			int iRetract = 0;
			int iAction = 0;
				
			// Proby konwersji jezeli podano
			if((sConditional != "") && (sConditional != "#NAN#"))
			{
				bool ok;
				iConditional = sConditional.toInt(&ok);
				if((!ok) || (iConditional < 0))
				{
					QMessageBox::information(NULL, "XTT Editor", "Incorrect table \"" + fTitle + "\" conditional context size: \"" + sConditional + "\" at line: " + list->at(line) + ".\nContext will be set as zero.", QMessageBox::Ok);
					iConditional = 0;
				}
			}
			if((sAssert != "") && (sAssert != "#NAN#"))
			{
				bool ok;
				iAssert = sAssert.toInt(&ok);
				if((!ok) || (iAssert < 0))
				{
					QMessageBox::information(NULL, "XTT Editor", "Incorrect table \"" + fTitle + "\" assert context size: \"" + sAssert + "\" at line: " + list->at(line) + ".\nContext will be set as zero.", QMessageBox::Ok);
					iAssert = 0;
				}
			}
			if((sRetract != "") && (sRetract != "#NAN#"))
			{
				bool ok;
				iRetract = sRetract.toInt(&ok);
				if((!ok) || (iRetract < 0))
				{
					QMessageBox::information(NULL, "XTT Editor", "Incorrect table \"" + fTitle + "\" retract context size: \"" + sRetract + "\" at line: " + list->at(line) + ".\nContext will be set as zero.", QMessageBox::Ok);
					iRetract = 0;
				}
			}
			if((sAction != "") && (sAction != "#NAN#"))
			{
				bool ok;
				iAction = sAction.toInt(&ok);
				if((!ok) || (iAction < 0))
				{
					QMessageBox::information(NULL, "XTT Editor", "Incorrect table \"" + fTitle + "\" action context size: \"" + sAction + "\" at line: " + list->at(line) + ".\nContext will be set as zero.", QMessageBox::Ok);
					iAction = 0;
				}
			}
			
			fnContextSize[0] = iConditional;
			fnContextSize[1] = iAssert;
			fnContextSize[2] = iRetract;
			fnContextSize[3] = iAction;
			
			// Suma kontekstow jest rowna ilosci kolumn, teraz sprawdzamy czy tak jest rzeczywiscie
			// NAJPIERW MUSZA ZOSTAC WCZYTANE WIERSZE	
			for(int r=0;r<(int)fRowCount;r++)
			{
				if(((int)fRows[r]->Length()) != ColCount())
				{
					//QMessageBox::critical(NULL, "XTT Editor", "To long/short row " + QString::number(r+1) + ". Row length was declared as " + QString::number(ColCount()) + ". Error in table \"" + fTitle + "\".", QMessageBox::Ok);
					return false;
				}
			}
			
			// Jezeli wszystko ok to przystepujemy do alokacji pamieci na pozostale parametry
			fWidth = new unsigned int[ColCount()];
               fAbstractColID = new unsigned int[ColCount()];
               fContextAtributes = new XTT_Attribute*[ColCount()];

               // Ustawianie wartosci domyslnych kolumn i ich kontekstow
               for(int i=0;i<ColCount();i++)
               {
                    fWidth[i] = 120;
                    fAbstractColID[i] = fNextAbstractID++;
				setColContext(i, ColContext(i));
               }
		}
			
		// "<column_attributes>", "<col_widths>", "<nrow_heights>"
          if((action == 5) || (action == 8)  || (action == 10))
               index = 0;
			
		// "<ca>"
          if(action == 6)
          {
			// Sprawdzanie czy zgadza sie ilosc kolumn w kazdym wierszu z suma wielkosci kontekstow
			int ctx_sum = ColCount();
			if(index >= ctx_sum)
			{
				QMessageBox::information(NULL, "XTT Editor", "Too many columns are defined. Error at line: " + list->at(line) + ".", QMessageBox::Ok);
				return false;
			}
			
			QString path = l;
			XTT_Attribute* attr = AttributeGroups->Attribute(path);
			if(attr == NULL)
			{
				QMessageBox::critical(NULL, "XTT Editor", "Undefined attribute: \"" + path + "\" at line: " + list->at(line) + ".", QMessageBox::Ok);
				return false;
			}
			
			// Jezeli odnajdziemy dany atrybut to mozemy go dodac
			fContextAtributes[index] = attr;
			
			// Oraz musimy go zapisac w kazdej komorce
			for(unsigned int r=0;r<fRowCount;r++)
				fRows[r]->Cell(index)->setAttrType(attr);
				
			// Powiekszenie indexu wczytywanej wartosci
			index++;
          }
		
		// "<localization"
          if(action == 7)
          {
			QString sXPos = XML_ReadValue(list->at(line), "xpos");
			QString sYPos = XML_ReadValue(list->at(line), "ypos");
			QString sHHeight = XML_ReadValue(list->at(line), "header_height");
			QString sLevel = XML_ReadValue(list->at(line), "level");
			QString sParent = XML_ReadValue(list->at(line), "parent");
			
			// Podanie nie jest wymagane, jezeli parametr nie jest podany to przyjmujemy wielkosc za 0
			// Usuwanie spacji z wczytanych wartosci
			for(;sXPos.contains(" ");)
				sXPos = sXPos.remove(sXPos.indexOf(" "), 1);
			for(;sYPos.contains(" ");)
				sYPos = sYPos.remove(sYPos.indexOf(" "), 1);
			for(;sHHeight.contains(" ");)
				sHHeight = sHHeight.remove(sHHeight.indexOf(" "), 1);
			for(;sLevel.contains(" ");)
				sLevel = sLevel.remove(sLevel.indexOf(" "), 1);
			for(;sParent.contains(" ");)
				sParent = sParent.remove(sParent.indexOf(" "), 1);
				
			// Parametry nie sa wymagane, jezeli nie zostana podane to zostana im
			// przypisane nastpujace wartosci domyslne
			int iXPos = 0;
			int iYPos = 0;
			int iHHeight = 20;
			int iLevel = 0;
			int iParent = -1;
				
			// Proby konwersji jezeli podano
			if((sXPos != "") && (sXPos != "#NAN#"))
			{
				bool ok;
				iXPos = sXPos.toInt(&ok);
				if(!ok)
					QMessageBox::information(NULL, "XTT Editor", "Incorrect value type. Value \"" + sXPos + "\" is not integer type. Error at line: " + list->at(line) + ".", QMessageBox::Ok);
				fPoint.setX(iXPos);
			}
			if((sYPos != "") && (sYPos != "#NAN#"))
			{
				bool ok;
				iYPos = sYPos.toInt(&ok);
				if(!ok)
					QMessageBox::information(NULL, "XTT Editor", "Incorrect value type. Value \"" + sYPos + "\" is not integer type. Error at line: " + list->at(line) + ".", QMessageBox::Ok);
				fPoint.setY(iYPos);
			}
			if((sHHeight != "") && (sHHeight != "#NAN#"))
			{
				bool ok;
				iHHeight = sHHeight.toInt(&ok);
				if(!ok)
					QMessageBox::information(NULL, "XTT Editor", "Incorrect value type. Value \"" + sHHeight + "\" is not integer type. Error at line: " + list->at(line) + ".", QMessageBox::Ok);
				fHeaderHeight = (unsigned int)iHHeight;
			}
			if((sLevel != "") && (sLevel != "#NAN#"))
			{
				bool ok;
				iLevel = sLevel.toInt(&ok);
				if(!ok)
					QMessageBox::information(NULL, "XTT Editor", "Incorrect value type. Value \"" + sLevel + "\" is not integer type. Error at line: " + list->at(line) + ".", QMessageBox::Ok);
				fLevel = iLevel;
			}
			if((sParent != "") && (sParent != "#NAN#"))
			{
				bool ok;
				iParent = sParent.toInt(&ok);
				if(!ok)
					QMessageBox::information(NULL, "XTT Editor", "Incorrect value type. Value \"" + sParent + "\" is not integer type. Error at line: " + list->at(line) + ".", QMessageBox::Ok);
				fParent = iParent;
			}
          }
		
		// "<cw>"
          if(action == 9)
          {
			// Mozliwe jest podanie mniejszej ilosci szerokosci niz jest zadeklarowanych kolumn, wtedy szerokosc bedzie ustawiona
			// w pierszych kolumnach od prawej a pozostale szerokosci beda ustawione na 120. Podanie zbyt duzej ilosci wymiarow
			// spowoduje pominiecie tych ostatnich pozycji ktore sa nadmiarowe
			
			
			// Sprawdzanie czy zgadza sie ilosc kolumn w kazdym wierszu z suma wielkosci kontekstow
			int ctx_sum = ColCount();
			if(index >= ctx_sum)
			{
				QMessageBox::information(NULL, "XTT Editor", "Too many widths of columns are defined. Error at line: " + list->at(line) + ".\nThis value wiil be ignored.", QMessageBox::Ok);
				continue;
			}
		
			bool ok_conv;
			unsigned int cw = (unsigned int)l.toInt(&ok_conv);
			
			if(!ok_conv)
			{
				QMessageBox::information(NULL, "XTT Editor", "Incorrect column " + QString::number(index+1) + " width. Value \"" + l + "\" is not integer type. Error at line: " + list->at(line) + ".\nWidth will be set as 10.", QMessageBox::Ok);
				cw = 10;
			}
			
			if(((int)cw) < 0)
			{
				QMessageBox::information(NULL, "XTT Editor", "Incorrect column " + QString::number(index+1) + " width. Value \"" + l + "\"  can't be less than zero. Error at line: " + list->at(line) + ".\nWidth will be set as 10.", QMessageBox::Ok);
				cw = 10;
			}
			
			fWidth[index] = cw;
			index++;
		}
			
		// "<rh>"
		if(action == 11)
		{
			// Sprawdzanie czy liczba wierszy nie zostala przekroczona
			if((unsigned int)index >= fRowCount)
			{
				QMessageBox::information(NULL, "XTT Editor", "Too many heights of row are defined. Error at line: " + list->at(line) + ".\nThis value wiil be ignored.", QMessageBox::Ok);
				continue;
			}
			
			bool ok_conv;
			unsigned int rh = (unsigned int)l.toInt(&ok_conv);
			
			if(!ok_conv)
			{
				QMessageBox::information(NULL, "XTT Editor", "Incorrect row " + QString::number(index+1) + " height. Value \"" + l + "\" is not integer type. Error at line: " + list->at(line) + ".\nHeight will be set as 10.", QMessageBox::Ok);
				rh = 10;
			}
			
			if(((int)rh) < 0)
			{
				QMessageBox::information(NULL, "XTT Editor", "Incorrect row " + QString::number(index+1) + " height. Value \"" + l + "\" can't be less than zero. Error at line: " + list->at(line) + ".\nHeight will be set as 10.", QMessageBox::Ok);
				rh = 10;
			}
			
			fHeight[index] = rh;
			index++;
		}
			
		// "</xtt_table>" - weryfikacja wszystkich ustawien
          if(action == 12)
          {
			// Sprawdzanie czy zgadza sie ilosc kolumn w kazdym wierszu z suma wielkosci kontekstow
			int ctx_sum = ColCount();
			
			for(unsigned int r=0;r<fRowCount;r++)
			{
				if((unsigned int)ctx_sum != fRows[r]->Length())
				{
					QMessageBox::critical(NULL, "XTT Editor", "Incorrect col count in row " + QString::number(r+1) + ". Column count must be equal sum of all contexts size. Error at line: " + list->at(line) + ".", QMessageBox::Ok);
					return false;
				}
				
				// Jezeli wiersz jest odpowiednio dlugi to musimy przeanalizowac komorki
				for(unsigned int c=0;c<fRows[r]->Length();c++)
				{
					// Zapisanie wartosci jezeli komorka jest pusta
					QString curr_content = fRows[r]->Cell(c)->Content();
					
					if((!fRows[r]->Cell(c)->Analyze(fRows[r]->Cell(c)->Content())) && (Settings->xttShowErrorCellMessage()))
						QMessageBox::warning(NULL, "XTT Editor", fRows[r]->Cell(c)->CellError(), QMessageBox::Ok);
					fRows[r]->Cell(c)->setContent(curr_content);
				}
			}
          }
     }

     // Odswiezenie widoku
     delete fGTable;
     fGTable = new GTable(this);
     fGTable->ReCreate();
     fGTable->Update();
     fGTable->RePosition();

     return res;
}
// -----------------------------------------------------------------------------

/**
 * Metoda wczytujaca dane o strybucie z pliku w formacie XTTML 2.0
 * @param QDomElement& root - Referencja do sekcji dotyczacej tej tabeli
 * @returns true jezeli dane sa ok, ianczej false
*/
bool XTT_Table::FromXTTML_2_0(QDomElement& root)      
{
	// Resetowanie ilosci kontekstow
	fnContextSize[0] = 0;
	fnContextSize[1] = 0;
	fnContextSize[2] = 0;
	fnContextSize[3] = 0;

     bool res = true;
	
	if(root.tagName().toLower() != "xtt_table")
		return false;

     // Reszta zakladamy ze jest ok
	
	QStringList tableTypes;
	tableTypes << "initial" << "middle" << "halt";

     // Okresla ktra wartosc wczytujemy z wartosci indexowanych, jak np fContextAtributes, lub fnContextSize
     int index;

	// Wczytywanie parametrow tagu glownego
	QString tableType = root.attribute("table_type", "#NAN#");
	QString tableTitle = root.attribute("name", "#NAN#");
	
	// Usuwanie spacji z wczytanych wartosci
	for(;tableType.contains(" ");)
		tableType = tableType.remove(tableType.indexOf(" "), 1);
	for(;tableTitle.contains(" ");)
		tableTitle = tableTitle.remove(tableTitle.indexOf(" "), 1);
	
	// Oba parametry sa wymagane wiec sprawdzanie czy wystepuja
	if((tableType == "") || (tableType == "#NAN#"))
	{
		QMessageBox::critical(NULL, "XTT Editor", "Missing table parameter \"table_type\" at line " + QString::number(root.lineNumber()) + ".", QMessageBox::Ok);
		return false;
	}
	
	// Jezeli okreslono typ tabeli to trzeba sprawdzic czy poprawnie
	if(!tableTypes.contains(tableType, Qt::CaseInsensitive))
	{
		QMessageBox::critical(NULL, "XTT Editor", "Incorrect table type: \"" + tableType + "\" at line " + QString::number(root.lineNumber()) + ".", QMessageBox::Ok);
		return false;
	}
	
	// Sprawdzanie podania nazwy tabeli
	if((tableTitle == "") || (tableTitle == "#NAN#"))
	{
		QMessageBox::critical(NULL, "XTT Editor", "Missing table parameter \"name\" at line " + QString::number(root.lineNumber()) + ".", QMessageBox::Ok);
		return false;
	}
	
	fTitle = tableTitle;
	fTableType = tableTypes.indexOf(tableType.toLower());
	
	
	// "<row"
	QDomElement DOMrow = root.firstChildElement("row");
	while(!DOMrow.isNull())
	{
		// Kiedy linia konczaca linie danych
		// Dodajemy nowy wiersz
		AddRow();

		// Wczytujemy dane z pliku
		res = res && fRows[fRowCount-1]->FromXTTML_2_0(DOMrow);
		
		// Wczytanie kolejnego wiersza
		DOMrow =  DOMrow.nextSiblingElement("row");
	}
	
	// "<description>"
	QString tDesc = root.firstChildElement("description").text();
	if(!tDesc.isEmpty())
		fDescription = tDesc;
		
	// "<context_size"
	QDomElement DOMcs = root.firstChildElement("context_size");
	if(!DOMcs.isNull())
	{
		QString sConditional = DOMcs.attribute("conditional", "#NAN#");
		QString sAssert = DOMcs.attribute("assert", "#NAN#");
		QString sRetract = DOMcs.attribute("retract", "#NAN#");
		QString sAction = DOMcs.attribute("action", "#NAN#");
		
		// Podanie nie jest wymagane, jezeli parametr nie jest podany to przyjmujemy wielkosc za 0
		// Usuwanie spacji z wczytanych wartosci
		for(;sConditional.contains(" ");)
			sConditional = sConditional.remove(sConditional.indexOf(" "), 1);
		for(;sAssert.contains(" ");)
			sAssert = sAssert.remove(sAssert.indexOf(" "), 1);
		for(;sRetract.contains(" ");)
			sRetract = sRetract.remove(sRetract.indexOf(" "), 1);
		for(;sAction.contains(" ");)
			sAction = sAction.remove(sAction.indexOf(" "), 1);
			
		int iConditional = 0;
		int iAssert = 0;
		int iRetract = 0;
		int iAction = 0;
			
		// Proby konwersji jezeli podano
		if((sConditional != "") && (sConditional != "#NAN#"))
		{
			bool ok;
			iConditional = sConditional.toInt(&ok);
			if((!ok) || (iConditional < 0))
			{
				QMessageBox::information(NULL, "XTT Editor", "Incorrect table \"" + fTitle + "\" conditional context size: \"" + sConditional + "\" at line " + QString::number(DOMcs.lineNumber()) + ".\nContext will be set as zero.", QMessageBox::Ok);
				iConditional = 0;
			}
		}
		if((sAssert != "") && (sAssert != "#NAN#"))
		{
			bool ok;
			iAssert = sAssert.toInt(&ok);
			if((!ok) || (iAssert < 0))
			{
				QMessageBox::information(NULL, "XTT Editor", "Incorrect table \"" + fTitle + "\" assert context size: \"" + sAssert + "\" at line " + QString::number(DOMcs.lineNumber()) + ".\nContext will be set as zero.", QMessageBox::Ok);
				iAssert = 0;
			}
		}
		if((sRetract != "") && (sRetract != "#NAN#"))
		{
			bool ok;
			iRetract = sRetract.toInt(&ok);
			if((!ok) || (iRetract < 0))
			{
				QMessageBox::information(NULL, "XTT Editor", "Incorrect table \"" + fTitle + "\" retract context size: \"" + sRetract + "\" at line " + QString::number(DOMcs.lineNumber()) + ".\nContext will be set as zero.", QMessageBox::Ok);
				iRetract = 0;
			}
		}
		if((sAction != "") && (sAction != "#NAN#"))
		{
			bool ok;
			iAction = sAction.toInt(&ok);
			if((!ok) || (iAction < 0))
			{
				QMessageBox::information(NULL, "XTT Editor", "Incorrect table \"" + fTitle + "\" action context size: \"" + sAction + "\" at line " + QString::number(DOMcs.lineNumber()) + ".\nContext will be set as zero.", QMessageBox::Ok);
				iAction = 0;
			}
		}
		
		fnContextSize[0] = iConditional;
		fnContextSize[1] = iAssert;
		fnContextSize[2] = iRetract;
		fnContextSize[3] = iAction;
		
		// Suma kontekstow jest rowna ilosci kolumn, teraz sprawdzamy czy tak jest rzeczywiscie
		// NAJPIERW MUSZA ZOSTAC WCZYTANE WIERSZE	
		for(int r=0;r<(int)fRowCount;r++)
		{
			if(((int)fRows[r]->Length()) != ColCount())
			{
				//QMessageBox::critical(NULL, "XTT Editor", "To long/short row " + QString::number(r+1) + ". Row length was declared as " + QString::number(ColCount()) + ". Error in table \"" + fTitle + "\".", QMessageBox::Ok);
				return false;
			}
		}
		
		// Jezeli wszystko ok to przystepujemy do alokacji pamieci na pozostale parametry
		fWidth = new unsigned int[ColCount()];
		fAbstractColID = new unsigned int[ColCount()];
		fContextAtributes = new XTT_Attribute*[ColCount()];

		// Ustawianie wartosci domyslnych kolumn i ich kontekstow
		for(int i=0;i<ColCount();i++)
		{
			fWidth[i] = 120;
			fAbstractColID[i] = fNextAbstractID++;
			setColContext(i, ColContext(i));
		}
	}
	
	// "<column_attributes>"
	QDomElement DOMcolattrs = root.firstChildElement("column_attributes");
	QDomElement DOMcolwidths = root.firstChildElement("col_widths");
	QDomElement DOMrowheights = root.firstChildElement("nrow_heights");
	
	// Wczytanie atrybutow dla kolumn
	index = 0;
	QDomElement DOMca = DOMcolattrs.firstChildElement("ca");
	while(!DOMca.isNull())
	{
		// Sprawdzanie czy zgadza sie ilosc kolumn w kazdym wierszu z suma wielkosci kontekstow
		int ctx_sum = ColCount();
		if(index >= ctx_sum)
		{
			QMessageBox::information(NULL, "XTT Editor", "Too many columns are defined. Error at line " + QString::number(DOMca.lineNumber()) + ".", QMessageBox::Ok);
			return false;
		}
		
		QString path = DOMca.text();
		XTT_Attribute* attr = AttributeGroups->Attribute(path);
		if(attr == NULL)
		{
			QMessageBox::critical(NULL, "XTT Editor", "Undefined attribute: \"" + path + "\" at line " + QString::number(DOMca.lineNumber()) + ".", QMessageBox::Ok);
			return false;
		}
		
		// Jezeli odnajdziemy dany atrybut to mozemy go dodac
		fContextAtributes[index] = attr;
		
		// Oraz musimy go zapisac w kazdej komorce
		for(unsigned int r=0;r<fRowCount;r++)
			fRows[r]->Cell(index)->setAttrType(attr);
			
		// Powiekszenie indexu wczytywanej wartosci
		index++;
		
		// Wczytanie kolejengo atrybutu
		DOMca = DOMca.nextSiblingElement("ca");
	}
	
	// Wczytywanie rozmiarow kolumn
	index = 0;
	QDomElement DOMcw = DOMcolwidths.firstChildElement("cw");
	while(!DOMcw.isNull())
	{
		// Mozliwe jest podanie mniejszej ilosci szerokosci niz jest zadeklarowanych kolumn, wtedy szerokosc bedzie ustawiona
		// w pierszych kolumnach od prawej a pozostale szerokosci beda ustawione na 120. Podanie zbyt duzej ilosci wymiarow
		// spowoduje pominiecie tych ostatnich pozycji ktore sa nadmiarowe
		
		
		// Sprawdzanie czy zgadza sie ilosc kolumn w kazdym wierszu z suma wielkosci kontekstow
		int ctx_sum = ColCount();
		if(index >= ctx_sum)
		{
			QMessageBox::information(NULL, "XTT Editor", "Too many widths of columns are defined. Error at line " + QString::number(DOMcw.lineNumber()) + ".\nThis value wiil be ignored.", QMessageBox::Ok);
			continue;
		}
	
		bool ok_conv;
		QString l = DOMcw.text();
		unsigned int cw = (unsigned int)l.toInt(&ok_conv);
		
		if(!ok_conv)
		{
			QMessageBox::information(NULL, "XTT Editor", "Incorrect column " + QString::number(index+1) + " width. Value \"" + l + "\" is not integer type. Error at line " + QString::number(DOMcw.lineNumber()) + ".\nWidth will be set as default.", QMessageBox::Ok);
			cw = 10;
		}
		
		if(((int)cw) < 0)
		{
			QMessageBox::information(NULL, "XTT Editor", "Incorrect column " + QString::number(index+1) + " width. Value \"" + l + "\"  can't be less than zero. Error at line " + QString::number(DOMcw.lineNumber()) + ".\nWidth will be set as default.", QMessageBox::Ok);
			cw = 10;
		}
		
		fWidth[index] = cw;
		index++;
	
		// Wczytanie kolejengo rozmiaru dla kolumny
		DOMcw = DOMcw.nextSiblingElement("cw");
	}
	
	// Wczytywanie rozmiarow wierszy
	index = 0;
	QDomElement DOMrh = DOMrowheights.firstChildElement("rh");
	while(!DOMrh.isNull())
	{
		// Sprawdzanie czy liczba wierszy nie zostala przekroczona
		if((unsigned int)index >= fRowCount)
		{
			QMessageBox::information(NULL, "XTT Editor", "Too many heights of row are defined. Error at line " + QString::number(DOMrh.lineNumber()) + ".\nThis value wiil be ignored.", QMessageBox::Ok);
			continue;
		}
		
		bool ok_conv;
		QString l = DOMrh.text();
		unsigned int rh = (unsigned int)l.toInt(&ok_conv);
		
		if(!ok_conv)
		{
			QMessageBox::information(NULL, "XTT Editor", "Incorrect row " + QString::number(index+1) + " height. Value \"" + l + "\" is not integer type. Error at line " + QString::number(DOMrh.lineNumber()) + ".\nHeight will be set as 10.", QMessageBox::Ok);
			rh = 10;
		}
		
		if(((int)rh) < 0)
		{
			QMessageBox::information(NULL, "XTT Editor", "Incorrect row " + QString::number(index+1) + " height. Value \"" + l + "\" can't be less than zero. Error at line " + QString::number(DOMrh.lineNumber()) + ".\nHeight will be set as 10.", QMessageBox::Ok);
			rh = 10;
		}
		
		fHeight[index] = rh;
		index++;
	
		// Wczytanie kolejengo rozmiaru dla wiersza
		DOMrh = DOMrh.nextSiblingElement("rh");
	}
	
	// "<localization"
	QDomElement DOMlocalization = root.firstChildElement("localization");
	if(!DOMlocalization.isNull())
	{
		QString sXPos = DOMlocalization.attribute("xpos", "#NAN#");
		QString sYPos = DOMlocalization.attribute("ypos", "#NAN#");
		QString sHHeight = DOMlocalization.attribute("header_height", "#NAN#");
		QString sLevel = DOMlocalization.attribute("level", "#NAN#");
		QString sParent = DOMlocalization.attribute("parent", "#NAN#");
		
		// Podanie nie jest wymagane, jezeli parametr nie jest podany to przyjmujemy wielkosc za 0
		// Usuwanie spacji z wczytanych wartosci
		for(;sXPos.contains(" ");)
			sXPos = sXPos.remove(sXPos.indexOf(" "), 1);
		for(;sYPos.contains(" ");)
			sYPos = sYPos.remove(sYPos.indexOf(" "), 1);
		for(;sHHeight.contains(" ");)
			sHHeight = sHHeight.remove(sHHeight.indexOf(" "), 1);
		for(;sLevel.contains(" ");)
			sLevel = sLevel.remove(sLevel.indexOf(" "), 1);
		for(;sParent.contains(" ");)
			sParent = sParent.remove(sParent.indexOf(" "), 1);
			
		// Parametry nie sa wymagane, jezeli nie zostana podane to zostana im
		// przypisane nastpujace wartosci domyslne
		int iXPos = 0;
		int iYPos = 0;
		int iHHeight = 20;
		int iLevel = 0;
		int iParent = -1;
			
		// Proby konwersji jezeli podano
		if((sXPos != "") && (sXPos != "#NAN#"))
		{
			bool ok;
			iXPos = sXPos.toInt(&ok);
			if(!ok)
				QMessageBox::information(NULL, "XTT Editor", "Incorrect value type. Value \"" + sXPos + "\" is not integer type. Error at line " + QString::number(DOMlocalization.lineNumber()) + ".", QMessageBox::Ok);
			fPoint.setX(iXPos);
		}
		if((sYPos != "") && (sYPos != "#NAN#"))
		{
			bool ok;
			iYPos = sYPos.toInt(&ok);
			if(!ok)
				QMessageBox::information(NULL, "XTT Editor", "Incorrect value type. Value \"" + sYPos + "\" is not integer type. Error at line " + QString::number(DOMlocalization.lineNumber()) + ".", QMessageBox::Ok);
			fPoint.setY(iYPos);
		}
		if((sHHeight != "") && (sHHeight != "#NAN#"))
		{
			bool ok;
			iHHeight = sHHeight.toInt(&ok);
			if(!ok)
				QMessageBox::information(NULL, "XTT Editor", "Incorrect value type. Value \"" + sHHeight + "\" is not integer type. Error at line " + QString::number(DOMlocalization.lineNumber()) + ".", QMessageBox::Ok);
			fHeaderHeight = (unsigned int)iHHeight;
		}
		if((sLevel != "") && (sLevel != "#NAN#"))
		{
			bool ok;
			iLevel = sLevel.toInt(&ok);
			if(!ok)
				QMessageBox::information(NULL, "XTT Editor", "Incorrect value type. Value \"" + sLevel + "\" is not integer type. Error at line " + QString::number(DOMlocalization.lineNumber()) + ".", QMessageBox::Ok);
			fLevel = iLevel;
		}
		if((sParent != "") && (sParent != "#NAN#"))
		{
			bool ok;
			iParent = sParent.toInt(&ok);
			if(!ok)
				QMessageBox::information(NULL, "XTT Editor", "Incorrect value type. Value \"" + sParent + "\" is not integer type. Error at line " + QString::number(DOMlocalization.lineNumber()) + ".", QMessageBox::Ok);
			fParent = iParent;
		}
	}
		
	// Po wczytaniu wszystkiego - weryfikacja wszystkich ustawien
	// Sprawdzanie czy zgadza sie ilosc kolumn w kazdym wierszu z suma wielkosci kontekstow
	int ctx_sum = ColCount();
	
	for(unsigned int r=0;r<fRowCount;r++)
	{
		if((unsigned int)ctx_sum != fRows[r]->Length())
		{
			QMessageBox::critical(NULL, "XTT Editor", "Incorrect col count in row " + QString::number(r+1) + ". Column count must be equal sum of all contexts size. Error at line: " + QString::number(root.lineNumber()) + ".", QMessageBox::Ok);
			return false;
		}
		
		// Jezeli wiersz jest odpowiednio dlugi to musimy przeanalizowac komorki
		for(unsigned int c=0;c<fRows[r]->Length();c++)
		{
			// Zapisanie wartosci jezeli komorka jest pusta
			QString curr_content = fRows[r]->Cell(c)->Content();
			
			if((!fRows[r]->Cell(c)->Analyze(fRows[r]->Cell(c)->Content())) && (Settings->xttShowErrorCellMessage()))
				QMessageBox::warning(NULL, "XTT Editor", fRows[r]->Cell(c)->CellError(), QMessageBox::Ok);
			fRows[r]->Cell(c)->setContent(curr_content);
		}
	}

     // Odswiezenie widoku
     delete fGTable;
     fGTable = new GTable(this);
     fGTable->ReCreate();
     fGTable->Update();
     fGTable->RePosition();

     return res;
}
// -----------------------------------------------------------------------------
