 /**
 *                                                                             *
 * @file XTT_Table.h                                                           *
 * Project name: HeKatE hqed                                                   *
 * $Id: XTT_Table.h,v 1.12 2008-03-03 22:04:34 kinio Exp $                      *
 * Author: Krzysztof Kaczor                                                    *
 * Date: 24.04.2007                                                            *
 *                                                                             *
 * Implementation by Krzysztof Kaczor <kinio4444@gmail.com>                    *
 *                                                                             *
 * Copyright (C) 2006-9 by the HeKatE Project                                  *
 *                                                                             *
 * HQEd has been develped within the HeKatE Project,                           *
 * see http://hekate.ia.agh.edu.pl                                             *
 *                                                                             *
 * This file is part of HQEd.                                                  *
 *                                                                             *
 * HQEd is free software: you can redistribute it and/or modify                *
 * it under the terms of the GNU General Public License as published by        *
 * the Free Software Foundation, either version 3 of the License, or           *
 * (at your option) any later version.                                         *
 *                                                                             *
 * HQEd is distributed in the hope that it will be useful,                     *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU General Public License for more details.                                *
 *                                                                             *
 * You should have received a copy of the GNU General Public License           *
 * along with HQEd.  If not, see <http://www.gnu.org/licenses/>.               *
 *                                                                             *
 * Comment: Plik zawiera definicje obiektu bedacego tabela XTT                 *
 *                                                                             *
 */

#ifndef XTT_TABLEH
#define XTT_TABLEH
// -----------------------------------------------------------------------------

#include <QtCore/QPoint>
#include <QtCore/QString>
#include <QWidget>
#include <QDomDocument>

#if QT_VERSION >= 0x040300
	#include <QXmlStreamWriter>
#endif

#include "XTT_Row.h"
#include "XTT_Attribute.h"
// -----------------------------------------------------------------------------

class GTable;
// -----------------------------------------------------------------------------

class XTT_Table
{
private:

     /// Tu jeszcze powinien byc vetkor atrybutow umieszczonych w tej tabeli

     /// Nastepna wartosc identyfikatora wiersza
     unsigned int* fNextRowId;

     /// Nastepna wartosc identyfikatora komorki
     unsigned int fNextCellId;

     /// Identyfikaotr tabeli
     unsigned int fTableID;

     /// Okresla typ tabeli 0-inital, 1-middle, 2-halt
     unsigned int fTableType;

     /// Tytul tabeli
     QString fTitle;

     /// Opis tabeli
     QString fDescription;

     /// Wielkosc poszczegolnych kontekstow
     unsigned int* fnContextSize;

     /// Wskazniki do atrybutow w poszczegolnych kontekstach
     XTT_Attribute** fContextAtributes;

     /// Ilosc wierszy tabeli
     unsigned int fRowCount;

     /// Tablica wierszy w tabeli
     XTT_Row** fRows;

     /// Zwiera abstrakcyjne identyfikatory kolumn
     unsigned int* fAbstractColID;

     /// Nastepne ID abstrakcyjne
     unsigned int fNextAbstractID;

     /// ------------------------ Polorzenie i wymiary --------------------------

     /// Obiekt rysujacy tabele
     GTable* fGTable;

     /// Wspolrzedne na Canvas
     QPoint fPoint;

     /// Dlugosc i szeroksoc tabeli
     unsigned int* fWidth;
     unsigned int* fHeight;

     /// Wysokosc naglowka
     unsigned int fHeaderHeight;

     /// Okresla na ktorym poziomie zagniezdzenia znaduje sie tabela
     unsigned int fLevel;

     /// Okresla kto jest bezposrednim rodzicem tabeli
     unsigned int fParent;

     /// Okresla czy tabela jest zmodyfikowana czy nie
     bool fChanged;

public:
      
     /// Konstruktor domyslny
     XTT_Table(void);

     /// Konstruktor ustawiajacy identyfikator tabeli i wskaznik do numerow
     /// id wierszy
     XTT_Table(unsigned int, unsigned int*);

     /// Destruktor
     ~XTT_Table(void);

     /// ----------------------- Metody obslugujace pliki -----------------------

     /// Wstawia wszystkie dane w formacie xttml 1.0
     QStringList* XTTML_1_0(void);
	
	#if QT_VERSION >= 0x040300
	/// Zwraca format zapisu zgodny z XTTML 1.0
	void XTTML_1_0(QXmlStreamWriter*);
	#endif
	
	/// Wstawia wszystkie dane w formacie xttml 2.0
     QStringList* XTTML_2_0(void);
	
	#if QT_VERSION >= 0x040300
	/// Zwraca format zapisu zgodny z XTTML 2.0
	void XTTML_2_0(QXmlStreamWriter*);
	#endif

     /// Wczytuje dane z pliku XTTML 1.0
     bool FromXTTML_1_0(QStringList*);
	
	/// Wczytuje dane z pliku XTTML 1.0
     bool FromXTTML_1_0(QDomElement&);
	
	/// Wczytuje dane z pliku XTTML 2.0
	bool FromXTTML_2_0(QStringList*) ;
	
	/// Wczytuje dane z pliku XTTML 2.0
	bool FromXTTML_2_0(QDomElement&);

     /// ----------------- Metody zwracajaca wartosci pol klasy -----------------

     /// Zwraca identyfikator tabeli
     unsigned int TableId(void){ return fTableID; }

     /// Zwraca typ tabeli
     unsigned int TableType(void){ return fTableType; }

     /// Zwraca tytul tabeli
     QString Title(void){ return fTitle; }

     /// Zwraca opis tabeli
     QString Description(void){ return fDescription; }

     /// Zwraca rozmiar wybranego kontekstu, jezeli podany index jest niepoprawyny
     /// to zwraca -1
     int ContextSize(unsigned int);

     /// Zwraca wskaznik do atrybutu w wybranej kolumnie, jezeli podany index jest
     /// niepoprawny to zwraca NULL
     XTT_Attribute* ContextAtribute(unsigned int);

     /// Zwraca ilosc wierszy tabeli
     unsigned int RowCount(void){ return fRowCount; }

     /// Zwraca wskaznik do wiersza, jezeli podany index jest niepoprawny to
     /// zwraca NULL
     XTT_Row* Row(unsigned int);

     /// Zwraca polorzenie tabeli
     QPoint Localization(void){return fPoint; }

     /// Zwraca szerokosc tabeli
     unsigned int Width(void);

     /// Zwraca wyokosc tabeli
     unsigned int Height(void);

     /// Zwraca szerokosc kolumny
     unsigned int ColWidth(unsigned int);

     /// Zwraca wyokosc wiersza
     unsigned int RowHeight(unsigned int);

     /// Zwraca poziom zagniezdzenia tabeli
     unsigned int Level(void){ return fLevel; }

     /// Zwraca ID rodzica tabeli
     unsigned int Parent(void){ return fParent; }

     /// Zwraca czy tabela zsotala zmodyfikowana
     bool Changed(void);

     /// Zwraca id kolumny
     int AbstractColID(unsigned int);

     /// Zwraca wysokosc naglowka
     unsigned int HeaderHeight(void){ return fHeaderHeight; }

     /// Zwraca obiet rysujacy tabele
     GTable* GraphicTable(void){ return fGTable; }

     /// ---------------- Metody ustawiajace wartosci pol klasy -----------------

     /// Usatwia tytul tabeli
     void setTitle(QString);

     /// Usatwia opis tabeli
     void setDescription(QString);

     /// Ustawia typ tabeli
     void setTableType(unsigned int);

     /// Pozwala zmienic atrybut przypisany do kolumny, jezeli podany index kolumny
     /// jest niepoprawny, albo wskaznik ma wartosc NULL to zwraca false inacze true
     bool setContextAtributes(unsigned int, XTT_Attribute*);

     /// Pozwala zmienic atrybut przypisany do kolumny, poprzez podanie indexu
     /// contextu i kolumny w kontekscie, jezeli podany index kolumny, lub kontekstu
     /// jest niepoprawny, albo wskaznik ma wartosc NULL to zwraca false inacze true
     bool setContextAtributes(unsigned int, unsigned int, XTT_Attribute*);

     /// Ustawia lokalizacje tabeli, jezeli podane wspolrzedne sa bledne to zwraca
     /// false inaczej true
     bool setLocalization(QPoint);

     /// Ustawia szerokosc tabeli
     bool setColWidth(unsigned int, unsigned int);

     /// Ustawia wysoksoc tabeli
     bool setRowHeight(unsigned int, unsigned int);

     /// Ustawia poziom zagniezdzenia tabeli
     void setLevel(unsigned int);

     /// Ustawia ID rodzica tabeli
     void setParent(unsigned int);

     /// Ustawia falge modyfikacji tabeli
     void setChanged(bool);

     /// Ustawia rozmiar kontekstu
     void setContextSize(unsigned int, unsigned int);

     /// Ustawia wysokosc naglowka
     void setHeaderHeight(unsigned int);

     /// -------------------------- Metody operacyjne ---------------------------

     /// Zwraca numer kontekstu kolumny o podanym indexie
     int ColContext(int);

     /// Zwraca index kolumny o podanym kotekscie i indexie w tym kontekscie
     int ColIndex(unsigned int, unsigned int);

     /// Zwraca ilosc kolumn tabeli
     int ColCount(void);

     /// Zamienia ze soba dwie kolumny z tego samego kontekstu
     bool Exchange(unsigned int, unsigned int);

     /// Dodaje nowy wiersz do tabeli
     void AddRow(void);

     /// Usuwa wybrany wiersz tabeli, metoda rowniez usuwa polaczenie z tym wierszem
     /// zwiazane, o ile oczywiscie takowe istnieje. Jezeli podany index wiersza jest
     /// niepoprawny zwraca false, inaczej true
     bool DeleteRow(unsigned int);

     /// Wstawia nowa kolumne w wybrane miejsce
     bool InsertCol(unsigned int);

     /// Usuwa wybrana kolumne, jezeli podany index jest niepoprawny to zwraca
     /// false inaczej true
     bool DeleteCol(unsigned int);

     /// Wstawia kolumne o nowym kotekscie, jaki jeszcze nie istnieje
     bool InsertNewContext(unsigned int);

     /// Zmienia pozycje kolumny
     bool Move(unsigned int, unsigned int);

     /// Ustawia kontekst dla calej kolumny
     void setColContext(unsigned int, unsigned int);

     /// Okresla ktory kontekst jest ostatni nie pusty
     int LastNotEmptyContext(void);

     /// Zwraca index elementu o danym id kolumny
     int IndexOfColId(unsigned int);

     /// Zwraca index atrubutu wystepujacego w tej tabeli o podanym ID
     int IndexOfAttributeID(unsigned int);

     /// Zamienia ze soba dwa wiersze
     bool ExchangeRows(unsigned int, unsigned int, bool _keepConnections = true);
	
	/// Wstawia nowy wiersz w dane miejsce
	bool insertRow(int);
	
	/// Zwraca index pierwszego wiersza do ktorego jest podlaczone jakies polaczenie
	int firstConnectionRowIndex(void);
	
	/// Zwraca liste polaczen przychodzacych do tej tabeli
	QList<XTT_Connections*>* inConnectionsList(void);
	
	/// Zwraca liste polaczen wychodzacych z tej tabeli
	QList<XTT_Connections*>* outConnectionsList(void);
};
// -----------------------------------------------------------------------------
#endif 
