 /**
 *                                                                             *
 * @file XTT_TableList.cpp                                                     *
 * Project name: HeKatE hqed                                                   *
 * $Id: XTT_TableList.cpp,v 1.12 2008-03-03 22:04:34 kinio Exp $                *
 * Author: Krzysztof Kaczor                                                    *
 * Date: 04.05.2007                                                            *
 *                                                                             *
 * Implementation by Krzysztof Kaczor <kinio4444@gmail.com>                    *
 *                                                                             *
 * Copyright (C) 2006-9 by the HeKatE Project                                  *
 *                                                                             *
 * HQEd has been develped within the HeKatE Project,                           *
 * see http://hekate.ia.agh.edu.pl                                             *
 *                                                                             *
 * This file is part of HQEd.                                                  *
 *                                                                             *
 * HQEd is free software: you can redistribute it and/or modify                *
 * it under the terms of the GNU General Public License as published by        *
 * the Free Software Foundation, either version 3 of the License, or           *
 * (at your option) any later version.                                         *
 *                                                                             *
 * HQEd is distributed in the hope that it will be useful,                     *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU General Public License for more details.                                *
 *                                                                             *
 * You should have received a copy of the GNU General Public License           *
 * along with HQEd.  If not, see <http://www.gnu.org/licenses/>.               *
 *                                                                             *
 * Comment: Zwiera definicje metod obiekt bedacego pojemnikiem na tabele       *
 *                                                                             *
 */

#include <QMessageBox>
#include <QtGui>
#include <QDialog>
#include <QObject>
#include <QString>
#include <QTreeWidgetItem>

#include "../../V/XTT/GTable.h"
#include "../../C/MainWin_ui.h"
#include "../../C/Settings_ui.h"
#include "../../C/XTT/MyGraphicsScene.h"
#include "XTT_Table.h"
#include "XTT_TableList.h"
//---------------------------------------------------------------------------

XTT_TableList* TableList;
//---------------------------------------------------------------------------

/**
 * Konstruktor domyslny obiektu
 * @returns Brak zwracanych wartosci
*/
XTT_TableList::XTT_TableList(void)
{
     fTables = NULL;
     fCount = 0;
     fNextTableID = 0;
     fNextRowID = 0;
	fChanged = false;
}
//---------------------------------------------------------------------------

/**
 * Destruktor obiektu
 * @returns Brak zwracanych wartosci
*/
XTT_TableList::~XTT_TableList(void)
{
     if(!fCount)
          return;

     for(unsigned int i=0;i<fCount;i++)
          delete fTables[i];
     delete [] fTables;
}
//---------------------------------------------------------------------------

/**
 * Metoda klasy oprozniajaca pojemnik
 * @returns Brak zwracanych wartoci
*/
void XTT_TableList::Flush(void)
{
     if(fCount == 0)
          return;

     for(unsigned int i=0;i<fCount;i++)
          delete fTables[i];
     delete [] fTables;

     fTables = NULL;
     fNextTableID = 0;
     fNextRowID = 0;
     fCount = 0;
	
	fChanged = true;
}
//---------------------------------------------------------------------------

/**
 * Metoda klasy zwracajaca wskaznik do wybranej tabeli
 * @param unsigned int _index - Index wybranej tabeli
 * @returns Wskaznik do wybranej tabeli, jezeli index jest niepoprawny to 
 * zwraca NULL
*/
XTT_Table* XTT_TableList::Table(unsigned int _index)
{
     if((((int)_index) < 0) || (_index >= fCount))
          return NULL;

     return fTables[_index];
}
//---------------------------------------------------------------------------

/**
 * Metoda zwracajaca wartosc okreslajaca czy dany obiekt zostal zmodyfikowany
 * @returns true jezeli zostal zmodyfikowany, inaczej false
 */
bool XTT_TableList::Changed(void)
{
	for(unsigned int i=0;i<fCount;i++)
		if(fTables[i]->Changed())
			return true;
			
	return fChanged;
}
//---------------------------------------------------------------------------

/**
 * Metoda ustawiajaca wartosc okreslajaca czy dany obiekt byl zmodyfikowany
 * @param bool _c - Nowa wartosc
 * @returns Brak zwracanych wartoci
 */
void XTT_TableList::setChanged(bool _c)
{
	for(unsigned int i=0;i<fCount;i++)
		fTables[i]->setChanged(_c);
			
	fChanged = _c;
}
//---------------------------------------------------------------------------

/**
 * Metoda klasy dodajaca nowa tabele do pojemnika
 * @returns Brak zwracanych wartoci
*/
void XTT_TableList::Add(void)
{
     XTT_Table** tmp = new XTT_Table*[fCount+1];

     // Kopiowanie zawartosci
     for(unsigned int i=0;i<fCount;i++)
          tmp[i] = fTables[i];

     // Usuniecie starej tablicy wskaznikow
     if(fCount)
          delete [] fTables;

     // Zapisanie nowej lokalizacji
     fTables = tmp;

     // Utworenie nowego elementu
     fTables[fCount] = new XTT_Table(++fNextTableID, &fNextRowID);
	
     // Powiekszenie pojemnika
     fCount++;
	
	// Zarejestrowanie ze sie zmienilo
	fChanged = true;
}
//---------------------------------------------------------------------------

/**
 * Metoda klasy usuwajaca wybrana tabele z pojemnika
 * @param unsigned int _index - Index wybranej tabeli
 * @returns true jezeli inex jest ok, inaczej false
*/
bool XTT_TableList::Delete(unsigned int _index)
{
     if((((int)_index) < 0) || (_index >= fCount))
          return false;

     XTT_Table** tmp = new XTT_Table*[fCount-1];

     // Kopiowanie zawartosci
     int tmp_i = 0;
     for(unsigned int i=0;i<fCount;i++)
          if(i != _index)
               tmp[tmp_i++] = fTables[i];

     // Usuniecie starej tablicy wskaznikow
     if(fCount)
     {
          delete fTables[_index];
          delete [] fTables;
     }

     // Zapisanie nowej lokalizacji
     fTables = tmp;

     // Pomniejszenie pojemnika
     fCount--;
	
	// Zarejestrowanie ze sie zmienilo
	fChanged = true;

     return true;
}
//---------------------------------------------------------------------------

/**
 * Metoda klasy tworzaca nowa nazwe tabeli na podstawie podanej nazwy bazowej
 * @param QString baseName - nazwa bazowa nazwy tabeli
 * @returns nazwe tabeli z dodanym indexem do podanej nazwy bazowej
*/
QString XTT_TableList::newBasicTableName(QString _baseName)
{
	// Ustawianie nowej nazwy tabeli
	int tableNamePostfix = 0;
	QString autoTableName;
	
	do
	{
		tableNamePostfix++;
		autoTableName = _baseName + QString::number(tableNamePostfix);
	}
	while(IndexOfTitle(autoTableName) > -1);
	
	return autoTableName;
}
//---------------------------------------------------------------------------

/**
 * Metoda klasy zwracajaca index tabeli o podanym tytule
 * @param QString _title - Tytul wybranej tabeli
 * @returns Index w pojemniku, jezeli nie znajdzie to -1
*/
int XTT_TableList::IndexOfTitle(QString _title)
{
     for(unsigned int i=0;i<fCount;i++)
          if(fTables[i]->Title() == _title)
               return i;

     return -1;
}
//---------------------------------------------------------------------------

/**
 * Metoda klasy zwracajaca wskaznik do tabeli o podanym tytule
 * @param QString _title - Tytul wybranej tabeli
 * @returns Wskaznik do tabeli, jezeli nie znajdzie to NULL
*/
XTT_Table* XTT_TableList::_IndexOfTitle(QString _title)
{
     int index = IndexOfTitle(_title);
     if(index == -1)
          return NULL;

     return fTables[index];
}
//---------------------------------------------------------------------------

/**
 * Metoda klasy zwracajaca index tabeli o podanym ID
 * @param unsigned int _id - ID wybranej tabeli
 * @returns Index w pojemniku, jezeli nie znajdzie to -1
*/
int XTT_TableList::IndexOfID(unsigned int _id)
{
     for(unsigned int i=0;i<fCount;i++)
          if(fTables[i]->TableId() == _id)
               return i;

     return -1;
}
//---------------------------------------------------------------------------

/**
 * Metoda klasy zwracajaca wskaznik do tabeli o podanym ID
 * @param unsigned int _id - ID wybranej tabeli
 * @returns Wskaznik do tabeli, jezeli nie znajdzie to NULL
*/
XTT_Table* XTT_TableList::_IndexOfID(unsigned int _id)
{
     int index = IndexOfID(_id);
     if(index == -1)
          return NULL;

     return fTables[index];
}
//---------------------------------------------------------------------------

/**
 * Metoda klasy, zwracajaca czy pojemnik jest pusty
 * @returns true jezeli pojemnik jest pusty, false gdy jakas tabela jest juz w 
 * pojemniku
*/
bool XTT_TableList::IsEmpty(void)
{
     return !fCount;
}
// ---------------------------------------------------------------------------

/**
 * Metoda klasy XTT_TableList, zwracajaca index pierwszej tabeli o podanym typie
 * @param unsigned int _type - Typ tabeli ktory chcemy wyszukac
 * @returns Index tabeli, jezeli nie ma takiej to -1
*/
int XTT_TableList::IndexOfType(unsigned int _type)
{
     for(unsigned int i=0;i<fCount;i++)
          if(fTables[i]->TableType() == _type)
               return i;

     return -1;
}
// ---------------------------------------------------------------------------

/**
 * Metoda klasy XTT_TableList, zwracajaca tytuly wszystkich tabeli
 * @returns Zwraca tytuly wszystkich tabeli jako lista stringow
*/
QStringList XTT_TableList::TablesTitles(void)
{
     QStringList res;
     res.clear();

     for(unsigned int i=0;i<fCount;i++)
          res << fTables[i]->Title();

     return res;
}
// ---------------------------------------------------------------------------

/**
 * Metoda odswiezajaca dana tabele
 * @param int _t - Index tabeli
 * @returns Brak zwracanych wartoci
*/
void XTT_TableList::Refresh(int _t)
{
     if((_t < 0) || (_t >= (int)fCount))
          return;
      
     fTables[_t]->GraphicTable()->ReCreate();
     fTables[_t]->GraphicTable()->Update();
     fTables[_t]->GraphicTable()->RePosition();

     ConnectionsList->RepaintConnectionRelatedWithTable(fTables[_t]->TableId());
     
	MainWin->xtt_scene->update();
     //MainWin->scaleView(1.001);
     //MainWin->scaleView(1./1.001);
}
// ---------------------------------------------------------------------------

/**
 * Metoda odswiezajaca wszystkie tabele
 * @returns Brak zwracanych wartoci
*/
void XTT_TableList::RefreshAll(void)
{
     for(unsigned int i=0;i<fCount;i++)
          Refresh(i);
}
// ---------------------------------------------------------------------------

/**
 * Metoda analizujaca wszystkie komorki
 * @returns Brak zwracanych wartoci
*/
void XTT_TableList::AnalyzeAllCells(void)
{
	for(unsigned int t=0;t<fCount;t++)
	{
		for(unsigned int r=0;r<Table(t)->RowCount();r++)
		{
			for(unsigned int c=0;c<Table(t)->Row(r)->Length();c++)
			{
				QString cnt = Table(t)->Row(r)->Cell(c)->Content();
				if((Settings->xttShowErrorCellMessage()) && (!Table(t)->Row(r)->Cell(c)->Analyze(cnt)))
					QMessageBox::critical(NULL, "XTT Editor", Table(t)->Row(r)->Cell(c)->CellError(), QMessageBox::Ok);
				Table(t)->Row(r)->Cell(c)->setContent(cnt);
			}
		}
	}
}
// ---------------------------------------------------------------------------

/**
 * Metoda zapisujaca wszystkie dane w formacie xttml 1.0
 * @returns Zwraca liste stringow w tym formacie
*/
QStringList* XTT_TableList::XTTML_1_0(void)
{
     QStringList* res = new QStringList;
     res->clear();

     *res << "<xtt_list_table next_table_id=\"" + QString::number(fNextTableID) + "\" next_row_id=\"" +  QString::number(fNextRowID)  + "\">";

     // Zapis tabel
     for(unsigned int a=0;a<fCount;a++)
     {
          QStringList* al = fTables[a]->XTTML_1_0();
          for(int l=0;l<al->size();l++)
               *res << "\t" + al->at(l);
          delete al;
     }
     
     *res << "</xtt_list_table>";

     return res;
}
// -----------------------------------------------------------------------------

#if QT_VERSION >= 0x040300
/**
 * Metoda zapisujaca wszystkie dane w formacie xttml 1.0
 * @param QXmlStreamWriter* _xmldoc - wskanik do dokumentu
 * @returns brak zwracanych wartosci
*/
void XTT_TableList::XTTML_1_0(QXmlStreamWriter* _xmldoc)
{
	_xmldoc->writeStartElement("xtt_list_table");
	_xmldoc->writeAttribute("next_table_id", QString::number(fNextTableID));
	_xmldoc->writeAttribute("next_row_id", QString::number(fNextRowID));
	
     // Zapis tabel
     for(unsigned int a=0;a<fCount;a++)
          fTables[a]->XTTML_1_0(_xmldoc);
     
     _xmldoc->writeEndElement();
}
#endif
// -----------------------------------------------------------------------------

/**
 * Metoda zapisujaca wszystkie dane w formacie xttml 2.0
 * @returns Zwraca liste stringow w tym formacie
*/
QStringList* XTT_TableList::XTTML_2_0(void)
{
     QStringList* res = new QStringList;
     res->clear();

     *res << "<xtt_list_table>";

     // Zapis tabel
     for(unsigned int a=0;a<fCount;a++)
     {
          QStringList* al = fTables[a]->XTTML_2_0();
          for(int l=0;l<al->size();l++)
               *res << "\t" + al->at(l);
          delete al;
     }
     
     *res << "</xtt_list_table>";

     return res;
}
// -----------------------------------------------------------------------------

#if QT_VERSION >= 0x040300
/**
 * Metoda zapisujaca wszystkie dane w formacie xttml 2.0
 * @param QXmlStreamWriter* _xmldoc - wskanik do dokumentu
 * @returns brak zwracanych wartosci
*/
void XTT_TableList::XTTML_2_0(QXmlStreamWriter* _xmldoc)
{
	_xmldoc->writeStartElement("xtt_list_table");

     // Zapis tabel
     for(unsigned int a=0;a<fCount;a++)
          fTables[a]->XTTML_2_0(_xmldoc);
     
     _xmldoc->writeEndElement();
}
#endif
// -----------------------------------------------------------------------------

/**
 * Metoda wczytujaca dane o strybucie z pliku w formacie XTTML 1.0
 * @param QStringList* list - Wskanik do sekcji dotyczacej tego atrybutu
 * @returns true jezeli dane sa ok, ianczej false
*/
bool XTT_TableList::FromXTTML_1_0(QStringList& list)
{
     bool res = true;
     
     // Wyczyszczenie pojemnika
     Flush();

     // Usuwanie spacji i tabulatorow wierszach
     for(int i=0;i<list.size();i++)
     {
          QString line = list.at(i);
          while(line.contains("\t"))
          {
               int index = line.indexOf("\t");
               line = line.remove(index, 1);
          }
          list.replace(i, line);
     }

     // Zakladamy ze pierwszy wiersz jest naglowkiem
     if(!list.at(0).contains("<xtt_list_table", Qt::CaseInsensitive))
          return false;
     if(!list.contains("</xtt_list_table>", Qt::CaseInsensitive))
          return false;

     // Reszta zakladamy ze jest ok

     // Spis tagow
     QStringList tags;
     tags.clear();
     tags << "<xtt_list_table" << "<xtt_table" << "</xtt_table>";

     // Sluza do odpowiedniego kopiowania lini dla atrybutu
     bool copy = false;
     QStringList buf;
     buf.clear();

     for(int line=0;line<list.size();line++)
     {
          //QMessageBox::warning(NULL, "tworzenie tabeli", list.at(line));
          // Sprawdzanie wystapienia tagu
          int action = -1;
          for(int tag=0;tag<tags.size();tag++)
               if(list.at(line).contains(tags.at(tag), Qt::CaseInsensitive))
                    action = tag;

          // Kopiowanie lini danych atrybutu
          if(copy)
               buf.append(list.at(line));
               
          if(action == -1)
               continue;

          QString end_tag = tags.at(action);
          end_tag = end_tag.insert(1, "/");
          QString l = list.at(line);
          l = l.remove(l.indexOf(tags.at(action), Qt::CaseInsensitive), tags.at(action).length());
          l = l.mid(0, l.indexOf(end_tag, Qt::CaseInsensitive));

          // Wczytywanie parametrow w zaleznosci od wykrytego tagu
          if(action == 0)
          {
               fNextTableID = (unsigned int)XML_ReadValue(list.at(line), "next_table_id").toInt();
               fNextRowID = (unsigned int)XML_ReadValue(list.at(line), "next_row_id").toInt();
          }
          if(action == 1)
          {
               // Jezeli tag rozpoczynajacy dane atrybutu, to uruchamiamy kopiowanie
               copy = true;
               buf.append(list.at(line));
          }
          if(action == 2)
          {
               // Kiedy linia konczaca linie danych atrybutu
               // Dodajemy nowy atrybut
               Add();

               /*QString msg = "";
               for(int i=0;i<buf.size();i++)
                    msg = msg + buf.at(i) + "\n";
               QMessageBox::warning(NULL, "tworzenie tabeli", QString::number(fCount));*/

               // Wczytujemy dane z pliku
               res = res && fTables[fCount-1]->FromXTTML_1_0(&buf);

               // Konczymy kopiowanie
               copy = false;
               
               // Czyscimy zawartosc danych atrybutu
               buf.clear();
          }
     }

     //QStringList* h = XTTML_1_0();
     //for(int i=0;i<h->size();i++)
     //QMessageBox::warning(NULL, "Application", "Wczytano liste tabel");

     return res;
}
// -----------------------------------------------------------------------------

/**
 * Metoda wczytujaca dane o strybucie z pliku w formacie XTTML 1.0
 * @param QDomElement& root - Referencja do sekcji dotyczacej tego atrybutu
 * @returns true jezeli dane sa ok, ianczej false
*/
bool XTT_TableList::FromXTTML_1_0(QDomElement& root)
{
	if(root.tagName().toLower() != "xtt_list_table")
		return false;
		
     bool res = true;
     
     // Wyczyszczenie pojemnika
     Flush();
	
	fNextTableID = (unsigned int)root.attribute("next_table_id", "0").toInt();
     fNextRowID = (unsigned int)root.attribute("next_row_id", "0").toInt();

	QDomElement DOMtable = root.firstChildElement("xtt_table");
     while(!DOMtable.isNull())
     {
		// Kiedy linia konczaca linie danych atrybutu
		// Dodajemy nowy atrybut
		Add();

		// Wczytujemy dane z pliku
		res = res && fTables[fCount-1]->FromXTTML_1_0(DOMtable);
		
		// Wczytanie kolejnej tabeli
		DOMtable = DOMtable.nextSiblingElement("xtt_table");
     }

     return res;
}
// -----------------------------------------------------------------------------

/**
 * Metoda wczytujaca dane o strybucie z pliku w formacie XTTML 2.0
 * @param QStringList* list - Wskanik do sekcji dotyczacej tego atrybutu
 * @returns true jezeli dane sa ok, ianczej false
*/
bool XTT_TableList::FromXTTML_2_0(QStringList& list)
{
     bool res = true;
     
     // Wyczyszczenie pojemnika
     Flush();

     // Usuwanie spacji i tabulatorow wierszach
     for(int i=0;i<list.size();i++)
     {
          QString line = list.at(i);
          while(line.contains("\t"))
          {
               int index = line.indexOf("\t");
               line = line.remove(index, 1);
          }
          list.replace(i, line);
     }

     // Zakladamy ze pierwszy wiersz jest naglowkiem
     if(!list.at(0).contains("<xtt_list_table>", Qt::CaseInsensitive))
          return false;
     if(!list.contains("</xtt_list_table>", Qt::CaseInsensitive))
          return false;

     // Reszta zakladamy ze jest ok

     // Spis tagow
     QStringList tags;
     tags.clear();
     tags << "<xtt_list_table>" << "<xtt_table" << "</xtt_table>";

     // Sluza do odpowiedniego kopiowania lini dla atrybutu
     bool copy = false;
     QStringList buf;
     buf.clear();

     for(int line=0;line<list.size();line++)
     {
          //QMessageBox::warning(NULL, "tworzenie tabeli", list.at(line));
          // Sprawdzanie wystapienia tagu
          int action = -1;
          for(int tag=0;tag<tags.size();tag++)
               if(list.at(line).contains(tags.at(tag), Qt::CaseInsensitive))
                    action = tag;

          // Kopiowanie lini danych atrybutu
          if(copy)
               buf.append(list.at(line));
               
          if(action == -1)
               continue;

          QString end_tag = tags.at(action);
          end_tag = end_tag.insert(1, "/");
          QString l = list.at(line);
          l = l.remove(l.indexOf(tags.at(action), Qt::CaseInsensitive), tags.at(action).length());
          l = l.mid(0, l.indexOf(end_tag, Qt::CaseInsensitive));

          // Wczytywanie parametrow w zaleznosci od wykrytego tagu
          if(action == 1)
          {
               // Jezeli tag rozpoczynajacy dane atrybutu, to uruchamiamy kopiowanie
               copy = true;
               buf.append(list.at(line));
          }
          if(action == 2)
          {
               // Kiedy linia konczaca linie danych atrybutu
               // Dodajemy nowy atrybut
               Add();

               /*QString msg = "";
               for(int i=0;i<buf.size();i++)
                    msg = msg + buf.at(i) + "\n";
               QMessageBox::warning(NULL, "tworzenie tabeli", QString::number(fCount));*/

               // Wczytujemy dane z pliku
               res = res && fTables[fCount-1]->FromXTTML_2_0(&buf);

               // Konczymy kopiowanie
               copy = false;
               
               // Czyscimy zawartosc danych atrybutu
               buf.clear();
			
			// Jezeli blad to nie czytamy dlaej
			if(!res)
				return false;
          }
     }

     //QStringList* h = XTTML_1_0();
     //for(int i=0;i<h->size();i++)
     //QMessageBox::warning(NULL, "Application", "Wczytano liste tabel");

     return res;
}
// -----------------------------------------------------------------------------

/**
 * Metoda wczytujaca dane o strybucie z pliku w formacie XTTML 2.0
 * @param QDomElement& root - Referencja do sekcji dotyczacej tego atrybutu
 * @returns true jezeli dane sa ok, ianczej false
*/
bool XTT_TableList::FromXTTML_2_0(QDomElement& root)
{
	if(root.tagName().toLower() != "xtt_list_table")
		return false;

     bool res = true;
     
     // Wyczyszczenie pojemnika
     Flush();

     // Reszta zakladamy ze jest ok

	QDomElement child = root.firstChildElement("xtt_table");
     while(!child.isNull())
     {
		// Kiedy linia konczaca linie danych atrybutu
		// Dodajemy nowy atrybut
		Add();

		// Wczytujemy dane z pliku
		res = res && fTables[fCount-1]->FromXTTML_2_0(child);
		
		// Jezeli blad to nie czytamy dlaej
		if(!res)
			return false;
				
		// Wczytujemy kolejna tabele
		child = child.nextSiblingElement("xtt_table");
     }

     return res;
}
// -----------------------------------------------------------------------------
