 /**
 *                                                                             *
 * @file XTT_TableList.h                                                       *
 * Project name: HeKatE hqed                                                   *
 * $Id: XTT_TableList.h,v 1.12 2008-03-03 22:04:34 kinio Exp $                  *
 * Author: Krzysztof Kaczor                                                    *
 * Date: 04.05.2007                                                            *
 *                                                                             *
 * Implementation by Krzysztof Kaczor <kinio4444@gmail.com>                    *
 *                                                                             *
 * Copyright (C) 2006-9 by the HeKatE Project                                  *
 *                                                                             *
 * HQEd has been develped within the HeKatE Project,                           *
 * see http://hekate.ia.agh.edu.pl                                             *
 *                                                                             *
 * This file is part of HQEd.                                                  *
 *                                                                             *
 * HQEd is free software: you can redistribute it and/or modify                *
 * it under the terms of the GNU General Public License as published by        *
 * the Free Software Foundation, either version 3 of the License, or           *
 * (at your option) any later version.                                         *
 *                                                                             *
 * HQEd is distributed in the hope that it will be useful,                     *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU General Public License for more details.                                *
 *                                                                             *
 * You should have received a copy of the GNU General Public License           *
 * along with HQEd.  If not, see <http://www.gnu.org/licenses/>.               *
 *                                                                             *
 * Comment: Zawiera definicje ciala klasy bedacej pojemnikiem na tabele        *
 *                                                                             *
 */


#ifndef XTT_TableListH
#define XTT_TableListH
//---------------------------------------------------------------------------

#include <QDomDocument>

#if QT_VERSION >= 0x040300
	#include <QXmlStreamWriter>
#endif
//---------------------------------------------------------------------------

class XTT_Table;
class QTreeWidgetItem;
//---------------------------------------------------------------------------

class XTT_TableList
{
private:

     /// Tablica wskaznkow do tabeli
     XTT_Table** fTables;

     /// Wielkosc zbiornika
     unsigned int fCount;

     /// Nastepne id tabeli
     unsigned int fNextTableID;

     /// Nastepne id wiersza
     unsigned int fNextRowID;
	
	/// Okresla czy dany obiekt jest zmodyfikowany
	bool fChanged;

     /// Metoda prywatna wspomagajaca tworzenie drzewa nazw tabel masek
     void AddChilds(QTreeWidgetItem*, unsigned int);

public:

     /// Konstruktor
     XTT_TableList(void);

     /// Destruktor
     ~XTT_TableList(void);

     /// ----------------------- Metody obslugujace pliki -----------------------

     /// Wstawia wszystkie dane w formacie xttml 1.0
     QStringList* XTTML_1_0(void);
	
	#if QT_VERSION >= 0x040300
	/// Zwraca format zapisu zgodny z XTTML 1.0
	void XTTML_1_0(QXmlStreamWriter*);
	#endif
	
	/// Wstawia wszystkie dane w formacie xttml 2.0
     QStringList* XTTML_2_0(void);
	
	#if QT_VERSION >= 0x040300
	/// Zwraca format zapisu zgodny z XTTML 2.0
	void XTTML_2_0(QXmlStreamWriter*);
	#endif

     /// Wczytuje dane z pliku XTTML 1.0
     bool FromXTTML_1_0(QStringList&);
	
	/// Wczytuje dane z pliku XTTML 1.0
     bool FromXTTML_1_0(QDomElement&);
	
	/// Wczytuje dane z pliku XTTML 2.0
     bool FromXTTML_2_0(QStringList&);
	
	/// Wczytuje dane z pliku XTTML 2.0
	bool FromXTTML_2_0(QDomElement&);


     /// ----------------- Metody zwracajaca wartosci pol klasy -----------------

     /// Zwraca nastepne id tabeli
     unsigned int NextTableID(void){ return fNextTableID; }

     /// Zwraca nastepne id wiersza
     unsigned int NextRowID(void){ return fNextRowID; }

     //// Zwraca rozmiar pojemnika
     unsigned int Count(void){ return fCount; }

     /// Zwraca wskaznik do tabeli
     XTT_Table* Table(unsigned int);
	
	/// Zwraca czy dany obiekt byl zmodyfikowany
	bool Changed(void);

     /// -------------------------- Metody operacyjne ---------------------------
	
	/// Usawia status modyfiakcji obiektu
	void setChanged(bool);

     /// Dodaje nowa tabele
     void Add(void);

     /// Usuwa wybrana tabele
     bool Delete(unsigned int);
	
	/// Tworzy nowa nazwe tabeli przy podnaej podstawie nazwy
	QString newBasicTableName(QString _baseName = "Table");

     /// Oproznia bufor
     void Flush(void);

     /// Zwraca index tabeli o podanym naglowku
     int IndexOfTitle(QString);

     /// Zwraca index tabeli o podanym ID
     int IndexOfID(unsigned int);

     /// Zwraca wskaznik tabeli o podanym naglowku
     XTT_Table* _IndexOfTitle(QString);

     /// Zwraca wskaznik tabeli o podanym ID
     XTT_Table* _IndexOfID(unsigned int);

     /// Zwraca index pierwszej tabeli o podanym typie
     int IndexOfType(unsigned int);

     /// Zwraca liste wszystkich naglowkow tabel
     QStringList TablesTitles(void);

     /// Zwraca czy pojenik jest pusty
     bool IsEmpty(void);

     /// Odswieza dana tabele
     void Refresh(int);

     /// Odswoerza widok wszystkich tabel
     void RefreshAll(void);
	
	/// Metoda analizujaca wszystkie komorki w projekcie
	void AnalyzeAllCells(void);
};
//---------------------------------------------------------------------------

extern XTT_TableList* TableList;
//---------------------------------------------------------------------------
#endif
