 /**
 *                                                                             *
 * @file gARDdependency.h                                                      *
 * Project name: HeKatE hqed                                                   *
 * $Id: gARDdependency.h,v 1.7 2008-03-03 22:04:34 kinio Exp $                 *
 * Author: Krzysztof Kaczor                                                    *
 * Date: 21.05.2007                                                            *
 *                                                                             *
 * Implementation by Krzysztof Kaczor <kinio4444@gmail.com>                    *
 *                                                                             *
 * Copyright (C) 2006-9 by the HeKatE Project                                  *
 *                                                                             *
 * HQEd has been develped within the HeKatE Project,                           *
 * see http://hekate.ia.agh.edu.pl                                             *
 *                                                                             *
 * This file is part of HQEd.                                                  *
 *                                                                             *
 * HQEd is free software: you can redistribute it and/or modify                *
 * it under the terms of the GNU General Public License as published by        *
 * the Free Software Foundation, either version 3 of the License, or           *
 * (at your option) any later version.                                         *
 *                                                                             *
 * HQEd is distributed in the hope that it will be useful,                     *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU General Public License for more details.                                *
 *                                                                             *
 * You should have received a copy of the GNU General Public License           *
 * along with HQEd.  If not, see <http://www.gnu.org/licenses/>.               *
 *                                                                             *
 * Comment: Plik zawierajacy definicje obiektu symbolizujacego                 *
 * zaleznosc pomiedzy wlasnosciami na diagramie ARD                            *
 *                                                                             *
 */

#ifndef GARDDEPENDENCYH
#define GARDDEPENDENCYH
//---------------------------------------------------------------------------

#include <QGraphicsItem>
#include <QPainter>
#include <QGraphicsSceneMouseEvent>
#include <QStyleOptionGraphicsItem>
//---------------------------------------------------------------------------

class gARDwire;
class gARDlevel;
class ARD_Dependency;
//---------------------------------------------------------------------------

class gARDdependency
{
private:

	/// Identyfikator polaczenia
	unsigned int fID;

     /// Okresla ile jest aktualnie wymaganych elemntow lini
     unsigned int fEssentialWiresQuantity;

     /// Tablica wskaznikow na polaczenia miedzy tabelami
     gARDwire* fWire[5];

     /// Obiekt polaczenia, ktore symbolizuje
     ARD_Dependency* fParent;
	
	/// Rodzica graficznej reprezentacji poziomu
	gARDlevel* fGparent;

     /// Pola wlasciwoasci lini

     /// Okresla stosunek podzialu pierszewj odleglosci poziomej przy 3 czesciowym polaczeniu
     float line_1_pos_3p;
	
	/// Okresla stosunek podzialu pierszewj odleglosci poziomej przy 5 czesciowym polaczeniu
     float line_1_pos_5p;
	
	/// Okresla stosunek podzialu pierszewj odleglosci pionowej przy 5 czesciowym polaczeniu
     float line_2_pos;
	
	/// Okresla stosunek podzialu drugiej odleglosci poziomej przy 5 czesciowym polaczeniu
     float line_5_pos;
	
	/// Punkt startowy i koncowy polaczenia
	QPointF startP, endP;

public:

     /// Konstruktor
     gARDdependency(unsigned int, ARD_Dependency*, gARDlevel*);

     /// Destruktor
     ~gARDdependency(void);
	
	/// Zwraca identyfikator polaczenia
	unsigned int id(void);
	
     /// Zwraca wskaznik do rodzica
     ARD_Dependency* parent(void);
	
	 /// Zwraca wskaznik do rodzica graficznego
     gARDlevel* gParent(void);

     /// Metoda rysujace placzenia
     void repaint(void);

     /// Metoda ustawiajaca wszystkie elemnty jako wybrane
     void selectAll(void);
     void clearSelection(void);

     /// Metoda zwracajaca wskazniki do lini
     gARDwire* wires(int);

     /// Metoda wykonywana w momecie zmiany pozycji elementu
     void posChange(int);
	
	/// Zwraca obszar zajmowany przez polaczenie
	QRectF area(void);
	
	/// Przesuwa elementy o zadany wektor
	void moveBy(float, float);
};
//---------------------------------------------------------------------------
#endif
