 /**
 *                                                                             *
 * @file gARDexpander.cpp                                                      *
 * Project name: HeKatE hqed                                                   *
 * $Id: gARDexpander.cpp,v 1.7 2008-03-03 22:04:34 kinio Exp $                 *
 * Author: Krzysztof Kaczor                                                    *
 * Date: 7.01.2008                                                             *
 *                                                                             *
 * Implementation by Krzysztof Kaczor <kinio4444@gmail.com>                    *
 *                                                                             *
 * Copyright (C) 2006-9 by the HeKatE Project                                  *
 *                                                                             *
 * HQEd has been develped within the HeKatE Project,                           *
 * see http://hekate.ia.agh.edu.pl                                             *
 *                                                                             *
 * This file is part of HQEd.                                                  *
 *                                                                             *
 * HQEd is free software: you can redistribute it and/or modify                *
 * it under the terms of the GNU General Public License as published by        *
 * the Free Software Foundation, either version 3 of the License, or           *
 * (at your option) any later version.                                         *
 *                                                                             *
 * HQEd is distributed in the hope that it will be useful,                     *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU General Public License for more details.                                *
 *                                                                             *
 * You should have received a copy of the GNU General Public License           *
 * along with HQEd.  If not, see <http://www.gnu.org/licenses/>.               *
 *                                                                             *
 * Comment: Plik zawierajacy definicje metod klasy bedacej obiektem            *
 * pozwalajacym na zwijanie i rozwijanie poziomow ARD                          *
 *                                                                             *
 */

#include <QMessageBox>
#include <QtGui>
#include <QDialog>
#include <QObject>

#include <QAction>
#include <QActionGroup>
#include <QLabel>
#include <QMenu> 
#include <QtGui>

#include <QGraphicsItem>

#include "../../C/ARD/ardGraphicsScene.h"

#include "gARDlevel.h"
#include "gARDexpander.h"
//---------------------------------------------------------------------------

#define EXPANDER_WIDTH 20
#define EXPANDER_HEIGHT 20
//---------------------------------------------------------------------------

/**
 * Konstruktor klasy
 * @param gARDlevel* _gParent - Wskanik do rodzica
 * @returns Brak zwracanych wartosci
*/
gARDexpander::gARDexpander(gARDlevel* _gParent) : QGraphicsItem(0)
{
	fGparent = _gParent;
	fPictureStr = ":/all_images/images/mu.bmp";
     setZValue(1);
	setAcceptsHoverEvents(true);
}
//---------------------------------------------------------------------------

/**
 * Destruktor klasy
 * @returns brak zwracanych wartosci
*/
gARDexpander::~gARDexpander(void)
{
}
//---------------------------------------------------------------------------

/**
 * Zwraca wskaznik do rodzica graficznego
 * @returns zwraca wskaznik do rodzica graficznego
*/
gARDlevel* gARDexpander::gParent(void)
{
	return fGparent;
}
//--------------------------------------------------------------------------

/**
 * Metoda klasy GRect rysujaca kwadrat na ekranie
 * @param QPainter* _p - Wskanik do plutna rysowania
 * @param const QStyleOptionGraphicsItem* - Wskanik do stylu rysowania
 * @param QWidget* widget - Wskanik nieuzywany
 * @returns Brak zwracanych wartoci
*/
void gARDexpander::paint(QPainter* _p, const QStyleOptionGraphicsItem*, QWidget*)
{
	QRectF target(0., 0., EXPANDER_WIDTH, EXPANDER_HEIGHT);
	QRectF source(0., 0., EXPANDER_WIDTH, EXPANDER_HEIGHT);
	QImage image(fPictureStr);

	_p->drawImage(target, image, source); 
}	
//---------------------------------------------------------------------------

/**
 * Metoda wywolywana w momencie zwolnienie przycisku myszki na obiekcie
 * @param QGraphicsSceneMouseEvent* _event - wskaznik do obiektu przechowujacego parametry zdarzenia
 * @returns brak zwracanych wartoci
*/
void gARDexpander::mousePressEvent(QGraphicsSceneMouseEvent* _event)
{
	if(_event->button() != Qt::LeftButton)
		return;
		
	fGparent->setExpanded(!fGparent->expanded());
	if(fGparent->expanded())
		fPictureStr = ":/all_images/images/ma.bmp";
	if(!fGparent->expanded())
		fPictureStr = ":/all_images/images/pa.bmp";
	update();
	QGraphicsItem::mousePressEvent(_event);
}
//---------------------------------------------------------------------------

/**
 * Metoda wywolywana w momencie najechania wskaznikiem myszki na obiekt
 * @param QGraphicsSceneHoverEvent* _event - wskaznik do obiektu przechowujacego parametry zdarzenia
 * @returns brak zwracanych wartoci
*/
void gARDexpander::hoverEnterEvent(QGraphicsSceneHoverEvent* _event)
{
	if(fGparent->expanded())
		fPictureStr = ":/all_images/images/ma.bmp";
	if(!fGparent->expanded())
		fPictureStr = ":/all_images/images/pa.bmp";
		
	QGraphicsItem::hoverEnterEvent(_event);
}
//---------------------------------------------------------------------------

/**
 * Metoda wywolywana w momencie zjechania wskaznikiem myszki z obiektu
 * @param QGraphicsSceneHoverEvent* _event - wskaznik do obiektu przechowujacego parametry zdarzenia
 * @returns brak zwracanych wartoci
*/
void gARDexpander::hoverLeaveEvent(QGraphicsSceneHoverEvent* _event)
{
	if(fGparent->expanded())
		fPictureStr = ":/all_images/images/mu.bmp";
	if(!fGparent->expanded())
		fPictureStr = ":/all_images/images/pu.bmp";
		
	QGraphicsItem::hoverLeaveEvent(_event);
}
//---------------------------------------------------------------------------

/**
 * Metoda klasy zwracajaca obszar prostokatny w ktorym sie znajduje obiekt
 * @returns Obszar prostokatny zawierajacy obiekt
*/
QRectF gARDexpander::boundingRect() const
{
	return QRectF(0., 0., EXPANDER_WIDTH, EXPANDER_HEIGHT);
}
//---------------------------------------------------------------------------
