 /**
 *                                                                             *
 * @file gARDwire.h                                                            *
 * Project name: HeKatE hqed                                                   *
 * $Id: gARDwire.h,v 1.7 2008-03-03 22:04:34 kinio Exp $                       *
 * Author: Krzysztof Kaczor                                                    *
 * Date: 07.01.2008                                                            *
 *                                                                             *
 * Implementation by Krzysztof Kaczor <kinio4444@gmail.com>                    *
 *                                                                             *
 * Copyright (C) 2006-9 by the HeKatE Project                                  *
 *                                                                             *
 * HQEd has been develped within the HeKatE Project,                           *
 * see http://hekate.ia.agh.edu.pl                                             *
 *                                                                             *
 * This file is part of HQEd.                                                  *
 *                                                                             *
 * HQEd is free software: you can redistribute it and/or modify                *
 * it under the terms of the GNU General Public License as published by        *
 * the Free Software Foundation, either version 3 of the License, or           *
 * (at your option) any later version.                                         *
 *                                                                             *
 * HQEd is distributed in the hope that it will be useful,                     *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU General Public License for more details.                                *
 *                                                                             *
 * You should have received a copy of the GNU General Public License           *
 * along with HQEd.  If not, see <http://www.gnu.org/licenses/>.               *
 *                                                                             *
 * Comment: Plik zawierajacy definicje lini symbolizujacej czesc calego        *
 * polaczenia miedzy zaleznosciami na diagramie ARD                            *
 *                                                                             *
 */


#ifndef GARDWIREH
#define GARDWIREH
//---------------------------------------------------------------------------

#include <QGraphicsItem>
#include <QPainter>
#include <QGraphicsSceneMouseEvent>
#include <QStyleOptionGraphicsItem>
//---------------------------------------------------------------------------

class gARDdependency;
//---------------------------------------------------------------------------

class gARDwire : public QGraphicsItem
{
private:

     /// Okresla czy linia jest pionowa
     bool fIsVertical;

     /// Okresla czy mozna przesuwac ten element
     bool fIsMoveable;

     /// Wskaznik do rodzica elementu
     gARDdependency* fParent;

     /// Index elementu u rodzica
     int fIndex;

     /// Dlugosc lini
     float fLength;

     /// Okresla czy dany obiekt jest selected
     bool fIsSelected;

     /// Okresla wspolrzedna x-owa i y-kowa. W zaleznosci od typu zmiennej nie moze jej zmieniac
     QPointF fPos;

     /// Okresla poziom zagniezdzenia funckji itemChange
     int level;
	
	/// Okresla etykiete polaczenia
	QString fLabel;
	
	/// Okresla czy etykieta ma byc widoczna
	bool fLabelVisible;
	
	/// Okresla sposob wysrodkowania etykiety -1 - do lewej, 0 - srodek, 1 - do prawej
	int fLabelAlignment;

public:

     /// Konstruktor klasy
     gARDwire(bool, bool, gARDdependency*, int, int);

     /// Metoda rysujaca element
     void paint(QPainter*, const QStyleOptionGraphicsItem*, QWidget* widget=0);

     /// Metoda oreslajaca obszar znajdowania sie elementu
     QRectF boundingRect() const;

     /// Metoda wywolywana w momencie zmiany wlasciwosci obiektu
     QVariant itemChange(GraphicsItemChange, const QVariant&);

     /// Metoda wywolywana w momencie dwukrotnego klikniecia na obiekcie
     void mouseDoubleClickEvent(QGraphicsSceneMouseEvent*);

     // Metoda ustawiajaca wartosc pol
     void setVerticality(bool _v){ fIsVertical = _v; }
	void setLabelVisible(bool _v){ fLabelVisible = _v; }
	void setLabel(QString _l){ fLabel = _l; }
     void setIndex(int _i){ fIndex = _i; }
     void setLength(float);
     void setActive(bool _s){ fIsSelected = _s; }
     void setMoveable(bool);
     void setCurPos(float, float);
	void setLabelAlignment(int);

     // Metody zwaracajce wartosci pol
	
	/// Zwraca czy dany obiekt moze zostac klikniety
     bool selected(void){ return fIsSelected; }
	
	/// Zwraca tresc etykiety
	QString label(void){ return fLabel; }
	
	/// Zwraca czy etykieta jest widoczna
	bool isLabelVisible(void){ return fLabelVisible; }
	
	/// Zwraca sposob wysrodkowania etykiety
	int labelAlignment(void){ return fLabelAlignment; }
};
//---------------------------------------------------------------------------
#endif
