 /**
 *                                                                             *
 * @file GNode.cpp                                                             *
 * Project name: HeKatE hqed                                                   *
 * $Id: GNode.cpp,v 1.12 2008-03-03 22:04:35 kinio Exp $                        *
 * Author: Krzysztof Kaczor                                                    *
 * Date: 20.05.2007                                                            *
 *                                                                             *
 * Implementation by Krzysztof Kaczor <kinio4444@gmail.com>                    *
 *                                                                             *
 * Copyright (C) 2006-9 by the HeKatE Project                                  *
 *                                                                             *
 * HQEd has been develped within the HeKatE Project,                           *
 * see http://hekate.ia.agh.edu.pl                                             *
 *                                                                             *
 * This file is part of HQEd.                                                  *
 *                                                                             *
 * HQEd is free software: you can redistribute it and/or modify                *
 * it under the terms of the GNU General Public License as published by        *
 * the Free Software Foundation, either version 3 of the License, or           *
 * (at your option) any later version.                                         *
 *                                                                             *
 * HQEd is distributed in the hope that it will be useful,                     *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU General Public License for more details.                                *
 *                                                                             *
 * You should have received a copy of the GNU General Public License           *
 * along with HQEd.  If not, see <http://www.gnu.org/licenses/>.               *
 *                                                                             *
 * Comment: Plik zawierajacy definicje metod obiektu bedacego wezlem           *
 * polaczenia                                                                  *
 *                                                                             *
 */

#include <QMessageBox>
#include <QtGui>
#include <QDialog>
#include <QObject>

#include "../../C/MainWin_ui.h"
#include "../../M/XTT/XTT_Table.h"
#include "../../M/XTT/XTT_TableList.h"
#include "../../M/XTT/XTT_Row.h"
#include "../../M/XTT/XTT_ConnectionsList.h"
#include "GLine.h"
#include "GRect.h"
#include "GText.h"
#include "GTable.h"
#include "GNode.h"
//---------------------------------------------------------------------------

/**
 * Konstruktor klasy
 * @param XTT_Table* _t_parent - Wskanik do tabeli rodzica
 * @param XTT_Row* _r_parent - Wskanik do wiersza rodzica
 * @returns Brak zwracanych wartosci
*/
GNode::GNode(XTT_Table* _t_parent, XTT_Row* _r_parent, bool _income)
{
     fTableParent = _t_parent;
     fRowParent = _r_parent;
     isIncome = _income;

     setCursor(Qt::CrossCursor);
     setFlag(ItemIsSelectable);
     setFlag(ItemIsFocusable);
     setFlag(ItemIsMovable);
     setZValue(7);
}
//---------------------------------------------------------------------------

/**
 * Metoda klasy rysujaca obiekt na ekrnia
 * @param QPainter* _p - Wskanik do plotna na ktorym rysujemy obiekt
 * @param const QStyleOptionGraphicsItem* - Wskanik do stylu rysowania
 * @param QWidget* widget - Wskanik do nie wiem czego
 * @returns Brak zwracanych wartoci
*/
void GNode::paint(QPainter* _p, const QStyleOptionGraphicsItem*, QWidget*)
{
     // Rysowanie kwadratu
     _p->setPen(Qt::NoPen);
     _p->setBrush(QColor(0, 0, 0, 255));
     QPointF points[3] = {
               QPointF(0.0, -5.0),
               QPointF(0.0, 5.0),
               QPointF(8.5, 0.0)
               };
     _p->drawPolygon(points, 3);
	
	// Rysowanie linii
	float x, y, dx, dy;
	x = .0;
	y = .0;
	dx = fPosition.x() - scenePos().x();
	if(isIncome)
		dx += 8.5;
	dy = fPosition.y() - scenePos().y();
	_p->setPen(Qt::SolidLine);
	_p->drawLine(QPointF(x, y), QPointF(dx, dy));
	//_p->drawRect(boundingRect());
}
//---------------------------------------------------------------------------

/**
 * Metoda klasy QGraphicsItem okreslajaca przedzial w jakim sie znajduje element
 * @returns Prostokat w ktorym musi sie znajdowac element
*/
QRectF GNode::boundingRect() const
{
	float x, y, dx, dy;
	
	x = .0;
	y = -5.0;
	dx = (fPosition.x() - scenePos().x());
	dy = (fPosition.y() - scenePos().y());
	
	if(dx < .0)
	{
		x = 8.5;
		dx -= 8.5;
	}
	if(dx >= .0)
		dx += 8.5;
		
	if(dy < .0)
	{
		y = 5.;
		dy -= 10.;
	}
	if(dy >= .0)
		dy += 10.;
	
     return QRectF(x, y, dx, dy);
}
//---------------------------------------------------------------------------

/**
 * Metoda wywolywana w momencie zwolinienia przycisku na scenie
 * @param QGraphicsSceneMouseEvent* _gsme - Wskanik do obiektu przechowujacego 
 * informacje o zdarzeniu
 * @returns Brak zwracanych wartoci
*/
void GNode::mouseReleaseEvent(QGraphicsSceneMouseEvent* _gsme)
{
     if(_gsme->button() != Qt::LeftButton)
          return;

     QList<QGraphicsItem*> res = scene()->items(_gsme->scenePos());

     // Sprawdzanie wystapienie elementu wezla pod zwolnionym przyciskiem
     for(int i=0;i<res.size();i++)
     {
          // Sprawdzanie czy to nie jest ten sam element ktory wlasnie przenosimy
          if(res.at(i)->isSelected())
               continue;

          // Tworzenie listy parametrow
          QStringList params = res.at(i)->toolTip().split(";");

          // Jezeli lista zaiwera nie poprawna ilosc znakow pusta
          if(params.size() != 3)
               continue;

          // Tworzenie polaczenia
          int stid = -1;                     // ID tabeli z ktorej wychodzi to polaczenie
          int srid = -1;                     // ID wiersza z ktorego wychodzi to polaczenie
          int dtid = -1;                     // ID tabeli do ktorej przychodzi to polaczenie
          int drid = -1;                     // ID wiersza do ktorego przychodzi to polaczenie

          if(params.at(0) == "true")
          {
               stid = fTableParent->TableId();                     
               srid = fRowParent->RowId();
               dtid = params.at(1).toInt();
               drid = params.at(2).toInt();
          }
          else if(params.at(0) == "false")
          {
               dtid = fTableParent->TableId();
               drid = fRowParent->RowId();
               stid = params.at(1).toInt();
               srid = params.at(2).toInt();
          }
          else if((params.at(0) != "false") && (params.at(0) != "true"))
               continue;
          if(params.at(0) == toolTip().split(";").at(0))
               continue;

          int conn_ind = -1;

          // Sprawdzanie czy dane polaczenie juz nie istnieje
          conn_ind = ConnectionsList->IndexOfOutConnections(stid, srid);
          if(conn_ind == -1)
          {
               ConnectionsList->Add();
               conn_ind = (int)ConnectionsList->Count()-1;
          }
		
          // Po to zeby nie wystapily bledy podczas odswiezenia rysunku
          ConnectionsList->Connection(conn_ind)->setSrcTable(0);
          ConnectionsList->Connection(conn_ind)->setSrcRow(0);
          ConnectionsList->Connection(conn_ind)->setDesTable(0);
          ConnectionsList->Connection(conn_ind)->setDesRow(0);

          // Wlasciew ustawianie wartosci
          ConnectionsList->Connection(conn_ind)->setSrcTable(stid);
          ConnectionsList->Connection(conn_ind)->setSrcRow(srid);
          ConnectionsList->Connection(conn_ind)->setDesTable(dtid);
          ConnectionsList->Connection(conn_ind)->setDesRow(drid);
		
		// Ustawienie pozycji polaczenia
		ConnectionsList->Connection(conn_ind)->Reposition();
		ConnectionsList->Connection(conn_ind)->Repaint();
     }

     // Powrot na stare smieci :)
     setPos(fPosition);
}
//---------------------------------------------------------------------------

/**
 * Ustawia pozycje elementu na ktora ma wrocic po przeniesieniu
 * @param float x - Wspolrzedna X-owa
 * @param float y - Wspolrzedna Y-kowa
 * @returns Brak zwracanych wartoci
*/
void GNode::setPosition(float x, float y)
{
     fPosition.setX(x);
     fPosition.setY(y);
}
//---------------------------------------------------------------------------

