 /**
 *                                                                             *
 * @file GRect.cpp                                                             *
 * Project name: HeKatE hqed                                                   *
 * $Id: GRect.cpp,v 1.12 2008-03-03 22:04:35 kinio Exp $                        *
 * Author: Krzysztof Kaczor                                                    *
 * Date: 15.05.2007                                                            *
 *                                                                             *
 * Implementation by Krzysztof Kaczor <kinio4444@gmail.com>                    *
 *                                                                             *
 * Copyright (C) 2006-9 by the HeKatE Project                                  *
 *                                                                             *
 * HQEd has been develped within the HeKatE Project,                           *
 * see http://hekate.ia.agh.edu.pl                                             *
 *                                                                             *
 * This file is part of HQEd.                                                  *
 *                                                                             *
 * HQEd is free software: you can redistribute it and/or modify                *
 * it under the terms of the GNU General Public License as published by        *
 * the Free Software Foundation, either version 3 of the License, or           *
 * (at your option) any later version.                                         *
 *                                                                             *
 * HQEd is distributed in the hope that it will be useful,                     *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU General Public License for more details.                                *
 *                                                                             *
 * You should have received a copy of the GNU General Public License           *
 * along with HQEd.  If not, see <http://www.gnu.org/licenses/>.               *
 *                                                                             *
 * Comment: Plik zawierajacy definicje metod klasy bedacej prostokatem         *
 * odpowiednikiem jednej z tabel                                               *
 *                                                                             *
 */


#include <QMessageBox>
#include <QtGui>
#include <QDialog>
#include <QObject>

#include <QAction>
#include <QActionGroup>
#include <QLabel>
#include <QMenu> 
#include <QtGui>

#include <QGraphicsItem>

#include "../../M/XTT/XTT_AttributeGroups.h"
#include "../../M/XTT/XTT_Table.h"
#include "../../M/XTT/XTT_TableList.h"
#include "../../C/XTT/TableEditor_ui.h"
#include "../../C/XTT/CellEditor_ui.h"
#include "../../C/XTT/MyGraphicsScene.h"
#include "../../C/XTT/Hinter_ui.h"
#include "../../C/MainWin_ui.h"
#include "../../C/Settings_ui.h"
#include "GRect.h"
//---------------------------------------------------------------------------

/**
 * Konstruktor klasy
 * @param XTT_Table* _parent - Wska�nik do tabeli rodzica
 * @returns Brak zwracanych wartosci
*/
GRect::GRect(XTT_Table* _parent) : QGraphicsItem(0)
{
	fParent = _parent;
     setFlag(ItemIsSelectable);
     setFlag(ItemIsFocusable);
     setFlag(ItemIsMovable);
     setZValue(1);
	
	MainWin->xtt_scene->setNewItem(this);
}
//---------------------------------------------------------------------------

/**
 * Metoda klasy GRect rysujaca kwadrat na ekranie
 * @param QPainter* _p - Wska�nik do plutna rysowania
 * @param const QStyleOptionGraphicsItem* - Wska�nik do stylu rysowania
 * @param QWidget* widget - Wska�nik nieuzywany
 * @returns Brak zwracanych warto�ci
*/
void GRect::paint(QPainter* _p, const QStyleOptionGraphicsItem*, QWidget*)
{
     // Rysowanie kwadratu
     _p->setPen(Qt::NoPen);
     _p->setBrush(Settings->xttTableColor());
     if(isSelected())
          _p->setBrush(Settings->xttSelectedTableColor());
     if(MainWin->ModelBW())
          _p->setBrush(QColor(255, 255, 255, 255));
     _p->drawRect(0, 0, fParent->Width(), fParent->Height());
}
//---------------------------------------------------------------------------

void GRect::mousePressEvent(QGraphicsSceneMouseEvent*)
{
	scene()->clearSelection();
	setSelected(true);
}
//---------------------------------------------------------------------------

/**
 * Metoda klasy GRect wywolywana w momencie dwukrotnego klikniecia na obiekcie, 
 * prostokacie
 * @param QGraphicsSceneMouseEvent* _e - Wska�nik do informacji o zdarzeniu
 * @returns Brak zwracanych warto�ci
*/
void GRect::mouseDoubleClickEvent(QGraphicsSceneMouseEvent* _e)
{
     if(_e->button() != Qt::LeftButton)
          return;
	
	/*
     // Wyszukiwanie i edycja tabeli
     int index = TableList->IndexOfID(fParent->TableId());

     if(index == -1)
          return;

     delete CellEdit;
     CellEdit = new CellEditor_ui(index, 0);
     CellEdit->show();
	*/
	
	// Sprawdzanie w ktorej komorce nastpilo klikniecie
	int col = -1;
	int row = -1;
	
	unsigned int width = 0;
	unsigned int height = fParent->HeaderHeight();
	
	// Okreslanie kolumny
	for(int w=0;w<fParent->ColCount();w++)
	{
		width += fParent->ColWidth(w);
		if(width >= _e->pos().x())
		{
			col = w;
			break;
		}
	}
	
	// Sprawdzanie czy klikniecie nie nastapilo w naglowku
	if(height >= _e->pos().y())
		row = -2;
	
	// Okreslanie wiersza
	if(row != -2)
	{
		for(unsigned int h=0;h<fParent->RowCount();h++)
		{
			height += fParent->RowHeight(h);
			if(height >= _e->pos().y())
			{
				row = h;
				break;
			}
		}
	}
	
	//QMessageBox::critical(NULL, "XTT Editor", "Event coordinates row = " + QString::number(row) + ", col = " + QString::number(col), QMessageBox::Ok);
	
	// Uruchmienie okienka edycji zawartosci komorki
	if(fParent == NULL)
		return;
		
	// Jezeli edytujemy komorke
	if(row > -1)
	{
		delete Hinter;
		
	     // Parametry uruchomienia okienka
	     int context = fParent->ColContext(col);
	     XTT_Attribute* attr = fParent->ContextAtribute(col);
	     XTT_Cell* cell = NULL;
	     if(row < (int)fParent->RowCount())
	          cell = fParent->Row(row)->Cell(col);
		
	     Hinter = new Hinter_ui(context, fParent, attr, cell, NULL);
		QObject::connect(Hinter, SIGNAL(ClickOK()), MainWin->xtt_scene, SLOT(update()));

	     Hinter->move(QCursor::pos());
	     Hinter->show();
	}
	
	// Jezeli edytujemy naglowek
	if(row == -2)
	{
		int tindex = TableList->IndexOfID(fParent->TableId());
		if(tindex == -1)
			return;
			
		delete TableEdit;
		TableEdit = new TableEditor_ui(false, tindex);
		TableEdit->show();
	}
}
//---------------------------------------------------------------------------

/**
 * Metoda klasy zwracajaca obszar prostokatny w ktorym sie znajduje obiekt
 * @returns Obszar prostokatny zawierajacy obiekt
*/
QRectF GRect::boundingRect() const
{
     return QRectF(0, 0, fParent->Width(), fParent->Height());
}
//---------------------------------------------------------------------------

/**
 * Metoda wywolywana w momencie zmiany polozenia obiektu
 * @param GraphicsItemChange change - Zawiera wartosc elementu ktora ulegla 
 * zmianie
 * @param const QVariant &value - Referencja do wartosci zmienionych
 * @returns Wartosc ktora ulela zmianie
*/
QVariant GRect::itemChange(GraphicsItemChange change, const QVariant &value)
{
     if((change == ItemSelectedChange) && (scene()))
     {
          bool isSel = value.toBool();
          if(isSel)
               MainWin->xtt_scene->setSelectedTableID(fParent->TableId());
          else
               MainWin->xtt_scene->setSelectedTableID(-1);
     }
     
     if((change == ItemPositionChange) && (scene()))
     {
          // Value jest nowa wartoscia polozenia
          QPointF newPos = value.toPointF();
          fParent->setLocalization(newPos.toPoint());
		
          if(Settings->xttAutoSceneResize())
		{
	          QRectF srect = scene()->sceneRect();
			QRectF brect = boundingRect();
			QRectF newItemRect = QRectF(newPos.x(), newPos.y(), brect.width(), brect.height());
			QRectF requiredRect = srect | newItemRect;
			
			if(requiredRect != srect)
				scene()->setSceneRect(requiredRect);
			
			/*
	          if(!srect.contains(QRectF(newPos.x(), newPos.y(), brect.width(), brect.height())))
	          {
				// Resize scene for new position
				
				// Current parameters
				float x = srect.x();
				float y = srect.y();
				float w = srect.width();
				float h = srect.height();
				float margin = 1.0;
				
				// Checking parameters for update
				if(newPos.x() < (x+margin))
				{
					x -= margin;
					w += margin;
				}
				if((newPos.x()+brect.width()) > (x+w-margin))
					w += margin;
				if(newPos.y() < (y+margin))
				{
					y -= margin;
					h += margin;
				}
				if((newPos.y()+brect.height()) > (y+h-margin))
					h += margin;
					
				scene()->setSceneRect(QRectF(x, y, w, h));
	          }   
			*/
		}
     }
     return QGraphicsItem::itemChange(change, value);
}
//---------------------------------------------------------------------------
