 /**
 *                                                                             *
 * @file GTable.cpp                                                            *
 * Project name: HeKatE hqed                                                   *
 * $Id: GTable.cpp,v 1.12 2008-03-03 22:04:35 kinio Exp $                       *
 * Author: Krzysztof Kaczor                                                    *
 * Date: 15.05.2007                                                            *
 *                                                                             *
 * Implementation by Krzysztof Kaczor <kinio4444@gmail.com>                    *
 *                                                                             *
 * Copyright (C) 2006-9 by the HeKatE Project                                  *
 *                                                                             *
 * HQEd has been develped within the HeKatE Project,                           *
 * see http://hekate.ia.agh.edu.pl                                             *
 *                                                                             *
 * This file is part of HQEd.                                                  *
 *                                                                             *
 * HQEd is free software: you can redistribute it and/or modify                *
 * it under the terms of the GNU General Public License as published by        *
 * the Free Software Foundation, either version 3 of the License, or           *
 * (at your option) any later version.                                         *
 *                                                                             *
 * HQEd is distributed in the hope that it will be useful,                     *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU General Public License for more details.                                *
 *                                                                             *
 * You should have received a copy of the GNU General Public License           *
 * along with HQEd.  If not, see <http://www.gnu.org/licenses/>.               *
 *                                                                             *
 * Comment: Plik zawierajacy definicje metod obiektu bedacego tabela           *
 * rysowana na plotnie                                                         *
 *                                                                             *
 */
 
#include <QMessageBox>
#include <QtGui>
#include <QDialog>
#include <QObject>

#include "../../C/XTT/MyGraphicsScene.h"
#include "../../C/MainWin_ui.h"
#include "../../M/XTT/XTT_Table.h"
#include "GLine.h"
#include "GRect.h"
#include "GText.h"
#include "GNode.h"
#include "GTable.h"
//----------------------------------------------------------------------------

/**
 * Konstruktor klasy przyjmujacy rodzica
 * @param XTT_Table* _p - Wskanik do rodzica tabeli
 * @returns Brak zwracanych wartosci
*/
GTable::GTable(XTT_Table* _p) : fParent(_p)
{
     fHLineCount = 0;
     fVLineCount = 0;
     fTextCount = 0;
     fNodeCount = 0;
     fHLines = NULL;
     fVLines = NULL;
     fBase = NULL;
     fErrBase = NULL;
     fText = NULL;
     fNodes = NULL;

     fBase = new GRect(fParent); 
     fBase->setPos(fParent->Localization().x(), fParent->Localization().y());
     MainWin->xtt_scene->addItem(fBase);
}
//----------------------------------------------------------------------------

/**
 * Destruktor klasy
 * @returns Brak zwracanych wartosci
*/
GTable::~GTable(void)
{
     if(fHLineCount)
     {
          for(unsigned int i=0;i<fHLineCount;i++)
               delete fHLines[i];
          delete [] fHLines;
     }

     if(fVLineCount)
     {
          for(unsigned int i=0;i<fVLineCount;i++)
               delete fVLines[i];
          delete [] fVLines;
     }

     if(fTextCount)
     {
          for(unsigned int i=0;i<fTextCount;i++)
               delete fText[i];
          delete [] fText;
     }

     if(fNodeCount)
     {
          for(unsigned int i=0;i<fNodeCount;i++)
               delete fNodes[i];
          delete [] fNodes;
     }

     if(fBase)
          delete fBase;

     if(fErrBase)
          delete fErrBase;
}
//----------------------------------------------------------------------------

/**
 * Znajduje tabele na scenie i przewija widok do niej
 * @returns Brak zwracanych wartoci
*/
void GTable::ensureVisible(void)
{
	if(fBase->scene() == NULL)
		return;

		
	QRectF itemRect = fBase->boundingRect();	
	int xMargin = 0;
	int yMargin = 0;
	float itemWidth = (xMargin*2.)+itemRect.width();
	float itemHeight = (yMargin*2.)+itemRect.height();
	
	for(int s=0;s<fBase->scene()->views().count();++s)
	{
		QRectF sRect = fBase->scene()->views().at(s)->sceneRect();
		if((itemWidth < sRect.width()) && (itemHeight < sRect.height()))
		{
			fBase->scene()->views().at(s)->ensureVisible(fBase, xMargin, yMargin);
			//QMessageBox::question(NULL, "XTT Editor", "itemWidth=" + QString::number(itemWidth) + ", itemHeight=" + QString::number(itemHeight) + "\nsceneWidth=" + QString::number(sRect.width()) + ", sceneHeight=" + QString::number(sRect.height()), QMessageBox::Ok);
		}
	}
	
	fBase->update();
}
//----------------------------------------------------------------------------

/**
 * Dodaje nowa linie pozioma
 * @returns Brak zwracanych wartoci
*/
void GTable::AddHorLine(void)
{
     GLine** tmp = new GLine*[fHLineCount+1];

     // Kopiowanie
     for(unsigned int i=0;i<fHLineCount;i++)
          tmp[i] = fHLines[i];

     // Usuniecie starej tablicy wskaznikow
     if(fHLineCount)
          delete [] fHLines;

     fHLines = tmp;

     fHLines[fHLineCount] = new GLine(fParent, false, fHLineCount);
     MainWin->xtt_scene->addItem(fHLines[fHLineCount]);
     fHLineCount++;
}
//----------------------------------------------------------------------------

/**
 * Metoda klasy usuwajaca ostatnia linie z listy
 * @returns Brak zwracanych wartoci
*/
void GTable::DeleteHorLine(void)
{
     GLine** tmp = new GLine*[fHLineCount-1];

     // Kopiowanie
     for(unsigned int i=0;i<fHLineCount-1;i++)
          tmp[i] = fHLines[i];

     // Usuniecie starej tablicy wskaznikow
     if(fHLineCount)
     {
          delete fHLines[fHLineCount-1];
          delete [] fHLines;
     }

     fHLines = tmp;

     fHLineCount--;
}
//----------------------------------------------------------------------------

/**
 * Dodaje nowa linie pionowa
 * @returns Brak zwracanych wartoci
*/
void GTable::AddVerLine(void)
{
     GLine** tmp = new GLine*[fVLineCount+1];

     // Kopiowanie
     for(unsigned int i=0;i<fVLineCount;i++)
          tmp[i] = fVLines[i];

     // Usuniecie starej tablicy wskaznikow
     if(fVLineCount)
          delete [] fVLines;

     fVLines = tmp;

     fVLines[fVLineCount] = new GLine(fParent, true, fVLineCount);
     MainWin->xtt_scene->addItem(fVLines[fVLineCount]);
     fVLineCount++;
}
//----------------------------------------------------------------------------

/**
 * Metoda klasy usuwajaca ostatnia linie z listy
 * @returns Brak zwracanych wartoci
*/
void GTable::DeleteVerLine(void)
{
     GLine** tmp = new GLine*[fVLineCount-1];

     // Kopiowanie
     for(unsigned int i=0;i<fVLineCount-1;i++)
          tmp[i] = fVLines[i];

     // Usuniecie starej tablicy wskaznikow
     if(fVLineCount)
     {
          delete fVLines[fVLineCount-1];
          delete [] fVLines;
     }

     fVLines = tmp;

     fVLineCount--;
}
//----------------------------------------------------------------------------

/**
 * Metoda klasy tworzaca nowy obiekt bedacy tekstem w tabeli
 * @param unsigned int _tt - Typ tekstu okresla ktory tekst ma zostac pobranry
 * @param unsigned int _row - Wiersz z ktorego trzeba pobrac tekst
 * @param unsigned int _col - Kolumna z ktorej trzeba pobrac tekst
 * @returns Brak zwracanych wartoci
*/
void GTable::AddText(unsigned int _tt, unsigned int _row, unsigned int _col)
{
     GText** tmp = new GText*[fTextCount+1];

     // Kopiowanie
     for(unsigned int i=0;i<fTextCount;i++)
          tmp[i] = fText[i];

     // Usuniecie starej tablicy wskaznikow
     if(fTextCount)
          delete [] fText;

     fText = tmp;

     fText[fTextCount] = new GText(fParent, _tt, _row, _col);
     MainWin->xtt_scene->addItem(fText[fTextCount]);
     fTextCount++;
}
//----------------------------------------------------------------------------

/**
 * Metoda klasy usuwajaca ostatni tekst
 * @returns Brak zwracanych wartoci
*/
void GTable::DeleteText(void)
{
     GText** tmp = new GText*[fTextCount-1];

     // Kopiowanie
     for(unsigned int i=0;i<fTextCount-1;i++)
          tmp[i] = fText[i];

     // Usuniecie starej tablicy wskaznikow
     if(fTextCount)
     {
          delete fText[fTextCount-1];
          delete [] fText;
     }

     fText = tmp;

     fTextCount--;
}
//----------------------------------------------------------------------------

/**
 * Dodaje nowy wezel
 * @returns Brak zwracanych wartoci
*/
void GTable::AddNode(unsigned int _row)
{
     GNode** tmp = new GNode*[fNodeCount+1];

     // Kopiowanie
     for(unsigned int i=0;i<fNodeCount;i++)
          tmp[i] = fNodes[i];

     // Usuniecie starej tablicy wskaznikow
     if(fNodeCount)
          delete [] fNodes;

     fNodes = tmp;

     fNodes[fNodeCount] = new GNode(fParent, fParent->Row(_row), false);
     MainWin->xtt_scene->addItem(fNodes[fNodeCount]);
     fNodeCount++;
}
//----------------------------------------------------------------------------

/**
 * Metoda klasy usuwajaca ostatni wezel z listy
 * @returns Brak zwracanych wartoci
*/
void GTable::DeleteNode(void)
{
     GNode** tmp = new GNode*[fNodeCount-1];

     // Kopiowanie
     for(unsigned int i=0;i<fNodeCount-1;i++)
          tmp[i] = fNodes[i];

     // Usuniecie starej tablicy wskaznikow
     if(fNodeCount)
     {
          delete fNodes[fNodeCount-1];
          delete [] fNodes;
     }

     fNodes = tmp;

     fNodeCount--;
}
//----------------------------------------------------------------------------

/**
 * Metoda klasy na nowo obliczajaca i ustwiajaca wartosci parametrow wszystkich elementow
 * @returns Brak zwracanych wartoci
*/
void GTable::ReCreate(void)
{
     //QMessageBox::information(NULL, "XTT Editor", "myk1", QMessageBox::Ok);
     // Obliczanie liczby pionowych lini
     unsigned int vline_quantity = fParent->ColCount() + 1;

     // Poprawa ilosc w zaleznosci od granic pomiedzy kolejnymi kontekstami
     // Kiedy istnieje granica pomiedzy conditional i assert lub retract
     if((fParent->ContextSize(1) > 0) && ((fParent->ContextSize(2) > 0) || (fParent->ContextSize(3) > 0)))
          vline_quantity += 1;

     // Kiedy istnieje granica pomiedzy action i assert lub retract
     if((fParent->ContextSize(4) > 0) && ((fParent->ContextSize(2) > 0) || (fParent->ContextSize(3) > 0)))
          vline_quantity += 1;

     // Kiedy tylko wystepuja konteksty conditional i action
     if((fParent->ContextSize(1) > 0) && (fParent->ContextSize(2) == 0) && (fParent->ContextSize(3) == 0) && (fParent->ContextSize(4) > 0))
          vline_quantity += 1;

     // Obliczanie liczby poziomych lini
     unsigned int hline_quantity = fParent->RowCount() + 2;

     // Obliczanie ilsoc tekstow
     unsigned int text_quantity = fParent->RowCount() * fParent->ColCount() + fParent->ColCount() + 1;

     // Obliczanie liczby potrzebnych wezlow
     unsigned int nodes_quantity = fParent->RowCount();
     if(fParent->TableType() == 1)
          nodes_quantity = fParent->RowCount()*2;

     // Alokacja pamieci na wybrane elementy

     // Na linie poziome
     for(;hline_quantity < fHLineCount;)
          DeleteHorLine();
     for(;hline_quantity > fHLineCount;)
          AddHorLine();

     // Na linie pionowe
     for(;vline_quantity < fVLineCount;)
          DeleteVerLine();
     for(;vline_quantity > fVLineCount;)
          AddVerLine();

     // Na teksty
     for(;text_quantity < fTextCount;)
          DeleteText();
     for(;text_quantity > fTextCount;)
          AddText(0, 0, 0);

     // Na wezly
     for(;nodes_quantity < fNodeCount;)
          DeleteNode();
     for(;nodes_quantity > fNodeCount;)
          AddNode(0);
      
     // Tworzenie kwadratu na tlo tabeli
     //if(fBase)
          //delete fBase;
     //fBase = new GRect(fParent);
     //fBase->setPos(fParent->Localization().x(), fParent->Localization().y());
     //MainWin->scene->addItem(fBase);
     //QMessageBox::information(NULL, "XTT Editor", "myk2", QMessageBox::Ok);
}
//----------------------------------------------------------------------------

/**
 * Metoda uaktualniajaca wszystkie parametry elementow
 * @returns Brak zwracanych wartoci
*/
void GTable::Update(void)
{
     //QMessageBox::information(NULL, "XTT Editor", "myk3", QMessageBox::Ok);
     // Ustawiamy parametry obiektow

     // Ustawianie lini poziomych
     for(unsigned int i=0;i<fHLineCount;i++)
          fHLines[i]->setVerticality(false);

     // Ustawianie lini pionowych
     for(unsigned int i=0;i<fVLineCount;i++)
          fVLines[i]->setVerticality(true);

     // Ustawianie parametrow tekstow

     // Teksty naglowkow
     for(int i=0;i<fParent->ColCount();i++)
     {
          fText[i]->setCol(i);
          fText[i]->setTextType(2);
     }

     // Teksty kmorek
     for(int i=0;i<fParent->ColCount();i++)
     {
          for(unsigned int j=0;j<fParent->RowCount();j++)
          {
               int index = fParent->ColCount()+i*fParent->RowCount()+j;
               fText[index]->setRow(j);
               fText[index]->setCol(i);
               fText[index]->setTextType(0);
          }
     }

     // Tekst tytulu, powinien byc ostatni
     fText[fTextCount-1]->setTextType(1);

     // Ustawianie wezlow
     for(unsigned int i=0;i<fParent->RowCount();i++)
     {
          fNodes[i]->setRowParent(fParent->Row(i));
          if(fParent->TableType() == 1)
               fNodes[fParent->RowCount()+i]->setRowParent(fParent->Row(i));
     }
          
     //QMessageBox::information(NULL, "XTT Editor", "myk4", QMessageBox::Ok);
}
//----------------------------------------------------------------------------

/**
 * Metoda ustawiajaca elementy na swoim miejscu
 * @returns Brak zwracanych wartoci
*/
void GTable::RePosition(void)
{
     //QMessageBox::information(NULL, "XTT Editor", "myk5", QMessageBox::Ok);
     // Kwadrat bazowy
     //fBase->setPos(fParent->Localization().x(), fParent->Localization().y());

     // Kolejne linie
     int next_x = fParent->Localization().x();
     int next_y = fParent->Localization().y();

     // Rysowanie lini poziomych
     fHLines[0]->setPos(next_x, next_y);
     next_y += fParent->HeaderHeight();
     for(unsigned int i=1;i<fHLineCount;i++)
     {
          fHLines[i]->setPos(next_x, next_y);
          next_y += fParent->RowHeight(i-1);
     }

     // Rysowanie lini pionowych
     next_y = fParent->Localization().y();
     for(int i=0;i<=fParent->ColCount();i++)
     {
          fVLines[i]->setPos(next_x, next_y);
          next_x += fParent->ColWidth(i);
     }

     // Rysowanie lini specjalnych (podwojnych)
     int dc = fVLineCount - fParent->ColCount();

     // Jezeli sa to dwie linie to trzeba je wstawic po kontekscie contidional i przed action
     if(dc == 3)
     {
          // Wstawianie po kontekscie conditional
          next_x = fParent->Localization().x();
          next_y = fParent->Localization().y();
          for(int i=0;i<fParent->ColCount();i++)
          {
               if(fParent->ColContext(i) != 1)
               {
                    fVLines[fVLineCount-2]->setPos(next_x+4, next_y);
                    break;
               }
               next_x += fParent->ColWidth(i);
          }

          // Wstawianie przed kontekst action
          next_x = fParent->Localization().x() + fParent->Width();
          next_y = fParent->Localization().y();
          for(unsigned int i=fParent->ColCount()-1;((int)i)>=0;i--)
          {
               if(fParent->ColContext(i) != 4)
               {
                    fVLines[fVLineCount-1]->setPos(next_x+4, next_y);
                    break;
               }
			next_x -= fParent->ColWidth(i);
          }
     }

     // Jezeli jest to tylko jedna linia nadmiarowa to trzeba ja wstawic po kontekscie
     // conditional, lub action
     if(dc == 2)
     {
          if(fParent->ContextSize(1) > 0)
          {
               // Wstawianie po kontekscie conditional
               next_x = fParent->Localization().x();
               next_y = fParent->Localization().y();
               for(int i=0;i<fParent->ColCount();i++)
               {
                    if(fParent->ColContext(i) != 1)
                    {
                         fVLines[fVLineCount-1]->setPos(next_x+4, next_y);
                         break;
                    }
                    next_x += fParent->ColWidth(i);
               }
          }

          if((fParent->ContextSize(4) > 0) && (fParent->ContextSize(1) == 0))
          {
               // Wstawianie przed kontekst action
               next_x = fParent->Localization().x() + fParent->Width();;
               next_y = fParent->Localization().y();
               for(unsigned int i=fParent->ColCount()-1;((int)i)>=0;i--)
               {
                    if(fParent->ColContext(i) != 4)
                    {
                         fVLines[fVLineCount-1]->setPos(next_x+4, next_y);
                         break;
                    }
                    next_x -= fParent->ColWidth(i-1);
               }
          }
     }

     // Wypisanie tekstow

     // Teksty naglowkow
     next_x = fParent->Localization().x();
     next_y = fParent->Localization().y();
     for(int i=0;i<fParent->ColCount();i++)
     {
          fText[i]->setPos(next_x, next_y);
          next_x += fParent->ColWidth(i);
     }

     // Teksty komorek
     next_x = fParent->Localization().x();
     for(int c=0;c<fParent->ColCount();c++)
     {
          next_y = fParent->Localization().y() + fParent->HeaderHeight();
          for(unsigned int r=0;r<fParent->RowCount();r++)
          {
               int index = fParent->ColCount() + c*fParent->RowCount() + r;
               fText[index]->setPos(next_x, next_y);

               next_y += fParent->RowHeight(r);
          }
          next_x += fParent->ColWidth(c);
     }

     // Tekst podpisu
     fText[fTextCount-1]->setPos(fParent->Localization().x(), fParent->Localization().y() + (int)fParent->Height());

     // Ustawianie wezlow

     next_x = fParent->Localization().x();
     next_y = fParent->Localization().y() + fParent->HeaderHeight();
     for(unsigned int i=0;i<fParent->RowCount();i++)
     {
          int y = next_y + (fParent->RowHeight(i)/2);

          // Gdy tabela jest typu initialize
          if(fParent->TableType() == 0)
          {
               fNodes[i]->setPos(next_x+(int)fParent->Width()+1, y);
               fNodes[i]->setPosition(next_x+(int)fParent->Width()+1, y);
               fNodes[i]->setIncome(false);
               // Zapisanie parametrow wezla do obiektu
               fNodes[i]->setToolTip("false;" + QString::number(fParent->TableId()) + ";" + QString::number(fParent->Row(i)->RowId()));
          }

          // Gdy tabela jest typu middle
          if(fParent->TableType() == 1)
          {
               fNodes[i]->setPos(next_x-8.5, y);
               fNodes[i]->setPosition(next_x-8.5, y);
               fNodes[i]->setIncome(true);
                // Zapisanie parametrow wezla do obiektu
               fNodes[i]->setToolTip("true;" + QString::number(fParent->TableId()) + ";" + QString::number(fParent->Row(i)->RowId()));

               fNodes[fParent->RowCount()+i]->setPos(next_x+(int)fParent->Width()+1, y);
               fNodes[fParent->RowCount()+i]->setPosition(next_x+(int)fParent->Width()+1, y);
               fNodes[fParent->RowCount()+i]->setIncome(false);
			
                // Zapisanie parametrow wezla do obiektu
               fNodes[fParent->RowCount()+i]->setToolTip("false;" + QString::number(fParent->TableId()) + ";" + QString::number(fParent->Row(i)->RowId()));
          }
          
          // Gdy tabela jest typu halt
          if(fParent->TableType() == 2)
          {
               fNodes[i]->setPos(next_x-8.5, y);
               fNodes[i]->setPosition(next_x-8.5, y);
               fNodes[i]->setIncome(true);
               
                // Zapisanie parametrow wezla do obiektu
               fNodes[i]->setToolTip("true;" + QString::number(fParent->TableId()) + ";" + QString::number(fParent->Row(i)->RowId()));
          }
               
          next_y += fParent->RowHeight(i);
     }
	
	// Uaktualnienie widoku sceny
	MainWin->xtt_scene->update();

     //QMessageBox::information(NULL, "XTT Editor", "myk6", QMessageBox::Ok);
}
//----------------------------------------------------------------------------

/**
 * Metoda ustawiajaca zaznaczanie danej tabeli
 * @param bool _sel - Status tabeli
 * @returns Brak zwracanych wartoci
*/
void GTable::setTableSelection(bool _sel)
{
     fBase->setSelected(_sel);
	fBase->update();
}
//----------------------------------------------------------------------------
