 /**
 *                                                                             *
 * @file GTable.h                                                              *
 * Project name: HeKatE hqed                                                   *
 * $Id: GTable.h,v 1.12 2008-03-03 22:04:35 kinio Exp $                         *
 * Author: Krzysztof Kaczor                                                    *
 * Date: 15.05.2007                                                            *
 *                                                                             *
 * Implementation by Krzysztof Kaczor <kinio4444@gmail.com>                    *
 *                                                                             *
 * Copyright (C) 2006-9 by the HeKatE Project                                  *
 *                                                                             *
 * HQEd has been develped within the HeKatE Project,                           *
 * see http://hekate.ia.agh.edu.pl                                             *
 *                                                                             *
 * This file is part of HQEd.                                                  *
 *                                                                             *
 * HQEd is free software: you can redistribute it and/or modify                *
 * it under the terms of the GNU General Public License as published by        *
 * the Free Software Foundation, either version 3 of the License, or           *
 * (at your option) any later version.                                         *
 *                                                                             *
 * HQEd is distributed in the hope that it will be useful,                     *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU General Public License for more details.                                *
 *                                                                             *
 * You should have received a copy of the GNU General Public License           *
 * along with HQEd.  If not, see <http://www.gnu.org/licenses/>.               *
 *                                                                             *
 * Comment: Plik zawierajacy definicje obiektu bedacego tabela rysowana        *
 * na plotnie                                                                  *
 *                                                                             *
 */


#ifndef GTableH
#define GTableH
//---------------------------------------------------------------------------

class XTT_Table;
class GLine;
class GRect;
class GText;
class GNode;
//---------------------------------------------------------------------------

class GTable
{
private:

     /// Tabela odpowiednik
     XTT_Table* fParent;

     /// Ilosc lini poziomych
     unsigned int fHLineCount;

     /// Ilosc lini pionowych
     unsigned int fVLineCount;

     /// Ilosc textow
     unsigned int fTextCount;

     /// Ilosc wezlow polaczen
     unsigned int fNodeCount;

     /// Wskazniki do lini poziomych
     GLine** fHLines;

     /// Wskazniki do lini pionowych
     GLine** fVLines;

     /// Wskaznik do prostokata, tla tabeli
     GRect* fBase;

     /// Wskaznik do prostokata, tla komorki z bledem
     GRect* fErrBase;

     /// Wskazniki do tekstow
     GText** fText;

     /// Wskazniki do wezlow
     GNode** fNodes;

public:

     /// Konstruktor
     GTable(XTT_Table*);

     /// Destruktor
     ~GTable(void);

     /// Zwraca ilosc pionowych lini
     unsigned int HorLineCount(void){ return fHLineCount; }

     /// Zwraca ilosc piozmych lini
     unsigned int VerLineCount(void){ return fVLineCount; }

     /// Zwraca ilosc tekxtow lini
     unsigned int TextCount(void){ return fTextCount; }

     /// Dodaje nowa linie pozioma
     void AddHorLine(void);

     /// Dodaje nowa linie pionowa
     void AddVerLine(void);

     /// Dodaje nowy tekst
     void AddText(unsigned int, unsigned int, unsigned int);

     /// Dodaje nowy wezel
     void AddNode(unsigned int);

     /// Usuwa ostatnia pionowa linie
     void DeleteVerLine(void);

     /// Usuwa ostatnia pionowa linie
     void DeleteHorLine(void);

     /// Usuwa ostatni text
     void DeleteText(void);

     /// Usuwa ostatni wezel
     void DeleteNode(void);

     /// Tworzy liste wszystkich komponentow od nowa i ustawia je
     void ReCreate(void);

     /// Uaktualnie parametry obiektow
     void Update(void);

     /// Ustawia elementy na swoim miejscu
     void RePosition(void);

     /// Ustawia tabele jako wybrana
     void setTableSelection(bool);
	
	/// Znajduje tabele na scenie
	void ensureVisible(void);
};
//---------------------------------------------------------------------------
#endif
