 /**
 *                                                                             *
 * @file GText.cpp                                                             *
 * Project name: HeKatE hqed                                                   *
 * $Id: GText.cpp,v 1.12 2008-03-03 22:04:35 kinio Exp $                        *
 * Author: Krzysztof Kaczor                                                    *
 * Date: 15.05.2007                                                            *
 *                                                                             *
 * Implementation by Krzysztof Kaczor <kinio4444@gmail.com>                    *
 *                                                                             *
 * Copyright (C) 2006-9 by the HeKatE Project                                  *
 *                                                                             *
 * HQEd has been develped within the HeKatE Project,                           *
 * see http://hekate.ia.agh.edu.pl                                             *
 *                                                                             *
 * This file is part of HQEd.                                                  *
 *                                                                             *
 * HQEd is free software: you can redistribute it and/or modify                *
 * it under the terms of the GNU General Public License as published by        *
 * the Free Software Foundation, either version 3 of the License, or           *
 * (at your option) any later version.                                         *
 *                                                                             *
 * HQEd is distributed in the hope that it will be useful,                     *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU General Public License for more details.                                *
 *                                                                             *
 * You should have received a copy of the GNU General Public License           *
 * along with HQEd.  If not, see <http://www.gnu.org/licenses/>.               *
 *                                                                             *
 * Comment: Plik zawierajacy definicje metod klasy GTect, bedacej              *
 * obiektem rysowanym na plotnie                                               *
 *                                                                             *
 */

#include <QMessageBox>
#include <QtGui>
#include <QFont>
#include <QDialog>
#include <QObject>

#include "../../M/XTT/XTT_Cell.h"
#include "../../M/XTT/XTT_Table.h"
#include "../../M/XTT/XTT_TableList.h"
#include "../../M/XTT/XTT_Attribute.h"
#include "../../M/XTT/XTT_AttributeGroups.h"
#include "../../C/XTT/Hinter_ui.h"
#include "../../C/XTT/MyGraphicsScene.h"
#include "../../C/XTT/AttributeEditor_ui.h"
#include "../../C/Settings_ui.h"
#include "../../C/MainWin_ui.h"
#include "GText.h"
//---------------------------------------------------------------------------

/**
 * Konstruktor klasy przyjmujacy jako wartosc rodzica i wspolrzedne komorki z 
 * ktorej trzeba pobrac text
 * @param XTT_Table* _parent - Wskanik do rodzica tabeli
 * @param int row - index wiersza komorki
 * @param int cel - index kolumny komorki
 * @returns Brak zwracanych wartosci
*/
GText::GText(XTT_Table* _parent, unsigned int _tt, unsigned int _row, unsigned int _col) : QObject(0), QGraphicsItem(0)
{
     setZValue(3);
     fParent = _parent;
     fTextType = _tt;
     fRow = _row;
     fCol = _col;
	isSelected = false;
	setAcceptsHoverEvents(true);
}
//---------------------------------------------------------------------------

/**
 * Metoda klasy uruchamiana w momencie najechania wskaznika myszy na elemnt
 * @param QGraphicsSceneHoverEvent* event - obiekt przehowujacy parametry zdarzenia
 * @returns Brak zwracanych wartoci
*/
void GText::hoverEnterEvent(QGraphicsSceneHoverEvent* event)
{
	if(scene() == NULL)
		return;

	isSelected = true;
	if((Settings->xttEnabledAutocentering()) && (scene()->mouseGrabberItem() != this))
		ensureVisible(boundingRect(), 0, 0);
	QGraphicsItem::hoverEnterEvent(event);
}
//---------------------------------------------------------------------------

/**
 * Metoda klasy uruchamiana w momencie opuszczenia wskaznika myszy z elemntu
 * @param QGraphicsSceneHoverEvent* event - obiekt przehowujacy parametry zdarzenia
 * @returns Brak zwracanych wartoci
*/
void GText::hoverLeaveEvent(QGraphicsSceneHoverEvent* event)
{
	if(fTextType == 0)
		fParent->Row(fRow)->Cell(fCol)->setSelected(false);
		
	isSelected = false;
	QGraphicsItem::hoverLeaveEvent(event);
}
//---------------------------------------------------------------------------

/**
 * Metoda klasy zwracajaca obszar prostokatny w ktorym sie znajduje obiekt
 * @returns Obszar prostokatny zawierajacy obiekt
*/
QRectF GText::boundingRect() const
{
	// Ustalanie rozmiarow tekstu
	int tWidth = 0;
	int tHeight = 0;
	
	// Jezeli zawartosc komorki
	if(fTextType == 0)
	{
		tWidth = fParent->ColWidth(fCol);
		tHeight = fParent->RowHeight(fRow);
	}
	
	// Jezeli tytul tabeli
	if(fTextType == 1)
	{
		tWidth = fParent->Width();
		tHeight = Settings->xttTableTitleFont().pointSize();
	}
	
	// Jezeli tresc naglowka
	if(fTextType == 2)
	{
		tWidth = fParent->ColWidth(fCol);
		tHeight = fParent->HeaderHeight();
	}
	
     return QRectF(-5, 0, tWidth, tHeight);
}
//---------------------------------------------------------------------------

/**
 * Metoda klasy GText rysujaca kwadrat na ekranie
 * @param QPainter* _p - Wskanik do plutna rysowania
 * @param const QStyleOptionGraphicsItem* - Wskanik do stylu rysowania
 * @param QWidget* widget - Wskanik nieuzywany
 * @returns Brak zwracanych wartoci
*/
void GText::paint(QPainter* _p, const QStyleOptionGraphicsItem*, QWidget*)
{
     QString text = "";
     QRect rect;
     int flags = Qt::AlignHCenter | Qt::AlignVCenter | Qt::TextWordWrap;
     
     if(fTextType == 0)
     {
          if((fRow >= fParent->RowCount()) || (fCol >= (unsigned int)fParent->ColCount()))
               return;
          text = fParent->Row(fRow)->Cell(fCol)->Content();
		setToolTip("");
		if(Settings->xttShowErrorInToolTip())
			setToolTip(fParent->Row(fRow)->Cell(fCol)->CellError(false));
          rect.setRect(0, 0, fParent->ColWidth(fCol), fParent->RowHeight(fRow));
		_p->setFont(Settings->xttTableFont());
		_p->setPen(Settings->xttTableFontColor());
     }
     else if(fTextType == 1)
     {
          text = "Table id: " + QString::number(fParent->TableId()) + " - " + fParent->Title();
          rect.setRect(0, 0, fParent->Width(), 20);
          flags = Qt::AlignLeft | Qt::AlignVCenter | Qt::TextWordWrap;
		_p->setFont(Settings->xttTableTitleFont());
		_p->setPen(Settings->xtTableTitleFontColor());
     }
     else if(fTextType == 2)
     {
          QString aname = AttributeGroups->CreatePath(fParent->ContextAtribute(fCol)) + fParent->ContextAtribute(fCol)->Name();
          if(aname == "#NAN")
          {
               QMessageBox::critical(NULL, "XTT Editor", "Missing pointer.", QMessageBox::Ok);
               return;
          }
          // Ustalanie contekstu i dodanie znacznika kontekstu
          QStringList ctx_smbl;
          ctx_smbl << "(?) " << "(+) " << "(-) " << "(->) ";
          aname = ctx_smbl.at(fParent->ColContext(fCol)-1) + aname;
          text = aname;
          rect.setRect(0, 0, fParent->ColWidth(fCol), fParent->HeaderHeight());
		_p->setFont(Settings->xttTableHeaderFont());
		_p->setPen(Settings->xttTableHeaderFontColor());
     }
	
	// Jezeli niepoprawna skladnia to podswietlamy komorke
	if((fTextType == 0) && (!fParent->Row(fRow)->Cell(fCol)->isCorrectSyntax()) && (Settings->xttHighlightErrorCell()))
		_p->fillRect(rect, QBrush(Settings->xttErrorCellColor(), MainWin->ModelBW()?Qt::Dense6Pattern:Qt::SolidPattern));
		
	// Jezeli komorka jest zaznaczona
	if((fTextType == 0) && ((isSelected) || (fParent->Row(fRow)->Cell(fCol)->isSelected())))
		_p->fillRect(rect, QBrush(Settings->xttHoveredCellColor(), MainWin->ModelBW()?Qt::Dense7Pattern:Qt::SolidPattern));
		
     _p->drawText(rect, flags, text);
}
//---------------------------------------------------------------------------

/*
 * Metoda wywolywana tworzaca polecenia menu kontekstowego
 * @returns Brak zwracanych wartosci
*/
void GText::createActions(void)
{
	aEidtTable = new QAction(QIcon(":/all_images/images/Edit1.bmp"), QObject::tr("Edit table..."), NULL);
     aEidtTable->setStatusTip(QObject::tr("Allows edit this table"));
     QObject::connect(aEidtTable, SIGNAL(triggered()), MainWin, SLOT(TableEditorEdit()));
	
	aCellEditor = new QAction(QIcon(":/all_images/images/Edit2.bmp"), QObject::tr("Cell editor..."), NULL);
     aCellEditor->setStatusTip(QObject::tr("Show cell editor"));
     QObject::connect(aCellEditor, SIGNAL(triggered()), MainWin, SLOT(ShowCellEditor()));
	
	aEditCell = new QAction(QIcon(":/all_images/images/Edit1.bmp"), QObject::tr("Edit cell..."), NULL);
     aEditCell->setStatusTip(QObject::tr("Allows edit this cell content"));
     QObject::connect(aEditCell, SIGNAL(triggered()), this, SLOT(showHinter()));
	
	aDeleteTable = new QAction(QIcon(":/all_images/images/DelTabele.bmp"), QObject::tr("Delete table"), NULL);
     aDeleteTable->setStatusTip(QObject::tr("Allows remove this table"));
     QObject::connect(aDeleteTable, SIGNAL(triggered()), MainWin, SLOT(deleteSelectedTable()));
	
	aInsertRowBefore = new QAction(QIcon(":/all_images/images/AddRowBefore.bmp"), QObject::tr("Insert row before"), NULL);
     aInsertRowBefore->setStatusTip(QObject::tr("Allows add new row before this row"));
     QObject::connect(aInsertRowBefore, SIGNAL(triggered()), this, SLOT(insertRowBefore()));
	
	aInsertRowAfter = new QAction(QIcon(":/all_images/images/AddRowAfter.bmp"), QObject::tr("Insert row after"), NULL);
     aInsertRowAfter->setStatusTip(QObject::tr("Allows add new row after this row"));
     QObject::connect(aInsertRowAfter, SIGNAL(triggered()), this, SLOT(insertRowAfter()));
	
	aMoveRowUp = new QAction(QIcon(":/all_images/images/RowMoveUp.bmp"), QObject::tr("Move row up"), NULL);
     aMoveRowUp->setStatusTip(QObject::tr("Allows move this row one position up"));
     QObject::connect(aMoveRowUp, SIGNAL(triggered()), this, SLOT(moveRowUp()));
	
	aMoveRowDown = new QAction(QIcon(":/all_images/images/RowMoveDown.bmp"), QObject::tr("Move row down"), NULL);
     aMoveRowDown->setStatusTip(QObject::tr("Allows move this row one position down"));
     QObject::connect(aMoveRowDown, SIGNAL(triggered()), this, SLOT(moveRowDown()));
	
	aDeleteRow = new QAction(QIcon(":/all_images/images/DelRow.bmp"), QObject::tr("Delete row"), NULL);
     aDeleteRow->setStatusTip(QObject::tr("Allows remove this row from this table"));
     QObject::connect(aDeleteRow, SIGNAL(triggered()), this, SLOT(deleteRow()));
	
	aDeleteCol = new QAction(QIcon(":/all_images/images/DelCol.bmp"), QObject::tr("Delete col"), NULL);
     aDeleteCol->setStatusTip(QObject::tr("Allows remove this column from this table"));
     QObject::connect(aDeleteCol, SIGNAL(triggered()), this, SLOT(deleteCol()));
	
	aAttributeEditor = new QAction(QIcon(":/all_images/images/Edit1.bmp"), QObject::tr("Edit attribute"), NULL);
     aAttributeEditor->setStatusTip(QObject::tr("Allows to edit attribute related with this column"));
     QObject::connect(aAttributeEditor, SIGNAL(triggered()), this, SLOT(editAttribute()));
	
	if(fTextType == 0)
	{
		QFont boldFont = aEditCell->font();
		boldFont.setBold(true);
		aEditCell->setFont(boldFont);
	}
	
	if(fTextType == 2)
	{
		QFont boldFont = aEidtTable->font();
		boldFont.setBold(true);
		aEidtTable->setFont(boldFont);
	}
}
//---------------------------------------------------------------------------

/*
 * Metoda wywolywana w momenie klikniecia na obiekt prawym przyciskiem myszy
 * @param QGraphicsSceneContextMenuEvent *contextMenuEvent - wskaznik do obiektu przechowujacego informacje i parametry zdarzenia
 * @returns Brak zwracanych wartosci
*/
void GText::contextMenuEvent(QGraphicsSceneContextMenuEvent* contextMenuEvent)
{
	if(fTextType == 1)
		return;
		
	createActions();
	
	QMenu mainMenu(NULL);
	
	if(fTextType != 2)
		mainMenu.addAction(aEditCell);
	mainMenu.addAction(aEidtTable);
	mainMenu.addAction(aCellEditor);
	
	mainMenu.addSeparator();
	if(fTextType != 2)
	{
		mainMenu.addAction(aInsertRowBefore);
		mainMenu.addAction(aInsertRowAfter);
		
		mainMenu.addSeparator();
		mainMenu.addAction(aMoveRowUp);
		mainMenu.addAction(aMoveRowDown);
		
		mainMenu.addSeparator();
		mainMenu.addAction(aDeleteRow);
	}
	mainMenu.addAction(aDeleteCol);
	
	mainMenu.addSeparator();
	mainMenu.addAction(aDeleteTable);
	mainMenu.addSeparator();
	mainMenu.addAction(aAttributeEditor);
	
	mainMenu.exec(contextMenuEvent->screenPos());
	
	delete aEidtTable;
	delete aCellEditor;
	delete aEditCell;
	delete aDeleteTable;
	delete aInsertRowBefore;
	delete aInsertRowAfter;
	delete aDeleteRow;
	delete aDeleteCol;
	delete aMoveRowDown;
	delete aMoveRowUp;
}
//---------------------------------------------------------------------------

/*
 * Metoda wywolujaca okno edycji zawartosci komorki
 * @returns Brak zwracanych wartosci
*/
void GText::showHinter(void)
{
	if(((int)fCol) >= 0)
	{
		delete Hinter;
		
	     // Parametry uruchomienia okienka
	     int context = fParent->ColContext(fCol);
	     XTT_Attribute* attr = fParent->ContextAtribute(fCol);
	     XTT_Cell* cell = NULL;
	     cell = fParent->Row(fRow)->Cell(fCol);
			
	     Hinter = new Hinter_ui(context, fParent, attr, cell, NULL);
		QObject::connect(Hinter, SIGNAL(ClickOK()), MainWin->xtt_scene, SLOT(update()));
	     Hinter->move(QCursor::pos());
	     Hinter->show();
	}
}
//---------------------------------------------------------------------------

/*
 * Metoda przesuwajaca wiersz o jeden nizej
 * @returns Brak zwracanych wartosci
*/
void GText::moveRowUp(void)
{
	fParent->ExchangeRows(fRow, fRow-1);
	scene()->update();
}
//---------------------------------------------------------------------------

/*
 * Metoda przesuwajaca wiersz o jeden wyzej
 * @returns Brak zwracanych wartosci
*/
void GText::moveRowDown(void)
{
	fParent->ExchangeRows(fRow, fRow+1);
	scene()->update();
}
//---------------------------------------------------------------------------

/*
 * Metoda wstawiajaca wiersz po aktualnym
 * @returns Brak zwracanych wartosci
*/
void GText::insertRowAfter(void)
{
	fParent->insertRow(fRow+1);
}
//---------------------------------------------------------------------------

/*
 * Metoda wstawiajaca wiersz w miejsce aktualnego
 * @returns Brak zwracanych wartosci
*/
void GText::insertRowBefore(void)
{
	fParent->insertRow(fRow);
}
//---------------------------------------------------------------------------

/*
 * Metoda usuwajaca aktualny wiersz tabeli
 * @returns Brak zwracanych wartosci
*/
void GText::deleteRow(void)
{
	if(QMessageBox::question(NULL, "XTT Editor", "Are you really want to remove selected row?", QMessageBox::Yes | QMessageBox::No) == QMessageBox::No)
		return;
	
	fParent->DeleteRow(fRow);
}
//---------------------------------------------------------------------------

/*
 * Metoda usuwajaca aktualna kolumne tabeli
 * @returns Brak zwracanych wartosci
*/
void GText::deleteCol(void)
{
	if(QMessageBox::question(NULL, "XTT Editor", "Are you really want to remove selected column?", QMessageBox::Yes | QMessageBox::No) == QMessageBox::No)
		return;
		
	fParent->DeleteCol(fCol);
	if(fParent->ColCount() == 0)
		TableList->Delete(TableList->IndexOfID(fParent->TableId()));
}
//---------------------------------------------------------------------------

/*
 * Metoda uruchamiajaca okno edycji atrybutu bedacego przypisanym do danej kolumny
 * @returns Brak zwracanych wartosci
*/
void GText::editAttribute(void)
{
	if(AttributeGroups->IsEmpty())
     {
          const QString str2 = "There is no attributes defined yet.";
          QMessageBox::information(NULL, "XTT Editor", str2, QMessageBox::Ok);
          return;
     }

     delete AttributeEditor;
     AttributeEditor = new AttrEditor(fParent->Row(fRow)->Cell(fCol)->AttrType()->Id(), false);
     AttributeEditor->show();
}
//---------------------------------------------------------------------------
