 /**
 *                                                                             *
 * @file GText.h                                                               *
 * Project name: HeKatE hqed                                                   *
 * $Id: GText.h,v 1.12 2008-03-03 22:04:35 kinio Exp $                          *
 * Author: Krzysztof Kaczor                                                    *
 * Date: 15.05.2007                                                            *
 *                                                                             *
 * Implementation by Krzysztof Kaczor <kinio4444@gmail.com>                    *
 *                                                                             *
 * Copyright (C) 2006-9 by the HeKatE Project                                  *
 *                                                                             *
 * HQEd has been develped within the HeKatE Project,                           *
 * see http://hekate.ia.agh.edu.pl                                             *
 *                                                                             *
 * This file is part of HQEd.                                                  *
 *                                                                             *
 * HQEd is free software: you can redistribute it and/or modify                *
 * it under the terms of the GNU General Public License as published by        *
 * the Free Software Foundation, either version 3 of the License, or           *
 * (at your option) any later version.                                         *
 *                                                                             *
 * HQEd is distributed in the hope that it will be useful,                     *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU General Public License for more details.                                *
 *                                                                             *
 * You should have received a copy of the GNU General Public License           *
 * along with HQEd.  If not, see <http://www.gnu.org/licenses/>.               *
 *                                                                             *
 * Comment: Plik zawierajacy definicje obiektu bedacego textem rysowanym       *
 * na plotnie                                                                  *
 *                                                                             *
 */


#ifndef GTextH
#define GTextH
//---------------------------------------------------------------------------

#include <QObject>
#include <QGraphicsItem>
#include <QPainter>
#include <QGraphicsSceneMouseEvent>
#include <QStyleOptionGraphicsItem>
//--------------------------------------------------------------------------

class XTT_Table;
//---------------------------------------------------------------------------

class GText : public QObject, public QGraphicsItem
{
	Q_OBJECT

private:

     /// Tabela rodzic
     XTT_Table* fParent;

     /// Wiersz
     unsigned int fRow;

     /// Kolumna
     unsigned int fCol;

     /// Okresla co nalezy pobrac z tabeli, jaki text
     /// 0 - zawartosc komorki okreslonej przez fRow, fCol
     /// 1 - Title
     /// 2 - Header okreslony przez fCol
     unsigned int fTextType;
	
	/// Okresla czy komorka ma zostac zaznaczona
	bool isSelected;
	
	/// Polecenia menu kontekstowego
	QAction* aEidtTable;
	QAction* aCellEditor;
	QAction* aEditCell;
	QAction* aDeleteTable;
	QAction* aInsertRowBefore;
	QAction* aInsertRowAfter;
	QAction* aDeleteRow;
	QAction* aDeleteCol;
	QAction* aMoveRowDown;
	QAction* aMoveRowUp;
	QAction* aAttributeEditor;
	
	/// Metoda tworzaca polenia menu
	void createActions(void);
	
protected:

	/// Redefinicja metody obslugujacaej menu kontekstowe obiektu
	void contextMenuEvent(QGraphicsSceneContextMenuEvent*);
	
	/// Obsluga zdarzenia najechania myszka nad komorke
	void hoverEnterEvent(QGraphicsSceneHoverEvent*);
	
	/// Obsluga zdarzenia zjechania myszka znad komorki
	void hoverLeaveEvent(QGraphicsSceneHoverEvent*);

public:

     /// Konstruktor przyjmujacy wartosc rodzica i parametry tekstu
     GText(XTT_Table*, unsigned int, unsigned int, unsigned int);

     /// Metoda rysowania obiektu
     void paint(QPainter*, const QStyleOptionGraphicsItem*, QWidget* widget=0);

     /// Metoda okreslajaca kwadrat w jakim sie znajduje obiekt
     QRectF boundingRect() const;

     /// Ponizsze metody pozwalajac ustawic parametry obiektu
     void setRow(unsigned int _r){ fRow = _r; }
     void setCol(unsigned int _c){ fCol = _c; }
     void setTextType(unsigned int _tt){ fTextType = _tt; }
	
public slots:

	/// Metoda pokazujaca edytor komorki
	void showHinter(void);
	
	/// Metoda przenoszaca wiersz o jeden w gore
	void moveRowUp(void);
	
	/// Metoda przenoszaca wiersz o jeden w dol
	void moveRowDown(void);
	
	/// Metoda wstawiajaca wiersz w miejsce aktualnego
	void insertRowBefore(void);
	
	/// Metoda wstawiajaca wiersz w miejsce zaraz po aktualnym
	void insertRowAfter(void);
	
	/// Metoda usuwajaca aktualny wiersz tabeli
	void deleteRow(void);
	
	/// Metoda usuwajaca aktualna kolumne tabeli
	void deleteCol(void);
	
	/// Metoda otwierajaca okno edycji atrybutu
	void editAttribute(void);
};
//---------------------------------------------------------------------------
#endif
