 /**
 *                                                                             *
 * @file GWire.cpp                                                             *
 * Project name: HeKatE hqed                                                   *
 * $Id: GWire.cpp,v 1.12 2008-03-03 22:04:35 kinio Exp $                        *
 * Author: Krzysztof Kaczor                                                    *
 * Date: 21.05.2007                                                            *
 *                                                                             *
 * Implementation by Krzysztof Kaczor <kinio4444@gmail.com>                    *
 *                                                                             *
 * Copyright (C) 2006-9 by the HeKatE Project                                  *
 *                                                                             *
 * HQEd has been develped within the HeKatE Project,                           *
 * see http://hekate.ia.agh.edu.pl                                             *
 *                                                                             *
 * This file is part of HQEd.                                                  *
 *                                                                             *
 * HQEd is free software: you can redistribute it and/or modify                *
 * it under the terms of the GNU General Public License as published by        *
 * the Free Software Foundation, either version 3 of the License, or           *
 * (at your option) any later version.                                         *
 *                                                                             *
 * HQEd is distributed in the hope that it will be useful,                     *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU General Public License for more details.                                *
 *                                                                             *
 * You should have received a copy of the GNU General Public License           *
 * along with HQEd.  If not, see <http://www.gnu.org/licenses/>.               *
 *                                                                             *
 * Comment: Plik zawierajacy definicje metod obiektu symbolizujacego           *
 * czesc polaczenia miedzy tabelami                                            *
 *                                                                             *
 */

#include <QGraphicsScene>

#include "../../C/XTT/ConnectionEditor_ui.h"
#include "../../M/XTT/XTT_Connections.h"
#include "../../M/XTT/XTT_ConnectionsList.h"
#include "../../C/MainWin_ui.h"
#include "../../C/Settings_ui.h"
#include "GConnection.h"
#include "GWire.h"
//---------------------------------------------------------------------------

/**
 * Konstruktor klasy, ustawiajacy wlasciwosci obiektu
 * @param bool _isVertical - Okresla czy jest to linia pionowa (isVertical == 
 * true)
 * @param bool _isMovable - Okresla czy mozna ja przesuwac na scenie
 * @param GConnection* _parent - Wskanik do rodzica pojemnika przechowujacego 
 * ten obiekt
 * @param int index - Index w tablicy rodzica tego obiektu
 * @returns Brak zwracanych wartosci
*/
GWire::GWire(bool _isVertical, bool _isMovable, GConnection* _parent, int _index, int _length)
{
     fIsVertical = _isVertical;
     fIsMoveable = _isMovable;
     fParent = _parent;
     fIndex = _index;
     fLength = _length;
     setFlag(ItemIsSelectable);
     setMoveable(fIsMoveable);
     setCursor(Qt::CrossCursor);

	fLabel = "";
	fLabelVisible = false;
     fIsSelected = false;
	fLabelAlignment = -1;
     level = 0;

     setZValue(5);
}
//---------------------------------------------------------------------------

/**
 * Ustawia wlasciwosc poruszania po scenie za pomoca myszki elementu 
 * @param bool _mov - Wartosc okreslajaca czy element moze sie poruszac
 * @returns Brak zwracanych wartoci
*/
void GWire::setMoveable(bool _mov)
{
     setFlag(ItemIsFocusable, _mov);
     setFlag(ItemIsMovable, _mov);
     fIsMoveable = _mov;
}
//---------------------------------------------------------------------------

/**
 * Ustawia wartosci zmiennych x-owych i y-kowych ktrych nie moze zmienic obiekt 
 * w zaleznosci od typu
 * @param float x - Wartosc x
 * @param float y - Wartosc y
 * @returns Brak zwracanych wartoci
*/
void GWire::setCurPos(float _x, float _y)
{
     fPos.setX(_x);
     fPos.setY(_y);
}
//---------------------------------------------------------------------------

/**
 * Metoda ustawiajaca dlugosc lini i zapamietujaca ja w toolTip
 * @param float _l - Nowa dlugosc lini
 * @returns Brak zwracanych wartoci
*/
void GWire::setLength(float _l)
{
     setToolTip(QString::number(_l));
     fLength = _l;
}
//---------------------------------------------------------------------------

/**
 * Metoda klasy GRect rysujaca kwadrat na ekranie
 * @param QPainter* _p - Wskanik do plutna rysowania
 * @param const QStyleOptionGraphicsItem* - Wskanik do stylu rysowania
 * @param QWidget* widget - Wskanik nieuzywany :)
 * @returns Brak zwracanych wartoci
*/
void GWire::paint(QPainter* _p, const QStyleOptionGraphicsItem*, QWidget*)
{
     // Ustalanie wspolrzednych
	float x = 0.;
	float y = 0.;
     float dx = 0.;
     float dy = 0.;

     if(!fIsVertical)
          dx = fLength;
     if(fIsVertical)
          dy = fLength;


     QLineF line(x, y, dx, dy);

     if(!MainWin->ModelBW())
     {
          if(!fParent->Parent()->IsCut())
               _p->setPen(QPen(Settings->xttConnectionColor(), 1, Qt::SolidLine, Qt::SquareCap, Qt::RoundJoin));
          else if(fParent->Parent()->IsCut())
               _p->setPen(QPen(Settings->xttCutConnectionColor(), 1, Qt::SolidLine, Qt::SquareCap, Qt::RoundJoin));
          if(fIsSelected)
               _p->setPen(QPen(Settings->xttSelectedConnectionColor(), 1, Qt::SolidLine, Qt::SquareCap, Qt::RoundJoin));
     }
     else if(MainWin->ModelBW())
     {
          if(!fParent->Parent()->IsCut())
               _p->setPen(QPen(QColor(0, 0, 0, 255), 1, Qt::SolidLine, Qt::SquareCap, Qt::RoundJoin));
          else if(fParent->Parent()->IsCut())
               _p->setPen(QPen(QColor(0, 0, 0, 255), 1, Qt::DashLine, Qt::SquareCap, Qt::RoundJoin));
          if(fIsSelected)
               _p->setPen(QPen(QColor(0, 0, 0, 255), 1, Qt::DotLine, Qt::SquareCap, Qt::RoundJoin));
     }
                                                                
     _p->drawLine(line);
	
	// Wyswietlenie etykiety
	if(fLabelVisible)
	{
		QString labelForShow = fLabel;
		QFontMetrics fm(Settings->xttConnectionLabelFont());
		_p->setFont(Settings->xttConnectionLabelFont());
		_p->setPen(Settings->xttConnectionLabelColor());
		
		// Odciecie nadmiernej ilosci znako w etykiecie
		bool showLabel;
		int labelLength = fm.width(labelForShow);
		for(int i=fLabel.size()-1;i>=0;--i)
		{
			showLabel = true;
			if(!(((fLength > .0) && ((labelLength) <= (fLength - 10))) || ((fLength <= .0) && ((-labelLength) >= (fLength + 10)))))
			{
				labelForShow = fLabel.left(i) + "...";
				labelLength = fm.width(labelForShow);
				showLabel = false;
			}
			else break;
		}
		
		double xp = (fLength>.0)?8:-8; 			// gdy fLabelAlignment == -1
		if(fLabelAlignment == 0)
			xp = (fLength/2.0) - ((double)(labelLength/2)); 
		if(fLabelAlignment == 1)
			xp = (fLength>.0)?fLength-8-(labelLength):-(labelLength)-8;
		
		// Warunek sprawdzajacy czy dlugosc linii polaczenia nie jest za krotka, jezeli tak to ukrywamy podpis
		//if(((fLength > .0) && ((labelLength) <= (fLength - 10))) || ((fLength <= .0) && ((-labelLength) >= (fLength + 10))))
		if(showLabel)
			_p->drawText((int)xp, -5, labelForShow);
	}
	
     //_p->drawRect(boundingRect());
}
//---------------------------------------------------------------------------

/**
 * Metoda klasy zwracajaca obszar prostokatny w ktorym sie znajduje obiekt
 * @returns Obszar prostokatny zawierajacy obiekt
*/
QRectF GWire::boundingRect() const
{
     // Margines
     int margin = fLabelVisible?5:2;

     float dsx = -margin;
     float dsy = -margin;
     
     float dx = margin*2.0;
     float dy = margin*2.0;

     if(!fIsVertical)
     {
          if(fLength < 0)
               dsx = -dsx;
          dx = fLength-(dsx*2);
     }
     if(fIsVertical)
     {
          if(fLength < 0)
               dsy = -dsy;
          dy = fLength-(dsy*2);
     }

     return QRectF(dsx, dsy, dx, dy);
}
//---------------------------------------------------------------------------

/**
 * Metoda wywolywana w momencie zmiany polozenia obiektu
 * @param GraphicsItemChange change - Zawiera wartosc elementu ktora ulegla 
 * zmianie
 * @param const QVariant &value - Referencja do wartosci zmienionych
 * @returns Wartosc ktora ulela zmianie
*/
QVariant GWire::itemChange(GraphicsItemChange change, const QVariant &value)
{
     if((change == ItemSelectedChange) && (scene()))
     {
          bool isSel = value.toBool();
          if(isSel)
               fParent->SelectAll();
          else
               fParent->ClearSelection();

     }

     if((change == ItemPositionChange) && (scene()) && (level == 0))
     {
          level = 1;
          QPointF newPos = value.toPointF();

          // Gdy pionowa to moze zmienic tylko wartosc x
          if(fIsVertical)
               newPos.setY(fPos.y());

          // Gdy pozioma to moze zmienic tylko wartosc y
          else if(!fIsVertical)
               newPos.setX(fPos.x());

          // Wykonaie operacji zwiazanych ze zmiana pozycji tego elementu
          if(scene()->mouseGrabberItem() == this)
               fParent->PosChange(fIndex);

          level = 0;
          return newPos;
     }
     return QGraphicsItem::itemChange(change, value);
}
//---------------------------------------------------------------------------

/**
 * Metoda klasy GWire wywolywana w momencie dwukrotnego klikniecia na obiekcie, 
 * prostokacie
 * @param QGraphicsSceneMouseEvent* _e - Wskanik do informacji o zdarzeniu
 * @returns Brak zwracanych wartoci
*/
void GWire::mouseDoubleClickEvent(QGraphicsSceneMouseEvent* _e)
{
     if(_e->button() != Qt::LeftButton)
          return;

     // Wyszukiwanie i edycja tabeli
     int index = ConnectionsList->IndexOfID(fParent->Parent()->ConnId());

     if(index == -1)
          return;

     delete ConnEdit;
     ConnEdit = new ConnectionEditor_ui(false, index);
     ConnEdit->show();
}
//---------------------------------------------------------------------------

/**
 * Metoda klasy ustawiajaca sposob wysrodkowania etykiety opisujacej polaczenie
 * @param int _align - Sposob wysrodkowania -1-do lewej, 0-do srodka, 1-do prawej
 * @returns Brak zwracanych wartoci
 */
void GWire::setLabelAlignment(int _align)
{
	if((_align != -1) && (_align != 0 ) && (_align != 1))
	{
		fLabelAlignment = -1;
		return;
	}
		
	fLabelAlignment = _align;
}
//---------------------------------------------------------------------------
