 /**
 *                                                                             *
 * @file main.cpp                                                              *
 * Project name: HeKatE hqed                                                   *
 * $Id: main.cpp,v 1.17 2008-03-03 22:04:30 kinio Exp $                         *
 * Author: Krzysztof Kaczor                                                     *
 * Date: 24.04.2007                                                            *
 * Implementation by Krzysztof Kaczor <kinio4444@gmail.com>                    *
 *                                                                             *
 * Copyright (C) 2006-9 by the HeKatE Project                                  *
 *                                                                             *
 * HQEd has been develped within the HeKatE Project,                           *
 * see http://hekate.ia.agh.edu.pl                                             *
 *                                                                             *
 * This file is part of HQEd.                                                  *
 *                                                                             *
 * HQEd is free software: you can redistribute it and/or modify                *
 * it under the terms of the GNU General Public License as published by        *
 * the Free Software Foundation, either version 3 of the License, or           *
 * (at your option) any later version.                                         *
 *                                                                             *
 * HQEd is distributed in the hope that it will be useful,                     *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 * GNU General Public License for more details.                                *
 *                                                                             *
 * You should have received a copy of the GNU General Public License           *
 * along with HQEd.  If not, see <http://www.gnu.org/licenses/>.               *
 *                                                                             *
 * Comment: Plik uruchamiajacy program                                         *
 *                                                                             *
 */

#include <QApplication>

#include "M/ARD/ARD_LevelList.h"
#include "M/ARD/ARD_AttributeList.h"
#include "M/ARD/ARD_PropertyList.h"
#include "M/ARD/ARD_DependencyList.h"

#include "M/XTT/XTT_AttributeGroups.h"
#include "M/XTT/XTT_TableList.h"
#include "M/XTT/XTT_ConnectionsList.h"

#include "C/XTT/Hinter_ui.h"
#include "C/XTT/ValEdit_ui.h"
#include "C/XTT/UnusedAttributes_ui.h"
#include "C/XTT/ConnectionManager_ui.h"
#include "C/XTT/DeleteConnection_ui.h"
#include "C/XTT/ConnectionEditor_ui.h"
#include "C/XTT/CellEditor_ui.h"
#include "C/XTT/DeleteTable_ui.h"
#include "C/XTT/AttributeEditor_ui.h"
#include "C/XTT/AttributeDelete_ui.h"
#include "C/XTT/AttributeManager_ui.h"
#include "C/XTT/GroupManager_ui.h"
#include "C/XTT/GroupEditor_ui.h"
#include "C/XTT/SelectGroup_ui.h"
#include "C/XTT/TableEditor_ui.h"
#include "C/XTT/ColumnEditor_ui.h"
#include "C/StartScreen_ui.h"
#include "C/Settings_ui.h"
#include "C/MainWin_ui.h"
#include "C/About_ui.h"
// -----------------------------------------------------------------------------

/**
 * Glowna funckja programu
 * @param int argc - Liczba argumentow w linii polecen
 * @param char *argv[] - Wskanik do listy argumantow z linii polecen
 * @returns Sukces do systemu operacyjnego
*/
int main(int argc, char *argv[])
{
     QApplication::setDesktopSettingsAware(false);
     QApplication::setEffectEnabled(Qt::UI_AnimateMenu);
     QApplication::setEffectEnabled(Qt::UI_FadeMenu);
     QApplication::setEffectEnabled(Qt::UI_AnimateCombo);
     QApplication::setEffectEnabled(Qt::UI_AnimateTooltip);
     QApplication::setEffectEnabled(Qt::UI_FadeTooltip);
     QApplication::setEffectEnabled(Qt::UI_AnimateToolBox);

     QApplication app(argc, argv);
	Q_INIT_RESOURCE(appresfile);
	app.setWindowIcon(QIcon(":/all_images/images/mainico.bmp")); 
	
	// Tworzenie okienka ustawien przed wszystim, aby nie bylo mozliwosci odwolan do nieistniejecych obiektow
	Settings = new Settings_ui;
	
	// Alokacja pamieci na pojemiki modelu ARD
	ardLevelList = new ARD_LevelList;
	ardAttributeList = new ARD_AttributeList;
	ardPropertyList = new ARD_PropertyList;
	ardDependencyList = new ARD_DependencyList;
	
	// Alokacja pamieci na pojemiki modelu XTT
     AttributeGroups = new XTT_AttributeGroups;
     TableList = new XTT_TableList;
     ConnectionsList = new XTT_ConnectionsList;

     // Alokacja pamieci na okienka
     AttributeEditor = new AttrEditor;
     AttributeDelete = new AttributeDelete_ui;
     AttrManager = new AttributeManager_ui;
     GroupMan = new GroupManager_ui;
     GroupEdit = new GroupEditor_ui;
     SelGroup = new SelectGroup_ui;
     TableEdit = new TableEditor_ui;
     ColumnEdit = new ColumnEditor_ui;
     DelTable = new DeleteTable_ui;
     CellEdit = new CellEditor_ui;
     ConnEdit = new ConnectionEditor_ui;
     DelConn = new DeleteConnection_ui;
     ConnManager = new ConnectionManager_ui;
     UnusedAttr = new UnusedAtrtribute_ui;
     AboutWin = new About_ui;
     ValEdit = new ValueEdit_ui;
     Hinter = new Hinter_ui;
	MainWin = new MainWin_ui;
	
	// Zastosowanie ustawien
	Settings->applyOptions();
	
	// Utworzenie okienka startowego
	if(Settings->useStartScreen())
		(StartScreen = new StartScreen_ui)->show();
	
     // Pokazanie glownego okienka
	if(!Settings->useStartScreen())
		MainWin->show();
		
     return app.exec();
}
// -----------------------------------------------------------------------------
